import java.text.MessageFormat

import java.util.Hashtable.ValueCollection;

import javax.servlet.http.HttpServletResponse

import com.sas.oprisk.framework.server.util.LabelValueBean

import com.sas.fsd.shared.util.StringUtil
import com.sas.oprisk.framework.export.RGFRow
import com.sas.oprisk.framework.export.RGFStyle
import com.sas.oprisk.framework.export.RGFWorkbook
import com.sas.oprisk.framework.export.RGFWorksheet
import com.sas.oprisk.framework.server.objects.BusinessObject.ClassObject
import com.sas.oprisk.framework.server.persistence.PersistenceSession
import com.sas.oprisk.framework.server.query.Query
import com.sas.oprisk.framework.server.query.QueryConstraint
import com.sas.oprisk.framework.server.query.QueryOrderByField
import com.sas.oprisk.framework.server.query.QueryOrdering
import com.sas.oprisk.monitor.web.AppRequestContext
import com.sas.oprisk.monitor.web.AppRequestContextHolder
import com.sas.oprisk.server.BusinessObjectTypeInfo
import com.sas.oprisk.server.ConfigDataProperties
import com.sas.oprisk.server.CustFieldDefn
import com.sas.oprisk.server.LinkType
import com.sas.oprisk.server.NamedList
import com.sas.oprisk.server.NamedListMapping
import com.sas.oprisk.server.base.CustFieldDefnQueryVariable
import com.sas.oprisk.server.base.CustFieldTypeCd
import com.sas.oprisk.server.base.NamedListMappingQueryVariable
import com.sas.oprisk.server.base.NamedListQuery
import com.sas.oprisk.server.base.NamedListQueryVariable
import com.sas.oprisk.server.services.ServiceLocator
import com.sas.oprisk.server.util.BusinessObjectUtils
import com.sas.oprisk.server.CustomObjectMetaData;
import com.sas.oprisk.server.CustomObjectRegistry;
import com.sas.oprisk.server.BusinessObjectLocalization
import com.sas.oprisk.framework.server.objects.BusinessObjectList
import com.sas.oprisk.server.services.ServiceLocator
import com.sas.oprisk.framework.server.util.ParsingUtils;


AppRequestContext requestContext = AppRequestContextHolder.getRequestContext();
ResourceBundle bundle = requestContext.getResourceBundle();
Locale locale = requestContext.getLocale();
PersistenceSession psession = ServiceLocator.getPersistenceSession();
HttpServletResponse response = requestContext.getResponse();

def localize={key,vals ->
   String message = bundle.getString(key);
   if(vals!=null){
      message = MessageFormat.format(message, vals)
   }
   return StringUtil.escapeHTML(message)
}

def getCustomizableBusinessObjectTypes(ResourceBundle bundle)
{
   BusinessObjectTypeInfo[] values = BusinessObjectTypeInfo.values();
   List<BusinessObjectTypeInfo> result = new ArrayList<BusinessObjectTypeInfo>(values.length);

   boolean allObjects = "true".equalsIgnoreCase(ConfigDataProperties.getProperty("monitor.listCustomFields.allObjects", "false"));
      
   for (BusinessObjectTypeInfo boType : values) {
      ClassObject classObject = boType.getClassObject();
      ClassObject nonvirtualClassObject = classObject.getNonvirtualClassObject();
      if (classObject != nonvirtualClassObject) {
         continue;
      }

      if (BusinessObjectUtils.isEnabled(boType) && (allObjects || (classObject.businessObjectTypeName != null && classObject.businessObjectTypeName.startsWith("customObject")))) {
         result.add(boType);         
      }
   }
   
   Collections.sort result, new Comparator<BusinessObjectTypeInfo>() {
      public int compare(BusinessObjectTypeInfo o1, BusinessObjectTypeInfo o2) {
         String label1 = bundle.getString(o1.getDisplayNameKey());
         String label2 = bundle.getString(o2.getDisplayNameKey());
         return label1.compareTo(label2);
      }
   };
   
   return result;
}


List<BusinessObjectTypeInfo> customizableBusinessObjectTypes =
   getCustomizableBusinessObjectTypes(bundle);


def getBusinessObjectTypeLabel(String businessObjectTypeNm, ResourceBundle bundle)
{
   BusinessObjectTypeInfo boType = BusinessObjectTypeInfo.getByBusinessObjectTypeName(businessObjectTypeNm);   
   return boType != null ? StringUtil.escapeHTML(bundle.getString(boType.getDisplayNameKey())) : "";
}

out.println("<html>")

out.println("<head>");

out.println("<link rel=stylesheet href='" + request.getContextPath() + "/docs/Documentation.css' type='text/css' />");

out.println "<script type='text/javascript'>";

out.println("function mySubmit() { var f = document.forms.custFieldQueryForm; f.name.value = ''; if (f.csvNm.value == 'xls') {" +
            "for(var i = 0; i < f.csvNm.length; i++) {" +
                "var option = f.csvNm[i];" +
                "if(option && option.selected) {" +
                    "option.classList.add('optionSelected');" +
                "} else {" +
                    "option.classList.remove('optionSelected');" +
                "}" +
            "}" +
            "var div = document.getElementById('customFieldsDocumentationListing'); if (div) div.style.display='none';}f.submit(); }")

out.println "</script>";

out.println("</head>");

out.println("<body>");


String businessObjectTypeNm =
StringUtil.trimToNull(request.getParameter("businessObjectTypeNm"));

String csvNm =
StringUtil.trimToNull(request.getParameter("csvNm"));

if (csvNm == null) {
   csvNm = "blank";
}

String action = request.getRequestURI() + "?rgfContentId=" + StringUtil.escapeHTML(ServiceLocator.getContentId());
out.println("<form id='custFieldQueryForm' method='POST' action='" + action + "'>");
out.println("<table class='layoutTable'>");
out.println("<tbody>");

out.println("<tr>");
out.println("   <th>"+localize("listCustomFields.businessobject.txt",null)+":</th>");
out.println("   <td>");
out.println("<select name='businessObjectTypeNm' id='businessObjectTypeNm' onchange='javascript:mySubmit();'>");

//-----add a default <ALL>-----
out.print("<option value='All'>");
out.print(StringUtil.escapeHTML(bundle.getString("application.all.txt")));
out.println("</option>");

for (BusinessObjectTypeInfo boType : customizableBusinessObjectTypes)
{
   if (boType.getBusinessObjectTypeName().equals(businessObjectTypeNm)) {
      out.print("<option value='" + StringUtil.escapeHTML(boType.getBusinessObjectTypeName()) + "' class='optionSelected' selected >");
   } else { 
      out.print("<option value='" + StringUtil.escapeHTML(boType.getBusinessObjectTypeName()) + "' >");
   }
   out.print(StringUtil.escapeHTML(bundle.getString(boType.getDisplayNameKey())));
   out.println("</option>");
}
out.println("</select>");
out.println("   </td>");


out.println("   <th>"+localize("listCustomFields.listingType.txt",null)+":</th>");
out.println("   <td>");
out.println("<select name='csvNm' onchange='javascript:mySubmit();'>");

if ("blank".equals(csvNm)) {
   out.println "<option value='blank' class='optionSelected' selected>"+localize("listCustomFields.customFields.txt",null)+"</option>";   
} else {
   out.println "<option value='blank'>"+localize("listCustomFields.customFields.txt",null)+"</option>";   
}

if ("csv".equals(csvNm)) {
   out.print("<option value='csv' class='optionSelected' selected>"+localize("listCustomFields.csvListing.txt",null)+"</option>");
} else {
   out.print("<option value='csv'>"+localize("listCustomFields.csvListing.txt",null)+"</option>");
}

if ("xls".equals(csvNm)) {
   out.print("<option value='xls' class='optionSelected' selected>"+localize("listCustomFields.xlsExcel.txt",null)+"</option>");
} else {
   out.print("<option value='xls'>"+localize("listCustomFields.xlsExcel.txt",null)+"</option>");
}

out.println("</select>");
out.println("   </td>");


out.println("</tr>");

out.println("</tbody>");
out.println("</table>");

if (businessObjectTypeNm != null) {
   boolean isAll = businessObjectTypeNm.equals("All");

   out.println("<br/>");
   out.println("<div id='customFieldsDocumentationListing'>");
   
   CustFieldDefnQueryVariable qvar = CustFieldDefn.ALL.result();
   List<CustFieldDefn> results = isAll ?
                                    CustFieldDefn.ALL.
                                       withOrdering(new QueryOrderByField(qvar.businessObjectTypeNm, true)).
                                       execute(psession) : 
                                    CustFieldDefn.ALL.
                                       withConstraint(Query.equalTo(qvar.businessObjectTypeNm, businessObjectTypeNm)).
                                       withOrdering(new QueryOrderByField(qvar.custFieldNm, true)).
                                       execute(psession);

   if (isAll) {
      String prefix = "customObject";
      int indexLen = prefix.length();
      Collections.sort results, new Comparator<CustFieldDefn>() {
         public int compare(CustFieldDefn c1, CustFieldDefn c2) {
            String type1 = getBusinessObjectTypeLabel(c1.getBusinessObjectTypeNm(), bundle);
            String type2 = getBusinessObjectTypeLabel(c2.getBusinessObjectTypeNm(), bundle);
            if (type1 == null) {
               return type22 == null ? (type1.equals(type2) ? c1.getCustFieldNm().compareTo(c2.getCustFieldNm()) : type1.compareTo(type2)) : 1;
            } else if (type2 == null) {
               return -1;
            } else {
               if (type1.equals(type2)) {
                  return c1.getCustFieldNm().compareTo(c2.getCustFieldNm()); 
               } else {
                  return type1.compareTo(type2);
               }
            }
         }
      };
   }
   
   out.println("<h3>" + (isAll ? businessObjectTypeNm : StringUtil.escapeHTML(getBusinessObjectTypeLabel(businessObjectTypeNm, bundle))) + "</h3>");

   if ("blank".equals(csvNm) || "xls".equals(csvNm)) {
      out.println("<table class='contentTable'>");
      out.println("<tbody>");
      out.println("<tr>");

      out.println("   <th>"+localize("listCustomFields.table.businessObjectType.txt",null)+"</th>");
      out.println("   <th>"+localize("listCustomFields.table.name.txt",null)+"</th>");
      out.println("   <th>"+localize("listCustomFields.table.description.txt",null)+"</th>");
      out.println("   <th>"+localize("listCustomFields.table.maxLength.txt",null)+"</th>");
      out.println("   <th>"+localize("listCustomFields.table.type.txt",null)+"</th>");
      out.println("   <th>"+localize("listCustomFields.table.namedList.txt",null)+"</th>");
      out.println("   <th>"+localize("listCustomFields.table.namedListValues.txt",null)+"</th>");
      out.println("</tr>");
   
      
      String saveBusinessObjectTypeNm = null;         
      for (CustFieldDefn c : results) {      
         if (isAll) {
            if (!c.getBusinessObjectTypeNm().equals(saveBusinessObjectTypeNm)) {
               out.println("<tr style='border-left-style:hidden'>");
               out.println("   <td style='border:0'></td>");
               out.println("</tr>");
                  
               out.println("<tr style='border-left-style:hidden'>");
               out.println("   <td style='border:0'>" + StringUtil.escapeHTML(getBusinessObjectTypeLabel(c.getBusinessObjectTypeNm(), bundle)) + "</td>");
               out.println("</tr>");
            }
         }
         
         out.println("<tr>");
         
         //if (isAll) {
            if (!c.getBusinessObjectTypeNm().equals(saveBusinessObjectTypeNm)) {
               out.println("   <td>" + c.getBusinessObjectTypeNm() + "</td>");
            } else {
               out.println("   <td></td>");
            }
            saveBusinessObjectTypeNm = c.getBusinessObjectTypeNm();   
         //}         
         
         out.println("   <td>" + StringUtil.escapeHTML(c.getCustFieldNm()) + "</td>");
         
         if (c.getCustFieldDesc() != null)
            out.println("   <td>" + StringUtil.escapeHTML(c.getCustFieldDesc()) + "</td>");
         else
            out.println("   <td></td>");
            
         if (c.getCustFieldMaxLength() != null)
            out.println("   <td>" + StringUtil.escapeHTML(c.getCustFieldMaxLength()) + "</td>");
         else
            out.println("   <td></td>");
         
         for (CustFieldTypeCd typeCd : CustFieldTypeCd.getAllValues()) {
            if (typeCd.getValue().equals(c.getCustFieldTypeCd().getValue())) {
               out.println("   <td>" + StringUtil.escapeHTML(bundle.getString(typeCd.getDisplayNameKey())) + "</td>");
            }
         }
         
         String namedListStr = NamedList.object.getNamedListNm(c.getBusinessObjectTypeNm(), c.getCustFieldNm(), psession);
         List<LabelValueBean> values = NamedList.object.getLabelValueBeansForCustField(c.getBusinessObjectTypeNm(), c.getCustFieldNm(), psession);
         if (!StringUtil.isEmpty(namedListStr)) {
            out.println("   <td>" + StringUtil.escapeHTML(namedListStr) + "</td>");
            
            if (values != null && values.size() > 0) {
               StringBuffer buf = new StringBuffer();
               for (LabelValueBean value : values) {
                  if (buf.size() > 0) {
                     buf.append(", ");
                  }
                  buf.append(value.getLabel());
                  buf.append(" ");
                  buf.append("[");
                  buf.append(value.getValue());
                  buf.append("]");
               }
               
               if (values.size() > 0) {
                  out.println("   <td>" + StringUtil.escapeHTML(buf.toString()) + "</td>");
               } else {
                  out.println("   <td></td>");
               }
            } else {
               out.println("   <td></td>");
            }
      
         } else {
            out.println("   <td></td>");
            out.println("   <td></td>");
         }
         
         out.println("</tr>");         
      }
   
      out.println("</tbody>");
      out.println("</table>");
   }

   if ("csv".equals(csvNm)) {
      //----------CustFieldDefns----------
      out.println("<br/><br/>");
      out.println("<h5>----------"+localize("listCustomFields.csvContents.fmt.txt",["CustFieldDefns"] as Object[])+"----------</h5>");
      out.println("<br/>");
      
      String saveBusinessObjectTypeNm = null;         
      out.println("custFieldDefn.businessObjectTypeNm,custFieldDefn.custFieldNm,custFieldDefn.custFieldTypeCd,custFieldDefn.custFieldMaxLength,custFieldDefn.custFieldDesc,");
      out.println("<br/>");
      for (CustFieldDefn c : results) {
         if (isAll) {
            if (saveBusinessObjectTypeNm != null && !saveBusinessObjectTypeNm.equals(c.getBusinessObjectTypeNm())) {
               out.println(",,,,,");
               out.println("<br/>");
            }
            saveBusinessObjectTypeNm = c.getBusinessObjectTypeNm();   
         }         
         String maxLen = c.getCustFieldMaxLength() == null ? "" : c.getCustFieldMaxLength();
         String desc = c.getCustFieldDesc() == null ? "" : c.getCustFieldDesc();
         out.println(StringUtil.escapeHTML(c.getBusinessObjectTypeNm() + "," + c.getCustFieldNm() + "," + c.getCustFieldTypeCd().getValue() + "," + maxLen + "," + desc + ","));
         out.println("<br/>");
      }

         
      //----------NamedListMappings----------
      NamedListMappingQueryVariable qvar1 = NamedListMapping.ALL.result();
      List<NamedListMapping> namedListMappings = isAll ?
                     NamedListMapping.ALL.
                                       withOrdering(new QueryOrderByField(qvar1.listNm, true)).
                                       execute(psession) :                                  
                     NamedListMapping.ALL.
                                       withConstraint(Query.equalTo(qvar1.businessObjectTypeNm, businessObjectTypeNm)).
                                       withOrdering(new QueryOrderByField(qvar1.listNm, true)).
                                       execute(psession);
                                       
      if (isAll) {
         String prefix = "customObject";
         int indexLen = prefix.length();
         Collections.sort namedListMappings, new Comparator<NamedListMapping>() {
            public int compare(NamedListMapping nlm1, NamedListMapping nlm2) {
               String type1 = nlm1.getBusinessObjectTypeNm();
               String type2 = nlm2.getBusinessObjectTypeNm();
               String numberStr1 = type1.startsWith(prefix) ? type1.substring(indexLen) : null;
               String numberStr2 = type2.startsWith(prefix) ? type2.substring(indexLen) : null;
               if (numberStr1 == null) {
                  return numberStr2 == null ? (type1.equals(type2) ? nlm1.getListNm().compareTo(nlm2.getListNm()) : type1.compareTo(type2)) : 1;
               } else if (numberStr2 == null) {
                  return -1;
               } else {
                  Integer number1 = Integer.valueOf(numberStr1);
                  Integer number2 = Integer.valueOf(numberStr2);
                  if (number1.equals(number2)) {
                     return nlm1.getListNm().compareTo(nlm2.getListNm()); 
                  } else {
                     return number1.compareTo(number2);
                  }
               }
            }
         };
      }
                                       
      out.println("<br/><br/>");
      out.println("<h5>----------"+localize("listCustomFields.csvContents.fmt.txt",["NamedListMappings"] as Object[])+"----------</h5>");
      out.println("<br/>");
      out.println("namedListMapping.listNm,namedListMapping.businessObjectTypeNm,namedListMapping.fieldNm");
      out.println("<br/>");
      Set<String> names = new HashSet<String>();                                
      saveBusinessObjectTypeNm = null;         
      for (NamedListMapping m : namedListMappings) {
         if (isAll) {
            if (saveBusinessObjectTypeNm != null && !saveBusinessObjectTypeNm.equals(m.getBusinessObjectTypeNm())) {
               out.println(",,");
               out.println("<br/>");
            }
            saveBusinessObjectTypeNm = m.getBusinessObjectTypeNm();   
         }         
         out.println(StringUtil.escapeHTML(m.getListNm() + "," + m.getBusinessObjectTypeNm() + "," + m.getFieldNm()));
         out.println("<br/>");
         names.add(m.getListNm());
      }

      
      //----------NamedLists----------
      NamedListQueryVariable qvar2 = NamedList.ALL.result();
      List<QueryOrdering> orderings = new ArrayList(Arrays.asList(new QueryOrderByField(qvar2.listNm, true), new QueryOrderByField(qvar2.rowNo, true)));
      List<QueryConstraint> constraints = isAll ? new ArrayList() : new ArrayList(Arrays.asList(Query.in(qvar2.listNm, names.toArray(new String[names.size()]))));
      NamedListQuery query2 = new NamedListQuery(qvar2, constraints, orderings);
      List<NamedList> namedLists = query2.execute(psession);
      
      Map<String, Boolean> orderByNameMap = new HashMap<String, Boolean>();      
      Map<String, StringBuffer> optionsMap = new HashMap<String, StringBuffer>();
      for (NamedList nl : namedLists) {
         String listNm = nl.getListNm();
         StringBuffer options = optionsMap.get(listNm);
         if (options == null) {
            options = new StringBuffer();
            optionsMap.put(listNm, options);
         } else {
            options.append(",");
         }
         options.append(nl.getOptionCd());
         
         if (nl.getRowNo() != null) {
            if (orderByNameMap.get(listNm) == null) {
               orderByNameMap.put(listNm, Boolean.FALSE);               
            }            
         }
      }
      out.println("<br/><br/>");
      out.println("<h5>----------"+localize("listCustomFields.csvContents.fmt.txt",["NamedLists"] as Object[])+"----------</h5>");
      out.println("<br/>");
      out.println("namedList.listNm,namedList.optionCds,namedList.orderByName");
      out.println("<br/>");
      
      if (isAll) {
         saveBusinessObjectTypeNm = null;         
         for (NamedListMapping m : namedListMappings) {
            if (saveBusinessObjectTypeNm != null && !saveBusinessObjectTypeNm.equals(m.getBusinessObjectTypeNm())) {
               out.println(",,");
               out.println("<br/>");
            }
            saveBusinessObjectTypeNm = m.getBusinessObjectTypeNm();
            
            String listNm = m.getListNm();
            StringBuffer options = optionsMap.get(listNm);
            String optionCds = "\"" + options.toString() + "\"";
            out.println(StringUtil.escapeHTML(listNm + "," + optionCds + "," + (orderByNameMap.get(listNm) != null ? "FALSE" : "")));
            out.println("<br/>");
         }
         
         //----------NamedLists----------
         boolean first = true;
         for (NamedList nl : namedLists) {
            String listNm = nl.getListNm();
            boolean found = false;
            for (NamedListMapping m : namedListMappings) {
               if (listNm.equals(m.getListNm())) {
                  found = true;
                  break;
               }
            }
            if (!found) {
               System.out.println("NamedList is NOT found: " + listNm);
               
               if (first) {
                  out.println(",,");
                  out.println("<br/>");
                  first = false;
               }
               
               StringBuffer options = optionsMap.get(listNm);
               String optionCds = "\"" + options.toString() + "\"";
               out.println(StringUtil.escapeHTML(listNm + "," + optionCds + "," + (orderByNameMap.get(listNm) != null ? "FALSE" : "")));
               out.println("<br/>");
            } else {
               // System.out.println("NamedList is found: " + listNm);
            }
         }
         System.out.println("namedListMappings size = " + namedListMappings.size());
         System.out.println("optionsMap size = " + optionsMap.size());
         //---------------------------
                  
      } else {
         for (String listNm : optionsMap.keySet()) {
            StringBuffer options = optionsMap.get(listNm);
            String optionCds = "\"" + options.toString() + "\"";
            out.println(StringUtil.escapeHTML(listNm + "," + optionCds + "," + (orderByNameMap.get(listNm) != null ? "FALSE" : "")));
            out.println("<br/>");
         }
      }

            
      //----------NamedListTranslations----------
      out.println("<br/><br/>");
      out.println("<h5>----------"+localize("listCustomFields.csvContents.fmt.txt",["NamedListTranslations"] as Object[])+"----------</h5>");
      out.println("<br/>");
      out.println("namedList.listNm,namedList.optionCd,businessObjectLocalization.languageCd,businessObjectLocalization.descTxt");
      out.println("<br/>");
      for (NamedList nl : namedLists) {
         BusinessObjectList<BusinessObjectLocalization> namedListLocalizations = nl.getLocalizations(psession);
         for (BusinessObjectLocalization nlLocalization : namedListLocalizations) {
            out.println(StringUtil.escapeHTML(nl.getListNm() + "," + nl.getOptionCd() + "," + nlLocalization.getLanguageCd() + "," + nlLocalization.getDescTxt()));
            out.println("<br/>");
         }
      }
      
      
      
      //----------LinkTypes----------
      out.println("<br/><br/>");
      out.println("<h5>----------"+localize("listCustomFields.csvContents.fmt.txt",["LinkTypes"] as Object[])+"----------</h5>");
      out.println("<br/>");
      out.println("linkType.sourceSystemCd, linkType.linkTypeId, linkType.businessObjectTypeNm1, linkType.businessObjectTypeNm2,linkType.defaultLanguageCd,linkType.otherLanguageCds, linkType.name1, linkType.description1, linkType.help1, linkType.name2, linkType.description2, linkType.help2");
      out.println("<br/>");
      List<LinkType> linkTypes = isAll ? LinkType.ALL.execute(psession) : LinkType.object.getEligibleLinkTypes(businessObjectTypeNm, psession);
      for (LinkType l : linkTypes) {
         out.println(StringUtil.escapeHTML(l.getSourceSystemCd() + "," + l.getLinkTypeId() + "," + l.getBusinessObjectTypeNm1() + "," + l.getBusinessObjectTypeNm2() + ",en," + "\"ar, de, es, fr, it, iw, ja, ko, nl, pl, pt_BR, pt, ru, uk, zh_CN, zh_TW\",,,,,,"));
         out.println("<br/>");
      }
      
      //----------LinkTypeTranslations----------
      out.println("<br/><br/>");
      out.println("<h5>----------"+localize("listCustomFields.csvContents.fmt.txt",["LinkTypeTranslations"] as Object[])+"----------</h5>");
      out.println("<br/>");
      out.println("linkType.linkTypeId,linkType.sourceSystemCd,businessObjectLocalization.languageCd,businessObjectLocalization.name1Txt,businessObjectLocalization.desc1Txt,businessObjectLocalization.help1Txt,businessObjectLocalization.name2Txt,businessObjectLocalization.desc2Txt,businessObjectLocalization.help2Txt");
      out.println("<br/>");
      for (LinkType l : linkTypes) {
         BusinessObjectList<BusinessObjectLocalization> linkTypeLocalizations = l.getLocalizations(psession);
         for (BusinessObjectLocalization loc : linkTypeLocalizations) {
            Locale currentLocale = ParsingUtils.parseLocale(loc.getLanguageCd());
            out.println(StringUtil.escapeHTML(l.getLinkTypeId() + "," + l.getSourceSystemCd() + "," + loc.getLanguageCd() + "," + l.getLinkTypeNames(currentLocale, psession)[0] + "," + l.getLinkTypeDescriptions(currentLocale, psession)[0] + "," + l.getLinkTypeHelp(currentLocale, psession)[0] + "," + l.getLinkTypeNames(currentLocale, psession)[1] + "," + l.getLinkTypeDescriptions(currentLocale, psession)[1] + "," + l.getLinkTypeHelp(currentLocale, psession)[1]));
            out.println("<br/>");
         }
      }
   }
   out.println("</div>");
   
   if ("xls".equals(csvNm)) {
      RGFWorkbook wb = new RGFWorkbook();
      
      RGFStyle headerStyle = wb.createStyle(RGFStyle.TEXT, RGFStyle.ALIGN_LEFT, RGFStyle.VERTICAL_TOP, false, wb.createDefaultFont());
      RGFStyle textStyle = wb.createStyle(RGFStyle.TEXT, RGFStyle.ALIGN_LEFT, RGFStyle.VERTICAL_TOP, false, wb.createDefaultFont());
      RGFStyle numberStyle = wb.createStyle(RGFStyle.TEXT, RGFStyle.ALIGN_RIGHT, RGFStyle.VERTICAL_TOP, false, wb.createDefaultFont());
      RGFStyle commentStyle = wb.createStyle(RGFStyle.TEXT, RGFStyle.ALIGN_CENTER, RGFStyle.VERTICAL_TOP, false, wb.createDefaultFont());
      
      //----------CustFieldDefns----------
      RGFWorksheet ws = wb.createWorksheet("CustFieldDefns");
      ws.setDefaultColumnWidth(35);
      RGFRow headerRow = ws.createRow();
      headerRow.createEmptyCell(commentStyle).setColumnWidth(3);
      headerRow.createTextCell("custFieldDefn.businessObjectTypeNm", headerStyle);
      headerRow.createTextCell("custFieldDefn.custFieldNm", headerStyle);
      headerRow.createTextCell("custFieldDefn.custFieldTypeCd", headerStyle);
      headerRow.createTextCell("custFieldDefn.custFieldMaxLength", headerStyle);
      headerRow.createTextCell("custFieldDefn.custFieldDesc", headerStyle);
      
      saveBusinessObjectTypeNm = null;         
      for (CustFieldDefn c : results) {
         RGFRow row = ws.createRow();
         if (isAll) {
            if (!c.getBusinessObjectTypeNm().equals(saveBusinessObjectTypeNm)) {
               row.createEmptyCell(textStyle);
               row = ws.createRow();
               if (saveBusinessObjectTypeNm != null) {
                  row.createEmptyCell(textStyle);
                  row = ws.createRow();
               }            
               row.createTextCell("# ", commentStyle).setColumnWidth(3);
               row.createTextCell(StringUtil.escapeHTML(getBusinessObjectTypeLabel(c.getBusinessObjectTypeNm(), bundle)), textStyle);
               row = ws.createRow();
               
               row.createEmptyCell(textStyle);
               row = ws.createRow();
            }
            saveBusinessObjectTypeNm = c.getBusinessObjectTypeNm();   
         }         
         row.createEmptyCell(textStyle);
         row.createTextCell(c.businessObjectTypeNm, textStyle);
         row.createTextCell(c.getCustFieldNm(), textStyle);
         row.createTextCell(c.getCustFieldTypeCd().getValue(), textStyle);
         (c.getCustFieldMaxLength() == null) ? row.createEmptyCell(numberStyle) : row.createIntegerCell(c.getCustFieldMaxLength(), numberStyle);
         (c.getCustFieldDesc() == null) ?  row.createEmptyCell(textStyle) : row.createTextCell(c.getCustFieldDesc(), textStyle);
      }

      //----------NamedListMappings----------
      ws = wb.createWorksheet("NamedListMappings");
      ws.setDefaultColumnWidth(40);
      headerRow = ws.createRow();
      headerRow.createTextCell("namedListMapping.listNm", headerStyle);
      headerRow.createTextCell("namedListMapping.businessObjectTypeNm", headerStyle);
      headerRow.createTextCell("namedListMapping.fieldNm", headerStyle);
      
      NamedListMappingQueryVariable qvar1 = NamedListMapping.ALL.result();
      List<NamedListMapping> namedListMappings = isAll ? 
                        NamedListMapping.ALL.
                                       withOrdering(new QueryOrderByField(qvar1.listNm, true)).
                                       execute(psession) :                                                
                        NamedListMapping.ALL.
                                       withConstraint(Query.equalTo(qvar1.businessObjectTypeNm, businessObjectTypeNm)).
                                       withOrdering(new QueryOrderByField(qvar1.listNm, true)).
                                       execute(psession);
      Set<String> names = new HashSet<String>();                                
      for (NamedListMapping m : namedListMappings) {
         RGFRow row = ws.createRow();
         row.createTextCell(m.getListNm(), textStyle);
         row.createTextCell(m.getBusinessObjectTypeNm(), textStyle);
         row.createTextCell(m.getFieldNm(), textStyle);
         names.add(m.getListNm());
      }
      
      //----------NamedLists----------
      ws = wb.createWorksheet("NamedLists");
      ws.setDefaultColumnWidth(60);
      headerRow = ws.createRow();
      headerRow.createTextCell("namedList.listNm", headerStyle);
      headerRow.createTextCell("namedList.optionCds", headerStyle);
      headerRow.createTextCell("namedList.orderByName", headerStyle);
      
      NamedListQueryVariable qvar2 = NamedList.ALL.result();
      List<QueryOrdering> orderings = new ArrayList(Arrays.asList(new QueryOrderByField(qvar2.listNm, true), new QueryOrderByField(qvar2.rowNo, true)));
      List<QueryConstraint> constraints = isAll ? new ArrayList() : new ArrayList(Arrays.asList(Query.in(qvar2.listNm, names.toArray(new String[names.size()]))));
      NamedListQuery query2 = new NamedListQuery(qvar2, constraints, orderings);
      List<NamedList> namedLists = query2.execute(psession);
      
      Map<String, Boolean> orderByNameMap = new HashMap<String, Boolean>();
      Map<String, StringBuffer> optionsMap = new HashMap<String, StringBuffer>();
      for (NamedList nl : namedLists) {
         String listNm = nl.getListNm();
         StringBuffer options = optionsMap.get(listNm);
         if (options == null) {
            options = new StringBuffer();
            optionsMap.put(listNm, options);
         } else {
            options.append(",");
         }
         options.append(nl.getOptionCd());
         
         if (nl.getRowNo() != null) {
            if (orderByNameMap.get(listNm) == null) {
               orderByNameMap.put(listNm, Boolean.FALSE);
            }
         }
      }
      
      for (String listNm : optionsMap.keySet()) {
         StringBuffer options = optionsMap.get(listNm);
         String optionCds = options.toString();
         RGFRow row = ws.createRow();
         row.createTextCell(listNm, textStyle);
         row.createTextCell(optionCds, textStyle);
         (orderByNameMap.get(listNm) != null) ?  row.createTextCell("FALSE", textStyle) : row.createEmptyCell(textStyle);
      }
            
      //----------NamedListTranslations----------
      ws = wb.createWorksheet("NamedListTranslations");
      ws.setDefaultColumnWidth(40);
      headerRow = ws.createRow();
      headerRow.createTextCell("namedList.listNm", headerStyle);
      headerRow.createTextCell("namedList.optionCd", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.languageCd", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.descTxt", headerStyle);
      for (NamedList nl : namedLists) {
         BusinessObjectList<BusinessObjectLocalization> namedListLocalizations = nl.getLocalizations(psession);
         for (BusinessObjectLocalization nlLocalization : namedListLocalizations) {
            RGFRow row = ws.createRow();
            row.createTextCell(nl.getListNm(), textStyle);
            row.createTextCell(nl.getOptionCd(), textStyle);
            row.createTextCell(nlLocalization.getLanguageCd(), textStyle);
            (nlLocalization.getDescTxt() != null) ?  row.createTextCell(nlLocalization.getDescTxt(), textStyle) : row.createEmptyCell(textStyle);
         }
      }
      
      //----------LinkTypes----------
      ws = wb.createWorksheet("LinkTypes");
      ws.setDefaultColumnWidth(40);
      headerRow = ws.createRow();
      headerRow.createTextCell("linkType.sourceSystemCd", headerStyle);
      headerRow.createTextCell("linkType.linkTypeId", headerStyle);
      headerRow.createTextCell("linkType.businessObjectTypeNm1", headerStyle);
      headerRow.createTextCell("linkType.businessObjectTypeNm2", headerStyle);
      headerRow.createTextCell("linkType.defaultLanguageCd", headerStyle);
      headerRow.createTextCell("linkType.otherLanguageCds", headerStyle);
      headerRow.createTextCell("linkType.name1", headerStyle);
      headerRow.createTextCell("linkType.description1", headerStyle);
      headerRow.createTextCell("linkType.help1", headerStyle);
      headerRow.createTextCell("linkType.name2", headerStyle);
      headerRow.createTextCell("linkType.description2", headerStyle);
      headerRow.createTextCell("linkType.help2", headerStyle);
      List<LinkType> linkTypes = isAll ? LinkType.ALL.execute(psession) : LinkType.object.getEligibleLinkTypes(businessObjectTypeNm, psession);
      for (LinkType l : linkTypes) {
         RGFRow row = ws.createRow();
         row.createTextCell(l.getSourceSystemCd(), textStyle);
         row.createTextCell(l.getLinkTypeId(), textStyle);
         row.createTextCell(l.getBusinessObjectTypeNm1(), textStyle);
         row.createTextCell(l.getBusinessObjectTypeNm2(), textStyle);
         row.createTextCell("en", textStyle);
         row.createTextCell("ar, de, es, fr, it, iw, ja, ko, nl, pl, pt_BR, pt, ru, uk, zh_CN, zh_TW", textStyle);
         row.createEmptyCell(textStyle);
         row.createEmptyCell(textStyle);
         row.createEmptyCell(textStyle);
         row.createEmptyCell(textStyle);
         row.createEmptyCell(textStyle);
         row.createEmptyCell(textStyle);         
      }
      
      //----------LinkTypeTranslations----------
      ws = wb.createWorksheet("LinkTypeTranslations");
      ws.setDefaultColumnWidth(40);
      headerRow = ws.createRow();
      headerRow.createTextCell("linkType.linkTypeId", headerStyle);
      headerRow.createTextCell("linkType.sourceSystemCd", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.languageCd", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.name1Txt", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.desc1Txt", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.help1Txt", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.name2Txt", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.desc2Txt", headerStyle);
      headerRow.createTextCell("businessObjectLocalization.help2Txt", headerStyle);
      for (LinkType l : linkTypes) {
         BusinessObjectList<BusinessObjectLocalization> linkTypeLocalizations = l.getLocalizations(psession);
         for (BusinessObjectLocalization loc : linkTypeLocalizations) {
            Locale currentLocale = ParsingUtils.parseLocale(loc.getLanguageCd());
            RGFRow row = ws.createRow();
            row.createTextCell(l.getLinkTypeId(), textStyle);
            row.createTextCell(l.getSourceSystemCd(), textStyle);
            row.createTextCell(loc.getLanguageCd(), textStyle);
            (l.getLinkTypeNames(currentLocale, psession)[0] != null) ? row.createTextCell(l.getLinkTypeNames(currentLocale, psession)[0], textStyle) : row.createEmptyCell(textStyle);
            (l.getLinkTypeDescriptions(currentLocale, psession)[0] != null) ? row.createTextCell(l.getLinkTypeDescriptions(currentLocale, psession)[0], textStyle) : row.createEmptyCell(textStyle);
            (l.getLinkTypeHelp(currentLocale, psession)[0] != null) ? row.createTextCell(l.getLinkTypeHelp(currentLocale, psession)[0], textStyle) : row.createEmptyCell(textStyle);
            (l.getLinkTypeNames(currentLocale, psession)[1] != null) ? row.createTextCell(l.getLinkTypeNames(currentLocale, psession)[1], textStyle) : row.createEmptyCell(textStyle);
            (l.getLinkTypeDescriptions(currentLocale, psession)[1] != null) ? row.createTextCell(l.getLinkTypeDescriptions(currentLocale, psession)[1], textStyle) : row.createEmptyCell(textStyle);
            (l.getLinkTypeHelp(currentLocale, psession)[1] != null) ? row.createTextCell(l.getLinkTypeHelp(currentLocale, psession)[1], textStyle) : row.createEmptyCell(textStyle);
         }
      }

      //----------Stream .xls file----------
      response.reset();
      CustomObjectMetaData meta = CustomObjectRegistry.getInstance().getByBusinessObjectTypeName(businessObjectTypeNm);
      
      String fileName = (meta == null || meta.getAlias() == null) ? ("All".equals(businessObjectTypeNm) ? "All_Objects" : businessObjectTypeNm) : meta.getAlias();
      fileName = fileName + "_" + ServiceLocator.getContentId() + (com.sas.oprisk.framework.server.ConfigDataProperties.getExportExcelUseXlsxFormat() ? ".xlsx" : ".xls");
       
      response.setContentType(com.sas.oprisk.framework.server.ConfigDataProperties.getExportExcelUseXlsxFormat() ? "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet" : "application/vnd.ms-excel"); // Set up mime type for .xls
      //String fileName = businessObjectTypeNm + ".xlsx";
      //response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"); // Set up mime type for .xlsx
      response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
      
      OutputStream outStream = response.getOutputStream();
         
      try {
         wb.write(outStream);
         response.flushBuffer();
         outStream.flush();
      } catch (Exception e) {
         e.printStackTrace();
      } finally {
         outStream.close();
      }
   }   
}

out.println("</form></body></html>");