

import groovy.xml.MarkupBuilder

import org.apache.commons.lang.StringEscapeUtils
import org.apache.commons.lang.StringUtils

import com.sas.oprisk.framework.server.util.CodeEnumerationUtils
import com.sas.oprisk.server.ApplicationProperties
import com.sas.oprisk.server.services.ServiceLocator
import com.sas.oprisk.server.util.LabelValueUtils
import com.sas.oprisk.framework.server.util.LabelValueBean
import com.sas.oprisk.server.CustomObjectMetaData
import com.sas.oprisk.server.util.BusinessObjectUtils
import com.sas.oprisk.server.CustomObjectRegistry


def getFilteredValues = {enumClass->
   def ps =ServiceLocator.getPersistenceSession()
   def locale =ServiceLocator.getLocale()

   def list = LabelValueUtils.getLabelValueCodeEnumerationBeans(CodeEnumerationUtils.getAllValues(Class.forName(enumClass)),locale);
   def filtered = [];
   for (LabelValueBean lv : list)
   {
      if (enumClass.equals("com.sas.oprisk.server.BusinessObjectDescriptor") && lv.getValue().startsWith("CustomObject")) {
         Integer index = Integer.valueOf(lv.getValue().substring("CustomObject".length(), lv.getValue().length()));
         CustomObjectMetaData meta = CustomObjectRegistry.getInstance().getByIndex(index);
         if (meta != null && BusinessObjectUtils.isEnabled(meta.getClassObject())) {
            filtered.add(lv);
         }
      }
      else if (enumClass.equals("com.sas.oprisk.server.base.ScreenDefinitionTypeCd") && lv.getValue().startsWith("CustomObject") && lv.getValue().endsWith("s")) {
         Integer index = Integer.valueOf(lv.getValue().substring("CustomObject".length(), lv.getValue().length() - 1));
         CustomObjectMetaData meta = CustomObjectRegistry.getInstance().getByIndex(index);
         if (meta != null && BusinessObjectUtils.isEnabled(meta.getClassObject())) {
            filtered.add(lv);
         }
      }
      else if (enumClass.equals("com.sas.oprisk.server.base.ScreenDefinitionTypeCd") && lv.getValue().startsWith("CustomObject") && lv.getValue().endsWith("c")) {
         Integer index = Integer.valueOf(lv.getValue().substring("CustomObject".length(), lv.getValue().length() - 1));
         CustomObjectMetaData meta = CustomObjectRegistry.getInstance().getByIndex(index);
         if (meta != null && BusinessObjectUtils.isEnabled(meta.getClassObject())) {
            filtered.add(lv);
         }
      }
      else if (enumClass.equals("com.sas.oprisk.server.base.ScreenDefinitionTypeCd") && lv.getValue().startsWith("CustomObject")) {
         Integer index = Integer.valueOf(lv.getValue().substring("CustomObject".length(), lv.getValue().length()));
         CustomObjectMetaData meta = CustomObjectRegistry.getInstance().getByIndex(index);
         if (meta != null && BusinessObjectUtils.isEnabled(meta.getClassObject())) {
            filtered.add(lv);
         }
      }
      else {
         filtered.add(lv);
      }
   }
   
   return filtered;
}


def getCodeTable = {enumClass->
   def ps =ServiceLocator.getPersistenceSession()
   def locale =ServiceLocator.getLocale()
   def writer = new StringWriter()

   def filtered = getFilteredValues(enumClass);

   filtered.sort().each{
      writer.append( " ${it.getValue()} = ${it.getLabel()} ")
      writer.append('<br/>')
   }
   return writer
}

def getCodeOnly = {enumClass->
   def locale =ServiceLocator.getLocale()
   def writer = new StringWriter()
   def x= new HashMap();
   
   def filtered = getFilteredValues(enumClass);
      
   filtered.each{
       x.put(it.getValue(), it.getLabel())
       
   }
   x.sort{ it.value.toLowerCase() }
   Map sorted = x.sort { a, b -> a.value <=> b.value }
   return sorted.keySet().join(",");
}

def enums=['com.sas.oprisk.server.BusinessObjectDescriptor','com.sas.oprisk.server.Dimension','com.sas.oprisk.server.DimensionKind','com.sas.oprisk.server.base.AttachmentTypeCd','com.sas.oprisk.server.base.AuxOpDim1AssocTypeCd','com.sas.oprisk.server.base.AuxOrgDim1AssocTypeCd','com.sas.oprisk.server.base.AuxRiskDim1AssocTypeCd','com.sas.oprisk.server.base.AuxRmgDim1AssocTypeCd','com.sas.oprisk.server.base.AuxRptDim1AssocTypeCd','com.sas.oprisk.server.base.BaselBlAssocTypeCd','com.sas.oprisk.server.base.BaselRiskCatAssocTypeCd','com.sas.oprisk.server.base.BlAssocTypeCd','com.sas.oprisk.server.base.CauseAssocTypeCd','com.sas.oprisk.server.base.ControlAssocTypeCd','com.sas.oprisk.server.base.CostCenterAssocTypeCd','com.sas.oprisk.server.base.CustFieldTypeCd','com.sas.oprisk.server.base.DirectionCd','com.sas.oprisk.server.base.FsiAssocTypeCd','com.sas.oprisk.server.base.GeographyAssocTypeCd','com.sas.oprisk.server.base.LegalOrgAssocTypeCd','com.sas.oprisk.server.base.ManagementOrgAssocTypeCd','com.sas.oprisk.server.base.NfiTypeAssocTypeCd','com.sas.oprisk.server.base.ObjectiveAssocTypeCd','com.sas.oprisk.server.base.ProcessAssocTypeCd','com.sas.oprisk.server.base.ProductAssocTypeCd','com.sas.oprisk.server.base.ProjectAssocTypeCd','com.sas.oprisk.server.base.ResourceAssocTypeCd','com.sas.oprisk.server.base.RiskCatAssocTypeCd','com.sas.oprisk.server.base.ScreenDefinitionTypeCd','com.sas.oprisk.server.base.StdProcAssocTypeCd','com.sas.oprisk.server.behavior.GlobalPermission']
def list=[]
enums.collect {
   def name = StringUtils.uncapitalize(it.substring(it.lastIndexOf(".")+1))
   def className=it
   list << [className :className, name: name, values: getCodeTable(className),codes:getCodeOnly(className)]

}





//--  Display them in a table

def createTable(data) {
   def locale =ServiceLocator.getLocale()
   def writer = new StringWriter()
   def bundle= ApplicationProperties.getBundle(locale);
   def mb= new MarkupBuilder(writer)
   mb.html{
      head{
         link(rel:"stylesheet", type:"text/css", href:"docs/Documentation.css")
         style(type:"text/css", '''
              table#mrm_list_enums_table {
                   border-collapse: collapse;
                   table-layout: fixed;
                   width:100%;
                   word-wrap:break-word;
               }
               
               table#mrm_list_enums_table,#mrm_list_enums_table th,#mrm_list_enums_table td {
                   border: 1px solid gray;
                  padding:10px;
               }
             ''')
         script(type:"textjavascript", '''
                   
                '''
         )

      }
      body{
         table(id:'mrm_list_enums_table',style:'text-align:left') {
            tr {
              th(bundle.getString("enums.name.txt"))
              th(bundle.getString("enums.configProperty.txt"))
              th(bundle.getString("enums.sample.txt"))
            }
            data.each { row ->
               tr() {
                  td(row.name)
                  td(width:'40%', "monitor.${row.name}.displayOrder = ${row.codes}")
                  td(row.values.toString())
               }



            }
         }
      }
   }
   writer.toString()
}

println StringEscapeUtils.unescapeHtml(createTable(list))
