import com.sas.services.webdav.*
import com.sas.services.webdav.DAVResource
import org.apache.tools.ant.*
import org.apache.tools.ant.taskdefs.*
import org.apache.commons.httpclient.URI
import org.apache.commons.httpclient.URIException

def debug = false
def r

// load incoming attributes
def _davServerUrl = attributes["url"]
def _serverUserId = attributes["user"]
def _serverUserPw = attributes["password"]

def _outputproperty = attributes["outputproperty"]
def _resultproperty = attributes["resultproperty"]

def log = new ReturnProperty(_outputproperty, _resultproperty, project)

// echo incoming attributes
Utility.antEcho(project, "url = \"${_davServerUrl}\"", debug)
Utility.antEcho(project, "userid = \"${_serverUserId}\"", debug)

Utility.antEcho(project, "_outputproperty = \"${_outputproperty}\"", debug)
Utility.antEcho(project, "_resultproperty = \"${_resultproperty}\"", debug)

try{
    //Utility.antEcho(project, "setting connnectionInfo", debug)
	// connect to the DAV server
	ConnectionInfo c = new ConnectionInfo(_davServerUrl, _serverUserId, _serverUserPw);
    //Utility.antEcho(project, "connnectionInfo set " +c.toString() + ">>"+c.getUrl() + c.getUser() + c.getPW(), debug)
    //Utility.antEcho(project, "creating DAVResource object", debug)
	r = new DAVResource(c)
    //Utility.antEcho(project, "DAVResource object Created", debug)
	Utility.antEcho(project, "DAVResource initial path: "  + r.getPath(), debug)
}
catch(Exception e){
	Utility.antEcho(project, "DAVResource: "  + e.getMessage(), debug)
	log.returnProperty("1", "Error connecting to the Content Server: " +
			"\n" + e.getMessage() +
			"\n  url: ${_davServerUrl}" +
			"\n  userid: ${_serverUserId}."
			);
	return;
}

String contentType = "";
	
// detect which content. CAM, GCM or QAM.  Look at {xyz}_version.properties.  {xyz} is acronym.
try{
	// root of any dav folders being moved
	def davPathRoot = "/SASContentServer/repository/default/sasdav/"
	
	// path of the projects to move and set ACL from
	String rgf73SourceParent = davPathRoot + "Products/SASRiskGovernanceFramework/RiskGovFrwkMid-Tier/"
	String rgf73TargetParent = null;

	boolean camContentExists = false;	
	boolean gcmContentExists = false;
	boolean qamContentExists = false;

	
	List children = r.getChildrenList(rgf73SourceParent)
	for (Iterator iterator = children.iterator(); iterator.hasNext();){
		String path = (String) iterator.next()
		
		if(path.equals(rgf73SourceParent + "cam_version.properties")){
			Utility.antEcho(project, "davContentDetected: CAM" , debug)
			camContentExists = true;
			contentType = "cam";
		}
		else if(path.equals(rgf73SourceParent + "gcm_version.properties")){
			Utility.antEcho(project, "davContentDetected: GCM" , debug)
			gcmContentExists = true;
			contentType = "gcm";
		}
		else if(path.equals(rgf73SourceParent + "qam_version.properties")){
			Utility.antEcho(project, "davContentDetected: QAM " , debug)
			qamContentExists = true;
			contentType = "qam";
		}
		else{
			Utility.antEcho(project, "No 7.3 Dav Content Detected" , debug)
		}
	}
	
	// If more than one detected set return property and exit.
	if (((camContentExists) && (gcmContentExists)) || 
	   ((camContentExists) && (qamContentExists)) ||
	   ((gcmContentExists) && (qamContentExists)) )
	{
		Utility.antEcho(project, "DAVResource: "  + "More than one content package exists.", debug)
		log.returnProperty("1", "More than one content package exists. " +
				"\n  url: ${_davServerUrl}" +
				"\n  userid: ${_serverUserId}."
				);
		return;
	}
	
	// If one content package exists, move it. Otherwise do nothing.
	if ((camContentExists) || (gcmContentExists) || (qamContentExists))
	{
		

	rgf73TargetParent = rgf73SourceParent + contentType + "/";
	
	// create folder 
	r.setPath(rgf73SourceParent + contentType + "/")
    r.makeCollection()
	Utility.antEcho(project, "creating new dav resource Path : "  + rgf73SourceParent + contentType + "/", debug)
	
	Acl moveTargetAcl
	
	children = r.getChildrenList(rgf73SourceParent)
	for (Iterator iterator = children.iterator(); iterator.hasNext();){
		String path = (String) iterator.next()
		String targetPath = path.replaceAll(rgf73SourceParent, rgf73TargetParent)
		
		if( (path.equals(rgf73SourceParent)) || (path.equals(rgf73TargetParent)) ||
			(path.equals(rgf73SourceParent + "rgf-system/")) ) {
		}
		else{
			Utility.antEcho(project, "davPathMove: "  + path + " -> " + targetPath, debug)
			
			r.setPath(path)
			r.move(targetPath)
		}
	
	    // get ACL of the target path to be set on children after moves
	    r.setPath(rgf73TargetParent);
	    moveTargetAcl = r.getAcl()
		
	    // encode the principal URI for each Ace
	    for(ace in moveTargetAcl.getAceList()){
		    Principal p = ace.getPrincipal()
		
		    if(Principal.REFERENCE == p.getType()){
			    String ref = p.getRef()
			
			    try{
				    Utility.antEcho(project, "Encoding URI on ref: "  + ref, false)
				    ace.setPrincipal(new Principal(Principal.REFERENCE, new org.apache.commons.httpclient.URI(ref, false).toString()))
			    }
			    catch (URIException u){
			    	// ignore exception - use existing principal in ace
			    	Utility.antEcho(project, "Handled URIException on ref: "  + ref, false)
			    }
		    }
	    }
	
	}
	
	Utility.antEcho(project, "setAclOnAllChildren: "  + rgf73TargetParent, debug)
	setAclOnAllChildren(r, rgf73TargetParent, moveTargetAcl)
	
	}
}
catch(Exception e){
	Utility.antEcho(project, "Error: "  + e.getMessage(), debug)
	log.returnProperty("1", "Error updating the Content Server paths: " +
			"\n" + e.getMessage() +
			"\n  url: ${_davServerUrl}" +
			"\n  userid: ${_serverUserId}."
			);
	return;
}

Utility.antEcho(project, "Successful end of script.", debug)
//log.returnProperty("0", "Successful end of script.")
log.returnProperty("0", contentType)


// declaring some class objects and functions being used in the script
void setAclOnAllChildren(r, parentPath, parentAcl){
	List children = r.getChildrenList(parentPath)
	for (Iterator iterator = children.iterator(); iterator.hasNext();){
		String path = (String) iterator.next()
		
		if(path.equals(parentPath)){
		}
		else{
			Utility.antEcho(project, "setAclOnChild: "  + path, false)
			
			r.setPath(path)
			r.setAcl(parentAcl)
			
			setAclOnAllChildren(r, path, parentAcl)
		}
	}
}
