// (c) 2015, SAS Institute Inc.
sap.ui.define([
    "jquery.sap.global",
    "./_ObjectInspectorNamedButton"
], function(jQuery, ObjectInspectorNamedButton) {
    "use strict";

    // icon path constants
    var SELECTED_ICON_PATH = sas.icons.HC.FAVORITEDINDICATOR,
        NOT_SELECTED_ICON_PATH = sas.icons.HC.FAVORITENOTSELECTED;

    /**
     * Constructor for a new FavoritesButton.
     *
     * Handles state with the "selected" property.
     * Depending on state, a different icon will show.
     *
     * @class
     * @name sas.hc.m.objectInspector.FavoritesButton
     * @extends sas.hc.m.objectInspector.ObjectInspectorNamedButton
     * @version 800001.10.0.20241202143000_f0htmcm80p
     * @author Jonathan Brink
     * @constructor
     * @public
     */
    var FavoritesButton = ObjectInspectorNamedButton.extend("sas.hc.m.objectInspector.FavoritesButton", /** @lends sas.hc.m.objectInspector.FavoritesButton.prototype */ {
        metadata: {
            library: "sas.hc.m",
            properties: {
                selected: {type: "boolean", defaultValue: false, group: "Appearance"}
            }
        },
        renderer: {
        }
    });

    /**
     * init lifecycle callback.
     * @private
     */
    FavoritesButton.prototype.init = function() {
        this.setType(ObjectInspectorNamedButton.FAVORITES);

        // set default icon
        this.setSelected(!!this.getSelected());

        this.attachPress(jQuery.proxy(this.toggleSelected, this));
    };

    /**
     * "selected" property setter override
     * prevent invalidation
     * manages icon property value
     * @param {boolean} bSelected value of selected
     * @public
     */
    FavoritesButton.prototype.setSelected = function(bSelected) {
        this.setProperty('selected', bSelected, true);
        if (bSelected === true) {
            this.setIcon(SELECTED_ICON_PATH);
        } else {
            this.setIcon(NOT_SELECTED_ICON_PATH);
        }
    };

    /**
     * toggle the "selected" property
     * @public
     */
    FavoritesButton.prototype.toggleSelected = function() {
        var bNewSelectedState = true;
        if (this.getSelected() === true) {
            bNewSelectedState = false;
        }
        this.setSelected(bNewSelectedState);
    };

    return FavoritesButton;
}, /* bExport= */ true);
