/**
 * Provides helper sas.hc.ui.commons.LargeTreeKeyboardDelegate
 *
 * @class Extension for sas.hc.ui.commons.LargeTreeKeyboardDelegate which handles keyboard related things.
 *
 * @extends sas.hc.ui.table.TableKeyboardDelegate
 * @author SAS
 * @version 800001.10.0.20241202143000_f0htmcm80p
 * @constructor
 * @private
 */

sap.ui.define(['sas/hc/ui/table/TableKeyboardDelegate', 'sas/hc/ui/table/TableUtils'],
    function(TableKeyboardDelegate, TableUtils) {
        "use strict";

        var LargeTreeKeyboardDelegate = TableKeyboardDelegate.extend("sas.hc.ui.commons.LargeTreeKeyboardDelegate", {});

        /*Opens the tree node when click on right arrow key and closes it when click on left arrow key*/
        LargeTreeKeyboardDelegate.prototype.onkeyup = function(oEvent) {
            if (oEvent.keyCode === jQuery.sap.KeyCodes.ARROW_RIGHT) {
                if (TableUtils.toggleGroupHeader(this, oEvent.target, true)) {
                    oEvent.preventDefault();
                    return;
                }
            } else if(oEvent.keyCode === jQuery.sap.KeyCodes.ARROW_LEFT) {
                if (TableUtils.toggleGroupHeader(this, oEvent.target, false)) {
                    oEvent.preventDefault();
                    return;
                }
            }
        };

        /*Navigate using up/down arrow key and press Enter key selects row*/
        LargeTreeKeyboardDelegate.prototype.onkeydown = function(oEvent) {
            if (oEvent.keyCode === jQuery.sap.KeyCodes.ENTER) {
                oEvent.preventDefault(); // To prevent the browser window to scroll down.
                oEvent.stopImmediatePropagation(true);
                this._onSelect(oEvent);
            }
        };

        return LargeTreeKeyboardDelegate;
    });
