jQuery.sap.declare("sas.risk.common.widgets.RichText");
jQuery.sap.require("sas.risk.common.widgets.RichTextToolbar");
jQuery.sap.require("sas.hc.ui.core.HTMLRenderer");

sas.hc.ui.core.HTML.extend("sas.risk.common.widgets.RichText", {
    metadata:{
        properties:{
      	  value:{type:"string"},
		     sanitizeContent : {type : "boolean", group : "Misc", defaultValue : true},
      	  editorWidth : {type : "sap.ui.core.CSSSize"},
  			  editorHeight : {type : "sap.ui.core.CSSSize"},
  			  language : {type: "string"},
  			  toolbarLevel : {type: "sas.risk.common.widgets.RichTextToolbarType", defaultValue: sas.risk.common.widgets.RichTextToolbarType.ADVANCE},
  			  enabled : {type: "boolean", defaultValue: true},
  			  addSavePlugin : {type: "boolean", defaultValue: false},
  			  inline : {type: "boolean"},
			  // rfw needs because we use HTML templates as input to the ckEditor -- SRINI others may not, therefore, setting default to false
			  allowedContent:{type:"boolean", defaultValue: false}
        },
        events:{
      	  save: {},
      	  change: {}
        }
    },
    renderer: "sas.hc.ui.core.HTMLRenderer"
});

sas.risk.common.widgets.RichText.prototype.init = function(){
	this.initializeContent();
	this.attachAfterRendering($.proxy(this.initializeEditor,this));
	//Listen to the destroyEditor event and destroy the editor instance accordingly. Otherwise many instances of CKEDITOR will be live.
	this.attachEvent("destroyEditor",function(e){
		this.editorInstance.destroy(true);
	});
};
sas.risk.common.widgets.RichText.prototype.initializeContent = function(){
	var tempContent = this.getValue()!==undefined ? this.getValue() : "";
	this.mainContent = DOMPurify.sanitize(tempContent,{KEEP_CONTENT: false});
	if(this.getInline()){
		this.setContent('<div class="riskRichText"><div id="'+this.getId()+'_inner" contenteditable="true" style="min-width:100px; min-height:300px; border: 1px solid #b4b4b4; padding:5px 8px;">'+this.mainContent+'</div></div>');
	}else{
		this.setContent('<div class="riskRichText"><div id="'+this.getId()+'_inner" style="min-width:100px; min-height:300px">'+this.mainContent+'</div></div>');
	}
	
};
sas.risk.common.widgets.RichText.prototype.getFocusDomRef=function(){/*teach it what exactly is the dom piece to focus on*/
	return this.editorInstance;
};

//Ckeditor4 
//sas.risk.common.widgets.RichText.prototype.initializeEditor = function(){
//	this.initializeContent();
//	var toolbarLevel=this.getToolbarLevel();
//	var customToolbar=sas.risk.common.widgets.RichTextToolbar.getToolbar(toolbarLevel);
//	var allowedContent=this.getAllowedContent();	// SRINI
//	var langCode=this.getLanguage();
//	if(langCode){
//		langCode=langCode.replace("_","-");
//	}
//	var instanceConfig={
//		width: this.getEditorWidth(),
//		height: this.getEditorHeight(),
//		language: langCode,
//		toolbar: customToolbar,
//		readOnly: !this.getEnabled(),
//		resize_dir: "both",		// rfw does not need, not sure what none is
//		allowedContent: allowedContent	// rfw needs because we use HTML templates as input to the ckEditor -- SRINI
//	};
//	if(!this.getAddSavePlugin()){
//		instanceConfig.removePlugins='save';
//	}
//	var editorId=this.getId()+"_inner";
//	//If current instance exists then destroy first - workaround since onAfterRendering is fired twice and only in 2nd time around the dom element exists
//	if(CKEDITOR.instances[editorId]){
//		CKEDITOR.instances[editorId].destroy(true);
//	}
//	if(this.getInline()){
//		CKEDITOR.inline(editorId, instanceConfig);
//	}else{
//		CKEDITOR.replace(editorId, instanceConfig);
//	}
//	this.editorInstance=CKEDITOR.instances[editorId];
//	this.editorInstance.on("change",$.proxy(this.handleChange,this));
//	this.editorInstance.on("instanceReady",$.proxy(this.handleEditorInstanceReady,this));
//	this.editorInstance.on('afterCommandExec', $.proxy(this.handleAfterCommandExec,this));
//	this.overrideEditorBehaviors();
//
//};

//CKEditor5
sas.risk.common.widgets.RichText.prototype.initializeEditor = function(){
	var that= this;
	this.initializeContent();
	var toolbarLevel=this.getToolbarLevel();
	var customToolbar=sas.risk.common.widgets.RichTextToolbar.getToolbar(toolbarLevel);
	var allowedContent=this.getAllowedContent();	// SRINI
	var langCode=this.getLanguage();
	if(langCode){
		langCode=langCode.replace("_","-");
	}

	this.editorId=this.getId()+"_inner";		
	ClassicEditor
	  .create(document.querySelector( "#"+ this.editorId ),{
		  language: langCode,
		  licenseKey: "TlVlT2lWWUF3Rmt2SlJ6ZDRjVjhKZWtFWnNXYmFWZE9ic3N0OHkwWkFQOHNxUnVzSkxCZUg3cnI3V24ybTdvcUNXWngxeEFBbWpVPS1NakF5TkRBME1qYz0=",
		  height: this.getEditorHeight()?this.getEditorHeight():"300px",
		  removePlugins: this.getRemovePlugins(this.getToolbarLevel()),
		  toolbar: this.getEnabled()?customToolbar:[],			  
		  isReadOnly: !this.getEnabled(),
		  allowedContent: allowedContent,	// rfw needs because we use HTML templates as input to the ckEditor -- SRINI
		  link: {
			   addTargetToExternalLinks: true,
			// Let the users decide whether to open link in a new tab
				   decorators: {
				     openInNewTab: {
				        mode: 'manual',
				        label: 'Open in a new tab',
				        attributes: {
				          target: '_blank',
				          rel: 'noopener noreferrer'
				        }
				      }
				    }
				},
	  })
	  .then( function(editor) {
		  that.editorInstance = editor;	
		  //setData specifically to this instance's content
		  that.editorInstance.setData(that.mainContent);		  		 
		  //Display all available toolbar items
		  console.log("Available toolbar items:\n", Array.from( editor.ui.componentFactory.names() ) );
		  //Prevent showing a warning notification when user is pasting a content from MS Word or Google Docs.
		  window.preventPasteFromOfficeNotification = false;
		  that.editorInstance.ui.view.editable.element.style.height=that.getEditorHeight()?that.getEditorHeight():"300px";	
		  that.handleEditorInstanceReady();		 
		  that.editorInstance.model.document.on("change",$.proxy(that.handleChange,that));	
		  //Make sure the editor area focused height correct
		  that.editorInstance.editing.view.document.on('change:isFocused', function() {
			  that.editorInstance.editing.view.change(function (writer) {
				  writer.setStyle('height', that.getEditorHeight()?that.getEditorHeight():"300px", that.editorInstance.editing.view.document.getRoot());
		      });
		    });
	  })
	  .catch(function(error) {
	    console.log(error);
	  });	
};

sas.risk.common.widgets.RichText.prototype.getRemovePlugins = function(toolbarLevel){
	var removePlugins=['outdent','Indent'];
	var pluginsNotInBasic=['SpecialCharacters','SpecialCharactersEssentials','Link','Table','HorizontalLine'];
	if (toolbarLevel==='BASIC'){		
			return removePlugins.concat(pluginsNotInBasic);
	}else
		return removePlugins;	
};

sas.risk.common.widgets.RichText.prototype.getData = function(){
	return this.editorInstance.getData() || "";
};

sas.risk.common.widgets.RichText.prototype.handleChange = function(e){
	var newData=this.editorInstance.getData();
	this.fireChange({data:newData});
};
sas.risk.common.widgets.RichText.prototype.handleEditorInstanceReady = function(event){
	if(!this.getEnabled()){
		//$(this.getDomRef()).find('[id^=cke_][id$=_top]').css("display","none");//hide the toolbar in readonly mode ckeditor4
		this.editorInstance.enableReadOnlyMode(this.editorId); // Enable readyOnlyMode in CKEditor5
	}
	var that=this;
	//Following workaround is for the links inside the editor to open in a new window
	//Otherwise if user wants to open the links that are inside the editor, they need to use cntrl + right click -> open
	//CKEditor5 doesn't use iframe
//	$('iframe').contents().click(function (e) {
//   	var url;
//   	url = e.target.href;
//   	if (url !== undefined && url !== "") {
//   		window.open(url);
//   	}
//   });
	$(this.getDomRef()).find('[id^=cke_][id$=_contents]').click(function (e) {
   	var url;
   	url = e.target.href;
   	if (url !== undefined && url !== "") {
   		window.open(url);
   	}
   });
};
//sas.risk.common.widgets.RichText.prototype.handleAfterCommandExec = function(event){
//	var commandName = event.data.name;
//	if (commandName === 'source'){
//		//When in source view make the content read-only. It helps in avoiding any possible security violations.
//		this.editorInstance.viewingSource=this.editorInstance.viewingSource ? false : true;
//		if(this.editorInstance.viewingSource){
//			var containerDiv=$(this.getDomRef()).find('[id^=cke_][id$=_contents]');
//			var textArea=containerDiv.children('textarea');
//			textArea.prop('disabled',true);
//		}
//	}
//};
//sas.risk.common.widgets.RichText.prototype.overrideEditorBehaviors = function(){
//	this.editorInstance.addCss( '.cke_editable p { margin: 0px }' );
//	this.editorInstance.model.document.on('dialogDefinition', function(ev) {
//   	var dialogName = ev.data.name;
//	   var dialogDefinition = ev.data.definition;
//
//	   if (dialogName === 'link') {
//	   	//Limit the linktype drop down options to only URL
//	   	var infoTab = dialogDefinition.getContents('info');
//	      var linkOptions = infoTab.get('linkType');
//	      linkOptions['items'] = [['URL', 'url']];
//
//	      //Limit the tabs to info and target only
//	      var info=dialogDefinition.getContents("info");
//	      var target=dialogDefinition.getContents("target");
//	      dialogDefinition.contents=[info, target];
//
//	      //Limit the protocols to http:// and https:// only
//	      info.get("protocol").items=[['http://', 'http://'],['https://', 'https://']];
//	   }
//   });	
//};
