jQuery.sap.declare("sas.risk.common.widgets.expreditor.ExprEditorHelper");
jQuery.sap.require("sas.hc.ui.core.HTML");

sas.risk.common.widgets.expreditor.ExprEditorHelper={
	createExprHelper:function(ee){
		var splitterLayout=new sas.hc.ui.layout.SplitterLayoutData({
			size: "auto"
		});
		ee.helper=new sas.hc.m.IconTabBar({
			layoutData:splitterLayout
		}).addStyleClass("ExprEditor-helper-tabs");
		_.each(["variables","operators"],function(type){
			ee.helper.addItem(sas.risk.common.widgets.expreditor.ExprEditorHelper.createExprHelperCategory(type,ee));
		});
		var splitterLayoutData=new sas.hc.ui.layout.SplitterLayoutData({
			collapsible:true,
			size: "300px"
	    });
		ee.usage=sas.risk.common.widgets.expreditor.ExprEditorHelper.createExprHelperUsage(ee);
	    var splitter=new sas.hc.ui.layout.Splitter({
	        contentAreas: [ee.helper, ee.usage],
	        orientation: sap.ui.core.Orientation.Vertical,
	        layoutData:splitterLayoutData
	    }).addStyleClass("ExprEditor-helper-splitter");
	    return splitter;
	},
	createExprHelperCategory:function(type,ee){
		var modelId=sas.risk.common.widgets.expreditor.ExprEditorHelper.getModelId(ee);
		var search=new sas.hc.m.SearchField({
			value:{
				path:ee.getModelId()+">/search",
				formatter:function(val){
					sas.risk.common.widgets.expreditor.ExprEditorHelper.generateModel(val,type,ee);
					return val;
				}
			},
			search:$.proxy(sas.risk.common.widgets.expreditor.ExprEditorHelper.onSearch,ee,type),
	        placeholder: ee.getLabel("filter")
	    });
		var tree=new sas.hc.ui.commons.Tree({
			width:"100%",
			height:"auto",
			showHeader:false,
			showHorizontalScrollbar:false,
			selectionMode:sap.ui.commons.TreeSelectionMode.Single,
			nodes:{
				path:modelId+'>/'+type,
				template:new sas.hc.ui.commons.TreeNode({
					text:"{"+modelId+">label}",
					icon: "{"+modelId+">icon}",
					expanded:true
				}),
				parameters:{
					arrayNames:['members']
				}
			},
            select: $.proxy(sas.risk.common.widgets.expreditor.ExprEditorHelper.onItemSelect,ee)
		}).addStyleClass("ExprEditor-helper-tree");
		tree.onAfterRendering=function(){
			//if (ee.treeRendered === undefined) {
				sas.hc.ui.commons.Tree.prototype.onAfterRendering.apply(this,arguments);
            //}
			$treeNodes=this.$().find("ul li.sapUiTreeNode:not(.sapUiTreeNodeCollapsed):not(.sapUiTreeNodeExpanded)");
			$treeNodes.dblclick($.proxy(sas.risk.common.widgets.expreditor.ExprEditorHelper.onItemAdd,ee));
			// $treeNodes.click($.proxy(sas.risk.common.widgets.expreditor.ExprEditorHelper.onItemSelect,ee));
            if (ee.selectedNode !== undefined) {
                tree.setSelection(ee.selectedNode);
                ee.selectedNode.focus();
                ee.selectedNode.scrollIntoView();
            } else if ($treeNodes.length > 0){
                tree.setSelection(sap.ui.getCore().byId($treeNodes[0].id));
            }
			//ee.treeRendered=true;
		};
		var splitterLayoutData=new sas.hc.ui.layout.SplitterLayoutData({
	        size: "300px",
	        collapsible: true,
	        hideCollapseIcon:false
	    });
		var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
			flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
			items:[search,tree],
			layoutData:splitterLayoutData
		}).addStyleClass("ExprEditor-helper-category");
		var tab=new sas.hc.m.IconTabFilter({
			text:ee.getLabel(type),
			content:[layout]
		});
		ee.tree=tree;
		return tab;
	},
	createExprHelperUsage:function(ee){
		var splitterLayoutData=new sas.hc.ui.layout.SplitterLayoutData({
	        size: "40px",
	        collapsible:true
	    });
		var html=new sas.hc.ui.core.HTML({
			content:"",
			layoutData:splitterLayoutData
		});
		return html;
	},
	getModelId:function(ee){
		return ee.getModelId()+"-helper";
	},
	generateModel:function(search,type,ee){
		search=search.trim().toUpperCase();
		var meta=ee.getMeta();
		var folders={};
		var singles=[];
		_.each(meta[type].items,function(item){
			if (typeof item.label !== 'undefined'){  // we don't show items without labels
				var match=item.label.toUpperCase().indexOf(search)!==-1;
				if(!match){
					match=_.find(item.aliases||[],function(alias){
						return alias.toUpperCase().indexOf(search)!==-1;
					});
				}
				if(match||search===""){
					var cat=_.find(meta[type].categories,function(c){
						return c.members.indexOf(item.id)!==-1;
					});
					if(cat){
						var catIcon=cat.icon&&sas.icons.HC[cat.icon.toUpperCase()]||sas.icons.HC.DATAITEMS;
						item.icon=item.icon||catIcon;
						var folder=folders[cat.label];
						if(!folder){
							folder={
									label:cat.label,
									members:[]
							};
							folders[cat.label]=folder;
						}
						folder.members.push(item);
					}else{
						singles.push(item);
					}
				}
			}
		});
		var unsorted=singles.concat(_.values(folders));
		var sorted=_.sortBy(unsorted,"label");
		var modelId=sas.risk.common.widgets.expreditor.ExprEditorHelper.getModelId(ee);
		var model=ee.getModel(modelId);
		if(!model){
			model=new sas.hc.ui.model.JSONModel({variables:[],operators:[]});
			ee.setModel(model,modelId);
		}
		model.setProperty("/"+type,sorted);
	},
	onSearch:function(type,event){
		var query=event.getParameter("query");
		sas.risk.common.widgets.expreditor.ExprEditorHelper.generateModel(query,type,this);
	},
	onItemSelect:function(event){
		var entry;
		if (event.getParameter && event.getParameter("node") && event.getParameter("nodeContext")) {
			entry = event.getParameter("nodeContext").getObject();
		} else{
            var node= sap.ui.getCore().byId($(event.target).closest("li")[0].id);
			entry=node.getBindingContext(sas.risk.common.widgets.expreditor.ExprEditorHelper.getModelId(this)).getObject();
		}
		var help=entry.help||this.getLabel("nohelp");
		this.usage.setContent(help);
	},
	onItemAdd:function(event){
		var node=sap.ui.getCore().byId($(event.target).closest("li")[0].id);
		var entry=node.getBindingContext(sas.risk.common.widgets.expreditor.ExprEditorHelper.getModelId(this)).getObject();
		var format=entry.format||{text:(entry.keyToken||entry.id)};
		this.selectedNode = node;
		this.cm.replaceSelection(format.text);
        
		this.cm.focus();
		if(_.has(format,"cursor")){
			var cursor = this.cm.getCursor();
		    var cursorLine = cursor.line;
		    var cursorCh = cursor.ch;
		    this.cm.setCursor({line: cursorLine , ch : cursorCh - format.text.length + format.cursor});
		}
	}
};
