(function(mod) {
	if (typeof exports == "object" && typeof module == "object") // CommonJS
		mod(require("../lib/codemirror"));
	else if (typeof define == "function" && define.amd) // AMD
		define([ "../../lib/codemirror" ], mod);
	else
		// Plain browser env
		mod(CodeMirror);
})(function(CodeMirror) {
	"use strict";

	CodeMirror.defineMode("datadef", function(config, parserConfig) {
		var meta=parserConfig.meta||{variables:{items:[]},operators:{items:[]}};
		function map(array,comp) {
			var keys = {};
			for (var i = 0; i < array.length; ++i) {
				if(!comp || comp(array[i])){
					keys[array[i].id] = array[i];
				}
			}
			return keys;
		};
		var operators = map(meta.operators.items,function(i){return !i.isFunction;});//css:operator
		var functions = map(meta.operators.items,function(i){return i.isFunction;});// css:keyword
		var variables = map(meta.variables.items);// css variable
		// css string
		// css number
		// css variable
		var numberRx = /[-]?((\d+(\.\d*)?)|(\.\d+))/;
		var rxs=buildRegExps(operators);
		var wordRx = rxs[0];
		var operatorRx = rxs[1];

		function tokenBase(stream, state) {
			var now={};
			// https://rndjira.sas.com/browse/RGFRMWRK-412..
			// we will match operator first.. var names containing IN, AND, OR (operators basically will be matched here)
			if (stream.match(operatorRx)){// HANDLE OPERATORS
				now = {
					token:stream.current(),
					type:"operator"
				};
			}
			// then we make sure it's an operator only and not a variable
			if (stream.match(numberRx)) {// HANDLE NUMBERS
				now = {
					token : stream.current(),
					type : "number"
				};
			} else if (stream.match(wordRx)) {// HANDLE WORDS
				var endsInOpenParens = stream.peek() == "(";
				var endsInCloseParens = stream.peek() == ")";
				var current = stream.current();
				now = {
					token : current
				};
				if (!endsInOpenParens) {// must be a variable or operator
					if (operators.hasOwnProperty(current)) {
						now.type = "operator";
					} else if (variables.hasOwnProperty(current)) {
						now.type = "variable";
					} else {
						now.type = null;
						now.err = true;
					}

				} else {// must be a function or operator
					if (operators.hasOwnProperty(current)) {
						now.type = "operator";
					} else if (functions.hasOwnProperty(current)) {
						now.type = "keyword";
					} else {
						now.type = null;
						now.err = true;
					}
				}
			} else {
				var ch = stream.next();
				if (ch == '(') {
					state.parenCount++;
				} else if (ch == ')') {
					state.parenCount--;
					if (state.parenCount < 0) {
						now.err = true;
					}
				} else if (ch == '"' || ch == "'") {
					state.tokenize = tokenString(ch);
					now.type = state.tokenize(stream, state);
				}
			}
			state.prev=now;
			if(now.err){
				return now.type?now.type+" error":"error";
			}else{
				return now.type||null;
			}
		};
		function buildRegExps(operators) {
			var multis=[];
			var singles="";
			for(var op in operators){
				var item=operators[op];
				if (item.id!=="power"){
					var keyToken=item.keyToken;
					if (keyToken&&keyToken.length===1){	// Srini.. just use the keyToken's that have a single character
						singles+=(keyToken==="-")?"\\-":keyToken;
					}else if ("||"===keyToken){	// need to escape the | because it's a keyword in regex
						singles+="\|";
					}
				}
				if(op.length==1){
					singles+=(op==="-")?"\\-":op;
				}else{
					var tmp="";
					for(var i=0;i<op.length;i++){
						if ("_"!==op[i]) {	//Srini.. we have an "_" in the id.. ignore it
							tmp += "[" + op[i] + "]";
						}
					}
					multis.push(tmp);
					// Srini.. we need to add "do", "then", "else" and "end" in case of IF
					if ("_if"===op){
						// Srini.. add 'null','true','false' and 'missing' 
						var others=["do","then","else","end","null","true","false","missing"];
						_.each(others,function(other){
							tmp="";
							for (var i=0;i<other.length;i++){
								tmp += "[" + other[i] +"]";
							}
							multis.push(tmp);
						});
						singles+=";";
					}
				}
			}
			if(singles){
				multis.push("["+singles+"]");
			}
			return [new RegExp("[^\"'(),\\s"+singles+"]+","i"),new RegExp(multis.join("|"),"i")];
		};
		function tokenString(quote) {
			return function(stream, state) {
				var escaped = false, next, end = false;
				while ((next = stream.next()) != null) {
					if (next == quote && !escaped) {
						end = true;
						break;
					}
					escaped = !escaped && next == "\\";
				}
				if (end || !escaped)
					state.tokenize = null;
				return "string";
			};
		}

		// Interface

		return {
			startState : function() {
				return {
					tokenize : null,
					parenCount : 0
				};
			},

			token : function(stream, state) {
				if (stream.eatSpace())
					return null;
				var style = (state.tokenize || tokenBase)(stream, state);
				return style;
			}
		};
	});

	CodeMirror.defineMIME("text/x-datadef", "datadef");

});
