jQuery.sap.declare("sas.risk.common.widgets.querybldr.QueryBuilder");
jQuery.sap.require("sas.risk.common.widgets.querybldr.QueryBuilderActions");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.ui.core.Item");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.AndFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.OrFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.AJunctionFilter");
jQuery.sap.require("sas.risk.common.widgets.querybldr.handler.FilterHandlerFactory");

jQuery.sap.require("sas.risk.common.layout.gridlayout.GridLayout");
jQuery.sap.require("sas.risk.common.layout.gridlayout.GridLayoutRenderer");
jQuery.sap.require("sas.risk.common.layout.gridlayout.GridLayoutData");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.cpb3.components.SpreadsheetComponent");


sas.risk.common.layout.gridlayout.GridLayout.extend("sas.risk.common.widgets.querybldr.QueryBuilder",{
	metadata:{
		properties:{
			value:{type:"any",defaultValue:{type:"and",children:[]}},
			meta:{type:"any"}
		},
		events:{
			change:{}
		}
	},
	constructor:function(){
		sas.risk.common.layout.gridlayout.GridLayout.apply(this,arguments);
		this.addStyleClass("riskQueryBldr");
		this.actions=new sas.risk.common.widgets.querybldr.QueryBuilderActions(this);
		this.setGridTemplateColumns("auto");
		this.setGridRowGap("18px");
		this.setWidth("100%");
		this.header=new sas.risk.common.layout.flexlayout.FlexLayout({
			justifyContent:sas.risk.common.layout.flexlayout.FlexLayoutJustifyContent.SpaceBetween,
			items:[this._createMainAndOrSelect(),this._createMainButtons()]
		}).addStyleClass("riskQueryBldrSelectable");
		this.actions.resetButtonState();
		this.body=new sas.risk.common.layout.gridlayout.GridLayout({
			width:"100%",
			gridRowGap:"10px",
			gridTemplateColumns:"20px auto auto auto auto 20px"
		});
		this.addItem(this.header);
		this.addItem(this.body);
	},
	renderer:sas.risk.common.layout.gridlayout.GridLayoutRenderer
});
sas.risk.common.widgets.querybldr.QueryBuilder.prototype.getMeta=function(){
	var metaJson=this.getProperty("meta");
	if(metaJson){
		if(metaJson instanceof sas.risk.common.widgets.spreadsheet.SpreadsheetMeta){
			return metaJson;
		}
		var meta=sas.ui.cpb3.components.SpreadsheetComponent.convertMeta(metaJson);
		return new sas.risk.common.widgets.spreadsheet.SpreadsheetMeta(meta);
	}
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype.updateValue=function(){
	if(this.activeFilter){
		this.activeFilterCoords=this.actions._getFilterCoords(this.activeFilter);
	}
	var filter=this.header.data("filter");
	var json=filter.toJson();
	this.setValue(json);
	this.fireChange({value:json});
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype.onBeforeRendering=function(){
	var value=this.getValue();
	this.body.removeAllItems();
	if(value){
		var filter=sas.risk.common.widgets.spreadsheet.filter.AFilter.hydrate(value);
		if(!(filter instanceof sas.risk.common.widgets.spreadsheet.filter.AJunctionFilter)){
			filter=new sas.risk.common.widgets.spreadsheet.filter.AndFilter([filter]);
		}
		this.header.data("filter",filter);
		if(this.activeFilterCoords){
			this.activeFilter=this.actions._getFilterByCoords(this.activeFilterCoords);
		}else{
			this.activeFilter=filter;
		}
		this._setAndOrSelect(this.andOrSelect,filter);
		var that=this;
		_.each(filter.getChildren(),function(f){
			that.body.addItem(that.buildRow(f));
		});
	}
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype.onAfterRendering=function(){
	var $this=this.$();
	var $rows=$this.find(".riskQueryBldrSelectable");
	var activeRow=this.actions._getActiveRow($rows);
	if(activeRow){$(activeRow).addClass("riskQueryBldrSelected");}
	$rows.attr("tabindex","-1");
	$rows.focus($.proxy(this.actions.onFocus,this.actions));
	this.actions.refreshButtonState();
//	$rows.blur($.proxy(this.actions.onBlur,this.actions));
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createButton=function(name,icon){
	this["button"+name]=new sas.hc.m.Button({
		icon:sas.icons.HC[icon],
		press:$.proxy(this.actions["on"+name],this.actions)
	});
	return this["button"+name];
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createMainButtons=function(){
	return new sas.risk.common.layout.flexlayout.FlexLayout({
		items:[
			this._createButton("Up","ARROWSMALLUP"),
			this._createButton("Down","ARROWSMALLDOWN"),
			this._createButton("Copy","COPY"),
			this._createButton("Group","NESTEDGROUP"),
			this._createButton("Add","ADD")
		]
	});
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createGroupButtons=function(){
	return new sas.risk.common.layout.flexlayout.FlexLayout({
		items:[
			this._createButton("DeleteGroup","DELETE"),
		]
	}).addStyleClass("riskQueryBldrGroupButtons");
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createFilterButtons=function(){
	return new sas.risk.common.layout.flexlayout.FlexLayout({
		items:[
			new sas.hc.m.ToggleButton({
				icon: sas.icons.HC.FORMULA,
            tooltip: "Toggle to expression editor",
            ariaLabel: "Toggle to expression editor",
            press: function(evt){
                console.log(evt);
            }
        }),
			this._createButton("DeleteFilter","DELETE"),
		]
	}).addStyleClass("riskQueryBldrFilterButtons");
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._setAndOrSelect=function(andOrSelect,filter){
	var isAnd=filter instanceof sas.risk.common.widgets.spreadsheet.filter.AndFilter;
	andOrSelect.setSelectedKey(isAnd?"AND":"OR");
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createAndOrSelect=function(){
	var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	return new sas.hc.m.Select({
		items:[
			new sas.hc.ui.core.Item({key:"AND",tooltip:rb.getText("querybldr.and.select.txt"),text:rb.getText("querybldr.and.select.txt")}),
			new sas.hc.ui.core.Item({key:"OR",tooltip:rb.getText("querybldr.or.select.txt"),text:rb.getText("querybldr.or.select.txt")})
		],
		change:$.proxy(this.actions.onAndOrSelect,this.actions)
	});
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createMainAndOrSelect=function(){
	this.andOrSelect=this._createAndOrSelect();
	return this.andOrSelect;
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createSubAndOrSelect=function(){
	return this._createAndOrSelect().addStyleClass("riskQueryBldrJunction");
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createColumnsSelect=function(filter){
	var select=new sas.hc.m.Select({
		change:$.proxy(this.actions.onColumnSelect,this.actions)
	});
	var meta=this.getMeta();
	var selectedItem=null;
	_.each(meta.getColumnIds(),function(columnId){
		var options=meta.getColumnOptions(columnId);
		var item=new sas.hc.ui.core.Item({
			key:columnId,
			tooltip:options.label,
			text:options.label
		});
		select.addItem(item);
		if(filter.getName()===columnId){
			selectedItem=item;
		}
	});
	select.setSelectedItem(selectedItem);
	return select;
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._createOperatorsSelect=function(filter,handler){
	var select=new sas.hc.m.Select({
		change:$.proxy(this.actions.onOperatorSelect,this.actions)
	});
	var selectedItem=null;
	_.each(handler.getOperators(),function(operator){
		var item=new sas.hc.ui.core.Item({
			key:operator.key,
			tooltip:operator.label,
			text:operator.label
		});
		if(operator.key===filter.getOperator()){
			selectedItem=item;
		}
		select.addItem(item);
	});
	select.setSelectedItem(selectedItem);
	return select;
};
//BUILDERS
sas.risk.common.widgets.querybldr.QueryBuilder.prototype.buildRow=function(filter){
	if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.AJunctionFilter){
		return this._buildJunctionRow(filter);
	}else{
		return this._buildFilterRow(filter);
	}
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._buildJunctionRow=function(filter){
	var grid=new sas.risk.common.layout.gridlayout.GridLayout({
		gridRowGap:"18px",
		gridTemplateColumns:"auto"
	}).setLayoutData(new sas.risk.common.layout.gridlayout.GridLayoutData({
		gridColumnStart:2,
		gridColumnEnd:6
	})).addStyleClass("riskQueryBldrGroup");
	var andOrSelect=this._createSubAndOrSelect();
	this._setAndOrSelect(andOrSelect,filter);
	var header=new sas.risk.common.layout.flexlayout.FlexLayout({
		items:[andOrSelect,this._createGroupButtons()]
	}).addStyleClass("riskQueryBldrSelectable");
	header.data("filter",filter);
	grid.addItem(header);
	
	var body=new sas.risk.common.layout.gridlayout.GridLayout({
		width:"100%",
		gridRowGap:"10px",
		gridTemplateColumns:"20px auto auto auto auto 20px"
	});
	var that=this;
	_.each(filter.getChildren(),function(filter){
		body.addItem(that.buildRow(filter));
	});
	grid.addItem(body);

	return grid;
};
sas.risk.common.widgets.querybldr.QueryBuilder.prototype._buildFilterRow=function(filter){
	var columnId=filter.getName();
	var handler=sas.risk.common.widgets.querybldr.handler.FilterHandlerFactory.getInstance().getHandler(this,columnId);
	var input=handler.getInput(filter);
	var row=new sas.risk.common.layout.flexlayout.FlexLayout({
		items:[
			this._createColumnsSelect(filter),
			this._createOperatorsSelect(filter,handler),
			input,
			this._createFilterButtons()
		]
	}).setLayoutData(new sas.risk.common.layout.gridlayout.GridLayoutData({
		gridColumnStart:2,
		gridColumnEnd:6
	})).addStyleClass("riskQueryBldrSelectable riskQueryBldrFilter");
	row.data("filter",filter);
	return row;
};