jQuery.sap.declare("sas.risk.common.widgets.querybldr.QueryBuilderActions");
jQuery.sap.require("sas.risk.common.widgets.querybldr.QueryBuilder");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.StringFilter");

sas.risk.common.widgets.querybldr.QueryBuilderActions=function(queryBuilder){
	this.queryBuilder=queryBuilder;
};
sas.risk.common.widgets.querybldr.QueryBuilderActions.prototype={
	onAndOrSelect:function(e){
		var filter=this._getFilter(e.getSource());
		var parent=this._findParentFilter(filter);
		var newFilter=null;
		var isAnd=e.getParameter("selectedItem").getKey()==="AND";
		var newFilter=new sas.risk.common.widgets.spreadsheet.filter[(isAnd?"And":"Or")+"Filter"](filter.getChildren());
		if(parent){
			var children=parent.getChildren();
			var index=children.indexOf(filter);
			children.splice(index,1,newFilter);
		}else{//root
			this.queryBuilder.header.data("filter",newFilter);
		}
		this.queryBuilder.updateValue();
	},
	onUp:function(e){
		var filter=this.queryBuilder.activeFilter;
		var parent=this._findParentFilter(filter);
		var children=parent.getChildren();
		var index=children.indexOf(filter);
		if(index){
			var peer=children[index-1];
			if(peer instanceof sas.risk.common.widgets.spreadsheet.filter.AJunctionFilter){
				children.splice(index,1);//remove from parent
				peer.getChildren().push(filter);//add to peer
			}else{
				children.splice(index-1,2,children[index],children[index-1]);//swap with previous
			}
		}else{
			var grandParent=this._findParentFilter(parent);
			var parentIndex=grandParent.getChildren().indexOf(parent);
			if(children.length===1){//only child
				children.length=0;
				grandParent.getChildren().splice(parentIndex,1,filter);//replace parent with child
			}else{
				children.splice(index,1);//remove child from parent
				grandParent.getChildren().splice(parentIndex,1,filter,parent);//add child before parent on grandparent
			}
		}
		this.queryBuilder.updateValue();
	},
	onDown:function(e){
		var filter=this.queryBuilder.activeFilter;
		var parent=this._findParentFilter(filter);
		var children=parent.getChildren();
		var index=children.indexOf(filter);
		var isLastChild=index===(children.length-1);
		if(!isLastChild){
			var peer=children[index+1];
			if(peer instanceof sas.risk.common.widgets.spreadsheet.filter.AJunctionFilter){
				children.splice(index,1);//remove from parent
				peer.getChildren().unshift(filter);//add at beginning of peer
			}else{
				children.splice(index,2,children[index+1],children[index]);//swap with next
			}
		}else{
			var grandParent=this._findParentFilter(parent);
			var parentIndex=grandParent.getChildren().indexOf(parent);
			if(children.length===1){//only child
				children.length=0;
				grandParent.getChildren().splice(parentIndex,1,filter);//replace parent with child
			}else{
				children.splice(index,1);//remove child from parent
				grandParent.getChildren().splice(parentIndex+1,0,filter);//add child after parent on grandparent
			}
		}
		this.queryBuilder.updateValue();
	},
	onCopy:function(e){
		var filter=this.queryBuilder.activeFilter;
		var newFilter=filter.clone();
		var parent=this._findParentFilter(filter);
		var children=parent.getChildren();
		var index=children.indexOf(filter);
		children.splice(index+1,0,newFilter);
		this.queryBuilder.updateValue();
	},
	onGroup:function(e){
		var filter=this.queryBuilder.activeFilter;
		var parent=this._findParentFilter(filter);
		var children=parent.getChildren();
		var index=children.indexOf(filter);
		var group=new sas.risk.common.widgets.spreadsheet.filter.AndFilter([filter]);
		children.splice(index,1,group);
		this.queryBuilder.updateValue();
	},
	onAdd:function(e){
		var meta=this.queryBuilder.getMeta();
		var columnId=meta.getColumnIds()[0];
		
		var handler=sas.risk.common.widgets.querybldr.handler.FilterHandlerFactory.getInstance().getHandler(this.queryBuilder,columnId);
		var newFilter=handler.createFilter();
		var options=meta.getColumnOptions(columnId);
		//var newFilter=new sas.risk.common.widgets.spreadsheet.filter.StringFilter(columnId,"contains","",options.label,"");
		var filter=this.queryBuilder.activeFilter;
		if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.AJunctionFilter){
			filter.getChildren().splice(filter.getChildren().length,0,newFilter);
		}else{
			var parent=this._findParentFilter(filter);
			var children=parent.getChildren();
			var index=children.indexOf(filter);
			children.splice(index+1,0,newFilter);
		}
		this.queryBuilder.updateValue();
	},
	onDeleteGroup:function(e){
		var filter=e.getSource().getParent().getParent().data("filter");
		var parent=this._findParentFilter(filter);
		var children=parent.getChildren();
		var index=children.indexOf(filter);
		children.splice.apply(children,[index,1].concat(filter.getChildren()));
		if(this.queryBuilder.activeFilter===filter){
			this.queryBuilder.activeFilter=null;
		}
		this.queryBuilder.updateValue();
	},
	onDeleteFilter:function(e){
		var filter=e.getSource().getParent().getParent().data("filter");
		var parent=this._findParentFilter(filter);
		var children=parent.getChildren();
		var index=children.indexOf(filter);
		children.splice(index,1);
		if(this.queryBuilder.activeFilter===filter){
			this.queryBuilder.activeFilter=null;
		}
		this.queryBuilder.updateValue();
	},
	onOperatorSelect:function(e){
		var select=e.getSource();
		var filter=this._getFilter(select);
		filter.setOperator(select.getSelectedKey());
		this.queryBuilder.updateValue();
	},
	onColumnSelect:function(e){
		var select=e.getSource();
		var columnId=select.getSelectedKey();
		var handler=sas.risk.common.widgets.querybldr.handler.FilterHandlerFactory.getInstance().getHandler(this.queryBuilder,columnId);
		var filter=handler.createFilter();
		var oldFilter=select.getParent().data("filter");
		var parent=this._findParentFilter(oldFilter);
		var children=parent.getChildren();
		var index=children.indexOf(oldFilter);
		children.splice(index,1,filter);
		this.queryBuilder.updateValue();
	},
//	onBlur:function(e){
//		this.resetButtonState();
//	},
	resetButtonState:function(){
		this.queryBuilder.buttonUp.setEnabled(false);
		this.queryBuilder.buttonDown.setEnabled(false);
		this.queryBuilder.buttonCopy.setEnabled(false);
		this.queryBuilder.buttonGroup.setEnabled(false);
		this.queryBuilder.buttonAdd.setEnabled(true);
	},
	onFocus:function(e){
		var row=sap.ui.getCore().byId(e.currentTarget.id);
		var filter=row.data("filter");
		this.queryBuilder.activeFilter=filter;
		this.refreshButtonState();
	},
	refreshButtonState:function(){
		var root=this.queryBuilder.header.data("filter");
		var filter=this.queryBuilder.activeFilter;
		var $rows=this.queryBuilder.$().find(".riskQueryBldrSelectable");
		var row=this._getActiveRow($rows);
		$rows.removeClass("riskQueryBldrSelected");
		if(row){
			$(row).addClass("riskQueryBldrSelected");
		}
		var parent=this._findParentFilter(filter);
		if(parent){//non-root
			var index=parent.getChildren().indexOf(filter);
			var isFirst=index===0 && parent===root;
			var isLast=index===(parent.getChildren().length-1) && parent===root;
			this.queryBuilder.buttonUp.setEnabled(!isFirst);
			this.queryBuilder.buttonDown.setEnabled(!isLast);
			this.queryBuilder.buttonCopy.setEnabled(true);
			this.queryBuilder.buttonGroup.setEnabled(true);
			this.queryBuilder.buttonAdd.setEnabled(true);
		}else{//root
			this.queryBuilder.buttonUp.setEnabled(false);
			this.queryBuilder.buttonDown.setEnabled(false);
			this.queryBuilder.buttonCopy.setEnabled(false);
			this.queryBuilder.buttonGroup.setEnabled(false);
			this.queryBuilder.buttonAdd.setEnabled(true);
		}
	},
	_getActiveRow:function($rows){
		var that=this;
		return _.find($rows,function(row){
			return sap.ui.getCore().byId(row.id).data("filter")===that.queryBuilder.activeFilter;
		});
	},
	_getFilterCoords:function(filter){
		var parent=this._findParentFilter(filter);
		if(parent){
			var index=parent.getChildren().indexOf(filter);
			var array=this._getFilterCoords(parent);
			array.splice(array.length,0,index);
			return array;
		}
		return [];
	},
	_getFilterByCoords:function(coords){
		var root=this.queryBuilder.header.data("filter");
		filter=_.reduce(coords,function(memo,idx){
			return memo.getChildren()[idx];
		},root);
		return filter;
	},
	_getFilter:function(source){
		var parent=source;
		while(!(parent instanceof sas.risk.common.layout.flexlayout.FlexLayout) || !parent.data("filter")){
			parent=parent.getParent();
		}
		return parent.data("filter");
	},
	_findParentFilter:function(filter){
		var root=this.queryBuilder.header.data("filter");
		return this._searchFilter(null,root,filter);
	},
	_searchFilter:function(parent,current,filter){
		if(current===filter){
			return parent;
		}else if(current.getChildren){
			var result=null;
			var children=current.getChildren();
			for(var i=0;result===null && i<children.length;i++){
				result=this._searchFilter(current,children[i],filter);
			}
			return result;
		}
		return null;
	}
};