jQuery.sap.declare("sas.risk.common.widgets.querybldr.handler.DateFilterHandler");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.hc.m.DatePicker");
jQuery.sap.require("sas.risk.common.widgets.querybldr.handler.format.RiskDateFormat");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.DateFilter");

sas.risk.common.widgets.querybldr.handler.DateFilterHandler=function(columnOptions,changeCallback){
	this.columnOptions=columnOptions;
	this.changeCallback=changeCallback;
};
sas.risk.common.widgets.querybldr.handler.DateFilterHandler.prototype={
	getOperators:function(){
		var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
		return [
			{label:rb.getText("table.dialog.advsearch.operator.eq.txt"),key:"eq"},
			{label:rb.getText("table.dialog.advsearch.operator.ne.txt"),key:"ne"},
			{label:rb.getText("table.dialog.advsearch.operator.gt.txt"),key:"gt"},
			{label:rb.getText("table.dialog.advsearch.operator.lt.txt"),key:"lt"},
			{label:rb.getText("table.dialog.advsearch.operator.ge.txt"),key:"ge"},
			{label:rb.getText("table.dialog.advsearch.operator.le.txt"),key:"le"}
		];
	},
	getInput:function(filter){
		var value=filter.getValue();
		if(value===undefined||value===null){
			value=this.today();
		}else if(_.isString(value)){
			value=new Date(value+"T12:00:00.000Z");
		}
		filter.setValue(value);
		filter.setValueLabel(this.toLocaleDateString(value));
		var that=this;
		var picker=new sas.hc.m.DatePicker({
			dateValue : value,
			valueFormat:"yyyy-MM-dd",
			change:function(e){
				var value=new Date(e.getParameter("newValue")+"T12:00:00.000Z");//noon-shift it
				if(isNaN(value.getTime())){
					filter.setValue(null);
					filter.setValueLabel(null);
				}else{
					filter.setValue(value);
					filter.setValueLabel(that.toLocaleDateString(value));
				}
				that.changeCallback();
			}
		}).addStyleClass("riskTabAdvDatePicker");
		return picker;
	},
	updateFilterValueLabel:function(filter){
		var def=$.Deferred().resolve();
		var value=filter.getValue();
		if(value===undefined||value===null){
			value=this.today();
		}else if(_.isString(value)){
			value=new Date(value+"T12:00:00.000Z");
		}
		filter.setValueLabel(this.toLocaleDateString(value));
		return def;
	},
	createFilter:function(){
		var columnId=this.columnOptions.id;
		var columnLabel=this.columnOptions.label;
		var value=this.today();
		return new sas.risk.common.widgets.spreadsheet.filter.DateFilter(columnId,"eq",value,columnLabel,this.toLocaleDateString(value));
	},
	today:function(){
   	var now=new Date();
   	return new Date(Date.UTC(now.getFullYear(),now.getMonth(),now.getDate(),12,0,0,0));
   },
   toLocaleDateString:function(dateObject){
   	var locale=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getLocale();
   	var inputFormat = sas.risk.common.widgets.querybldr.handler.format.RiskDateFormat.getInstance({style: "medium"}, locale);
   	return inputFormat.format(dateObject);
   }
};