jQuery.sap.declare("sas.risk.common.widgets.querybldr.handler.TimestampFilterHandler");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.hc.m.DatePicker");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.TimestampFilter");

sas.risk.common.widgets.querybldr.handler.TimestampFilterHandler=function(columnOptions,changeCallback){
	this.columnOptions=columnOptions;
	this.changeCallback=changeCallback;
};
sas.risk.common.widgets.querybldr.handler.TimestampFilterHandler.prototype={
	getOperators:function(){
		var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
		return [
			{label:rb.getText("table.dialog.advsearch.operator.gt.txt"),key:"gt"},
			{label:rb.getText("table.dialog.advsearch.operator.lt.txt"),key:"lt"},
			{label:rb.getText("table.dialog.advsearch.operator.ge.txt"),key:"ge"},
			{label:rb.getText("table.dialog.advsearch.operator.le.txt"),key:"le"}
		];
	},
	getInput:function(filter){
		var value=filter.getValue();
		var operator=filter.getOperator();
		if(value===undefined||value===null){
			//value=this.today();
			value=that.adjustedDatetimeValue(operator,null);
		}
//			else if(_.isString(value)){
//			value=new Date(value+"T12:00:00.000Z");
//		}
//		filter.setValue(value);
//		filter.setValueLabel(this.toLocaleDateString(value));
		var that=this;
		var picker=new sas.hc.m.DatePicker({
			dateValue : value,
			valueFormat:"yyyy-MM-dd",
			change:function(e){
				//var value=new Date(e.getParameter("newValue")+"T12:00:00.000Z");//noon-shift it
				var value=e.getParameter("newValue");
				var operator=filter.getOperator();
				value= that.adjustedDatetimeValue(operator,value,true);	
				if(isNaN(value.getTime())){
					filter.setValue(null);
					filter.setValueLabel(null);
				}else{
					filter.setValue(value);
					filter.setValueLabel(that.toLocaleDateString(value));
				}
			}
		}).addStyleClass("riskTabAdvDatePicker");
		if (_.isDate(picker.getDateValue())){
			var value = that.adjustedDatetimeValue(operator,picker.getDateValue(),false);
			filter.setValue(value);
			filter.setValueLabel(that.toLocaleDateString(value,true));
		}
		return picker;
	},
	updateFilterValueLabel:function(filter){
		var def=$.Deferred().resolve();
		var value=filter.getValue();
		if(value===undefined||value===null){
			value=that.adjustedDatetimeValue(operator,null);
		}else if (_.isDate(value)){
			value=this.adjustedDatetimeValue(operator,value,false);
		}
		filter.setValueLabel(this.toLocaleDateString(value,true));
		return def;
	},
	createFilter:function(){
		var columnId=this.columnOptions.id;
		var columnLabel=this.columnOptions.label;
		var value=this.today();
		return new sas.risk.common.widgets.spreadsheet.filter.TimestampFilter(columnId,"gt",value,columnLabel,this.toLocaleDateString(value));
	},
	today:function(){
   	var now=new Date();
   	return new Date(Date.UTC(now.getFullYear(),now.getMonth(),now.getDate(),12,0,0,0));
   },
   toLocaleDateString:function(dateObject,bUTC){
   	var locale=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getLocale();
   	var inputFormat = sas.risk.common.widgets.querybldr.handler.format.RiskDateFormat.getInstance({style: "medium"}, locale);
   	return inputFormat.format(dateObject,bUTC);
   },
   adjustedDatetimeValue:function(type,value,isISO){	
		var parts;
		if (value!==null&& isISO){
				parts = value.split('-');
		}
   	switch  (type){
   		case 'gt':
   		case 'le':
   			if (value&&isISO){
   				return new Date(Date.UTC(parts[0], parts[1]-1,parts[2], 23, 59, 59, 999));
   			}else if (value && !isISO)
   				return new Date(Date.UTC(value.getFullYear(),value.getMonth(),value.getDate(),23,59,59,999));
   			else{
   				var now = new Date();
   				return new Date(Date.UTC(now.getFullYear(),now.getMonth(),now.getDate(),23,59,59,999));
   			}   				
   			break;
   		case 'ge':
   		case 'lt':
   			if (value&&isISO){
   				return new Date(Date.UTC(parts[0], parts[1]-1,parts[2], 0, 0, 0, 0));
   			}else if (value && !isISO)
   				return new Date(Date.UTC(value.getFullYear(),value.getMonth(),value.getDate(),0,0,0,0));
   			else{
   				var now = new Date();
   				return new Date(Date.UTC(now.getFullYear(),now.getMonth(),now.getDate(),0,0,0,0));
   			}  
   			break;  			
   	}
   }
};