// Provides class sap.ui.core.format.DateFormat
sap.ui.define(['jquery.sap.global', 'sap/ui/core/library', 'sap/ui/core/format/DateFormat',  'sap/ui/core/LocaleData', 'sap/ui/core/date/UniversalDate',],
	function(jQuery, library, DateFormat, LocaleData, UniversalDate) {
	"use strict";
	
	// shortcut
	var CalendarType = library.CalendarType;

	var RiskDateFormat = function() {
		// Do not use the constructor
		throw new Error();
	};
	
	RiskDateFormat.oDateInfo = {
			oDefaultFormatOptions: {
				style: "short",
				relativeScale: "day",
				relativeStyle: "wide"
			},
			aFallbackFormatOptions: [
				{style: "short"},
				{style: "medium"},
				{pattern: "yyyy-MM-dd"},
				{pattern: "yyyyMMdd", strictParsing: true}
			],
			bShortFallbackFormatOptions: true,
			getPattern: function(oLocaleData, sStyle) {
				var shortLocaleFormat=oLocaleData.mData["ca-gregorian"]["dateFormats"]["short"]; //CPB: override
				var match=shortLocaleFormat.match(/[y]{4}/) || shortLocaleFormat.match(/[y]{2}/) || shortLocaleFormat.match(/[y]/);
				if(match && match.length>0){
					shortLocaleFormat = shortLocaleFormat.replace(match[0],"yyyy");
					oLocaleData.mData["ca-gregorian"]["dateFormats"]["short"]=shortLocaleFormat;
				}
				return oLocaleData.getDatePattern(sStyle);
			},
			oRequiredParts: {
				"text": true, "year": true, "weekYear": true, "month": true, "day": true
			},
			aRelativeScales: ["year", "month", "week", "day"],
			aRelativeParseScales: ["year", "quarter", "month", "week", "day", "hour", "minute", "second"]
	};
	
	RiskDateFormat.getInstance = function(oFormatOptions, oLocale) {
		var dateFormatInstance=DateFormat.createInstance(oFormatOptions, oLocale, RiskDateFormat.oDateInfo);
		dateFormatInstance.parse=RiskDateFormat.prototype.parse;
		dateFormatInstance.format=RiskDateFormat.prototype.format;
		return dateFormatInstance;
	};
	RiskDateFormat.getDateTimeInstance = function(oFormatOptions, oLocale) {
		var dateTimeFormatInstance=DateFormat.getDateTimeInstance(oFormatOptions, oLocale);
		dateTimeFormatInstance.parse=RiskDateFormat.prototype.parse;
		dateTimeFormatInstance.format=RiskDateFormat.prototype.format;
		return dateTimeFormatInstance;
	};
	
	//Overriding format from DateFormat with few changes marked with comments
	RiskDateFormat.prototype.format = function(oJSDate, bUTC) {
		if (bUTC === undefined) {
			bUTC = this.oFormatOptions.UTC;
		}

		if (!oJSDate || isNaN(oJSDate.getTime())) {
			return "";
		}

		if (this.oFormatOptions.relative) {
			var sRes = this.formatRelative(oJSDate, bUTC, this.oFormatOptions.relativeRange);
			if (sRes) { //Stop when relative formatting possible, else go on with standard formatting
				return sRes;
			}
		}

		var sCalendarType = this.oFormatOptions.calendarType;
		var oDate = UniversalDate.getInstance(oJSDate, sCalendarType);

		var aBuffer = [],
			oPart,
			sCalendarType = this.oFormatOptions.calendarType,
			iDay = bUTC ? oDate.getUTCDay() : oDate.getDay(),
			iDate = bUTC ? oDate.getUTCDate() : oDate.getDate(),
			iMonth = bUTC ? oDate.getUTCMonth() : oDate.getMonth(),
			iYear = bUTC ? oDate.getUTCFullYear() : oDate.getFullYear(),
			iEra = bUTC ? oDate.getUTCEra() : oDate.getEra(),
			iWeek = bUTC ? oDate.getUTCWeek() : oDate.getWeek(),
			iMilliseconds = bUTC ? oDate.getUTCMilliseconds() : oDate.getMilliseconds(),
			iSeconds = bUTC ? oDate.getUTCSeconds() : oDate.getSeconds(),
			iMinutes = bUTC ? oDate.getUTCMinutes() : oDate.getMinutes(),
			iHours = bUTC ? oDate.getUTCHours() : oDate.getHours(),
			iTZOffset = Math.abs(oDate.getTimezoneOffset()),
			bPositiveOffset = oDate.getTimezoneOffset() > 0,
			iHourOffset = Math.floor(iTZOffset / 60),
			iMinuteOffset = iTZOffset % 60,
			iQuarter = Math.floor(iMonth / 3),
			sYear,
			sWeek,
			sHours,
			sResult;

		for (var i = 0; i < this.aFormatArray.length; i++) {
			oPart = this.aFormatArray[i];
			switch (oPart.type) {
				case "text":
					aBuffer.push(oPart.value);
					break;
				case "day":
					aBuffer.push(jQuery.sap.padLeft(String(iDate), "0", oPart.digits));
					break;
				case "dayNameInWeek":
					if (oPart.digits < 4) {
						aBuffer.push(this.aDaysAbbrev[iDay]);
					} else if (oPart.digits === 4) {
						aBuffer.push(this.aDaysWide[iDay]);
					} else if (oPart.digits === 5) {
						aBuffer.push(this.aDaysNarrow[iDay]);
					} else {
						aBuffer.push(this.aDaysShort[iDay]);
					}
					break;
				case "dayNameInWeekStandalone":
					if (oPart.digits < 4) {
						aBuffer.push(this.aDaysAbbrevSt[iDay]);
					} else if (oPart.digits === 4) {
						aBuffer.push(this.aDaysWideSt[iDay]);
					} else if (oPart.digits === 5) {
						aBuffer.push(this.aDaysNarrowSt[iDay]);
					} else {
						aBuffer.push(this.aDaysShortSt[iDay]);
					}
					break;
				case "dayNumberOfWeek":
					aBuffer.push(iDay || 7);
					break;
				case "month":
					if (oPart.digits === 3) {
						aBuffer.push(this.aMonthsAbbrev[iMonth]);
					} else if (oPart.digits === 4) {
						aBuffer.push(this.aMonthsWide[iMonth]);
					} else if (oPart.digits > 4) {
						aBuffer.push(this.aMonthsNarrow[iMonth]);
					} else {
						aBuffer.push(jQuery.sap.padLeft(String(iMonth + 1), "0", oPart.digits));
					}
					break;
				case "monthStandalone":
					if (oPart.digits === 3) {
						aBuffer.push(this.aMonthsAbbrevSt[iMonth]);
					} else if (oPart.digits === 4) {
						aBuffer.push(this.aMonthsWideSt[iMonth]);
					} else if (oPart.digits > 4) {
						aBuffer.push(this.aMonthsNarrowSt[iMonth]);
					} else {
						aBuffer.push(jQuery.sap.padLeft(String(iMonth + 1), "0", oPart.digits));
					}
					break;
				case "quarter":
					if (oPart.digits === 3) {
						aBuffer.push(this.aQuartersAbbrev[iQuarter]);
					} else if (oPart.digits === 4) {
						aBuffer.push(this.aQuartersWide[iQuarter]);
					} else if (oPart.digits > 4) {
						aBuffer.push(this.aQuartersNarrow[iQuarter]);
					} else {
						aBuffer.push(jQuery.sap.padLeft(String(iQuarter + 1), "0", oPart.digits));
					}
					break;
				case "quarterStandalone":
					if (oPart.digits === 3) {
						aBuffer.push(this.aQuartersAbbrevSt[iQuarter]);
					} else if (oPart.digits === 4) {
						aBuffer.push(this.aQuartersWideSt[iQuarter]);
					} else if (oPart.digits > 4) {
						aBuffer.push(this.aQuartersNarrowSt[iQuarter]);
					} else {
						aBuffer.push(jQuery.sap.padLeft(String(iQuarter + 1), "0", oPart.digits));
					}
					break;
				case "era":
					if (oPart.digits <= 3) {
						aBuffer.push(this.aErasAbbrev[iEra]);
					} else if (oPart.digits === 4) {
						aBuffer.push(this.aErasWide[iEra]);
					} else {
						aBuffer.push(this.aErasNarrow[iEra]);
					}
					break;
				case "year":
				case "weekYear":
					sYear = "" + iYear;
					if (oPart.digits === 2 && sYear.length > 2) {
						sYear = sYear.substr(sYear.length - 2);
					}
					// When parsing we assume dates less than 100 to be in the current/last century,
					// so when formatting we have to make sure they are differentiable by prefixing with zeros
					if (sCalendarType !== CalendarType.Japanese && oPart.digits === 1 && iYear < 100) {
						sYear = jQuery.sap.padLeft(sYear, "0", 4);
					}
					aBuffer.push(jQuery.sap.padLeft(sYear, "0", oPart.digits));
					break;
				case "weekInYear":
					if (iWeek === undefined) {
						//Date object doesn't support week calculation
						break;
					}
					sWeek = "" + iWeek;
					if (oPart.digits < 3) {
						sWeek = jQuery.sap.padLeft(sWeek, "0", oPart.digits);
					} else {
						sWeek = this.oLocaleData.getCalendarWeek(oPart.digits === 3 ? "narrow" : "wide", jQuery.sap.padLeft(sWeek, "0", 2));
					}
					aBuffer.push(sWeek);
					break;
				case "hour0_23":
					aBuffer.push(jQuery.sap.padLeft(String(iHours), "0", oPart.digits));
					break;
				case "hour1_24":
					if (iHours === 0) {
						sHours = "24";
					} else {
						sHours = String(iHours);
					}
					aBuffer.push(jQuery.sap.padLeft(sHours, "0", oPart.digits));
					break;
				case "hour0_11":
					if (iHours > 11) {
						sHours = String(iHours - 12);
					} else {
						sHours = String(iHours);
					}
					aBuffer.push(jQuery.sap.padLeft(sHours, "0", oPart.digits));
					break;
				case "hour1_12":
					if (iHours > 12) {
						sHours = String(iHours - 12);
					} else if (iHours === 0) {
						sHours = "12";
					} else {
						sHours = String(iHours);
					}
					aBuffer.push(jQuery.sap.padLeft(sHours, "0", oPart.digits));
					break;
				case "minute":
					aBuffer.push(jQuery.sap.padLeft(String(iMinutes), "0", oPart.digits));
					break;
				case "second":
					aBuffer.push(jQuery.sap.padLeft(String(iSeconds), "0", oPart.digits));
					break;
				case "fractionalsecond":
					var sMilliseconds = String(iMilliseconds),
						sFractionalseconds = jQuery.sap.padLeft(sMilliseconds, "0", 3);
					sFractionalseconds = sFractionalseconds.substr(0, oPart.digits);
					sFractionalseconds = jQuery.sap.padRight(sFractionalseconds, "0", oPart.digits);
					aBuffer.push(sFractionalseconds);
					break;
				case "amPmMarker":
					var iDayPeriod = iHours < 12 ? 0 : 1;
					aBuffer.push(this.aDayPeriods[iDayPeriod]);
					break;
				case "timezoneGeneral":
					//TODO getTimezoneLong and getTimezoneShort does not exist on Date object
					//-> this is a preparation for a future full timezone support (only used by unit test so far)
					if (oPart.digits > 3 && oDate.getTimezoneLong()) {
						aBuffer.push(oDate.getTimezoneLong());
						break;
					} else if (oDate.getTimezoneShort()) {
						aBuffer.push(oDate.getTimezoneShort());
						break;
					}
					else if (oPart.digits === 1){//nibdey: if user gets timezone in "Z" pattern as input source format option then it should handle it that way during formatting instead of defaulting to GMT-xx:00 format
						aBuffer.push("Z");
						break;
					}
					aBuffer.push("GMT");
					// falls through
				case "timezoneISO8601":
					if (!bUTC && iTZOffset !== 0) {
						aBuffer.push(bPositiveOffset ? "-" : "+");
						aBuffer.push(jQuery.sap.padLeft(String(iHourOffset), "0", 2));
						aBuffer.push(":");
						aBuffer.push(jQuery.sap.padLeft(String(iMinuteOffset), "0", 2));
					} else {
						aBuffer.push("Z");
					}
					break;
				case "timezoneRFC822":
					if (!bUTC && iTZOffset !== 0) {
						aBuffer.push(bPositiveOffset ? "-" : "+");
						aBuffer.push(jQuery.sap.padLeft(String(iHourOffset), "0", 2));
						aBuffer.push(jQuery.sap.padLeft(String(iMinuteOffset), "0", 2));
					}
					break;

			}
		}

		sResult = aBuffer.join("");
		if (sap.ui.getCore().getConfiguration().getOriginInfo()) {
			sResult = new String(sResult);// eslint-disable-line no-new-wrappers
			sResult.originInfo = {
				source: "Common Locale Data Repository",
				locale: this.oLocale.toString(),
				style: this.oFormatOptions.style,
				pattern: this.oFormatOptions.pattern
			};
		}

		return sResult;
	};

	//Overriding parse from DateFormat with few changes marked with comments
	RiskDateFormat.prototype.parse = function(oValue, bUTC, bStrict) {
			if (bUTC === undefined) {
				bUTC = this.oFormatOptions.UTC;
			}

			if (bStrict === undefined) {
				bStrict = this.oFormatOptions.strictParsing;
			}

			var oDate,
				iIndex = 0,
				iDay = null,
				iMonth = null,
				iYear = null,
				iEra = null,
				iHours = null,
				iMinutes = null,
				iSeconds = null,
				iMilliseconds = null,
				iQuarter = null,
				bPM = false,
				oPart,
				sPart,
				iTZDiff = null,
				bValid = true,
				oFound,
				bFound,
				iCurrentEra = this.aErasWide.length - 1,
				oRequiredParts = this.oRequiredParts,
				sCalendarType = this.oFormatOptions.calendarType,
				aDaysVariants = [this.aDaysWide, this.aDaysWideSt, this.aDaysAbbrev, this.aDaysAbbrevSt, this.aDaysShort, this.aDaysShortSt, this.aDaysNarrow, this.aDaysNarrowSt],
				aMonthsVariants = [this.aMonthsWide, this.aMonthsWideSt, this.aMonthsAbbrev, this.aMonthsAbbrevSt, this.aMonthsNarrow, this.aMonthsNarrowSt],
				aQuartersVariants = [this.aQuartersWide, this.aQuartersWideSt, this.aQuartersAbbrev, this.aQuartersAbbrevSt, this.aQuartersNarrow, this.aQuartersNarrowSt],
				aErasVariants = [this.aErasWide, this.aErasAbbrev, this.aErasNarrow];

			function isNumber(iCharCode) {
				return iCharCode >= 48 && iCharCode <= 57;
			}

			function findNumbers(iMaxLength) {
				var iLength = 0;
				while (iLength < iMaxLength && isNumber(oValue.charCodeAt(iIndex + iLength))) {
					iLength++;
				}
				return oValue.substr(iIndex, iLength);
			}

			function findEntry(aList) {
				var iFoundIndex = -1,
					iMatchedLength = 0;

				for (var j = 0; j < aList.length; j++) {
					if (aList[j] && aList[j].length > iMatchedLength && oValue.indexOf(aList[j], iIndex) === iIndex) {
						iFoundIndex = j;
						iMatchedLength = aList[j].length;
					}
				}
				return {
					index: iFoundIndex,
					value: iFoundIndex === -1 ? null : aList[iFoundIndex]
				};
			}

			function parseTZ(bISO) {
				var iTZFactor = oValue.charAt(iIndex) === "+" ? -1 : 1;
				iIndex++; //"+" or "-"
				sPart = findNumbers(2);
				var iTZDiffHour = parseInt(sPart, 10);
				iIndex = iIndex + 2; //hh: 2 digits for hours
				if (bISO) {
					iIndex++; //":"
				}
				sPart = findNumbers(2);
				iIndex = iIndex + 2; //mm: 2 digits for minutes
				iTZDiff = parseInt(sPart, 10);
				iTZDiff = (iTZDiff + 60 * iTZDiffHour) * iTZFactor;
			}

			function checkValid(sType, bPartInvalid) {
				if (sType in oRequiredParts && bPartInvalid) {
					bValid = false;
				}
			}

			function matchInArray(aArray) {
				oFound = findEntry(aArray);
				if (oFound.index !== -1) {
					iIndex += oFound.value.length;
					return true;
				}
			}

			oValue = jQuery.trim(oValue);

			var oJSDate = this.parseRelative(oValue, bUTC);
			if (oJSDate) { //Stop when relative parsing possible, else go on with standard parsing
				return oJSDate;
			}

			for (var i = 0; i < this.aFormatArray.length; i++) {
				oPart = this.aFormatArray[i];
				switch (oPart.type) {
					case "text":
						if (oValue.indexOf(oPart.value, iIndex) === iIndex) {
							iIndex += oPart.value.length;
						} else {
							// only require text, if next part is also required
							checkValid(oPart.type, !!this.aFormatArray[i + 1] && this.aFormatArray[i + 1].type in oRequiredParts); // SAS Edit  check for array element out of bounds
						}
						break;
					case "day":
						sPart = findNumbers(Math.max(oPart.digits, 2));
						checkValid(oPart.type, sPart === "");
						iIndex += sPart.length;
						iDay = parseInt(sPart, 10);
						if (bStrict && (iDay > 31 || iDay < 1)) {
							bValid = false;
						}
						break;
					case "dayNameInWeek":
					case "dayNameInWeekStandalone":
						aDaysVariants.some(matchInArray);
						break;
					case "dayNumberOfWeek":
						sPart = findNumbers(oPart.digits);
						iIndex += sPart.length;
						break;
					case "month":
					case "monthStandalone":
						if (oPart.digits < 3) {
							sPart = findNumbers(Math.max(oPart.digits, 2));
							checkValid(oPart.type, sPart === "");
							iMonth = parseInt(sPart, 10) - 1;
							iIndex += sPart.length;
							if (bStrict && (iMonth > 11 || iMonth < 0)) {
								bValid = false;
							}
						} else {
							bFound = aMonthsVariants.some(matchInArray);
							if (bFound) {
								iMonth = oFound.index;
							} else {
								checkValid(oPart.type, true);
							}
						}
						break;
					case "quarter":
					case "quarterStandalone":
						if (oPart.digits < 3) {
							sPart = findNumbers(Math.max(oPart.digits, 2));
							checkValid(oPart.type, sPart === "");
							iQuarter = parseInt(sPart, 10) - 1;
							iIndex += sPart.length;
							if (bStrict && iQuarter > 3) {
								bValid = false;
							}
						} else {
							bFound = aQuartersVariants.some(matchInArray);
							if (bFound) {
								iQuarter = oFound.index;
							} else {
								checkValid(oPart.type, true);
							}
						}
						break;
					case "era":
						bFound = aErasVariants.some(matchInArray);
						if (bFound) {
							iEra = oFound.index;
						} else {
							checkValid(oPart.type, true);
							iEra = iCurrentEra;
						}
						break;
					case "year":
					case "weekYear":
						if (oPart.digits === 1) {
							sPart = findNumbers(4);
						} else if (oPart.digits === 2) {
							sPart = findNumbers(2);
						} else {
							sPart = findNumbers(oPart.digits);
						}
						iIndex += sPart.length;
						checkValid(oPart.type, sPart === "");
						iYear = parseInt(sPart, 10);
						// Find the right century for two-digit years
						if (sCalendarType !== CalendarType.Japanese && sPart.length <= 2) {
							var oCurrentDate = UniversalDate.getInstance(new Date(), sCalendarType),
								iCurrentYear = oCurrentDate.getFullYear(),
								iCurrentCentury = Math.floor(iCurrentYear / 100),
								iYearDiff = iCurrentCentury * 100 + iYear - iCurrentYear;
							if (iYearDiff < -70) {
								iYear += (iCurrentCentury + 1) * 100;
							} else if (iYearDiff < 30 ) {
								iYear += iCurrentCentury * 100;
							} else {
								iYear += (iCurrentCentury - 1) * 100;
							}
						}
						break;
					case "weekInYear":
						if (oPart.digits < 3) {
							sPart = findNumbers(2);
							iIndex += sPart.length;
							checkValid(oPart.type, !sPart);
						} else {
							sPart = this.oLocaleData.getCalendarWeek(oPart.digits === 3 ? "narrow" : "wide");
							sPart = sPart.replace("{0}", "[0-9]+");
							var rWeekNumber = new RegExp(sPart),
								oResult = rWeekNumber.exec(oValue.substring(iIndex));
							if (oResult) {
								iIndex += oResult[0].length;
							} else {
								checkValid(oPart.type, true);
							}
						}
						break;
					case "hour0_23":
						sPart = findNumbers(Math.max(oPart.digits, 2));
						checkValid(oPart.type, sPart === "");
						iIndex += sPart.length;
						iHours = parseInt(sPart, 10);
						if (bStrict && iHours > 23) {
							bValid = false;
						}
						break;
					case "hour1_24":
						sPart = findNumbers(Math.max(oPart.digits, 2));
						checkValid(oPart.type, sPart === "");
						iIndex += sPart.length;
						iHours = parseInt(sPart, 10);
						if (iHours === 24) {
							iHours = 0;
						}
						if (bStrict && iHours > 23) {
							bValid = false;
						}
						break;
					case "hour0_11":
						sPart = findNumbers(Math.max(oPart.digits, 2));
						checkValid(oPart.type, sPart === "");
						iIndex += sPart.length;
						iHours = parseInt(sPart, 10);
						if (bStrict && iHours > 11) {
							bValid = false;
						}
						break;
					case "hour1_12":
						sPart = findNumbers(Math.max(oPart.digits, 2));
						checkValid(oPart.type, sPart === "");
						iIndex += sPart.length;
						iHours = parseInt(sPart, 10);
						if (iHours === 12) {
							iHours = 0;
							// 12:00 defaults to 12:00 PM
							bPM = true;
						}
						if (bStrict && iHours > 11) {
							bValid = false;
						}
						break;
					case "minute":
						sPart = findNumbers(Math.max(oPart.digits, 2));
						checkValid(oPart.type, sPart === "");
						iIndex += sPart.length;
						iMinutes = parseInt(sPart, 10);
						if (bStrict && iMinutes > 59) {
							bValid = false;
						}
						break;
					case "second":
						sPart = findNumbers(Math.max(oPart.digits, 2));
						checkValid(oPart.type, sPart === "");
						iIndex += sPart.length;
						iSeconds = parseInt(sPart, 10);
						if (bStrict && iSeconds > 59) {
							bValid = false;
						}
						break;
					case "fractionalsecond":
						sPart = findNumbers(oPart.digits);
						iIndex += sPart.length;
						sPart = sPart.substr(0, 3);
						sPart = jQuery.sap.padRight(sPart, "0", 3);
						iMilliseconds = parseInt(sPart, 10);
						break;
					case "amPmMarker":
						var sAM = this.aDayPeriods[0],
							sPM = this.aDayPeriods[1];
						if (oValue.indexOf(sAM, iIndex) === iIndex) {
							bPM = false;
							iIndex += sAM.length;
						} else if (oValue.indexOf(sPM, iIndex) === iIndex) {
							bPM = true;
							iIndex += sPM.length;
						}
						break;
					case "timezoneGeneral": //e.g. GMT-02:00 or GMT+02:00
						var oTZ = oValue.substring(iIndex, iIndex + 3);
						if (oTZ === "GMT" || oTZ === "UTC") {
							iIndex = iIndex + 3;
						} else if (oValue.substring(iIndex, iIndex + 2) === "UT") {
							iIndex = iIndex + 2;
						} else if (oValue.charAt(iIndex) === "Z") {
							iIndex = iIndex + 1;
							iTZDiff = 0;
							break;
						} else {
							jQuery.sap.log.error(oValue + " cannot be parsed correcly by sap.ui.core.format.DateFormat: The given timezone is not supported!");
							break;
						}
						// falls through
					case "timezoneISO8601": //e.g. -02:00 or +02:00 or Z
						if (oValue.charAt(iIndex) === "Z") {
							iIndex = iIndex + 1;
							iTZDiff = 0;
							break;
						}
						parseTZ(true);
						break;
					case "timezoneRFC822": //e.g. -0200 or +0200
						parseTZ(false);
						break;

				}
				if (!bValid) {
					break;
				}
			}

			// If input value has not been completely parsed, mark it as invalid
			if (iIndex < oValue.length) {
				bValid = false;
			}

			if (bPM) {
				iHours += 12;
			}

			if (iQuarter !== null && iMonth === null && iDay === null) {
				iMonth = 3 * iQuarter;
				iDay = 1;
			}

			if (bValid) {
				if (bUTC || iTZDiff != null) {
					oDate = UniversalDate.getInstance(new Date(0), sCalendarType);
					oDate.setUTCEra(iEra || UniversalDate.getCurrentEra(sCalendarType));
					oDate.setUTCFullYear(iYear || 1970);
					oDate.setUTCMonth(iMonth || 0);
					oDate.setUTCDate(iDay || 1);
					oDate.setUTCHours(iHours || 12);//nibdey: If time is not provided then set default time to 12:00:00 to avoid time-zone problem and date showing up 1 day off
					oDate.setUTCMinutes(iMinutes || 0);
					oDate.setUTCSeconds(iSeconds || 0);
					oDate.setUTCMilliseconds(iMilliseconds || 0);
					if (bStrict && (iDay || 1) !== oDate.getUTCDate()) {
						// check if valid date given - if invalid, day is not the same (31.Apr -> 1.May)
						bValid = false;
						oDate = undefined;
					} else if (iTZDiff) {
						// Set TZDiff after checking for valid day, as it may switch the day as well
						oDate.setUTCMinutes((iMinutes || 0) + iTZDiff);
					}
				} else {
					oDate = UniversalDate.getInstance(new Date(1970, 0, 1, 0, 0, 0), sCalendarType);
					oDate.setEra(iEra || UniversalDate.getCurrentEra(sCalendarType));
					oDate.setFullYear(iYear || 1970);
					oDate.setMonth(iMonth || 0);
					oDate.setDate(iDay || 1);
					oDate.setHours(iHours || 12);//nibdey: If time is not provided then set default time to 12:00:00 to avoid time-zone problem and date showing up 1 day off
					oDate.setMinutes(iMinutes || 0);
					oDate.setSeconds(iSeconds || 0);
					oDate.setMilliseconds(iMilliseconds || 0);
					if (bStrict && (iDay || 1) !== oDate.getDate()) {
						// check if valid date given - if invalid, day is not the same (31.Apr -> 1.May)
						bValid = false;
						oDate = undefined;
					}
				}

				if (bValid) {
					oDate = oDate.getJSDate();
					return oDate;
				}
			}

			if (!this.bIsFallback) {
				jQuery.each(this.aFallbackFormats, function(i, oFallbackFormat) {
					oDate = oFallbackFormat.parse(oValue, bUTC, bStrict);
					if (oDate) {
						return false;
					}
				});
				return oDate;
			}

			return null;

		};
	
	return RiskDateFormat;

}, /* bExport= */ true);
