// Provides class sap.ui.core.format.NumberFormat
sap.ui.define(['jquery.sap.global', 'sap/ui/core/format/NumberFormat'],
	function(jQuery, NumberFormat) {
	"use strict";
	
	var RiskNumberFormat = function() {
		// Do not use the constructor
		throw new Error();
	};
	
	RiskNumberFormat.getInstance = function(oFormatOptions, oLocale) {
		var numberFormatInstance=NumberFormat.getInstance(oFormatOptions, oLocale);
		numberFormatInstance.format=RiskNumberFormat.prototype.format;
		numberFormatInstance.parse=RiskNumberFormat.prototype.parse;
		return numberFormatInstance;
	};
	RiskNumberFormat.getFloatInstance = function(oFormatOptions, oLocale) {
		var numberFloatFormatInstance=NumberFormat.getFloatInstance(oFormatOptions, oLocale);
		numberFloatFormatInstance.format=RiskNumberFormat.prototype.format;
		numberFloatFormatInstance.parse=RiskNumberFormat.prototype.parse;
		return numberFloatFormatInstance;
	};
	
	//Overriding format from NumberFormat with few changes marked with comments (CPB: override)
	RiskNumberFormat.prototype.format = function(oValue, sMeasure) {
		var sNumber = this.convertToDecimal(oValue),
		sIntegerPart = "",
		sFractionPart = "",
		sGroupedIntegerPart = "",
		sResult = "",
		iPosition = 0,
		iLength = 0,
		bNegative = oValue < 0,
		iDotPos = -1,
		oOptions = this.oFormatOptions;

		if (sNumber === "NaN") {
			return sNumber;
		}
		
		//handle measure
		if (oOptions.type === NumberFormat.CURRENCY) {
			var iDigits = this.oLocaleData.getCurrencyDigits(sMeasure);
			oOptions.maxFractionDigits = iDigits;
			oOptions.minFractionDigits = iDigits;
		}
		
		// if number is negative remove minus
		if (bNegative) {
			sNumber = sNumber.substr(1);
		}

		//CPB: override - start
		//Handle rounding : HALF-EVEN
		function round(num, maxFractionDigits){
//			var intDigits = num.substr(0, num.indexOf("."));
			var fracDigits = num.substr(num.indexOf(".") + 1);
			var n = Math.pow(10, maxFractionDigits);
			if(fracDigits.length > maxFractionDigits){
				if(num.charAt(num.length - 1) === "5"){
					var prevStr = num.substr(num.length-2,1);
					var prev = parseInt(prevStr);
					return ((prev % 2) === 0) ? Math.floor(num * n)/n : Math.ceil(num * n)/n;
				}
			}
			return Math.round(num * n) / n;
		}
		//round using fraction digit = min (current number fraction digit , maxFractionDigit)
		var maxFracDigits=oOptions.maxFractionDigits;
		if (sNumber.indexOf(".") > -1) {
			var fraction = sNumber.substr(sNumber.indexOf(".") + 1);
			if(fraction.length > maxFracDigits){
				sNumber = round(sNumber, maxFracDigits).toString();
			}
		}
   	//CPB: override - end
		
		// if number contains fraction, extract it
		iDotPos = sNumber.indexOf(".");
		if (iDotPos > -1) {
			sIntegerPart = sNumber.substr(0, iDotPos);
			sFractionPart = sNumber.substr(iDotPos + 1);
		}
		else {
			sIntegerPart = sNumber;
		}

		// integer part length
		if (sIntegerPart.length < oOptions.minIntegerDigits) {
			sIntegerPart = jQuery.sap.padLeft(sIntegerPart, "0", oOptions.minIntegerDigits);
		}
		else if (sIntegerPart.length > oOptions.maxIntegerDigits) {
			sIntegerPart = jQuery.sap.padLeft("", "?", oOptions.maxIntegerDigits);
		}

		
		// fraction part length
		if (sFractionPart.length < oOptions.minFractionDigits) {
//			sFractionPart = jQuery.sap.padRight(sFractionPart, "0", oOptions.minFractionDigits); //CPB: override
		}
		else if (sFractionPart.length > oOptions.maxFractionDigits) {
			sFractionPart = sFractionPart.substr(0, oOptions.maxFractionDigits);
		}

		// grouping
		iLength = sIntegerPart.length;
		if (oOptions.groupingEnabled && iLength > 3) {
			iPosition = iLength % 3 || 3;
			sGroupedIntegerPart = sIntegerPart.substr(0, iPosition);
			while (iPosition < sIntegerPart.length) {
				sGroupedIntegerPart += oOptions.groupingSeparator;
				sGroupedIntegerPart += sIntegerPart.substr(iPosition, 3);
				iPosition += 3;
			}
			sIntegerPart = sGroupedIntegerPart;
		}

		// combine
		if (bNegative) {
			sResult = oOptions.minusSign;
		}
		sResult += sIntegerPart;
		if (sFractionPart) {
			sResult += oOptions.decimalSeparator + sFractionPart;
		}
		
		if (sMeasure && oOptions.showMeasure) {
			if (oOptions.type === NumberFormat.CURRENCY) {
//				var sCurrencyPlaceholder = '\u00a4';
				var sPattern = oOptions.pattern;
				
				sPattern = sPattern.replace(/\u00a4/, this.oLocaleData.getCurrencySymbol(sMeasure));
				sPattern = sPattern.replace(/[0#.,]+/, sResult);
				
				sResult = sPattern;
			}
		}

		if (sap.ui.getCore().getConfiguration().getOriginInfo()) {
			sResult = new String(sResult);// eslint-disable-line no-new-wrappers
			sResult.originInfo = {
				source: "Common Locale Data Repository",
				locale: this.oLocale.toString()
			};
		}

		return sResult;	
	};
	
	RiskNumberFormat.prototype.parse = function(sValue) {
		var oOptions = this.oFormatOptions,
			sRegExpFloat = "^\\s*([+-]?(?:[0-9\\" + oOptions.groupingSeparator + "]+|[0-9\\" + oOptions.groupingSeparator + "]*\\" + oOptions.decimalSeparator + "[0-9]+)([eE][+-][0-9]+)?)\\s*$",
			sRegExpInt = "^\\s*([+-]?[0-9\\" + oOptions.groupingSeparator + "]+)\\s*$",
			oGroupingRegExp = new RegExp("\\" + oOptions.groupingSeparator, "g"),
			oDecimalRegExp = new RegExp("\\" + oOptions.decimalSeparator, "g"),
			oRegExp,
			oResult = 0;
		
		// Check for valid syntax
		if (oOptions.isInteger) {
			oRegExp = new RegExp(sRegExpInt);
		} else {
			oRegExp = new RegExp(sRegExpFloat);
		}
		if (!oRegExp.test(sValue)) {
			return NaN;
		}
		
		// Remove grouping separator and replace locale dependant decimal separator, 
		// before calling parseInt/parseFloat
		sValue = sValue.replace(oGroupingRegExp, "");
		if (oOptions.isInteger) {
			oResult = parseInt(sValue, 10);
		}
		else {
			sValue = sValue.replace(oDecimalRegExp, ".");
			oResult = parseFloat(sValue);
		}
		return oResult;
	};


	return RiskNumberFormat;

}, /* bExport= */ true);