jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.FetchCriteria");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.AndFilter");

sas.risk.common.widgets.spreadsheet.FetchCriteria=function(){
	this.filterSimple=null;
	this.filterAdvanced=null;
	this.sorts=[];
	this.columnIds=[];
};
sas.risk.common.widgets.spreadsheet.FetchCriteria.prototype={
	clear:function(){
		this.filterSimple=null;
		this.filterAdvanced=null;
		this.sorts=[];
		this.columnIds=[];
	},
	setColumnIds:function(columnIds){
		this.columnIds=columnIds;
	},
	getColumnIds:function(){
		return this.columnIds;
	},
	addSort:function(columnId,sortOrder){
		this.sorts.push({columnId:columnId,sortOrder:sortOrder});
	},
	getSorts:function(){
		return this.sorts;
	},
	getFilter:function(){
		if(this.filterAdvanced && this.filterSimple){
			return new sas.risk.common.widgets.spreadsheet.filter.AndFilter([this.filterAdvanced,this.filterSimple]);
		}
		return this.filterAdvanced||this.filterSimple;
	},
	getFilterSimple:function(){
		return this.filterSimple;
	},
	andSimpleFilter:function(filter){
		if(this.filterSimple){
			if(this.filterSimple instanceof sas.risk.common.widgets.spreadsheet.filter.AndFilter){
				this.filterSimple.addFilter(filter);
			}else{
				this.filterSimple=new sas.risk.common.widgets.spreadsheet.filter.AndFilter([this.filterSimple,filter]);
			}
		}else{
			this.filterSimple=filter;
		}
	},
	buildQueryParameters:function(meta){
		var queryParams={};
		var filter=this.getFilter();
		if(filter){
			queryParams.filter=filter.toString();
		}
		if(this.sorts.length){
			var sortColumnOptions=meta.getColumnOptions(this.sorts[0].columnId);
			if(sortColumnOptions.sortProperty && this.sorts[0].sortOrder){
				queryParams.sortBy=sortColumnOptions.sortProperty+":"+this.sorts[0].sortOrder.toLowerCase();
			}
		}
		if(this.columnIds.length){
			queryParams.fields=this.columnIds.join(",");
			queryParams.fields = queryParams.fields + ",links";
		}
		
		return $.param(queryParams);
	},
	equals:function(other){
		var filter=this.getFilter();
		var equals=filter?(this.getFilter().equals(other.getFilter())):!other.getFilter();
		equals&=(this.sorts.length===other.sorts.length);//eslint-disable-line no-bitwise
		if(equals && this.sorts.length){
			equals&=(this.sorts[0].columnId===other.sorts[0].columnId);//eslint-disable-line no-bitwise
			equals&=(this.sorts[0].sortOrder===other.sorts[0].sortOrder);//eslint-disable-line no-bitwise
		}
		equals&=(this.columnIds.join("|")===other.columnIds.join("|"));//eslint-disable-line no-bitwise
		return !!equals;
	},
	clone:function(){
		var fetchCriteria=new sas.risk.common.widgets.spreadsheet.FetchCriteria();
		fetchCriteria.filterSimple=this.filterSimple?this.filterSimple.clone():null;
		fetchCriteria.filterAdvanced=this.filterAdvanced?this.filterAdvanced.clone():null;
		fetchCriteria.sorts=$.extend(true,[],this.sorts);
		fetchCriteria.columnIds=$.extend([],this.columnIds);
		return fetchCriteria;
	}
};
//STATICS
sas.risk.common.widgets.spreadsheet.FetchCriteria.loadFetchCriteria=function(json){
	var fc=new sas.risk.common.widgets.spreadsheet.FetchCriteria();
	fc.filterSimple=sas.risk.common.widgets.spreadsheet.filter.AFilter.hydrate(json.filterSimple);
	fc.filterAdvanced=sas.risk.common.widgets.spreadsheet.filter.AFilter.hydrate(json.filterAdvanced);
	if(fc.filterAdvanced){fc.filterAdvanced.isAdvancedFilter=true;}
	if(json.sortColumnId && json.sortOrder){
		fc.addSort(json.sortColumnId,json.sortOrder);
	}
	fc.setColumnIds(json.columnIds);
	return fc;
};