jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory");

sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory=function(){
	this.registry={};
	this.cache={};
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.prototype={
	register:function(type,klassName){
		this.registry[type]=klassName;
	},
	request:function(type){
		var typeCache=this.cache[type];
		if(typeCache && typeCache.length){
			return typeCache.pop();
		}else{
			var klassName=this.registry[type];
			if(!klassName){
				console.error("No Spreadsheet Cell of type "+type+" has been registered. Default to string.");// eslint-disable-line no-console
				klassName=this.registry["string"];
			}
			jQuery.sap.require(klassName);
			var klass=sap.ui.require(klassName.replace(/\./g, "/"));
			sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.counter++;
			return new klass({cellType:type});
		}
	},
	release:function(control){
		var type=control.getCellType();
		var typeCache=this.cache[type]||[];
		control.clearCell();
		typeCache.push(control);
		this.cache[type]=typeCache;
	}
};
//STATICS
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.counter=0;
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory._instance=new sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory();
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance=function(){
	return sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory._instance;
};

sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("date","sas.risk.common.widgets.spreadsheet.control.DateCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("timestamp","sas.risk.common.widgets.spreadsheet.control.DateTimeCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("date_clear","sas.risk.common.widgets.spreadsheet.control.DateCellClear");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("timestampX","sas.risk.common.widgets.spreadsheet.control.DateTimeCellX");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("number","sas.risk.common.widgets.spreadsheet.control.NumberCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("number_0","sas.risk.common.widgets.spreadsheet.control.NumberCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("number_2","sas.risk.common.widgets.spreadsheet.control.NumberCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("number_4","sas.risk.common.widgets.spreadsheet.control.NumberCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("percent","sas.risk.common.widgets.spreadsheet.control.PercentCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("string","sas.risk.common.widgets.spreadsheet.control.StringCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("dropdown","sas.risk.common.widgets.spreadsheet.control.DropdownCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("formula","sas.risk.common.widgets.spreadsheet.control.FormulaCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("formula_string","sas.risk.common.widgets.spreadsheet.control.FormulaStringCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("icon","sas.risk.common.widgets.spreadsheet.control.IconCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("array_length","sas.risk.common.widgets.spreadsheet.control.ArrayLengthCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("checkbox","sas.risk.common.widgets.spreadsheet.control.CheckboxCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("dual","sas.risk.common.widgets.spreadsheet.control.DualSelectorCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("com.sas.oprisk.server.User","sas.risk.common.widgets.spreadsheet.control.UserCell");

sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("java.lang.String","sas.risk.common.widgets.spreadsheet.control.StringCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("java.util.Date","sas.risk.common.widgets.spreadsheet.control.DateCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("java.lang.Double","sas.risk.common.widgets.spreadsheet.control.NumberCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("java.sql.Timestamp","sas.risk.common.widgets.spreadsheet.control.DateTimeCell");

sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("qb_TEMP.QBMETA","sas.risk.common.widgets.spreadsheet.control.QueryBuilderCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("disabled","sas.risk.common.widgets.spreadsheet.control.DisabledCell");
sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register("ee_TEMP.EEMETA","sas.risk.common.widgets.spreadsheet.control.ExpressionEditorCell");

for(var i=1;i<=250;i++){
	var customType = "com.sas.oprisk.server.CustomObject"+i;
	sas.risk.common.widgets.spreadsheet.SpreadsheetCellFactory.getInstance().register(customType,"sas.risk.common.widgets.spreadsheet.control.CustomObjectCell");
}
