jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo");

sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo=function(sheetId,columnId,cellId,spreadsheet){
	this.sheetId=sheetId;
	this.columnId=columnId;
	this.cellId=cellId;
	this.spreadsheet=spreadsheet;
	this.meta=spreadsheet.getMetaInfo(sheetId).meta;
	this.state=spreadsheet.getState();
	this.cellKey=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.createKey(sheetId,columnId,cellId);//initialize cell key
	this.dependsOn=[];//these i depend on
	this.dependents=[];//these depend on me
	this.init();
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.prototype={
	init:function(){
		this.initialized=false;
		var table=this.spreadsheet.getSheetTable(this.sheetId);
		this.columnIndex=-1;
		var that=this;
		_.find(table._getVisibleColumns(),function(col,index){
			that.columnIndex=index;
			return col.getName()===that.columnId;
		});
		var row=_.find(table.getRows(),function(row){
			return row.getCells()[0].getCellId()===that.cellId;
		});
		if(!row){return;}
		this.rowIndex=row.getIndex();
		this.aliasRef=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.calcColIndexToAlias(this.columnIndex);
		this.aliasRef+=(this.rowIndex+1);//1-based
		this.initialized=true;
	},
	getSInitialized:function(){
		return this.initialized;
	},
	getSCellKey:function(){
		return this.cellKey;
	},
	getSColumnIndex:function(){
		return this.columnIndex;
	},
	getSRowIndex:function(){
		return this.rowIndex;
	},
	getSSheetId:function(){
		return this.sheetId;
	},
	getSCellId:function(){
		return this.cellId;
	},
	getSColumnId:function(){
		return this.columnId;
	},
	getSOptions:function(){
		return this.meta.getCellOptions(this.columnId,this.cellId);
	},
	getSModel:function(){
		return this.spreadsheet.getModel(this.sheetId);
	},
	getValue:function(){
		return this.getSModel().getValue(this.cellId,this.columnId,this.getSOptions());
	},
	setValue:function(newValue){
		this.getSModel().setValue(this.cellId,this.columnId,newValue,this.getSOptions());
	},
	getSMeta:function(){
		return this.meta;
	},
	getSState:function(){
		return this.state;
	},
	getSAliasReference:function(){
		return this.aliasRef;
	},
	aliasExpression:function(expr){
		var nameExprs=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr,"NAME");
		var that=this;
		_.each(nameExprs,function(nameExpr){
			nameExpr.name=that.resolveCellName(nameExpr.name);
		});
	},
	resolveExpression:function(expr){
		var nameExprs=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr,"NAME");
		var that=this;
		_.each(nameExprs,function(nameExpr){
			nameExpr.name=that.resolveCellAlias(nameExpr.name);
		});
	},
	analyzeExpression:function(expr,commit){
		var keys=this._getDependsOnKeys(expr);
		var state=this.state;
		var chain=[this];
		_.each(keys,function(key){
			state.getCell(key).circularCheck(chain);
		});
		if(commit){
			this.setCellDependency(keys);
		}
	},
	_getDependsOnKeys:function(expr){
		var nameExprs=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr,"NAME");
		var keys=[];
		var that=this;
		_.each(nameExprs,function(nameExpr){
			nameExpr.name=that.resolveCellName(nameExpr.name);
			if(sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.matchCellNameType(nameExpr.name)){
				keys.push(nameExpr.name);
			}
		});
		return keys;
	},
	evaluateExpression:function(){
		var def=$.Deferred();
		try{
			var value=this.getValue();
			var expr=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.parse(value);
			this.aliasExpression(expr);
			var that=this;
			return sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.evaluate(expr,this.state.getFormulaContext()).done(function(result){
				that.state.setCachedValue(that.sheetId,that.cellKey,result);
				def.resolve(result);
			}).fail(function(error){
				console.error(error);// eslint-disable-line no-console
				def.reject(error);
			});
		}catch(error){
			console.error(error);// eslint-disable-line no-console
			def.reject(error);
		}
		return def;
	},
	//DEPENDENCY METHODS
	addDependent:function(cell){
		this.dependents.push(cell);
	},
	removeDependent:function(cell){
		this._removeCell(cell,this.dependents);
	},
	addDependsOn:function(cell){
		this.dependsOn.push(cell);
	},
	removeDependsOn:function(cell){
		this._removeCell(cell,this.dependsOn);
	},
	_removeCell:function(cell,array){
		var index=array.indexOf(cell);
		if(index!==-1){
			array.splice(index,1);
		}
	},
	circularCheck:function(chain){
		chain=chain||[];
		if(chain.indexOf(this)!==-1){
			throw {message:"circular dependency",cell1:chain[0],cell2:this};// eslint-disable-line shikari/sas-i18n-ems 
		}
		chain.push(this);
		_.each(this.dependsOn,function(cell){
			cell.circularCheck(chain);
		});
		chain.pop();
	},
	setCellDependency:function(keys){
		var that=this;
		_.each(this.dependsOn,function(cell){
			that.removeDependsOn(cell);
			cell.removeDependent(that);
		});
		_.each(keys,function(key){
			var cell=that.state.getCell(key);
			cell.addDependent(that);
			that.addDependsOn(cell);
		});
		_.each(this.dependents,function(cell){
			that.state.clearCachedValue(cell.getSSheetId(),cell.getSCellKey());
		});
		_.each(this.getParentKeys(),function(parentKey){
			that.state.clearCachedValue(that.sheetId,parentKey);
		});
	},
	getParentKeys:function(){
		var parents=[];
		var path=this.state.getRowPathFromKey(this.sheetId,this.cellId);
		var prefix="/"+this.getSModel().getBindId()+"/";
		if(path.indexOf(prefix)===0){
			var paths=path.substring(prefix.length).split("/");
			while(true){
				paths.pop();
				if(paths.length){
					var parentPath=prefix+paths.join("/");
					parents.push(this.state.getRowKeyFromPath(this.sheetId,parentPath));
				}else{
					break;
				}
			}
		}
		return parents;
	},
	resolveCellName:function(alias){
		if(sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo._isStringType(alias)){
			// return alias.substring(1,alias.length-1).split('""').join('"');
			return alias.split('""').join('"');
		}
		if($.isNumeric(alias)){
			return parseFloat(alias);
		}
		if(sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.matchCellNameType(alias)){
			return alias;
		}
		var match=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.matchCellNameAlias(alias);
		if(match){
			if(!match[1]){
				match[1]=this.sheetId;
			}
			var sheetId=match[1];
			var columnId=this.state.getColumnId(sheetId,match[2]);//ADD SHEETID HERE AND HERE AND HERE
			var rowId=this.state.getRowId(sheetId,match[3]);
			if(sheetId && columnId && rowId){
				return sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.createKey(sheetId,columnId,rowId);
			}else{
				throw "not a valid cell reference";
			}
		}else{
			throw "not a valid name";
		}
	},
	resolveCellAlias:function(name){
		var match=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.matchCellNameType(name);
		if(match){
			var sheetId=match[1]||this.sheetId;
			var columnId=match[2];
			var cellId=match[3];
			var columnAlias=this.state.getColumnAlias(sheetId,columnId);
			if(columnAlias){
				var rowAlias=this.state.getRowAlias(sheetId,cellId);
				if(rowAlias){
					if(sheetId===this.sheetId){
						return columnAlias+rowAlias;
					}else{
						sheetAlias=sheetId.indexOf("'")!==-1?"'"+sheetId+"'":sheetId;
						return sheetId+"!"+columnAlias+rowAlias;
					}
				}
			}
		}
		return name;
	}
};
//STATICS
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.createKey=function(sheetId,columnId,cellId){
	sheetId=sheetId.split("'").join("''");
	return (sheetId.indexOf("!")===-1)?sheetId+"!"+columnId+"|"+cellId:
											 "'"+sheetId+"'!"+columnId+"|"+cellId;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.REGEX_CELLALIAS=/^(?:([^!]+)\!)?([A-Z]+)([0-9]+)$/g;
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.REGEX_QUOTED_CELLALIAS=/^(?:'(.+)'\!)?([A-Z]+)([0-9]+)$/g;
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.REGEX_CELLREF=/^(?:([^!]+)\!)?([^|]+)\|(.+)$/g;
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.REGEX_QUOTED_CELLREF=/^(?:'(.+)'\!)?([^|!]+)\|(.+)$/g;
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.parseKey=function(key,spreadsheet){
//	var index1=key.indexOf("!");
	var match=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.matchCellNameType(key);
	var sheetId=match[1]?match[1].split("''").join("'"):spreadsheet.getSelectedSheetId();
	var columnId=match[2];
	var cellId=match[3];
	return new sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo(sheetId,columnId,cellId,spreadsheet);
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo._exec=function(regexName,txt){
	var regex=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo[regexName];
	var value=regex.exec(txt);
	if(value!==null){
		while(regex.exec(txt)!==null){}
	}
	return value;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.matchCellNameType=function(value){
	var match=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo._exec("REGEX_QUOTED_CELLREF",value);
	if(!match){
		match=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo._exec("REGEX_CELLREF",value);
	}
	return match;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.matchCellNameAlias=function(value){
	var match=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo._exec("REGEX_QUOTED_CELLALIAS",value);
	if(!match){
		match=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo._exec("REGEX_CELLALIAS",value);
	}
	return match;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo._isStringType=function(value){
	if(value.length>1 && value[0]==='"' && value[value.length-1]==='"'){
		var stripped=value.substring(1,value.length-1);
		var chunks=stripped.split('""');
		var badChunk=_.find(chunks,function(chunk){
			return chunk.indexOf('"')!==-1;
		});
		return !badChunk;
	}
	return false;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.calcColIndexToAlias=function(num){ 
   var floor=Math.floor(num/26);
   var result="";
   if(floor>0){
      result+=this.calcColIndexToAlias(floor-1);
   }
   result+=String.fromCharCode(num%26+65);
   return result;
};