jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.SpreadsheetMeta");
jQuery.sap.require("sap.ui.base.EventProvider");

sap.ui.base.EventProvider.extend("sas.risk.common.widgets.spreadsheet.SpreadsheetMeta",{
	metadata:{
		events:{
			change:{}
		}
	},
	constructor:function(settings,id){
		sap.ui.base.EventProvider.apply(this);
		settings=this.validateSettings(settings);
		this.tableId=id||settings.table.id||"abc";
		this.tableOptions=settings.table.options||{};
		this.baseColumnOptions=settings.table.columnOptions||{};
		this.baseCellOptions=settings.table.cellOptions||{};
		var that=this;
		this.columnOptions={};
		this.columnIds=[];
		_.each(settings.columns,function(options){
			var opts=$.extend({},that.baseColumnOptions,options.options);
			sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.processMultiLabels(opts);		
			that.columnOptions[options.options.id]=opts;
			that.columnIds.push(options.options.id);
		});
		this.columnCellOptions={};
		_.each(settings.columns,function(options){
			var opts=$.extend({},that.baseCellOptions,options.cellOptions||{});
			that.columnCellOptions[options.options.id]=opts;
		});
		this.cellOptions={};
		_.each(settings.cells||{},function(columns,cellId){
			that.cellOptions[cellId]={};
			_.each(columns,function(options,columnId){
				that.cellOptions[cellId][columnId]=$.extend({},that.columnCellOptions[columnId],options);
			});
		});
	}
});
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.validateSettings=function(settings){
	if(!settings || !settings.table || !settings.columns){
		console.error("Attempt to use an invalid Meta.",settings);// eslint-disable-line no-console
		return {table:{},columns:[]};
	}
	return settings;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.toJson=function(){
	var json={table:{},columns:[]};
	json.table.id=this.tableId;
	json.table.options=this.tableOptions;
	json.table.columnOptions=this.baseColumnOptions;
	json.table.cellOptions=this.baseCellOptions;
	var that=this;
	_.each(this.columnIds,function(columnId){
		var coljson={options:{}};
		_.each(that.columnOptions[columnId],function(colOptionValue,colOptionName){
			if(that.baseColumnOptions[colOptionName]!==colOptionValue){
				coljson.options[colOptionName]=colOptionValue;
			}
		});
		_.each(that.columnCellOptions[columnId],function(colCellOptionValue,colCellOptionName){
			if(that.baseCellOptions[colCellOptionName]!==colCellOptionValue){
				coljson.cellOptions=coljson.cellOptions||{};
				coljson.cellOptions[colCellOptionName]=colCellOptionValue;
			}
		});
		json.columns.push(coljson);
	});
	if(_.size(this.cellOptions)>0){
		json.cells={};
		_.each(this.cellOptions,function(cellOptionValues,cellId){
			json.cells[cellId]={};
			_.each(that.cellOptions[cellId],function(cellColOptionValues,colId){
				json.cells[cellId][colId]={};
				_.each(that.cellOptions[cellId][colId],function(cellOptionValue,cellOptionName){
					if(that.columnCellOptions[colId][cellOptionName]!==cellOptionValue){
						json.cells[cellId][colId][cellOptionName]=cellOptionValue;
					}
				});
			});
		});
	}
	return json;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.getTableId=function(){
	return this.tableId;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.getColumnIds=function(){
	return this.columnIds;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.getTableOptions=function(){
	return this.tableOptions;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.getColumnOptions=function(columnId){
	return columnId?this.columnOptions[columnId]:this.columnOptions;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.getCellOptions = function(columnId, rowId) {
	var cellOptions = this.cellOptions[rowId];
	if (!cellOptions && this.tableOptions.cellOverrideisRegex) {
		var matchingCellOptions = _.find(this.cellOptions, function(val, key) {
			var regOExpObj = new RegExp(key);
			return regOExpObj.test(rowId);
		});
		cellOptions = matchingCellOptions;
	}
	return (cellOptions && cellOptions[columnId]) || this.columnCellOptions[columnId];
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.setCellOption=function(option,value,columnId,cellId){
	var cellOptionsByColumn=this.cellOptions[cellId]||{};
	if(!cellOptionsByColumn[columnId]){
		cellOptionsByColumn[columnId]=$.extend({},this.columnCellOptions[columnId]);
	}
	cellOptionsByColumn[columnId][option]=value;
	this.cellOptions[cellId]=cellOptionsByColumn;
	this.fireChange({type:"cellOption",meta:this,tableId:this.tableId,option:option,value:value,columnId:columnId,cellId:cellId});
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.overrideVisibleColumnIds=function(visibleColumnIds){
	var columnIds=this.getColumnIds();
	var that=this;
	_.each(columnIds,function(columnId){
		var columnOptions=that.getColumnOptions(columnId);
		columnOptions.visible=(visibleColumnIds.indexOf(columnId)!==-1);
	});
	_.each(visibleColumnIds,function(columnName){
		var index=columnIds.indexOf(columnName);
		columnIds.splice(index,1);
	});
	columnIds.unshift.apply(columnIds,visibleColumnIds);
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.attachChange=function(oData, fnFunction, oListener){
	this.attachEvent("change", oData, fnFunction, oListener);
	return this;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.detachChange=function(fnFunction, oListener) {
	this.detachEvent("change", fnFunction, oListener);
	return this;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.prototype.fireChange=function(mArguments) {
	this.fireEvent("change", mArguments);
	return this;
};
sas.risk.common.widgets.spreadsheet.SpreadsheetMeta.processMultiLabels=function(options){
	if(_.isArray(options.label)){
		options.multiLabels=[];
		_.each(options.label,function(label){
			options.multiLabels.push(new sas.hc.m.Label({text:label}));
		});
		delete options.label;
	}
};