jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.SpreadsheetState");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.control.FormulaCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo");

sas.risk.common.widgets.spreadsheet.SpreadsheetState=function(spreadsheet){
	this.spreadsheet=spreadsheet;
	this.isDblClick=false;
	this.inFormulaMode=false;
	this.currentFormulaCell=null;
	this.columnAliasToId={};
	this.columnIdToAlias={};
	this.rowKeyToPath={};
	this.rowPathToKey={};
	this.rowIndexToKey={};
	this.rowKeyToIndex={};
	this.formulaContext=null;
	this._cellMap={};
	this._evalCache={};
	this._focusedCellMap={};
};
sas.risk.common.widgets.spreadsheet.SpreadsheetState.prototype={
	clearCells:function(){
		this._cellMap={};
	},
	_getMap:function(objName,sheetId){
		var map=this[objName][sheetId]||{};
		this[objName][sheetId]=map;
		return map;
	},
	getCachedValue:function(sheetId,cellKey){
		return this._getMap("_evalCache",sheetId)[cellKey];
	},
	clearCachedValue:function(sheetId,cellKey){
		delete this._getMap("_evalCache",sheetId)[cellKey];
	},
	setCachedValue:function(sheetId,cellKey,value){
		return this._getMap("_evalCache",sheetId)[cellKey]={value:value};
	},
	getCell:function(cellKey,reinitialize){
		var cell=this._cellMap[cellKey];
		if(!cell){
			cell=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.parseKey(cellKey,this.spreadsheet);
			if(cell.getSInitialized()){
				this._cellMap[cellKey]=cell;
			}
		}
		if(reinitialize){
			cell.init();
		}
		return cell;
	},
	setFormulaContext:function(formulaContext){
		this.formulaContext=formulaContext;
		this.formulaContext.setState(this);
	},
	getFormulaContext:function(){
		return this.formulaContext;
	},
	// START: CELL EVENTS //
	onCellClick:function(cell){
		var sheetId=cell.cellInfo.getSSheetId();
		this.blurPreviousCell(sheetId,cell);
		this._focusedCellMap[sheetId]=cell;
		cell.focusCell();
	},
	blurPreviousCell:function(sheetId,cell){
		var prevFocusedCell=this._focusedCellMap[sheetId];
		if(prevFocusedCell && prevFocusedCell!==cell && prevFocusedCell.getDomRef()){
			prevFocusedCell.blurCell();
		}
	},
	// END: CELL EVENTS//
//	ORIGonCellClick:function(cell){
//		this.isDblClick=false;
//		if(this.inFormulaMode){
//			this.onCellClickDelayed(cell);
//		}else{
//			_.delay($.proxy(this.onCellClickDelayed,this),250,cell);
//		}
//	},
//	onCellClickDelayed:function(cell){
//		if(!this.isDblClick){
//			if(this.isInFormulaMode()){
//				this.currentFormulaCell.addCellToFormula(cell);				
//			}
//		}
//	},
//	onCellDoubleClick:function(cell){
//		this.isDblClick=true;
//		if(this.isInFormulaMode()){
//			return;
//		}else{
//			if(cell instanceof sas.risk.common.widgets.spreadsheet.control.FormulaCell && 
//				!cell.cellInfo.getSOptions().readonly){
//				this.currentFormulaCell=cell;
//				this.setFormulaMode(true);
//			}
//			cell.showEditPopup();
//		}
//	},
	clearColumnAliases:function(sheetId){
		delete this.columnAliasToId[sheetId];
		delete this.columnIdToAlias[sheetId];
	},
	clearRowAliases:function(sheetId){
		delete this.rowIndexToKey[sheetId];
		delete this.rowKeyToIndex[sheetId];
	},
	addRowPath:function(sheetId,key,path){
		this._getMap("rowKeyToPath",sheetId)[key]=path;
		this._getMap("rowPathToKey",sheetId)[path]=key;
	},
	getRowPathFromKey:function(sheetId,rowKey){
		return this._getMap("rowKeyToPath",sheetId)[rowKey];
	},
	getRowKeyFromPath:function(sheetId,path){
		return this._getMap("rowPathToKey",sheetId)[path];
	},
	addRowAlias:function(sheetId,key,index){
		this._getMap("rowIndexToKey",sheetId)[index]=key;
		this._getMap("rowKeyToIndex",sheetId)[key]=index;
	},
	addColumnAlias:function(sheetId,columnName){
		var map=this._getMap("columnIdToAlias",sheetId);
		var index=_.size(map);
		var alias=sas.risk.common.widgets.spreadsheet.SpreadsheetCellInfo.calcColIndexToAlias(index);
		map[columnName]=alias;
		this._getMap("columnAliasToId",sheetId)[alias]=columnName;
	},
	getColumnId:function(sheetId,columnAlias){
		return this._getMap("columnAliasToId",sheetId)[columnAlias];
	},
	getColumnAlias:function(sheetId,columnName){
		return this._getMap("columnIdToAlias",sheetId)[columnName]||"";
	},
	getRowAlias:function(sheetId,rowKey){
		return this._getMap("rowKeyToIndex",sheetId)[rowKey];
	},
	getRowId:function(sheetId,rowAlias){
		return this._getMap("rowIndexToKey",sheetId)[rowAlias];
	},
	setFormulaMode:function(formulaModeOn){
		this.inFormulaMode=formulaModeOn;
		var $tabs=$(this.spreadsheet.tabs.getDomRef());
		var that=this;
		if(formulaModeOn){
			$(document).on("click.spreadsheet_formula",function(e){
				if(!$(e.target).hasClass("riskSpreadsheetFormulaEditTarget") && !$tabs.find(e.target).length){
					$(document).off("click.spreadsheet_formula");
					that.currentFormulaCell.clickedBeyondFormulaScope();
				}else if($(e.target.parentElement).hasClass("sapMITBFilter")){
					var editingCellSheetId=that.currentFormulaCell.cellInfo.getSSheetId();
					var selectedTab=that.spreadsheet.tabs.getSelectedItem();
					var $editingCell=$(that.currentFormulaCell.editControl.getDomRef().parentElement);
					if(editingCellSheetId===selectedTab.getText()){
						that.currentFormulaCell.editing=true;//rerender of cell on tab revisit wipes this out
						$editingCell.css("display","flex");
					}else{
						$editingCell.css("display","none");	
					}
				}
			});
		}else{
			$(document).off("click.spreadsheet_formula");
		}
		this.spreadsheet.refreshFormulaMode();
	},
	isInFormulaMode:function(){
		return this.inFormulaMode;
	}
};