jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.ACell");
jQuery.sap.require("sap.ui.core.Control");
jQuery.sap.require("sas.hc.m.Image");
jQuery.sap.require("sap.ui.layout.VerticalLayout");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetCellStyleFactory");

sap.ui.core.Control.extend("sas.risk.common.widgets.spreadsheet.control.ACell",{
	metadata:{
		properties:{
			cellType:{type:"string"},
			readonly:{type:"boolean"},
			errorState:{type:"string"},
			errorText:{type:"string"},
			ValidatorType:{type:"string"}
		}
	},
	constructor:function(){
		sap.ui.core.Control.apply(this,arguments);
		this.proxyHandleKeydown=$.proxy(this.handleKeydown,this);
		if(!this.getErrorState()) {
			this.setErrorState(sap.ui.core.ValueState.None);
		}
	}
});
//API
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.clearCell=function(){
	this.editing=false;
	delete this.cellInfo;
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.loadCell=function(cellInfo){
	this.editing=false;
	this.cellInfo=cellInfo;
	if(!cellInfo.getSModel()){return;}
	this.viewControl=this.createViewControl();
	this.errorControl=this.createErrorControl();
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.focusCell=function(){
	this.originalValue=this.cellInfo.getValue();
	this.showEditPopup();
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.blurCell=function(){
	if(this.editing||this.popup){//sometimes cells rerender, causing editing to be flipped back to false
		this.popup&&this.popup.close();
		this.storeNewValue();
		this.refreshViewControl();
		this.editing=false;
	}
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.createErrorControl=function(imgSrc){
	var statusIcon;
	var errorType = this.getErrorState() || "";
	errorType = errorType.toUpperCase();
	switch(errorType){
		case "INFO" || "INFORMATION":
			statusIcon =new sas.hc.m.Image({src: sas.icons.HC.ERRORSTATUS, inlineSVG: true});
			break;
		case "ERROR":
			statusIcon =new sas.hc.m.Image({src: sas.icons.HC.ERRORSTATUS, inlineSVG: true});
			break;
		case "SUCCESS":
			statusIcon =new sas.hc.m.Image({src: sas.icons.HC.ERRORSTATUS, inlineSVG: true});
			break;
		case "WARNING":
			statusIcon =new sas.hc.m.Image({src: sas.icons.HC.ERRORSTATUS, inlineSVG: true});
			break;
		default:
			statusIcon = null;
	}
	if(statusIcon) {
		var cellIconContainer = new sap.ui.layout.VerticalLayout({
			content:[
				statusIcon
			],
			width: "16px"
		});
		if(this.getTextAlign && this.getTextAlign() === sap.ui.core.TextAlign.End) {
			this.addStyleClass('endAligned');
		}
		return cellIconContainer;
	} else {
		return null;
	}
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.createValidator=function(){
	sas.risk.common.widgets.spreadsheet.control.validator.SpreadsheetValidatorFactory.getInstance().request(this.cellInfo.getSOptions().ValidatorType, this);
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.processKeyOther=function(keycode){};
//EVENT REGISTRATION
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.handleKeydown=function(e){
	switch(e.keyCode){
		case jQuery.sap.KeyCodes.ESCAPE:
			this.revertValue();
		break;
		case jQuery.sap.KeyCodes.F2:
			this.enterDeepEdit();
		break;
		case jQuery.sap.KeyCodes.SPACE:
		case jQuery.sap.KeyCodes.BACKSPACE:
		case jQuery.sap.KeyCodes.DELETE:
		case jQuery.sap.KeyCodes.ENTER:
			this.processKeyOther(e);
		default:
			if(e.originalEvent){
				var ee=$.Event(e.originalEvent.type);
				ee.keyCode=e.originalEvent.keyCode;
				ee.code=e.originalEvent.code;
				$(this.getDomRef().parentElement.parentElement.parentElement).trigger(ee);
			}
		break;
	}
};
//HELPERS
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.createViewControl=function(){};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.createEditControl=function(){};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.enterDeepEdit=function(){};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.refreshViewControl=function(){};

sas.risk.common.widgets.spreadsheet.control.ACell.prototype.isEditable=function(){
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.showEditPopup=function(){
	if(!this.editing && (!this.cellInfo.getSOptions().readonly || (this.cellInfo.getSOptions().readonly && this.cellInfo.getSOptions().validatorType === "error")) && this.cellInfo.getSState().spreadsheet.getEnabled()){
		this.editing=true;
		var $td=this.$().closest("td");
		this.editControl=this.createEditControl();
		this.popup=this.createPopup(this.editControl);
		this.popup.attachOpened($.proxy(this.onPopupOpened,this));
		this.popup.attachClosed($.proxy(this.onPopupClosed,this));
		this.popup.open(0,sap.ui.core.Popup.Dock.BeginTop,sap.ui.core.Popup.Dock.BeginTop,$td,"0 0","fit",true);
		if(this.editControl.getItems) {
			_.each(this.editControl.getItems(), function(ec, index) {
				if(typeof ec.performLiveValidation == 'function') { 
					ec.performLiveValidation(); 
				}
			});
		}
	}
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.onPopupClosed=function(event){
	this.storeNewValue();
	this.refreshViewControl();
	this.editing=false;
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.onPopupOpened=function(event){
	var $editControl=this.editControl.$();
	$editControl.off("keydown",this.proxyHandleKeydown);
	$editControl.on("keydown",this.proxyHandleKeydown);
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.createPopup=function(editControl){
	var td=this.$().closest("td");
	var popupLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		items:[
			new sas.risk.common.layout.flexlayout.FlexLayout({
				items:[editControl],
				alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
				width:td.width()+"px"
			}).addStyleClass("riskSpreadsheetCellPopup")
		]
	});
	var popup = new sap.ui.core.Popup(popupLayout,false,false,true);
	popup.editingCell=this.cellInfo;
	return popup;
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.invalidateParent = function() {
	var id = $("#" + this.sId).parent().attr('id');
	sap.ui.getCore().getElementById(id).invalidate();
};
//VALUE MANAGEMENT
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.getNewValue=function(){
	return this.editControl.getValue();
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.storeNewValue=function(){
	var value=this.getNewValue();
	if(value!==this.originalValue){
		this.originalValue=value;
		if(this.popup && this.popup.editingCell) {
			this.popup.editingCell.setValue(value);
		}
	}
};
sas.risk.common.widgets.spreadsheet.control.ACell.prototype.revertValue=function(){
	var currentValue=this.cellInfo.getValue();
	if(currentValue!==this.originalValue){
		this.cellInfo.setValue(this.originalValue);
	}
};
jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.ACellRenderer");
sas.risk.common.widgets.spreadsheet.control.ACellRenderer={
	render:function(rm,cell){
		rm.write("<DIV");
		rm.writeControlData(cell);
		rm.writeAttribute("tabindex", "-1");
		rm.addStyle("flex-grow","2");
		rm.addStyle("min-height","31px");
		rm.addStyle("width","100%");
		rm.addStyle("align-items","center");
		rm.addStyle("display","flex");
		rm.addClass("riskSpreadsheetCell");
		//CSS Identifiers
		rm.addClass("SHEET_" + cell.cellInfo.sheetId);
		rm.addClass("ROW_" + cell.cellInfo.cellId);
		rm.addClass("COLUMN_" + cell.cellInfo.columnId);
		//plug in for custom styling of cells
		var stylerKey=cell.cellInfo.getSOptions().styler;
		var styler=stylerKey&&sas.risk.common.widgets.spreadsheet.SpreadsheetCellStyleFactory.getInstance().request(stylerKey);
		styler&&styler.style(rm,cell);
		
		rm.writeClasses();
		rm.writeStyles();
		rm.write(">");
		if(!cell.editing && cell.viewControl && cell.cellInfo.getSModel()){
			cell.refreshViewControl();
			rm.renderControl(cell.viewControl);
			rm.write("<DIV");
			rm.addClass("cellIconContainer");
			rm.writeClasses();
			rm.write(">");
			rm.renderControl(cell.errorControl);
			rm.write("</DIV>");
		}
		rm.write("<div class='riskSpreadsheetRowNumber'>"+(cell.cellInfo.getSRowIndex()+1)+"</div>");
		rm.write("</DIV>");
	}	
};