jQuery.sap.require("sas.risk.common.layout.gridlayout.GridLayout");
jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.risk.common.widgets.expreditor.ExprEditor");
jQuery.sap.require("sap.ui.model.json.JSONModel");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetMeta");
jQuery.sap.require("sas.ui.cpb3.components.SpreadsheetComponent");

sap.ui.jsview("sas.risk.common.widgets.spreadsheet.control.ExpressionEditorDialog", {
    getControllerName : function() {
        return "sas.risk.common.widgets.spreadsheet.control.ExpressionEditorDialog";
    },
    /*
     * 
     */
    onAfterRendering: function() {
       // RGFRMWRK-402
       // the dialog
       var elem=this.$()[0].parentElement.parentElement.parentElement.parentElement; 
       // zIndex to be set
       var zIndex= 10; // document.getElementById($(elem)[0].id).style.zIndex;
       setTimeout(function() {
           zIndex=(document.getElementById($(elem)[0].id).style.zIndex) + 1;
           // the hints are on the document therefore, we need to use that to set the variable
           document.body.style.setProperty("--zIndex", zIndex);
       }, 0);
    },
    createContent : function(controller) {
       var that=this;
       this.createButtons(controller);
       this.query=new sas.risk.common.widgets.expreditor.ExprEditor({
           value:"{configure>/rawExpression}",
           meta:"{configure>/meta}",
           change:function(e){
                that.parsedExpression=e.getParameter("parsedExpression");
               //that.convertedExpression=e.getParameter("convertedExpression");
                that.validationErrors=e.getParameter("validationErrors");
           }
       }).setLayoutData(new sas.risk.common.layout.gridlayout.GridLayoutData({
           gridColumnStart:2,
           gridColumnEnd:4
       }));
       return new sas.risk.common.layout.gridlayout.GridLayout({
           gridTemplateColumns:"32px auto auto",
           gridRowGap:"10px",
           items:[this.query]
       });
    },
    createButtons : function(controller) {
        var rb=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
       this.okButton=new sas.risk.common.widgets.Button({
            text:rb.getText("spreadsheet.cell.dialog.button.ok.txt"),
            press:$.proxy(controller.actionOk,controller)
        });
       this.okButton.addStyleClass("sasPrimaryButton");
        this.cancelButton=new sas.risk.common.widgets.Button({
            text:rb.getText("spreadsheet.cell.dialog.button.cancel.txt"),
            press:$.proxy(controller.actionCancel,controller)
        });
    },
    getDialogSettings : function() {
        var viewData=this.getViewData();
        return {
         contentWidth:"90%",
         beginButton: this.okButton,
         endButton: this.cancelButton,
         initialFocus: this.query.editor,
         escapeHandler:viewData.callback,
         showClose:false
        };
    }
});
sap.ui.controller("sas.risk.common.widgets.spreadsheet.control.ExpressionEditorDialog", {
    onInit:function(){
        var view=this.getView();
        var that=this;
        this.loadMeta().done(function(meta){
            view.setModel(that.createModel(meta),"configure");
        });
    },
    loadMeta:function(){
       var deferred = $.Deferred();
       var view=this.getView();
       var viewData=view.getViewData();
       var value=viewData.meta;
       if(!value){
          var value=viewData.cpbForm.getCpbModelManager().getUIContextValue(viewData.variable);
          if(_.isString(value)){
              value=JSON.parse(value);
          }/*else {
              deferred.resolve(value); // resolve whatever we find... the value could be a JSON.. see if it matches ny criteria below
          }*/
       }
       // Check if the property cacheValues has been set: if not, assume values are not to be cached
       var forceUrlLoad = value.cacheValues? !value.cacheValues : true;
       // Check if the values have already been loaded
       var alreadyLoaded = value.values && value.values.length > 0;
       // Check if the property validValuesUrl has been set.
       var isDataUrlSet = value.validValuesUrl && value.validValuesUrl.length > 0;
       // Conditionally Load/Reload data from URL
       if(isDataUrlSet && (forceUrlLoad || !alreadyLoaded)) {
           $.getJSON(value.validValuesUrl, function( data ) {
               value.values=data;
               deferred.resolve(value);
             })
            .fail(function(status, xhr) {
               console.log("FAILED TO LOAD VALUES FROM URL: " + value.dataValuesUrl);
               // deferred.fail(xhr);
               deferred.resolve(value);
            });
       } else {
           deferred.resolve(value);
       }
       return deferred/*$.Deferred().resolve(value)*/;
    },
    actionOk:function(){
        var view=this.getView();
        this.resetValues(view);
        var configureModel=view.getModel("configure");
        var data=configureModel.getData();
        /*var value={
            rawExpression:data.rawExpression,
            parsedExpression:view.parsedExpression,
            convertedExpression:view.convertedExpression,
            validationErrors:view.validationErrors
        };
        view.fireClosing({
            isOk:true,
            value:value
        });
        view.getDialog().close();*/
        this._translateParsedExpression()
            .always(function(result){
                // update the items required.. so that wehn the dialog is reloaded the information is present
                var value={
                    rawExpression:data.rawExpression,
                    parsedExpression:view.parsedExpression,
                    convertedExpression:result.convertedExpression,
                    validationErrors:view.validationErrors
                };
                view.fireClosing({
                    isOk:true,
                    value:value
                });
                view.getDialog().close();
            });
            //.fail(function)
    },
    actionCancel:function(){
        var view=this.getView();
        this.resetValues(view);
        view.getDialog().close();
        view.fireClosing({
            isOk:false
        });
    },
    resetValues:function(view){
        var viewData=view.getViewData();
        var value=viewData.meta;
        if(!value){
           var value=viewData.cpbForm.getCpbModelManager().getUIContextValue(viewData.variable);
           if(_.isString(value)){
               value=JSON.parse(value);
           }
        } 
        // Check if the property cacheValues has been set: if not, assume values are not to be cached
        var forceUrlLoad = value.cacheValues? !value.cacheValues : true;
        if (forceUrlLoad) {
           value.values=[];
        }
    },
    createModel: function(meta){
        var view=this.getView();
        var viewData=view.getViewData();
        var data={meta:meta};
        if(viewData.value && viewData.value.rawExpression){
            data.rawExpression=viewData.value.rawExpression;
            data.parsedExpression=viewData.value.parsedExpression;
            data.convertedExpression=viewData.value.convertedExpression;
            data.validationErrors=viewData.value.validationErrors;
        }
        return new sap.ui.model.json.JSONModel(data);
    },
    _translateParsedExpression:function(){
        var deferred=$.Deferred();
        var view=this.getView();
        var modelData=view.getModel("configure").getData();
        var data={filterJson:view.parsedExpression||{}, metaJson:modelData.meta};

        var rb=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
        // call groovy to translate the given expression
        $.ajax({
            contentType: 'application/json',
            data: JSON.stringify(data),
            dataType: 'json',
            processData: false,
            type: 'POST',
            url: modelData.meta.conversionURL || "/SASRiskGovernanceFramework/groovy/rmc/Json2Sql.groovy?contentType=json" // default
        })
        .done (function(result){
            // update the items required.
            deferred.resolve({
                rawExpression:modelData.rawExpression,
                convertedExpression:result.convertedExpression||result.result
            });
        })
        .fail (function(xhr,status,error){
            console.error("Result: " + status + " " + error + " " + xhr.status + " " + xhr.statusText);
            var errorJSON={
                message:rb.getText("expreditor.conversionURL.fail.txt"),
                remediation: rb.getText("expreditor.conversionURL.fail.txt"),
                details:[
                    rb.getText("expreditor.conversionURL.fail.status.txt", xhr.status),
                    rb.getText("expreditor.conversionURL.fail.status.txt", xhr.statusText),
                ]
            };
            /* sas.risk.common.widgets.MessageDetailsBox.error(
                errorJSON, rb.getText("expreditor.title.txt"), rb
            ); */
            view.validationErrors.translationError=errorJSON;
            // failed to convert
            deferred.fail({
                rawExpression:modelData.rawExpression,
                convertedExpression:null
            });
        });
        return deferred;
    }
});