jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.FormulaCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.control.StringCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil");

sas.risk.common.widgets.spreadsheet.control.StringCell.extend("sas.risk.common.widgets.spreadsheet.control.FormulaCell",{
	renderer:"sas.risk.common.widgets.spreadsheet.control.ACellRenderer"
});
//this is called when about to show the edit popup
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.createShallowEditControl=function(){
	var value=this.calcValue();
	this.shallowEditControl=new sas.hc.m.Input({
       type: sap.m.InputType.Text,
       value: value,
       editable:false
   }).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
   this.shallowEditControl.validator=this;
	this.shallowEditControl.attachBrowserEvent("keypress.thiscell",this.proxyHandleShallowEdit);
	var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%"
	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
	layout.addItem(new sas.hc.ui.core.Icon({
      src: sas.icons.HC.FORMULA
   }));
	layout.addItem(this.shallowEditControl);
	layout.addStyleClass("riskSpreadsheetFormulaEdit");
   return layout;
};
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.createDeepEditControl=function(){
	var value=this.calcValue();
	this.deepEditControl=new sas.hc.m.Input({
       type: sap.m.InputType.Text,
       value: value
   }).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
   this.deepEditControl.validator=this;
	var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%"
	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
	layout.addItem(new sas.hc.ui.core.Icon({
      src: sas.icons.HC.FORMULA
   }));
	layout.addItem(this.deepEditControl);
	layout.addStyleClass("riskSpreadsheetFormulaEdit");
   return layout;
};
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.calcValue=function(){
	var value=this.cellInfo.getValue();
	value=_.isNumber(value)?value.toString():value;
	try{
		var expr=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.parse(value);
		this.cellInfo.resolveExpression(expr);
		value=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.print(expr);
	}catch(error){
		console.error(error);// eslint-disable-line no-console
	}
	return value;
};
//API
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.createViewControl=function(){
	return new sas.hc.m.Text({
		textAlign:sap.ui.core.TextAlign.End
	}).addStyleClass("riskSpreadsheetStringCellView");
};
//called by renderer whenever its about to render the view-only version of this cell
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.refreshViewControl=function(){
	this.syncViewControl(this.cellInfo.getValue());
};
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.syncViewControl=function(value){
	var cellValue=this.cellInfo.getValue();
	if(value!==cellValue){this.cellInfo.setValue(value);}
	value=_.isNumber(value)?value.toString():value;
	if(value){
		var that=this;
		this.cellInfo.state.getFormulaContext().clearExpectedFormulas();
		this.cellInfo.evaluateExpression().done(function(result){
			that.viewControl.setText(result);
		}).fail(function(error){
			console.error(error);// eslint-disable-line no-console
			that.viewControl.setText("#ERROR!"+error);//eslint-disable-line shikari/sas-i18n-ems
		});
	}else{
		this.viewControl.setText("");
	}
};
//VALIDATION
//this is the formula live validation. The function name must be textLiveValidation to play nicely with input widget
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.textLiveValidation=function(value){
	var result={valueText:"",valueType:sap.ui.core.ValueState.None};
	try{
		var expr=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.parse(value);
		this.cellInfo.analyzeExpression(expr);
	}catch(error){
		var bundle=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
		result.valueType=sap.ui.core.ValueState.Error;
		if(error.message){
			result.customErrorText=bundle.getText("spreadsheet.formula.circular.txt");
		}else{
			result.customErrorText=bundle.getText("spreadsheet.formula.invalid.txt");
		}
	}
	return result;
};
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.textValidation=sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.textLiveValidation;
//VALUE MANAGEMENT
//called by storeNewValue, which is called whenever the edit popup is closed
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.getNewValue=function(){
	var val=this.deepEditControl.getValue();
	try{
		var expr=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.parse(val);
		this.cellInfo.analyzeExpression(expr,true);
		return sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.print(expr);
	}catch(error){
		return val;
	}
};
sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.storeNewValue=function(){
	sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.storeNewValue.apply(this,arguments);
	this.cellInfo.getSState().setFormulaMode(false);
};
//CLICK EVENT HANDLERS
//sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.clickedBeyondFormulaScope=function(){
//	var e = jQuery.Event("keydown");
//	e.which = jQuery.sap.KeyCodes.ESCAPE;
//	e.keyCode = jQuery.sap.KeyCodes.ESCAPE;
//	this.editControl.$().trigger(e);
//};
//sas.risk.common.widgets.spreadsheet.control.FormulaCell.prototype.addCellToFormula=function(cell){
//	var prefix=this.cellInfo.getSSheetId()!==cell.cellInfo.getSSheetId()?cell.cellInfo.getSSheetId()+"!":"";
//	var cellRef=prefix+cell.cellInfo.getSAliasReference();
//	var elementId=this.editInput.getId()+"-inner";
//	var pos = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getCaretPosition(elementId);// Find the caret in the formula
//	var oldFormula = this.editInput.getValue();// The original formula
//	var newFormula = oldFormula.substr(0, pos) + cellRef + oldFormula.substr(pos);// Insert cell name into formula at caret position to create new formula
//	this.editInput.setValue(newFormula);// Set the formula field value with the new formula
//	this.editInput.fireLiveChange();//fire live validation
//	sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.setCaretPosition(elementId, pos + cellRef.length);// Set the caret to just after the inserted cell name in the formula
//};