jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.NumberCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.control.StringCell");
jQuery.sap.require("sap.ui.model.type.Float");
jQuery.sap.require("sas.ui.cpb3.model.format.CpbNumberFormat");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sap.ui.core.format.NumberFormat");

sas.risk.common.widgets.spreadsheet.control.StringCell.extend("sas.risk.common.widgets.spreadsheet.control.NumberCell",{
	metadata:{
		properties:{
			textAlign: {type: "string"}
		}
	},
	constructor: function() {
		sas.risk.common.widgets.spreadsheet.control.StringCell.apply(this, arguments);
		if(!this.getTextAlign()){ this.setTextAlign(sap.ui.core.TextAlign.End);}
	},
	renderer:"sas.risk.common.widgets.spreadsheet.control.ACellRenderer"
});
sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.createShallowEditControl=function(){
	var value=this.cellInfo.getValue();
	value=this.cleanValue(value);
	this.shallowEditControl = new sas.hc.m.Input({
       type: sap.m.InputType.Number,
       textAlign:sap.ui.core.TextAlign.End,
       value: value,
       editable:false
   	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2})).addStyleClass("riskSpreadsheetNumberCellEdit");
	this.shallowEditControl.attachBrowserEvent("keypress.thiscell",this.proxyHandleShallowEdit);
	var validator = this.createValidator();
	if(validator){
		this.shallowEditControl.setValidator(validator);
	}
   	return this.shallowEditControl;
};
sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.createDeepEditControl=function(){
	var value=this.cellInfo.getValue();
	value=this.cleanValue(value);
	this.deepEditControl=new sas.hc.m.Input({
       type: sap.m.InputType.Number,
       textAlign:sap.ui.core.TextAlign.End,
       value: value
  	 }).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2})).addStyleClass("riskSpreadsheetNumberCellEdit");
   	var validator = this.createValidator();
	if(validator){
		this.deepEditControl.setValidator(validator);
	}
   	return this.deepEditControl;
};
//API
sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.createViewControl=function(){
	return new sas.hc.m.Text({
		textAlign:sap.ui.core.TextAlign.End
	}).addStyleClass("riskSpreadsheetStringCellView");
};
sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.syncViewControl=function(val){
	var val=this.cleanValue(val);
	this.viewControl.setText(sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.formatNumber(val));
};
//VALUE MANAGEMENT
sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.getNewValue=function(){
	var val=this.deepEditControl.getRawNumber();
	val = this.parseValue(val);
	val=this.cleanValue(val);
	if($.isNumeric(val)){
		return Number(val);
	}
	return "";
};
sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.cleanValue=function(val){
	var cellTypeInfo=this.getCellType().split("_");
	var decimals=-1;
	if(cellTypeInfo.length===2){
		decimals=parseInt(cellTypeInfo[1]);
	}

	if(decimals > 0 && val !== undefined){
		val=this.round(val.toString(),decimals);
	}
	return val;
};
sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.round=function(num, maxFractionDigits){
	var fracDigits = num.substr(num.indexOf(".") + 1);
	
	var n = Math.pow(10, maxFractionDigits);
	if(fracDigits.length > maxFractionDigits){
		if(num.charAt(num.length - 1) === "5"){
			var prevStr = num.substr(num.length-2,1);
			var prev = parseInt(prevStr);
			return ((prev % 2) === 0) ? Math.floor(num * n)/n : Math.ceil(num * n)/n;
		}
	}
	return Math.round(num * n) / n;
};

sas.risk.common.widgets.spreadsheet.control.NumberCell.prototype.parseValue=function(value) {
	var cellTypeInfo=this.getCellType().split("_");
	var decimals=-1;
	if(cellTypeInfo.length===2){
		decimals=parseInt(cellTypeInfo[1]);
	}

	//Only tested for number and number_0
	//See also SpreadsheetUtil.js - needs parse, not just format
	//https://openui5.hana.ondemand.com/1.28.3/docs/guide/91f2f2866f4d1014b6dd926db0e91070.html
	try{
	   var options = decimals === -1 ? {minFractionDigits:null, maxFractionDigits:15} : {minFractionDigits:null, maxFractionDigits:decimals};
		var floatFormat = sap.ui.core.format.NumberFormat.getFloatInstance(options, sas.ui.cpb3.CpbUtil.getLocale()); 
		fValue = floatFormat.parse(value);
		return fValue;
	}catch(e){//last ditch attempt
		var num=Number(value);
		if(_.isNaN(num))throw e;
		return num;
	}
};