jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.PercentCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.control.StringCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");

sas.risk.common.widgets.spreadsheet.control.StringCell.extend("sas.risk.common.widgets.spreadsheet.control.PercentCell",{
	metadata:{
		properties:{
			textAlign: {type: "string"}
		}
	},
	constructor: function() {
		sas.risk.common.widgets.spreadsheet.control.StringCell.apply(this, arguments);
		if(!this.getTextAlign()){ this.setTextAlign(sap.ui.core.TextAlign.End);}
	},
	renderer:"sas.risk.common.widgets.spreadsheet.control.ACellRenderer"
});
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.createShallowEditControl=function(){
	var value=this.cellInfo.getValue();
	value=this.convertToPercent(value);
	this.shallowEditControl=new sas.hc.m.Input({
       type: sap.m.InputType.Number,
       textAlign:sap.ui.core.TextAlign.End,
       value: value,
       editable:false
   }).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2})).addStyleClass("riskSpreadsheetPercentCellEdit");
	this.shallowEditControl.attachBrowserEvent("keypress.thiscell",this.proxyHandleShallowEdit);
   return this.shallowEditControl;
};
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.createDeepEditControl=function(){
	var value=this.cellInfo.getValue();
	value=this.convertToPercent(value);
	this.deepEditControl=new sas.hc.m.Input({
       type: sap.m.InputType.Number,
       textAlign:sap.ui.core.TextAlign.End,
       value: value
   }).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2})).addStyleClass("riskSpreadsheetPercentCellEdit");
//   var rangeValidator = new sas.hc.m.RangeValidationAdapter({
//       minValue: 0,
//       maxValue: 100,
//       inclusive: true
//   });
//   input.setValidator(rangeValidator);
   return this.deepEditControl;
};
//API
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.createViewControl=function(){
	return new sas.hc.m.Text({
		textAlign:sap.ui.core.TextAlign.End
	}).addStyleClass("riskSpreadsheetStringCellView");
};
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.syncViewControl=function(value,isPercent){
	if(!isPercent){
		value=this.convertToPercent(value);
	}
	if(value||value===0){
		value=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.formatNumber(value)+"%";
	}
	this.viewControl.setText(value);
};
//VALUE MANAGEMENT
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.getNewValue=function(){
	var value=this.deepEditControl.getRawNumber();
	return this.convertFromPercent(value);
};
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.syncShallowEditor=function(){
	//really strange but getInputValue and getRawNumber are both out of date if you hit enter off of shallow
	//luckily _getInputValue has the right answer
	var deepValue=this.deepEditControl._getInputValue();
	if(this.popup && this.popup.editingCell) {
		this.popup.editingCell.setValue(sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.formatNumber(this.convertFromPercent(deepValue)));
	}
	this.shallowEditControl.setValue(deepValue);
	this.syncViewControl(deepValue,true);
};
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.syncDeepEditor=function(){
	var shallowValue=this.shallowEditControl.getRawNumber();
	this.deepEditControl.setValue(shallowValue);
	this.syncViewControl(shallowValue,true);
};
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.convertToPercent=function(value){
	if($.isNumeric(value)){
		value=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.round(value*100,2);
	}
	return value;
};
sas.risk.common.widgets.spreadsheet.control.PercentCell.prototype.convertFromPercent=function(value){
	if($.isNumeric(value)){
		var number=Number(value);
		return sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.round(number/100,4);
	}
	return null;
};
