jQuery.sap.require("sas.risk.common.layout.gridlayout.GridLayout");
jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.risk.common.widgets.querybldr.QueryBuilder");
jQuery.sap.require("sap.ui.model.json.JSONModel");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetMeta");
jQuery.sap.require("sas.ui.cpb3.components.SpreadsheetComponent");

sap.ui.jsview("sas.risk.common.widgets.spreadsheet.control.QueryBuilderDialog", {
	getControllerName : function() {
		return "sas.risk.common.widgets.spreadsheet.control.QueryBuilderDialog";
	},
	createContent : function(controller) {
		var rb=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	   var that=this;
	   var viewData=this.getViewData();
	   this.createButtons(controller);
	   this.query=new sas.risk.common.widgets.querybldr.QueryBuilder({
	   	value:"{configure>/filter}",
	   	meta:"{configure>/meta}"
	   }).setLayoutData(new sas.risk.common.layout.gridlayout.GridLayoutData({
	   	gridColumnStart:2,
	   	gridColumnEnd:4
	   }));
	   return new sas.risk.common.layout.gridlayout.GridLayout({
	   	gridTemplateColumns:"32px auto auto",
	   	gridRowGap:"10px",
	   	items:[this.query]
	   });
	},
	createButtons : function(controller) {
		var rb=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	   this.okButton=new sas.risk.common.widgets.Button({
			text:rb.getText("spreadsheet.cell.dialog.button.ok.txt"),
			press:$.proxy(controller.actionOk,controller)
		});
	   this.okButton.addStyleClass("sasPrimaryButton");
		this.cancelButton=new sas.risk.common.widgets.Button({
			text:rb.getText("spreadsheet.cell.dialog.button.cancel.txt"),
			press:$.proxy(controller.actionCancel,controller)
		});
	},
	getDialogSettings : function() {
		var viewData=this.getViewData();
		return {
         contentWidth:"70%",
         beginButton: this.okButton,
         endButton: this.cancelButton,
         initialFocus: this.query,
         escapeHandler:viewData.callback,
         showClose:false
		};
	}
});
sap.ui.controller("sas.risk.common.widgets.spreadsheet.control.QueryBuilderDialog", {
	onInit:function(){
		var view=this.getView();
		var viewData=view.getViewData();
		var that=this;
		this.loadMeta().done(function(result){
			var meta=sas.ui.cpb3.components.SpreadsheetComponent.convertMeta(result);
			var defaultCellType=meta.table.cellOptions&&meta.table.cellOptions.cellType;
			meta.columns=_.filter(meta.columns,function(col){
				if((col.cellOptions&&col.cellOptions.cellType)||defaultCellType){
					switch((col.cellOptions&&col.cellOptions.cellType)||defaultCellType)
					{
						case "java.lang.String":
							col.cellOptions&&(col.cellOptions.cellType="string");
							return true;
						case "java.util.Date":
						case "java.sql.Timestamp":
							col.cellOptions&&(col.cellOptions.cellType="timestamp");
							return true;
						case "java.lang.Double":
							col.cellOptions&&(col.cellOptions.cellType="number");
							return true;
						case "java.lang.Boolean":
							col.cellOptions&&(col.cellOptions.cellType="boolean");
							return true;
						case "dropdown":
						case "string":
						case "boolean":
						case "number":
						case "timestamp":
						case "percent":
						case "date":
							return true;
					}
				}
				return false;
			});
			meta=new sas.risk.common.widgets.spreadsheet.SpreadsheetMeta(meta);
			view.setModel(that.createModel(meta),"configure");
		});
	},
	loadMeta:function(){
		var view=this.getView();
		var viewData=view.getViewData();
		var value=viewData.cpbForm.getCpbModelManager().getUIContextValue(viewData.variable);
		if(_.isString(value)){
			value=JSON.parse(value);
		}
		return $.Deferred().resolve(value);
	},
	actionOk:function(){
		var view=this.getView();
		view.getDialog().close();
		var value={
			filter:view.query.getValue()
		};
		if(!(value.filter)||!(value.filter.children) || !(value.filter.children.length)){
			value.filter=null;
		}
		value.restFilter=value.filter?sas.risk.common.widgets.spreadsheet.filter.AFilter.hydrate(value.filter).toString():null;
		view.fireClosing({
			isOk:true,
			value:value
		});
	},
	actionCancel:function(){
		var view=this.getView();
		view.getDialog().close();
		view.fireClosing({
			isOk:false
		});
	},
	createModel: function(meta){
		var view=this.getView();
		var viewData=view.getViewData();
		var data={meta:meta};
		if(viewData.value && viewData.value.filter){
			data.filter=$.extend(true,{},viewData.value.filter);
		}
		return new sap.ui.model.json.JSONModel(data);
	}
});