jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.StringCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.control.ACell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");
jQuery.sap.require("sas.risk.common.widgets.RegexValidationAdapter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.control.validator.SpreadsheetValidatorFactory");

sas.risk.common.widgets.spreadsheet.control.ACell.extend("sas.risk.common.widgets.spreadsheet.control.StringCell",{
	constructor:function(){
		sas.risk.common.widgets.spreadsheet.control.ACell.apply(this,arguments);
		this.proxyHandleClose=$.proxy(this.handleClose,this);
		this.proxyHandleShallowEdit=$.proxy(this.handleShallowEdit,this);
		this.proxyHandleDoubleClick=$.proxy(this.handleDoubleClick,this);
	},
	renderer:"sas.risk.common.widgets.spreadsheet.control.ACellRenderer"
});
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.createEditControl=function(){
	return new sas.risk.common.layout.flexlayout.FlexLayout({
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		items:[this.createShallowEditControl(),this.createDeepEditControl()]
	}).addStyleClass("riskEditInputCell riskHideDeepEditInputCell").setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.createShallowEditControl=function(){
	var value=this.cellInfo.getValue();
	this.shallowEditControl=new sas.hc.m.Input({
		type:sap.m.InputType.Text,
		value:value,
		editable:false,
	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
	this.shallowEditControl.attachBrowserEvent("keypress.thiscell",this.proxyHandleShallowEdit);
	var validator = this.createValidator();
	if(validator){
		this.shallowEditControl.setValidator(validator);
	}
	return this.shallowEditControl;
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.createDeepEditControl=function(){
	var value=this.cellInfo.getValue();
	this.deepEditControl=new sas.hc.m.Input({
		type:sap.m.InputType.Text,
		value:value,
		editable:this.cellInfo.getSOptions().readonly ? false : true
	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
	var validator = this.createValidator();
	if(validator){
		this.deepEditControl.setValidator(validator);
	}
	return this.deepEditControl;
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.createValidator=function(){
	var cellTypeInfo=this.getCellType().split("_");
	if(cellTypeInfo.length >= 2) {
		var regex=null;
		var msg=null;
		if(cellTypeInfo.length>2){
			cellTypeInfo.shift();
			var msgKey="uiextensions.validation."+cellTypeInfo.shift()+".txt";
			var rb=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
			msg=rb.getText(msgKey);
			regex=new RegExp(cellTypeInfo.join("_"));
		}
		if(regex){
			return new sas.risk.common.widgets.RegexValidationAdapter({regex:regex,errorText:msg});
		}
		return null;
	}
	return sas.risk.common.widgets.spreadsheet.control.validator.SpreadsheetValidatorFactory.getInstance().request(this.cellInfo.getSOptions().validatorType, this.cellInfo.getSOptions().validatorOptions, this);
};
//API
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.createViewControl=function(){
	return new sas.hc.m.Text({}).addStyleClass("riskSpreadsheetStringCellView");
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.refreshViewControl=function(){
	this.syncViewControl(this.cellInfo.getValue());
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.blurCell=function(){
	sas.risk.common.widgets.spreadsheet.control.ACell.prototype.blurCell.apply(this,arguments);
	delete this.shallowEditMode;
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.processKeyOther=function(e){
	switch(e.keyCode){
		case jQuery.sap.KeyCodes.SPACE:
		case jQuery.sap.KeyCodes.BACKSPACE:
			this.deepEditControl.setValue("");
			this.shallowEditMode=true;
			this.enterDeepEdit();
			e.preventDefault();
			e.stopImmediatePropagation();
		break;
		case jQuery.sap.KeyCodes.DELETE:
			this.deepEditControl.setValue("");
			this.shallowEditControl.setValue("");
			this.storeNewValue();
			this.refreshViewControl();
			e.preventDefault();
			e.stopImmediatePropagation();
		break;
		case jQuery.sap.KeyCodes.ENTER:
			this.syncShallowEditor();
			this.exitDeepEdit();
			this.shallowEditMode=true;
			event.preventDefault();
			var ee=$.Event("keydown");
			ee.keyCode=jQuery.sap.KeyCodes.ARROW_DOWN;
			ee.code="ArrowDown";
			$(this.getDomRef().parentElement.parentElement.parentElement).trigger(ee);
		break;
	}
};
//VALUE MANAGEMENT
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.getNewValue=function(){
	return this.deepEditControl.getValue();
};
//QUICK EDIT
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.handleShallowEdit=function(event){
	event.stopImmediatePropagation();
	event.preventDefault();
	var charcode = event.charCode;
	if(charcode) {
		var char = String.fromCharCode(charcode);
		if(!this.cellInfo.getSOptions().readonly) {
			this.deepEditControl.setValue(char);
		}
		this.shallowEditMode=true;
	   	this.enterDeepEdit();
	}
};
//DEEP EDIT
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.exitDeepEdit=function(){
	this.editControl.$().addClass("riskHideDeepEditInputCell");
	var focusElem=this.shallowEditControl.$().firstFocusableDomRef();
	jQuery.sap.focus(focusElem);
	this.detachDeepEvents();
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.enterDeepEdit=function(){
	this.editControl.$().removeClass("riskHideDeepEditInputCell");
	var focusElem=this.deepEditControl.$().firstFocusableDomRef();
	jQuery.sap.focus(focusElem);
	sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.setCaretPosition(focusElem);
	this.attachDeepEvents();
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.attachDeepEvents=function(){
	this.deepEditControl.attachBrowserEvent("keydown.thiscell",this.proxyHandleClose);
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.detachDeepEvents=function(){
	this.deepEditControl.detachBrowserEvent("keydown.thiscell",this.proxyHandleClose);
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.syncShallowEditor=function(){
	var deepValue=this.deepEditControl.getValue();
	this.shallowEditControl.setValue(deepValue);
	this.syncViewControl(deepValue);
	if(this.popup && this.popup.editingCell) {
		this.popup.editingCell.setValue(deepValue);
	}
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.syncDeepEditor=function(){
	var shallowValue=this.shallowEditControl.getValue();
	this.deepEditControl.setValue(shallowValue);
	this.syncViewControl(shallowValue);
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.syncViewControl=function(value){
	this.viewControl.setText(value);
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.handleClose=function(event){
	switch(event.keyCode){
		case jQuery.sap.KeyCodes.ESCAPE:
			this.syncDeepEditor();
			this.exitDeepEdit();
			this.shallowEditMode=true;
			event.preventDefault();
		break;
		case jQuery.sap.KeyCodes.ENTER:
			this.syncShallowEditor();
			this.exitDeepEdit();
			this.shallowEditMode=true;
			event.preventDefault();
			var domRef=this.getDomRef();
			var table=this.cellInfo.spreadsheet.getSelectedSheetTable();
			var focusedRow=table._rowFromRef(domRef);
			if(focusedRow.getIndex()>=table._getRowCount()-1){//is last row
				var cell=table._getSelectedCell();
				cell.blurCell();
				table.onFocusAgain();
			}else{
				var ee=$.Event("keydown");
				ee.keyCode=jQuery.sap.KeyCodes.ARROW_DOWN;
				ee.code="ArrowDown";
				$(this.getDomRef().parentElement.parentElement.parentElement).trigger(ee);
			}
		break;
		case jQuery.sap.KeyCodes.F2:
		   this.shallowEditMode=false;
			this.enterDeepEdit();
			event.preventDefault();
		break;
		case jQuery.sap.KeyCodes.ARROW_LEFT:
		case jQuery.sap.KeyCodes.ARROW_RIGHT:
		case jQuery.sap.KeyCodes.ARROW_UP:
		case jQuery.sap.KeyCodes.ARROW_DOWN:
			if(this.shallowEditMode){
				this.syncShallowEditor();
				this.exitDeepEdit();
				return;//let this one filter through
			}
		break;
	}
	event.stopImmediatePropagation();
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.handleDoubleClick=function(e){
	if(this.shallowEditMode || this.shallowEditMode===undefined){
		this.enterDeepEdit();
	   this.shallowEditMode=false;
		this.enterDeepEdit();
		e.preventDefault();
		e.stopImmediatePropagation();
	}
};
sas.risk.common.widgets.spreadsheet.control.StringCell.prototype.onPopupOpened=function(event){
	sas.risk.common.widgets.spreadsheet.control.ACell.prototype.onPopupOpened.apply(this,arguments);
	$shallowEditControl=this.shallowEditControl.$();
	$shallowEditControl.off("dblclick",this.proxyHandleDoubleClick);
	$shallowEditControl.on("dblclick",this.proxyHandleDoubleClick);
};
