jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator");
jQuery.sap.require("sas.hc.m.BaseValidationAdapter");

//This validator doesn't perform any validatation and always shows an error when one is shown.

sas.hc.m.BaseValidationAdapter.extend("sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator",{
	metadata:{
		properties:{
			validatorOptions:{type: "any"},
			errorState: {type: "any"},
			errorText:{type:"string"},//localized error message
			cell: {type: "any"}
		}
	},
	constructor: function() {
		sas.hc.m.BaseValidationAdapter.apply(this, arguments);

		if(this.getValidatorOptions()) {
			//if no errorText is passed in the constructor check for it in validatorOptions and set
			if(!this.getErrorText() && this.getValidatorOptions().errorText){
				this.setErrorText(this.getValidatorOptions().errorText);
			}

			//takes errorType string and converts to sap valuestate object
			var errorType = this.getErrorState() || (this.getValidatorOptions().forceState && this.getValidatorOptions().errorState) || "";
			errorType = errorType.toUpperCase();
			switch(errorType){
				case "INFO" || "INFORMATION":
					this.setErrorState(sap.ui.core.ValueState.Information);
					break;
				case "ERROR":
					this.setErrorState(sap.ui.core.ValueState.Error);
					break;
				case "SUCCESS":
					this.setErrorState(sap.ui.core.ValueState.Success);
					break;
				case "WARNING":
					this.setErrorState(sap.ui.core.ValueState.Warning);
					break;
				default:
					this.setErrorState(sap.ui.core.ValueState.None);
			}
			
		}

	}
});


sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.textLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.textValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.numericLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.numericValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.telephoneLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.telephoneValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.urlLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.urlValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.emailLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype.emailValidation = function(value){
	return this._validation(value);
};

sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype._validation = function(value) {
	var validation = this._validate(value);
	if(this.getCell() && validation){
		if(this.getErrorText()) {
			this.getCell().setErrorText(this.getErrorText());
			this.getCell().deepEditControl.setValueStateText(this.getErrorText());
			this.getCell().shallowEditControl.setValueStateText(this.getErrorText());
		}
		this.getCell().setErrorState(validation.valueType);
		this.getCell().invalidateParent();
		var that = this;
		setTimeout(function(){
			that.getCell().deepEditControl.setValueState(validation.valueType);
			that.getCell().deepEditControl.setShowValueHelp(true);
			that.getCell().deepEditControl.openValueStateMessage();
		}, 0);
			
	}
	return validation;
};
sas.risk.common.widgets.spreadsheet.control.validator.ErrorMessageValidator.prototype._validate = function(value) {
	var result = {
		valueText: "",
		valueType: this.getErrorState() ||sap.ui.core.ValueState.None
	};
	if(this.getErrorText()) {
		result.customErrorText = this.getErrorText();
	}
	return result;
};
