jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator");
jQuery.sap.require("sas.hc.m.BaseValidationAdapter");

sas.hc.m.BaseValidationAdapter.extend("sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator",{
	metadata:{
		properties:{
			validatorOptions:{type: "any"},
			regex: {type: "any"},
			errorText: {type: "string"}, 
			errorState: {type: "any"},
			cell: {type: "any"}
		}
	},
	constructor: function() {
		sas.hc.m.BaseValidationAdapter.apply(this, arguments);

		if(this.getValidatorOptions()) {
			//if no regex is passed in the constructor check for it in validatorOptions and set
			if(!this.getRegex() && this.getValidatorOptions().expression) {
				this.setRegex(new RegExp(this.getValidatorOptions().expression));
			}
			
			//if no errorText is passed in the constructor check for it in validatorOptions and set
			if(!this.getErrorText() && this.getValidatorOptions().errorText){
				this.setErrorText(this.getValidatorOptions().errorText);
			}

			//takes errorType string and converts to sap valuestate object
			var errorType = this.getErrorState() || (this.getValidatorOptions().forceState && this.getValidatorOptions().errorState) || "";
			errorType = errorType.toUpperCase();
			switch(errorType){
				case "INFO" || "INFORMATION":
					this.setErrorState(sap.ui.core.ValueState.Information);
					break;
				case "NONE":
					this.setErrorState(sap.ui.core.ValueState.None);
					break;
				case "SUCCESS":
					this.setErrorState(sap.ui.core.ValueState.Success);
					break;
				case "WARNING":
					this.setErrorState(sap.ui.core.ValueState.Warning);
					break;
				default:
					this.setErrorState(sap.ui.core.ValueState.Error);
			}	
		}
	}
});

sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.textLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.textValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.numericLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.numericValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.telephoneLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.telephoneValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.urlLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.urlValidation = function(value){
	return this._validation(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.emailLiveValidation = function(value){
	return this._validate(value);
};
sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype.emailValidation = function(value){
	return this._validation(value);
};

sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype._validation = function(value) {
	var validation = this._validate(value);
	if(this.getCell() && validation){
		if(this.getErrorText()) {
			this.getCell().setErrorText(this.getErrorText());
		}
		this.getCell().setErrorState(validation.valueType);
		this.getCell().invalidateParent();
	}
	return validation;
};

sas.risk.common.widgets.spreadsheet.control.validator.RegexCellValidator.prototype._validate = function(value){
	var result = {
		valueText: "",
		valueType: sap.ui.core.ValueState.None
	};
	if(this.getRegex() && !this.getRegex().test(value)){	
		if(this.getErrorText()) {
			result.customErrorText = this.getErrorText();
		}
		result.valueType = this.getErrorState();
	}
	return result;
};
