jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.TokenType");
jQuery.sap.require("sap.ui.core.format.NumberFormat");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.Math");

jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.Parser");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.Lexer");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.eval.NameEval");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.eval.OperatorEval");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.eval.ComparisonEval");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.eval.ConditionalEval");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.eval.PostfixEval");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.eval.CallEval");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.eval.PrefixEval");

sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil={
	print:function(expr,fnContext,builder){
		var ExpressionUtil=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil;
		var TokenType=sas.risk.common.widgets.spreadsheet.formula.TokenType;
		var isRoot=(builder===undefined);
		if(!expr){return "";}
		switch(expr.type){
			case "CALL":
				if(!builder){
					builder='';
				}
				builder=ExpressionUtil.print(expr.func,fnContext,builder);
				builder+='(';
				for(var i=0;i<expr.args.length;i++){
					builder=ExpressionUtil.print(expr.args[i],fnContext,builder);
					if(i<expr.args.length-1){
						builder+=/*sas.stress.wkbench.StressUtil.whatListSeparator()*/', ';
					}
				}
				return builder+=')';
			case "COMPARISON":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.left,fnContext,builder);
				builder+=' '+TokenType[expr.comparator].punctuator+' ';
				builder=ExpressionUtil.print(expr.right,fnContext,builder);
				return builder+((isRoot)?'':')');
			case "CONDITIONAL":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.condition,fnContext,builder);
				builder+=' ? ';
				builder=ExpressionUtil.print(expr.thenArm,fnContext,builder);
				builder+=' : ';
				builder=ExpressionUtil.print(expr.elseArm,fnContext,builder);
				return builder+((isRoot)?'':')');
			case "NAME":
				if(!builder){
					builder='';
				}
				if(fnContext){
					switch(ExpressionUtil._getType(expr.name)){
						case "STRING":
							return builder+expr.name;
						case "NUMBER":
							var num=expr.name;
							if (num.indexOf(".")>=0) {
								//num = sas.stress.wkbench.StressUtil.replaceAll(num,".", sas.stress.wkbench.StressUtil.whatDecimalSeparator());
							}
							return builder+num;
						//break;
						case "BOOLEAN":
							return builder+expr.name.toUpperCase();
						//break;
						default:
							return builder+fnContext(expr.name);
					}
				}
				return builder+expr.name;
			case "OPERATOR":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.left,fnContext,builder);
				builder+=' '+TokenType[expr.operator].punctuator+' ';
				builder=ExpressionUtil.print(expr.right,fnContext,builder);
				return builder+((isRoot)?'':')');
			case "POSTFIX":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.left,fnContext,builder);
				return builder+TokenType[expr.operator].punctuator+((isRoot)?'':')');
			case "PREFIX":
				var skipParens=(builder && builder.indexOf(", ",builder.length-2)!==-1);//skip parenthesis if it is a function call arg
				builder=isRoot?'':builder+(skipParens?'':'(');
				builder+=TokenType[expr.operator].punctuator;
				builder=ExpressionUtil.print(expr.right,fnContext,builder);
				return builder+((isRoot||skipParens)?'':')');
		}
	},
	toString:function(expr,tab){
		var ExpressionUtil=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil;
		var TokenType=sas.risk.common.widgets.spreadsheet.formula.TokenType;
		var tab=tab?tab:0;
		switch(expr.type){
			case "CALL":
				var buff='CALL:';
				buff+=ExpressionUtil._tab(tab)+'|--(FUNCTION)'+ExpressionUtil.toString(expr.func,tab+1);
				for(var i=0;i<expr.args.length;i++){
					buff+=ExpressionUtil._tab(tab)+'|--(ARG['+i+'])'+ExpressionUtil.toString(expr.args[i],tab+1);
				}
				return buff;
			case "COMPARISON":
				var buff='COMPARISON:'+TokenType[expr.comparator].punctuator;
				buff+=ExpressionUtil._tab(tab)+'|--(LEFT)'+ExpressionUtil.toString(expr.left,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(RIGHT)'+ExpressionUtil.toString(expr.right,tab+1);
				return buff;
			case "CONDITIONAL":
				var buff='CONDITIONAL:';
				buff+=ExpressionUtil._tab(tab)+'|--(CONDITION)'+ExpressionUtil.toString(expr.condition,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(THEN)'+ExpressionUtil.toString(expr.thenArm,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(ELSE)'+ExpressionUtil.toString(expr.elseArm,tab+1);
				return buff;
			case "NAME":
				var buff='NAME:'+expr.name;
				return buff;
			case "OPERATOR":
				var buff='OPERATOR:'+TokenType[expr.operator].punctuator;
				buff+=ExpressionUtil._tab(tab)+'|--'+ExpressionUtil.toString(expr.left,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--'+ExpressionUtil.toString(expr.right,tab+1);
				return buff;
			case "POSTFIX":
				var buff='POST-OPERATOR:';
				buff+=ExpressionUtil._tab(tab)+'|--(VALUE)'+ExpressionUtil.toString(expr.left,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(OPERATOR)'+TokenType[expr.operator].punctuator;
				return buff;
			case "PREFIX":
				var buff='PRE-OPERATOR:'+TokenType[expr.operator].punctuator;
				buff+=ExpressionUtil._tab(tab)+'|--'+ExpressionUtil.toString(expr.right,tab+1);
				return buff;
		}
	},
	eval:function(expr,fnContext){
		if(expr){
			switch(expr.type){
				case "NAME":
					return sas.risk.common.widgets.spreadsheet.formula.eval.NameEval.eval(expr,fnContext);
				case "OPERATOR":
					return sas.risk.common.widgets.spreadsheet.formula.eval.OperatorEval.eval(expr,fnContext);
				case "COMPARISON":
					return sas.risk.common.widgets.spreadsheet.formula.eval.ComparisonEval.eval(expr,fnContext);
				case "CONDITIONAL":
					return sas.risk.common.widgets.spreadsheet.formula.eval.ConditionalEval.eval(expr,fnContext);
				case "POSTFIX":
					return sas.risk.common.widgets.spreadsheet.formula.eval.PostfixEval.eval(expr,fnContext);
				case "CALL":
					return sas.risk.common.widgets.spreadsheet.formula.eval.CallEval.eval(expr,fnContext);
				case "PREFIX":
					return sas.risk.common.widgets.spreadsheet.formula.eval.PrefixEval.eval(expr,fnContext);
			}
		}
		return $.Deferred().resolve(null);
	},
	collect:function(expr,type){
		var hits=[];
		if(expr){
			if(expr.type===type){
				hits.push(expr);
			}
			switch(expr.type){
				case "CALL":
					for(var i=0;i<expr.args.length;i++){
						hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.args[i],type));
					}
					return hits;
				case "COMPARISON":
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.left,type));
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.right,type));
					return hits;
				case "CONDITIONAL":
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.condition,type));
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.thenArm,type));
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.elseArm,type));
					return hits;
				case "NAME":
					return hits;
				case "OPERATOR":
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.left,type));
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.right,type));
					return hits;
				case "POSTFIX":
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.left,type));
					return hits;
				case "PREFIX":
					hits=hits.concat(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr.right,type));
					return hits;
			}
		}
		return hits;
	},
	evaluate:function(formulaStr,fnContext){//pass in formula string or already parsed expression
		var expr=_.isString(formulaStr)?sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.parse(formulaStr):formulaStr;
		var callExprs=sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.collect(expr,"CALL");
		var expectedFormulas=_.map(callExprs,function(val){return val.func.name;});
		fnContext.addExpectedFormulas(expectedFormulas);
		return sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil.eval(expr,fnContext);
	},
	parse:function(formulaStr){ 
		var lexer=new sas.risk.common.widgets.spreadsheet.formula.Lexer(formulaStr);
		var parser=new sas.risk.common.widgets.spreadsheet.formula.Parser(lexer);
		return parser.parseExpression();
	},
	create:function(type,arg1,arg2,arg3){
		var expr={type:type};
		switch(type){
			case "CALL":
				expr.func=arg1;
				expr.args=arg2;
				return expr;
			case "COMPARISON":
				expr.left=arg1;
				expr.comparator=arg2.name;
				expr.right=arg3;
				return expr;
			case "CONDITIONAL":
				expr.condition=arg1;
				expr.thenArm=arg2;
				expr.elseArm=arg3;
				return expr;
			case "NAME":
				expr.name=arg1;
				return expr;
			case "OPERATOR":
				expr.left=arg1;
				expr.operator=arg2.name;
				expr.right=arg3;
				return expr;
			case "POSTFIX":
				expr.left=arg1;
				expr.operator=arg2.name;
				return expr;
			case "PREFIX":
				expr.operator=arg1.name;
				expr.right=arg2;
				return expr;
		}
	},
	_getType:function(name){
		if(isNaN(name)){
			if(sas.risk.common.widgets.spreadsheet.formula.ExpressionUtil._isQuoted(name)){
				return "STRING";
			}
			var uppCase=name.toUpperCase();
			if(uppCase==="TRUE"||uppCase==="FALSE"){
				return "BOOLEAN";
			}
			return "REFERENCE";
		} else {
			return "NUMBER";
		}
	},
	_isQuoted:function(str){
		return str.length>=2 &&
			(str[0]==='"' || str[0]==="'") &&
			(str[str.length-1]==="'" || str[str.length-1]==='"');
	},
	_tab:function(size){
		if(!size){
			size=0;
		}
		var tab='\t';
		for(var i=0;i<size;i++){
			tab+='\t';
		}
		return '\n'+tab;
	}
};