/**
 * Created by sinsrj on 1/8/2018.
 *
 * converts the floating point numbers to integers, does the arithmetic operation and then divides by the number of decimals.
 *
 * https://stackoverflow.com/questions/1458633/how-to-deal-with-floating-point-number-precision-in-javascript
 */

sas.risk.common.widgets.spreadsheet.formula.Math ={
    a: function () {
        var f = _cf.apply(null, arguments);
        if(f === undefined) return undefined;
        function cb(x, y) {
            return x + ~~(f*y);// eslint-disable-line no-bitwise
        }
        var value=Array.prototype.reduce.call(arguments, cb, 0) / f;
        return sas.risk.common.widgets.spreadsheet.formula.Math.precisionRound(value, 10);
    },

    s: function (l,r) {
        var f = _cf(l,r);
        return (~~(l * f) - ~~(r * f)) / f;// eslint-disable-line no-bitwise
    },

    m: function () {
        var f = _cf.apply(null, arguments);
        function cb(x, y) {
            return (x*f) * (y*f) / (f * f);
        }
        var value=Array.prototype.reduce.call(arguments, cb, 1);
        return sas.risk.common.widgets.spreadsheet.formula.Math.precisionRound(value, 10);
    },

    d: function (l,r) {
        var f = _cf(l,r);
        return ~~(l * f) / ~~(r * f);// eslint-disable-line no-bitwise
    },
    /**
     * Excel like rounding
     *
     * @param number        The number to round
     * @param precision     The number of decimal places to preserve
     * @returns {number}
     */
    round: function(number, precision) {
        var factor = Math.pow(10, precision);
        var tempNumber = number * factor;
        var roundedTempNumber = Math.round(tempNumber);
        return roundedTempNumber / factor;
    },
    /**
     * Excel like rounding Up
     *
     * @param number        The number to round
     * @param precision     The number of decimal places to preserve
     * @returns {number}
     */
    roundUp: function(number, precision) {
        precision = Math.pow(10, precision);
        return Math.ceil(number * precision) / precision;
    },
    /**
     * Excel like rounding Down
     *
     * @param number        The number to round
     * @param precision     The number of decimal places to preserve
     * @returns {number}
     */
    roundDown: function(number, precision) {
        precision = Math.pow(10, precision);
        return Math.floor(number * precision) / precision;
    },
    /**
     * Excel MROUND function
     *
     * @param number
     * @param roundto
     * @returns {number}
     */
    roundTo: function(number, roundto){
        var value= roundto * Math.round(number/roundto);
        return sas.risk.common.widgets.spreadsheet.formula.Math.precisionRound(value, 10);
    },
    precisionRound: function (number, precision) {
        precision=precision===undefined?10:precision;
        var factor = Math.pow(10, precision);
        return Math.round(number * factor) / factor;
    }
};
var _cf = (function() {
    function _shift(x) {
        var parts = x.toString().split('.');
        return (parts.length < 2) ? 1 : Math.pow(10, parts[1].length);
    }
    return function() {
        return Array.prototype.reduce.call(arguments, function (prev, next) {
            return prev === undefined || next === undefined ? undefined : Math.max(prev, _shift (next));
        }, -Infinity);
    };
})();

