jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.formula.Parser");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.TokenType");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.Precedence");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.NameParselet");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.ComparisonParselet");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.GroupParselet");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.CallParselet");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.ConditionalParselet");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.PostfixOperatorParselet");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.PrefixOperatorParselet");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.formula.parselet.BinaryOperatorParselet");

sas.risk.common.widgets.spreadsheet.formula.Parser=function(tokens){
	this.tokens=tokens;
	this.read=[];
};
sas.risk.common.widgets.spreadsheet.formula.Parser.prototype={
	init:function(){
		var TokenType=sas.risk.common.widgets.spreadsheet.formula.TokenType;
		var Precedence=sas.risk.common.widgets.spreadsheet.formula.Precedence;

		this.registerPrefix(TokenType.NAME,new sas.risk.common.widgets.spreadsheet.formula.parselet.NameParselet());
		this.registerPrefix(TokenType.LEFT_PAREN,new sas.risk.common.widgets.spreadsheet.formula.parselet.GroupParselet());
		this.prefix(TokenType.PLUS, Precedence.PREFIX);
		this.prefix(TokenType.MINUS, Precedence.PREFIX);
		this.prefix(TokenType.TILDE, Precedence.PREFIX);
		this.prefix(TokenType.BANG, Precedence.PREFIX);
		this.prefix(TokenType.PERCENTAGE, Precedence.POSTFIX);
		this.prefix(TokenType.GREATER_THAN, Precedence.PREFIX);
		this.prefix(TokenType.GREATER_THAN_EQUAL, Precedence.PREFIX);

		this.registerInfix(TokenType.EQUAL,new sas.risk.common.widgets.spreadsheet.formula.parselet.ComparisonParselet());
		this.registerInfix(TokenType.NOT_EQUAL,new sas.risk.common.widgets.spreadsheet.formula.parselet.ComparisonParselet());
		this.registerInfix(TokenType.LESS_THAN,new sas.risk.common.widgets.spreadsheet.formula.parselet.ComparisonParselet());
		this.registerInfix(TokenType.LESS_THAN_EQUAL,new sas.risk.common.widgets.spreadsheet.formula.parselet.ComparisonParselet());
		this.registerInfix(TokenType.GREATER_THAN,new sas.risk.common.widgets.spreadsheet.formula.parselet.ComparisonParselet());
		this.registerInfix(TokenType.GREATER_THAN_EQUAL,new sas.risk.common.widgets.spreadsheet.formula.parselet.ComparisonParselet());
		this.registerInfix(TokenType.QUESTION,new sas.risk.common.widgets.spreadsheet.formula.parselet.ConditionalParselet());
		this.registerInfix(TokenType.LEFT_PAREN,new sas.risk.common.widgets.spreadsheet.formula.parselet.CallParselet());

		this.infixLeft(TokenType.PLUS, Precedence.SUM);
		this.infixLeft(TokenType.MINUS, Precedence.SUM);
		this.infixLeft(TokenType.ASTERISK, Precedence.PRODUCT);
		this.infixLeft(TokenType.SLASH, Precedence.PRODUCT);
		this.infixRight(TokenType.CARET, Precedence.EXPONENT);

		this.postfix(TokenType.BANG, Precedence.POSTFIX);
		this.postfix(TokenType.PERCENTAGE, Precedence.POSTFIX);
	},
	prefixParselets:{},
	infixParselets:{},
	registerPrefix:function(token,parselet){
		this.prefixParselets[token.name]=parselet;
	},
	registerInfix:function(token,parselet){
		this.infixParselets[token.name]=parselet;
	},
	parseExpression:function(precedence){
		if(arguments.length===0){
			precedence=0;
		}
		var token,prefix,left;
		var text=this.tokens.text;
		// S1431431.. if text startsWith ' and does not contain a '! it is a comment do not parse anything
		if (text.startsWith("'") && text.indexOf("'!") === -1) {
			prefix=this.prefixParselets["NAME"];
			token=new sas.risk.common.widgets.spreadsheet.formula.Token(sas.risk.common.widgets.spreadsheet.formula.TokenType.NAME, text.substr(1));
			left=prefix.parse(this,token);
			return left;
		}
		token=this.consume();
		prefix=this.prefixParselets[token.getType().name];
		if(!prefix){
			if(!token.getText()){return null;}
			throw "Could not parse \""+token.getText()+"\".";
		}
		left=prefix.parse(this,token);
		while(precedence<this.getPrecedence()){
			token=this.consume();
			var infix=this.infixParselets[token.getType().name];
			left=infix.parse(this,left,token);
		}
		return left;
	},
	match:function(expectedType){
		var token=this.lookAhead(0);
		if(token.getType()!== expectedType){
			return false;
		}
		this.consume();
		return true;
	},
	consume:function(expectedType){
		var token=this.lookAhead(0);
		if(expectedType){
			if(token.getType()!==expectedType){
				throw "Expected token "+expectedType.name+" and found "+token.getType().name;
			}
		}
		return this.read.shift();
	},
	lookAhead:function(distance){
		while(distance>=this.read.length){
			this.read.push(this.tokens.next());
		}
		return this.read[distance];
	},
	getPrecedence:function(){
		var parser=this.infixParselets[this.lookAhead(0).getType().name];
		return (parser)?parser.getPrecedence():0;
	},
	prefix:function(tokenType,precedence){
		this.registerPrefix(tokenType,new sas.risk.common.widgets.spreadsheet.formula.parselet.PrefixOperatorParselet(precedence));
	},
	infixLeft:function(tokenType,precedence){
		this.registerInfix(tokenType,new sas.risk.common.widgets.spreadsheet.formula.parselet.BinaryOperatorParselet(precedence));
	},
	infixRight:function(tokenType,precedence){
		this.registerInfix(tokenType,new sas.risk.common.widgets.spreadsheet.formula.parselet.BinaryOperatorParselet(precedence,true));
	},
	postfix:function(tokenType,precedence){
		this.registerInfix(tokenType,new sas.risk.common.widgets.spreadsheet.formula.parselet.PostfixOperatorParselet(precedence));
	}
};
sas.risk.common.widgets.spreadsheet.formula.Parser.prototype.init();