jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService");
jQuery.sap.require("sas.hc.ui.services.RestService");

sas.hc.ui.services.RestService.extend("sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService",{
	constructor:function(options){
		this.eventProvider=new sap.ui.base.EventProvider();
		this.mEventRegistry = {};
		this.oEventPool = new sap.ui.base.ObjectPool(sap.ui.base.Event);
		sas.hc.ui.services.RestService.apply(this,arguments);
	}
});
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.addDraftRow=function(newRow){
   var patch=this.model.patch;
   var dataKey=this.model.getDataKey();
   var rowPatch=[];
   patch[newRow[dataKey]]=rowPatch;
   _.each(newRow,function(value,key){
      if(key!==dataKey){//dont add the key column in the patch
         rowPatch.push({op:"add",path:key,value:value});
      }
   });
   this.model.fireChange({action:'add_draft'});
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.setModel=function(model){
	this.model=model;
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.detachLoaded=function(fnFunction, oListener){
	return this.eventProvider.detachEvent("loaded",fnFunction,oListener);
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.attachLoaded=function(oData, fnFunction, oListener){
	return this.eventProvider.attachEvent("loaded",oData,fnFunction,oListener);
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.fireLoaded=function(mParameters, bAllowPreventDefault, bEnableEventBubbling) {
	return this.eventProvider.fireEvent("loaded",mParameters, bAllowPreventDefault, bEnableEventBubbling);
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.detachLoading=function(fnFunction, oListener){
	return this.eventProvider.detachEvent("loading",fnFunction,oListener);
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.attachLoading=function(oData, fnFunction, oListener){
	return this.eventProvider.attachEvent("loading",oData,fnFunction,oListener);
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.fireLoading=function(mParameters, bAllowPreventDefault, bEnableEventBubbling) {
	return this.eventProvider.fireEvent("loading",mParameters, bAllowPreventDefault, bEnableEventBubbling);
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.removeRows=function(keys){
	var keysLength=keys.length;
	var missed=this._removeDraftRows(keys);
	if(keysLength && !(missed.length)){
		this.model.fireChange({action:'remove_rows'});
	}
	return missed;
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.setValue=function(cellId,columnId,value){
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.getDraftKeys=function(){
   var patch=this.model.patch;
   var draftKeys=[];
   _.each(patch,function(patchEntries,key){//remove all patch entries for draft rows
      if(key.startsWith('_draft_')){
         draftKeys.push(key);
      }
   });
   return draftKeys;
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.clearDrafts=function(){
   var draftKeys=this.getDraftKeys();
   var patch=this.model.patch;
   _.each(patch,function(patchEntries,key){//remove all patch entries for draft rows
      if(_.indexOf(draftKeys,key)!==-1){
         delete patch[key];
      }
   });
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.applyPatch=function(results){
   var that=this;
   var patch=this.model.patch;
   var dataKey=this.model.getDataKey();
   var draftKeys=this.getDraftKeys();
   //add draft rows at the top for rest based service
   if(!results.start){//are we dealing with the first page of the results (start=0)
      _.each(patch,function(patchEntries,key){
         if(_.indexOf(draftKeys,key)!==-1){
            var prevRow=_.find(results.items,function(item){return item[dataKey]===key;});//in case row is already there (local)
            if(!prevRow){
               var row={};
               row[dataKey]=key;
               results.items.unshift(row);
               results.count++;
            }
         }
      });
   }
   if(_.size(patch)){//here we apply the patch on top of the incoming results, patch is empty?(size=0) no need to do anything
      var deletedItems=[];
      _.each(results.items,function(item){
         var patchEntries=patch[item[dataKey]];//patch is an array of objects for each column of the row
         if(patchEntries){//found a patch for this row
            var patchedItem=that.applyPatchToRow(item,patchEntries);
            if(!patchedItem){//got deleted
               deletedItems.push(item);
            }
         }
      });
      if(deletedItems.length){//we have items to delete from the results, lets filter them out
         results.items=_.filter(results.items,function(item){return deletedItems.indexOf(item)===-1;});
      }
   }
};
sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.applyPatchToRow=function(row,patchEntries){
   var patchedRow=row;
   _.each(patchEntries,function(patch){
      switch(patch.op){
         case 'add':
         case 'replace':
            sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.addPropValue(row,patch.path,patch.value);
            break;
         case 'remove':
            if(patch.path){
               sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.delPropValue(row,patch.path);
            }else{
               patchedRow=null;//we hit a remove with not path, that means the whole row has been deleted
            }
            break;
      }
   });
   return patchedRow;
};

