jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService");

sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.extend("sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService", {
	constructor: function (options) {
		options = options || {};
		options.methods = ['get', 'post', 'delete', 'put'];
		sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.call(this, options);
		this.data = options.data || {
			limit: 0,
			count: 0,
			items: []
		};
	}
});
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.setData = function (data) {
	this.data = data;
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.addDraftRow = function (newRow) {
	var dataKey = this.model.getDataKey();
	this.data.items.unshift(newRow);
	sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.addDraftRow.apply(this, arguments);
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.clearDrafts = function () {
	this.removeRows(this.getDraftKeys());
	sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.prototype.clearDrafts.apply(this, arguments);
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.removeRows = function (keys) {
	var keysLength = keys.length;
	var dataKey = this.model.getDataKey(); //key
	var patch = this.model.patch;
	var that = this;
	_.each(keys, function (key) {
		var row = _.find(that.data.items, function (r) {
			return r[dataKey] === key;
		});
		if (row) {
			var index = that.data.items.indexOf(row);
			that.data.items.splice(index, 1);
			that.data.count--;
		} else {
			console.error("Couldn't find row with key " + key + " to remove from table"); // eslint-disable-line no-console
		}
		delete patch[key]; // eslint-disable-line no-console
	});
	if (keysLength) {
		this.model.fireChange({
			action: 'remove_rows'
		});
	}
	return $.Deferred().resolve(true);
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.getValues = function (cellId, columnId, value) {
	var values = {
		items: $.extend([], this.data.items),
		start: 0
	};
	values.limit = values.items.length;
	values.count = values.items.length;
	this.applyPatch(values);
	return values;
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.setValue = function (cellId, columnId, value) {
	var row = this.findRow(cellId);
	if (value === null) {
		delete row[columnId];
	} else {
		row[columnId] = value;
	}
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.get = function (options) {
	if (!this.model.isActive()) {
		return $.Deferred().resolve({
			items: [],
			start: 0,
			count: 0
		});
	}
	var values = {
		items: $.extend([], this.data.items),
		start: 0
	};
	this.applyPatch(values);
	var filteredData = this._findData(values, options);
	if (options.fetchCriteria) {
		if (options.fetchCriteria.getFilter()) {
			var filteredItems = this._filter(filteredData.items, options.fetchCriteria.getFilter(), options.meta);
			filteredData = {
				limit: filteredItems.length,
				count: filteredItems.length,
				start: 0,
				items: filteredItems
			};
		}
		var that = this;
		filteredData.items = _.reduce(options.fetchCriteria.getSorts(), function (filteredItems, sortInfo) {
			return that._sort(filteredItems, sortInfo, options.meta);
		}, filteredData.items);
	}
	filteredData.count = filteredData.items ? filteredData.items.length : 0;
	if (options.limit < filteredData.count) {
		this.model.setPageSize(filteredData.count);
	}
	this.fireLoaded({
		limit: filteredData.limit,
		count: filteredData.count
	});
	return $.Deferred().resolve(filteredData);
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.findRow = function (key) {
	var bindId = this.model.getBindId(); //items
	var dataKey = this.model.getDataKey(); //key
	var arrayNames = this.model.arrayNames;
	var roots = this.allRows();
	this.data[bindId];
	var that = this;
	var match = null;
	_.find(roots, function (row) {
		match = that._recurseFindRow(row, key, dataKey, arrayNames);
		return !!match;
	});
	return match;
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype.allRows = function (draftOnly) {
	var bindId = this.model.getBindId(); //items
	var arrayNames = this.model.arrayNames;
	var roots = this.data[bindId];
	var that = this;
	var rows = [];
	_.each(roots, function (row) {
		rows = rows.concat(that._recurseAllRows(row, arrayNames, draftOnly));
	});
	return rows;
};
//HELPERS
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype._findData = function (data, options) {
	if (!options.parentKeys || !options.parentKeys.length) {
		return data;
	}
	var bindId = this.model.getBindId();
	var parent = _.reduce(options.parentKeys, function (memo, parentKey) {
		var arrayName = _.find(options.arrayNames, function (arrayName) {
			return !!memo[arrayName];
		}) || bindId;
		return _.find(memo[arrayName], function (row) {
			return row.key === parentKey;
		});
	}, data);
	var arrayName = _.find(options.arrayNames, function (arrayName) {
		return !!parent[arrayName];
	});
	var childData = arrayName ? parent[arrayName] : [];
	return {
		limit: childData.length,
		count: childData.length,
		items: childData
	};
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype._recurseAllRows = function (row, arrayNames, draftOnly) {
	var that = this;
	var rows = [];
	if (draftOnly) { //looking only for drafts
		var draftKeys = this.getDraftKeys();
		if (_.indexOf(draftKeys, row.key) !== -1) { //if row's key is in the draft keys array, add it.
			rows.push(row);
		}
	} else {
		rows.push(row);
	}
	var arrayName = _.find(arrayNames, function (name) {
		return !!row[name];
	});
	if (arrayName) {
		_.each(row[arrayName], function (child) {
			rows = rows.concat(that._recurseAllRows(child, arrayNames, draftOnly));
		});
	}
	return rows;
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype._recurseFindRow = function (row, key, dataKey, arrayNames) {
	var that = this;
	var rowKey = row[dataKey];
	rowKey = _.isNumber(rowKey) ? rowKey.toString() : rowKey;
	if (rowKey === key) {
		return row;
	}
	var arrayName = _.find(arrayNames, function (name) {
		return !!row[name];
	});
	var match = null;
	if (arrayName) {
		_.find(row[arrayName], function (child) {
			match = that._recurseFindRow(child, key, dataKey, arrayNames);
			return !!match;
		});
	}
	return match;
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype._filter = function (items, filter, meta) {
	var that = this;
	return _.filter(items, function (row) {
		var cellId = row.key;
		return filter.evaluate(function (columnId) {
			var cellOptions = meta.getCellOptions(columnId, cellId);
			return that.model.getValue(cellId, columnId, cellOptions, row);
		});
	});
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService.prototype._sort = function (items, sortInfo, meta) {
	var that = this;
	var locale = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getLocale();
	var factor = (sortInfo.sortOrder === "Ascending") ? 1 : -1;
	var columnId = sortInfo.columnId;
	var sortProperty = meta.getColumnOptions(columnId).sortProperty;
	switch (meta.getColumnOptions(sortProperty).columnType) {
		case "number":
			var numericCollator = new Intl.Collator(locale.getLanguage(), {
				numeric: true
			});
			items.sort(function (a, b) {
				var aVal = that.model.getValue(a.key, sortProperty, meta.getCellOptions(sortProperty, a.key));
				var bVal = that.model.getValue(b.key, sortProperty, meta.getCellOptions(sortProperty, b.key));
				return factor * numericCollator.compare(aVal, bVal);
			});
			break;
		default: //string
			var stringCollator = new Intl.Collator(locale.getLanguage(), {});
			items.sort(function (a, b) {
				var aVal = that.model.getValue(a.key, sortProperty, meta.getCellOptions(sortProperty, a.key));
				var bVal = that.model.getValue(b.key, sortProperty, meta.getCellOptions(sortProperty, b.key));
				return factor * stringCollator.compare(aVal, bVal);
			});
			break;
	}
	return items;
};