jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.model.SpreadsheetPageableJSONMixin");

sas.risk.common.widgets.spreadsheet.model.SpreadsheetPageableJSONMixin={
	proto:{
		_initialize:function(settings){
			this.delta=settings.delta||[];
			this.undo=settings.undo||[];
			this._calcPatch(settings.patch);
			this.dataKey=settings.dataKey||"key";
			this.arrayNames=settings.arrayNames||[];
		},
		_calcPatch:function(defaultPatch){
			this.patch=defaultPatch;
			var that=this;
			_.each(this.delta,function(deltaOp){
				that._updatePatch(deltaOp);
			});
		},
		setService:function(service){
			service.setModel(this);
			sas.hc.ui.model.PageableJSONModel.prototype.setService.apply(this,arguments);
		},
		//Must be called each time you want to reactive the service endpoint
		setServiceData:function(serviceData){
			sas.hc.ui.model.PageableJSONModel.prototype.setServiceData.apply(this,arguments);
			this._isActive=!!(serviceData&&serviceData.activate);
		},
		isActive:function(){
			var val=this._isActive||false;
			//this._isActive=false;
			return val;
		},
		getDataKey:function(){
			return this.dataKey;
		},
		_setData:function(data){
			sas.hc.ui.model.PageableJSONModel.prototype._setData.apply(this,arguments);
			this._fillUpDataCache(data);
		},
		_fillUpDataCache:function(data){
			var that=this;
			this._cacheMap=_.reduce(data,function(memo,row){
				memo[row[that.dataKey]]=row;
				return memo;
			},this._cacheMap||{});
		},
		getValue:function(cellId,columnId,cellOptions,row){
			row=row||this._cacheMap[cellId];
			if(!row){return null;}
			var path=cellOptions.path;
			var colPath=path? path+"/"+columnId: columnId;
			var entryDelta=this.patch[cellId]||[];
			var delta=_.find(entryDelta,function(d){
				return d.path===colPath;
			});
			return delta?(delta.op==="remove"?null:delta.value):this.findValueInRow(row,path?path.split("/"):[],columnId);
		},
		findValueInRow:function(base,paths,columnId){
			var path=paths.shift();
			if(path){
				var parent=base[path];
				if(parent){
					return this.findValueInRow(parent,paths,columnId);
				}
			}else{
				return base[columnId];
			}
			return null;
		},
		setValue:function(cellId,columnId,value,cellOptions,updatePatchOnly){
			var row=this._cacheMap[cellId];
			if(!row){return null;}
			var path=cellOptions.path;
			var colPath=path?path+"/"+columnId:columnId;
			var patchPath=cellId+"/"+colPath;
			var prevValue=this.getValue(cellId,columnId,cellOptions);
			if(_.isNull(prevValue)||_.isUndefined(prevValue)){
				this.undo.push({op:"remove",path:patchPath});
				var deltaOp={op:"add",   path:patchPath,value:value};
				this.delta.push(deltaOp);
				this._updatePatch(deltaOp);
			}else if(_.isNull(value)||_.isUndefined(value)){
				this.undo.push({op:"add",   path:patchPath,value:prevValue});
				var deltaOp={op:"remove",path:patchPath};
				this.delta.push(deltaOp);
				this._updatePatch(deltaOp);
			}else{
				this.undo.push({op:"replace",path:patchPath,value:prevValue});
				var deltaOp={op:"replace",path:patchPath,value:value};
				this.delta.push(deltaOp);
				this._updatePatch(deltaOp);
			}
			this.service.setValue(cellId,columnId,value);
			if(!updatePatchOnly && (prevValue || value)){
				this.fireChange({newValue:value,cellId:cellId,columnId:columnId});
			} 
		},
		_updatePatch:function(deltaOp){
			var paths=deltaOp.path.split("/");
			var cellId=paths.splice(0,1);
			var colPath=paths.join("/");
			var entryPatch=this.patch[cellId]||[];
			var index=_.findIndex(entryPatch,function(delta){
				return delta.path===colPath;
			});
			if(index>=0){
				entryPatch.splice(index,1);
			}
			entryPatch.push($.extend({},deltaOp,{path:colPath}));
			this.patch[cellId]=entryPatch;
		},
		discardPatch:function(){
			this.service.clearDrafts();
			this.undo=[];
			this.delta=[];
			this.patch={};
			this.fireChange({});
		},
		getDelta:function(){
			return this.delta;
		},
		getPatch:function(){
			return this.patch;
		},
		getUndo:function(){
			return this.undo;
		},
		attachChange:function(oData, fnFunction, oListener){
			this.attachEvent("change", oData, fnFunction, oListener);
			return this;
		},
		detachChange:function(fnFunction, oListener) {
			this.detachEvent("change", fnFunction, oListener);
			return this;
		},
		getRowKeys:function(){
			return Object.keys(this._cacheMap);
		},
		fireChange:function(mArguments) {
			this.fireEvent("change", mArguments);
			return this;
		}		
	}
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetPageableJSONMixin.mixin=function(klass){
	$.extend(klass.prototype,sas.risk.common.widgets.spreadsheet.model.SpreadsheetPageableJSONMixin.proto);
};