jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService");
jQuery.sap.require("sas.hc.ui.services.RestService");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");

sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.extend("sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService",{
	constructor:function(options){
		options=options||{};
		options.methods=['get','post','delete','put'];
		sas.risk.common.widgets.spreadsheet.model.ASpreadsheetService.call(this,options);
		this.importData=null;
	}
});
sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService.prototype.removeRows=function(keys,options){
	var def=$.Deferred();
	var patch=this.model.patch;
	var draftKeys=this.getDraftKeys();
	_.each(keys,function(key){
	   if(_.indexOf(draftKeys,key)!==-1){//for drafts we just take out the patch entry entirely, server doesnt know about it
	      delete patch[key];
	   }else{//override any existing patch info with a delete
	      var opsInPatch =[];
	      opsInPatch.push({op:"remove"});
	      patch[key]= opsInPatch;
	   }
	});
	this.model.fireChange({action:'remove_rows'});
	def.resolve(true);
	return def;
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService.prototype.deleteRows=function(keys,options){
	var defs=[];
	_.each(keys,function(key){
		defs.push($.ajax({
			headers: options.getDeleteHeaders(key),
			method: options.getDeleteMethod(key),
			url: options.getDeleteUrl(key)
		}));
	});
	return $.when.apply($,defs).done($.proxy(this.deleteCallbackSuccess,this)).fail($.proxy(this.deleteCallbackFailure,this));
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService.prototype.deleteCallbackSuccess=function(){
	this.model.fireChange({action:'remove_rows'});
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService.prototype.deleteCallbackFailure=function(){
};
sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService.prototype.get=function(options){
	if(!this.model.isActive()){return $.Deferred().resolve({items:[],start:0,count:0});}
	if(options.fetchCriteria){
		var hasParams=this.url.indexOf("?")!==-1;
		var queryParams=options.fetchCriteria.buildQueryParameters(options.meta);
		options.url=(queryParams)?this.url+(hasParams?"&":"?")+queryParams:this.url;
		options.data=$.extend(true,{},options.data); 
		delete options.fetchCriteria;
		delete options.data.fetchCriteria;
		delete options.sortBy;
		delete options.data.sortBy;
	}
	if(options.meta){
		delete options.meta;
		delete options.data.meta;
	}
	
	var importData = this.importData;
	
	var that=this;
	var def=$.Deferred();
	this.fireLoading();
	sas.hc.ui.services.RestService.prototype.get.call(this,options).done(function(results){
		if (importData) {
		   that.importData=null;
		   if (that.save_results) {
		     results=that.save_results;
		   }
		}
		
		that.applyPatch(results);
		if(results.items && results.items.length<results.limit){
			//if the limit exceeds the items.length, the table adds a bunch of empty rows to fill in the difference, resulting in a vert scrollbar
			//overriding the limit so that these two match exactly and no extra rows are added
			results.limit=results.items.length;
		}
		if(results.items && ((results.limit < results.items.length) || (!results.limit && results.items.length))) {
			that.model.setPageSize(results.items.length);
		}
		
		that.save_results=results;
		
		def.resolve(results);
		that.fireLoaded({limit:results.limit,count:results.count});
	}).always(function(args) {
		that.fireLoaded(args);
	});	
	return def;
};
