jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.ui.core.Item");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.StringFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.OrFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.AndFilter");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch",{
	metadata:{
		properties:{
			spreadsheet:{type:"any"},
			toolbar1:{type:"any"},
			toolbar2:{type:"any"},
			footer:{type:"any"},
			menu:{type:"any"},
			spreadsheetToolbarType:{type:"any"}
		}
	},
	constructor:function(){
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
		this.addStyleClass("riskSpreadsheetBroadSearch");
		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center);
	},
	renderer:sas.risk.common.layout.flexlayout.FlexLayoutRenderer
});
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.setSpreadsheet=function(spreadsheet){
	this.setProperty("spreadsheet",spreadsheet);
	spreadsheet.attachPreFetchData($.proxy(this.onPreFetchData,this));
	spreadsheet.attachPostFetchData($.proxy(this.onPostFetchData,this));
	spreadsheet.attachPostMetaChange($.proxy(this.onPostMetaChange,this));
	spreadsheet.attachSelect($.proxy(this.onSelect,this));
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.onPostMetaChange=function(event){
	var spreadsheet=this.getSpreadsheet();
	var sheetId=spreadsheet.getSelectedSheetId();
	var meta=spreadsheet.getMetaInfo(sheetId).meta;
	this.removeAllItems();
	this.search=new sas.hc.m.SearchField({
		search:function(){
			var sheetId=spreadsheet.getSelectedSheetId();
			
			spreadsheet.getSelectedSheetTable().setFirstVisibleRow(0);
			spreadsheet.getSelectedSheetTable().setBusyIndicatorDelay(0);
			spreadsheet.getSelectedSheetTable().setBusy(true);
			spreadsheet.fetchData(sheetId);
		}
	});
	this.search.getEnabled=function(val){return true;};//SearchField should be always enabled irrespective of whether table is readonly or not
	this.searchColumns=_.filter(meta.getColumnOptions(),function(columnOption){
		return !!columnOption.quickSearch;
	});
	switch(this.searchColumns.length){
		case 0:
			this.setVisible(false);
		break;
		default:
			this.setVisible(true);
			this.addItem(this.search);
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.onPostFetchData=function(event){
	var sheetId=this.getSpreadsheet().getSelectedSheetId();
	var fetchCriteriaMap=event.getParameter("fetchCriteria");
	this._refresh(fetchCriteriaMap[sheetId]);
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.onSelect=function(event){
	if(this.search){
		var spreadsheet=this.getSpreadsheet();
		var table=spreadsheet.getSelectedSheetTable();
		var sheetId=table.data("sheetId");
		var fetchCriteriaMap=spreadsheet.getFetchCriteria();
		this._refresh(fetchCriteriaMap[sheetId]);
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype._refresh=function(fetchCriteria){
	var filter=fetchCriteria.getFilterSimple();
	var columnIds=_.pluck(this.searchColumns,"id");
	var term=this.processAllTypes(filter,columnIds);
	if(!term && filter instanceof sas.risk.common.widgets.spreadsheet.filter.AndFilter){
		var that=this;
		_.find(filter.getChildren(),function(f){
			term=that.processAllTypes(f,columnIds);
			return !!term;
		});
	}
	if(term && term.length){
		this.search.setValue(term.join(" "));
	}else{
		if(this.search.getValue()){
			this.search.setValue("");
		}
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.processAllTypes=function(filter,columnIds){
	if(columnIds.length===1){
		return this.processSingleTermSingleColumn(filter,columnIds[0])||this.processMultiTermSingleColumn(filter,columnIds[0]);
	}else if(columnIds.length>1){
		return this.processSingleTermMultiColumn(filter,columnIds)||this.processMultiTermMultiColumn(filter,columnIds);
	}
	return null;	
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.processMultiTermMultiColumn=function(filter,columnIds){
	if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.OrFilter && filter.getChildren().length && columnIds.length){
		var children=filter.getChildren();
		var terms=[];
		var cols=[];
		for(var i=0;i<children.length;i++){
			var match=this.processMultiTermSingleColumn(children[i]);
			if(!term){term=match;}//first time around
			if(!match || term.length!==match.length || _.difference(term,match).length){return null;}//short circuit
			cols.push(children[i].getChildren()[0].getName());
		}
		return (_.uniq(cols).length===columnIds.length)?terms:null;
	}
	return null;
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.processSingleTermMultiColumn=function(filter,columnIds){
	if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.OrFilter && filter.getChildren().length===columnIds.length && columnIds.length){
		var children=filter.getChildren();
		var term=null;
		var cols=[];
		for(var i=0;i<children.length;i++){
			var match=this.processSingleTermSingleColumn(children[i]);
			if(!term){term=match;}//first time around
			if(!match || term[0]!==match[0]){return null;}//short circuit
			cols.push(children[i].getName());
		}
		return (_.uniq(cols).length===columnIds.length)?[term]:null;
	}
	return null;
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.processSingleTermSingleColumn=function(filter,columnId){
	if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.StringFilter && filter.getOperator()==="contains"){
		return (!columnId||(columnId===filter.getName()))?[filter.getValue().trim()]:null;
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.processMultiTermSingleColumn=function(filter,columnId){
	if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.AndFilter){
		var collectedTerms=[];
		var children=filter.getChildren();
		var columns=[];
		for(var i=0;i<children.length;i++){
			var match=this.processSingleTermSingleColumn(children[i],columnId);
			if(match){
				collectedTerms.push(match[0]);
				columns.push(children[i].getName());
			}else{
				return null;
			}
		}
		return _.uniq(columns).length===1?collectedTerms:null;
	}
	return null;
};
sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch.prototype.onPreFetchData=function(event){
	var sheetId=this.getSpreadsheet().getSelectedSheetId();
	var fetchCriteria=event.getParameter("fetchCriteria")[sheetId];
	var searchValue=this.search.getValue();
	if(searchValue){
		if(this.searchColumns.length){
			var chunks=searchValue.trim().split(" ");
			if(chunks.length===1){//SINGLE SEARCH TERM
				if(this.searchColumns.length===1){//SINGLE SEARCH COLUMN
					fetchCriteria.andSimpleFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter(this.searchColumns[0].id,"contains",chunks[0],this.searchColumns[0].label,searchValue));
				}else{//MULTI SEARCH COLUMN
					var orFilter=new sas.risk.common.widgets.spreadsheet.filter.OrFilter();
					_.each(this.searchColumns,function(column){
						orFilter.addFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter(column.id,"contains",chunks[0],column.label,searchValue));
					});
					fetchCriteria.andSimpleFilter(orFilter);
				}
			}else{//MULTI SEARCH TERMS
				var that=this;
				if(this.searchColumns.length===1){//SINGLE SEARCH COLUMN
					var andFilter=new sas.risk.common.widgets.spreadsheet.filter.AndFilter();
					_.each(chunks,function(chunk){
						andFilter.addFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter(that.searchColumns[0].id,"contains",chunk,that.searchColumns[0].label,chunk));
					});
					fetchCriteria.andSimpleFilter(andFilter);
				}else{//MULTI SEARCH COLUMN
					var orFilter=new sas.risk.common.widgets.spreadsheet.filter.OrFilter();
					_.each(this.searchColumns,function(column){
						var andFilter=new sas.risk.common.widgets.spreadsheet.filter.AndFilter();
						_.each(chunks,function(chunk){
							andFilter.addFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter(column.id,"contains",chunk,column.label,chunk));
						});
						orFilter.addFilter(andFilter);
					});
					fetchCriteria.andSimpleFilter(orFilter);
				}
			}
		}
	}
};
