jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.ColumnOrder");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");

sas.risk.common.widgets.spreadsheet.toolbar.ColumnOrder=function(spreadsheet){
	this.spreadsheet=spreadsheet;
	this.spreadsheet.attachPreFetchData($.proxy(this.handleColumnOrderPreFetchData,this));
	this.spreadsheet.attachPostFetchData($.proxy(this.handleColumnOrderPostFetchData,this));
	this.spreadsheet.attachPreMetaChange($.proxy(this.onPreMetaChange,this));
	this.attached={};
};
sas.risk.common.widgets.spreadsheet.toolbar.ColumnOrder.prototype={
		onPreMetaChange:function(event){//spreadsheet initializing
			var that=this;
			var defs=event.getParameter("defs");
			var def=$.Deferred();
			defs.unshift(def);
			defs.pop().done(function(applied){
				var innerDefs=[];
				_.each(that.spreadsheet.getMeta(),function(metaInfo){
					var sheetId=metaInfo.id;
					var meta=that.spreadsheet.getMetaInfo(sheetId).meta;
					var preferencesKey=that.spreadsheet.getPreferencesKey()[sheetId];
					var innerDef=$.Deferred();
					innerDefs.push(innerDef);
					if(applied||!preferencesKey){return innerDef.resolve();}
					that.spreadsheet.getPreferencesAPI().loadPreferences(preferencesKey).done(function(columnsStr){
						if(columnsStr){
							meta.overrideVisibleColumnIds(columnsStr.split("|"));
						}
						innerDef.resolve();
					});
				});
				$.when.apply($,innerDefs).done(function(){
					def.resolve();
				});
			});
		},
		_getDialog:function(){
			var table=this.spreadsheet.getSelectedSheetTable();
			var omb=table.getAggregation("tableOptionsMenuButton");
			return omb.getAggregation("tableOptionsColumnsDialog");
		},
		_getDualSelector:function(){
			var dialog=this._getDialog();
			return dialog.getContent()[0];
		},
		addDefaultsButton:function(){
			var dialog=this._getDialog();
			var middleButton=dialog.getMiddleButton();
			if(!middleButton){
				var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
				middleButton=new sas.hc.m.Button({text:rb.getText("spreadsheet.managecolumns.defaults.txt"),press:$.proxy(this.restoreDefaults,this)});
				//SAS overrides to Dialog prevents us from adding a middle button
				var onBeforeRenderingOrig=dialog.onBeforeRendering;
				var onBeforeRenderingOverride=function(){
					onBeforeRenderingOrig.apply(this,arguments);
					this.setMiddleButton(middleButton);
					var toolbar=dialog._getToolbar();
					var beginButton=dialog.getBeginButton();
					var endButton=dialog.getEndButton();
					var spacer=toolbar.getContent()[0];
					toolbar.removeAllContent();
					toolbar.addContent(spacer);
					toolbar.addContent(beginButton);
					toolbar.addContent(middleButton);
					toolbar.addContent(endButton);
				};
				dialog.onBeforeRendering=onBeforeRenderingOverride;
			}
		},
		addOkButtonValidator:function(){
			var dialog=this._getDialog();
			var okButton=dialog.getBeginButton();
			okButton.mEventRegistry.press=[{fFunction:$.proxy(this.handleColumnPickerChange,this,okButton.mEventRegistry.press)}];
		},
		restoreDefaults:function(){
			var table=this.spreadsheet.getSelectedSheetTable();
			var sheetId=this.spreadsheet.getSelectedSheetId();
			var meta=this.spreadsheet.getMetaInfo(sheetId).meta;
			var defaultVisibles=[];
			_.each(meta.getColumnOptions(),function(columnOption){
				if(columnOption.defaultVisible || columnOption.defaultVisible === undefined){
					defaultVisibles.push(columnOption.id);
				}
			});
			var ds=this._getDualSelector();
			var dsVisibleColumns=[];
			var dsHiddenColumns=[];
			var defaultVisiblesIds=[];
			_.each(table.getColumns(),function(column){
				var index=defaultVisibles.indexOf(column.getName());
				if(index!==-1){
					defaultVisiblesIds[index]=column.getId();
				}
			});
			var dsValues=ds.getTargetList().getModel().getProperty("/itemList").concat(ds.getModel().getProperty("/columns"));
			_.each(dsValues,function(dsValue){
				if(defaultVisiblesIds.indexOf(dsValue.id)===-1){
					dsHiddenColumns.push(dsValue);
				}else{
					dsVisibleColumns.push(dsValue);
				}
			});
			dsVisibleColumns=_.sortBy(dsVisibleColumns,function(dsValue){
				return defaultVisiblesIds.indexOf(dsValue.id);
			});
			ds.getModel().setProperty("/columns",dsHiddenColumns);
			ds.getTargetList().getModel().setProperty("/itemList",dsVisibleColumns);
		},
		handleColumnPickerChange:function(listenerChain,event,args1){
			var sheetId=this.spreadsheet.getSelectedSheetId();
			var key=this.spreadsheet.getPreferencesKey()[sheetId];
			if(key){
				var preferencesAPI=this.spreadsheet.getPreferencesAPI();
				var ds=this._getDualSelector();
				var columnIdArray=_.pluck(ds.getTargetList().getModel().getProperty("/itemList"),"id");
				var tableColumns=this.spreadsheet.getSelectedSheetTable().getColumns();
				var columnNameArray=_.map(columnIdArray,function(columnId){
					var columnMatch=_.find(tableColumns,function(col){
						return col.getId()===columnId;
					});
					return columnMatch.getName();
				});
				var value=columnNameArray.join("|");
				var that=this;
				preferencesAPI.updatePreferences(key,value).done(function(){
					_.each(listenerChain,function(listener){
						listener.fFunction.call(listener.oListener||event.getSource(),event,args1);
					});
					
					that.spreadsheet.getSelectedSheetTable().setBusyIndicatorDelay(0);
					that.spreadsheet.getSelectedSheetTable().setBusy(true);
					that.spreadsheet.fetchData(sheetId);
				}).fail(function(e,jqxhr){
					console.error(arguments);// eslint-disable-line no-console
					var json=e.responseJSON||(jqxhr&&jqxhr.responseJSON);
					jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");
			    	sas.risk.common.widgets.MessageDetailsBox.error(json, sas.config.appName, sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle());					
				});
			}else{
				_.each(listenerChain,function(listener){
					listener.fFunction.call(listener.oListener||event.getSource(),event,args1);
				});
				this.spreadsheet.getSelectedSheetTable().setBusyIndicatorDelay(0);
				this.spreadsheet.getSelectedSheetTable().setBusy(true);
				this.spreadsheet.fetchData(sheetId);
			}
		},
		handleColumnOrderPreFetchData:function(event){
			var table=this.spreadsheet.getSelectedSheetTable();
			var sheetId=table.data("sheetId");
			var fetchCriteria=event.getParameter("fetchCriteria");
			var visibleColumns=[];
			_.each(table.getColumns(),function(column){
				if(column.getVisible()){
					visibleColumns.push(column.getName());
				}
			});
			fetchCriteria[sheetId].setColumnIds(visibleColumns);
		},
		handleColumnOrderPostFetchData:function(event){
			var table=this.spreadsheet.getSelectedSheetTable();
			var sheetId=table.data("sheetId");
			if(this.attached[sheetId]){
				table.detachBeforeColumnsDialogOpen($.proxy(this.addDefaultsButton,this));
				table.detachAfterColumnsDialogOpen($.proxy(this.addOkButtonValidator,this));
			}
			table.attachBeforeColumnsDialogOpen($.proxy(this.addDefaultsButton,this));
			table.attachAfterColumnsDialogOpen($.proxy(this.addOkButtonValidator,this));
			this.attached[sheetId]=true;
			var fetchCriteria=event.getParameter("fetchCriteria")[sheetId];
			var columnIds=fetchCriteria.getColumnIds();
			var visibleColumns=[];
			var hiddenColumns=[];
			var isDifferent=false;
			var visibleCount=0;
			_.each(table.getColumns(),function(column){
				if(column.getVisible()){
					if(column.getName()!==columnIds[visibleCount]){
						isDifferent=true;
					}
					visibleCount++;
				}
				if(columnIds.indexOf(column.getName())!==-1){
					column.setVisible(true);
					visibleColumns.push(column);
				}else{
					column.setVisible(false);
					hiddenColumns.push(column);
				}
			});
			if(isDifferent){
				table.removeAllColumns();
				_.each(visibleColumns.concat(hiddenColumns),function(column){
					table.addColumn(column);
				});
			}
		}
};