jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.ExportButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.AButtonAction");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
//dialog imports
jQuery.sap.require("sap.ui.commons.layout.HorizontalLayout");
jQuery.sap.require("sas.hc.m.Text");

jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.hc.m.MessageSummary");
jQuery.sap.require("sap.ui.model.json.JSONModel");

sas.risk.common.widgets.spreadsheet.toolbar.AButtonAction.extend("sas.risk.common.widgets.spreadsheet.toolbar.ExportButton", {
	constructor: function () {
		sas.risk.common.widgets.spreadsheet.toolbar.AButtonAction.apply(this, arguments);
		this.getEnabled=function(){return true;};
		this.setIcon(sas.icons.HC.EXPORT);
	},
	renderer: "sas.hc.m.ButtonRenderer"
});
sas.risk.common.widgets.spreadsheet.toolbar.ExportButton.prototype.onPress = function () {

	var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
	var viewData = {
		title: rb.getText("spreadsheet.export.dialog.title.txt"),
		formattingKey: sas.risk.rgf.util.RGFUtil.buildDialogFormattingKey("ExportExcelDialog", this.getSpreadsheet().getParent().getParent().getCpbForm(), "SpreadSheet", this.getSpreadsheet().getParent().getParent().getCpbName()),
		spreadsheet: this.getSpreadsheet()
	};
	sas.risk.rgf.util.DialogManager.open("ExportExcelDialog", viewData, null, null);
};

/**
 * Export Excel Dialog
 */

sap.ui.jsview("sas.risk.rgf.view.dialog.ExportExcelDialog", {
	getControllerName: function () {
		return "sas.risk.rgf.view.dialog.ExportExcelDialog";
	},
	createContent: function (controller) {
		this.controller = controller;
		var that = this;
		var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
		this.fileSucessfullyUploaded = false;
		this.viewData = this.getViewData();
		$.proxy(this.createButtons(controller), this);

		this.message = new sas.hc.m.Text({
			text: rb.getText("spreadsheet.export.dialog.warning.txt")
		});

		this.layout = new sas.risk.common.layout.flexlayout.FlexLayout({
			flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
			items: [
				this.message
			]
		 });
		 return this.layout;
	},
	createButtons: function (controller) {
		var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
		this.okButton = new sas.risk.common.widgets.Button({
			text: rb.getText("button.ok.action.label"),
			press: $.proxy(controller.actionOk, this, controller)
		});
		this.cancelButton = new sas.risk.common.widgets.Button({
			text: rb.getText("button.cancel.action.label"),
			press: $.proxy(controller.actionCancel, this, controller)
		});
	},
	getDialogSettings: function () {
		return {
			contentWidth: "25%",
			minWidth: 400,
			beginButton: this.okButton,
			endButton: this.cancelButton
		};
	}
});
sap.ui.controller("sas.risk.rgf.view.dialog.ExportExcelDialog", {

	actionOk: function (controller) {
		var dialog = controller.getView().getDialog();
		dialog.setBusyIndicatorDelay(0);
		dialog.setBusy(true);
		var spreadsheet = this.viewData.spreadsheet;

		//construct data
		var fileData;
		if(!(spreadsheet.getModel(spreadsheet.getSheetIds()[0]).getService() instanceof sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService)){
			//SpreadsheetLocalService
			fileData = spreadsheet.getValues();
			if (!Array.isArray(fileData)) {
				fileData = [fileData];
			}
		} else {
			//SpreadsheetRestService
			that=this;
			fileData = [];
			_.each(spreadsheet.getSheetIds(), function(sheetId, index) {
				var model = spreadsheet.getModel(sheetId);
				var data = model.getData();
				fileData.push(spreadsheet.getValues());
				fileData[index].items = data.items;
				fileData[index].count = data.items.length;
			});
		}
		
		//construct meta
		var fileMeta = [];
		_.each(spreadsheet.getSheetIds(), function (sheetId) {
			fileMeta.push(sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.objectMetaToFileMeta(spreadsheet.getMetaInfo(sheetId)));
		});

		var exportURL = sas.risk.rgf.util.RGFUtil.getRestUrl() + "/utils/exportSpreadsheet";
		if (!exportURL) {
			console.error("exportURL is not provided. Can't perform Export to Excell."); /*I18nOK:EMS*/
			return;
		}
		exportURL = sas.risk.rgf.util.URLParser.parseURL(exportURL).removeParam("start");
		exportURL = sas.risk.rgf.util.URLParser.parseURL(exportURL).removeParam("limit");

		var xhr = new XMLHttpRequest();
		xhr.open("POST", exportURL, true);
		xhr.setRequestHeader("content-type", "application/json;charset=UTF-8");
		xhr.responseType = "blob";
		xhr.onload = function (e) {
			if (this.status === 200) {
				var contentDispositionHeader = xhr.getResponseHeader("content-disposition");
				var fileName = contentDispositionHeader.slice(contentDispositionHeader.indexOf("filename")).replace("filename=\"", "").replace("\"", "");
				var blob = this.response;
				if (window.navigator.msSaveBlob) {
					window.navigator.msSaveBlob(blob, fileName);
				} else {
					var downloadUrl = URL.createObjectURL(blob);
					var a = document.createElement("a");
					a.href = downloadUrl;
					a.download = fileName;
					document.body.appendChild(a);
					a.click();
				}
			} else {
				console.error("recieved non-normal status code from the server:", e);
			}
		};
		xhr.onerror = function (e) {
			console.log(e);
			alert('Unable to download excel for dataload.'); //I18nOK:EMS
		};
		xhr.send(JSON.stringify({
			"data": fileData,
			"meta": fileMeta
		}));

		dialog.setBusy(false);
		dialog.close();
	},
	actionCancel: function (controller) {
		var dialog = controller.getView().getDialog();
		dialog.close();
	}
});