jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.Filters");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.ui.core.Item");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.StringFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.NumberFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.AndFilter");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetUtil");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.common.widgets.spreadsheet.toolbar.Filters", {
	metadata: {
		properties: {
			spreadsheet: {
				type: "any"
			},
			toolbar1: {
				type: "any"
			},
			toolbar2: {
				type: "any"
			},
			footer: {
				type: "any"
			},
			menu: {
				type: "any"
			},
			spreadsheetToolbarType: {
				type: "any"
			}
		}
	},
	constructor: function () {
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this, arguments);
		this.addStyleClass("riskSpreadsheetFilters");
		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Baseline);
	},
	renderer: sas.risk.common.layout.flexlayout.FlexLayoutRenderer
});
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype.setSpreadsheet = function (spreadsheet) {
	this.setProperty("spreadsheet", spreadsheet);
	spreadsheet.attachPreFetchData($.proxy(this.onPreFetchData, this));
	spreadsheet.attachPostFetchData($.proxy(this.onPostFetchData, this));
	spreadsheet.attachPostMetaChange($.proxy(this.onPostMetaChange, this));
	spreadsheet.attachSelect($.proxy(this.onSelect, this));
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype.onPostMetaChange = function (event) {
	var spreadsheet = this.getSpreadsheet();
	var sheetId = spreadsheet.getSelectedSheetId();
	var meta = spreadsheet.getMetaInfo(sheetId).meta;
	this._refreshFilterSelects(meta);
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype.getFilterColumns = function () {
	var that = this;
	return new Promise(function (resolve, reject) {
		var start = Date.now();
		var timeout = 10;

		function checkFilter() {
			if (Date.now() > start + timeout * 1000) {
				reject("Loading filter columns timed out after " + timeout + " seconds");
			} else if (!that.filterColumns) {
				setTimeout(checkFilter, 100);
			} else {
				resolve(that.filterColumns);
			}
		}
		checkFilter();
	});
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype._refreshFilterSelects = function (meta) {
	this.removeAllItems();
	var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	var that = this;
	this.filterColumns = _.filter(meta.getColumnOptions(), function (columnOption) {
		return !!columnOption.filterable;
	});
	this.setVisible(!!this.filterColumns.length);
	var spreadsheet = that.getSpreadsheet();
	var model = spreadsheet.initFilterModel(this.filterColumns.length);
	var sheetId = spreadsheet.getSelectedSheetId();
	
	var meta = spreadsheet.getMetaInfo(sheetId).meta;
	
	if(meta.tableOptions.filterPanelOpen != null)
		model.setProperty("/filterPanelOpen", meta.tableOptions.filterPanelOpen);

	_.each(this.filterColumns, function (filterColumn, index) {
		var labelText;
		if (filterColumn.multiLabels) {
			labelText = filterColumn.multiLabels[filterColumn.multiLabels.length - 1].getText();
		} else {
			labelText = filterColumn.label;
		}
		var filterLabel = new sap.m.Label({
			text: rb.getText("table.toolbar.filter.label.fmt.txt", [labelText])
		}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
			flexShrink: 0
		}));
		var filter = new sas.hc.m.Select({
			change: function (evt) {
				var selected = !(evt.getParameter("selectedItem").getKey() === "___SPECIAL_ALLITEMS___");
				model.setProperty("/filterSelected/" + index, selected);

				//model.setProperty("/filterSelected", existing + selected);

				spreadsheet.getSelectedSheetTable().setFirstVisibleRow(0);
				spreadsheet.getSelectedSheetTable().setBusyIndicatorDelay(0);
				spreadsheet.getSelectedSheetTable().setBusy(true);
				spreadsheet.fetchData(sheetId);
			}
		});
		filter.data("meta", filterColumn, false);
		filter.getEnabled = function (val) {
			return true;
		}; //filter should be always enabled irrespective of whether table is readonly or not
		filterLabel.setLabelFor(filter);
		if (filterColumn.useAllFilter) {
			filter.addItem(new sas.hc.ui.core.Item({
				key: "___SPECIAL_ALLITEMS___",
				text: rb.getText("table.toolbar.filter.all.txt"),
			}));
		}
		if (filterColumn.useNoneFilter) {
			filter.addItem(new sas.hc.ui.core.Item({
				key: "",
				text: rb.getText("select.none.selected.txt")
			}));
		}
		var cellOptions = meta.getCellOptions(filterColumn.id);
		_.each(cellOptions.choices, function (choice) {
			filter.addItem(new sas.hc.ui.core.Item({
				key: choice.value,
				tooltip: choice.label,
				text: choice.label
			}));
		});
		var labeledFilter = new sas.risk.common.layout.flexlayout.FlexLayout({
			flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.row,
			items: [
				filterLabel,
				filter
			]
		}).addStyleClass("riskSpreadsheetFiltersContainer");
		that.addItem(labeledFilter);
	});
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype.onSelect = function (event) {
	if (this.getItems().length) {
		var spreadsheet = this.getSpreadsheet();
		var table = spreadsheet.getSelectedSheetTable();
		var sheetId = table.data("sheetId");
		var meta = spreadsheet.getMetaInfo(sheetId).meta;
		var fetchCriteria = spreadsheet.getFetchCriteria()[sheetId];
		this._refreshFilterSelects(meta);
		this._refreshFilters(fetchCriteria);
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype.onPostFetchData = function (event) {
	var sheetId = this.getSpreadsheet().getSelectedSheetId();
	var fetchCriteria = event.getParameter("fetchCriteria")[sheetId];
	this._refreshFilters(fetchCriteria);
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype._refreshFilters = function (fetchCriteria) {
	var filter = fetchCriteria.getFilter();
	var that = this;
	if (this.getItems() && this.getItems().length) {
		_.each(this.getItems(), function (outerLayout) {
			if (outerLayout.getItems() && outerLayout.getItems().length) {
				_.each(outerLayout.getItems(), function (f) {
					if (f instanceof sas.hc.m.Select) {
						var meta = f.data("meta");
						var matchingFilter = that.findMatchingFilter(filter, meta.id);
						if (matchingFilter) {
							f.setSelectedKey(matchingFilter.getValue());
						}
					}
				});
			}
		});
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype.findMatchingFilter = function (filter, columnId) {
	if (filter) {
		if ((filter instanceof sas.risk.common.widgets.spreadsheet.filter.StringFilter ||
			filter instanceof sas.risk.common.widgets.spreadsheet.filter.NumberFilter)   //RGFRMWRK-605
			&& filter.getOperator() === "eq") {
			if (columnId === filter.getName()) {
				return filter;
			}
		} else if (filter instanceof sas.risk.common.widgets.spreadsheet.filter.AndFilter && !filter.isAdvancedFilter) {
			var that = this;
			return _.find(filter.getChildren(), function (child) {
				return !!that.findMatchingFilter(child, columnId);
			});
		}
	}
	return null;
};
sas.risk.common.widgets.spreadsheet.toolbar.Filters.prototype.onPreFetchData = function (event) {
	var sheetId = this.getSpreadsheet().getSelectedSheetId();
	var fetchCriteria = event.getParameter("fetchCriteria")[sheetId];
	var that = this.getSpreadsheet().getMetaInfo(sheetId).meta;  //RGFRMWRK-605

	if (this.getItems() && this.getItems().length) {
		_.each(this.getItems(), function (outerLayout) {
			if (outerLayout.getItems() && outerLayout.getItems().length) {
				_.each(outerLayout.getItems(), function (filter) {
					if (filter instanceof sas.hc.m.Select) {
						var item = filter.getSelectedItem();
						if(item) {
							var meta = filter.data("meta");
							var value = item.getKey();
							if (value !== "___SPECIAL_ALLITEMS___") {
								if (that.columnCellOptions[meta.id].cellType === "number") {  //RGFRMWRK-605
									fetchCriteria.andSimpleFilter(new sas.risk.common.widgets.spreadsheet.filter.NumberFilter(meta.id, "eq", value, meta.label, item.getText()));
								} else {
									fetchCriteria.andSimpleFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter(meta.id, "eq", value, meta.label, item.getText()));
								}
							}
						}
					}
				});
			}
		});
	}
};