jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.ImportButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.AButtonAction");
jQuery.sap.require("sas.risk.rgf.util.DialogManager");
//dialog imports
jQuery.sap.require("sap.ui.commons.layout.HorizontalLayout");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.hc.m.MessageSummary");
jQuery.sap.require("sap.ui.model.json.JSONModel");


sas.risk.common.widgets.spreadsheet.toolbar.AButtonAction.extend("sas.risk.common.widgets.spreadsheet.toolbar.ImportButton", {
   constructor: function () {
      sas.risk.common.widgets.spreadsheet.toolbar.AButtonAction.apply(this, arguments);
      this.setIcon(sas.icons.HC.IMPORT);
   },
   renderer: "sas.hc.m.ButtonRenderer"
});
sas.risk.common.widgets.spreadsheet.toolbar.ImportButton.prototype.onPress = function () {

   var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
   var viewData = {
      title: rb.getText("spreadsheet.import.dialog.title.txt"),
      formattingKey: sas.risk.rgf.util.RGFUtil.buildDialogFormattingKey("ImportExcelDialog", this.getSpreadsheet().getParent().getParent().getCpbForm(), "SpreadSheet", this.getSpreadsheet().getParent().getParent().getCpbName()),
      spreadsheet: this.getSpreadsheet()
   };
   sas.risk.rgf.util.DialogManager.open("ImportExcelDialog", viewData, null, null);
};


/**
 * Import Excel Dialog
 */
sap.ui.jsview("sas.risk.rgf.view.dialog.ImportExcelDialog", {
   getControllerName: function () {
      return "sas.risk.rgf.view.dialog.ImportExcelDialog";
   },
   createContent: function (controller) {
      this.controller = controller;
      var that = this;
      var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
      this.fileSucessfullyUploaded = false;
      this.viewData = this.getViewData();
      $.proxy(this.createButtons(controller), this);

      this.uploaderMessage = new sas.hc.m.Label({
         text: rb.getText("spreadsheet.import.dialog.import.title.txt"),
         required: true
      }).addStyleClass('spreadsheetImportDialogUploaderTitle');;

      this.uploader = new sas.hc.ui.unified.FileUploader({
         name: "file",
         uploadUrl: "",
         buttonOnly: false,
         uploadOnChange: false,
         sendXHR: true,
         width: "100%",
         typeMissmatch: function (oEvent) {
            bValidSelection = false;
            // Get the name and type of the file
            var sName = oEvent.getParameter("fileName");
            var sType = oEvent.getParameter("fileType");
            var sMimeType = this.getMimeType();
            if (!sMimeType) {
               sMimeType = this.getFileType();
            }
            console.error("file mimetype missmatch");
         },
         filesSelected: function (oEvent) {
            that.okButton.setEnabled(true);
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
            that.okButton.setText(rb.getText("spreadsheet.import.dialog.import.txt"));
            that.fileAttachment = oEvent.getParameter('files')[0];
            that.oMessageSummary.setVisible(false);
         }
      }).addStyleClass('spreadsheetImportDialogUploader');
      this.oMessageSummary = new sas.hc.m.MessageSummary({
         showToolbar: false,
         visible: false
      });
      var messages = new sap.ui.model.json.JSONModel({
         messages: []
      });
      this.oMessageSummary.setModel(messages);

      this.innerLayout = new sas.risk.common.layout.flexlayout.FlexLayout({
         flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
         items: [
            this.oMessageSummary
         ]
      });
      this.layout = new sas.risk.common.layout.flexlayout.FlexLayout({
         flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
         items: [
            this.uploaderMessage,
            this.uploader,
            this.innerLayout
         ]
      });
      return this.layout;
   },
   createButtons: function (controller) {
      var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
      this.okButton = new sas.risk.common.widgets.Button({
         text: this.fileSucessfullyUploaded ? rb.getText("button.ok.action.label") : rb.getText("spreadsheet.import.dialog.import.txt"),
         press: $.proxy(controller.actionOk, this, controller),
         enabled: false
      });
      this.cancelButton = new sas.risk.common.widgets.Button({
         text: rb.getText("button.cancel.action.label"),
         press: $.proxy(controller.actionCancel, this, controller)
      });
   },
   getDialogSettings: function () {
      return {
         contentWidth: "60%",
         minWidth: 400,
         beginButton: this.okButton,
         endButton: this.cancelButton
      };
   }
});
sap.ui.controller("sas.risk.rgf.view.dialog.ImportExcelDialog", {
   actionOk: function (controller) {
      var dialog = controller.getView().getDialog();
      dialog.setBusyIndicatorDelay(0);
      dialog.setBusy(true);
      if (!this.fileSucessfullyUploaded) {

         //generate meta from spreadsheet
         var fileMeta = [];
         var that = this;
         _.each(this.viewData.spreadsheet.getSheetIds(), function (sheetId) {
            fileMeta.push(sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.objectMetaToFileMeta(that.viewData.spreadsheet.getMetaInfo(sheetId)));
         });

         //get original spreadsheet data from UI
         var originalData = this.viewData.spreadsheet.getValues();
         if (!Array.isArray(originalData)) {
            originalData = [originalData];
         }

         var importUrl = sas.risk.rgf.util.RGFUtil.getRestUrl() + "/utils/importSpreadsheet";
         if (!importUrl) {
            console.error("Import URL is not provided. Can't perform import to spreadsheet."); /*I18nOK:EMS*/
            return;
         }
         importUrl = sas.risk.rgf.util.URLParser.parseURL(importUrl).removeParam("start");
         importUrl = sas.risk.rgf.util.URLParser.parseURL(importUrl).removeParam("limit");

         //create new ajax form
         var formData = new FormData();
         var blob = new Blob([JSON.stringify(fileMeta)], {
            type: "application/json"
         });
         formData.append("meta", blob);
         if (this.fileAttachment) {
            formData.append("excel", this.uploader.FUEl.files[0]);
         }
         var dataBlob = new Blob([JSON.stringify(originalData)], {
            type: "application/json"
         });
         formData.append("originalData", dataBlob);
         $.ajax({
            type: "POST",
            url: importUrl,
            data: formData,
            processData: false,
            contentType: false,
            cache: false
         }).done(function (data, textStatus, jqXHR) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");
            that.okButton.setText(rb.getText("button.ok.action.label"));

            var spreadsheet = that.viewData.spreadsheet;
            _.each(spreadsheet.getSheetIds(), function (sheetId) {
               spreadsheet.getModel(sheetId).getService().clearDrafts();
               if (!(spreadsheet.getModel(sheetId).getService() instanceof sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService)) {
                  if(spreadsheet.getModel(sheetId).discardPatch) {spreadsheet.getModel(sheetId).discardPatch();}
               } else {
                  spreadsheet.getModel(sheetId).getService().importData=true;
                  sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.importRestServiceDataPatch(data,spreadsheet);
               }
            });
            
            spreadsheet.setValues(data.data);
            spreadsheet.fireChange({
               action: "change"
            });

            that.fileSucessfullyUploaded = true;
            that.controller.getView().getDialog().setBusy(false);
            that.controller.getView().getDialog().close();

            _.each(spreadsheet.getSheetIds(), function (sheetId) {
               if (spreadsheet.getModel(sheetId).getService() instanceof sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService) {
               	spreadsheet.getModel(sheetId).getService().importData=null;
               }
            });
            
         }).fail(function (e) {
            that.controller.getView().getDialog().setBusy(false);
            //remove old view of errors.
            that.controller.getView().getDialog().getContent()[0].innerLayout.removeItem(0);

            //recreate message summary
            that.oMessageSummary = new sas.hc.m.MessageSummary({
               showToolbar: false,
               visible: false
            });
            var tempMessages = [];
            _.each(e.responseJSON.messages, function (message) {
               var severity;
               if (message.severity === "warning" || message.severity === "warn" || message.severity === "Warning" || message.severity === "Warn" || message.severity === "WARN" || message.severity === "WARNING") {
                  severity = "Warning";
               } else if (message.severity === "Err" || message.severity === "err" || message.severity === "error" || message.severity === "Error" || message.severity === "ERR" || message.severity === "ERROR") {
                  severity = "Error";
               } else if (message.severity === "success" || message.severity === "SUCCESS" || message.severity === "Success") {
                  severity = "Success";
               } else {
                  severity = "Information";
               }

               tempMessages.push({
                  type: severity,
                  title: message.text
               });
            });
            var messages = new sap.ui.model.json.JSONModel({
               messages: tempMessages
            });
            that.oMessageSummary.setModel(messages);
            that.controller.getView().getDialog().getContent()[0].innerLayout.insertItem(that.oMessageSummary);
            that.oMessageSummary.setVisible(true);
            that.fileSucessfullyUploaded = false;
         });
      } else {
         dialog.setBusy(false);
         dialog.close();
      }
   },
   actionCancel: function (controller) {
      var dialog = controller.getView().getDialog();
      dialog.close();
   }
});