jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.Search");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.ui.core.Item");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.StringFilter");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.common.widgets.spreadsheet.toolbar.Search",{
	metadata:{
		properties:{
			spreadsheet:{type:"any"},
			toolbar1:{type:"any"},
			toolbar2:{type:"any"},
			footer:{type:"any"},
			menu:{type:"any"},
			spreadsheetToolbarType:{type:"any"}
		}
	},
	constructor:function(){
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
		this.caseSensitive=arguments[0].spreadsheetToolbarType.endsWith("|cs");
		this.addStyleClass("riskSpreadsheetSearch");
		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center);
		this.select=new sas.hc.m.Select().setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexShrink:0}));
		this.select.getEnabled=function(){return true;};
	},
	renderer:sas.risk.common.layout.flexlayout.FlexLayoutRenderer
});
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.setSpreadsheet=function(spreadsheet){
	this.setProperty("spreadsheet",spreadsheet);
	spreadsheet.attachPreFetchData($.proxy(this.onPreFetchData,this));
	spreadsheet.attachPostFetchData($.proxy(this.onPostFetchData,this));
	spreadsheet.attachPostMetaChange($.proxy(this.onPostMetaChange,this));
	spreadsheet.attachSelect($.proxy(this.onSelect,this));
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.onPostMetaChange=function(event){
	var spreadsheet=this.getSpreadsheet();
	var sheetId=spreadsheet.getSelectedSheetId();
	var meta=spreadsheet.getMetaInfo(sheetId).meta;
	this.search=new sas.hc.m.SearchField({
		search:function(){
			
			spreadsheet.getSelectedSheetTable().setFirstVisibleRow(0);
			spreadsheet.getSelectedSheetTable().setBusyIndicatorDelay(0);
			spreadsheet.getSelectedSheetTable().setBusy(true);
			spreadsheet.fetchData(spreadsheet.getSelectedSheetId());
		}
	});
	this.search.getEnabled=function(val){return true;};//SearchField should be always enabled irrespective of whether table is readonly or not
	this._refreshSelect(meta);
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype._getColumnLabel=function(columnOptions){
	if(_.isArray(columnOptions.multiLabels)&&columnOptions.multiLabels.length){
		return columnOptions.multiLabels[columnOptions.multiLabels.length-1].getText();
	}
	return columnOptions.label;
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype._refreshSelect=function(meta){
	this.searchColumns=_.filter(meta.getColumnOptions(),function(columnOption){
		return !!columnOption.quickSearch;
	});
	this.removeAllItems();
	switch(this.searchColumns.length){
		case 0:
			this.setVisible(false);
		break;
		case 1:
			this.setVisible(true);
			this.addItem(this.search);
			this.addItem(new sas.hc.m.Text({text:this._getColumnLabel(this.searchColumns[0])}).addStyleClass("riskSpreadsheetSearchLabel"));
		break;
		default:
			this.setVisible(true);
			this.addItem(this.search);
			this.select.removeAllItems();
			var that=this;
			_.each(this.searchColumns,function(searchColumn){
				var colLabel=that._getColumnLabel(searchColumn);
				that.select.addItem(new sas.hc.ui.core.Item({
					key:searchColumn.id,
					tooltip:colLabel,
					text:colLabel
				}));
			});
			this.addItem(this.select);
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.onPostFetchData=function(event){
	var sheetId=this.getSpreadsheet().getSelectedSheetId();
	var fetchCriteriaMap=event.getParameter("fetchCriteria");
	this._refresh(fetchCriteriaMap[sheetId]);
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.onSelect=function(event){
	if(this.search){
		var spreadsheet=this.getSpreadsheet();
		var table=spreadsheet.getSelectedSheetTable();
		var sheetId=table.data("sheetId");
		var fetchCriteriaMap=spreadsheet.getFetchCriteria();
		var meta=spreadsheet.getMetaInfo(sheetId).meta;
		this._refreshSelect(meta);
		this._refresh(fetchCriteriaMap[sheetId]);
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype._refresh=function(fetchCriteria){
	var filter=fetchCriteria.getFilter();
	var columnIds=_.pluck(this.searchColumns,"id");
	var matchingFilters=this.findMatchingFilters(filter,columnIds);
	if(matchingFilters.length){
		var value=this.extractValueFromFilters(matchingFilters);
		this.search.setValue(value);
		if(this.searchColumns.length>1){
			this.select.setSelectedKey(matchingFilters[0].getName());
		}
	}else{
		if(this.search.getValue()){
			this.search.setValue("");
		}
		if(this.searchColumns.length>1 && (this.select && !this.select.getSelectedKey())){
			this.select.setSelectedKey();
		}
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.findMatchingFilters=function(filter,columnIds){
	var filters=[];
	if(filter){
		var that=this;
		_.find(columnIds,function(columnId){
			if(that.isMatchingFilter(filter,columnId)){
				filters.push(filter);
			}else if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.AndFilter){
				_.each(filter.getChildren(),function(child){
					if(that.isMatchingFilter(child,columnId)){
						filters.push(child);
					}
				});
			}
			return !!filters.length;
		});
	}
	return filters;
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.isMatchingFilter=function(filter,columnId){
	if(filter instanceof sas.risk.common.widgets.spreadsheet.filter.StringFilter && filter.getOperator()==="contains"){
		return columnId===filter.getName();
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.extractValueFromFilters=function(filters){
	var result="";
	_.each(filters,function(filter){
		result+=" "+filter.getValue();
	});
	return result.trim();
};
sas.risk.common.widgets.spreadsheet.toolbar.Search.prototype.onPreFetchData=function(event){
	var sheetId=this.getSpreadsheet().getSelectedSheetId();
	var fetchCriteria=event.getParameter("fetchCriteria")[sheetId];
	var searchValue=this.search.getValue();
	if(searchValue){
		var columnId,columnLabel;
		switch(this.searchColumns.length){
			case 0:
			break;
			case 1:
				columnId=this.searchColumns[0].id;
				columnLabel=this._getColumnLabel(this.searchColumns[0]);
			break;
			default:
				columnId=this.select.getSelectedKey();
				columnLabel=this.select.getSelectedItem().getText();
		}
		if(columnId&&columnLabel){
			var chunks=searchValue.trim().split(" ");
			if(chunks.length===1){
				var simpleFilter=new sas.risk.common.widgets.spreadsheet.filter.StringFilter(columnId,"contains",searchValue,columnLabel,searchValue);
				simpleFilter.setCaseSensitive(this.caseSensitive);
				fetchCriteria.andSimpleFilter(simpleFilter);
			}else{
				var andFilter=new sas.risk.common.widgets.spreadsheet.filter.AndFilter();
				_.each(chunks,function(chunk){
					var simpleFilter=new sas.risk.common.widgets.spreadsheet.filter.StringFilter(columnId,"contains",chunk,columnLabel,searchValue);
					simpleFilter.setCaseSensitive(this.caseSensitive);
					andFilter.addFilter(simpleFilter);
				});
				fetchCriteria.andSimpleFilter(andFilter);
			}
		}
	}
};
