jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.Sort");

sas.risk.common.widgets.spreadsheet.toolbar.Sort=function(spreadsheet){
	this.spreadsheet=spreadsheet;

	//attach sort click to spreadsheet
	this.handleSortAttach();

	this.spreadsheet.attachPostMetaChange($.proxy(this.handleSortPostMetaChange, this));
	this.spreadsheet.attachPreFetchData($.proxy(this.handleSortPreFetchData,this));
	this.spreadsheet.attachPostFetchData($.proxy(this.handleSortPostFetchData,this));
	this.spreadsheet.attachSelect($.proxy(this.handleSortSelect,this));
};

sas.risk.common.widgets.spreadsheet.toolbar.Sort.prototype={
		handleSort:function(event){
			var table=event.getSource();
			var sheetId=table.data("sheetId");
			var column=event.getParameter("column");
			var columnId=column.getName();
			var sortOrder=event.getParameter("sortOrder");
			var meta=this.spreadsheet.getMetaInfo(sheetId).meta;
			var sortedColumn=_.find(meta.getColumnOptions(),function(columnOption){
				return !!columnOption.sortProperty && columnOption.sorted;
			});
			if(sortedColumn){
				if(columnId===sortedColumn.id && sortOrder===sortedColumn.sortOrder && sortedColumn.sorted){
					return;//no differences
				}
				delete sortedColumn.sortOrder;
				delete sortedColumn.sorted;
			}
			if(sortOrder){
				var newSortedColumn=meta.getColumnOptions(columnId);
				newSortedColumn.sorted=true;
				newSortedColumn.sortOrder=sortOrder;
			}
			event.preventDefault();
			
			this.spreadsheet.getSelectedSheetTable().setFirstVisibleRow(0);
			this.spreadsheet.getSelectedSheetTable().setBusyIndicatorDelay(0);
			this.spreadsheet.getSelectedSheetTable().setBusy(true);
			this.spreadsheet.fetchData(sheetId);
		},
		handleSortPreFetchData:function(event){
			var sheetId=this.spreadsheet.getSelectedSheetId();
			var meta=this.spreadsheet.getMetaInfo(sheetId).meta;
			var sortedColumn=_.find(meta.getColumnOptions(),function(columnOption){
				return !!columnOption.sortProperty && columnOption.sorted;
			});
			if(sortedColumn){
				var fetchCriteriaMap=event.getParameter("fetchCriteria");
				var fetchCriteria=fetchCriteriaMap[sheetId];
				fetchCriteria.addSort(sortedColumn.id,sortedColumn.sortOrder||"Ascending");
			}
		},
		handleSortPostFetchData:function(event){
			var table=this.spreadsheet.getSelectedSheetTable();
			var fetchCriteriaMap=event.getParameter("fetchCriteria");
			this._handleSortPostFetchData(table,fetchCriteriaMap);
		},
		handleSortDetech: function() {
			//detach sort handles from each of the tables/tabs
			var that = this;
			_.each(that.spreadsheet.getSheetTables(),function(table){
				table.detachSort($.proxy(that.handleSort,that));
			});
		},
		handleSortAttach: function() {
			//attach sort handles to each of the tables/tabs
			var that = this;
			_.each(that.spreadsheet.getSheetTables(),function(table){
				table.attachSort($.proxy(that.handleSort,that));
				that._handleSortPostFetchData(table,that.spreadsheet.getFetchCriteria());
			});
		},
		handleSortPostMetaChange: function(event) {
			//detach all sorts and reapply them.
			this.handleSortDetech();
			this.handleSortAttach();
		},
		handleSortSelect:function(event){
			var table=this.spreadsheet.getSelectedSheetTable();
			var fetchCriteriaMap=this.spreadsheet.getFetchCriteria();
			this._handleSortPostFetchData(table,fetchCriteriaMap);
		},
		_handleSortPostFetchData:function(table,fetchCriteriaMap){
			var sheetId=table.data("sheetId");
			//Fix for RGFRMWRK-267
			var fc = this.spreadsheet.getFetchCriteria(table.data("sheetId")); //gets the Fetch Criteria
            var m1=this.spreadsheet.getMetaInfo(sheetId).meta;
            var sc1=_.find(m1.getColumnOptions(),function(columnOption){return !!columnOption.sorted;});
            if (sc1 !==undefined) {
                fc.abc.sorts=[{columnId: sc1.id, sortOrder: sc1.sortOrder}];
            }
			// End of fix for RGFRMWRK-267
			var prevColumnWidget=_.find(table.getColumns(),function(column){return column.getSorted();});
			if(prevColumnWidget){
				prevColumnWidget.setProperty("sorted", false, true);
				prevColumnWidget.setProperty("sortOrder", null, true);
				delete prevColumnWidget._oSorter;
				var binding=table.getBinding("rows");
				if(binding){binding.aSorters=[];}
				prevColumnWidget._updateIcons();
			}
			var fetchCriteria=fetchCriteriaMap[sheetId];
			var sorts=fetchCriteria?fetchCriteria.getSorts():[];
			if(sorts.length){
				var columnId=sorts[0].columnId;
				var meta=this.spreadsheet.getMetaInfo(sheetId).meta;
				var newSortedColumn=_.find(meta.getColumnOptions(),function(columnOption){
					return !!columnOption.sortProperty && columnOption.id===columnId;
				});
				var sortOrder=sorts[0].sortOrder;
				if(newSortedColumn){
					newSortedColumn.sorted=true;
					newSortedColumn.sortOrder=sortOrder;
					var columnWidget=_.find(table.getColumns(),function(column){return column.getName()===columnId;});
					columnWidget.setProperty("sorted", true, true);
		         columnWidget.setProperty("sortOrder", sortOrder, true);
		         columnWidget._updateIcons();
				}
			}
		}
};