jQuery.sap.declare("sas.risk.common.widgets.spreadsheet.toolbar.Views");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.risk.common.layout.gridlayout.GridLayout");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.risk.common.layout.gridlayout.GridLayoutData");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.common.widgets.spreadsheet.toolbar.Views",{
	metadata:{
		properties:{
			spreadsheet:{type:"any"}
		}
	},
	constructor:function(){
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
		this.views={};
		this.addStyleClass("riskSpreadsheetViews");
		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center);
		var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
		this.saveViewMenuItem=new sas.hc.m.MenuItem({
			press:$.proxy(this.saveView,this),
			text:rb.getText("table.toolbar.menu.saveView.txt"),
			enabled:false
		});
		this.deleteViewMenuItem=new sas.hc.m.MenuItem({
			press:$.proxy(this.deleteView,this),
			text:rb.getText("table.toolbar.menu.deleteView.txt"),
			enabled:false
		});
		this.setAsDefaultViewMenuItem=new sas.hc.m.MenuItem({
			press:$.proxy(this.setAsDefaultView,this),
			text:rb.getText("table.toolbar.menu.setAsDefaultView.txt"),
			enabled:false
		});
		var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
		var that=this;
		var dropdownLabel = new sap.m.Label({
	   	text:rb.getText("table.toolbar.extra.view.txt")
	   }).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
	   	flexShrink:0
	   }));
		this.dropdown=new sas.hc.m.Select({
			change:function(event){
				var view=event.getParameter("selectedItem").getModel("meta").getData();
				var spreadsheet=that.getSpreadsheet();
				var sheetId=spreadsheet.getSelectedSheetId();
				var fetchCriteria=spreadsheet.getFetchCriteria();
				fetchCriteria[sheetId]=view.clone();
				that.getSpreadsheet().setFetchCriteria(fetchCriteria);
			}
		});
		dropdownLabel.setLabelFor(this.dropdown);
		this.noneView=new sas.risk.common.widgets.spreadsheet.View("___SPECIAL_NONE___",rb.getText("table.view.non.txt"),"",false);
		noneItem=new sas.hc.ui.core.Item({
			key:this.noneView.getViewKey(),
			text:this.noneView.getViewName()
		});
		noneItem.setModel(new sap.ui.model.json.JSONModel(this.noneView),"meta");
		this.customItem=new sas.hc.ui.core.Item({
			key:"___SPECIAL_CUSTOM___",
			text:rb.getText("table.view.custom.txt")
		});
		this.dropdown.addItem(noneItem);
		this.addItem(new sas.hc.m.Label({text:rb.getText("table.toolbar.extra.view.txt"),labelFor:this.dropdown}));
		this.addItem(this.dropdown);
	},
	renderer:sas.risk.common.layout.flexlayout.FlexLayoutRenderer
});
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype._getViews=function(sheetId){
	return this.views[sheetId];
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype._setViews=function(sheetId, views){
	this.views[sheetId] = views;
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.setSpreadsheet=function(spreadsheet){
	this.setProperty("spreadsheet",spreadsheet);
	spreadsheet.attachPreMetaChange($.proxy(this.onPreMetaChange,this));
	spreadsheet.attachPostFetchData($.proxy(this.onPostFetchData,this));
	spreadsheet.attachPostMetaChange($.proxy(this.onPostMetaChange,this));
	spreadsheet.attachSelect($.proxy(this.onSelect,this));
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.onPreMetaChange=function(event){
	var that=this;
	var defs=event.getParameter("defs");
	var def=$.Deferred();
	defs.unshift(def);
	defs.pop().done(function(){
		var innerDefs=[];
		var spreadsheet=that.getSpreadsheet();
		_.each(spreadsheet.getMeta(),function(metaInfo){
			var sheetId=metaInfo.id;
			var meta=spreadsheet.getMetaInfo(sheetId).meta;
			var viewsKey=spreadsheet.getViewsKey()[sheetId];
			var innerDef=$.Deferred();
			innerDefs.push(innerDef);
			if(!viewsKey){
				that.views[sheetId]=[];
				return innerDef.resolve(false);
			}
			spreadsheet.getViewsAPI().getViews(viewsKey).done(function(views){
				that.views[sheetId]=views;
				that.refreshDropdown(sheetId);
				var match=_.find(views,function(view){
					return view.getViewIsDefault();
				});
				if(match){
					meta.overrideVisibleColumnIds(match.getColumnIds());
					var fetchCriteriaMap=spreadsheet.getProperty("fetchCriteria")||{};
					fetchCriteriaMap[sheetId]=match.clone();
					spreadsheet.setProperty("fetchCriteria",fetchCriteriaMap,true);
				}
				innerDef.resolve(!!match);
			});
		});
		$.when.apply($,innerDefs).done(function(){
			def.resolve();
		});
	});
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.onSelect=function(event){
	if(this.dropdown){
		var spreadsheet=this.getSpreadsheet();
		var sheetId=spreadsheet.getSelectedSheetId();
		this.refreshDropdown(sheetId);
		this.refreshSelection(sheetId,spreadsheet.getFetchCriteria());
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.onPostMetaChange=function(event){
	var spreadsheet=this.getSpreadsheet();
	var sheetId=spreadsheet.getSelectedSheetId();
	var meta=spreadsheet.getMetaInfo(sheetId).meta;
	var sortedColumn=_.find(meta.getColumnOptions(),function(columnOption){
		return !!columnOption.sorted;
	});
	if(sortedColumn){
		this.noneView.addSort(sortedColumn.id,sortedColumn.sortOrder);
	}
	var visibleColumnIds=_.filter(meta.getColumnIds(),function(columnId){
		var columnOption=meta.getColumnOptions(columnId);
		return !!columnOption.visible;
	});
	this.noneView.setColumnIds(visibleColumnIds);
	this.refreshDropdown(sheetId);
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.onPostFetchData=function(event){
	var sheetId=this.getSpreadsheet().getSelectedSheetId();
	var fetchCriteriaMap=event.getParameter("fetchCriteria");
	this.refreshSelection(sheetId,fetchCriteriaMap);
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.refreshSelection=function(sheetId,fetchCriteriaMap){
	var fetchCriteria=fetchCriteriaMap[sheetId];
	var match=_.find(this._getViews(sheetId),function(view){
		return fetchCriteria.equals(view);
	});
	if(match){
		this.dropdown.removeItem(this.customItem);
		this.dropdown.setSelectedKey(match.getViewKey());
		this.setMenuItemsEnabled(false,true,!match.getViewIsDefault());
	}else{
		var isNone=fetchCriteria.equals(this.noneView);
		if(isNone){
			this.dropdown.removeItem(this.customItem);
			this.dropdown.setSelectedKey("___SPECIAL_NONE___");
			this.setMenuItemsEnabled(false,false,false);
		}else{
			this.dropdown.addItem(this.customItem);
			this.dropdown.setSelectedKey("___SPECIAL_CUSTOM___");
			this.setMenuItemsEnabled(true,false,false);
		}
	}
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.refreshDropdown=function(sheetId){
	var views=this._getViews(sheetId);
	var noneItem=this.dropdown.getFirstItem();
	this.dropdown.removeAllItems();
	this.dropdown.addItem(noneItem);
	var that=this;
	_.each(views,function(view){
		that.dropdown.addItem(that._createViewItem(view));
	});
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype._createViewItem=function(view){
	var item=new sas.hc.ui.core.Item({
		key:view.getViewKey(),
		text:{
			parts:["meta>/name","meta>/isDefault"],
			formatter:function(name,isDefault){
				return name+(isDefault?" *":""); 
			}
		}
	});
	item.setModel(new sap.ui.model.json.JSONModel(view),"meta");
	return item;
};
//MENU ITEMS
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.refreshMenuItems=function(){
	_.each(this.dropdown.getItems(),function(item){
		var model=item.getModel("meta");
		if(model){
			model.checkUpdate(true);
		}
	});
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.setMenuItemsEnabled=function(saveView,deleteView,setAsDefaultView){
	this.saveViewMenuItem.setEnabled(saveView);
	this.deleteViewMenuItem.setEnabled(deleteView);
	this.setAsDefaultViewMenuItem.setEnabled(setAsDefaultView);
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.getMenuItems=function(){
	return [this.saveViewMenuItem,this.deleteViewMenuItem,this.setAsDefaultViewMenuItem];
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.saveView=function(){
	var layout=new sas.risk.common.layout.gridlayout.GridLayout({
		gridRowGap:"20px",
		gridTemplateColumns:"auto auto"
	}).addStyleClass("riskSpreadsheetViewsSave");
	var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	var dialog=new sas.hc.m.Dialog({
		title:rb.getText("table.saveView.title.txt"),
		content:[layout]
	});
	dialog.nameInput=new sas.hc.m.Input({
		type:sap.m.InputType.Text,
		required:true,
		liveChange:function(event){
			var liveName=event.getParameter("value");
			okButton.setEnabled(!!liveName.length);
		}
	});
	layout.addItem(new sas.hc.m.Label({
		text:rb.getText("table.saveView.viewName.label.txt"),
		required:true,
		labelFor:dialog.nameInput
	}));
	layout.addItem(dialog.nameInput);
	dialog.descriptionInput=new sas.hc.m.TextArea({});
	layout.addItem(new sas.hc.m.Label({
		text:rb.getText("table.saveView.viewDescription.label.txt"),
		labelFor:dialog.descriptionInput
	}));
	layout.addItem(dialog.descriptionInput);
	dialog.defaultCheckBox=new sas.hc.m.CheckBox({
		text:rb.getText("table.toolbar.menu.setAsDefaultView.txt")
	}).setLayoutData(new sas.risk.common.layout.gridlayout.GridLayoutData({
		gridColumnStart:1,
		gridColumnEnd:3
	}));
	layout.addItem(dialog.defaultCheckBox);
	var okButton=new sas.hc.m.Button({
		text:rb.getText("table.saveView.ok.txt"),
		press:$.proxy(this.onSaveOk,this,dialog),
		enabled:false
	});
	dialog.setLeftButton(okButton);
	dialog.setDefaultButton(okButton);
	var cancelButton=new sas.hc.m.Button({
		text:rb.getText("table.saveView.cancel.txt"),
		press:function(){dialog.close();}
	});
	dialog.setRightButton(cancelButton);
	dialog.open();
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.onSaveOk=function(dialog,event){
	var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	var spreadsheet=this.getSpreadsheet();
	var sheetId=spreadsheet.getSelectedSheetId();
	var fetchCriteria=spreadsheet.getFetchCriteria()[sheetId];
	var viewsAPI=spreadsheet.getViewsAPI();
	var newViewName=dialog.nameInput.getValue();
	var newViewDescription=dialog.descriptionInput.getValue();
	var newViewIsDefault=dialog.defaultCheckBox.getSelected();
	var key=newViewName;
	var newView=new sas.risk.common.widgets.spreadsheet.View(key,newViewName,newViewDescription,newViewIsDefault);
	newView.mergeFetchCriteria(fetchCriteria);
	var viewsKey=spreadsheet.getViewsKey()[sheetId];
	var that=this;
	viewsAPI.getViews(viewsKey).done(function(views){
		var match=_.find(views,function(view){
			return view.name===newViewName;
		});
		if(match){
			jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");
	    	sas.risk.common.widgets.MessageDetailsBox.error({
	    		message:rb.getText("table.saveView.duplicateName.msg.txt")
 			}, sas.config.appName, sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle());
		}else{
			viewsAPI.saveView(viewsKey,newView).done(function(views){
				if(!views) {
					that._getViews(sheetId).push(newView);
					that.dropdown.removeItem(that.customItem);
					that.dropdown.addItem(that._createViewItem(newView));
				} else {
					that._setViews(sheetId, views);
					that.refreshDropdown(sheetId);
				}
				that.dropdown.setSelectedKey(newView.getViewKey());
				that.refreshMenuItems();
				that.setMenuItemsEnabled(false,true,!newView.getViewIsDefault());
				dialog.close();
			});
		}
	});
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.deleteView=function(){
	var layout=new sas.risk.common.layout.gridlayout.GridLayout({gridTemplateColumns:"auto"});
	var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	var dialog=new sas.hc.m.Dialog({
		title:rb.getText("table.deleteView.title.txt"),
		content:[layout]
	});
	dialog.deleteButton=new sas.hc.m.Button({
		icon:sas.icons.HC.DELETE,
		press:$.proxy(this.onDeleteView,this,dialog),
		enabled:false
	});
	layout.addItem(new sas.hc.m.ResponsiveToolbar({content : [new sas.hc.m.ToolbarSpacer(),dialog.deleteButton]}));
	dialog.table=new sas.hc.ui.table.Table({
		selectionMode:sas.hc.ui.table.SelectionMode.Single,
		showTableOptions: false,
		columns:[
			new sas.hc.ui.table.Column({
				label:rb.getText("table.deleteView.columns.name.txt"),
				template:new sas.hc.m.Text().bindProperty("text","name"),
				sortProperty:"name"
			}),
			new sas.hc.ui.table.Column({
				label:rb.getText("table.deleteView.columns.description.txt"),
				template:new sas.hc.m.Text().bindProperty("text","description"),
				sortProperty:"description"
			})			
		],
		rowSelectionChange:function(event){
			dialog.deleteButton.setEnabled(this.getSelectedIndex()!==-1);
		},
		rows:"{/rows}"
	});
	var spreadsheet=this.getSpreadsheet();
	var sheetId=spreadsheet.getSelectedSheetId();
	dialog.table.setModel(new sap.ui.model.json.JSONModel({rows:this._getViews(sheetId)}));
	layout.addItem(dialog.table);
	var cancelButton=new sas.hc.m.Button({
		text:rb.getText("table.deleteView.close.txt"),
		press:function(){dialog.close();}
	});
	dialog.setRightButton(cancelButton);
	dialog.open();
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.onDeleteView=function(dialog,event){
	dialog.deleteButton.setEnabled(false);
	var spreadsheet=this.getSpreadsheet();
	var sheetId=spreadsheet.getSelectedSheetId();
	var viewsAPI=spreadsheet.getViewsAPI();
	var viewsKey=spreadsheet.getViewsKey()[sheetId];
	var that=this;
	var selectedView=dialog.table.getRows()[dialog.table.getSelectedIndex()].getBindingContext().getObject();
	viewsAPI.deleteView(viewsKey,selectedView).done(function(){
		var views=that._getViews(sheetId);
		var index=views.indexOf(selectedView);
		views.splice(index,1);
		dialog.table.getModel().checkUpdate(true);
		that.dropdown.removeItem(that.dropdown.getSelectedItem());
		that.dropdown.addItem(that.customItem);
		that.dropdown.setSelectedKey("___SPECIAL_CUSTOM___");
		that.setMenuItemsEnabled(true,false,false);
	});
};
sas.risk.common.widgets.spreadsheet.toolbar.Views.prototype.setAsDefaultView=function(){
	var spreadsheet=this.getSpreadsheet();
	var sheetId=spreadsheet.getSelectedSheetId();
	var viewsAPI=spreadsheet.getViewsAPI();
	var viewsKey=spreadsheet.getViewsKey()[sheetId];
	var that=this;
	var defaultView=this.dropdown.getSelectedItem().getModel("meta").getData();
	var defaultViewName=defaultView.getViewName();
	viewsAPI.setAsDefaultView(viewsKey,defaultView).done(function(){
		_.each(that._getViews(sheetId),function(view){
			view.setViewIsDefault(view.getViewName()===defaultViewName);
		});
		that.refreshMenuItems();
		that.setMenuItemsEnabled(false,true,false);
	});
};