jQuery.sap.declare("sas.risk.rgf.api.CommonAPI");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.ui.mashup.util.PerspectiveUtil");

sas.risk.rgf.api.CommonAPI = {
	_userDetails:null,
	_taskTypes:null,
	_dataLoaders:null,
	_defaultMultiEval:true, // Forcing DefaultMultEval to be TRUE always RGFRMWRK-516
	_GZIPCompressionThreshold:50000, // Threshold value to turn on GZIP compression for rest calls RGFRMWRK-552
	                              // values -1: off 
	                              //         0: always send compressed 
	                              //         X: if payload is > X send it compressed
	_multiEvalPayloadDebugging:false,  //To turn on debugging info in devtools network tab to inspect multieval payload
	_timezoneMidday12hourEnabled:true,
	_iTZDiffEnabled:true,
	_altTimeoutActivityInterval:-1,
	_notificationPollingInterval:0, // in milliseconds, the default is 0 (without setting the notification polling interval and just using html commons default 60 seconds)  
	_lboScrollPreventDefault:false,

	loadUserDetails:function(){
		var def=$.Deferred();
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/delegationInfo"+sas.ui.mashup.util.PerspectiveUtil.encodedParam()
		}).done(function(json){
			jQuery.sap.require("sas.risk.rgf.biz.UserDetails");
			sas.risk.rgf.api.CommonAPI._userDetails=sas.risk.rgf.biz.UserDetails.createFromJson(json);
			def.resolve(sas.risk.rgf.api.CommonAPI._userDetails);
		});

		return def;
	},
	loadCPB3Resources:function(perspectiveId){
		var def=$.Deferred();
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/cpb3Resources"+sas.ui.mashup.util.PerspectiveUtil.encodedParam()
		}).done(function(json){
			sas.risk.rgf.util.RGFUtil._rb._bundleJSON = json;
			sap.ui.getCore().getLibraryResourceBundle("sas.risk.common");/*cache resource bundle for risk commons*/
			def.resolve(sas.risk.rgf.util.RGFUtil._rb._bundleJSON);
		});
		return def;
	},
	getUserDetails:function(){
		return sas.risk.rgf.api.CommonAPI._userDetails;
	},
	impersonate:function(impersonationId){
		var def=$.Deferred();
		sas.ui.mashup.App.getInstance().setBusy(true);
		$.ajax({
			type:"PUT",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/impersonate/"+impersonationId,
			contentType: "application/json; charset=UTF-8"
		}).done(function(){
			def.resolve();
			sas.ui.mashup.App.getInstance().setBusy(false);
		}).fail(function(e){
			jQuery.sap.require("sas.ui.cpb3.CpbUtil");
			sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
			sas.ui.mashup.App.getInstance().setBusy(false);
		});
		return def;
	},
	searchCollection:function(collectionName,filter){
		var def=$.Deferred();
		var url=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/"+collectionName;
		if(filter){
			url+="?filter="+filter.toString();
		}
		$.getJSON(url,function(json){
			def.resolve(json.items);
		})
		return def;
	},
//	loadCollection:function(collectionName,keys){
//		var def=$.Deferred();
//		var url=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/"+collectionName;
//		var filter="in(key,"+keys.join(",")+")&limit="+keys.length;
//		$.getJSON(url+"?filter="+filter,function(json){
//			def.resolve(json.items);
//		});
//		return def;
//	},
   loadCollection:function(collectionName,keys){
      var url=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/"+collectionName;
      var chunks = [];
      var chunk = [];
      for (var i = 0; i < keys.length; i++)
      {
             if (i > 0 && i % 200 === 0)
             {
                   chunks.push(chunk);
                   chunk = [];
             }
             chunk.push(keys[i]);
      }
      if (chunk.length > 0)
      {
             chunks.push(chunk);
      }

      var defs = [];
      for (var i = 0; i < chunks.length; i++)
      {
             var filter="in(key,"+chunks[i].join(",")+")&limit="+chunks[i].length;
             defs.push($.getJSON(url+"?filter="+filter));
      }
      
      var def=$.Deferred();
      $.when.apply($,defs).done(function(){
           var resps=Array.prototype.slice.call(arguments,0);
           var items = [];
           if(defs.length===1){
                  resps=[resps];//the format for a single getJSON comes back differently than multiples. wrapping it so that the structures match
           }
           _.each(resps,function(resp){
                  items=items.concat(resp[0].items);
           });
           def.resolve(items);
      });
      return def;
},

	getTableMeta:function(collectionName){
		var def=$.Deferred();
		var url=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/fieldMeta/"+collectionName;
		$.getJSON(url,function(json){
			def.resolve(json);
		});
		return def;
	},
	loadCustomizationInfo:function(URL){
		var def=$.Deferred();
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:URL
		}).done(function(json){
			def.resolve(json);
		}).fail(function(e){
			sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
		});
		return def;
	},
	loadTaskTypes:function(){
		var def=$.Deferred();
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/taskTypes"
		}).done(function(json){
			jQuery.sap.require("sas.risk.rgf.biz.TaskType");
			sas.risk.rgf.api.CommonAPI._taskTypes=[];
			_.each(json.items,function(item){
				sas.risk.rgf.api.CommonAPI._taskTypes.push(sas.risk.rgf.biz.TaskType.createFromJson(item));
			});
			def.resolve(sas.risk.rgf.api.CommonAPI._taskTypes);
		});
		return def;		
	},
	getTaskTypes:function(){
		return sas.risk.rgf.api.CommonAPI._taskTypes;
	},
	loadDataLoaders:function(){
		var def=$.Deferred();
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/dataLoaders"
		}).done(function(json){
			jQuery.sap.require("sas.risk.rgf.biz.DataLoader");
			sas.risk.rgf.api.CommonAPI._dataLoaders=[];
			_.each(json.items,function(item){
				sas.risk.rgf.api.CommonAPI._dataLoaders.push(sas.risk.rgf.biz.DataLoader.createFromJson(item));
			});
			def.resolve(sas.risk.rgf.api.CommonAPI._dataLoaders);
		});
		return def;		
	},
	getDataLoaders:function(){
		return sas.risk.rgf.api.CommonAPI._dataLoaders;
	},
	loadDefaultMultiEval:function(){

		return true;  // HARDCODING to return true always.... RGFRMWRK-516
		   
		var def=$.Deferred();
      	$.ajax({
			headers: {Accept:"application/json; charset=UTF-8", "Content-Type":"application/json; charset=UTF-8"},
			async: false,
			type: "GET",
			dataType: "json",
			url: sas.risk.rgf.util.RGFUtil.getRestUrl() + "/utils/defaultMultiEvalFlag"
	   	}).done(function(json){
			sas.risk.rgf.api.CommonAPI._defaultMultiEval=json.defaultMultiEvalFlag;
			def.resolve(sas.risk.rgf.api.CommonAPI._defaultMultiEval);
		}).fail(function(e){
     	   jQuery.sap.require("sas.ui.cpb3.CpbUtil");
         	sas.risk.rgf.util.RGFUtil.handleFail(e);
        	def.reject();
        	sas.risk.rgf.api.CommonAPI._defaultMultiEval=false;
      	});
		return def;
	},
	getDefaultMultiEval:function(){
		return sas.risk.rgf.api.CommonAPI._defaultMultiEval;
	},
	loadGZIPCompressionThreshold:function(){	   
		var def=$.Deferred();
      	$.ajax({
			headers: {Accept:"application/json; charset=UTF-8", "Content-Type":"application/json; charset=UTF-8"},
			async: false,
			type: "GET",
			dataType: "json",
			url: sas.risk.rgf.util.RGFUtil.getRestUrl() + "/utils/GZIPCompressionThreshold"
	   	}).done(function(json){
			sas.risk.rgf.api.CommonAPI._GZIPCompressionThreshold=json.GZIPCompressionThreshold;
			def.resolve(sas.risk.rgf.api.CommonAPI._GZIPCompressionThreshold);
		}).fail(function(e){
     	   jQuery.sap.require("sas.ui.cpb3.CpbUtil");
         	sas.risk.rgf.util.RGFUtil.handleFail(e);
        	def.reject();
      	});
		return def;
	},
	getGZIPCompressionThreshold:function(){
		return sas.risk.rgf.api.CommonAPI._GZIPCompressionThreshold;
	},
	loadMultiEvalPayloadDebugging:function(){	   
		var def=$.Deferred();
      	$.ajax({
			headers: {Accept:"application/json; charset=UTF-8", "Content-Type":"application/json; charset=UTF-8"},
			async: false,
			type: "GET",
			dataType: "json",
			url: sas.risk.rgf.util.RGFUtil.getRestUrl() + "/utils/MultiEvalPayloadDebugging"
	   	}).done(function(json){
			sas.risk.rgf.api.CommonAPI._multiEvalPayloadDebugging=json.MultiEvalPayloadDebugging;
			def.resolve(sas.risk.rgf.api.CommonAPI._multiEvalPayloadDebugging);
		}).fail(function(e){
     	   jQuery.sap.require("sas.ui.cpb3.CpbUtil");
         	sas.risk.rgf.util.RGFUtil.handleFail(e);
        	def.reject();
        	sas.risk.rgf.api.CommonAPI._multiEvalPayloadDebugging=false;
      	});
		return def;
	},
	getMultiEvalPayloadDebugging:function(){
		return sas.risk.rgf.api.CommonAPI._multiEvalPayloadDebugging;
	},
	loadConfigdataOptions:function(){	   
		var def=$.Deferred();
      	$.ajax({
			headers: {Accept:"application/json; charset=UTF-8", "Content-Type":"application/json; charset=UTF-8"},
			async: false,
			type: "GET",
			dataType: "json",
			url: sas.risk.rgf.util.RGFUtil.getRestUrl() + "/utils/configuration"
	   	}).done(function(json){
				sas.risk.rgf.api.CommonAPI._timezoneMidday12hourEnabled = json.timezoneMidday12hourEnabled;
				sas.risk.rgf.api.CommonAPI._iTZDiffEnabled = json.iTZDiffEnabled;
				sas.risk.rgf.api.CommonAPI._altTimeoutActivityInterval = json.altTimeoutActivityInterval;
				sas.risk.rgf.api.CommonAPI._notificationPollingInterval = json.notificationPollingInterval;
				sas.risk.rgf.api.CommonAPI._lboScrollPreventDefault = json.lboScrollPreventDefault;
				def.resolve(sas.risk.rgf.api.CommonAPI._timezoneMidday12hourEnabled);
				def.resolve(sas.risk.rgf.api.CommonAPI._iTZDiffEnabled);
				def.resolve(sas.risk.rgf.api.CommonAPI._altTimeoutActivityInterval);
				def.resolve(sas.risk.rgf.api.CommonAPI._notificationPollingInterval);
				def.resolve(sas.risk.rgf.api.CommonAPI._lboScrollPreventDefault);
		}).fail(function(e){
     	   jQuery.sap.require("sas.ui.cpb3.CpbUtil");
     	   sas.risk.rgf.util.RGFUtil.handleFail(e);
        	def.reject();
        	sas.risk.rgf.api.CommonAPI._timezoneMidday12hourEnabled=true;
        	sas.risk.rgf.api.CommonAPI._iTZDiffEnabled=true;
			sas.risk.rgf.api.CommonAPI._altTimeoutActivityInterval = -1;
			sas.risk.rgf.api.CommonAPI._notificationPollingInterval = 0;
        	sas.risk.rgf.api.CommonAPI._lboScrollPreventDefault=false;
      	});
		return def;
	},
	getTimezoneMidday12hourEnabled:function(){
		return sas.risk.rgf.api.CommonAPI._timezoneMidday12hourEnabled;
	},
	getITZDiffEnabled:function(){
		return sas.risk.rgf.api.CommonAPI._iTZDiffEnabled;
	},
	getAltTimeoutActivityInterval:function(){
		return sas.risk.rgf.api.CommonAPI._altTimeoutActivityInterval;
	},
	getNotificationPollingInterval:function(){
		return sas.risk.rgf.api.CommonAPI._notificationPollingInterval;
	},
	getLboScrollPreventDefault:function(){
		return sas.risk.rgf.api.CommonAPI._lboScrollPreventDefault;
	}
}