jQuery.sap.declare("sas.risk.rgf.api.DimensionalLocationAPI");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.risk.rgf.biz.DimPoint");
jQuery.sap.require("sas.risk.rgf.biz.Dimension");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");

sas.risk.rgf.api.DimensionalLocationAPI={
	loadDimPoint:function(pointRk){
		return $.getJSON(sas.risk.rgf.util.RGFUtil.getRestUrl()+"/dimensionalPoints/"+pointRk);
	},
    loadDimPoints:function(pointRks){
       var filter="in(dimPointRk,"+pointRks.join(",")+")&limit="+pointRks.length;
       return $.getJSON(sas.risk.rgf.util.RGFUtil.getRestUrl()+"/dimensionalPoints?filter="+filter);
    },
	loadNodes:function(dimName,keys,filter){
		var def=$.Deferred();
		var url=sas.risk.rgf.biz.Dimension.getRestURL(dimName);
		var params= $.extend({},filter);
		if (keys && keys.length) {
			params.filter="in(key,"+keys.join(",")+")";
			params.limit=keys.length;
		}
		params.includeInactive=true;
		var requestStr = url+"?"+$.param(params);
		$.getJSON(requestStr,function(json){
			def.resolve(json.items);
		});
		return def;
	},
	loadNodeChildren:function(dimName,nodeRk,securityKey,filter, showInactive){
		var url=sas.risk.rgf.biz.Dimension.getRestURL(dimName)+"/"+nodeRk+"/children";
		var securityFilter=securityKey?securityKey.createSecurityFilter():{};
		var filters=$.extend({limit:1000},securityFilter,filter);
		url=url+"?"+$.param(filters);
		if(showInactive) {
			url = url + "&includeInactive=true";
		}
		return sas.risk.rgf.api.UtilsAPI.getCumulativeDeferred(url);
	},
	searchNodes:function(dimName,query,securityKey,filter,loadAllAvailable, showInactive){
		var url=sas.risk.rgf.biz.Dimension.getRestURL(dimName);
		var securityFilter=securityKey?securityKey.createSecurityFilter():{};
		var filters=$.extend({},securityFilter,filter);
		var fullUrl=url+"?"+query.toString()+"&"+$.param(filters);
		if(showInactive) {
			fullUrl = fullUrl + "&includeInactive=true";
		}
		if(loadAllAvailable){
			return sas.risk.rgf.api.UtilsAPI.getCumulativeDeferred(fullUrl);
		}else{
			var def=$.Deferred();
			$.getJSON(fullUrl).done(function(json){
				def.resolve(json.items,json.count,json.start);
			});
			return def;
		}
	},
	loadAllFavorites:function(){
		var def=$.Deferred();
		$.getJSON(sas.risk.rgf.util.RGFUtil.getRestUrl()+"/favoriteAreas?limit=1000",function(json){
			def.resolve(json.items);
		});
		return def;
	},
	loadFavoritePoints:function(favoriteKey){
		var def=$.Deferred();
		$.getJSON(sas.risk.rgf.util.RGFUtil.getRestUrl()+"/favoriteAreas/"+favoriteKey+"/dimensionalPoints?limit=1000",function(json){
			def.resolve(json.items);
		});
		return def;		
	},
	loadPointsFromNodes:function(dimNodes){
		var def=$.Deferred();
		var i,dimNode;
		var handles=[];
		for(i=0;i<dimNodes.length;i++){
			dimNode=dimNodes[i];
			handles.push("'"+dimNode.getHandle()+"'");
		}
		var filter="crossProduct("+handles.join(",")+")";
		var url=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/dimensionalPoints?"+$.param({filter:filter})+"&limit=1000";
      return sas.risk.rgf.api.UtilsAPI.getCumulativeDeferred(url);
	},
	createFavorite:function(name,dimPoints){
		var points=_.pluck(dimPoints,"pointRk");
		var def=$.Deferred();
      $.ajax({
			type : "POST",
			url : sas.risk.rgf.util.RGFUtil.getRestUrl()+"/favoriteAreas",
			contentType : "application/json; charset=UTF-8",
			data : JSON.stringify({
				favoriteNm:name,
				dimensionalPoints:points
			})
		}).done(function(data, textStatus, jqXHR) {
			def.resolve(data);
		}).fail(function(e){
			sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
		});
		return def;
	},
	deleteFavorite : function(favoriteKey) {
		var def = $.Deferred();
		$.ajax({
			type : "DELETE",
			url : sas.risk.rgf.util.RGFUtil.getRestUrl()+"/favoriteAreas/" + favoriteKey
		}).done(function() {
			def.resolve();
		}).fail(function(e){
			sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
		});
		return def;
	},
	loadNodePath:function(dimName,pathArray,filter){
		var def = $.Deferred();
		var url=sas.risk.rgf.biz.Dimension.getRestURL(dimName);
		var nameAttr=sas.risk.rgf.biz.DimensionDefinition[dimName.toLowerCase()].nameAttr;
		var filters=$.extend({},filter,{nameAttr:pathArray.join("|")});
		url=url+"?"+$.param(filters);
      $.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:url
      }).done(function(data){
      	def.resolve(data.items);
      }).fail(function(e){
      	sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
      });
      return def;
	}
}