jQuery.sap.declare("sas.risk.rgf.api.UtilsAPI");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.risk.rgf.type.CurrencyType");

sas.risk.rgf.api.UtilsAPI = {
	_restMediaTypes:null,
	convertCurrency : function(amount, currencyCode, baseAmount, date) {
		var def = $.Deferred();
		var params={
			amt:amount,
			baseAmt:baseAmount,
			currCd:currencyCode,
			dt:date
		};
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/twoWayConvertCurrency?"+$.param(params)
		}).done(function(json){
			if(json.conversionError){
				def.reject(json.conversionError);
			}else{
				if(baseAmount){
					sas.risk.rgf.type.CurrencyType.addFormat(json.amtDouble,json.amt);
				}else{
					sas.risk.rgf.type.CurrencyType.addFormat(json.baseAmtDouble,json.baseAmt);
				}
				def.resolve(json);
			}
		}).fail(function(e){
			jQuery.sap.require("sas.ui.cpb3.CpbUtil");
			sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
		});
		return def;
	},
	formatCurrency:function(amount) {
		var def = $.Deferred();
		var params={
			amt:amount
		};
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/twoWayConvertCurrency?"+$.param(params)
		}).done(function(json){
			if(json.conversionError){
				def.reject(json.conversionError);
			}else{
				sas.risk.rgf.type.CurrencyType.addFormat(json.baseAmtDouble,json.baseAmt);
				def.resolve({amtDouble:json.baseAmtDouble,amt:json.baseAmt});
			}
		}).fail(function(e){
			jQuery.sap.require("sas.ui.cpb3.CpbUtil");
			sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
		});
		return def;		
	},
	isValidUrl : function(urlTxt) {
       var def = $.Deferred();
       var params={
           urlTxt:urlTxt
       };
       $.getJSON(sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/isValidUrl?"+$.param(params),function(json){
           def.resolve(json);
       }).fail(function(e){
      	 jQuery.sap.require("sas.ui.cpb3.CpbUtil");
          sas.risk.rgf.util.RGFUtil.handleFail(e);
          def.reject();
       });
       return def;	   
	},
	getRestMediaTypes:function(){
		return sas.risk.rgf.api.UtilsAPI._restMediaTypes;
	},
	loadRestMediaTypes:function(){
		var def=$.Deferred();
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/restMediaTypes"
		}).done(function(json){
			sas.risk.rgf.api.UtilsAPI._restMediaTypes=json.boTypes;
			def.resolve(sas.risk.rgf.api.UtilsAPI._restMediaTypes);
		}).fail(function(e){
			jQuery.sap.require("sas.ui.cpb3.CpbUtil");
			sas.risk.rgf.util.RGFUtil.handleFail(e);
			def.reject();
		});
		return def;
	},
	getVisibleRowCounts:function(){
		var def=$.Deferred();
		var url=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/visibleRowCounts";
		$.getJSON(url,function(json){
			sas.risk.rgf.util.RGFUtil._visibleRows=json.visibleRows;
			sas.risk.rgf.util.RGFUtil._dialogVisibleRows=json.dialogVisibleRows;
			def.resolve(json);
		});
		return def
	},
	getCumulativeDeferred:function(url){
		var def=$.Deferred();
		$.getJSON(url,function(json){
			var totalCount=json.count;
			var limit=json.items.length;
			if(totalCount>limit){
				var defs=[];
				for(var readCount=limit;readCount<totalCount;readCount+=limit){
					limit=Math.min(totalCount-readCount,limit);
					defs.push($.getJSON(url+"&limit="+limit+"&start="+readCount));
				}
				$.when.apply($,defs).done(function(){
					var resps=Array.prototype.slice.call(arguments,0);
					if(defs.length===1){
						resps=[resps];//the format for a single getJSON comes back differently than multiples. wrapping it so that the structures match
					}
					_.each(resps,function(resp){
						json.items=json.items.concat(resp[0].items);
					});
					def.resolve(json.items,totalCount,json.start);
				}).fail(function(e){
					def.reject(e);
				});
			}else{
				def.resolve(json.items,totalCount,json.start);
			}
		}).fail(function(e){
			def.reject(e);
		});
		return def;
	},
	getTickets:function(){
		var def=$.Deferred();
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/tickets"
		}).done(function(json){
			def.resolve(json);
		});
		return def;		
	},
	exportToMip:function(url){
		var def=$.Deferred();
      var xhr=new XMLHttpRequest();
  	   xhr.open("GET",url,true);
  	   xhr.responseType="blob";
  	   xhr.onload=function(e){
  	   	if(this.status==200){
  	   		var responseCode = xhr.getResponseHeader("ResponseCode");
  	   		if(parseInt(responseCode) === 0){
  	   			def.resolve(responseCode);
  	   		}else {
  	  	         var contentDispositionHeader = xhr.getResponseHeader("content-disposition");
  	  	         var fileName = contentDispositionHeader 
  	  	         				? contentDispositionHeader.slice(contentDispositionHeader.indexOf("filename")).replace("filename=", "")
  	  	         				: undefined;
  	  	   		var blob=(this.response && !!contentDispositionHeader && !!fileName) ? this.response : undefined;
 	   			def.reject({responseCode:responseCode, fileName:fileName, blob:blob});
  	   		}
  	   	}
  	   };
  	   xhr.onerror=function(e){
  	   	def.reject(e);
  	   };
  	   xhr.send();
  	   return def;
	},
	getRemoteFileList:function(url,rk){
		var def=$.Deferred();
      url=url+"&usageRk="+rk;
		$.getJSON(url)
      .done(function(data) {
          def.resolve(data);
      }).fail(function(e){
      	 jQuery.sap.require("sas.ui.cpb3.CpbUtil");
          sas.risk.rgf.util.RGFUtil.handleFail(e);
          def.reject();
      });
		return def
	},
	transferRemoteFile:function(fileName,workgroupKey,url,groupNumber,model,rk,etag){
		var that=this;
		var def=$.Deferred();
      url=url
      	+"&fileName="+fileName
      	+"&workgroup="+workgroupKey
      	+"&usageRk="+rk
      	+"&groupNo="+groupNumber
      	+"&etag="+etag;
		$.getJSON(url)
      .done(function(data, textStatus, jqXHR) {        
         sas.risk.rgf.api.ContentManagementAPI._updateParentETag(model, jqXHR);
      	sas.risk.rgf.api.ContentManagementAPI.addToUIContext(data, model, groupNumber);
         def.resolve(data);
      }).fail(function(e){
      	 jQuery.sap.require("sas.ui.cpb3.CpbUtil");
          sas.risk.rgf.util.RGFUtil.handleFail(e);
          def.reject();
      });
		return def
	},
	getAreaLabel:function(areaHandle) {
		var def=$.Deferred();
      var params={
          handle:areaHandle
      };
		$.ajax({
			headers: {
				Accept: "application/json; charset=UTF-8",
				"Content-Type": "application/json; charset=UTF-8"
			},
			type:"GET",
			dataType:"json",
			url:sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/locationText?"+$.param(params)
		}).done(function(json){
			def.resolve(json.label);
		}).fail(function(e){
     	   jQuery.sap.require("sas.ui.cpb3.CpbUtil");
         sas.risk.rgf.util.RGFUtil.handleFail(e);
         def.reject();
      });
		return def;	
	},
	execScript:function(url,fileContents) {
		var def=$.Deferred();
		var formData = new FormData();
	   formData.append("fileContents", fileContents);
      $.ajax({
	       type : "POST",
	       url : url,
	       data : formData,
	       processData : false,
	       contentType : false,
	       cache : false,
	       datatype: "application/json;charset=utf-8"
	   }).done(function(json){
			def.resolve(json);
		}).fail(function(e){
     	   jQuery.sap.require("sas.ui.cpb3.CpbUtil");
         sas.risk.rgf.util.RGFUtil.handleFail(e);
         def.reject();
      });
		return def;	
	},
	getDelegationMaxTime:function(){
		var out;
		var def=$.Deferred();
      $.ajax({
			headers: {Accept:"application/json; charset=UTF-8", "Content-Type":"application/json; charset=UTF-8"},
			async: false,
			type: "GET",
			dataType: "json",
			url: sas.risk.rgf.util.RGFUtil.getRestUrl() + "/utils/delegationMaxTime"
	   }).done(function(json){
			def.resolve(json);
			out=json.delegationMaxTime;
		}).fail(function(e){
     	   jQuery.sap.require("sas.ui.cpb3.CpbUtil");
         sas.risk.rgf.util.RGFUtil.handleFail(e);
         def.reject();
         out=false;
      });
		return out;
	}
	
}