jQuery.sap.declare("sas.risk.rgf.biz.DimNode");
jQuery.sap.require("sas.risk.rgf.biz.DimNodeRegistry");
jQuery.sap.require("sas.risk.rgf.api.DimensionalLocationAPI");
jQuery.sap.require("sas.risk.rgf.biz.DimensionDefinition");

sas.risk.rgf.biz.DimNode=function(key,levelNo,selectable,leaf,name,localizedPath,dimName,path,links){
	this.key=key;
	this.levelNo=levelNo;
	this.leaf=leaf;
	this.name=name;
	this.localizedPath=localizedPath;
	this.nodePath=path;
	this.dimName=dimName.toLowerCase();
	this.selectable=selectable;
	this.links=links;
};
sas.risk.rgf.biz.DimNode.prototype={
	getDimNodeKey:function(){
		return this.key;
	},
	isRoot:function(){
		return this.key===-1;
	},
	getLevelNo:function(){
		return this.levelNo;
	},
	getName:function(){
		return this.name;
	},
	getNodePath:function(){
		return this.nodePath;
	},
	getLocalizedPath:function(){
		if(this.localizedPath.length){
			var rb=sas.risk.rgf.util.RGFUtil.getBundle();
			var sep=" "+rb.getText("dimloc.pathsep.txt")+" ";
			return this.localizedPath.join(sep);
		}
		return this.name;
	},
	getLocalizedLabel:function(){
		if(this.localizedPath.length){
			return this.localizedPath[this.localizedPath.length-1];
		}
		return this.name;
	},
	getParentPath:function(rootLocalizedName){
		var parentPath=this.localizedPath.slice();
		parentPath.splice(0,0,rootLocalizedName);
		var rb=sas.risk.rgf.util.RGFUtil.getBundle();
		var sep=" "+rb.getText("dimloc.pathsep.txt")+" ";
		return parentPath.join(sep);
	},
	calcSelectable:function(feature){
		var dim=feature.getDimension(this.getDimensionName());
		return this.selectable && dim &&(!dim.getLeafOnly() || this.leaf && dim.getLeafOnly());
	},
	getDimensionName:function(){
		return this.dimName;
	},
	getLeaf:function(){
		return this.leaf;
	},
	setLeaf:function(leaf){
		this.leaf=leaf;
	},
	loadChildren:function(securityKey,filter, showInactive){
		var def=$.Deferred();
		var dimName=this.dimName;
		sas.risk.rgf.api.DimensionalLocationAPI.loadNodeChildren(this.dimName,this.key,securityKey,filter,showInactive).done(function(items){
			var i,item,node;
			var nodes=[];
			for(i=0;i<items.length;i++){
				item=items[i];
				node=sas.risk.rgf.biz.DimNode.createFromJson(dimName,item);
				sas.risk.rgf.biz.DimNodeRegistry.addToCache(node,securityKey);
				nodes.push(node);
			}
			def.resolve(nodes);
		}).fail(function(info){
			if(info.status===404){
				def.resolve([]);
			}
		});
		return def;
	},
	getHandle:function(){
		return sas.risk.rgf.biz.DimensionDefinition[this.dimName].name+"|"+this.key+"|";
	},
	equals:function(other){
		return other instanceof sas.risk.rgf.biz.DimNode &&
			    this.key === other.key &&
			    this.dimName === other.dimName;
	}
};
//STATICS
sas.risk.rgf.biz.DimNode.createFromJson=function(dimName,json){
	return new sas.risk.rgf.biz.DimNode(
			json.key,
			json.levelNo,
			json.selectable,
			json.leaf,
			json.localization.name,
			json.localization.localizedPath,
			dimName,
			json.path,
			json.links);
};
sas.risk.rgf.biz.DimNode.loadAndProcessChildren=function(dimName,dimKey,securityKey,filter){
	var def=$.Deferred();
	sas.risk.rgf.api.DimensionalLocationAPI.loadNodeChildren(dimName,dimKey,securityKey,filter).done(function(items){
		var i,item,node;
		var nodes=[];
		for(i=0;i<items.length;i++){
			item=items[i];
			node=sas.risk.rgf.biz.DimNode.createFromJson(dimName,item);
			sas.risk.rgf.biz.DimNodeRegistry.addToCache(node,securityKey);
			nodes.push(node);
		}
		def.resolve(nodes);
	});
	return def;
};