jQuery.sap.declare("sas.risk.rgf.biz.Feature");
jQuery.sap.require("sas.risk.rgf.biz.DimensionDefinition");
jQuery.sap.require("sas.risk.rgf.biz.Dimension");
jQuery.sap.require("sas.risk.rgf.biz.DimNodeRegistry");
jQuery.sap.require("sas.risk.rgf.biz.DimNode");

sas.risk.rgf.biz.Feature=function(name,businessObjectType,allowAncestors,allowDescendants,allowUnconstrained,hasConstraints,mode,excludedDimensions){
	this.name=name;
	this.businessObjectType=businessObjectType;
	this.allowAncestors=allowAncestors,
	this.allowDescendants=allowDescendants;
	this.allowUnconstrained=allowUnconstrained;
	this.hasConstraints=hasConstraints;
	this.mode=mode;
	this.dimensions=[];
	this.excludedDimensions=excludedDimensions||[];
	this.requiredDimensions=null;
	this.dimNames=null;
	this.reqdDimNames=null;
	this.allDimNames=[];
};
sas.risk.rgf.biz.Feature.prototype={
	getName:function(){
		return this.name;
	},
	getAllowUnconstrained:function(){
		return this.allowUnconstrained;
	},
	getHasConstraints:function(){
		return this.hasConstraints;
	},
	addDimension:function(dim){
		var lowerName=dim.getDimName().toLowerCase();
		this.allDimNames.push(lowerName);
		var incomingName=sas.risk.rgf.biz.DimensionDefinition[lowerName].name;
		if(this.excludedDimensions.indexOf(incomingName)===-1){
			this.dimensions.push(dim);
			this.requiredDimensions=null;
			this.dimNames=null;
			this.reqdDimNames=null;
		}
	},
	getDimension:function(dimName){
		return _.find(this.dimensions,function(dim){return dim.getDimName()===dimName;});
	},
	getDimensions:function(){
		return this.dimensions;
	},
	setExcludedDimensions:function(excludedDimensions){
		this.excludedDimensions=excludedDimensions;
		this.dimensions=_.filter(this.dimensions,function(dim){
			var def=sas.risk.rgf.biz.DimensionDefinition[dim.getDimName().toLowerCase()];
			var incomingName=def.name;
			var incomingAlias=def.alias||"";
			return excludedDimensions.indexOf(incomingName)===-1 && excludedDimensions.indexOf(incomingAlias)===-1 ;
		});
		this.requiredDimensions=null;
		this.dimNames=null;
		this.reqdDimNames=null;		
	},
	getRequiredDimensions:function(){
		if(this.requiredDimensions){return this.requiredDimensions;}
		this.requiredDimensions=[];
		var i,dim;
		for(i=0;i<this.dimensions.length;i++){
			dim=this.dimensions[i];
			if(dim.getIsRequired()){
				this.requiredDimensions.push(dim);
			}
		}
		return this.requiredDimensions;
	},
	getVisibleByDefaultDimensions: function() {
		this.visibleByDefaultDimensions=[];
		var i,dim;
		for(i=0;i<this.dimensions.length;i++){
			dim=this.dimensions[i];
			if(dim.visibleByDefault){
				this.visibleByDefaultDimensions.push(dim);
			}
		}
		return this.visibleByDefaultDimensions;
	},
	getDimensionNames:function(){
		if(this.dimNames){return this.dimNames;}
		this.dimNames=_.pluck(this.dimensions,'name');
		return this.dimNames;
	},
	getRequiredDimensionNames:function(){
		if(this.reqdDimNames){return this.reqdDimNames;}
		this.reqdDimNames=_.pluck(this.getRequiredDimensions(),'name');
		return this.reqdDimNames;
	},
	indexOfDimension:function(dimName){
		return this.allDimNames.indexOf(dimName.toLowerCase());
	}
};
sas.risk.rgf.biz.Feature.parseFromJson=function(json,capability){
	var feature=new sas.risk.rgf.biz.Feature(json.name,json.businessObjectType,json.allowAncestors,json.allowDescendants,json.allowUnconstrained,json.hasConstraints,json.mode);
	var securityKey=new sas.risk.rgf.biz.SecurityKey(feature,capability);
	var isForFiltering=!securityKey.getCapability();
	var i,dimInfo,dimension,dimNode,node;
	for(i=0;i<json.dimensionInfo.length;i++){
   	dimInfo=json.dimensionInfo[i];
   	dimension=new sas.risk.rgf.biz.Dimension(dimInfo.name,dimInfo.label,dimInfo.alias,dimInfo.dimNode.localization.name,(isForFiltering?false:dimInfo.isRequired),dimInfo.allowsMultiple,dimInfo.leafOnly,dimInfo.visibleByDefault,dimInfo.primary);
   	feature.addDimension(dimension);
   	dimNode=dimInfo.dimNode;
   	node=sas.risk.rgf.biz.DimNode.createFromJson(dimInfo.name,dimNode);
   	sas.risk.rgf.biz.DimNodeRegistry.addToCache(node,securityKey);
   }
	return feature;
};