jQuery.sap.declare("sas.risk.rgf.components.CalendarComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.risk.rgf.widget.RGFTable");
jQuery.sap.require("sas.risk.slimtable.meta.filter.AndFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.OrFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.ValueFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.TimestampFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.DateFilter");
jQuery.sap.require("sas.hc.m.MessageBox");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.ui.cpb3.model.format.CpbDateFormat");
jQuery.sap.require("sas.ui.cpb3.model.type.CpbDate");
jQuery.sap.require("sas.ui.cpb3.widget.HTML");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.rgf.util.NavigationUtil");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.CalendarComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
           dataUrl:{type:"string"},
      	  meta:{type:"any"},
      	  calendarType:{type:"string"},
           visibleRows:{type:"int"},
           titleAttribute:{type:"string"},
           plannedStartDateAttribute:{type:"string"},
           plannedStartDateType:{type:"string"},
           plannedEndDateAttribute:{type:"string"},
           plannedEndDateType:{type:"string"},
           actualStartDateAttribute:{type:"string"},
           actualStartDateType:{type:"string"},
           actualEndDateAttribute:{type:"string"},
           actualEndDateType:{type:"string"},
           plannedColor:{type:"string"},
           actualColor:{type:"string"},
           yearRange:{type:"int"},
           cpbValue:{type:"any"},//override type
           formattingKey:{type:"any"}
        }
    },
    constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.addStyleClass("RGFCalendarComp");
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.CalendarComponent);

sas.risk.rgf.components.CalendarComponent.prototype._createComponent = function() {
	this.getCpbForm().attachEvent('reload',$.proxy(this.onReload,this));
   var metaProp=this.getProperty("meta");
	if(!this.getCpbLabel() && metaProp.title){
      this.setCpbLabel(metaProp.title);
   }
   this.dataUrlProp=this.getProperty("dataUrl");
   this.comp = this.getAdvTableComponent(this.getCalendarType(), this.getProperty("visibleRows"));
   sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.comp,"enabled");
   
   this.comp.bindProperty("location","cpb_data>/LOCATION");
   this.comp.attachRowClick($.proxy(this.doRowClick,this));
   this._label.addStyleClass("RGFTabTitleLabel");
   return this.comp;
};
sas.risk.rgf.components.CalendarComponent.prototype.getTableTopActionsPlugin=function(){
	var that=this;
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	this.previousLink=new sas.risk.common.widgets.Button({
		icon:sas.icons.HC.ARROWSMALLLEFT,
		text:rb.getText("calendar.button.previous.title.txt"),
      press: $.proxy(this.actionPrevious12Months,this)
   }).addStyleClass("RGFCalActionLink");
	this.nextLink=new sas.risk.common.widgets.Button({
		icon:sas.icons.HC.ARROWSMALLRIGHT,
		text:rb.getText("calendar.button.next.title.txt"),
      press: $.proxy(this.actionNext12Months,this)
   }).addStyleClass("RGFCalActionLink");

   this.tableTopActionsPlugin=new sas.risk.slimtable.plugins.TablePlugin({
     	position: sas.risk.slimtable.TableLayout.TOP_END,
   	items:[this.previousLink, this.nextLink]
   });
   this.tableTopActionsPlugin.handleDisabled=function(){
   	//No changes needed to the actions in case of read-only
   };
   return this.tableTopActionsPlugin;
};
sas.risk.rgf.components.CalendarComponent.prototype.getAdvTableComponent=function(calendarType, visibleRowsProp){
   return new sas.risk.rgf.widget.RGFTable({
   	  formattingKey:this.getFormattingKey(),
        meta: this.getMeta(),
        data: $.proxy(this._fetchData,this),
        visibleRows: visibleRowsProp,
        showSelectionControls:false,
        plugins:[this.getTableTopActionsPlugin(),
                 new sas.risk.slimtable.plugins.Search(),
                 new sas.risk.slimtable.plugins.Filter(),
                 new sas.risk.slimtable.plugins.Counter(),
                 new sas.risk.slimtable.plugins.TablePlugin({
                   	position: sas.risk.slimtable.TableLayout.BOTTOM_CENTER,
                   	items:[this.createLegend()]
                 }),
                 new sas.risk.slimtable.plugins.Pager()
        ]
     }).addStyleClass("RGFCalTable");
}
sas.risk.rgf.components.CalendarComponent.prototype.createLegend=function(){
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var legend=new sas.ui.cpb3.widget.HTML({
		content:
			'<table class="borderedTable" cellpadding="2" cellspacing="0" border="0" align="center" style="width: auto;">'+
			   '<tbody><tr class="odd">'+
			      '<td width="50%" style="border-style: none;">'+
			         '<table>'+
			            '<tbody><tr>'+
			               '<td style="border-style: none;">'+
			                  '<table>'+
			                     '<tbody><tr>'+
			                        '<td class="RGFCalLegendColor" style="background-color: '+this.getPlannedColor()+'; font-size: 3pt;" width="22px">&nbsp;&nbsp;&nbsp;&nbsp;</td>'+
			                     '</tr>'+
			                  '</tbody></table>'+
			               '</td>'+
			               '<td style="border-style: none;">' + rb.getText("calendar.legend.planned.txt") +'</td>'+
			            '</tr>'+
			         '</tbody></table>'+
			      '</td>'+
			      '<td align="right" width="50%" style="border-style: none;">'+
			         '<table>'+
			            '<tbody><tr>'+
			               '<td align="right" style="border-style: none;">'+
			                  '<table>'+
			                     '<tbody><tr>'+
			                        '<td class="RGFCalLegendColor" style="background-color: '+this.getActualColor()+'; font-size: 3pt;" width="22px">&nbsp;&nbsp;&nbsp;&nbsp;</td>'+
			                     '</tr>'+
			                  '</tbody></table>'+
			               '</td>'+
			               '<td align="right" style="border-style: none;">' + rb.getText("calendar.legend.actual.txt") +'</td>'+
			            '</tr>'+
			         '</tbody></table>'+
			      '</td>'+
			   '</tr>'+
			'</tbody></table>'
	});
	return legend;
};
sas.risk.rgf.components.CalendarComponent.prototype.actionPrevious12Months=function(){
	var date=this.getCpbValue();
	this.setCpbValue(new Date(date.getFullYear()-1,date.getMonth(),1));
};
sas.risk.rgf.components.CalendarComponent.prototype.actionNext12Months=function(){
	var date=this.getCpbValue();
	this.setCpbValue(new Date(date.getFullYear()+1,date.getMonth(),1));
};
sas.risk.rgf.components.CalendarComponent.prototype.getCPBControl=function(){
	return this.comp;
};
sas.risk.rgf.components.CalendarComponent.prototype.setCpbValue=function(value){
	this.setProperty("cpbValue",value);
	var selectedStart=this.calcSelectedStartDate();
	var selectedEnd=this.calcSelectedEndDate(selectedStart);
   var rgfTableQuery=this.comp.getQuery().clone();
   var newFilter=new sas.risk.slimtable.meta.filter.AndFilter([
      new sas.risk.slimtable.meta.filter.ValueFilter("__MONTH__","eq",selectedStart.getMonth().toString()),
      new sas.risk.slimtable.meta.filter.ValueFilter("__YEAR__","eq",selectedStart.getFullYear().toString()),
   ]);
   newFilter.addFilter(rgfTableQuery.getFilterByName(this.getTitleAttribute(),"SIMPLE")[0]);
   rgfTableQuery.setQueryFilter(newFilter,"SIMPLE");
   this.updateMonthColumns(selectedStart.getMonth(),rgfTableQuery);
   this.updateYearRange(selectedStart.getFullYear());
   if(!rgfTableQuery.equal(this.comp.getQuery(),true)){
   	if(this.comp.table.built){
   		this.comp.doApplyQuery(rgfTableQuery);
   	}else{
   		this.comp.doApplyDefaultQuery(rgfTableQuery);
   	}
   }
};
sas.risk.rgf.components.CalendarComponent.prototype.updateMonthColumns=function(startingMonth,query){
	var colNames=[];
	colNames.push(this.getTitleAttribute());
	for(var i=0;i<12;i++){
	  colNames.push("__M"+((i+startingMonth)%12)+"__");
	}
   var orderedCols=this.comp.getSource().getTableMeta().getOrderedColumnsByNames(colNames);
   query.setOrderedColumns(orderedCols);
};
sas.risk.rgf.components.CalendarComponent.prototype.updateYearRange=function(selectedYear){
	var locale=sas.ui.cpb3.CpbUtil.getLocale();
	var range=Math.min(this.getYearRange()-1,100);
	var max=selectedYear+Math.ceil(range/2);
	var min=selectedYear-Math.floor(range/2);
	var newChoices=[];
	for(var i=min;i<=max;i++){
		//FIXME https://rndjira.sas.com/browse/RGFRMWRK-437
		//FIXME Thai: date and time under Data Created are not converted for different calendar
		// Change the label to Thai solar year, keep underlying year the same to keep things working.
		var choiceLabel = i.toString();
		if (locale.sLocaleId === "th_TH" && i > 1900 && i < 2200) {
			choiceLabel = (i + 543).toString();
		}
		newChoices.push({label:choiceLabel,value:i.toString()});
		//newChoices.push({label:i.toString(),value:i.toString()});
	}
	var colMeta=this.comp.table.getSource().getTableMeta().getFilterableColumns()[1];
	colMeta.setColChoices(newChoices);
};
sas.risk.rgf.components.CalendarComponent.prototype.createYearItem=function(year,column){
	return new sas.risk.slimtable.widget.Item({
	   key:year.toString(),
	   text:year.toString(),
	   payload:{
	   	value:year.toString(),
	   	colMeta:column
	   }
	});
};
sas.risk.rgf.components.CalendarComponent.prototype.calcSelectedStartDate=function(){
	var selectedStart=this.getCpbValue();
	if(!selectedStart){
		selectedStart=new Date();
	}
	selectedStart.setDate(1);
	selectedStart.setHours(0);
	selectedStart.setMinutes(0);
	selectedStart.setSeconds(0);
	selectedStart.setMilliseconds(0);
	return selectedStart;
};
sas.risk.rgf.components.CalendarComponent.prototype.calcSelectedEndDate=function(selectedStart){
	return new Date(new Date(selectedStart.getFullYear()+1,selectedStart.getMonth(),1).getTime()-1);
};
sas.risk.rgf.components.CalendarComponent.prototype._fetchData = function(query) {
	var clone=query.clone();
	var selectedMonth=query.getFilterByName("__MONTH__","SIMPLE")[0].getValue();
	var selectedYear=query.getFilterByName("__YEAR__","SIMPLE")[0].getValue();
	var selectedStart=new Date(selectedYear,selectedMonth,1);
	this.setCpbValue(selectedStart);
	var selectedEnd=this.calcSelectedEndDate(selectedStart);
   this.updateMonthColumns(selectedStart.getMonth(),query);
   this.updateYearRange(selectedStart.getFullYear());
	var pS=this.getPlannedStartDateAttribute();
	var pStS=this.getPlannedStartDateType()==="timestamp";
	var pE=this.getPlannedEndDateAttribute();
	var pEtS=this.getPlannedEndDateType()==="timestamp";
	var aS=this.getActualStartDateAttribute();
	var aStS=this.getActualStartDateType()==="timestamp";
	var aE=this.getActualEndDateAttribute();
	var aEtS=this.getActualEndDateType()==="timestamp";
	var filter=new sas.risk.slimtable.meta.filter.OrFilter([
      new sas.risk.slimtable.meta.filter.AndFilter(
		pStS?[
         new sas.risk.slimtable.meta.filter.TimestampFilter(pS,"le",selectedEnd),
         new sas.risk.slimtable.meta.filter.TimestampFilter(pE,"ge",selectedStart)
      ]:[
         new sas.risk.slimtable.meta.filter.DateFilter(pS,"le",selectedEnd),
         new sas.risk.slimtable.meta.filter.DateFilter(pE,"ge",selectedStart)
      ]),
      new sas.risk.slimtable.meta.filter.AndFilter(
		aStS?[
 	      new sas.risk.slimtable.meta.filter.TimestampFilter(aS,"le",selectedEnd),
 	      new sas.risk.slimtable.meta.filter.TimestampFilter(aE,"ge",selectedStart)
      ]:[
 	      new sas.risk.slimtable.meta.filter.DateFilter(aS,"le",selectedEnd),
 	      new sas.risk.slimtable.meta.filter.DateFilter(aE,"ge",selectedStart)
      ])
	]);
	var titleFilter=query.getFilterByName(this.getTitleAttribute(),"SIMPLE")[0];
	if(titleFilter){
		_.each(filter.getChildren(),function(f){f.addFilter(titleFilter);});
	}
	clone.setQueryFilter(filter,"SIMPLE");
	var that=this;
	var formatter=new sas.ui.cpb3.model.type.CpbDate({style: "medium"});
	var pStartAttr=that.getPlannedStartDateAttribute();
	var pEndAttr=that.getPlannedEndDateAttribute();
	var aStartAttr=that.getActualStartDateAttribute();
	var aEndAttr=that.getActualEndDateAttribute();
	var plannedColor=this.getPlannedColor();
	var actualColor=this.getActualColor();
	var i,item,pStart,pEnd,aStart,aEnd,pStartDate,pEndDate,aStartDate,aEndDate,range,description;
	var def=$.Deferred();
	this.getCPBControl().fetchData(this.dataUrlProp,clone).done(function(json){
		for(i=0;i<json.items.length;i++){
			item=json.items[i];
			pStart=item[pStartAttr]||(item.customFields && item.customFields[pStartAttr]);
			pEnd=item[pEndAttr]||(item.customFields && item.customFields[pEndAttr]);
			aStart=item[aStartAttr]||(item.customFields && item.customFields[aStartAttr]);
			aEnd=item[aEndAttr]||(item.customFields && item.customFields[aEndAttr]);
			if(pStart && pEnd){
				pStartDate=that.parseDateString(pStart);
				pEndDate=that.parseDateString(pEnd);
				description=formatter.formatValue(pStartDate,"string")+" - "+formatter.formatValue(pEndDate,"string");
				range=that.calcVisibleRange(pStartDate,pEndDate,selectedStart,selectedEnd,"planned",plannedColor,description);
				$.extend(true,item,range);
			}
			if(aStart && aEnd){
				aStartDate=that.parseDateString(aStart);
				aEndDate=that.parseDateString(aEnd);
				description=formatter.formatValue(aStartDate,"string")+" - "+formatter.formatValue(aEndDate,"string");
				range=that.calcVisibleRange(aStartDate,aEndDate,selectedStart,selectedEnd,"actual",actualColor,description);
				$.extend(true,item,range);
			}
		}
		def.resolve(json);
	});
	return def;
};
sas.risk.rgf.components.CalendarComponent.prototype.parseDateString=function(dateStr){
	var inputFormat = sas.ui.cpb3.model.format.CpbDateFormat.getInstance({pattern: "yyyy-MM-ddz"}, sas.risk.rgf.util.RGFUtil.getLocale());
	return inputFormat.parse(dateStr);
};
sas.risk.rgf.components.CalendarComponent.prototype.calcVisibleRange = function(rangeStart,rangeEnd,selectedStart,selectedEnd,rangeName,color,description) {
	var rangeStartIsPrior=rangeStart<selectedStart;
	var rangeStartIsBeyond=rangeStart>selectedEnd;
	var rangeEndIsPrior=rangeEnd<selectedStart;
	var rangeEndIsBeyond=rangeEnd>selectedEnd;

	var state=rangeEndIsPrior?"ALL_PREVIOUS":
			    rangeStartIsBeyond?"ALL_AFTER":
			    rangeStartIsPrior && !rangeEndIsPrior && !rangeEndIsBeyond?"STRADLE_START":
	          !rangeStartIsPrior && !rangeStartIsBeyond && rangeEndIsBeyond?"STRADLE_END":
	          !rangeStartIsPrior && !rangeEndIsBeyond?"WITHIN":"STRADLE_BOTH";
	switch(state){
		case "ALL_PREVIOUS":
		case "ALL_AFTER":
			return {};
		case "STRADLE_START":
			return this.formatRange(selectedStart,rangeEnd,rangeName,color,description);
		case "STRADLE_END":
			return this.formatRange(rangeStart,selectedEnd,rangeName,color,description);
		case "WITHIN":
			return this.formatRange(rangeStart,rangeEnd,rangeName,color,description);
		case "STRADLE_BOTH":
			return this.formatRange(selectedStart,selectedEnd,rangeName,color,description);
	}
};
sas.risk.rgf.components.CalendarComponent.prototype.formatRange = function(rangeStart,rangeEnd,rangeName,color,description) {
	var range={};
	var daysInMonth=this.getDaysInMonth(rangeStart);
	//HANDLE START
	var startMonthName="__M"+rangeStart.getMonth()+"__";
	range[startMonthName]={};
	range[startMonthName][rangeName]={
		isStart:true,
		isEnd:false,
		daysInMonth:daysInMonth,
		start:rangeStart.getDate(),
		end:daysInMonth,
		color:color,
		description:description
	}
	//HANDLE MIDDLE
	var monthStart=this.incrementMonth(new Date(rangeStart.getFullYear(),rangeStart.getMonth(),1));
	var midMonthName;
	while(monthStart<rangeEnd){
		daysInMonth=this.getDaysInMonth(monthStart);
		midMonthName="__M"+monthStart.getMonth()+"__";
		range[midMonthName]={};
		range[midMonthName][rangeName]={
			isStart:false,
			isEnd:false,
			daysInMonth:daysInMonth,
			start:1,
			end:daysInMonth,
			color:color,
			description:description
		};
		monthStart=this.incrementMonth(monthStart);
	}
	//HANDLE END
	var endMonthName="__M"+rangeEnd.getMonth()+"__";
	range[endMonthName]=range[endMonthName]||{};
	var endMonthEntry=range[endMonthName][rangeName];
	if(endMonthEntry){//merge in
		endMonthEntry.isEnd=true;
		endMonthEntry.end=rangeEnd.getDate();
	}else{//add new entry
		range[endMonthName][rangeName]={
			isStart:false,
			isEnd:true,
			daysInMonth:this.getDaysInMonth(rangeEnd),
			start:1,
			end:rangeEnd.getDate(),
			color:color,
			description:description
		};
	}
	return range;
};
sas.risk.rgf.components.CalendarComponent.prototype.incrementMonth = function(date) {
	if(date.getMonth()==11){
		return new Date(date.getFullYear()+1,0,date.getDate());
	}else{
		return new Date(date.getFullYear(),date.getMonth()+1,date.getDate());
	}
};
sas.risk.rgf.components.CalendarComponent.prototype.getDaysInMonth = function(date) {
	if(date.getMonth()==11){
		return 31;
	}else{
		return new Date(new Date(date.getFullYear(),date.getMonth()+1,1).getTime()-1).getDate();
	}
};
sas.risk.rgf.components.CalendarComponent.prototype.doRowClick=function(oEvent){
   console.log(oEvent);
   var type = this.getCalendarType();
   var rowData=this.comp.getSource().getData()[oEvent.getParameter("rowIdx")];
   sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
		this.getCpbForm(),
		{
			event_name:"object_selected",
			type:type,
			key:rowData.key
		}
	);   
}

sas.risk.rgf.components.CalendarComponent.prototype.doCPBRefresh = function() {
	this.getCPBControl().doRefresh();
};
sas.risk.rgf.components.CalendarComponent.prototype.onReload = function(event) {
	var type=event.getParameter("type");
	if(type===this.getCalendarType() || type==="*"){
		this.doCPBRefresh();
	}
};
//STATICS
sas.risk.rgf.components.CalendarComponent.create=function(fieldJson, modelId, cpbForm) {
	if(fieldJson.calendarType&&!fieldJson.dataUrl){//making dataUrl optional,just derive url from query table type
		var restMediaTypes=sas.risk.rgf.api.UtilsAPI.getRestMediaTypes();
		var mediaType=_.find(restMediaTypes,function(type){return type.boTypeNm===fieldJson.calendarType;});
		fieldJson.calendarType=mediaType.restUri;
		fieldJson.dataUrl=sas.risk.rgf.util.RGFUtil.aimRestUrl(fieldJson.calendarType);
	}else{
		fieldJson.dataUrl=sas.risk.rgf.util.RGFUtil.aimRestUrl(fieldJson.dataUrl);
	}
	var calendarComponent=new sas.risk.rgf.components.CalendarComponent($.extend(
		sas.ui.cpb3.WidgetBindingUtil.getHelpSettings(fieldJson,modelId),
		{
			meta : fieldJson.meta,
			dataUrl : fieldJson.dataUrl,
			visibleRows : fieldJson.visibleRows,
			calendarType : fieldJson.calendarType,
			titleAttribute : fieldJson.titleAttribute,
			plannedStartDateAttribute : fieldJson.plannedStartDateAttribute,
			plannedStartDateType : fieldJson.plannedStartDateType,
			plannedEndDateAttribute : fieldJson.plannedEndDateAttribute,
			plannedEndDateType : fieldJson.plannedEndDateType,
			actualStartDateAttribute : fieldJson.actualStartDateAttribute,
			actualStartDateType : fieldJson.actualStartDateType,
			actualEndDateAttribute : fieldJson.actualEndDateAttribute,
			actualEndDateType : fieldJson.actualEndDateType,
			yearRange : fieldJson.yearRange,
			plannedColor : fieldJson.plannedColor,
			actualColor : fieldJson.actualColor,
			cpbName:fieldJson.cpbName?fieldJson.cpbName.toUpperCase():fieldJson.cpbName,
			cpbForm : cpbForm,
			cpbId : fieldJson.widgetId,
			cpbAlign:fieldJson.cpbAlign,
			cpbVisible:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbVisible),
         formattingKey :sas.risk.rgf.util.RGFUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
			cpbLabel:{
				path:"cpb_labels>/label"+fieldJson.cpbLabel,
				formatter:sas.ui.cpb3.WidgetBindingUtil.formatterLabel
			}
		}
	));
	jQuery.sap.require("sas.ui.cpb3.model.type.CpbDate");
	calendarComponent.bindProperty("cpbValue",{
		path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,fieldJson.cpbName),
		type: 'sas.ui.cpb3.model.type.CpbDate',
		formatOptions: {field:calendarComponent}
	});
	return calendarComponent;
};