jQuery.sap.declare("sas.risk.rgf.components.CodeEditorComponent");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.hc.ui.commons.codeEditor.CodeEditor");
jQuery.sap.require("sas.hc.ui.commons.Toolbar");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.CodeEditorComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata : {
		properties : {
			"cpbWidth" : {
				type : "sap.ui.core.CSSSize"
			},
			"cpbHeight" : {
				type : "sap.ui.core.CSSSize"
			},
			"language" : {
				type : "string"
			},
			"cpblanguage" : {
				type : "string"
			},
			"inline" : {
				type : "boolean",
				defaultValue : false
			},
			"autocomplete" : {
				type : "boolean",
				defaultValue : false
			}
		}
	},
	constructor : function(args) {
		this.supportsLayout = true;
		this.supportsLabel = true;
		this.initLoad = true;
		this.jsonFields = args;
		
		//Editor Settings
		var settings =  {};
		//Start with a blank editor
		settings.content = "";
		settings.autoCompleteEnabled = false;
		settings.snippetsAutoCompleteEnabled = false;
		settings.pairQuotations = false;
		settings.pairBrackets = false;		
		if(args.autocomplete === true) {
			settings.autoCompleteEnabled = true;
			settings.snippetsAutoCompleteEnabled = true;
			settings.pairQuotations = true;
			settings.pairBrackets = true;
		}
		
		//Check if language mode set
		if (args.cpblanguage) {
			settings.langMode = args.cpblanguage;
		}		
		var codeEditorID = args.cpbName + Math.round(window.performance.now());
		
		this.codeEditor = new sas.hc.ui.commons.codeEditor.CodeEditor(codeEditorID, settings);
		this.codeEditor._ctrl.core().cleanText=function(text){return text;};
		var rb=sas.risk.rgf.util.RGFUtil.getBundle();
		
		var printLineNumber = true;
		var that = this;
		this.codeEditorToolbar = new sas.hc.m.ResponsiveToolbar({
			content : [ new sas.hc.m.Button({
				icon : sas.icons.HC.CUT,
				id : codeEditorID + "-cut",
				press : function() {
					that.codeEditor.cut();
				}
			}), new sas.hc.m.Button({
				icon : sas.icons.HC.COPY,
				id : codeEditorID + "-copy",
				press : function() {
					that.codeEditor.copy();
				}
			}), new sas.hc.m.Button({
				icon : sas.icons.HC.CLEARALL,
				id : codeEditorID + "-clearall",
				press : function() {
					that.codeEditor.clear();
				}
			}), new sas.hc.m.Button({
				icon : sas.icons.HC.FIND,
				id : codeEditorID + "-find",
				press : function() {
					that.codeEditor.showFindReplaceDialog();
				}
			}), new sas.hc.m.Button({
				icon : sas.icons.HC.REDO,
				id : codeEditorID + "-redo",
				press : function() {
					that.codeEditor.redo();
				}
			}), new sas.hc.m.Button({
				icon : sas.icons.HC.UNDO,
				id : codeEditorID + "-undo",
				press : function() {
					that.codeEditor.undo();
				}
			}),
			new sas.hc.m.Button({
            icon: sas.icons.HC.PRINT,
            id: codeEditorID + "-print",
            press: function() {
                var themeMgr = sas.hc.ui.commons.codeEditor.ThemeMgr.getInstance();
                var setTheme = function(doc, themeData, theme, addClass) {
                    var dom = $(doc.body).find('.textview');
                    if (dom.length > 0) {
                        if (addClass) dom.addClass('sce-' + theme);
                        dom.css('backgroundColor', themeData[theme]['background-color']);
                    }
                };
                var setDarkTheme = function(doc, themeData) {
                    var dark = 'dark';
                    if (themeMgr.getCurrentThemeName() === dark){
                        var darkStyle = sas.hc.ui.core.util.getStyleSheet('sas.hc.ui.core');
                        doc.write('<link type="text/css" rel="stylesheet" href=' + darkStyle.href + '>');
                        setTheme(doc, themeData, dark);
                    }
                };
                var style = sas.hc.ui.core.util.getStyleSheet('sas.hc.ui.commons.codeEditor');
                var w = window.open(), doc = w.document;
                doc.write('<link type="text/css" rel="stylesheet" href=' + style.href + '>');
                doc.write(that.codeEditor.getHTML({ lineNumber: printLineNumber }));
                doc.body.style.overflow = 'auto';
                var language = that.codeEditor._language;
                doc.close();
                window.setTimeout(function() {
                    w.print();
                }, 1000);
            }
			})
			]
		}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData(
				{alignSelf:sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.FlexEnd}));
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this, arguments);

	},
	renderer : "sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));

sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.CodeEditorComponent);

sas.risk.rgf.components.CodeEditorComponent.prototype.wireOnChange = function(onChangeFn) {
	if(onChangeFn){
		this.codeEditor.attachTextChanged(onChangeFn);
	}
};

sas.risk.rgf.components.CodeEditorComponent.prototype._createComponent = function(args) {	
	var that=this;
	this.codeEditor.attachTextChanged(function(e){
		that.setCpbValue(that.codeEditor.getText(), true);
	});
	
	//Listen to the destroyEditor event and delegate to the RichText accordingly. 
	this.attachEvent("destroyEditor",function(e){
		if( that.codeEditor ) { that.codeEditor.exit(); }
	});
	
	//Override onAfterRendering
	that.codeEditor._oAR = this.codeEditor.onAfterRendering;
	this.codeEditor.onAfterRendering = function() {
		that.codeEditor._oAR();
		that.codeEditor.refresh();
	}
	
	if(this.getCpbAlign() === "top" && this._label) {
		this._label.addStyleClass('cpbCodeEditorLabel');
	}
		
	var layoutWidth = this.jsonFields.cpbWidth ? this.jsonFields.cpbWidth : "100%";
	return new sas.risk.common.layout.flexlayout.FlexLayout({
		width : layoutWidth,
		alignItems: sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Stretch,
		alignContent: sas.risk.common.layout.flexlayout.FlexLayoutAlignContent.Stretch,
		flexDirection : sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		justifyContent : sas.risk.common.layout.flexlayout.FlexLayoutJustifyContent.SpaceBetween,
		items : [ this.codeEditorToolbar, this.codeEditor
		]
	})
};
sas.risk.rgf.components.CodeEditorComponent.prototype.setCPBValue = function(value) {
	this.setCpbValue(value);
};

sas.risk.rgf.components.CodeEditorComponent.prototype.setCpbValue = function(val, fromEventListener) {
	this.setProperty("cpbValue", val, true);
	if(val && !fromEventListener) { 
		this.codeEditor.setText(val)
	}
	
	if(!this.getCpbEnabled())  {
		this.codeEditor.setReadOnly(true);
		this.codeEditor.setLineHighlightingVisible(false);
		this.codeEditorToolbar.getContent()[0].setEnabled(false);
		this.codeEditorToolbar.getContent()[2].setEnabled(false);
		this.codeEditorToolbar.getContent()[4].setEnabled(false);
		this.codeEditorToolbar.getContent()[5].setEnabled(false);
	}
	
	var ceHeight = this.getCpbHeight() ? this.getCpbHeight() : '400px';
	this.codeEditor.resize('100%', ceHeight);
};
sas.risk.rgf.components.CodeEditorComponent.prototype.getData = function() {
	this.codeEditor.getText();
};
sas.risk.rgf.components.CodeEditorComponent.prototype.handleSave = function(e) {
	alert("RichTextComponent handle save");
};
// STATICS
sas.risk.rgf.components.CodeEditorComponent.create = function(fieldJson, modelId, cpbForm) {
	var settings = sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson, modelId, cpbForm);
	return new sas.risk.rgf.components.CodeEditorComponent($.extend(settings, {
		cpbValue : sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.cpbName),
		cpbWidth : fieldJson.cpbWidth,
		cpbHeight : fieldJson.cpbHeight,
		language : fieldJson.language,
		cpblanguage : fieldJson.cpblanguage,
		autocomplete : fieldJson.autocomplete,
		inline : fieldJson.inline
	}));
};