jQuery.sap.declare("sas.risk.rgf.components.ColorDropdownComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.slimtable.widget.Item");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sap.ui.layout.VerticalLayout");
jQuery.sap.require("sas.risk.slimtable.widget.Select");
jQuery.sap.require("sas.ui.cpb3.widget.HTML");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
/*
 *  This component displays dropdown and associated color box. 
 */
sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.ColorDropdownComponent",sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
				baseValues:{type:"any"},
				optionColorMap:{type:"any"}
        }
    },
    constructor:function(){
   	 this.supportsLayout=true;
   	 this.supportsLabel=true;
   	 this.supportsAlign=true;
   	 sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
   	 this.addStyleClass("RGFColor");
   	 this.addStyleClass("cpbFieldLikeComponent");
    }
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.ColorDropdownComponent);

sas.risk.rgf.components.ColorDropdownComponent.prototype.setCPBValue = function(value) {
	this.setCpbValue(value);
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.setCPBValues = function(value) {
	this.setCpbValues(value);
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.wireOnChange=function(onChangeFn){
	if(onChangeFn){
		this.onChangeFns=this.onChangeFns||[];
		this.onChangeFns.push(onChangeFn);
	}
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.setCpbValues = function(values) {
	this.setProperty("cpbValues",values);
	if(values){
		this.doBuildSelection(values);
		if(this.getCpbEnabled()){
			this.comp.getModel().setData(this.labelValueColorData);//update dropdown values accordingly
		}
	}
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.handleCurrentVal=function() {
	var currentVal=this.getCpbValue();
	if(_.isNumber(currentVal)){
		currentVal=currentVal.toString();
	}
	var selection=_.find(this.labelValueColorData,function(item){return item.value===currentVal});
	if(selection){
		this.currentValueMap={label:selection.label, value:selection.value, color:selection.color};
	}
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.handleReadonly = function(){
	var val=this.getCpbEnabled();
	if(val===false){
		this.comp = this.text;
		this.select.setVisible(false);
		this.text.setVisible(true);
		if(this.currentValueMap){
			this.comp.setText(this.currentValueMap.label);
		}
		else{
			this.comp.setText("");
		}
	}else{
		this.comp = this.select;
		this.select.setVisible(true);
		this.text.setVisible(false);
	}
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.setCpbEnabled = function(value) {
	if(this.getProperty("cpbEnabled")===value){return;}//skip if its the same
	this.setProperty("cpbEnabled",value);
	this._refresh();
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.setCpbValue = function(value) {
	if(this.getProperty("cpbValue")===value){return;}//skip if its the same
	this.setProperty("cpbValue",value);
	this._refresh();
};
sas.risk.rgf.components.ColorDropdownComponent.prototype._refresh = function() {
	if(this.getCpbEnabled()!==undefined &&	this.getCpbValue()!==undefined){
		this.handleCurrentVal();
		this.handleReadonly();
		this.doHandleNoneSelection();
	} else if(this.getCpbEnabled()!==undefined) {
		this.handleReadonly();
	}
};
sas.risk.rgf.components.ColorDropdownComponent.prototype._createComponent = function() {
	var that=this;
	this.select = new sas.risk.slimtable.widget.Select({
      items: {
      	path: "/",
         template: new sas.risk.slimtable.widget.Item({
         	text: '{label}',
         	key: '{value}',
         	payload: '{color}'
         })
      },
      change:function(e){
      	var item=e.getParameter("selectedItem");
      	that.setCpbValue(item.getKey());
      	_.each(that.onChangeFns,function(onChangeFn){onChangeFn();});
      }
   });
	this.text = new sap.ui.commons.TextView({visible:false});
	this.comp=this.select;
	this.doBuildSelection(this.getBaseValues());
	var model=new sap.ui.model.json.JSONModel(this.labelValueColorData);
   this.comp.setModel(model);
   this.comp.attachChange(function(event){
		that.colorBox.rerender(); //For the colorBox to update accordingly
   });
   this.colorBox=new sas.ui.cpb3.widget.HTML({
   	content:"<div class='RGFColorDropDown'></div>",
   	afterRendering:function(){
   		var selectedColor=that.comp.getSelectedItem&&that.comp.getSelectedItem() ? that.comp.getSelectedItem().getPayload()||"#FFFFFF" : that.currentValueMap ? that.currentValueMap.color : "#FFFFFF";
      	if(selectedColor){
      		$(that.colorBox.getDomRef()).css("background-color",selectedColor);
      	}
   	}
   });
   this.layout=new sas.risk.common.layout.flexlayout.FlexLayout({
   	items:[this.colorBox,this.select,this.text]
   });
   return this.layout;
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.doBuildSelection=function(values){
	var rb=sas.risk.rgf.util.RGFUtil.getBundle();
	this.labelValueColorData=values;
	var optionColorMap=this.getOptionColorMap();
	var i;
	if(optionColorMap){
		for(i=0;i<this.labelValueColorData.length;i++){
			var item=this.labelValueColorData[i];
			if(optionColorMap[item.value]) {item.color=optionColorMap[item.value];}
		}
	}
	if(this.labelValueColorData && this.labelValueColorData.length){
		var first=this.labelValueColorData[0];
		var containsNoneValue=first.value==="" && first.color==="#FFFFFF";
		if(!containsNoneValue){
			this.labelValueColorData.unshift({label:rb.getText("select.none.selected.txt"),value:"",color:"#FFFFFF"});
		}
	}
};
sas.risk.rgf.components.ColorDropdownComponent.prototype.doHandleNoneSelection = function() {
	if(!this.getCpbEnabled()) return; //In case of read-only, no need to handle this block which is specific to adding none selected to the selection
	var rb=sas.risk.rgf.util.RGFUtil.getBundle();
	var currentVal=this.getCpbValue();
	var ifRequired=this.getCpbRequired();
	var first=this.labelValueColorData[0];
	var containsNoneValue=first && first.value==="" && first.color==="#FFFFFF";
	if((!ifRequired || !currentVal) && !containsNoneValue){
		this.labelValueColorData.unshift({label:rb.getText("select.none.selected.txt"),value:"",color:"#FFFFFF"});
		this.comp.getModel().setData(this.labelValueColorData);
	}
	else if(ifRequired && currentVal && containsNoneValue){
		this.labelValueColorData.shift();
		this.comp.getModel().setData(this.labelValueColorData);
	}
	var selection=_.find(this.labelValueColorData,function(item){return item.value===currentVal});
	if(selection){this.comp.setSelectedKey(selection.value);}
};
//STATICS
sas.risk.rgf.components.ColorDropdownComponent.create=function(fieldJson, modelId, cpbForm) {
	jQuery.sap.require("sas.ui.cpb3.model.type.CpbString");
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	var colorDropDownComponent=new sas.risk.rgf.components.ColorDropdownComponent($.extend(
		settings,
		{
			cpbValues : sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbValues),
			baseValues : fieldJson.baseValues,
			optionColorMap : fieldJson.optionColorMap
		}
	));
	colorDropDownComponent.bindProperty("cpbValue",{
		path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,fieldJson.cpbName),
		type: 'sas.ui.cpb3.model.type.CpbString',
		formatOptions: {field:colorDropDownComponent}
	});
	return colorDropDownComponent;
};