jQuery.sap.declare("sas.risk.rgf.components.CurrencyInput");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.risk.common.utils.CompositeUtil");
jQuery.sap.require("sas.ui.cpb3.widget.InputField");
jQuery.sap.require("sas.risk.slimtable.widget.Select");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.risk.rgf.api.UtilsAPI");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");

var ValueState = sap.ui.core.ValueState;

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.CurrencyInput", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata:{
		properties:{
			amountField:{type:"string"},
			baseAmountField:{type:"string"},
			currencyCdField:{type:"any"},
			conversionDtField:{type:"any"},
			allowEditOfBaseAmount:{type:"boolean"},
			updateConversionDt:{type:"boolean"},
			baseCurrencyCode:{type:"string"}
		},
		aggregations : {
			currencyCodes : {type : "sap.ui.core.Item", multiple : true, singularName : "currencyCode", bindable : "bindable"} 
		},
		events : {
			change : {}
		}
	},
   constructor:function(){
  	 this.supportsLayout=true;
  	 this.supportsLabel=true;
  	 this.supportsAlign=true;
  	 sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
   },
	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.CurrencyInput);
sas.risk.rgf.components.CurrencyInput.prototype.applySettings=function(settings,scope){
	sas.risk.common.layout.flexlayout.FlexLayout.prototype.applySettings.apply(this,arguments);
	this.addStyleClass("RGFCurrency");
	var rb=sas.risk.rgf.util.RGFUtil.getBundle();
	this.amountInput=new sas.ui.cpb3.widget.InputField({
		type: sap.m.InputType.Number,
		editable:{
			parts:settings.cpbEnabled.parts,
			formatter:settings.cpbEnabled.formatter
		},
		change:$.proxy(this.actionChange,this,"AMT")
	}).addStyleClass("RGFCurrencyInput");
	this.amountCurrencyCd=new sas.risk.slimtable.widget.Select({
		enabled:{
			parts:settings.cpbEnabled.parts,
			formatter:settings.cpbEnabled.formatter
		},		
		change:$.proxy(this.actionChange,this,"CD")
	});
	this.baseAmountInput=new sas.ui.cpb3.widget.InputField({
		type: sap.m.InputType.Number,
		editable:{
			parts:settings.cpbEnabled.parts,
			formatter:function(){
				return settings.allowEditOfBaseAmount && settings.cpbEnabled.formatter.apply(this,arguments);
			}
		},
		change:$.proxy(this.actionChange,this,"BASE")
	}).addStyleClass("RGFCurrencyInput");
	this.baseCurrencyCd=new sas.hc.m.Label().addStyleClass("RGFCurrencyBaseLabel");
	this.convertButton=new sas.risk.common.widgets.Button({
		text:rb.getText("currency.convert.button.txt"),
		visible:{
			parts:settings.cpbEnabled.parts,
			formatter:function(){
				return settings.allowEditOfBaseAmount && settings.cpbEnabled.formatter.apply(this,arguments);
			}
		},
		press:$.proxy(this.actionConvert,this)
	});
	this.actionButtonStateProxy=$.proxy(this.actionButtonState,this);
	this.actionConvertProxy=$.proxy(this.actionConvert,this);
	var infoImage=new sas.hc.ui.commons.Image({src:sas.icons.HC.INFORMATIONSTATUS});
	var infoMessage=new sas.hc.ui.commons.TextView({text:rb.getText("currency.convert.instruction.txt")});
	this.infoBar=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		visible:{
			parts:settings.cpbEnabled.parts,
			formatter:function(){
				return settings.allowEditOfBaseAmount && settings.cpbEnabled.formatter.apply(this,arguments);
			}
		},
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row,
		items:[infoImage,infoMessage]
	});
	//COMPOSITES
   sas.risk.common.utils.CompositeUtil.addComposite(this,"amountField",this.amountInput,"value");
   sas.risk.common.utils.CompositeUtil.addComposite(this,"baseAmountField",this.baseAmountInput,"value");
   sas.risk.common.utils.CompositeUtil.addComposite(this,"allowEditOfBaseAmount",this.baseAmountInput,"editable");
   sas.risk.common.utils.CompositeUtil.addComposite(this,"currencyCodes",this.amountCurrencyCd,"items");
   sas.risk.common.utils.CompositeUtil.addComposite(this,"currencyCdField",this.amountCurrencyCd,"selectedKey");
   sas.risk.common.utils.CompositeUtil.addComposite(this,"baseCurrencyCode",this.baseCurrencyCd,"text");
   //COMPONENT FIX TODO:Fix ABaseControl->AComponent/AField/ALayout usage of applySettings
	if(this.supportsLayout){
		this.prepLayout();
	}
	this.wireOnChange(this.getProperty("cpbOnChangeFn"));
};
sas.risk.rgf.components.CurrencyInput.prototype.onBeforeRendering=function(){
	this.amountInput.addAriaLabelledBy(this.getCpbLabelControl());
	var readonly=!this.getCpbEnabled();
	var editBase=this.getAllowEditOfBaseAmount();
	this.actionButtonState();
	if(!readonly){
		if(editBase){//TODO: move this out to applySettings
			this.amountInput.detachLiveChange(this.actionButtonStateProxy);
			this.amountInput.attachLiveChange(this.actionButtonStateProxy);
			this.baseAmountInput.detachLiveChange(this.actionButtonStateProxy);
			this.baseAmountInput.attachLiveChange(this.actionButtonStateProxy);
		}
	}
};
sas.risk.rgf.components.CurrencyInput.prototype.setCPBValue = function(value) {
	if(typeof value === "number"){
		var type=this.amountInput.getBinding("value").getType();
		value=type.formatValue(value,"string");
	}
	this.setAmountField(value);
	this.actionChange("AMT");
};
sas.risk.rgf.components.CurrencyInput.prototype.actionChange = function(source) {
	var readonly=!this.getCpbEnabled();
	if(!readonly && !this.getAllowEditOfBaseAmount()){//readonly base amount
		this.baseAmountInput.setValue("");
		this.actionConvert();
	}
};
sas.risk.rgf.components.CurrencyInput.prototype.actionConvert=function(){
	var rawAmt=this.amountInput.getValue();
	var currencyCode=this.amountCurrencyCd.getSelectedKey();
	var date=(!this.getUpdateConversionDt())?this.getConversionDtField():null;
	var rawBaseAmount=this.getAllowEditOfBaseAmount()?this.baseAmountInput.getValue():"";
	var that=this;
	sas.risk.rgf.api.UtilsAPI.convertCurrency(rawAmt,currencyCode,rawBaseAmount,date).done(function(formatted){
		that.setBaseAmountField(formatted.baseAmt);
		that.setAmountField(formatted.amt);
		that.convertButton.setEnabled(false);
		that.warnBar.setVisible(false);
		//RGFRMWRK-666 Currency input field state needs to back to normal
		that.baseAmountInput.setValueState(ValueState.None);
		that.amountInput.setValueState(ValueState.None);		
		that.fireChange(formatted.amtDouble);
	}).fail(function(message){
		that.warnMessage.setText(message);
		that.warnBar.setVisible(true);
	});
};
sas.risk.rgf.components.CurrencyInput.prototype.actionButtonState=function(){
	var amount=this.amountInput.getValue();
	var baseAmount=this.baseAmountInput.getValue();
	this.convertButton.setEnabled(amount==="" || baseAmount==="");
};
sas.risk.rgf.components.CurrencyInput.prototype._createComponent = function() {
	var rb=sas.risk.rgf.util.RGFUtil.getBundle();
	var top=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row,
		items:[this.amountInput,this.amountCurrencyCd,this.convertButton]
	}).addStyleClass("RGFCurrencyTop");
	var bottom=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row,
		items:[this.baseAmountInput,this.baseCurrencyCd]
	}).addStyleClass("RGFCurrencyBottom");
	var warnImage=new sas.hc.ui.commons.Image({src:sas.icons.HC.WARNINGSTATUS});
	this.warnMessage=new sas.hc.ui.commons.TextView({text:""});
	this.warnBar=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row,
		items:[warnImage,this.warnMessage],
		visible:false
	});
	var messages=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		items:[this.infoBar,this.warnBar]
	}).addStyleClass("RGFCurrencyMessages");
	var comp=new sas.risk.common.layout.flexlayout.FlexLayout({
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		items:[top,bottom,messages]
	}).addStyleClass("RGFCurrencyFields");
   return comp;
};
sas.risk.rgf.components.CurrencyInput.prototype.getCPBControl = function() {
	return this;
};
//STATICS
sas.risk.rgf.components.CurrencyInput.create=function(fieldJson, modelId, cpbForm) {
	jQuery.sap.require("sas.risk.rgf.type.CurrencyType");
	jQuery.sap.require("sas.risk.rgf.type.CurrencyCodeType");
	var settings=$.extend(
		sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm),
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
			conversionDtField : sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.conversionDtField),
			allowEditOfBaseAmount : !!fieldJson.allowEditOfBaseAmount,
			updateConversionDt : !!fieldJson.updateConversionDt
		}
	);
	var currencyInput=new sas.risk.rgf.components.CurrencyInput(settings);
	currencyInput.bindProperty("currencyCdField",{
		path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId, fieldJson.currencyCdField),
		type: 'sas.risk.rgf.type.CurrencyCodeType',
		formatOptions:{field:currencyInput,baseCurrencyCode:fieldJson.meta.baseCurrencyCode}
	});
	currencyInput.bindProperty("amountField",{
		path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId, fieldJson.amountField),
		type: 'sas.ui.cpb3.model.type.CpbNumber',
		formatOptions: {
			decimals:2
		}
	});
	currencyInput.bindProperty("baseAmountField",{
		path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId, fieldJson.baseAmountField),
		type: 'sas.ui.cpb3.model.type.CpbNumber',			
		formatOptions: {
			decimals:2
		}
	});
	var ccArray=fieldJson.meta.currencyCodes;
	var i,currencyCd;
	for(i=0;i<ccArray.length;i++){
		currencyCd=ccArray[i];
		currencyInput.addCurrencyCode(new sap.ui.core.Item({
			text:currencyCd,
			key:currencyCd
		}));	
	}
	currencyInput.setBaseCurrencyCode(fieldJson.meta.baseCurrencyCode);
	return currencyInput;
};
sas.risk.common.utils.CompositeUtil.mixinAggregationSupport(sas.risk.rgf.components.CurrencyInput);
sas.risk.common.utils.CompositeUtil.mixinEventSupport(sas.risk.rgf.components.CurrencyInput);