jQuery.sap.declare("sas.risk.rgf.components.DelegateesComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.rgf.widget.RGFTable");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.rgf.api.DelegationAPI");
jQuery.sap.require("sas.risk.slimtable.util.RowActionUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.DelegateesComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata : {
        properties : {
            meta:{type : "any"},
            visibleRows:{type:"int"},
            data:{type:"any"},
            formattingKey:{type:"any"}
        }
    },
	constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.DelegateesComponent);

sas.risk.rgf.components.DelegateesComponent.prototype._createComponent = function() {
	var meta=this.getMeta(); 
	var that=this;
	this.table = new sas.risk.rgf.widget.RGFTable({
		formattingKey:this.getFormattingKey(),
		clientSide:true,
		meta : meta,
		title : meta.title,
		data : $.proxy(this.fetchData, this),
		visibleRows : this.getVisibleRows(),
		plugins:[
         this.getActionsPlugin(),
			new sas.risk.slimtable.plugins.Search(),
			new sas.risk.slimtable.plugins.Filter(),
			new sas.risk.slimtable.plugins.Counter(),
			new sas.risk.slimtable.plugins.Pager()
		],
		rowClick:function(e){
			var rowData=this.getSource().getData()[e.getParameter("rowIdx")];
			that.editDelegation(rowData);
		}
	});
	this.table.addAriaLabelledBy(this.table.titleField);
	var actionColumn=sas.risk.slimtable.util.RowActionUtil.createActionColumn($.proxy(this.createActionMenuItems,this),this);
	this.table.addActionColumn(actionColumn);
	return this.table;
};
//TABLE STUFF
sas.risk.rgf.components.DelegateesComponent.prototype.getActionsPlugin=function(){
   var that=this;
	var createNewButton=new sas.risk.common.widgets.Button({
      press:function(){
      	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
      	var viewData={
   			delegateLabel:that.getColumnLabel("delegateName"),
   			startDateLabel:that.getColumnLabel("startDate"),
   			endDateLabel:that.getColumnLabel("endDate"),
   			title:rb.getText("delegation.add.title.txt"),
   			serviceId:that.getCpbForm().getServiceId()
			};
	   	sas.risk.rgf.util.DialogManager.open("AddEditDelegateDialog",viewData,that.actionSaveDelegation,that);
      },
      icon:sas.icons.HC.NEWSTARBURST
   }).addStyleClass("QueryTableCreateButton");
   var tableTopActionsPlugin=new sas.risk.slimtable.plugins.TablePlugin({
     	position: sas.risk.slimtable.TableLayout.TOP_END,
   	items:[createNewButton]
   });
   tableTopActionsPlugin.handleDisabled=function(){
   	that.createNewButton.setVisible(false);
   };
   return tableTopActionsPlugin;
};
sas.risk.rgf.components.DelegateesComponent.prototype.createActionMenuItems=function(extra){
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var that=this;
	
	var rowData = extra.rowData;
	
	var today=new Date();
	today.setHours(0);
	today.setMinutes(0);
	today.setSeconds(0);
	today.setMilliseconds(0);
	
	var readonly = new Date(rowData.startDate).getTime()<today.getTime() && new Date(rowData.endDate).getTime()<today.getTime();
	var editItem;
	if (!readonly)
	{
		editItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
			text : rb.getText("delegation.action.edit.txt"),
			callback : function(){
				var rowData=this.getParent().getPayload().rowData;
				that.editDelegation(rowData);
			}
		});
	}
	var deleteItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
		text : rb.getText("delegation.action.delete.txt"),
		callback : function(){
			var rowData=this.getParent().getPayload().rowData;
			sas.hc.m.MessageDialog.warning({
				text:rb.getText("delegation.action.delete.confirmation.txt"),
				title:rb.getText("delegation.action.delete.confirmation.title.txt"),
				buttonActions:[sas.hc.m.MessageDialogAction.Delete,sas.hc.m.MessageDialogAction.Cancel],
				buttonPress:$.proxy(that.actionDeleteDelegation,that,rowData)
			});
		}
	});
	return readonly ? [deleteItem] : [editItem, deleteItem];
};
sas.risk.rgf.components.DelegateesComponent.prototype.fetchData = function(query) {
	var cloneData=$.extend([],this.getData());
	var results={
		items:cloneData,
		start:0,
		count:cloneData.length,
		limit:cloneData.length
	};
	return $.Deferred().resolve(results);
};
sas.risk.rgf.components.DelegateesComponent.prototype.editDelegation=function(rowData){
	var that=this;
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var viewData={
		delegationId:rowData.id,
		delegate:{displayNm:rowData.delegateName,userId:rowData.delegateId},
		startDate:new Date(rowData.startDate),
		endDate:new Date(rowData.endDate),
		delegateLabel:that.getColumnLabel("delegateName"),
		startDateLabel:that.getColumnLabel("startDate"),
		endDateLabel:that.getColumnLabel("endDate"),
		title:rb.getText("delegation.edit.title.txt"),
		serviceId:that.getCpbForm().getServiceId()
	};
	sas.risk.rgf.util.DialogManager.open("AddEditDelegateDialog",viewData,that.actionSaveDelegation,that);
};
//ACTIONS
sas.risk.rgf.components.DelegateesComponent.prototype.actionDeleteDelegation=function(rowData,event){
	var action=event.getParameter("origin").getMessageDialogAction();
	if(action==="DELETE"){
		var that=this;
		var data=this.getData();
		this.setBusy(true);
		var item=_.find(data,function(row){return row.id===rowData.id;});
		sas.risk.rgf.api.DelegationAPI.deleteDelegation(item.id).done(function(){
			var index=data.indexOf(item);
			data.splice(index,1);
			that.table.doRefresh();
		}).always(function(){
			that.setBusy(false);
		});
	}
};
sas.risk.rgf.components.DelegateesComponent.prototype.actionSaveDelegation=function(e){
	var that=this;
	var dialog=e.getParameter("dialog");
	var moveDate=function(date,hourOfDay,minute,second){
		var d=new Date(date.getTime());
		d.setHours(hourOfDay);
		d.setMinutes(minute);
		d.setSeconds(second);
		return d;
	};
	var currentUser=sas.risk.rgf.api.CommonAPI.getUserDetails();
	var delegationId=e.getParameter("delegationId");
	var delegate=e.getParameter("delegate");
	var startDate=e.getParameter("startDate");
	var endDate=e.getParameter("endDate");
	if(delegationId===undefined){//ADD case
		var newRow={
			delegateId:delegate.userId,//"asolomon"
			delegateName:delegate.displayNm,//"Ashok Solomon"
			delegatorId:currentUser.getUserId(),//"rdt0003"
			delegatorName:currentUser.getName(),//"rdt0003-Adminstration"
			startDate:moveDate(startDate,0,0,0).toISOString(),//2017-06-12T04:00:00.000Z
			endDate:moveDate(endDate,23,59,59).toISOString()//2017-06-13T03:59:59.000Z
		};
		sas.risk.rgf.api.DelegationAPI.addDelegation(newRow.delegateId,newRow.startDate,newRow.endDate).done(function(delegation){
			newRow.id=delegation.key;
			that.getData().push(newRow);
			that.table.doRefresh();
		}).always(function(){
			dialog.close();
		});
	}else{//EDIT case
		var data=this.getData();
		var editRow=_.find(data,function(row){return row.id===delegationId;});
		editRow.delegateId=delegate.userId;
		editRow.delegateName=delegate.displayNm;
		editRow.startDate=moveDate(startDate,0,0,0).toISOString();
		editRow.endDate=moveDate(endDate,23,59,59).toISOString();
		sas.risk.rgf.api.DelegationAPI.updateDelegation(delegationId,editRow.delegateId,editRow.startDate,editRow.endDate).done(function(){
			that.table.doRefresh();
		}).always(function(){
			dialog.close();
		});
	}
};
//HELPERS
sas.risk.rgf.components.DelegateesComponent.prototype.getColumnLabel=function(colName){
	var meta=this.getMeta();
	var column=_.find(meta.columns,function(col){return col.rawName===colName;});
	return column.label;
};
//STATICS
sas.risk.rgf.components.DelegateesComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	return new sas.risk.rgf.components.DelegateesComponent($.extend(
		settings,
		{
			meta : fieldJson.meta,
			visibleRows : fieldJson.visibleRows,
			formattingKey :sas.risk.rgf.util.RGFUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
			data : fieldJson.data
		}
	));
};