jQuery.sap.declare("sas.risk.rgf.components.DimensionNodeComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.ui.commons.Tree");
jQuery.sap.require("sas.hc.ui.commons.TreeNode");
jQuery.sap.require("sas.risk.rgf.biz.DimNodeRegistry");
jQuery.sap.require("sas.risk.rgf.biz.Feature");
jQuery.sap.require("sas.risk.rgf.widget.RGFTable");
jQuery.sap.require("sas.risk.slimtable.widget.Item");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.ui.cpb3.model.format.CpbDateFormat");
jQuery.sap.require("sap.ui.commons.TextView");
jQuery.sap.require("sas.risk.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.rgf.biz.SecurityKey");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.DimensionNodeComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata : {
		properties : {
			"dimension" : {type:"string"},
			"rootVisible" : {type:"boolean"},
			"leafOnly" : {type:"boolean"},
			"feature" : {type:"string"},
			"secureAction" : {type:"string"},
			"meta" : {type:"any"},
			"cpbWidth" : {type : "sap.ui.core.CSSSize"},
			"cpbHeight" : {type : "sap.ui.core.CSSSize"},
			formattingKey:{type:"any"}
		},
		aggregations:{
      	"headerPlugin":{type:"sas.risk.common.layout.flexlayout.FlexLayout", multiple:false}
      },
		events : {
			change : {},
			select : {
				parameters : {
					dimNode : {type : "any"} //DimNode
				}
			}
		}
	},
	constructor : function() {
		this.supportsLayout=true;
		this.supportsLabel=true;
		this.supportsAlign=true;
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
	},
	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.DimensionNodeComponent);

sas.risk.rgf.components.DimensionNodeComponent.prototype.init=function(){
	this.dimNodeContainer=this.createDimNodeContainer();
	this.headerPluginContainer=new sas.risk.common.layout.flexlayout.FlexLayout();
//	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbWidth",this.dimNodeContainer,"width");
//	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbHeight",this.dimNodeContainer,"height");
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.onBeforeRendering=function(){
	if(!this.initialized){
		this.initialize();
	}
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.setHeaderPlugin=function(headerPlugin){
	this.setAggregation("headerPlugin",headerPlugin);
	this.headerPluginContainer.addItem(headerPlugin);
	return this;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.initialize=function(){
	var that=this;
	this.securityKey=this.createSecurityKey();
	this.hideInactive=true;
	this.rootDimNodes=[];
	this.systemFeature=sas.risk.rgf.biz.Feature.parseFromJson(this.getMeta().system);
	//If dimension is passed then use it. Otherwise use the system feature to work with all possible dimensions.
	var dimension=this.getDimension();
	if(dimension){ //it's the dimension's tagged pointer handle
		var split=dimension.split("|");
		var dimensionName=split[0].toLowerCase();
		var key=split[1];
		var rootDimNode=sas.risk.rgf.biz.DimNodeRegistry.getNode(dimensionName,key,this.getSecurityKey());
		that.rootDimNodes.push(rootDimNode);
	}else{
		var dimensions=this.systemFeature.getDimensions();
		dimensions=_.sortBy(dimensions,function(dimension){ //sort the dimensions by display label to match with past behavior
			return dimension.getDimRootLabel();
		});
		_.each(dimensions,function(dimension){
			var dimNode=sas.risk.rgf.biz.DimNodeRegistry.getNode(dimension.getDimName(),-1,that.getSecurityKey());
			that.rootDimNodes.push(dimNode);
		});
	}
	//If it is a draft scenario initiated from Dimension Browser, then make use of any init parentPath set during the course
	var model=this.getCpbForm().getCpbService().createModelManagerObject(this);
	var isDraft=model.getUIContextValue("NEVERBEENSAVED");
	if(isDraft && typeof(Storage)!==undefined){
		var initParentPath=JSON.parse(sessionStorage.getItem("init_parentPath"));
		if(initParentPath){
			this.setCpbValue(initParentPath);
			sessionStorage.removeItem("init_parentPath");//clean up
		}
	}
	if(this.getCpbEnabled()){
		this.createEditable();
	}else{
		this.createReadOnly();
	}
	if(this.getCpbValue()){
		this.handleValue();
	}
	this.initialized=true;
};

sas.risk.rgf.components.DimensionNodeComponent.prototype.wireOnChange=function(onChangeFn){
	if (onChangeFn) {
		this.attachChange(onChangeFn);
	}
};

sas.risk.rgf.components.DimensionNodeComponent.prototype._createComponent=function(){
	this.addStyleClass("RGFDimensionNode");
	return this.dimNodeContainer;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createEditable=function(){
	var editContainer=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column
	});//.addStyleClass("RGFDimNodeContainer");
	editContainer.addItem(this.createHeader()); 
	editContainer.addItem(this.createBody());
	editContainer.addItem(this.createFooter());
	this.dimNodeContainer.addItem(editContainer);
	this.refreshTree();
	this.refreshSearch();
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createReadOnly=function(){
	this.readOnlyTree=new sap.ui.commons.TextView().addStyleClass("RGFDimNodeReadOnly");
	this.dimNodeContainer.addItem(this.readOnlyTree);
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createDimNodeContainer=function(){
	var container=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column
	}).addStyleClass("RGFDimNodeContainer");
	return container;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createHeader=function(){
	this.header=new sas.risk.common.layout.flexlayout.FlexLayout({
		width: "100%",
		items: [this.createSearchLayout(),this.headerPluginContainer],
		justifyContent: sas.risk.common.layout.flexlayout.FlexLayoutJustifyContent.SpaceBetween
	}).addStyleClass("RGFDimNodeHeader");
	return this.header;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createBody=function(){
	this.innerLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:this.getCpbWidth() || "100%",
		height:this.getCpbHeight()
	}).addStyleClass("RGFDimNodeBody");
	var treeLayout=this.createTreeLayout();
	var searchTable=this.createSearchTable();
	this.innerLayout.addItem(treeLayout);
	this.innerLayout.addItem(searchTable);
	return this.innerLayout;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createFooter=function(){
	var footerLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		items:[new sas.hc.m.CheckBox({
         	text: this.getMeta().labels.hideInactive,
         	selected: true,
         	select: $.proxy(this.actionHideInactive,this)
			})
		]
	});
	return footerLayout;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createSearchLayout=function(){
	var rb=sas.risk.rgf.util.RGFUtil.getBundle();
	this.searchField=new sas.risk.slimtable.widget.SearchField({
		placeholder:rb.getText("dimloc.search.placeholder.txt"),
		search:$.proxy(this.actionSearch,this)
	});
   this.searchLabel = new sap.ui.commons.Label().addStyleClass("RGFDimNodeSearchLabel");
   this.searchSelect = new sas.risk.slimtable.widget.Select().addStyleClass("RGFDimNodeSearchList");
   var searchLayout = new sas.risk.common.layout.flexlayout.FlexLayout({
//   	width:"100%",
   	alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
      items: [this.searchField,this.searchLabel,this.searchSelect]
   }).addStyleClass("RGFDimNodeSearchLayout");
   return searchLayout;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createTreeLayout=function(){
	this.tree=new sas.hc.ui.commons.Tree({
		width:"100%",
		showHeader:false,
		selectionMode:sap.ui.commons.TreeSelectionMode.Single
	});
	return this.tree;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createTreeNode=function(text,dimNode){
	var that=this;
	var treeNode=new sas.hc.ui.commons.TreeNode({
		text: dimNode.getLocalizedLabel(),
		expanded: false,
		selectable: true,
		hasExpander: !dimNode.getLeaf(),
		customData:[{
	   	key:"key",
	   	value:dimNode
	   }],
		toggleOpenState:function(e){
			var expand=e.getParameter("opened");
			if(expand && this.getNodes().length===0){
				that.actionExpandNode(this.getCustomData()[0].getValue(),this);
			}
		},
		selected:function(e){
			that.selectedDimNode=this.getCustomData()[0].getValue();
			that.selectedTreeNode=this;
			that.actionDimNodeSelection(this);
		}
	});
	return treeNode;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.createSearchTable=function(){
	var rb=sas.risk.rgf.util.RGFUtil.getBundle();
	this.searchTable=new sas.risk.rgf.widget.RGFTable({
		formattingKey:this.getFormattingKey(),
		showSelectionControls:false,
		meta: {
			columns:[
				{	
					filterable: false,
					label: rb.getText("dimloc.search.table.col.name.txt"),
					name: "name",
					quickSearch: false,
					sortPreference: 9999,
					sortable: true,
					type: "java.lang.String",
					userDisplayOrder: 0,
					userVisible: true,
					visible: true
				},
				{	
					filterable: false,
					label: rb.getText("dimloc.search.table.col.path.txt"),
					name: "path",
					quickSearch: false,
					sortPreference: 9999,
					sortable: true,
					type: "java.lang.String",
					userDisplayOrder: 1,
					userVisible: true,
					visible: true
				}
			]
		},
		data: $.proxy(this.actionSearchTable,this),
		visibleRows: 5,
		plugins:[new sas.risk.slimtable.plugins.Counter(),
		         new sas.risk.slimtable.plugins.Pager()
		]
	});
	this.searchTable.attachRowClick($.proxy(this.actionSearchTableRowClick,this));
	this.searchTable.setVisible(false);
	return this.searchTable;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.handleValue=function(){ //shows the parentPath value selected in the tree or displayed in the readonly textView 
	var that=this;
	var securityKey=this.getSecurityKey();
	var def=$.Deferred();
	var parentPath=this.getCpbValue();
	if(parentPath){//If the parent path has name/s then pop the last one which is the parent node i.e. the selection for Dimension Node component
		var split=parentPath.split("|");
		var dimName=split[0].toLowerCase();
		var key=split[1];
		sas.risk.rgf.biz.DimNodeRegistry.loadNodes(dimName,key,securityKey)
		.done(function(items){
			var dimNode=sas.risk.rgf.biz.DimNodeRegistry.getNode(dimName,key,securityKey);
			that.selectedDimNode=dimNode;
			if(that.getCpbEnabled()){
				//RGFRMWRK-780 avoid calling actionExpanded() twice
				//that.refreshTreeSelection(); 
				that.refreshRootTreeNodeSelection();
			}else{
				var rootDimNode=sas.risk.rgf.biz.DimNodeRegistry.getNode(dimNode.getDimensionName(),-1,securityKey);
				var rootLocalizedName=rootDimNode.getName();
				var parentPath=dimNode.getParentPath(rootLocalizedName);
				that.readOnlyTree.setText(parentPath);
			}
		});
	}
};
//REFRESH
sas.risk.rgf.components.DimensionNodeComponent.prototype.refreshSearch=function(){
	var that=this;
	var securityKey=this.getSecurityKey();
	var dimensions=[];
	_.each(this.rootDimNodes,function(dimNode){
		dimensions.push(that.systemFeature.getDimension(dimNode.getDimensionName()));
	});
	dimensions=_.sortBy(dimensions,function(dimension){ //sort the dimensions by display label
		return dimension.getDimRootLabel();
	});
	this.searchSelect.destroyItems();
	_.each(dimensions,function(dimension){
		var dimNode=sas.risk.rgf.biz.DimNodeRegistry.getNode(dimension.getDimName(),-1,securityKey);
		that.searchSelect.addItem(new sas.risk.slimtable.widget.Item({
			text:dimension.getDimLabel(),
			payload:dimNode
		}));
	});
	if(dimensions.length===1){
		this.searchLabel.setText(dimensions[0].getDimLabel());
		this.searchSelect.setVisible(false);
	}else{
		this.searchLabel.setVisible(false);
	}
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.refreshTree=function(){
	var that=this;
	this.tree.destroyNodes();
	if(this.getDimension()){ //it's the dimension's tagged pointer handle
		var rootDimNode=this.rootDimNodes[0];
		var treeNode=that.createTreeNode(rootDimNode.getName(),rootDimNode);
		that.tree.addNode(treeNode);
		that.actionExpandNode(rootDimNode,treeNode);
	}else{
		_.each(this.rootDimNodes,function(dimNode){
			that.tree.addNode(that.createTreeNode(dimNode.getName(),dimNode));
		});
	}
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.refreshRootTreeNodeSelection=function() {
	var that = this;
	var securityKey=this.getSecurityKey();
	var treeNode = _.find(this.tree.getNodes(), function(node) {
		var treeDimNode = node.getCustomData()[0].getValue();
		return treeDimNode.equals(that.selectedDimNode);
	});
	// if treeNode is not a root, then just return; nothing to do
	if(this.selectedDimNode.getNodePath().length === 0) {
		treeNode.select();
		sas.risk.rgf.biz.DimNodeRegistry.loadNodes(this.selectedDimNode.getDimensionName(),null,securityKey).done(function(items) {
			var rootNode=sas.risk.rgf.biz.DimNodeRegistry.getNode(that.selectedDimNode.getDimensionName(), -1, securityKey);
			that.selectedDimNode = rootNode;
			var oldRootDimNodes = that.rootDimNodes;
			that.rootDimNodes = [];
			_.each(oldRootDimNodes, function(oldRootDimNode) {
				if(oldRootDimNode.dimName === rootNode.dimName) {
					that.rootDimNodes.push(rootNode);
				} else {
					that.rootDimNodes.push(oldRootDimNode);
				}
			});
			// re-sort the items and then refresh the tree
			that.rootDimNodes = _.sortBy(that.rootDimNodes, function(dimension) {
				return dimension.getDimRootLabel();
			});
			that.refreshTree();
			
			var rootTreeNode = _.find(that.tree.getNodes(), function(node) {
				var treeDimNode=node.getCustomData()[0].getValue();
				return treeDimNode.equals(that.selectedDimNode);
			});
			that.selectedTreeNode = rootTreeNode;
			rootTreeNode.select();
		});
	}
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.refreshTreeSelection=function(refreshTree){
	var that=this;
   var deferred = $.Deferred();
   
   try {
		if(refreshTree){
			this.refreshTree();
		}
		//If there is a selected node, then open the required hierarchy with the right node selected
		if(this.selectedDimNode){
			//If the path is empty that means it is one of the root nodes, find locally, select, and refresh
			if(this.selectedDimNode.getNodePath().length===0){
				this.refreshRootTreeNodeSelection();
			}
			
			// Find the top level dimension node.
			var treeDimNode = undefined;
			var treeNode = _.find(that.tree.getNodes(), function(node) {
				treeDimNode = node.getCustomData()[0].getValue();
				return treeDimNode.dimName === that.selectedDimNode.dimName;
			});
			
			if (treeNode) {
				that.actionExpandNode(treeDimNode, treeNode).done(function() {
				
					// We will use the localizedPath to navigate the tree to the node searched for
					var pathDepth = that.selectedDimNode.localizedPath.length - 1;
					
					// Function to find, select and focus on child node that matches the given value
					var selectAndFocusNode = function(aNode, pathValue) {
						var selectedTreeNode = _.find(aNode.getNodes(), function(node) {
							var nodeText = node.getText();
							return pathValue === nodeText;
						});
						selectedTreeNode.select();
						setTimeout(function() {
						    selectedTreeNode.focus();
						}, 100);
					}; // selectAndFocusNode
					
					// Function to recurse down the tree to the desired node.
					var goDeeper = function (idx, pathDepth, path, tNode) {
						var childTreeNode = _.find(tNode.getNodes(), function(node) {
							var nodeText = node.getText();
							return path[idx] === nodeText;
						});
						if (childTreeNode) {
							that.actionExpandNode(childTreeNode.getCustomData()[0].getValue(), childTreeNode).done(function() {
								if (++idx < pathDepth) {
									goDeeper(idx, pathDepth, path, childTreeNode);
								} else {
									selectAndFocusNode(childTreeNode, path[idx]);
								}
							});
						}
					}; // goDeeper
					
					// If what is searched for is buried under several layers of the tree, then we
					// will recurse into the tree to find and display it.
					if (treeNode && pathDepth > 0) {
						goDeeper(0, pathDepth, that.selectedDimNode.localizedPath, treeNode);
					} else {
						// If what is searched for is in the first level of the dimension, 
						// select and display it.
						selectAndFocusNode(treeNode, that.selectedDimNode.localizedPath[0]);
					}
					
				});
			} // if treeNode

		} else {
			deferred.resolve();
		}
   } catch(e) {
   	deferred.resolve();
   }
	return deferred;
};

//ACTIONS
sas.risk.rgf.components.DimensionNodeComponent.prototype.actionDimNodeSelection=function(treeNode){
	this.setProperty("cpbValue",this.selectedDimNode.getHandle());
	this.fireSelect({"dimNode":this.selectedDimNode, "treeNode":treeNode});
	this.fireChange();
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.actionExpandNode=function(dimNode,treeNode){
	var that=this;
   var deferred = $.Deferred();
   try {
		var securityKey=this.getSecurityKey();
		var hideInactiveFilter=this.getHideInactiveFilter();
		dimNode.loadChildren(securityKey,hideInactiveFilter).done(function(children){
			var i,childDimNode,childTreeNode;
			children = _.sortBy(children, function(child) {
				return child.getLocalizedLabel();
			});
			for(i=0;i<children.length;i++){
				childDimNode=children[i];
				childTreeNode=that.createTreeNode(childDimNode.getName(),childDimNode);
				treeNode.addNode(childTreeNode);
			}
			treeNode.setExpanded(true);
			deferred.resolve();
		});
   } catch(e) {
   	deferred.resolve();
   }
	return deferred;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.actionSearch=function(e){
	if(e.getParameter("query")){
		this.tree.setVisible(false);
		this.searchTable.setVisible(true);
		this.searchTable.doRefresh();
	}else{
		this.tree.setVisible(true);
		this.searchTable.setVisible(false);
	}
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.actionSearchTable=function(query){
	var def=$.Deferred();
	if(this.searchField.getValue()){
		var searchText=this.searchField.getValue();
		var dimNode=this.searchSelect.getSelectedItem().getPayload();
		var dimension=this.systemFeature.getDimension(dimNode.getDimensionName());
		var securityKey=this.getSecurityKey();
		var hideInactiveFilter=this.getHideInactiveFilter();
		dimension.search(searchText,query,securityKey,hideInactiveFilter).done(function(nodes,count,start){
			var json={count:count,start:start,items:[]};
			var node,i;
			for(i=0;i<nodes.length;i++){
				node=nodes[i];
				json.items.push({name:node.getName(),path:node.getLocalizedPath(),payload:node});
			}
			def.resolve(json);
		});
	}else{
		def.resolve({items:[],count:0,start:0});
	}
	return def;
};

sas.risk.rgf.components.DimensionNodeComponent.prototype.actionSearchTableRowClick=function(e){
	var that = this;
	var rowData=this.searchTable.getSource().getData()[e.getParameter("rowIdx")];
	this.selectedDimNode=rowData.payload;
	if (this.selectedDimNode) {
		this.setProperty("cpbValue",this.selectedDimNode.getHandle());
		this.fireSelect({"dimNode":this.selectedDimNode, "treeNode":undefined});
		this.fireChange();
	}
	this.tree.setVisible(true);
	this.searchTable.setVisible(false);
	
   this.refreshTreeSelection(true);
};

sas.risk.rgf.components.DimensionNodeComponent.prototype.actionHideInactive=function(e){
	this.hideInactive=e.getParameter("selected");
	this.refreshTree();
};

//HELPERS
sas.risk.rgf.components.DimensionNodeComponent.prototype.createSecurityKey=function(){
	var feature=this.getFeature();
	var capability=this.getSecureAction();
	if(feature&&capability){
		return new sas.risk.rgf.biz.SecurityKey(feature,capability);
	}
	return null;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.getSecurityKey=function(){
	return this.securityKey;
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.getHideInactiveFilter=function(){
	return this.hideInactive ? {} : {"includeInactive":true};
};
sas.risk.rgf.components.DimensionNodeComponent.prototype.findLink=function(links,relName){
	return _.find(links,function(link){
		return link.rel===relName;
	});
};

sas.risk.rgf.components.DimensionNodeComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	return new sas.risk.rgf.components.DimensionNodeComponent($.extend(
		settings,
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
			dimension : fieldJson.dimension,
			rootVisible : fieldJson.rootVisible,
			leafOnly : fieldJson.leafOnly,
			feature : fieldJson.feature,
			secureAction : fieldJson.secureAction,
			meta : fieldJson.meta,
			cpbWidth : fieldJson.cpbWidth,
			formattingKey :sas.risk.rgf.util.RGFUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
			cpbHeight : fieldJson.cpbHeight
		}
	));
};