jQuery.sap.declare("sas.risk.rgf.components.EditableMatrix");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.rgf.biz.Link");
jQuery.sap.require("sas.risk.rgf.widget.RGFTable");
jQuery.sap.require("sas.ui.cpb3.CPBForm");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.risk.slimtable.plugins.TablePlugin");
jQuery.sap.require("sas.risk.slimtable.plugins.TableOptions");
jQuery.sap.require("sas.risk.slimtable.plugins.Counter");
jQuery.sap.require("sas.risk.rgf.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.CustomizationInfoItem");
jQuery.sap.require("sas.risk.rgf.util.LinksUtil");
jQuery.sap.require("sas.risk.rgf.util.InPlaceEditUtil");
jQuery.sap.require("sas.risk.common.utils.CompositeUtil");
jQuery.sap.require("sas.risk.slimtable.util.RowActionUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.EditableMatrix", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata : {
		properties : {
			linkContainer:{type:"any"},
			nestedSaves:{type:"any"},
			nestedDeletes:{type:"any"},
			sourceSystemCode:{type:"string"},
			linkTypeId:{type:"string"},
			linkTypeKey:{type:"int"},
			collectionName:{type:"string"},
			meta : {type : "any"},
			sourceObjectKey:{type:"int"},
			visibleRows : {type:"int"},
			bizObjLabel:{type:"string"},
			showAddRowLink:{type:"boolean",defaultValue:false},
			showEditRowLink:{type:"boolean",defaultValue:false},
			showDeleteRowLink:{type:"boolean",defaultValue:false},
			showColumnsLink:{type:"boolean",defaultValue:false},
			showRowsCounter:{type:"boolean",defaultValue:false},
			errors:{type:"any"},
			formattingKey:{type:"any"}
		},
		events:{
			change:{}
		}
	},
	constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		this.cpbForms={};
 		this.handleToKey={};
 		this.savedChildren=[];
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.addStyleClass("rgfEMTable");
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));

sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.EditableMatrix);

sas.risk.rgf.components.EditableMatrix.prototype.wireOnChange = function(onChangeFn) {
	if(onChangeFn){
		this.attachChange(onChangeFn);
	}
};
sas.risk.rgf.components.EditableMatrix.prototype.setLinks=function(value){
	this.links=value;
};
sas.risk.rgf.components.EditableMatrix.prototype.getLinks=function(){
	return this.links;
};
sas.risk.rgf.components.EditableMatrix.prototype.getSavedChildren=function(){
	return this.savedChildren;
};
sas.risk.rgf.components.EditableMatrix.prototype.onBeforeRendering=function(){
	if(this.getShowEditRowLink() || this.getShowDeleteRowLink()){	//action column
		var actionColumn=sas.risk.slimtable.util.RowActionUtil.createActionColumn($.proxy(this.createActionMenuItems,this),this);
		this.table.addActionColumn(actionColumn);
	}
};
sas.risk.rgf.components.EditableMatrix.prototype.fetchData = function() {
	return sas.risk.rgf.util.LinksUtil.fetchData(this);
};
//CPB STUFF
sas.risk.rgf.components.EditableMatrix.prototype.doCPBRefresh = function(){
	this.table.doRefresh();
};
sas.risk.rgf.components.EditableMatrix.prototype.setCpbValue = function(value) {
	this.mProperties["cpbValue"]=null;//force the set in the next line to update the array instance rather than the get bypassed by the clever "no change" check from ManagedObject.setProperty
   this.setProperty("cpbValue",value);
	this.setSourceObjectKey(this.getCpbForm().getCpbModelManager().getBusinessObjectRK());
	this.setLinkContainer(value);
};
sas.risk.rgf.components.EditableMatrix.prototype.setCpbEnabled = function(val) {
	this.setProperty("cpbEnabled",val)
	this.table.setEnabled(val);
	this.table.setClickScope(val?sas.risk.slimtable.ClickScope.Cell:sas.risk.slimtable.ClickScope.Row);
	if(val){
		this.table.removeStyleClass("rgfEMReadonly");
	}else{
		this.table.addStyleClass("rgfEMReadonly");
	}
};
sas.risk.rgf.components.EditableMatrix.prototype.fireChange = function() {
	var that=this;
	var checkUpdate=function(propName,forceUpdate){
		var bi=that.getBindingInfo(propName);
		if(bi && bi.binding && bi.binding.getModel()){
			bi.binding.getModel().checkUpdate(forceUpdate);
		}
	};
	checkUpdate("cpbValue",false);
	checkUpdate("nestedSaves",true);
	checkUpdate("nestedDeletes",true);
	this.fireEvent("change",arguments);
};
//TABLE STUFF
sas.risk.rgf.components.EditableMatrix.prototype._createComponent = function() {
	var multiSelection=this.getShowDeleteRowLink();
	this.table = new sas.risk.rgf.widget.RGFTable({
		formattingKey : this.getFormattingKey(),
		meta : this.getMeta(),
		data : $.proxy(this.fetchData, this),
		clientSide : true,
		showSelectionControls: multiSelection,
		multiSelect : multiSelection,
		visibleRows : this.getVisibleRows(),
		clickScope:sas.risk.slimtable.ClickScope.Cell
	});
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbLabel",this.table,"title");
	if(this.getShowAddRowLink() || this.getShowDeleteRowLink()){
		this.table.addPlugin(this.getTableTopActionsPlugin());
	}
	if(this.getShowColumnsLink()){
		this.table.addPlugin(         
			new sas.risk.slimtable.plugins.TableOptions({
		   	menuItems: this.getTableMenuItems()
		   })
		);
	}
	if(this.getShowRowsCounter()){
		this.table.addPlugin(new sas.risk.slimtable.plugins.Counter());
	}
	this.table.setRequired(this.getCpbRequired());
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbHelp",this.table,"help");
	this.table.attachRowSelect($.proxy(this.actionTableSelectionChanged,this));
	this.table.attachCellClick($.proxy(this.actionTableCellEdit,this));
	var source=this.table.getSource();
	var that=this;
	var origFormatter=source.formatCell;
	source.formatCell=function(rowIndex,colIndex,parent){
		origFormatter.apply(this,arguments);
		var data=source.data[rowIndex];
		if(data){
			var cpbForm=that.getChildCpbForm(data.key);
			var hasErrors=cpbForm.getCpbModelManager().hasErrors();
			$(parent.parentNode.parentNode).toggleClass("rgfEMRowError",hasErrors);
			var tm=that.table.getTableMeta();
			if(that.table.getShowActionColumn()){--colIndex;}//action column throws index off by 1
			var cm=tm.getVisibleColumns()[colIndex];
			if(cm){
				var control=cpbForm.getCPBControl(cm.getColRawName());
				if(control && control.getCpbEnabled() && sas.risk.rgf.components.EditableMatrix.IN_PLACE_CONTROLS.indexOf(control.getMetadata().getName())!==-1){
					var isEditing=!!(control.getDomRef());//is it being currently shown?
					parent.parentNode.className="SlimTableDataCell rgfEMCellEditable"+(isEditing?" rgfEMCellEditing":"");
				}else{
					parent.parentNode.className="SlimTableDataCell rgfEMCellNonEditable";
				}
			}
		}
	};
	this._label.addStyleClass("RGFTabTitleLabel");
	return this.table;
};
sas.risk.rgf.components.EditableMatrix.prototype.getTableTopActionsPlugin=function(){
	var that=this;
	this.addRowButton=new sas.risk.common.widgets.Button({
		icon:sas.icons.HC.ADDROW,
		enabled:true,
      press: $.proxy(this.actionAddRow,this)
   });
	this.deleteSelectedRowsButton=new sas.risk.common.widgets.Button({
		icon:sas.icons.HC.DELETEROW,
		enabled:false,
      press: $.proxy(this.actionDeleteSelectedRows,this)
   });
   this.tableTopActionsPlugin=new sas.risk.slimtable.plugins.TablePlugin({
     	position: sas.risk.slimtable.TableLayout.TOP_END
   });
   if(this.getShowAddRowLink()){
   	this.tableTopActionsPlugin.addItem(this.addRowButton);
   }
   if(this.getShowDeleteRowLink()){
   	this.tableTopActionsPlugin.addItem(this.deleteSelectedRowsButton);
   }
   this.tableTopActionsPlugin.handleDisabled=function(){
   	that.addRowButton.setVisible(false);
   	that.deleteSelectedRowsButton.setVisible(false);
   };
   return this.tableTopActionsPlugin;
};
sas.risk.rgf.components.EditableMatrix.prototype.getTableMenuItems=function(){
	var menuItems=[new sas.risk.rgf.widget.menu.ColumnsItem({serviceId:this.getCpbForm().getCpbService().getServiceId()})];
	var customizationInfoMeta=this.getMeta().customizationInfoMeta;
	if(customizationInfoMeta){
		var custInfoUrl=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/showCustomizationInfo/linkTypeId/"+this.getLinkTypeId()+"/linkTypeSSC/"+this.getSourceSystemCode()+"/sideOfLink/0";
		if (customizationInfoMeta.fileName) {
			custInfoUrl += "?fileName="+customizationInfoMeta.fileName;
		}
		menuItems.push(new sas.risk.rgf.widget.menu.CustomizationInfoItem({url:custInfoUrl,meta:customizationInfoMeta}));
	}
   return menuItems;
};
sas.risk.rgf.components.EditableMatrix.prototype.createActionMenuItems=function(extra){
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var that=this;
	var menuItems=[];
	if(this.getShowEditRowLink()){
		var editItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
			text : rb.getText("em.action.edit.txt"),
			callback : function(item){
				var rowData = item.rowData;
				var link=rowData.__LINK__;
				that.actionEditRow(link);
			}
		});
		menuItems.push(editItem);
	}
	if(this.getShowDeleteRowLink()){
		var deleteItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
			text : rb.getText("em.action.delete.txt"),
			callback : function(item){
				var rowData = item.rowData;
				var link=rowData.__LINK__;
				that.actionDeleteRows([link]);
			}
		});
		menuItems.push(deleteItem);
	}
	return menuItems;
};
//ACTIONS
sas.risk.rgf.components.EditableMatrix.prototype.actionTableCellEdit=function(e){
	var that=this;
	var cell=e.getParameter("cell");
	var rowIdx=e.getParameter("rowIdx");
	var colIdx=e.getParameter("colIdx");
	if(this.table.getShowActionColumn()){--colIdx;}//action column throws index off by 1
	var data=this.table.getSource().data[rowIdx];
	var cpbForm=this.getChildCpbForm(data.key);
	var link=sas.risk.rgf.util.LinksUtil.findLink(data.key,this.getLinks());
	var payload={cpbForm:cpbForm,link:link,table:this.table,source:this};
	cpbForm.attachOnChangeFinished(payload,sas.risk.rgf.util.LinksUtil.syncLinkWithCpbForm);
	$(cell).addClass("rgfEMCellEditing");
	sas.risk.rgf.util.InPlaceEditUtil.show(this.table,rowIdx,colIdx,cpbForm,cell).done(function(syncNeeded){
		$(cell).removeClass("rgfEMCellEditing");
		if(syncNeeded){
			sas.risk.rgf.util.LinksUtil.syncLinkWithCpbForm(null,payload);
		}
	});
};
sas.risk.rgf.components.EditableMatrix.prototype.actionTableSelectionChanged=function(){
	this.deleteSelectedRowsButton.setEnabled(!!this.table.table.getSource().getSelections().length);
};
sas.risk.rgf.components.EditableMatrix.prototype.actionDeleteSelectedRows=function(){
	var source=this.table.table.getSource();
	var indices=source.getSelections();
	var rows=source.getData();
	var i,row,linksToDelete=[];
	for(i=0;i<indices.length;i++){
		row=rows[indices[i]];
		linksToDelete.push(row.__LINK__);
	}
	var def=this.actionDeleteRows(linksToDelete,true);
	def.done(function(success){
		if(success){source.setSelections([]);}//clear selection unless you hit cancel
	});
};
sas.risk.rgf.components.EditableMatrix.prototype.actionDeleteRows=function(linksToDelete,viaSelection){
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var def=$.Deferred();
	var that = this;
	var callBackFn = function(event) {
		var action=event.getParameter("origin").getMessageDialogAction();
		if(action==="CANCEL"){def.resolve(false);return;}
		that.deleteSelectedRowsButton.setEnabled(false);
		sas.risk.rgf.util.LinksUtil.deleteLinks(linksToDelete,that,false/*immediateSave*/,true/*weakEntity*/);//update link container
		if (linksToDelete.length > 0) that.fireChange();
		that.table.doRefresh();
		def.resolve(true);
	};
	var labelKey=(viaSelection)?"em.action.confirmation.fmt.txt":"em.action.confirmation.txt";
	var labelArgs=[this.getBizObjLabel()];
	var titleKey="em.action.confirmation.title.fmt.txt";
	sas.hc.m.MessageDialog.warning({
		text:rb.getText(labelKey,labelArgs),
		title:rb.getText(titleKey,labelArgs),
		buttonActions:[{key:"DELETE",text:rb.getText("em.action.confirmation.delete.txt")}, sas.hc.m.MessageDialogAction.Cancel],
		buttonPress:callBackFn
	});
	return def;
};
sas.risk.rgf.components.EditableMatrix.prototype.actionAddRow=function(){
	var that=this;
	this.setBusyIndicatorDelay(0);
	this.setBusy(true);
	sas.risk.rgf.util.LinksUtil.createCpbFormForLinkedObject(this).done(function(json){
		var key=parseInt(json.cpb_data.BUSINESSOBJECT.handle.split("|")[1]);
		var cpbForm=that.getChildCpbForm(key);
		var sourceKey=that.getSourceObjectKey();
		var bizKey1=sourceKey;
		var bizKey2=key;
		var creator=parseInt(json.cpb_data.CURRENTUSER.handle.split("|")[1]);
		var restLinks=[{rel:"creator",title:json.cpb_data.CURRENTUSER.name}];
		var link=new sas.risk.rgf.biz.Link(bizKey1,bizKey2,null,creator,null,null,null,
				  that.getLinkTypeKey(),that.getSourceSystemCode(),{},sourceKey,null,restLinks);
		link.setTargetJson({key:key,customFields:{}});
		sas.risk.rgf.util.LinksUtil.updateLink(link,that);
		that.fireChange();
		sas.risk.rgf.util.LinksUtil.syncLinkWithCpbForm(null,{
			cpbForm:cpbForm,
			link:link,
			table:that.table,
			source:that
		});
		that.table.getTable().scrollToBottom();
		that.setBusy(false);
	});
};
sas.risk.rgf.components.EditableMatrix.prototype.actionEditRow=function(link){
	var cpbForm=this.getChildCpbForm(link.getTargetKey());
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var viewData={
		cpbForm:cpbForm,
		bizObjLabel:this.getBizObjLabel(),
		title:rb.getText("em.dialog.title.fmt.txt",[this.getBizObjLabel()])
	};
	var dataClone=$.extend(true,{},viewData.cpbForm.getCpbModelManager().getUIContext());
	var restoreOrigModel=function(e){
      var dialog=e.getParameter("dialog");
		var revert=e.getParameter("revert");
		if(revert){
			var oldUiCtxt=cpbForm.getCpbModelManager().getUIContext();
			(this.getNestedSaves()[this.getCpbId()]||[]).splice(oldUiCtxt);//clean up nested saves
			viewData.cpbForm.getCpbModelManager().getUIContextModel().setData(dataClone);
		}
		var payload={cpbForm:cpbForm,link:link,table:this.table,source:this};
      dialog.destroyContent();
		sas.risk.rgf.util.LinksUtil.syncLinkWithCpbForm(null,payload);
		dialog.close();
	};
	sas.risk.rgf.util.DialogManager.open("EditableMatrixObjectDialog",viewData,restoreOrigModel,this);
};
//HELPERS
sas.risk.rgf.components.EditableMatrix.prototype.getChildCpbForm=function(key){
	if(!key){return null;}
	return this.cpbForms[key.toString()];
};
sas.risk.rgf.components.EditableMatrix.prototype.addChildCpbForm=function(key,cpbForm){
	this.cpbForms[key.toString()]=cpbForm;
	this.handleToKey[cpbForm.getCpbModelManager().getBusinessObjectHandle()]=key;
};
sas.risk.rgf.components.EditableMatrix.prototype.getChildCpbFormByHandle=function(tpHandle){
	var key=this.handleToKey[tpHandle];
	return this.getChildCpbForm(key);
};
//STATICS
sas.risk.rgf.components.EditableMatrix.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	settings.cpbLabel.formatter=function(value){
		if(!value || !value.message){
			return fieldJson.meta.title;
		}else{
			return sas.ui.cpb3.WidgetBindingUtil.formatterLabel(value);
		}
	};
	return new sas.risk.rgf.components.EditableMatrix($.extend(
		settings,
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,'LINKEDBUSINESSOBJECTS/objectLinks',true),
			nestedSaves:sas.ui.cpb3.model.CpbModelManagerObject.bindNestedSaves("/"),
			nestedDeletes:sas.ui.cpb3.model.CpbModelManagerObject.bindNestedDeletes("/"),
			meta : fieldJson.meta,
			visibleRows : fieldJson.visibleRows,
			sourceSystemCode: fieldJson.sourceSystemCd,
			linkTypeKey: fieldJson.linkTypeKey,
			linkTypeId: fieldJson.linkTypeId,
			collectionName: fieldJson.collectionName,
			bizObjLabel:fieldJson.bizObjLabel,
			showAddRowLink:fieldJson.showAddRowLink,
			showEditRowLink:fieldJson.showEditRowLink,
			showDeleteRowLink:fieldJson.showDeleteRowLink,
			showColumnsLink:fieldJson.showColumnsLink,
			showRowsCounter:fieldJson.showRowsCounter,
			formattingKey :sas.risk.rgf.util.RGFUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
			errors:{
				path:sas.ui.cpb3.model.CpbModelManagerObject.bindMessagePath("/"),
				formatter:function(value){//called on first-page load, and on successful and error saves
					var that=this;
					var parentId=this.getCpbId();
					var localNestedSaves=this.getNestedSaves()[parentId]||[];
					var errorMessages=_.filter(value.messages,function(msg){return msg.status==="ERROR";});//retrieve all error messages
					var localMessages=_.filter(errorMessages,function(msg){return parseInt(msg.parentWidgetId)===parentId;});//filter down to error messages belonging to this instance of editable matrix
					var groups=_.groupBy(localMessages,'parentWidgetIndex');//group the filtered err messages by row index
					var hasErrors=!!errorMessages.length;
					for(var i=0;i<localNestedSaves.length;i++){
						var newUICtxt=localNestedSaves[i];
						var handle=newUICtxt["TPHANDLE"];
						var cpbForm=that.getChildCpbFormByHandle(handle);
						var cpb_messages={messages:[]};
						if(_.has(groups,i)){
							var messages=groups[i];
							var cloneMessages=$.extend(true,[],messages);
							_.each(cloneMessages,function(msg){delete msg["parentWidgetId"];delete msg["parentWidgetIndex"];});
							cpb_messages=cpbForm.getCpbAPI()._processValidationErrors(cloneMessages);
						}
						if(cpbForm){
							cpbForm.getCpbModelManager().applyErrors({cpb_messages:cpb_messages,cpb_data:newUICtxt});
						}
					}
					if(!hasErrors){//success path (everything got saved), clean out localNestedSaves
						this.getNestedSaves()[parentId]=[];
					}
				}
			}
		}
	));
};
sas.risk.rgf.components.EditableMatrix.IN_PLACE_CONTROLS=[
   "sas.risk.rgf.components.ColorDropdownComponent",
   "sas.risk.rgf.components.CurrencyInput",
   "sas.risk.rgf.components.SubScreenChooserComponent",
   "sas.risk.rgf.components.UserChooserComponent",
	"sas.ui.cpb3.field.StringField",
	"sas.ui.cpb3.field.NumberField",
	"sas.ui.cpb3.field.DateField",
	"sas.ui.cpb3.field.BooleanField",
	"sas.ui.cpb3.field.CheckBoxField",
	"sas.ui.cpb3.field.DropDownField",
	"sas.ui.cpb3.field.RadioField",
	"sas.ui.cpb3.field.TextAreaField"   
];
sas.risk.common.utils.CompositeUtil.mixinPropertySupport(sas.risk.rgf.components.EditableMatrix);