jQuery.sap.declare("sas.risk.rgf.components.FileComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.hc.ui.unified.FileUploader");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.rgf.api.UtilsAPI");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.FileComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
   metadata:{
       properties:{
           "name":{type:"string"},
           "uploadScript":{type:"string"}
       }
   },
   constructor:function(){
       this.supportsLayout=true;
       this.supportsLabel=true;
       this.supportsAlign=true;
       sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
       this.addStyleClass("RGFFile");
       this.addStyleClass("cpbFieldLikeComponent");
   },
   renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.FileComponent);

sas.risk.rgf.components.FileComponent.prototype._createComponent = function() {
   var rb = sas.risk.rgf.util.RGFUtil.getBundle();
   this.uploader = new sas.hc.ui.unified.FileUploader({
      name: "file",
      uploadUrl: "",
      buttonText: rb.getText("file.browseButton.txt"),
      uploadOnChange: false,
      sameFilenameAllowed: true,
      sendXHR:true, //using this to get the response in a better format.
      change: $.proxy(this.fileUploadChange,this),
      "aria-required":true,
      visible: false
   });
   return this.uploader;
},
sas.risk.rgf.components.FileComponent.prototype.setCpbValue = function(value) {
   this.setProperty("cpbValue",value);
},
sas.risk.rgf.components.FileComponent.prototype.onBeforeRendering=function() {
   if (!this.renderedOnce) {
      var value = this.getProperty("cpbValue");
      var rb = sas.risk.rgf.util.RGFUtil.getBundle();

      if (this.getCpbEnabled()) {
         //if the value is set, change the placeholder text to be the current filename
         //this initializes the placeholder text to the current file name in the case that
         //we are opening an existing file
         var placeholderTxt = value ? this._buildExistingFileText(value) : rb.getText("file.placeholder.txt");
         this.uploader.setPlaceholder(placeholderTxt);

         this.uploader.setVisible(true);
         this.addItem(this.uploader);
      }
      else if (value && value.fileName) {
         this.label=new sas.hc.m.Label({
            text:this._buildExistingFileText(value)
         });
         this.addItem(this.label);
      }
      this.renderedOnce = true;
   }
},
sas.risk.rgf.components.FileComponent.prototype.fileUploadChange = function() {
   //update the value in the uicontext, so that content can pick it up and validate it
	 var that = this;
	 this._buildSelectedFileJson().done(function(json){
		 that.setProperty("cpbValue", json);
		 if (that.fileOnChange){ 
			 	that.fileOnChange();	
		 }
	 });
},
sas.risk.rgf.components.FileComponent.prototype.getSelectedFile = function() {
   if (this.uploader && this.uploader.getVisible() && this.uploader.oFileUpload && this.uploader.oFileUpload.files) {
      return this.uploader.oFileUpload.files.item(0);
   }
},
sas.risk.rgf.components.FileComponent.prototype.wireOnChange = function(onChangeFn){
	if(onChangeFn){
		this.fileOnChange=onChangeFn;
	}
},
sas.risk.rgf.components.FileComponent.prototype._buildSelectedFileJson = function() {
	var def=$.Deferred();
   if (this.uploader) {
      var newFile = this.uploader.oFileUpload.files.item(0);
      if (newFile) {
      	var results= {fileName: newFile.name, fileSize: newFile.size};
      	if (this.getUploadScript()){
      		sas.risk.rgf.api.UtilsAPI.execScript(this.getUploadScript(),newFile).done(function(json){
      			   if (json){
      			   	results.data=json;	
      			   }
      				def.resolve(results);
      		});
      	}else {
      		def.resolve(results);
      	}
      } else {
      	def.resolve();
      }
   }else{
   	def.resolve();
   }
   return def;
},
sas.risk.rgf.components.FileComponent.prototype._buildExistingFileText = function(value) {
   var rb = sas.risk.rgf.util.RGFUtil.getBundle();
   var bytes = value.fileSize;

   var fileSize = bytes;
   var labelKey = "file.fileAndSize.b.fmt.txt";
   if (bytes > 1073741824)
   {
      fileSize = (bytes / 1073741824).toFixed(2);
      labelKey = "file.fileAndSize.gb.fmt.txt";
   }
   else if (bytes > 1048576)
   {
      fileSize = (bytes / 1048576).toFixed(2);
      labelKey = "file.fileAndSize.mb.fmt.txt";
   }
   else if (bytes > 1024)
   {
      fileSize = (bytes / 1024).toFixed(2);
      labelKey = "file.fileAndSize.kb.fmt.txt";
   }

   var labelArgs=[value.fileName, fileSize];
   return rb.getText(labelKey, labelArgs);
}
//STATICS
sas.risk.rgf.components.FileComponent.create=function(fieldJson, modelId, cpbForm) {
   var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
   return new sas.risk.rgf.components.FileComponent($.extend(
     settings,
     {
   	  cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
   	  meta : fieldJson.meta,
   	  uploadScript: fieldJson.uploadScript
     }
   ));
};