jQuery.sap.declare("sas.risk.rgf.components.LinkedBusinessObjectChooserComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.rgf.widget.ObjectChooser");
jQuery.sap.require("sas.risk.rgf.widget.LinkedBusinessObjectsMixin");
jQuery.sap.require("sas.risk.rgf.biz.Feature");
jQuery.sap.require("sas.risk.rgf.biz.Favorite");
jQuery.sap.require("sas.risk.slimtable.meta.TableMeta");
jQuery.sap.require("sas.risk.rgf.util.LinksUtil");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.components.LinkedBusinessObjectChooserComponent", sas.risk.rgf.widget.LinkedBusinessObjectsMixin.mixinMeta(sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
      	  location:{type:"string"},
  			  icon : {type : "sap.ui.core.URI", group : "Appearance", defaultValue : sas.icons.HC.SEARCH},
  			  placeholderText:{type:"string"},
  			  actionTooltip:{type:"string"},
  			  dialogTitle:{type:"string"},
  			  showDimensionChooserAsTree:{type:"boolean",defaultValue:false},
  			  tableMeta:{type:"any"}
        },
        events:{
      	  change:{}
        }
    },
 	constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		this.supportsAlign=true;
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Baseline);
 		this.addStyleClass("cpbFieldLikeComponent");
 		this.setMultiSelect(false);
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
})));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.LinkedBusinessObjectChooserComponent);
sas.risk.rgf.widget.LinkedBusinessObjectsMixin.mixin(sas.risk.rgf.components.LinkedBusinessObjectChooserComponent);

sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.setCPBValue=function(newLinks){
	if(!newLinks) {return;}
	var linkTypeToLinksMap=_.groupBy(newLinks,function(link){return link.linkType});
	if($.isEmptyObject(linkTypeToLinksMap)){//empty array passed to clean up the LBO table
		this.deleteAll();
		return;
	}
	var that=this;
	_.each(linkTypeToLinksMap,function(map,key){
		if(that.getLinkTypeKey()===parseInt(key)){
			var linkContainer=that.getLinkContainer();
			var matchLinksOfType=_.filter(linkContainer,function(item){return item.linkType===that.getLinkTypeKey();});
			var i;
			for(i=0; i<matchLinksOfType.length; i++){
				var l=matchLinksOfType[i];
				l.sourceKey=that.getSourceObjectKey();
				matchLinksOfType[i]=sas.risk.rgf.biz.Link.fromJson(l);
			}
			sas.risk.rgf.util.LinksUtil.deleteLinks(matchLinksOfType,that,that.getImmediateSave());
			var links={};
			_.each(map,function(newLink){
				newLink.sourceKey=that.getSourceObjectKey();
				var link=sas.risk.rgf.biz.Link.fromJson(newLink);
				sas.risk.rgf.util.LinksUtil.updateLink(link,that);
			});
			if ((map.length > 0 || matchLinksOfType.length > 0) && !that.getImmediateSave()) that.fireChange();
		}
	},this);
	this.doCPBRefresh();
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.setCpbValue=function(value){
	this.mProperties["cpbValue"]=null;//force the set in the next line to update the array instance rather than the get bypassed by the clever "no change" check from ManagedObject.setProperty
   this.setProperty("cpbValue",value);
	var modelId=this.getBindingInfo("cpbValue").parts[0].model;
	var model=this.getModel(modelId);
	this.sourceBusinessObjectKey=parseInt(model.getProperty("/BUSINESSOBJECT/handle").split("|")[1]);//TODO: something easier needed
	this.setLinkContainer(value);
	this.setSourceObjectKey(this.sourceBusinessObjectKey);
	if(this._isReadyToCreate()){
		this._loadLinkData();
	}
	this.doCPBRefresh();
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.setImmediateSave=function(value){
	this.setProperty("immediateSave",value);
	if(this._isReadyToCreate()){
		this._loadLinkData();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.setCpbEnabled=function(value){
	this.setProperty("cpbEnabled",value);
	this.field.setEnabled(value);
	if(this._isReadyToCreate()){
		this._loadLinkData();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype._isReadyToCreate=function(value){
	return this.getImmediateSave()!==undefined && this.getCpbEnabled()!==undefined && this.getCpbValue()!==undefined;
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype._loadLinkData=function(){
	return sas.risk.rgf.util.LinksUtil.fetchData(this);
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.doCPBRefresh=function(){
	var link=_.find(this.getLinks());
	if(link){
		var containerLink=_.find(sas.risk.rgf.util.LinksUtil.findMatchingLinksInContainer(this));
		if(!containerLink || !sas.risk.rgf.biz.Link.matchesLink(containerLink, link, this.getSourceObjectKey())){
			this.setLinks(null);
		}
	}
	this.doRefresh();
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.getBaseFilter=function(){
	var qp=this.getQueryProcessor();
	//RGFRMWRK-526 Don't forget to check queryProcessor for UI filters
	if(!qp && this.getQueryProcessorFieldName()){
		var cpbModelManager=this.getCpbForm().getCpbModelManager();
		qp=cpbModelManager.getUIContext()[this.getQueryProcessorFieldName().toUpperCase()];
	} 
	if(qp){
		switch(qp.type){
			case "UserQuery":
				var location=null;
				if(this.getLocationFieldName()){
					location=this.getCpbForm().getCpbModelManager().getUIContext()[this.getLocationFieldName().toUpperCase()];
					if(!location){
						alert(this.getMissingLocationMessage());
						return;
					}
				}
				var handle=location||"1|10000|";//(eveywhere)
				var objectType=this.getSourceTypeName();
				var action="'"+(qp.action||"")+"'";
				params={filter:"secureUserQueryProcessor("+action+",'"+objectType+"','"+handle+"','"+qp.showSuperUsers+"','"+qp.roles.join("','")+"')"};
				return $.param(params).substring("filter=".length);
			break;
			case "FilterExpression":
				params={filter:qp.filter};
				return $.param(params).substring("filter=".length);
			break;
		}
	}
	return null;
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.doRefresh=function(){
	var that=this;
	this._loadLinkData().done(function(){
		var link=_.find(that.getLinks());
		if(link){
			var targetJson=link.getTargetJson();
			that.field.setValueLabel(targetJson&&(targetJson.name || targetJson.displayNm || targetJson.userId || targetJson.objectId)||link.businessObject2);
		}else{
			that.field.setValueLabel("");
		}
	});
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype._createComponent = function() {
   var rb = sas.risk.rgf.util.RGFUtil.getBundle();
   var that=this;
   this.field=new sas.risk.rgf.widget.ObjectChooser({
   	collectionName:this.getCollectionName(),
   	placeholderTxt:this.getPlaceholderText()?this.getPlaceholderText():rb.getText("objectChooser.placeholder.txt"),
   	searchAttr:this.getCollectionName()==="users"?"displayNm":"name",
   	press:function(e){
	   	 var link=_.find(that.getLinks());//extract first link it finds from links object
   		 that.actionTargetAdd(e,link,{title:that.getDialogTitle()});
	   },
	   change:function(e){
	   	var newValue=e.getParameter("newValue");
	   	that.saveLinks(_.toArray(that.getLinks()),newValue?[newValue.key]:[]);
	   },
	   icon:this.getIcon()
   });
   this.field.getBaseFilter=$.proxy(this.getBaseFilter,this);//override get base filter to retrieve latest location for security
   this.field.button.setTooltip(this.getActionTooltip()?this.getActionTooltip():this.getLinkLabel());
   return this.field;
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.prototype.wireOnChange=function(onChangeFn){
	if(onChangeFn){
		this.attachChange(onChangeFn);
	}
};
//STATICS
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.risk.rgf.components.LinkedBusinessObjectChooserComponent._createSettings(fieldJson,modelId,cpbForm);
	return new sas.risk.rgf.components.LinkedBusinessObjectChooserComponent(settings);
};
sas.risk.rgf.components.LinkedBusinessObjectChooserComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	var meta=fieldJson.meta;
	var link=meta.link;
	var icon=fieldJson.icon?sas.icons.HC[fieldJson.icon]:sas.icons.HC.SEARCH;
	return $.extend(
		settings,
		{
			linkType:link.linkType,
			isLocated:link.isLocated,
			showLocationChooser:meta.showLocationChooser,
			isDimension:link.isDimension,
			collectionName:link.collectionName,
			linkLabel:link.linkLabel,
			targetFeature:link.targetFeature?sas.risk.rgf.biz.Feature.parseFromJson(link.targetFeature):null,
			favorites:sas.risk.rgf.biz.Favorite.parseAll(meta.favorites),
			targetMeta:link.table_meta_target,
			editDialogWidth:meta.editDialogWidth,
			editDialogHeight:meta.editDialogHeight,
			queryProcessor:meta.queryProcessor,
			sourceTypeName:meta.sourceTypeName,
			locationFieldName:meta.locationFieldName,
			queryProcessorFieldName:meta.queryProcessorFieldName,
			equalTypes:link.equalTypes,
			sideOfLink:link.sideOfLink,
			enforceSide:link.enforceSide,
			sourceSystemCode:link.sourceSystemCode,
			linkTypeKey:link.linkTypeKey,
			immediateSave:{
				parts:[sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,"SCREEN_READ_ONLY"),
				       sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,"NEVERBEENSAVED"),
				       sas.ui.cpb3.WidgetBindingUtil.getExpPath(fieldJson.cpbReadonly)
				],
				formatter:function(screenReadOnly,neverBeenSaved,fieldReadOnly){
					var immediateSave=fieldJson.meta.immediateSave;
					var readonly=fieldReadOnly!==undefined ? fieldReadOnly : screenReadOnly;
					var saveImmediately=!neverBeenSaved && (immediateSave || (screenReadOnly && !readonly));
					return saveImmediately;
				}
         },
         cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,'LINKEDBUSINESSOBJECTS/objectLinks',true),
         tableMeta:new sas.risk.slimtable.meta.TableMeta(link.table_meta),
         cpbLabel:{
				path:"cpb_labels>/label"+fieldJson.cpbLabel,
				formatter:function(value){
					if(value && !value.message){
						value.message=link.table_meta.title;
					}
					return sas.ui.cpb3.WidgetBindingUtil.formatterLabel(value);
				}
			},
			placeholderText:fieldJson.placeholderText,
			actionTooltip:fieldJson.actionTooltip,
			dialogTitle:fieldJson.dialogTitle,
			missingLocationMessage:meta.missingLocationMessage,
			icon:icon
		}
	);
};