jQuery.sap.declare("sas.risk.rgf.components.LinkedBusinessObjectsComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.rgf.widget.LinkedBusinessObjectsTable");
jQuery.sap.require("sas.risk.rgf.biz.Feature");
jQuery.sap.require("sas.risk.rgf.biz.Favorite");
jQuery.sap.require("sas.risk.rgf.meta.formatter.LinkInstanceCreatorFormatter");
jQuery.sap.require("sas.risk.rgf.meta.formatter.LinkInstanceDateFormatter");
jQuery.sap.require("sas.risk.rgf.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.CustomizationInfoItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.LBOExcelItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.LBODataloadItem");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");

sap.ui.layout.VerticalLayout.extend("sas.risk.rgf.components.LinkedBusinessObjectsComponent",sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata : {
		properties : {
			"meta" : {type : "any"},
			"visibleRows" : {type:"int"},
			"minVisibleRows" : {type:"int"},
			"immediateSave" : {type:"boolean",defaultValue:false},
			formattingKey:{type:"any"}
		},
		events:{
			change:{}
		}
	},
	constructor : function() {
		sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbVisible",this,"visible");
		sap.ui.layout.VerticalLayout.apply(this,arguments);
	},
	renderer : "sap.ui.layout.VerticalLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.rgf.components.LinkedBusinessObjectsComponent);

sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.setCPBValue=function(newLinks){
	if(!newLinks) {return;}
	var linkTypeToLinksMap=_.groupBy(newLinks,function(link){return link.linkType});
	if($.isEmptyObject(linkTypeToLinksMap)){//empty array passed to clean up the LBO table
		this.doDeleteAll();
		return;
	}
	_.each(linkTypeToLinksMap,function(map,key){
		_.each(this.tables,function(table){
			if(table.getLinkTypeKey()===parseInt(key)){
				var linkContainer=table.getLinkContainer();
				var matchLinksOfType=_.filter(linkContainer,function(item){return item.linkType===table.getLinkTypeKey();});
				var i;
				for(i=0; i<matchLinksOfType.length; i++){
					var l=matchLinksOfType[i];
					l.sourceKey=table.getSourceObjectKey();
					matchLinksOfType[i]=sas.risk.rgf.biz.Link.fromJson(l);
				}
				sas.risk.rgf.util.LinksUtil.deleteLinks(matchLinksOfType,table,table.getImmediateSave());
				var links={};
				_.each(map,function(newLink){
					newLink.sourceKey=table.getSourceObjectKey();
					var link=sas.risk.rgf.biz.Link.fromJson(newLink);
					sas.risk.rgf.util.LinksUtil.updateLink(link,table);
				});
			   if ((map.length > 0 || matchLinksOfType.length > 0) && !table.getImmediateSave()) table.fireChange();
			}
		});
	},this);
	this.doCPBRefresh();
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.setCPBVisible=function(value){
	this.setVisible(value);
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.setImmediateSave=function(value){
	this.setProperty("immediateSave",value);
	if(this._isReadyToCreate()){
		this._createInners();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.setCpbEnabled=function(value){
	this.setProperty("cpbEnabled",value);
	if(this._isReadyToCreate()){
		this._createInners();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype._isReadyToCreate=function(value){
	return this.getImmediateSave()!==undefined && this.getCpbEnabled()!==undefined && this.getCpbValue()!==undefined;
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.setCpbValue=function(value){
	this.mProperties["cpbValue"]=null;//force the set in the next line to update the array instance rather than the get bypassed by the clever "no change" check from ManagedObject.setProperty
   this.setProperty("cpbValue",value);
	var modelId=this.getBindingInfo("cpbValue").parts[0].model;
	var model=this.getModel(modelId);
	this.sourceBusinessObjectKey=parseInt(model.getProperty("/BUSINESSOBJECT/handle").split("|")[1]);//TODO: something easier needed
	var that=this;
	_.each(this.tables,function(table){
		table.setLinkContainer(null);//CLEAR IT OUT... to force a set on next link
		table.setLinkContainer(value);
		table.setSourceObjectKey(that.sourceBusinessObjectKey);
	});
	if(this._isReadyToCreate()){
		this._createInners();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.getExportUrl=function(collectionName){
   var exportURL= sas.risk.rgf.util.RGFUtil.getRestUrl() + "/" + collectionName + "/export";
	return exportURL;
};

sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype._createInners=function() {
   var cpbForm=this.getCpbForm();
   if(!this.tables){
   	this.setWidth("100%");
   	var that = this;
   	var meta = this.getProperty("meta");
   	var immediateSaveProp = this.getImmediateSave();
   	var model=this.getModel("cpb_data");
   	if (!meta.showCreateLinkSpecified && model !== null && model.getData("cpb_data")["NEVERBEENSAVED"]) {
   		meta.showCreateLink=false; // not showing create link on 360 tables in a fresh new business object
   	}
      this.tables=[];
      var isMulti=meta.links.length>1;
		_.each(meta.links,function(link){
			var table=new sas.risk.rgf.widget.LinkedBusinessObjectsTable({
				formattingKey : that.getFormattingKey(),
				meta:link.table_meta,
				visibleRows:that.getVisibleRows(),
				minVisibleRows:that.getMinVisibleRows(),
				enabled:that.getCpbEnabled(),
				showSelectionControls: true,
				multiSelect: true,
				clientSide:true,
				plugins:[new sas.risk.slimtable.plugins.TableOptions({
		             	  menuItems: that.getTableMenuItems(link,meta)
		               }),
		               new sas.risk.slimtable.plugins.Counter()
		      ],
				linkType:link.linkType,
				showCreateLink:meta.showCreateLink&&link.canCreate,
				showAddTargetLink:meta.showAddTargetLink,
				showRemoveTargetLink:meta.showRemoveTargetLink,
				showDeleteTargetLink:meta.showDeleteTargetLink,
				showEditLink:meta.showEditLink,
				showNoLinkTypesMessage:meta.showNoLinkTypesMessage,
				noLinkTypesMessage:link.noLinkTypesMessage,
				bizObjLabel:link.bizObjLabel,
				createLabel:link.createLabel,
				linkLabel:link.linkLabel,
				unlinkLabel:link.unlinkLabel,
				deleteLabel:link.deleteLabel,
				linkTypeKey:link.linkTypeKey,
				sideOfLink:link.sideOfLink,
				enforceSide:link.enforceSide,
				sourceSystemCode:link.sourceSystemCode,
				queryProcessor:meta.queryProcessor,
				sourceTypeName:meta.sourceTypeName,
				locationFieldName:meta.locationFieldName,
				queryProcessorFieldName:meta.queryProcessorFieldName,
				missingLocationMessage:meta.missingLocationMessage,
				
				disablement:function(source){
				   var data=source.getData();
				   var disableds=[];
				   var i,key;
				   for(i=0;i<data.length;i++){
				      var link = data[i];
				      var isDisabled = link.__LINK__.isDisabled;
				      
				      if (isDisabled !== undefined && isDisabled){
				         disableds.push(i);
				      }
				   }
				   source.setDisableds(disableds);
				},
				
				//POPUP
				collectionName:link.collectionName,
				editDialogWidth:meta.editDialogWidth,
				editDialogHeight:meta.editDialogHeight,
				immediateSave:immediateSaveProp,
				isDimension:link.isDimension,
				isLocated:link.isLocated,
				showLocationChooser:meta.showLocationChooser,
				targetFeature:link.targetFeature?sas.risk.rgf.biz.Feature.parseFromJson(link.targetFeature):null,
				favorites:sas.risk.rgf.biz.Favorite.parseAll(meta.favorites),
				targetMeta:link.table_meta_target,
				equalTypes:link.equalTypes,
				//ORA
				leafOnly:meta.ora.leafOnly,
				nonLeafOnly:meta.ora.nonLeafOnly,
				rootVisible:meta.ora.rootVisible,
				searchable:meta.ora.searchable,
				showDimensionChooserAsTree:meta.ora.showDimensionChooserAsTree,
				showFooter:meta.ora.showFooter,
				showInactive:meta.ora.showInactive,
				//Export
		      exportURL: that.getExportUrl(link.collectionName),
				
				//CPB
				cpbForm:cpbForm,
				//EVENTS
				change:$.proxy(that.fireChange,that)
			});
			that.tables.push(table);
			if(isMulti){
				table.setTitle(link.table_meta.title);
			}else{
				table.bindProperty("title", {
					path:"cpb_labels>"+that.getBindingPath("cpbLabel"),
					formatter:function(value){
						if(!value || !value.message){
							return link.table_meta.title;
						}
						return value.message;
					}
				});
				table.setRequired(that.getCpbRequired());
				table.setHelp(that.getCpbHelp());
			}
			table.setLinkContainer(that.getCpbValue());
			table.setSourceObjectKey(that.sourceBusinessObjectKey);
			table.cbAddToLbo=$.proxy(that.onAddToLbo,that,table);
			cpbForm.attachEvent('addToLbo',table.cbAddToLbo);
			table.cbDelFromLbo=$.proxy(that.onDelFromLbo,that,table);
			cpbForm.attachEvent('delFromLbo',table.cbDelFromLbo);
			table.cbReload=$.proxy(that.onReloadLbo,that,table);
			cpbForm.attachEvent('reload',table.cbReload);
			if (sas.risk.rgf.api.CommonAPI.getLboScrollPreventDefault()) {
				//RGFRMWRK-553 simple way to prevent mouse wheel scrolling table from scrolling the full page                
	         //table.attachBrowserEvent("wheel", e => e.preventDefault(), {passive:false});
	         table.attachBrowserEvent("wheel", function(e){
	         	return e.preventDefault()}, {passive:false});
			}
			that.addContent(table);
			table.build();//initialize the table
		});
   }
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.getTableMenuItems=function(metaLink,meta){
	var menuItems=[new sas.risk.rgf.widget.menu.ColumnsItem({serviceId:this.getCpbForm().getCpbService().getServiceId()})];
	var customizationInfoMeta=metaLink.table_meta.customizationInfoMeta;
	if(customizationInfoMeta){
		var custInfoUrl=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/showCustomizationInfo/linkTypeId/"+metaLink.linkType["LINKTYPE.LINKTYPEID"]+"/linkTypeSSC/"+metaLink.linkType["LINKTYPE.SOURCESYSTEMCD"]+"/sideOfLink/"+metaLink.sideOfLink;
		if (customizationInfoMeta.fileName) {
			custInfoUrl += "?fileName="+customizationInfoMeta.fileName;
		}
		menuItems.push(new sas.risk.rgf.widget.menu.CustomizationInfoItem({url:custInfoUrl,meta:customizationInfoMeta}));
	}
	
	if (meta) {
		if (meta.showExportToExcel) {
			menuItems.push(new sas.risk.rgf.widget.menu.LBOExcelItem());
		}
		
		if (meta.showExportForDataLoad) {
			var dataLoaders=sas.risk.rgf.api.CommonAPI.getDataLoaders();
			var dataLoader=_.find(dataLoaders,function(dataLoader){return dataLoader.getCollectionName()===metaLink.collectionName;});
			if (dataLoader && dataLoader.getDataLoaderName()) {
				menuItems.push(new sas.risk.rgf.widget.menu.LBODataloadItem());
			}
		}
	}
   return menuItems;
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.destroy=function(){
	var that=this;
	_.each(this.tables,function(table){
		that.getCpbForm().detachEvent('addToLbo',table.cbAddToLbo);
		that.getCpbForm().detachEvent('delFromLbo',table.cbDelFromLbo);
		that.getCpbForm().detachEvent('reload',table.cbReload);
	});
	sap.ui.layout.VerticalLayout.prototype.destroy.apply(this,arguments);
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.setCpbRequired=function(value){
	this.setProperty("cpbRequired",value);
	if(this.tables && this.tables.length===1){
		this.tables[0].setRequired(value);
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.onAddToLbo=function(table,event) {
	var link=event.getParameter("link");
	var sourceKey=table.getSideOfLink()?link.getBusinessObject2():link.getBusinessObject1();
	var enforceSide=table.getEnforceSide();
	var tableSourceKey=table.getSourceObjectKey();
	if((  (enforceSide && tableSourceKey===sourceKey) || //if side is enforced than the table source key must be on the correct side 
			(!enforceSide && (tableSourceKey===link.getBusinessObject2() || tableSourceKey===link.getBusinessObject1())) //no side enforced , so table source key can be on either side
	   ) && table.getLinkTypeKey()===link.getType()){
		link.setSourceKey(sourceKey);
		sas.risk.rgf.util.LinksUtil.updateLink(link,table);
		if (!table.getImmediateSave()) table.fireChange();
		table.doRefresh();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.onDelFromLbo=function(table,event) {
	var link=event.getParameter("link");
	var sourceKey=table.getSideOfLink()?link.getBusinessObject2():link.getBusinessObject1();
	var enforceSide=table.getEnforceSide();
	var tableSourceKey=table.getSourceObjectKey();
	if((  (enforceSide && tableSourceKey===sourceKey) || //if side is enforced than the table source key must be on the correct side 
			(!enforceSide && (tableSourceKey===link.getBusinessObject2() || tableSourceKey===link.getBusinessObject1())) //no side enforced , so table source key can be on either side
		) && table.getLinkTypeKey()===link.getType()){
		link.setSourceKey(tableSourceKey);
		sas.risk.rgf.util.LinksUtil.deleteLinks(link,that,that.getImmediateSave());
		if (!that.getImmediateSave()) that.fireChange();
		table.doRefresh();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.onReloadLbo=function(table,event) {
	var type=event.getParameter("type");
	var key=event.getParameter("key");
	if(table.getCollectionName()===type){
		var link=sas.risk.rgf.util.LinksUtil.findLink(key,table.getLinks());
		if(link){
			link.setTargetJson(null);
			table.doRefresh();
		}
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.doCPBRefresh = function(){
	var i;
	for(i=0;i<this.tables.length;i++){
		this.tables[i].doRefresh();
	}
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.prototype.doDeleteAll = function(){
	var i;
	for(i=0;i<this.tables.length;i++){
		this.tables[i].deleteAll();
		this.tables[i].doRefresh();
	}
};
//STATICS
sas.risk.rgf.components.LinkedBusinessObjectsComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.risk.rgf.components.LinkedBusinessObjectsComponent._createSettings(fieldJson,modelId,cpbForm);
	return new sas.risk.rgf.components.LinkedBusinessObjectsComponent(settings);
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	return $.extend(
		settings,
		{
			immediateSave:{
				parts:[sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,"SCREEN_READ_ONLY"),
				       sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,"NEVERBEENSAVED"),
				       sas.ui.cpb3.WidgetBindingUtil.getExpPath(fieldJson.cpbReadonly)
				],
				formatter:function(screenReadOnly,neverBeenSaved,fieldReadOnly){
					var immediateSave=fieldJson.meta.immediateSave;
					var readonly=fieldReadOnly!==undefined ? fieldReadOnly : screenReadOnly;
					var saveImmediately=!neverBeenSaved && (immediateSave || (screenReadOnly && !readonly));
					return saveImmediately;
				}
         }
		},
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,'LINKEDBUSINESSOBJECTS/objectLinks',true),
			meta : fieldJson.meta,
			formattingKey :sas.risk.rgf.util.RGFUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
			visibleRows : fieldJson.visibleRows,
			minVisibleRows : fieldJson.minVisibleRows
		}
	);
};
sas.risk.rgf.components.LinkedBusinessObjectsComponent.formatter = {
	"COM.SAS.OPRISK.SERVER.USER_linkInstanceCreator":new sas.risk.rgf.meta.formatter.LinkInstanceCreatorFormatter("COM.SAS.OPRISK.SERVER.USER_linkInstanceCreator"),
	"JAVA.UTIL.DATE_linkInstanceLastModifiedDt":new sas.risk.rgf.meta.formatter.LinkInstanceDateFormatter("JAVA.UTIL.DATE_linkInstanceLastModifiedDt")
};
sas.risk.slimtable.meta.ColumnFormatterRegistry.registerFactory(sas.risk.rgf.components.LinkedBusinessObjectsComponent.formatter,"LinkedBusinessObjects");
