jQuery.sap.declare("sas.risk.rgf.components.QueryTableComponent");
jQuery.sap.require("sas.risk.rgf.components.RGFAdvancedTableComponent");
jQuery.sap.require("sas.risk.rgf.meta.ExportMipMenuItemsFactory");
jQuery.sap.require("sas.risk.slimtable.widget.menu.SeparatorItem");
jQuery.sap.require("sas.risk.slimtable.widget.menu.AdvSearchItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.CustomizationInfoItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.ExcelItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.AttachmentsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.DataloadItem");
jQuery.sap.require("sas.risk.rgf.widget.RGFViews");
jQuery.sap.require("sas.risk.slimtable.plugins.Search");
jQuery.sap.require("sas.risk.slimtable.plugins.Filter");
jQuery.sap.require("sas.hc.m.SegmentedButton");
jQuery.sap.require("sas.risk.rgf.components.Visual360QueryComponent");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");

sas.risk.rgf.components.RGFAdvancedTableComponent.extend("sas.risk.rgf.components.QueryTableComponent",{
    metadata:{
        properties:{
            "location":{type:"string"},
            "exportMipUrl":{type:"string"},
            "exportMipUrlProgram":{type:"string"},
            "exportMipPath":{type:"string"},
            "exportMipSizeLimit":{type:"int"},
            "exportMipModelStatuses":{type:"string"},
            "businessObjectTypeName":{type:"string"},
            "showVisual360":{type:"string"},
            "altViewName":{type:"string"}
        }
    },
    renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});
sas.risk.rgf.components.QueryTableComponent.prototype.init=function(){
	this.addStyleClass("cpbControl");
	this.addStyleClass("RGFQueryTable");
};
sas.risk.rgf.components.QueryTableComponent.prototype.onAfterRendering=function(){
	//Load the default view on first time load only
	if(!this.rendered){
		this.rendered=true;
		this.handleViews();
	}
	this.rendered=true;
};
sas.risk.rgf.components.QueryTableComponent.prototype._createComponent = function() {
	var that=this;
	sas.risk.rgf.components.RGFAdvancedTableComponent.prototype._createComponent.apply(this,arguments);
	
   if (this.comp && this.comp.table) {
      this.comp.table.setSortScope(sas.risk.slimtable.SortScope.MultiSort);
   }
	
	this.hasDefaultView=!!_.find(this.getMeta().views,function(view){
		return !!view.isDefault;
	});
	
	
	//LOCATION - is specific to query tables
   this.comp.bindProperty("location","cpb_data>/LOCATION");
   
   //Query tables also support views. Add them now
   var viewName = this.getQueryTableType();
   var viewBaseUrl = sas.risk.rgf.util.RGFUtil.getRestUrl()+"/savedViews/"+viewName;
   if (this.getAltViewName()) 
   	viewBaseUrl = viewBaseUrl + "|" + this.getAltViewName();
   this.views=new sas.risk.rgf.widget.RGFViews({
   	rgfTable:this.comp,
   	viewBaseURL:viewBaseUrl
   });
   this.tableTopActionsPlugin.addItem(this.views);
   
   //Add plugin to show visual360 where applicable 
   if(this.getShowVisual360()){
   	this.comp.addPlugin(this.getVisual360Plugin());
   }
   
	var that=this;
	this.comp.attachEvent("locationChangeByView",function(event){
		that.locationUpdateByView=true;
	});
	return this.comp;
};
sas.risk.rgf.components.QueryTableComponent.prototype.getVisual360Plugin=function(){
  	var that=this;
   var Visual360Network=new sas.risk.rgf.components.Visual360QueryComponent({
   	visible:false,
   	initNodeType:this.getBusinessObjectTypeName()
   });
   Visual360Network.setInitNodeType(this.getBusinessObjectTypeName());
   this.comp.addItem(Visual360Network);
   //pass on data from table to graph
   this.comp.attachEvent("dataReceived",function(e){
   	var tableData=e.getParameter("data");
   	Visual360Network.setData(tableData);
   });
   var modePlugin=new sas.risk.slimtable.plugins.TablePlugin({
     	position: sas.risk.slimtable.TableLayout.TOP_BEGIN,
   	items:[new sas.hc.m.SegmentedButton({
         buttons: [ new sas.hc.m.Button({
					tooltip:"Table mode",//TODO: localize
					icon:sas.icons.HC.LISTVIEW,
					press:function(){
						Visual360Network.setVisible(false);
						that.comp.table.setVisible(true);
						that.comp.footerLayout.setVisible(true);
					}
				}),
				new sas.hc.m.Button({
					tooltip:"Graph mode",//TODO : localize
					icon: sas.icons.HC.NEURALNETWORK,
					press:function(){
						Visual360Network.setVisible(true);
   					that.comp.table.setVisible(false);
   					that.comp.footerLayout.setVisible(false);
					}
				})]
   	})]
   });
   return modePlugin;
};
sas.risk.rgf.components.QueryTableComponent.prototype.setLocation=function(locationHandle){
	this.setProperty("location",locationHandle);
	if(!this.locationUpdateByView){
		if(this.rendered){ 
			//Update the preference only in case of location change by user and ignore the initial load's setLocation binding call
			if (!!this.getMeta().prefAreaKey) {
				this.getCpbForm().getQueryAPI().updatePreferences(this.getMeta().prefAreaKey,locationHandle);
			}
			//Update the base view (if one exists) with the location preference update
			if(this.views && this.views.views){
				this.updateBaseViewWithLocationPreference();
			}
		}
	}
	this.locationUpdateByView=false;
};
sas.risk.rgf.components.QueryTableComponent.prototype.getTableMenuItems=function(){
	var menuItems=[new sas.risk.slimtable.widget.menu.AdvSearchItem(),new sas.risk.rgf.widget.menu.ColumnsItem({serviceId:this.getCpbForm().getCpbService().getServiceId()})];
	var customizationInfoMeta=this.getMeta().customizationInfoMeta;
	if(customizationInfoMeta){
		var custInfoUrl=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/showCustomizationInfo/queryTableType/"+this.getQueryTableType();
		if (customizationInfoMeta.fileName) {
			custInfoUrl += "?fileName="+customizationInfoMeta.fileName;
		}
		menuItems.push(new sas.risk.rgf.widget.menu.CustomizationInfoItem({url:custInfoUrl,meta:customizationInfoMeta}));
	}
	
	var exportMipForType=sas.risk.rgf.meta.ExportMipMenuItemsFactory[this.getQueryTableType()];
	var exportMipMenuItem = exportMipForType ? exportMipForType.apply(this) : null;
	var showExportMip=exportMipMenuItem && this.getExportMipUrl() !== undefined && this.getExportMipUrlProgram() !== undefined;
	if (showExportMip || this.getShowExportToExcel() || this.getShowExportForDataLoad() || this.getShowExportAttachments()){
		menuItems.push(new sas.risk.slimtable.widget.menu.SeparatorItem());	
	}
	
	if (showExportMip) {
		menuItems.push(exportMipMenuItem);
	}
	if (this.getShowExportToExcel()){
		menuItems.push(new sas.risk.rgf.widget.menu.ExcelItem());	
	}
	if (this.getShowExportForDataLoad()){
		menuItems.push(new sas.risk.rgf.widget.menu.DataloadItem());	
	}
	if (this.getShowExportAttachments()){
		menuItems.push(new sas.risk.rgf.widget.menu.AttachmentsItem());	
	}
	
   return menuItems;
};
sas.risk.rgf.components.QueryTableComponent.prototype._fetchData = function(query) {
	if(this.hasDefaultView && !this.rendered){
		return new $.Deferred().resolve({items:[],start:0,count:0});
	}
	var colNames=[];
	_.each(query.getOrderedColumns(),function(colMeta){
		colNames.push(colMeta.getColName())
	});
	var fields="links,"+colNames.toString();//links are not added by default. Need to ask for specifically
	if(this.getShowVisual360()){
		fields="objectLinks,"+fields;
	}
	var url=this.dataUrlProp+(this.dataUrlProp.indexOf("?") > 0 ? "&" : "?")+"fields="+fields; 
	return this.getCPBControl().fetchData(url,query);
};
sas.risk.rgf.components.QueryTableComponent.prototype.handleViews = function(){
	var that=this;
   this.views.loadViews().done(function(){
      var defaultView=that.views.getDefaultView();
      //If there is location preference then add it to the base view <None> at this point during first load
      //This is in addition to the same call in setLocation during preference save of location since on initial load, the setLocation block is reached way early before the base view is even created 
      if(!this.locationUpdateByView){
      	that.updateBaseViewWithLocationPreference();
      }
      if(defaultView.getKey() !== -1){
      	that.comp.doApplyDefaultQuery(defaultView.getQueryMeta());
      }
   })
};
sas.risk.rgf.components.QueryTableComponent.prototype.updateBaseViewWithLocationPreference = function(){
	var baseView=_.find(this.views.views,function(view){return view.getKey()===-1});
   baseView.getQueryMeta().setLocation(this.getLocation());
};
//STATICS
sas.risk.rgf.components.QueryTableComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.risk.rgf.components.QueryTableComponent._createSettings(fieldJson,modelId,cpbForm);
	return sas.ui.cpb3.components.AdvancedTableComponent._createComponent(settings,sas.risk.rgf.components.QueryTableComponent);
};
sas.risk.rgf.components.QueryTableComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	if(fieldJson.queryTableType&&!fieldJson.dataUrl){//making dataUrl optional,just derive url from query table type
		var restMediaTypes=sas.risk.rgf.api.UtilsAPI.getRestMediaTypes();
		var mediaType=_.find(restMediaTypes,function(type){return type.boTypeNm===fieldJson.queryTableType;});
		if (mediaType && mediaType.restUri) 
			fieldJson.queryTableType=mediaType.restUri;
		fieldJson.dataUrl=sas.risk.rgf.util.RGFUtil.aimRestUrl(fieldJson.queryTableType);
	}
	if (fieldJson.queryTableType&&fieldJson.dataUrl) {
		var restMediaTypes=sas.risk.rgf.api.UtilsAPI.getRestMediaTypes();
		var mediaType=_.find(restMediaTypes,function(type){return type.boTypeNm===fieldJson.queryTableType;});
		if (mediaType && mediaType.restUri) 
			fieldJson.queryTableType=mediaType.restUri;
	}
	var settings=sas.risk.rgf.components.RGFAdvancedTableComponent._createSettings(fieldJson,modelId,cpbForm);
	$.extend(settings,{
		location: sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,"location"),
		exportMipUrl : fieldJson.exportMipUrl,
		exportMipUrlProgram : fieldJson.exportMipUrlProgram,
		exportMipPath : fieldJson.exportMipPath,
		exportMipSizeLimit : fieldJson.exportMipSizeLimit,
		exportMipModelStatuses : fieldJson.exportMipModelStatuses,
		businessObjectTypeName : fieldJson.businessObjectTypeName,
		showVisual360 : fieldJson.showVisual360,
		altViewName : fieldJson.altViewName
	});
	return settings;
};