jQuery.sap.declare("sas.risk.rgf.components.TaskListComponent");
jQuery.sap.require("sas.risk.rgf.components.RGFAdvancedTableComponent");
jQuery.sap.require("sas.risk.slimtable.widget.menu.AdvSearchItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.CustomizationInfoItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.BulkActionItem");
jQuery.sap.require("sas.risk.slimtable.util.TableUtil");
jQuery.sap.require("sas.risk.slimtable.meta.filter.OrFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.AndFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.StringFilter");
jQuery.sap.require("sas.risk.rgf.meta.filter.FilterMetaFactory");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");

sas.risk.rgf.components.RGFAdvancedTableComponent.extend("sas.risk.rgf.components.TaskListComponent", {
   metadata:{
      properties:{
        "refreshFilters":{type:"boolean",defaultValue:true},
        "defaultType" : {type:"string", defaultValue:null},
        "bulkAction":{type:"boolean",defaultValue:true}
      },
      events:{
      	change:{}
      }
   },
   renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});

sas.risk.rgf.components.TaskListComponent.prototype.getAdvTableComponent=function(queryTableType, visibleRowsProp){
	this._label.addStyleClass("RGFTabTitleLabel");
	var metaProp=this.getMeta();
	var rgfTable = new sas.risk.rgf.widget.RGFTable({
		formattingKey:this.getFormattingKey(),
      meta: metaProp,
      data: $.proxy(this._fetchData,this),
      visibleRows: visibleRowsProp,
      showSelectionControls:this.getBulkAction(),
      multiSelect:true,
      plugins:[new sas.risk.slimtable.plugins.Search(),
               new sas.risk.slimtable.plugins.Filter({
               	createFilterForSelectHook: $.proxy(this.createFilterForSelect,this),
               	checkSpecialFilterHook: $.proxy(this.checkSpecialFilter,this),
               	checkCurrentFilterValHook: $.proxy(this.checkCurrentFilterVal,this)
               }),
               new sas.risk.slimtable.plugins.Counter(),
               new sas.risk.slimtable.plugins.Pager()
      ],
      beforeRefreshAllHook: $.proxy(this.beforeRefreshAll,this),
      afterRefreshAllHook: $.proxy(this.afterRefreshAll,this)
   });
	rgfTable.addPlugin(new sas.risk.slimtable.plugins.TableOptions({
		menuItems: this.getTableMenuItems(metaProp, rgfTable)
   }));
	return rgfTable;
};

sas.risk.rgf.components.TaskListComponent.prototype.getTableMenuItems=function(meta,table){
	var menuItems=[
      new sas.risk.slimtable.widget.menu.AdvSearchItem(),
      new sas.risk.rgf.widget.menu.ColumnsItem({serviceId:this.getCpbForm().getCpbService().getServiceId()})
   ];
   if (this.getBulkAction()) {
		menuItems.push(new sas.risk.rgf.widget.menu.BulkActionItem({rgfTable:table}));
   }
	var customizationInfoMeta=meta.customizationInfoMeta;
	if(customizationInfoMeta){
		var custInfoUrl=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/showCustomizationInfo/"+customizationInfoMeta.boType;
		menuItems.push(new sas.risk.rgf.widget.menu.CustomizationInfoItem({url:custInfoUrl,meta:customizationInfoMeta}));
	}
   return menuItems;
};

sas.risk.rgf.components.TaskListComponent.prototype.doRowClick=function(oEvent){
   var type = this.getQueryTableType();
   var rowData=this.comp.getSource().getData()[oEvent.getParameter("rowIdx")];
   var boLink = this.findLink(rowData, "businessObject");
   if (boLink)
   {
      var boTypes = boLink.uri.split("/");
      sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
			this.getCpbForm(),
			{
				key:boTypes[3],
				type:boTypes[2],
				event_name:"object_selected"
			}
		);
   }
};

sas.risk.rgf.components.TaskListComponent.prototype.onReload = function(event) {
    this.doCPBRefresh();
};
//HOOKS
//beforeRefreshAllHook for RGFTable
sas.risk.rgf.components.TaskListComponent.prototype.beforeRefreshAll=function(json){
   if (this.getRefreshFilters()) {
      var filterableColumns=this.comp.getTableMeta().getFilterableColumns();
      var typeColumn=_.find(filterableColumns,function(fc){return fc.getColName()==="type"});
      var statusColumn=_.find(filterableColumns,function(fc){return fc.getColName()==="objectStatus"});
      this._refreshColumnChoices(json,typeColumn,statusColumn);
   }
};
//afterRefreshAllHook for RGFTable
sas.risk.rgf.components.TaskListComponent.prototype.afterRefreshAll=function(){
   if (this.getRefreshFilters() && !!this.defaultTypeFilter) {
      this.defaultTypeFilter.setSelectedKey(this.getDefaultType()||-1);
      this.defaultTypeFilter.fireChange({selectedItem: this.defaultTypeFilter.getItemByKey(this.getDefaultType())});
      this.defaultTypeFilter = null;
      this.setDefaultType(null);
   }
};
//createFilterForSelectHook for RGFTable
//Creates a query filter for the dropdown filter that takes into account possible multiple values for the selected label
sas.risk.rgf.components.TaskListComponent.prototype.createFilterForSelect=function(selectFilter){
   if (this.getRefreshFilters()) {
   	var name=selectFilter.getName();
   	var selectedItem=selectFilter.getSelectedItem();
   	var payload=sas.risk.slimtable.util.TableUtil.findCustomData(selectedItem,"payload");
   	var value=payload.value;
   	var baseFilter=new sas.risk.slimtable.meta.filter.StringFilter(name,"eq",value);
   	var valueMap = this.filterColumnValuesMap[name];
      var label = selectedItem.getText();      
      if (name==="objectStatus" && valueMap != undefined && valueMap[label] != undefined && valueMap[label].length > 0) {
      	var orFilters = [];
         for(j = 0; j < valueMap[label].length; j++) {
            var columnValue = valueMap[label][j];
            var filter1 = new sas.risk.slimtable.meta.filter.StringFilter(name,"eq",columnValue.value);
            // use a special "eq2" operator to prevent Type filter dropdown to select this value.  See line#142 in Filter.js.
            var filter2 = new sas.risk.slimtable.meta.filter.StringFilter("type","eq2",columnValue.type);
            var andFilters = [filter1, filter2];
            orFilters.push(new sas.risk.slimtable.meta.filter.AndFilter(andFilters));
         }         
         if (orFilters.length === 1) {
            baseFilter = orFilters[0];      	
         } else {
            baseFilter = new sas.risk.slimtable.meta.filter.OrFilter(orFilters);      	
         }
      }
      return baseFilter;
   }
   return null;
};
//checkSpecialFilterHook for RGFTable
//Sets the default type on the type filter dropdown if needed
sas.risk.rgf.components.TaskListComponent.prototype.checkSpecialFilter=function(name, choice, filter){
   if (this.getRefreshFilters() && name==="type") {
      if (this.getDefaultType() !== null && this.getDefaultType() !== "" && choice.value===this.getDefaultType()) {
         this.defaultTypeFilter = filter;
      }
   }
   if(name==="type" && this.typeFilter!==filter){
   	this.typeFilter=filter;
   	filter.attachChange($.proxy(this._onTypeChange,this));
   	filter.mEventRegistry.change.unshift(filter.mEventRegistry.change.pop());//make it the first one to be notified
   }else if(name==="objectStatus" && this.statusFilter!==filter){
   	this.statusFilter=filter;
   }
};
//checkCurrentFilterValHook for RGFTable
//Translates what could be multiple values into a single value so that the filter dropdown can set selected item
sas.risk.rgf.components.TaskListComponent.prototype.checkCurrentFilterVal=function(currentFilterVal){
   var result = currentFilterVal;
   if (this.getRefreshFilters() && currentFilterVal != null && this.doneFilterColumnValuesMap) {
      var isString=_.isString(currentFilterVal);
      //currentFilterVal = isString ? currentFilterVal : currentFilterVal[0];
      result = isString ? currentFilterVal : currentFilterVal[0];
   }
   return result;
};

//HELPERS
sas.risk.rgf.components.TaskListComponent.prototype._refreshColumnChoices=function(json,typeColumn,statusColumn){
	this.filterColumnValuesMap={};
	var typeChoices=json.choices&&json.choices[typeColumn.getColName()]||{};
	var statusChoices=json.choices&&json.choices[statusColumn.getColName()]||{};
	this.filterColumnValuesMap={};
	this.filterColumnValuesMap[typeColumn.getColName()]=this._buildChoicesMap(typeChoices);
	this.filterColumnValuesMap[statusColumn.getColName()]=this._buildChoicesMap(statusChoices);
	this._compressChoices(typeColumn);
	this._compressChoices(statusColumn);
	this.doneFilterColumnValuesMap = true;
};
sas.risk.rgf.components.TaskListComponent.prototype._compressChoices=function(column){
	var choices=this.filterColumnValuesMap[column.getColName()];
	column.choices=_.reduce(choices,function(memo,choices){
		memo.push(choices[0]);
		return memo;
	},[]);
};
sas.risk.rgf.components.TaskListComponent.prototype._buildChoicesMap=function(choices){
	return _.reduce(choices,function(memo,choice){
		memo[choice.label]=memo[choice.label]||[];
		memo[choice.label].push(choice);
		return memo;
	},{});
};
sas.risk.rgf.components.TaskListComponent.prototype._onTypeChange=function(){
	if(this.statusFilter){
		this.statusFilter.setSelectedItemId();
	}
};
sas.risk.rgf.components.TaskListComponent.prototype.____refreshColumnChoices=function(json, column, i){
   this.doneFilterColumnValuesMap = false;
   this.filterColumnValuesMap = this.filterColumnValuesMap ? this.filterColumnValuesMap : {};
   if (json.choices !== undefined && json.choices[column.getColName()] !== undefined) {
      var colName = column.getColName();
      var arr = json.choices[column.getColName()];
      var obj = {};
      var arr2 = [];
      for(var j=0; j < arr.length; j++) {
         var label = arr[j].label;
         if (!(arr[j].label in obj)) {
            var valueMap = this.filterColumnValuesMap[colName];
            if (valueMap === undefined) {
               valueMap = {};
            }
            valueMap[label]=[arr[j]];
            this.filterColumnValuesMap[colName]=valueMap;
            arr2.push(arr[j]);
            obj[arr[j].label] = true;
         } else {
            var valueMap = this.filterColumnValuesMap[column.getColName()];
            valueMap[label].push(arr[j]);
            this.filterColumnValuesMap[colName]=valueMap;
         }
      }
      var filterPlugin=_.find(this.comp.getPlugins(),function(plugin){
      	return plugin instanceof sas.risk.slimtable.plugins.Filter
      });
      if(filterPlugin){
         // TODO: AJ : need to find a better way to do this.
         if (filterPlugin.filters[i] === undefined || 
         	(filterPlugin.filters[i].getSelectedItem() != undefined && 
             (filterPlugin.filters[i].getSelectedItem().getKey() == -1 || 
              (i==1 && 
               filterPlugin.filters[0].getSelectedItem().getKey() != -1 && 
               arr2.length > 2)))) {
            column.choices=arr2;
         }
      }
      this.doneFilterColumnValuesMap = true;
   }
};
sas.risk.rgf.components.TaskListComponent.prototype.findLink=function(item, name)
{
   if (item.links){
      return _.find(item.links, function (link) {return link.rel === name});
   }
};
//STATICS
sas.risk.rgf.components.TaskListComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.risk.rgf.components.TaskListComponent._createSettings(fieldJson, modelId, cpbForm);
	return sas.ui.cpb3.components.AdvancedTableComponent._createComponent(settings,sas.risk.rgf.components.TaskListComponent);
};
sas.risk.rgf.components.TaskListComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=sas.risk.rgf.components.RGFAdvancedTableComponent._createSettings(fieldJson,modelId,cpbForm);
	$.extend(settings,{
		refreshFilters: fieldJson.refreshFilters,
		formattingKey :sas.risk.rgf.util.RGFUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
      defaultType: fieldJson.defaultType,
      bulkAction: fieldJson.bulkAction
	});
	return settings;
};