jQuery.sap.declare("sas.risk.rgf.components.UsersComponent");
jQuery.sap.require("sas.risk.rgf.components.QueryTableComponent");
jQuery.sap.require("sas.risk.slimtable.widget.menu.AdvSearchItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.CustomizationInfoItem");
jQuery.sap.require("sas.risk.slimtable.util.TableUtil");
jQuery.sap.require("sas.risk.slimtable.meta.filter.OrFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.StringFilter");
jQuery.sap.require("sas.risk.rgf.api.BatchJobsAPI");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.rgf.util.NavigationUtil");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.hc.m.BusyDialog");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.MessageSummary");
jQuery.sap.require("sap.ui.model.json.JSONModel");

sas.risk.rgf.components.QueryTableComponent.extend("sas.risk.rgf.components.UsersComponent", {
   metadata:{
      properties:{
        "showSyncButton" : {type:"boolean", defaultValue:false}
      }
   },
   renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});


sas.risk.rgf.components.UsersComponent.prototype.getAdvTableComponent=function(queryTableType, visibleRowsProp){
  	var metaProp=this.getMeta();
	this.rgfTable = new sas.risk.rgf.widget.RGFTable({
		formattingKey:this.getFormattingKey(),
      meta: metaProp,
      data: $.proxy(this._fetchData,this),
      exportURL: this.getExportUrl(),
      visibleRows: visibleRowsProp,
      plugins:[this.getTableTopActionsPlugin(),
               new sas.risk.slimtable.plugins.Search(),
               new sas.risk.slimtable.plugins.Filter(),
               new sas.risk.slimtable.plugins.Counter(),
               new sas.risk.slimtable.plugins.Pager()
      ]
   });
	this.rgfTable.addPlugin(new sas.risk.slimtable.plugins.TableOptions({
		menuItems: this.getTableMenuItems(metaProp, this.rgfTable)
   }));
	return this.rgfTable;
};

sas.risk.rgf.components.UsersComponent.prototype.getExportUrl=function(){
	var dataUrl = this.getDataUrl();
   var i = dataUrl.indexOf("?");
	var exportUrl = i > 0 ? (dataUrl.substring(0,i) + "/export" + dataUrl.substring(i,dataUrl.length)) : (dataUrl + "/export");
	console.debug("exportUrl=" + exportUrl);
	return exportUrl;
};

sas.risk.rgf.components.UsersComponent.prototype.getTableMenuItems=function(meta,table){
	var menuItems=[
      new sas.risk.slimtable.widget.menu.AdvSearchItem(),
      new sas.risk.rgf.widget.menu.ColumnsItem({serviceId:this.getCpbForm().getCpbService().getServiceId()})
   ];
	var customizationInfoMeta=this.getMeta().customizationInfoMeta;
	if(customizationInfoMeta){
		var custInfoUrl=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/showCustomizationInfo/queryTableType/users";
		menuItems.push(new sas.risk.rgf.widget.menu.CustomizationInfoItem({url:custInfoUrl,meta:customizationInfoMeta}));
	}
	menuItems.push(new sas.risk.slimtable.widget.menu.SeparatorItem());
	menuItems.push(new sas.risk.rgf.widget.menu.ExcelItem());
   return menuItems;
};

sas.risk.rgf.components.UsersComponent.prototype.onReload = function(event) {
   this.doCPBRefresh();
};

sas.risk.rgf.components.UsersComponent.prototype.getTableTopActionsPlugin=function() {
	var that=this;
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
   
   this.tableTopActionsPlugin=new sas.risk.slimtable.plugins.TablePlugin({
     	position: sas.risk.slimtable.TableLayout.TOP_END,
   	items:[]
   });
   if(this.getShowSyncButton()) {
	   this.syncButton=new sas.risk.common.widgets.Button({
	      press: $.proxy(this.doSyncButtonClick,this),
	      icon:sas.icons.HC.REFRESH,
	      tooltip: rb.getText("usersTable.action.refresh.tooltip.txt")
	   });
	   this.tableTopActionsPlugin.addItem(this.syncButton);
	   this.tableTopActionsPlugin.handleDisabled=function(){
	   	that.syncButton.setVisible(false);
	   };
   }
   return this.tableTopActionsPlugin;
};

sas.risk.rgf.components.UsersComponent.prototype.doSyncButtonClick=function(oEvent){
	var that = this;
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var busyDialog = new sas.hc.m.BusyDialog({
      title: rb.getText("usersTable.action.refresh.busy.message.txt"),
      showCancelButton: false
	});
	busyDialog.open();
	
	sas.risk.rgf.api.BatchJobsAPI.startUserSync().done(function (result) {
		// Get the link that will give us status
		var statusLink = that.findLink(result, "self");
		// Repeat every 3 seconds until complete
		var timer = setInterval(function() {
	      sas.risk.rgf.api.BatchJobsAPI.getSyncStatus(statusLink).done(function(statusResult) {
				if(statusResult.state === "completed") {
					clearInterval(timer);
					if(busyDialog) {
						var progressMsg = rb.getText("table.action.refresh.busy.message.progress.fmt.txt", [Math.floor(statusResult.completionPercentage)]);
						busyDialog.setTitle(progressMsg);
		   			busyDialog.close();
		   			busyDialog.destroy();
		   			that.showSyncMessages(statusResult);
			      }
					if(that.rgfTable) {
						that.rgfTable.refreshAll();
					}
				} else {
					if(busyDialog) {
						var progressMsg = rb.getText("table.action.refresh.busy.message.progress.fmt.txt", [Math.floor(statusResult.completionPercentage)]);
						busyDialog.setTitle(progressMsg);
					}
				}
			}).fail(function(e) {
				clearInterval(timer);
				if(busyDialog) {
	   			busyDialog.close();
	   			busyDialog.destroy();
		      }
				sas.risk.rgf.util.RGFUtil.handleFail(e);
			});
	   }, 3000); // every 3 seconds
  }).fail(function (e) {
	  if(busyDialog) {
			busyDialog.close();
			busyDialog.destroy();
     }
	  if (e.status && e.status === 404 && that.meta) {
       //The parent object no longer exists
       var errorJson = {"message" : that.meta.missingParentMsg};
       sas.risk.common.widgets.MessageDetailsBox.error(errorJson, sas.config.appName, sas.risk.rgf.util.RGFUtil.getBundle());
     }
  });
};

sas.risk.rgf.components.UsersComponent.prototype.showSyncMessages=function(statusResult) {
	var dialog;
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
   var closeButton = new sas.hc.m.Button({
       text: rb.getText("table.action.refresh.results.dialog.button.close.txt"),
       press: function() {
      	 dialog.close();
       }
   });
   var oMessageSummary = new sas.hc.m.MessageSummary({ 
      title: rb.getText("usersTable.action.refresh.results.dialog.title.txt"),
      layoutMode: sas.hc.m.MessageSummaryLayoutModeType.List,
      showToolbar: false,
      messagesInitiallyExpanded: true,
      showInformationFilter: true,
      showWarningFilter: true,
      showErrorFilter: true,
      showSuccessFilter: true
   });
   
   var msgModel = new sap.ui.model.json.JSONModel();
   var syncMessagesJson = this.getSyncMessagesJson(statusResult);
   msgModel.setData(syncMessagesJson);
   oMessageSummary.setModel(msgModel);

   dialog = new sas.hc.m.Dialog({
       title: rb.getText("usersTable.action.refresh.results.dialog.title.txt"),
       content:[oMessageSummary],
       minWidth: 380,
       minHeight: 300,
       resizable: true,
       rightButton: closeButton
   });
   dialog.open();
};

sas.risk.rgf.components.UsersComponent.prototype.getSyncMessagesJson=function(statusResult) {
	var messagesArray = [];
	this.getSyncMessagesArrayByType(messagesArray, 
			                          statusResult, 
			                          sas.hc.m.MessageSummaryFilterButtonType.Success, // or, could be .Information?
			                          "infoMessages");
	this.getSyncMessagesArrayByType(messagesArray, 
                                   statusResult, 
                                   sas.hc.m.MessageSummaryFilterButtonType.Error, 
                                   "errorMessages");
	this.getSyncMessagesArrayByType(messagesArray, 
                                   statusResult, 
                                   sas.hc.m.MessageSummaryFilterButtonType.Warning, 
                                   "warningMessages");
	var json = {
			"messages": messagesArray
	};
	return json;
}

sas.risk.rgf.components.UsersComponent.prototype.getSyncMessagesArrayByType=function(messagesArray, statusResult, messageType, messageListName) {
   if(statusResult[messageListName]) {
		for(var i = 0; i < statusResult[messageListName].length; i++) {
			var msg = {
				"type": messageType
			};
			var messageList = statusResult[messageListName];
			msg.title = messageList[i];
			messagesArray.push(msg);
		}
   }
	return messagesArray;
}

sas.risk.rgf.components.UsersComponent.prototype.findLink=function(item, name)
{
   if (item.links){
      return _.find(item.links, function (link) {return link.rel === name});
   }
};

//STATICS
sas.risk.rgf.components.UsersComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.risk.rgf.components.UsersComponent._createSettings(fieldJson,modelId,cpbForm);
	return sas.risk.rgf.components.UsersComponent._createComponent(settings,fieldJson);
};
sas.risk.rgf.components.UsersComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=sas.risk.rgf.components.RGFAdvancedTableComponent._createSettings(fieldJson,modelId,cpbForm);
	$.extend(settings,{
		formattingKey :sas.risk.rgf.util.RGFUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
      showSyncButton: fieldJson.showSyncButton
	});
	return settings;
};
sas.risk.rgf.components.UsersComponent._createComponent=function(settings,fieldJson) {
	var def=$.Deferred();
	var comp = new sas.risk.rgf.components.UsersComponent(settings)
	comp.fullMeta = fieldJson;
	def.resolve(comp);
	return def;
};