jQuery.sap.declare("sas.risk.rgf.nav.RGFToolbarActionsObject");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.risk.rgf.widget.Button");
jQuery.sap.require("sas.hc.m.ActionSheet");
jQuery.sap.require("sas.risk.rgf.util.DialogManager");
jQuery.sap.require("sas.risk.common.utils.DialogManager");
jQuery.sap.require("sas.risk.rgf.data.RGFModelManagerObject");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.ui.cpb3.nav.ToolbarActionsObject");
jQuery.sap.require("sas.risk.rgf.api.CommonAPI");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.rgf.biz.Link");

sas.ui.cpb3.nav.ToolbarActionsObject.extend("sas.risk.rgf.nav.RGFToolbarActionsObject", {
	constructor:function(){
		sas.ui.cpb3.nav.ToolbarActionsObject.apply(this,arguments);
		this.pollAndRefreshDebounced=_.debounce(this.pollAndRefresh,2000);
	},
   renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.applySettings=function(){
	sas.ui.cpb3.nav.ToolbarActionsQuery.prototype.applySettings.apply(this,arguments);
	var cpbModelManager=this.getCpbForm().getCpbModelManager();
	this.saveButton=this.createSaveButton();
	this.closeButton=this.createCloseButton();
	var that=this;
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	this.wfActions = new sas.hc.m.Menu({
		items: {
			path: sas.risk.rgf.data.RGFModelManagerObject.bindWorkflowPath("/activity/transitions"),
			template: new sas.hc.m.MenuItem({
				text: {
					path:sas.risk.rgf.data.RGFModelManagerObject.bindWorkflowPath('title'),
					formatter:function(title){
						return title||"";
					}
				},
				press: function(e){
					that.updateWorkflowInfo();
					var transition = sas.risk.rgf.data.RGFModelManagerObject.getWorkflowBindingContext(this).getObject();
					var viewData={
						title:rb.getText('form.changereason.txt'),
						transition:transition,
						reasonTxt:rb.getText('form.reasonText.txt'),
						saveButtonTxt:rb.getText("changeReason.add.dialog.save.txt"),
						cancelButtonTxt:rb.getText("changeReason.add.dialog.cancel.txt"),
						changeReason: that.getProperty("changeReason")
					};
					if (transition.isChangeReasonRequired()){
						sas.risk.common.utils.DialogManager.open("sas.ui.cpb3.nav.ChangeReasonDialog",viewData,that.actionWorkflow,that);	
					}else{
						that.transitionKey=transition.getKey();
						that.actionSave($.proxy(that.closePageOnTransition,that,transition));
					}
				}
			})
		}
	});
	this.openButton = new sas.hc.m.MenuButton({
		text:rb.getText("form.button.workflow.txt"),
		visible:{
			parts:[
				sas.risk.rgf.data.RGFModelManagerObject.bindWorkflowPath("/activity"),
				sas.ui.cpb3.model.CpbModelManager.bindDataPath("/SCREEN_READ_ONLY"),
				sas.ui.cpb3.model.CpbModelManager.bindDataPath("/TEMP.POLLING_URL"),
				sas.ui.cpb3.model.CpbModelManager.bindDataPath("/BUSINESSOBJECT/handle"),
				sas.ui.cpb3.model.CpbModelManager.bindDataPath("/TEMP.POLLING_MAX_WAIT")
			],
			formatter:function(activity,screenReadOnly,pollingUrl,handle,maxWait){
				if(screenReadOnly && pollingUrl && handle){
					that.pollAndRefreshDebounced(pollingUrl,handle,1,maxWait);
				}
				return (!!activity)&&(!!(activity.getTransitions().length));
			}
		},
		design: sas.hc.m.MenuButton.Design.LightweightNeutral,
		menu:this.wfActions
	}).addStyleClass("FormWfButton");
	this.histButton=new sas.risk.common.widgets.Button({
	       icon: sas.icons.HC.HISTORY,
	       enabled: {
	      	 path:"cpb_data>/NEVERBEENSAVED",
	      	 formatter:function(val){return !val;}
	       },
	       press: function(){
	      	 	that.updateObjectInfo();
	      	 	var meta=cpbModelManager.getViewHistoryMeta();
		       	var viewData={
		       		objectType:	that.objectType,
		       		objectId:   that.objectId,
		       		meta:meta,
		       		title:meta.historyDialogLabel,
		       		formattingKey:sas.risk.rgf.util.RGFUtil.buildDialogFormattingKey("ViewHistoryDialog")
		   		};
	       		sas.risk.rgf.util.DialogManager.open("ViewHistoryDialog",viewData,that.actionSave,that);	
	       }
	}).addStyleClass("FormSaveButton");
	if(sas.risk.rgf.util.RGFUtil.isScreenEditable(this.incomingRouteName)){
		this.editScreenButton=this.createScreenEditor();
		this.addItem(this.editScreenButton);
		this.addItem(new sap.ui.commons.TextView({text:"|"}).addStyleClass("RGFScreenEditorSeparator"));
	}
	this.addItem(this.histButton);
	this.addItem(this.saveButton);
	this.addItem(new sap.ui.commons.TextView({
		text:"|",
		visible:{
			path:sas.risk.rgf.data.RGFModelManagerObject.bindWorkflowPath("/activity"),
			formatter:function(activity){
				return (!!activity)&&(!!(activity.getTransitions().length));
			}
		}
	}).addStyleClass("RGFScreenEditorSeparator"));
	this.addItem(this.openButton);
	this.addItem(this.closeButton);
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.pollAndRefresh=function(url,handle,wait,maxWait){
	var that=this;
	var wait=wait||1;
	var maxWait=maxWait||60;
	if(wait >= maxWait)
	   wait = 1;
	setTimeout(function(){
		$.getJSON(url,function(results){
			var dataModel=that.getModel(sas.ui.cpb3.model.CpbModelManager._nameData);
			var newHandle=dataModel&&dataModel.getProperty("/BUSINESSOBJECT/handle");
			var stillOnThePage=!!(that.closeButton.$().length) && handle===newHandle;
			if(!stillOnThePage){
				console.info("moved away from object page, cancelling poll for workflow status");
				return;
			}else if(results.items.length){
				wait=Math.min(wait*2, maxWait);
				console.info("workflow status unchanged, retrying in "+wait+" seconds");
				that.pollAndRefreshDebounced(url,handle,wait,maxWait);
			}else{
				console.warn("workflow status update: refreshing page");
				var current=hasher.getHash();
				hasher.setPrivateHash("");
				hasher.setHash(current);
			}
		})
	},wait*1000);
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.createSaveButton=function(){
	var saveButton=sas.ui.cpb3.nav.ToolbarActionsObject.prototype.createSaveButton.apply(this,arguments);
	saveButton.bindProperty("visible",{
		path:sas.risk.rgf.data.RGFModelManagerObject.bindWorkflowPath("/activity"),
		formatter:function(activity){
			return activity&&!!activity.getSaveTransition();
		}
	});
	return saveButton;
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.isChangeReasonRequired=function(){
	var oMod=this.getCpbForm().getCpbModelManager();
	this.updateWorkflowInfo();
	var activity=oMod.getWorkflowActivity();
	var saveLink=oMod.getSaveLink();
	var changeReasonConfig=sas.ui.cpb3.nav.ToolbarActionsObject.prototype.isChangeReasonRequired.apply(this,arguments);
	return (!!activity)&&activity.getSaveTransition().isChangeReasonRequired() || (!!saveLink)&& changeReasonConfig;
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.closePageOnTransition=function(transition){
	if(transition.closePageAfterApply){
		this.actionClose();
	}
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.saveObjectContext=function(uiContext){
	return this.getCpbForm().getCpbAPI().saveObjectContext(this.objectType,this.objectId,uiContext,this.getChangeReason(),this.getNestedSaves(),this.getNestedDeletes(),this.activityKey,this.transitionKey)
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.draftSaveCallback=function(){
	sas.ui.cpb3.nav.ToolbarActionsObject.prototype.draftSaveCallback.apply(this,arguments);
	this.incomingRouteName=this.objectType;
	this.incomingRouteArgs={key:this.objectId};
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.saveSuccess=function(json,neverBeenSaved){
	var cpbModelManager=this.getCpbForm().getCpbModelManager();
	if(!sas.risk.rgf.util.RGFUtil.isMainTab()){
		var linkInfo=_.find(cpbModelManager.getUIContextValue("LINKEDBUSINESSOBJECTS").objectLinks,function(l){return l.isDisabled;});
		if(linkInfo){
			var newKey=parseInt(cpbModelManager.getUIContextValue("HANDLE").split("|")[0]);
			var newLink=_.find(json.cpb_data.LINKEDBUSINESSOBJECTS.objectLinks,
			      function(l){return linkInfo.linkType===l.linkType && (
			            (linkInfo.businessObject1 && linkInfo.businessObject1===l.businessObject1 && l.businessObject2===newKey) || 
			            (linkInfo.businessObject2 && linkInfo.businessObject2===l.businessObject2 && l.businessObject1===newKey)
			       );});
			if(newLink){
			   var link = sas.risk.rgf.biz.Link.fromJson(newLink);
			   sas.ui.cpb3.CpbUtil.fireStorageEvent('add_lbo',link.getLinkContainerJsonRep());
			}else {console.error("New link pointing to parent is missing from link container.");} /*I18nOK:EMS*/
		}else{console.error("Temp Link pointing to parent is missing from link container.");} /*I18nOK:EMS*/
		var allOtherLinks=_.filter(cpbModelManager.getUIContextValue("LINKEDBUSINESSOBJECTS").objectLinks,function(l){return !l.isDisabled;});
		_.each(allOtherLinks,function(link){
			sas.ui.cpb3.CpbUtil.fireStorageEvent('add_lbo',sas.risk.rgf.biz.Link.fromJson(link).getLinkContainerJsonRep());
		});
	}
	if(!neverBeenSaved){
		var origLinks=cpbModelManager.getUIContextValue("ORIG_LINKEDBUSINESSOBJECTS").objectLinks;
		var currentLinks=cpbModelManager.getUIContextValue("LINKEDBUSINESSOBJECTS").objectLinks;
		var deletedLinks=_.differenceObj(origLinks,currentLinks,function(a,b){ 
			return a.linkInstanceId===b.linkInstanceId;
		});
		_.each(deletedLinks,function(link){
			sas.ui.cpb3.CpbUtil.fireStorageEvent('del_lbo',sas.risk.rgf.biz.Link.fromJson(link).getLinkContainerJsonRep());
      });
	}
	sas.ui.cpb3.nav.ToolbarActionsObject.prototype.saveSuccess.apply(this,arguments);
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.updateWorkflowInfo=function(){
	var cpbModelManager=this.getCpbForm().getCpbModelManager();
	this.transitionKey=null;
	this.activityKey=undefined;
	var activity=cpbModelManager.getWorkflowActivity();
	if(!activity){
		var saveLink=cpbModelManager.getSaveLink();
		if(saveLink){
			this.objectType=saveLink.objectType;
			this.objectId=saveLink.objectId;			
		}
	}else{
		this.activityKey=activity.getKey();
		this.objectType=activity.getObjectType();
		this.objectId=activity.getObjectId();
		var saveTransition=activity.getSaveTransition();
		if(saveTransition){
			this.transitionKey=saveTransition.getKey();
		}
	}
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.updateObjectInfo=function(){
	var cpbModelManager=this.getCpbForm().getCpbModelManager();
	var uiContext=cpbModelManager.getUIContext();
	this.objectType=uiContext.BUSINESSOBJECT.collectionName;
	this.objectId=uiContext.BUSINESSOBJECT.handle.split("|")[1];
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.actionWorkflow = function(e) {
	var that=this;
	var transition=e.getParameter("transition"); 
	this.transitionKey=transition.getKey();
	var changeReason= e ? e.getParameter("changeReason") : null;
	this.setChangeReason(changeReason);
	var dialog=e.getParameter("dialog");	
	var successFunc=function(){
		that.setChangeReason(null);
		dialog.close();
		if(transition.shouldClosePageAfterApply()){
			this.actionClose();
		}
	};
	var errorFunc=function(){
		dialog.close();
	}
	this.actionSave($.proxy(successFunc,this),errorFunc)
	.fail(function(e){dialog.close();});
};
sas.risk.rgf.nav.RGFToolbarActionsObject.prototype.actionClose = function(e) {
	this.setChangeReason(null);
	return sas.risk.rgf.nav.RGFToolbarActionsObject.actionClose(this.getCpbForm());
};
sas.risk.rgf.nav.RGFToolbarActionsObject.actionClose=function(cpbForm) {
	if (sas.risk.rgf.util.RGFUtil.isMainTab()) {
		var screenId=cpbForm.getScreenId();
		var type=_.find(sas.risk.rgf.api.UtilsAPI.getRestMediaTypes(),function(type){
			return type.screenId===screenId;
		});
		sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
			cpbForm,
			{
				event_name:"close_object",
				parent:type.parentRoute
			}
		);
	} else {
		$(window).unbind('beforeunload'); //to not show the browser default warning message while leaving a page with changes
		window.top.close();
	}
};