jQuery.sap.declare("sas.risk.rgf.util.LinksUtil");
jQuery.sap.require("sas.ui.cpb3.model.CpbModelManager");

sas.risk.rgf.util.LinksUtil={
	//REST INTEGRATION
	fetchData:function(source){
		var def=$.Deferred();
		var links=source.getLinks();
		if(!links || !_.size(links)){//first load
			links=sas.risk.rgf.util.LinksUtil.findMatchingLinksInContainer(source);
			source.setLinks(links);
		}
		//find links that dont have their target json already loaded
		var linksToLoad=_.filter(links,function(l){return !l.hasTargetJson();});
		var linkKeys=_.map(linksToLoad,function(l){return l.getTargetKey();});
		if(linkKeys.length){
			sas.risk.rgf.api.CommonAPI.loadCollection(source.getCollectionName(),linkKeys).done(function(items){
				var loadedLinkKeys=_.map(items,function(l){return l.key;});
				var missingLinkKeys=_.difference(linkKeys,loadedLinkKeys);
				if(missingLinkKeys.length){
					_.each(missingLinkKeys,function(key){
						var missingLink=links[key.toString()];
						if(missingLink && missingLink.getIsDisabled()){
							source.partialViewPending=true;//There are one or more links that could not be shown because they are from an object that has not yet been saved.
						}else{
							source.partialViewPerms=true;//There are one or more items that you are not allowed to view.
						}
					});
				}
				var defs=[];
				var i;
				for(i=0;i<items.length;i++){
					var link=links[items[i].key.toString()];
					link.setTargetJson(items[i]);
					if(source.addChildCpbForm){
						defs.push(sas.risk.rgf.util.LinksUtil.loadCpbFormForLinkedObject(link,source));
					}
				}
				$.when.apply($,defs).done(function(){
					def.resolve(sas.risk.rgf.util.LinksUtil.doConvertLinksToJson(links));
				});
			});
			return def;
		}
		def.resolve(sas.risk.rgf.util.LinksUtil.doConvertLinksToJson(links));
		return def;
	},
	findMatchingLinksInContainer:function(source){
		var considerSide=(source.getSideOfLink)?(!source.getEqualTypes()||source.getEnforceSide()):false;
		var sideOfLink=considerSide?source.getSideOfLink():null
		return sas.risk.rgf.biz.Link.findByType(source.getLinkContainer(),source.getLinkTypeKey(),source.getSourceObjectKey(),sideOfLink);
	},
	doConvertLinksToJson:function(links){
		var json={count:0,start:0,limit:0,items:[]};
		var i,link,jsonRep;
		for(key in links){
			link=links[key];
			if(link.hasTargetJson()){//these are the ones that we had perms to load
				jsonRep=link.getJsonRep();
				jsonRep.__LINK__=link;
				json.items.push(jsonRep);
			}
		}
		json.count=json.items.length;
		json.limit=json.items.length;
		return json;
	},
	//LINK MANAGEMENT
	updateLink:function(link,source){
		var linkContainer=source.getLinkContainer();
		var index=sas.risk.rgf.util.LinksUtil.findLinkIndex(link,linkContainer);
		if(index!==-1){//UPDATE
			linkContainer[index]=link.getLinkContainerJsonRep();
		}else{//CREATE
			linkContainer.push(link.getLinkContainerJsonRep());
		}
		source.getLinks()[link.getTargetKey().toString()]=link;
	},
	deleteLinks:function(links,source,immediateSave,weakEntity){
		links=$.isArray(links)?links:[links];
		var sourceLinks=source.getLinks();
		var i,index,link,targetKey;
		var linkContainer=source.getLinkContainer();
		for(i=0;i<links.length;i++){
			link=links[i];
			targetKey=link.getTargetKey();
			delete sourceLinks[targetKey.toString()];
			index=sas.risk.rgf.util.LinksUtil.findLinkIndex(link,linkContainer);
			if(index!==-1){
				var linkContainer=linkContainer;
				linkContainer.splice(index,1);
			}
			if(weakEntity){
				var savedChildren=source.getSavedChildren();
				var childIndex=savedChildren.indexOf(targetKey);
				if(childIndex!==-1){
					if(immediateSave){
						
					}else{
						//ADD to NESTED DELETES
						var childCpbModelManager=source.getChildCpbForm(targetKey).getCpbModelManager();
						var childHandle=childCpbModelManager.getBusinessObjectHandle();
						var nestedDeletes=source.getNestedDeletes();
						var toDelete=nestedDeletes[source.getCpbId()]||[];
						toDelete.push(childHandle);
						nestedDeletes[source.getCpbId()]=toDelete;
						source.fireChange();
					}
				} else {
					//CLEAR from NESTED SAVES
					var childCpbModelManager=source.getChildCpbForm(targetKey).getCpbModelManager();
					var nestedSaves=source.getNestedSaves()[source.getCpbId()]||[];
					var nestedSaveIndex=nestedSaves.indexOf(childCpbModelManager.getUIContext());
					if(nestedSaveIndex>-1){
						nestedSaves.splice(nestedSaveIndex);
					}
					source.fireChange();
				}
			}
		}
	},
	findLink:function(key,links){
		return _.find(links,function(link){
			return link.getTargetKey()===key;
		});
	},
	findLinkIndex:function(link,linkContainer){
		var i,json;
		var type=link.getType();
		var biz1=link.getBusinessObject1();
		var biz2=link.getBusinessObject2();
		var targetKey=link.getTargetKey();
		for(i=0;i<linkContainer.length;i++){
			json=linkContainer[i];
			if(json.linkType===type && json.businessObject1===biz1 && json.businessObject2===biz2){
				return i;
			}
		}
		return -1;
	},
	//CPBForm INTEGRATION
	loadCpbFormForLinkedObject:function(link,source){
		var key=link.getTargetJson().key;
		var objectType=source.getCollectionName();
		var typeMeta=_.find(sas.risk.rgf.api.UtilsAPI.getRestMediaTypes(),function(type){
			return type.objectType===objectType;
		});
		var screenId=typeMeta.screenId;
		var cpbForm = new sas.ui.cpb3.CPBForm({//create a new cpbForm for the item being loaded
			serviceId : source.getCpbForm().getServiceId(),
			screenId : screenId,
			data : sas.ui.cpb3.model.CpbModelManager.bindData('/')
		});
		var cpbModelManager=cpbForm.getCpbService().createModelManagerObject(cpbForm);//create a new clean model to host the item's ui context
		var tempParent=source.getCpbForm().getCpbModelManager().getUIContext();
		var overrides={
			"TEMP.PARENT":tempParent,
			"TEMP.LINKTYPE":source.getLinkTypeKey(),
			"TEMP.SSC":source.getSourceSystemCode()
		};
		if (tempParent.LOCATION) { //do not allow null in UI context
		   overrides["PREVIOUSLOCATION"] = tempParent.LOCATION;
		}
		return cpbForm.getCpbAPI().loadObjectContext(screenId, objectType, key, overrides).done(function(json){
			json.cpb_data["TEMP.PARENT"]=tempParent;
			cpbModelManager.refresh(json);//imprint results on the cpbForm
			source.addChildCpbForm(key,cpbForm);//store the cpbForm in the cache by item key
			source.getSavedChildren().push(key);//store key of children that already exist
			var payload={cpbForm:cpbForm,link:link,table:source.table,source:source};
			sas.risk.rgf.util.LinksUtil.syncLinkWithCpbForm(null,payload,true);
		});
	},
	createCpbFormForLinkedObject:function(source){
		var objectType=source.getCollectionName();
		var typeMeta=_.find(sas.risk.rgf.api.UtilsAPI.getRestMediaTypes(),function(type){
			return type.objectType===objectType;
		});
		var screenId=typeMeta.screenId;
		var cpbForm = new sas.ui.cpb3.CPBForm({//create a new cpbForm for the item being loaded
			serviceId : source.getCpbForm().getServiceId(),
			screenId : screenId,
			data : sas.ui.cpb3.model.CpbModelManager.bindData('/')
		});
		var cpbModelManager=cpbForm.getCpbService().createModelManagerObject(cpbForm);//create a new clean model to host the item's ui context
        var tempParent=source.getCpbForm().getCpbModelManager().getUIContext();
      var overrides={
            "TEMP.PARENT":tempParent,
            "TEMP.LINKTYPE":source.getLinkTypeKey(),
            "TEMP.SSC":source.getSourceSystemCode()
      };
      if (tempParent.LOCATION) { //do not allow null in UI context
         overrides["PREVIOUSLOCATION"] = tempParent.LOCATION;
      }
      return cpbForm.getCpbAPI().createObjectContext(screenId, objectType, null, overrides).done(function(json){
            json.cpb_data["TEMP.PARENT"]=tempParent;
			cpbModelManager.refresh(json);//imprint results on the cpbForm
			source.addChildCpbForm(cpbModelManager.getBusinessObjectRK(),cpbForm);//store the cpbForm in the cache 
		});
	},
	syncLinkWithCpbForm:function(e,payload,skipRefresh){
		var cpbForm=payload.cpbForm;
		var cpbModelManager=cpbForm.getCpbModelManager();
		var skipSave=!cpbModelManager.isFlaggedDirty() && !cpbModelManager.isDirty();
		cpbForm.updateUIContextWithRequiredFields();
		var link=payload.link;
		var table=payload.table;
		var source=payload.source;
		var tm=table.getTableMeta();
		var cpbData=cpbForm.getCpbModelManager().getPageContext().cpb_data;
		var targetJson=link.getTargetJson();
		_.each(tm.getColumns(),function(col){
			var upperRawName=col.getColRawName().toUpperCase();
			if(col.isColCustom()){
				if(_.has(targetJson.customFields,col.getColName())||_.has(cpbData,upperRawName)){
					targetJson.customFields=targetJson.customFields||{};
					targetJson.customFields[col.getColName()]=cpbData[upperRawName];
				}
			}else{
				if(_.has(targetJson,col.getColName())||_.has(cpbData,upperRawName)){
					targetJson[col.getColName()]=cpbData[upperRawName];
				}
			}
		});
		var nestedSaves=source.getNestedSaves();
		var toSave=nestedSaves[source.getCpbId()]||[];
		var uictxt=cpbForm.getCpbModelManager().getUIContext();
		if(!skipSave || uictxt.NEVERBEENSAVED){
			if(toSave.indexOf(uictxt)===-1){
				toSave.push(uictxt);
				nestedSaves[source.getCpbId()]=toSave;
				source.fireChange();
			}
		}
		if(!skipRefresh){
			table.doRefresh();
		}
	}
};