jQuery.sap.require("sas.risk.rgf.widget.ObjectChooser");
jQuery.sap.require("sas.risk.slimtable.meta.filter.AndFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.NumberFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.BooleanFilter");
jQuery.sap.require("sas.risk.rgf.api.DelegationAPI");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsview("sas.risk.rgf.view.dialog.AddEditDelegateDialog", {
	getControllerName : function() {
		return "sas.risk.rgf.view.dialog.AddEditDelegateDialog";
	},
	createContent : function(controller) {
		var that=this;
		var that=this;
		var isReadyCheck=function(){
			var delegate=that.delegate.getValue();
			var startDate=that.startDate.getDateValue();
			var endDate=that.endDate.getDateValue();
			var isReady=!!(delegate && startDate && endDate && startDate.getTime()<=endDate.getTime());
			that.okButton.setEnabled(isReady);
		}
		var rb = sas.risk.rgf.util.RGFUtil.getBundle();
		var activeUsersFilter=new sas.risk.slimtable.meta.filter.BooleanFilter("activeFlg","eq",true);
		var currentUser=sas.risk.rgf.api.CommonAPI.getUserDetails();
		var notCurrentUserFilter=new sas.risk.slimtable.meta.filter.NumberFilter("key","ne",currentUser.getUserRk());
		var viewData=this.getViewData();
		this.delegate=new sas.risk.rgf.widget.ObjectChooser({
			collectionName:"users",
			placeholderTxt:rb.getText("objectChooser.users.placeholder.txt"),
			dialogTitleTxt:rb.getText("objectChooser.users.dialog.title"),
			icon:sas.icons.HC.USER,
			searchAttr:"displayNm",
			change:isReadyCheck,
			baseFilter:new sas.risk.slimtable.meta.filter.AndFilter([activeUsersFilter,notCurrentUserFilter]),
			serviceId:viewData.serviceId
		});
		var today=new Date();
		today.setHours(0);
		today.setMinutes(0);
		today.setSeconds(0);
		today.setMilliseconds(0);
		if(!viewData.delegationId){//"add" mode
			viewData.startDate=today;
			viewData.endDate=today;
		}else{//"edit" mode
			this.delegate.setValue(viewData.delegate);
			this.delegate.setValueLabel(viewData.delegate.displayNm);
		}
		
		var inputFormat = sap.ui.core.format.DateFormat.getInstance({style:"medium"});
		var getDateValue=function(){
			return inputFormat.parse(this.getValue());
		};
		var setDateValue=function(oDate){
			return this.setValue(inputFormat.format(oDate));
		};
		var now=new Date();
		this.startDate=new sas.hc.m.DatePicker({displayFormat:"medium",valueFormat:"medium",dateValue:viewData.startDate});
		if(viewData.startDate.getTime()<today.getTime()){
			this.startDate.setEnabled(false);
		}else{
			this.startDate.setMinDate(now);
		}
		this.startDate.getDateValue=getDateValue;
		this.startDate.setDateValue=setDateValue;
		this.endDate=new sas.hc.m.DatePicker({displayFormat:"medium",valueFormat:"medium",dateValue:viewData.endDate});
		if(viewData.endDate.getTime()<today.getTime()){
			this.endDate.setEnabled(false);
			this.delegate.setEnabled(false);
		}else{
			this.endDate.setMinDate(now);
		}
		this.endDate.getDateValue=getDateValue;
		this.endDate.setDateValue=setDateValue;
		this.startDate.attachChange(function(e){
			var val=e.getSource().getDateValue();
			that.endDate.setMinDate(val);
			if(val){
				var toVal=that.endDate.getDateValue();
				if(toVal && val.getTime()>toVal.getTime()){
					that.endDate.setDateValue(val);
				}
			}
			isReadyCheck();
		});
		this.endDate.attachChange(function(e){
			var val=e.getSource().getDateValue();
			that.startDate.setMaxDate(val);
			if(val){
				var fromVal=that.startDate.getDateValue();
				if(fromVal && val.getTime()<fromVal.getTime()){
					that.startDate.setDateValue(val);
				}
			}
			isReadyCheck();
		});
		//labels
		var addSeparator=function(label){
			return jQuery.sap.formatMessage(label.replace(/'/g,"''")+"{0}",[":"]);
		};
		var delegateLabel=new sas.hc.m.Label({
			text:addSeparator(viewData.delegateLabel),
			labelFor:this.delegate,
			required:true
		});
		var startDateLabel=new sas.hc.m.Label({
			text:addSeparator(viewData.startDateLabel),
			labelFor:this.startDate,
			required:true
		});
		var endDateLabel=new sas.hc.m.Label({
			text:addSeparator(viewData.endDateLabel),
			labelFor:this.endDate,
			required:true
		});
	   this.maxTime = sas.risk.rgf.api.UtilsAPI.getDelegationMaxTime();
		var maxTimeLabel=new sas.hc.m.Label({
			text:rb.getText("delegation.maximumTime.fmt.txt",[this.maxTime])
		});
		//layout
		var matrix=new sap.ui.commons.layout.MatrixLayout({layoutFixed:false});
		matrix.createRow(delegateLabel,this.delegate);
		matrix.createRow(startDateLabel,this.startDate);
		matrix.createRow(endDateLabel,this.endDate);
		if (this.maxTime > 0) {
			matrix.createRow(maxTimeLabel);
		}
		
		var layout = new sas.hc.ui.commons.layout.VerticalLayout({
			width : "100%",
			content : [ this.createMessageBar(), matrix]
		});
		this.createButtons(controller);
		return layout;
	},
	createMessageBar : function(){
	  	this.messageBar = new sas.risk.common.layout.flexlayout.FlexLayout({flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column}).addStyleClass("cpbErrorMessageBar cpbMessageBar cpbMessageBarHidden");
	   this.errorMsgTxt=new sas.hc.m.Label().addStyleClass("cpbMessage");
	   this.errorMsgTxt2=new sas.hc.m.Label().addStyleClass("cpbMessage");
	   var rb = sas.risk.rgf.util.RGFUtil.getBundle();
		this.errorMsgTxt.setText(rb.getText("delegation.validation.notUnique.txt"));
		this.errorMsgTxt2.setText(rb.getText("delegation.validation.maximumTime.txt"));
	   this.messageBar.addItem(this.errorMsgTxt);
	   this.messageBar.addItem(this.errorMsgTxt2);
	   return this.messageBar;
	},
	createButtons : function(controller) {
		var rb = sas.risk.rgf.util.RGFUtil.getBundle();
		var that=this;
		this.okButton = new sas.risk.common.widgets.Button({
			text : rb.getText("delegation.dialog.button.ok.txt"),
			press : $.proxy(controller.actionOk,controller),
			enabled : false
		}).addStyleClass("sasPrimaryButton");
		this.cancelButton = new sas.risk.common.widgets.Button({
			text : rb.getText("delegation.dialog.button.cancel.txt"),
			press : $.proxy(controller.actionCancel,controller)
		});
	},
	getDialogSettings : function() {
		return {
			beginButton : this.okButton,
			endButton : this.cancelButton,
			initialFocus : this.delegate
		};
	}
});
sap.ui.controller("sas.risk.rgf.view.dialog.AddEditDelegateDialog", {
	actionOk:function(){
		var view=this.getView();
		var dialog=view.getDialog();
		dialog.setBusy(true);
		var delegationId=view.getViewData().delegationId;
		var delegate=view.delegate.getValue();
		var startDate=view.startDate.getDateValue();
		var endDate=view.endDate.getDateValue();
		var maxTime=view.maxTime;
		var serviceId=view.getViewData().serviceId;
		var rb = sas.risk.rgf.util.RGFUtil.getBundle();
		view.errorMsgTxt.setText(rb.getText("delegation.validation.notUnique.txt"));
		sas.risk.rgf.api.DelegationAPI.isDelegationValid(delegationId,delegate,startDate,endDate,serviceId,maxTime).done(function(json){
			if(json.uniqueError || json.maxTimeError){
				view.errorMsgTxt2.setText("");
				if(json.maxTimeError && json.uniqueError){ // both errors
					view.errorMsgTxt.setText(rb.getText("delegation.validation.notUnique.txt"));
					view.errorMsgTxt2.setText(rb.getText("delegation.validation.maximumTime.txt"));
				} else if(json.maxTimeError){ // maxTime error only
					view.errorMsgTxt.setText(rb.getText("delegation.validation.maximumTime.txt"));
				} else { // unique error only  
					view.errorMsgTxt.setText(rb.getText("delegation.validation.notUnique.txt"));
				}
	   		$(view.messageBar.getDomRef()).removeClass("cpbMessageBarHidden");
				dialog.setBusy(false);
			}else{
				view.fireClosing({delegationId:delegationId,delegate:delegate,startDate:startDate,endDate:endDate});
	   		$(view.messageBar.getDomRef()).addClass("cpbMessageBarHidden");
			}
		}).fail(function(e){
			console.error(e);
			dialog.setBusy(false);
		});
	},
	actionCancel:function(){
		this.getView().getDialog().close();
	}
});