jQuery.sap.require("sas.risk.rgf.api.UtilsAPI");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");

sap.ui.jsview("sas.risk.rgf.view.dialog.AttachmentPropertiesDialog", {
    getControllerName : function() {
        return "sas.risk.rgf.view.dialog.AttachmentPropertiesDialog";
    },
    createContent : function(controller) {
       this.addStyleClass("rgfObjectViewPopup");
       var viewData=this.getViewData();
       var that=this;
       var rb = sas.risk.rgf.util.RGFUtil.getBundle();
       this.createButtons(controller);
       var messages = new sas.risk.common.layout.flexlayout.FlexLayout({
          width : "100%",
          flexDirection : sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
          items : {
              path : '/messages',
              factory : function(sId, context) {
                  var messageBar = new sas.risk.common.layout.flexlayout.FlexLayout({
                      items : [ 
                       new sas.hc.m.Label({
                              text : "{message}"
                          }).addStyleClass("cpbMessage")
                      ]
                  }).addStyleClass("cpbErrorMessageBar").addStyleClass("cpbMessageBar");
                  // possible add other possible types of message css classes here
                  // in future
                  return messageBar;
              }
          }
      });

       this.layout=new sas.risk.common.layout.flexlayout.FlexLayout({
      	  width : "100%",
           flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
           items:[
               messages,
               this.createAttachmentFormLayout(controller)
           ]
       });
       this.okButton.setEnabled(controller.actionIsOkEnabled());
       return this.layout;
   },
   createAttachmentFormLayout : function(controller) {
      var that = this;
      var rb = sas.risk.rgf.util.RGFUtil.getBundle();
      var viewData=this.getViewData();
      
      this.formLayout = new sas.hc.ui.commons.layout.MatrixLayout({
         layoutFixed : true,
         columns : 2,
         width : "auto",
         widths : [ "30%", "70%"]
      });
            
      this.linkIdInput=new sas.hc.m.Input({
         type:sap.m.InputType.Text,
         maxLength:32,
         required:true,
         disableValidation:false,
         value:viewData.item.objectId,
         liveChange:function(e){
           that.okButton.setEnabled(controller.actionIsOkEnabled());
         },
         "aria-required":true,
         enabled:false
      });
      this.linkIdLabel= new sas.hc.m.Label({text:viewData.meta.linkId,required:true,labelFor:this.linkIdInput}).addStyleClass("RGFAttachmentsFieldLbl");

      this.linkNameInput=new sas.hc.m.Input({
         type:sap.m.InputType.Text,
         maxLength:100,
         required:true,
         disableValidation:false,
         value:viewData.item.name,
         liveChange:function(e){
           that.okButton.setEnabled(controller.actionIsOkEnabled());
         },
         "aria-required":true
      });
      this.linkNameLabel= new sas.hc.m.Label({text:viewData.meta.linkName,required:true,labelFor:this.linkNameInput}).addStyleClass("RGFAttachmentsFieldLbl");
      this.urlInput=new sas.hc.m.Input({
         type:sap.m.InputType.Text,
         maxLength:1000,
         required:true,
         disableValidation:false,
         value:viewData.item.urlTxt,
         liveChange:function(e){
             that.okButton.setEnabled(controller.actionIsOkEnabled());
         },
         "aria-required":true
      });
      this.urlLabel= new sas.hc.m.Label({text:viewData.meta.linkUrl,required:true,labelFor:this.urlInput}).addStyleClass("RGFAttachmentsFieldLbl");
      
      var idRow = new sas.hc.ui.commons.layout.MatrixLayoutRow({
         cells: [
                 new  sas.hc.ui.commons.layout.MatrixLayoutCell({
                     content :[this.linkIdLabel],
                     separation: sap.ui.commons.layout.Separation.Small
                  }),
                  new  sas.hc.ui.commons.layout.MatrixLayoutCell({
                      content :[this.linkIdInput],
                      separation: sap.ui.commons.layout.Separation.Small
                   })
         ]
      });
      this.formLayout.addRow(idRow);
      
      var nameRow = new sas.hc.ui.commons.layout.MatrixLayoutRow({
         cells: [
                 new  sas.hc.ui.commons.layout.MatrixLayoutCell({
                     content :[this.linkNameLabel],
                     separation: sap.ui.commons.layout.Separation.Small
                  }),
                  new  sas.hc.ui.commons.layout.MatrixLayoutCell({
                      content :[this.linkNameInput],
                      separation: sap.ui.commons.layout.Separation.Small
                   })
         ]
      });
      this.formLayout.addRow(nameRow);
      
      var urlRow = new sas.hc.ui.commons.layout.MatrixLayoutRow({
             cells: [
                     new  sas.hc.ui.commons.layout.MatrixLayoutCell({
                         content :[this.urlLabel],
                         separation: sap.ui.commons.layout.Separation.Small
                      }),
                      new  sas.hc.ui.commons.layout.MatrixLayoutCell({
                          content :[this.urlInput],
                          separation: sap.ui.commons.layout.Separation.Small
                      })
             ]
      });
      this.formLayout.addRow(urlRow);
      
      return this.formLayout;
   },   
    createButtons : function(controller) {
        var rb = sas.risk.rgf.util.RGFUtil.getBundle();
        var that=this;
        this.okButton = new sas.risk.common.widgets.Button({
           text : rb.getText("attachments.dialog.ok.txt"),
           press : $.proxy(controller.actionOkCreateUpdate,controller)
       });
       this.cancelButton = new sas.risk.common.widgets.Button({
          text : rb.getText("attachments.dialog.cancel.txt"),
          press : $.proxy(controller.actionCancel,controller),
      });
    },
    getDialogSettings : function() {
       var viewData=this.getViewData();
       return {
          beginButton : this.okButton,
          endButton : this.cancelButton,
          contentWidth:viewData.width+"px",
          contentHeight:viewData.height+"px"
      };
    },
});
sap.ui.controller("sas.risk.rgf.view.dialog.AttachmentPropertiesDialog", {
   onInit:function(){
      var view=this.getView();
      view.setModel(new sas.ui.cpb3.model.CpbJSONModel({}));
      var viewData = view.getViewData();      
      var bo = viewData.model.getData("cpb_data")["BUSINESSOBJECT"];
  },
  actionOkCreateUpdate:function(){
     var that = this;
     var view = this.getView();
     var viewData = view.getViewData();
        
     var groupNo = viewData.meta.groupNumber;
     var model = viewData.model;

     var sscValue, sscLink;
     //must have links set so ssc will display properly
     sscValue = viewData.meta.sourceSystemCd || viewData.item.sourceSystemCd;
     
     var ssc = viewData.sscchoices;
     for (var i = 0; i < ssc.length; i++)
     {
        if (ssc[i].value === sscValue)
        {
           sscLink = {
                 "rel": "sourceSystemCd",
                 "title": ssc[i].label
              };              
        }
     }
     
     var normalizedUrl = sas.risk.rgf.api.AttachmentsAPI.normalizeURL(view.urlInput.getValue());
     
     //if we are not immediate save, then validate now
     var validateDeferred = sas.risk.rgf.api.UtilsAPI.isValidUrl(normalizedUrl);
     validateDeferred.done(function(json) {
        if (json && !json.result) {
           var e = {};
           e.responseJSON = {};
           e.responseJSON.details = ["Please provide a valid URL."];
           view.getModel().setProperty("/messages",that._buildMessagesForCpb(e));            
        }
        else
        {
           //validate was ok, continue
           if (viewData.create)
           {
             var item = {
                   sourceSystemCd: sscValue,
                   objectId: view.linkIdInput.getValue(),
                   typeCd: "LNK",
                   name: view.linkNameInput.getValue(), 
                   urlTxt: normalizedUrl,
                   key: viewData.item.key,
                   links: [sscLink]};
             
             if (groupNo) item.groupNo = groupNo;

             var def = sas.risk.rgf.api.AttachmentsAPI.addAttachment(model, groupNo, viewData.meta.immediateSave, item).done(function() {
                view.fireClosing();
                view.getDialog().close();  
             }).fail(function(e){
                if (e.status && e.status === 400)
                {
                   view.getModel().setProperty("/messages",that._buildMessagesForCpb(e));
                }
                else if (e.status && e.status === 404)
                {
                   //The parent object no longer exists
                   var errorJson = {"message" : viewData.meta.missingParentMsg};      
   	             sas.risk.common.widgets.MessageDetailsBox.error(errorJson, sas.config.appName, sas.risk.rgf.util.RGFUtil.getBundle());	
                }
                else
                {
                   sas.risk.rgf.util.RGFUtil.handleFail(e);
                }
             });      
           }
           else
           {        
             var item = viewData.item;
             item.sourceSystemCd = sscValue;
             item.objectId = view.linkIdInput.getValue();
             item.name = view.linkNameInput.getValue();
             item.urlTxt = normalizedUrl;
             item.key = viewData.item.key;
             item.links = _.reject(item.links, function(lnk) {return lnk.rel === "sourceSystemCd"});
             item.links.push(sscLink);

             if (groupNo) item.groupNo = groupNo;
             
             var def = sas.risk.rgf.api.AttachmentsAPI.updateAttachment(model, groupNo, viewData.meta.immediateSave, item).done(function(extra) {
                view.fireClosing();
                view.getDialog().close();  
             }).fail(function(e){
                if (e.status && e.status === 400)
                {
                   view.getModel().setProperty("/messages",that._buildMessagesForCpb(e));
                }
                else
                {
                   sas.risk.rgf.util.RGFUtil.handleFail(e);
                }
             });
           }   
        }
     });
  },
  actionCancel:function() {
     this.getView().getDialog().close();
  },
  actionIsOkEnabled:function(){
     var view=this.getView();
     return view.linkIdInput.getValue().length > 0 && 
        view.linkNameInput.getValue().length > 0 &&
        view.urlInput.getValue().length > 0;
  },
  _buildMessagesForCpb:function(e) {
     var msgs = [];
     var details = e.responseJSON.details;
     for (var i = 0; i < details.length; i++)
     {
        var msg = {message:details[i]};
        msgs.push(msg);
     }
     return msgs;
  },
});