jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.rgf.api.BulkActionAPI");
jQuery.sap.require("sas.ui.cpb3.CPBService");

sap.ui.jsview("sas.risk.rgf.view.dialog.BulkActionDialog", {
	init: function(){
		this.hasSubmitted=false;
	},
	getControllerName : function() {
       return "sas.risk.rgf.view.dialog.BulkActionDialog";
   },
   createContent : function(controller) {
      this.rb = sas.risk.rgf.util.RGFUtil.getBundle();
      var innerLayout=this.createInnerLayout(controller);
      
      var layout = new sas.hc.ui.commons.layout.VerticalLayout({
         width : "100%",
         content : [ innerLayout]
      });
      this.createButtons(controller);
      return layout;
   },
   createButtons : function(controller) {
      this.cancelButton = new sas.risk.common.widgets.Button({
      	text: this.rb.getText("bulkAction.closeBtn.txt"),
      	press:$.proxy(controller.actionCancel,controller) 
      });
   },
	getDialogSettings : function() {
		return {
			contentWidth:"80%",
			endButton : this.cancelButton
		};
	},
	createInnerLayout : function(controller){
		var that=this;
		var viewData=this.getViewData();
		var onChangeFn=function(hasAtLeastOneAction,hasActionMessage){
			that.submitButton.setEnabled(hasAtLeastOneAction&&hasActionMessage);
		}
		this.allActionsSelect=new sas.hc.m.Select({
			change:function(event){
				var selectedText=event.getParameter("selectedItem").getText();
				_.each(viewData.cachedData.items,function(item){
					if(item.action.selectedKey){
						delete item.action.selectedKey;
					}
					var match=_.find(item.action.choices,function(choice){return choice.label===selectedText;});
					if(match){
						item.action.selectedKey=match.value;
						item.currentActivity.selectedKey=match.activity;
					}
				});
				that.bulkActionTable.doRefresh();
				var data=that.bulkActionTable.getSource().getData();
				var hasAtLeastOneAction=!!_.find(data,function(row){return !!row.action.selectedKey;});
				onChangeFn(hasAtLeastOneAction,!!that.changeReasonTextArea.getValue());
			}
		});
		var tableTopActionsPlugin=new sas.risk.slimtable.plugins.TablePlugin({
	     	position: sas.risk.slimtable.TableLayout.TOP_END,
	     	items:[new sas.risk.common.layout.flexlayout.FlexLayout({
   			items:[
      			    new sas.hc.m.Label({text:this.rb.getText("bulkAction.allActionsLbl.txt"),labelFor:this.allActionSelect}),
   			       this.allActionsSelect
   		      ]
      		}).addStyleClass("RGFBulkActionDiagAllActionsSelect")
      	]
	   });
	   tableTopActionsPlugin.handleDisabled=function(){
	   	
	   };
		this.bulkActionTable=new sas.risk.rgf.widget.RGFTable({
			formattingKey:viewData.formattingKey,
			title:this.rb.getText("bulkAction.table.title.txt"),
         meta: viewData.meta,
         data: viewData.data,
         clientSide:true,
         visibleRows: viewData.meta.dialogVisibleRows,
   		plugins:[tableTopActionsPlugin,
                  new sas.risk.slimtable.plugins.Counter()
         ],
         rowChange:function(){
				var data=that.bulkActionTable.getSource().getData();
				var hasAtLeastOneAction=!!_.find(data,function(row){return !!row.action.selectedKey;});
         	onChangeFn(hasAtLeastOneAction,!!that.changeReasonTextArea.getValue());
         }
      });
		var that=this;
	   this.changeReasonTextArea=new sas.hc.m.TextArea({
			required:true,
			rows: 2,
			width:"100%",
			maxLength:viewData.changeReasonMaxLen,
			value: viewData.changeReason,
			liveChange:function(e){
				var data=that.bulkActionTable.getSource().getData();
				var hasAtLeastOneAction=!!_.find(data,function(row){return !!row.action.selectedKey;});
				onChangeFn(hasAtLeastOneAction,!!e.getParameter("newValue"));
			}
		});
	   var changeReasonLabel= new sas.hc.m.Label({
	   	text:this.rb.getText("bulkAction.submitLbl.txt"),
	   	required:true,
	   	labelFor:this.changeReasonTextArea
   	}).addStyleClass("RGFDimLocFavNameLbl");
		this.submitButton=new sas.risk.common.widgets.Button({
			text:this.rb.getText("bulkAction.submitBtn.txt"),
			enabled:false,
			press:$.proxy(controller.actionSubmit,controller)
		}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
			alignSelf:sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.FlexEnd
		})).addStyleClass("sasPrimaryButton").addStyleClass("RGFBulkActionDiagSubmit");
		var inputLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
			width:"100%",
			flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
			items:[changeReasonLabel,this.changeReasonTextArea,this.submitButton]
		});
		var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
			width:"100%",
			flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
			items:[this.bulkActionTable,inputLayout]
		});
      return layout;
	}
});
sap.ui.controller("sas.risk.rgf.view.dialog.BulkActionDialog", {
	actionSubmit : function(){
		var view=this.getView();
		view.getDialog().setBusy(true);
		view.bulkActionTable.setEnabled(false);
		view.changeReasonTextArea.setEnabled(false);
		view.submitButton.setEnabled(false);
		var source=view.bulkActionTable.getSource();
		var data=source.getData();
		var selected=_.filter(data,function(row,index){row.rowIdx=index;return !!row.action.selectedKey});
		var changeReason=view.changeReasonTextArea.getValue();
		var items=_.map(selected,function(row){
			return {
				rowIdx:row.rowIdx,
				screenId:row.screenId,
				objectType:row.objectType,
				objectKey:row.objectKey,
				cpbForm:row.cpbForm,
				activityKey:row.currentActivity.selectedKey,//activityKey,
				transitionKey:row.action.selectedKey
			};
		});
		if(items.length){view.hasSubmitted=true;}
		var cpbService=sas.ui.cpb3.CPBService.getInstance(view.getViewData().serviceId);
		var uiContextAPI=cpbService.getUIContextAPI();
		var bulkActionAPI=new sas.risk.rgf.api.BulkActionAPI(uiContextAPI);
		bulkActionAPI.submitBulkAction(items,changeReason).done(function(items){
			_.each(data,function(row){
				var submittedItem=_.find(items,function(item){return item.rowIdx===row.rowIdx;});
				if(submittedItem){
					row.result=submittedItem.result;
				}else{
					row.result="";
				}
			});
			source.update(view.bulkActionTable.getQuery(),source.getData());
			view.getDialog().setBusy(false);
		});
	},
	actionCancel : function(){
		var view=this.getView();
		if(view.hasSubmitted){
			view.fireClosing({});
		}
		view.getDialog().close();
	}
});