jQuery.sap.require("sas.risk.rgf.widget.DualSelector");
jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsview("sas.risk.rgf.view.dialog.ColumnsCustomizationDialog", {
   getControllerName : function() {
      return "sas.risk.rgf.view.dialog.ColumnsCustomizationDialog";
   },
   createContent : function(controller) {
      this.rb = sas.risk.rgf.util.RGFUtil.getBundle();
      var that=this;
      this.viewData=this.getViewData();
      this.rgfTable=this.viewData.source;
      var oData=this.getCleansedData();
      this.model=new sap.ui.model.json.JSONModel({"columns":oData});
      
      var innerLayout=this.createInnerLayout();
      
      var layout = new sas.hc.ui.commons.layout.VerticalLayout({
         height : "100%",
         width : "100%",
         content : [ innerLayout]
      });
      $.proxy(this.createButtons(controller),this);
      return layout;
   },
   createButtons : function(controller) {
      this.okButton = new sas.risk.common.widgets.Button({
         text : this.rb.getText("table.dialog.button.ok.txt"),
         press : $.proxy(controller.actionOK,this,controller),
      });
      this.cancelButton = new sas.risk.common.widgets.Button({
         text : this.rb.getText("table.dialog.button.cancel.txt"),
         press : $.proxy(controller.actionCancel,this,controller),
      });
      this.defaultsButton = new sas.risk.common.widgets.Button({
         text : this.rb.getText("table.dialog.button.defaults.txt"),
         press : $.proxy(controller.actionDefaults,this,controller),
      });
   },
   getDialogSettings : function() {
      return {
      	contentWidth:"50%",
      	leftButton: this.okButton,
      	middleButton: this.cancelButton,
      	rightButton: this.defaultsButton
      };
   },
   getCleansedData : function(){
      var oData=[].concat(this.rgfTable.table.getSource().tableMeta.getColumns() || []);
      
      //Don't include the action column in the dual selector
      var actionColumn=this.rgfTable.table.getSource().tableMeta.getColumnByName("action");
      if(actionColumn){oData.splice(oData.indexOf(actionColumn),1);}
      
      //Don't include link type fields - they need to be non-hidden so we can search on them, but
      //we are not yet ready to handle displaying them.
      var filtered = _.filter(oData, function(col) {return !col.linkField });
      return filtered;
   },
   createInnerLayout : function(){
   	this.oDualSelector = new sas.risk.rgf.widget.DualSelector({
         contextPath: "/columns",
         labelField: "label",
         sourceLabel: this.rb.getText("table.dialog.columnCustomization.available.txt"),
         targetLabel: this.rb.getText("table.dialog.columnCustomization.displayed.txt"),
         showSearch: true,
         maxTargetLength: 30
      });
   	this.oDualSelector.attachDualSelectorCreationComplete($.proxy(this.handleDualSelectItems, this));
      this.oDualSelector.setModel(this.model);
      return this.oDualSelector;
   },
   handleDualSelectItems : function(){
      if (this.oDualSelector && this.oDualSelector.getModel()) {
         this.oDualSelector.selectItems(this.rgfTable.table.getSource().getTableMeta().getVisibleColumns());
      }
   }
});
sap.ui.controller("sas.risk.rgf.view.dialog.ColumnsCustomizationDialog", {
	onAfterRendering:function(){
		this.getView().getDialog()._reposition();
	},
	actionOK:function(controller){
		var rb=sas.risk.rgf.util.RGFUtil.getBundle();
      var dialog=controller.getView().getDialog();
      var displayedList=this.oDualSelector.getTargetList().getModel().getData().itemList;
      var i,colNames=[],preferenceCols=[];
      for(i=0;i<displayedList.length;i++){
      	colNames.push(displayedList[i].name);
      	preferenceCols.push(displayedList[i].rawName);
      }
      if(colNames.length===0){
      	sas.risk.common.widgets.MessageDetailsBox.error({message:rb.getText("table.toolbar.menu.columns.noColumns.txt")}, sas.config.appName, sas.risk.rgf.util.RGFUtil.getBundle());
      	return;
      }
      var preferenceKey=this.rgfTable.table.getSource().getTableMeta().preferenceKey;
      var preferenceValue=preferenceCols.join("|");
      var that=this;
      var serviceId=this.getViewData().serviceId;
      var queryAPI=(serviceId)?sas.ui.cpb3.CPBService.getInstance(serviceId).getQueryAPI():this.rgfTable.getParent().getCpbForm().getQueryAPI();
      queryAPI.updatePreferences(preferenceKey,preferenceValue).done(function(result){
      	if(!result){return;} //error case - when preference save failed
      	var rawColNames=result.value.split("|");
         var orderedCols=that.rgfTable.table.getSource().getTableMeta().getOrderedColumnsByNames(colNames);
         var sortedCols=that.rgfTable.table.getSource().getTableMeta().getSortedColumns();
         var rgfTableQuery=that.rgfTable.getQuery().clone();
         rgfTableQuery.setOrderedColumns(orderedCols);

         _.each(sortedCols, function(sortedCol){
            if (sortedCol.sortOrder === 0) {
               rgfTableQuery.setSort(sortedCol.name,sortedCol.sortDirection,sortedCol);
            } else if (sortedCol.sortOrder === 1) {
               rgfTableQuery.setSort2(sortedCol.name,sortedCol.sortDirection,sortedCol);
            } else if (sortedCol.sortOrder === 2) {
               rgfTableQuery.setSort3(sortedCol.name,sortedCol.sortDirection,sortedCol);
            }
         })

        // rgfTableQuery.setSortedColumns(sortedCols);


         that.rgfTable.fireEvent("tablePreferenceChanged",{"columns":orderedCols,"rawColumnNames":rawColNames}); //sequence is important here. This line must be before the next line.
         that.rgfTable.doApplyQuery(rgfTableQuery);
      });
      dialog.close();
   },
   actionCancel:function(controller){
      var dialog=controller.getView().getDialog();
      dialog.close();
   },
   actionDefaults:function(controller){
      var dialog=controller.getView().getDialog();
      if (this.oDualSelector && this.oDualSelector.getModel()) {
      	var defaultVisibleCols=this.rgfTable.table.getSource().getTableMeta().getDefaultVisibleColumns();
      	this.oDualSelector.oMoveManager.removeItemsFromSelectedList(this.oDualSelector.oTargetLayout.oTargetList.getItems(),false,true)
      	this.oDualSelector.getModel().setProperty("/columns",[]);
      	
      	var oData=controller.getView().getCleansedData();
         this.oDualSelector.getModel().setProperty("/columns",oData);
         
         var indexedData =  this.oDualSelector.getModel().getProperty("/columns");
         var i,selectedItems = [];
         for(i = 0; i < defaultVisibleCols.length; i++) {
         	var col=_.find(indexedData,function(item){return item.getColRawName()===defaultVisibleCols[i].getColRawName();})
            if (col) {
               selectedItems.push(col);
            }
         }
      	this.oDualSelector.selectItems(selectedItems);
      }
   }
});       