jQuery.sap.require("sas.risk.rgf.widget.dimloc.DimensionalLocation");
jQuery.sap.require("sas.risk.rgf.meta.formatter.FileSizeFormatter");
jQuery.sap.require("sas.risk.slimtable.util.RowActionUtil");
jQuery.sap.require("sas.risk.slimtable.widget.menu.AdvSearchItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.CustomizationInfoItem");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");

sap.ui.jsview("sas.risk.rgf.view.dialog.ContentHistoryDialog", {
    getControllerName : function() {
        return "sas.risk.rgf.view.dialog.ContentHistoryDialog";
    },
    createContent : function(controller) {
        this.addStyleClass("rgfObjectViewPopup");
        var viewData=this.getViewData();
        var that=this;
        var rb = sas.risk.rgf.util.RGFUtil.getBundle();
        
        //FORMATTERS
        this.formatter = {"JAVA.LANG.LONG_fileSize":new sas.risk.rgf.meta.formatter.FileSizeFormatter("JAVA.LANG.LONG_fileSize")}; 
        sas.risk.slimtable.meta.ColumnFormatterRegistry.registerFactory(this.formatter,"ContentHistoryDialog");
        
        this.createButtons(controller);
        
        this.layout=new sas.risk.common.layout.flexlayout.FlexLayout({
            flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
            items:[]
        });
        var getTableMenuItems = function(){
      		var menuItems=[new sas.risk.rgf.widget.menu.ColumnsItem({serviceId:viewData.serviceId})];
      		var customizationInfoMeta=viewData.meta.tableMetaHistory.customizationInfoMeta;
      		if(customizationInfoMeta){
      			var custInfoUrl=sas.risk.rgf.util.RGFUtil.getRestUrl()+"/utils/showCustomizationInfo/content/history/labelOverridePrefix/"+viewData.labelOverridePrefix;
      			if (customizationInfoMeta.fileName) {
      				custInfoUrl += "?fileName="+customizationInfoMeta.fileName;
      			}
      			menuItems.push(new sas.risk.rgf.widget.menu.CustomizationInfoItem({url:custInfoUrl,meta:customizationInfoMeta}));
      		}
      	   return menuItems;
        };
        this.table=new sas.risk.rgf.widget.RGFTable({
      	   formattingKey : viewData.formattingKey,
            data : $.proxy(this.fetchData, this),
            meta: viewData.meta.tableMetaHistory,
            clientSide:true,
            showActionColumn:true,
            visibleRows: viewData.meta.tableMetaHistory.dialogVisibleRows,
            selectionMode:sap.ui.table.SelectionMode.None,
            showSelectionControls:false,
            plugins:[new sas.risk.slimtable.plugins.Search(),
                     new sas.risk.slimtable.plugins.Filter(),
                     new sas.risk.slimtable.plugins.TableOptions({
                     	  menuItems: getTableMenuItems()
                       }),
                     new sas.risk.slimtable.plugins.Counter()
            ]
        });

        //ACTION COLUMN
        var actionColumn=sas.risk.slimtable.util.RowActionUtil.createActionColumn($.proxy(this.createActionMenuItems,this), this.table);
        this.table.addColumn(actionColumn);
        
        this.table.attachRowClick($.proxy(this.actionTableRowClicked,this));
        
        //When the table preference changes, update the local table meta json's column information accordingly
        this.table.attachEvent("tablePreferenceChanged",function(event){
      	  var colNames=event.getParameter("rawColumnNames");
      	  sas.risk.slimtable.meta.TableMeta.updateTableMetaJsonUserDisplayColumns(viewData.meta.tableMetaHistory,colNames);
        });
        
        this.layout.addItem(this.table);
        
        return this.layout;
    },
    createActionMenuItems : function(extra){
        var rb = sas.risk.rgf.util.RGFUtil.getBundle();
        var that=this;
        var viewData=this.getViewData();
        var downloadItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
           text : viewData.meta.downloadAction,
           callback : function(){that.actionRowDownload(this);}
        });
        
        return [downloadItem];
    },
    createButtons : function(controller) {
        var rb = sas.risk.rgf.util.RGFUtil.getBundle();
        var that=this;
        this.okButton = new sas.risk.common.widgets.Button({
            text : rb.getText("content.dialog.ok.txt"),
            press : $.proxy(controller.actionOk,controller),
        });
    },
    getDialogSettings : function() {
        var viewData=this.getViewData();
        return {
            beginButton : this.okButton,
            endButton : this.okButton,
            contentWidth:"50%"
        };
    },
    fetchData : function() {
       var that = this;
       var viewData = this.getViewData();
       
       this.getDialog().setBusyIndicatorDelay(0);
       this.getDialog().setBusy(true);
       
       var def;
       if (viewData.historyLink)
       {
          //history of a single file
          def = sas.risk.rgf.api.ContentManagementAPI.fetchAttachmentHistory(this.getViewData().historyLink);
       }
       else
       {
          //history across all files
          def = sas.risk.rgf.api.ContentManagementAPI.fetchAttachmentsHistory(this.getViewData().model,this.getViewData().meta.groupNumber);
       }
       def.done(function () {
          that.getDialog().setBusy(false);                      
       }).fail(function (e) {
      	 that.getDialog().setBusy(false);
          if (e.status && e.status === 404)
          {
             //The parent object no longer exists
             var errorJson = {"message" : viewData.meta.missingParentMsg};      
    	     sas.risk.common.widgets.MessageDetailsBox.error(errorJson, sas.config.appName, sas.risk.rgf.util.RGFUtil.getBundle());	
    	     that.getDialog().close();
          }
          else
          {
             sas.risk.rgf.util.RGFUtil.handleFail(e);
          }
       });
       return def;
    },
    actionRowDownload : function(item) {
       var rowData = item.getParent().getPayload().rowData;
       if (rowData)
       {
          sas.risk.rgf.api.ContentManagementAPI.downloadAttachmentContent(rowData);
       }
    },
    actionTableRowClicked : function(e) {
       //CLICKING ON ROW WILL DOWNLOAD
       var rowData = e.getSource().getSource().getData()[e.getParameter("rowIdx")];
       sas.risk.rgf.api.ContentManagementAPI.downloadAttachmentContent(rowData);
    },
    actionRowViewHistory : function(item) {
       var rowData = item.getParent().getPayload().rowData;
       var link = sas.risk.rgf.api.ContentManagementAPI.findLink(rowData, "history");
       if (link && link.href)
       {
          var viewData = this.getController().getView().getViewData();
          this.getController().getView().getDialog().close();

          viewData.historyLink=link;
          sas.risk.rgf.util.DialogManager.open("ContentHistoryDialog",viewData);
       }
    },
});
sap.ui.controller("sas.risk.rgf.view.dialog.ContentHistoryDialog", {
    onInit:function(){
        var that=this;
        var viewData=this.getView().getViewData();
    },
    actionOk:function(){
       this.getView().getDialog().close();
    }
});