jQuery.sap.require("sap.ui.commons.layout.MatrixLayout");
jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.risk.slimtable.widget.menu.SeparatorItem");
jQuery.sap.require("sas.risk.slimtable.widget.menu.AdvSearchItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.rgf.widget.menu.ExcelItem");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");

sap.ui.jsview("sas.risk.rgf.view.dialog.ExportToDataloadDialog", {
    getControllerName : function() {
        return "sas.risk.rgf.view.dialog.ExportToDataloadDialog";
    },
    createContent : function(controller) {
   	 this.viewData=this.getViewData();
   	 this.rgfTable=this.viewData.source;
       var innerLayout=this.createInnerLayout();
       
       var layout = new sas.hc.ui.commons.layout.VerticalLayout({
          height : "100%",
          width : "100%",
          content : [ innerLayout]
       });
       $.proxy(this.createButtons(controller),this);
       return layout;
   },
   createButtons : function(controller) {
      var rb = sas.risk.rgf.util.RGFUtil.getBundle();
      this.okButton = new sas.risk.common.widgets.Button({
         text : rb.getText("table.dialog.button.ok.txt"),
         press : $.proxy(controller.actionOk,this,controller)
     });
      this.cancelButton = new sas.risk.common.widgets.Button({
          text : rb.getText("table.dialog.button.cancel.txt"),
          press : $.proxy(controller.actionCancel,this,controller)
      });
   },
   getDialogSettings : function() {
      return {
         contentWidth:"50%",
         beginButton : this.okButton,
         endButton : this.cancelButton
      };
   },
   createInnerLayout : function(){
   	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
   	var innerLayout=new sap.ui.commons.layout.MatrixLayout({
   		layoutFixed: false
   	});
   	this.includeDimsCheckbox=new sas.hc.m.CheckBox({
   		text:rb.getText("table.exportToDataload.dimension.txt")
   	}).addStyleClass("RGFExportToDataloadCheckbox");
   	this.displayedColsOnlyCheckbox=new sas.hc.m.CheckBox({
   		selected: true,
   		text:rb.getText("table.exportToDataload.displayedColumns.txt")
		}).addStyleClass("RGFExportToDataloadCheckbox");
   	innerLayout.createRow(this.includeDimsCheckbox);
   	innerLayout.createRow(this.displayedColsOnlyCheckbox);
   	return innerLayout;
   }
});
sap.ui.controller("sas.risk.rgf.view.dialog.ExportToDataloadDialog", {
   actionOk:function(controller){
      var dialog=controller.getView().getDialog();
      dialog.setBusyIndicatorDelay(0);
      dialog.setBusy(true);
      var exportURL=this.rgfTable.getExportURL();
      if(!exportURL) {
         console.error("exportURL is not provided. Can't perform Export to dataload."); /*I18nOK:EMS*/
         return; 
      }
      
      
  		var component = this.viewData.source.getParent();
  		
  		var fields;
      if (this.viewData.fromLBO) {
     		var visibleColumns = component.tables[0].table.state.source.tableMeta.visibleColumns
     		for(i = 0; i < visibleColumns.length; i++) {
     			var varName = visibleColumns[i].name;
     			if (!fields) {
     				fields = "fields=" + varName;
     			} else {
     				fields = fields + "," + varName;
     			}
     		}
      }
  		
      var filter;
      if (this.viewData.fromLBO) {
      	var data = component.tables[0].table.state.source.data;
      	for(i = 0; i < data.length; i++) {
      		var key = data[i].key;
      		if (i == 0) {
      			filter = "filter=in(key," + key;
      		} else {
      			filter = filter + "," + key;
      		} 
      	}
      	filter = filter ? filter + ")" : "filter=in(key,-1)";
      }
      
   	
      var includeDims=this.includeDimsCheckbox.getSelected() ? true : false;
      var includeAllCols=this.displayedColsOnlyCheckbox.getSelected() ? false : true;
      var i = exportURL.indexOf("?");
      exportURL=(i>0)?exportURL+"&":exportURL+"?";
      exportURL=exportURL+"exportForDataLoad=true&includeAllCols="+includeAllCols+"&includeDims="+includeDims+"&"+this.rgfTable.query.toString();
      exportURL=sas.risk.rgf.util.URLParser.parseURL(exportURL).removeParam("start");
      exportURL=sas.risk.rgf.util.URLParser.parseURL(exportURL).removeParam("limit");
      
      // adding visible fields
      exportURL=(this.viewData.fromLBO && fields) ? exportURL+ (exportURL.indexOf("?") > 0 ? "&" : "?") + fields : exportURL;
      
      // adding the current data/rows in LBO table
      exportURL=(this.viewData.fromLBO && filter) ? exportURL+ (exportURL.indexOf("?") > 0 ? "&" : "?") + filter : exportURL;
      
  	   var xhr=new XMLHttpRequest();
  	   xhr.open("GET",exportURL,true);
  	   xhr.responseType="blob";
  	   xhr.onload=function(e){
  	   	if(this.status==200){
  	         var contentDispositionHeader = xhr.getResponseHeader("content-disposition");
  	         var fileName = contentDispositionHeader.slice(contentDispositionHeader.indexOf("filename")).replace("filename=\"", "").replace("\"","");
  	   		var blob=this.response;
  	   		if(window.navigator.msSaveBlob){
  	   			window.navigator.msSaveBlob(blob,fileName);
  	   		}else{
  		         var downloadUrl = URL.createObjectURL(blob);
  		         var a = document.createElement("a");
  				   a.href = downloadUrl;
  				   a.download = fileName;
  				   document.body.appendChild(a);
  				   a.click();
  	   		}
  	   	}
  	   };
  	   xhr.onerror=function(e){
  	   	console.log(e);
  	   	alert('Unable to download excel for dataload.'); /*I18nOK:EMS*/
  	   };
  	   xhr.send();
      dialog.close();
   },
   actionCancel:function(controller){
   	var dialog=controller.getView().getDialog();
   	dialog.close();
   }
});