jQuery.sap.require("sas.risk.rgf.meta.filter.FilterMetaFactory");
jQuery.sap.require("sas.risk.rgf.api.RGFTableAPI");
jQuery.sap.require("sas.hc.ui.commons.layout.MatrixLayout");
jQuery.sap.require("sas.risk.rgf.biz.View");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsview("sas.risk.rgf.view.dialog.SaveViewDialog", {
	getControllerName : function() {
       return "sas.risk.rgf.view.dialog.SaveViewDialog";
   },
   createContent : function(controller) {
   	this.viewData=this.getViewData();
   	this.rgfTable=this.viewData.source;
      this.rb = sas.risk.rgf.util.RGFUtil.getBundle();
      var innerLayout=this.createInnerLayout();//$.proxy(this.createInnerLayout,this)();
      var layout = new sas.hc.ui.commons.layout.VerticalLayout({
         height : "100%",
         width : "100%",
         content : [ innerLayout]
      });
      $.proxy(this.createButtons(controller),this);
      return layout;
   },
   createButtons : function(controller) {
      var rb = sas.risk.rgf.util.RGFUtil.getBundle();
      this.okButton = new sas.risk.common.widgets.Button({
         text:this.rb.getText("table.saveView.ok.txt"),
         enabled:false,
         press:$.proxy(controller.actionOk,this,controller) 
     });
     this.cancelButton = new sas.risk.common.widgets.Button({
         text:this.rb.getText("table.saveView.cancel.txt"),
         press:$.proxy(controller.actionCancel,controller) 
     });
   },
	getDialogSettings : function() {
		return {
			contentWidth:"40%",
			beginButton : this.okButton,
			endButton : this.cancelButton,
			initialFocus : this.viewNameInput
		};
	},
	createInnerLayout : function(){
		var that=this;
		var innerInputLayout=new sas.hc.ui.commons.layout.MatrixLayout({
			widths : [ "25%", "75%" ]
   	});
      //Duplicate name error message area
		var dummyLabel=new sas.hc.m.Label({text:""}).addStyleClass("SaveViewDialogLabel"); //dummy label
		this.duplicateNameTxt=new sas.hc.m.Text({
			visible: false,
   		text:this.rb.getText("table.saveView.duplicateName.msg.txt")
   	}).addStyleClass("SaveViewDialogDuplicateNameErrorTxt");
		innerInputLayout.createRow(dummyLabel, this.duplicateNameTxt);
		
      //View name field
      this.viewNameLabel=new sas.hc.m.Label({
      	text:this.rb.getText("table.saveView.viewName.label.txt"),
      	required:true
      }).addStyleClass("SaveViewDialogField");
      this.viewNameInput = new sas.hc.m.Input({
         required:true,
         liveChange:function(e){
             var newValue=e.getParameter("newValue");
             var dupName=_.find(that.viewData.rgfViews.views,function(view){return newValue===view.getName();});
             that.okButton.setEnabled(!dupName && !!newValue); //TODO: add error message explaining the problem
             that.replaceViewCheckBox.setVisible(!!dupName);
             that.duplicateNameTxt.setVisible(!!dupName);
         }
      }).addStyleClass("SaveViewDialogField");
      innerInputLayout.createRow(this.viewNameLabel, this.viewNameInput);
     
      //view description field
      this.viewDescLabel=new sas.hc.m.Label({
      	text:this.rb.getText("table.saveView.viewDescription.label.txt")
      }).addStyleClass("SaveViewDialogField");
      this.viewDescArea = new sas.hc.m.TextArea({
         autoExpandable: true
      }).addStyleClass("SaveViewDialogField");
      innerInputLayout.createRow(this.viewDescLabel, this.viewDescArea);
      
      //Set as default
      this.setAsDefaultViewCheckBox=new sas.hc.m.CheckBox({text:this.rb.getText("table.toolbar.menu.setAsDefaultView.txt")});
      // replace view check box
      this.replaceViewCheckBox=new sas.hc.m.CheckBox({
     	 	visible: false,
     	   select: function(e){
     	   	that.okButton.setEnabled(e.getParameters().selected);
     	   },
     	   text:this.rb.getText("table.saveView.replace.txt")
      });
      
      var innerLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
         items: [innerInputLayout, this.setAsDefaultViewCheckBox, this.replaceViewCheckBox],
         flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column
      });
      return innerLayout;
	}
});
sap.ui.controller("sas.risk.rgf.view.dialog.SaveViewDialog", {
	actionCancel:function(){
		this.getView().getDialog().close();
	},
	actionOk:function(controller){
		var dialogView=controller.getView();
		var dialog=dialogView.getDialog();
      dialog.setBusyIndicatorDelay(0);
      dialog.setBusy(true);
      var viewName=dialogView.viewNameInput.getValue();
      var viewDesc=dialogView.viewDescArea.getValue();
      this.tableMeta=this.rgfTable.table.getSource().getTableMeta();
      var columns=this.tableMeta.getVisibleColumns();
      var query=this.rgfTable.query;// TODO: too much information?!?!
      // query.sortedColumns=_.filter(query.orderedColumns, function(column){return column.sortDirection !== "None";})
      var setDefault=dialogView.setAsDefaultViewCheckBox.getSelected();
      var oldDefaultView=this.viewData.rgfViews.getDefaultView();
      var queryLocationPoints=query.getLocationPoints() ? query.getLocationPoints().split(",") : null;
      var newView=new sas.risk.rgf.biz.View(null,viewName,viewDesc,columns,
                      query.getSort().sortName,query.getSort().sortDirection,
                      query.getSort2().sortName,query.getSort2().sortDirection,
                      query.getSort3().sortName,query.getSort3().sortDirection,
                      query.getQueryFilter("SIMPLE"),query.getQueryFilter("ADVANCED"),
                      queryLocationPoints,setDefault,oldDefaultView.version,query.sortedColumns);
      
      var that=this; 
      if(controller.getView().replaceViewCheckBox.getVisible()){ //Replace an existing view with this new one
      	var oldView=_.find(this.viewData.rgfViews.views,function(view){return view.name===viewName;});
      	var oldViewIndex=this.viewData.rgfViews.views.indexOf(oldView);
      	newView.setKey(oldView.getKey());
      	sas.risk.rgf.api.RGFTableAPI.updateView(newView,this.viewData.rgfViews.getViewBaseURL()+"/"+oldView.getKey()).done(function(v){
      		$.when(sas.risk.rgf.meta.filter.FilterMetaFactory.createFilterFromJson(JSON.parse(v.simpleFilter||null),that.tableMeta),
      				 sas.risk.rgf.meta.filter.FilterMetaFactory.createFilterFromJson(JSON.parse(v.advancedFilter||null),that.tableMeta))
         	.done(function(filterSimple,filterAdvanced){
               var newTableMetaView=new sas.risk.rgf.biz.View(v.key,v.name,v.description,that.tableMeta.getOrderedColumnsByNames(v.fields),
                                                              v.sortColumn,v.sortDirection,
                                                              v.sortColumn2,v.sortDirection2,
                                                              v.sortColumn3,v.sortDirection3,
                                                               filterSimple,filterAdvanced,
                                                               v.dimensionalPoints,v.default,v.version,
                                                               that.rgfTable.query.sortedColumns);
            	if(newTableMetaView.getIsDefault() && oldDefaultView){
            		oldDefaultView.setIsDefault(false);
            	}
            	that.viewData.rgfViews.views[oldViewIndex]=newTableMetaView;
            	that.viewData.rgfViews.currentView=newTableMetaView;
            	that.viewData.rgfViews.updateViewList();
            	that.viewData.rgfViews.viewList.fireChange({
               	selectedItem: that.viewData.rgfViews.viewList.getSelectedItem()
            	});
               dialog.close();
         	});
         });
      }
      else{ //Save this new view
         sas.risk.rgf.api.RGFTableAPI.createView(newView,this.viewData.rgfViews.getViewBaseURL()).done(function(v){
         	$.when(sas.risk.rgf.meta.filter.FilterMetaFactory.createFilterFromJson(JSON.parse(v.simpleFilter||null),that.tableMeta),
         			 sas.risk.rgf.meta.filter.FilterMetaFactory.createFilterFromJson(JSON.parse(v.advancedFilter||null),that.tableMeta))
         	.done(function(filterSimple,filterAdvanced){
               var newTableMetaView=new sas.risk.rgf.biz.View(v.key,v.name,v.description,that.tableMeta.getOrderedColumnsByNames(v.fields),
                                                               v.sortColumn,v.sortDirection,
                                                               v.sortColumn2,v.sortDirection2,
                                                               v.sortColumn3,v.sortDirection3,
                                                               filterSimple,filterAdvanced,
                                                               v.dimensionalPoints,v.default,v.version,
                                                               that.rgfTable.query.sortedColumns);
	           	if(newTableMetaView.getIsDefault() && oldDefaultView){
	           		oldDefaultView.setIsDefault(false);
	           	}
	           	that.viewData.rgfViews.addView(newTableMetaView);
	           	that.viewData.rgfViews.currentView=newTableMetaView;
	           	that.viewData.rgfViews.updateViewList();
	           	that.viewData.rgfViews.viewList.fireChange({
	              	selectedItem: that.viewData.rgfViews.viewList.getSelectedItem()
	           	});
	           	dialog.close();
         	});
         });
      }
	}
});    