jQuery.sap.declare("sas.risk.rgf.widget.ObjectChooser");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.cpb3.widget.InputField");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.risk.slimtable.widget.Item");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.risk.rgf.util.DialogManager");
jQuery.sap.require("sas.risk.rgf.api.CommonAPI");
jQuery.sap.require("sas.risk.slimtable.meta.filter.AndFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.StringFilter");
jQuery.sap.require("sas.risk.common.widgets.Button");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.rgf.widget.ObjectChooser", {
	metadata:{
		properties:{
			key:{type:"string"},
			collectionName:{type:"string"},
			enabled:{type:"boolean",defaultValue:true},
			placeholderTxt:{type:"string"},
			dialogTitleTxt:{type:"string"},
			searchAttr:{type:"string",defaultValue:"name"},
			icon : {type : "sap.ui.core.URI", group : "Appearance", defaultValue : sas.icons.HC.SEARCH},
			baseFilter:{type:"any"},
			valueLabel : {type : "string", group : "Data", defaultValue : null, bindable : "bindable"},
			value : {type : "any", group : "Data", defaultValue : null, bindable : "bindable"},
			serviceId:{type:"string"}
		},
		events:{
			change : {
				parameters : {
					newValue : {type : "string"}
				}
			},
			press : {}
		}
	},
 	constructor:function(){
 		this.field = new sas.ui.cpb3.widget.InputField({
 	      showSuggestion: true,
 	      filterSuggests : false
 		});
 		this.button=new sas.risk.common.widgets.Button({icon:sas.icons.HC.SEARCH});
 		sas.risk.common.utils.CompositeUtil.addComposite(this,"valueLabel",this.field,"value");
 		sas.risk.common.utils.CompositeUtil.addComposite(this,"press",this.button);
 		sas.risk.common.utils.CompositeUtil.addComposite(this,"icon",this.button);
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.addStyleClass("RGFChooser");
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});
sas.risk.rgf.widget.ObjectChooser.prototype.init = function() {
   var that=this;
   this.field.attachSuggest(function(e){
      var v = e.getParameter("suggestValue");
      var f = e.getSource();
      sas.risk.rgf.api.CommonAPI.searchCollection(that.getCollectionName(),that.buildFilter(v)).done(function(data){
         f.removeAllSuggestionItems();
         _.each(data, function(item){
            f.addSuggestionItem(that.createSuggestionItem(item));
         });
      });
   });
   //Cannot use both "attachSuggestionItemSelected" and "attachChange" events. timing issue bug on base code. 
   //Therefore adding the attachSuggestionItemSelected logic and generic attachChange logic all in one "attachChange".
   this.field.attachChange(function(item){
   	if(that.field._oList && that.field.getSuggestionItems() && that.field.getSuggestionItems().length>0){ //If this change is coming from selecting a suggested item
   		if(that.field._oList.getSelectedItem()){ //selection through key press
   			var selItem=that.field._oList.getSelectedItem();
   			var index=that.field._oList.indexOfItem(selItem);
   			var selectedItem=that.field.getSuggestionItems()[index];
   			that.setValue(selectedItem.getPayload());
   			that.fireChange({newValue:selectedItem.getPayload()});
   		}else{
   			var selection=$("#"+that.field._oList.getId()+">ul>li:hover");//selection through mouse click
   			if(!selection || selection.length===0){//If user didn't select anything from the suggested items then we don't want to capture the user entered text as the value for user field
   				that.fireChange({newValue:null});
   				return;
   			}else{//Hack to get the suggestion selected item
   				var selectedItemId=$("#"+that.field._oList.getId()+">ul>li:hover")[0].id;
   				var suggestionItems=that.field.getSuggestionItems();
   				var selectedSuggestionItem=_.find(suggestionItems,function(item){return selectedItemId.startsWith(item.getId());});
               if (selectedSuggestionItem===undefined && suggestionItems.length == 1) {
                  selectedSuggestionItem=suggestionItems[0];
               }
   				that.setValue(selectedSuggestionItem.getPayload());
   				that.fireChange({newValue:selectedSuggestionItem.getPayload()});
   			}
   		}
   	}else{
   		that.setValue(null);
   		that.fireChange({newValue:null});
   	}
   });
   this.button.attachPress(function(e){
      sas.risk.rgf.api.CommonAPI.getTableMeta(that.getCollectionName()).done(function(meta){
         var rb=sas.risk.rgf.util.RGFUtil.getBundle();
         sas.risk.rgf.util.DialogManager.open("ObjectChooserDialog",{
            serviceId:that.getServiceId(),
            collectionName:that.getCollectionName(),
            chooserTableMeta:meta,
            filter:that.buildFilter(),
            title:that.getDialogTitleTxt()||rb.getText("objectChooser.dialog.title"),
            formattingKey:sas.risk.rgf.util.RGFUtil.buildDialogFormattingKey("ObjectChooserDialog")
         },$.proxy(that.selectAction,that),that);
      });
	});
   this.addItem(this.field);
   this.addItem(this.button);
};
sas.risk.rgf.widget.ObjectChooser.prototype.setEnabled = function(value){
	this.setProperty("enabled",value);
	this.field.setEnabled(value);
	if(value===false){this.field.setPlaceholder("");}
	this.button.setVisible(value);

};
sas.risk.rgf.widget.ObjectChooser.prototype.attachPress = function(fn) {
   var that=this;
   _.each(this.button.mEventRegistry["press"],function(eventListener){//clear out previously attached press events
   	that.button.detachEvent("press",eventListener.fFunction,eventListener.oListener);
   });
   this.attachEvent("press",fn);
};
sas.risk.rgf.widget.ObjectChooser.prototype.onBeforeRendering = function() {
   var rb = sas.risk.rgf.util.RGFUtil.getBundle();
   var placeholderTxt=this.getPlaceholderTxt()||rb.getText("objectChooser.placeholder.txt");
   this.field.setPlaceholder(this.getEnabled()?placeholderTxt:"");
   this.button.setIcon(this.getIcon());
};
sas.risk.rgf.widget.ObjectChooser.prototype.createSuggestionItem=function(item){
	return new sas.risk.slimtable.widget.Item({
      key: item.key,
      text: this.createItemLabel(item),
      payload: item
   })
};
sas.risk.rgf.widget.ObjectChooser.prototype.selectAction=function(e){
	var item=e.getParameter("item");
	this.field.removeAllSuggestionItems();
	this.field.setValue(this.createItemLabel(item));
	this.setValue(item);
   this.fireChange({newValue:item});
};
sas.risk.rgf.widget.ObjectChooser.prototype.createItemLabel=function(item){
	return item[this.getSearchAttr()]||(item.displayNm + ' (' + item.userId + ')');
};
sas.risk.rgf.widget.ObjectChooser.prototype.setKey=function(key){
	this.setProperty("key",key);
	var that=this;
	sas.risk.rgf.api.CommonAPI.loadCollection(this.getCollectionName(),[key]).done(function(items){
		that.field.setValue(that.createItemLabel(items[0]));
	});
};
sas.risk.rgf.widget.ObjectChooser.prototype.buildFilter = function(value){
	var baseFilter=this.getBaseFilter();
	if(value){
		var extraFilter=new sas.risk.slimtable.meta.filter.StringFilter(this.getSearchAttr(),"contains",value);
		if(baseFilter){
			return new sas.risk.slimtable.meta.filter.AndFilter([extraFilter,baseFilter]);
		}else{
			return extraFilter;
		}
	}
	return baseFilter?baseFilter:null;
};
sas.risk.common.utils.CompositeUtil.mixinPropertySupport(sas.risk.rgf.widget.ObjectChooser);
sas.risk.common.utils.CompositeUtil.mixinEventSupport(sas.risk.rgf.widget.ObjectChooser);