jQuery.sap.declare("sas.risk.rgf.widget.RGFFacetedSearch");
jQuery.sap.require("sas.risk.common.widgets.search.FacetedSearch");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.hc.m.DatePicker");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.risk.rgf.util.RGFUtil");
jQuery.sap.require("sas.risk.common.widgets.search.SearchAttachmentItem");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.hc.m.ResponsivePopover");
jQuery.sap.require("sas.risk.rgf.api.GlobalSearchAPI");
jQuery.sap.require("sas.risk.rgf.widget.ObjectChooser");
jQuery.sap.require("sas.ui.cpb3.widget.HTML");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");

sas.risk.common.widgets.search.FacetedSearch.extend("sas.risk.rgf.widget.RGFFacetedSearch",{
	renderer:sas.risk.common.layout.flexlayout.FlexLayoutRenderer
});
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createItemWidget=function(item){
	var itemWidget;
	if(item instanceof sas.risk.common.widgets.search.SearchAttachmentItem){
		itemWidget=this.createAttachmentItemWidget(item);
	}else{
		itemWidget=this.createObjectItemWidget(item);
	}
	itemWidget.data("payload",item);
	return itemWidget;
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.getResourceBundle=function(){
	if(!this.rb){
		this.rb = sas.risk.rgf.util.RGFUtil.getBundle();
	}
	return this.rb;
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createAttachmentItemWidget=function(item){
	var itemWidget=this.createObjectItemWidget(item);
	var layout=itemWidget.getContent()[0];
	var row=layout.getRows()[0];
	var cell=row.getCells()[2];
	var title=cell.getContent()[0];
	var flex=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%"
	});
	title.setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
		flexGrow:2
	}));
	var attachButton=new sas.risk.common.widgets.Button({
		text:item.getAttachmentName(),
		icon:sas.icons.HC.ATTACH,
		type:sap.m.ButtonType.Unstyled,
		tooltip:this.getResourceBundle().getText("global.search.download.attachment.fmt.txt",[item.getAttachmentName()]),
		press:$.proxy(this.downloadAttachment,this,item)
	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
		flexShrink:0
	}));
	attachButton.ontap=function(e){
		e.stopPropagation();//prevent the row select from triggering and navigating away
		return sap.m.Button.prototype.ontap.apply(this,arguments);
	};
	flex.addItem(title);
	flex.addItem(attachButton);
	cell.removeAllContent();
	cell.addContent(flex);
	return itemWidget;
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype._navigate=function(e){
	if(e.getParameter("selected")){
		var listItem=e.getParameter("listItem");
		var item=listItem.data("payload");
		if(item.getRouteName() && item.getParentKey()){
			this.navigateByRouteName(item.getRouteName(),item.getParentKey());
			this.fireResultSelected({selectedItem:item,results:this.getResults()});
		}else{
			console.error("SOLR Index needs reindexing. There is no navigation route for this item");
		}
	}
};

sas.risk.rgf.widget.RGFFacetedSearch.prototype._search=function(params){
	this.fireSearchChange({filter:params});
	return sas.risk.rgf.api.GlobalSearchAPI.search(params);
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.downloadAttachment=function(item){
	sas.risk.rgf.api.GlobalSearchAPI.downloadAttachmentLink(item.getAttachmentUri());
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.openDetails=function(button,item){
	var detailsTitle=this.createObjectDetailsTitle(item).addStyleClass("RGFDetailsHeader");
	var detailsCell=detailsTitle.getRows()[0].getCells()[3];
	detailsCell.getParent().removeCell(detailsCell);
	var highlights=new sap.ui.commons.layout.MatrixLayout({columns:2,layoutFixed:false}).addStyleClass("RGFDetailsBody");
	_.each(item.getHighlights(),function(highlight){
		var keyCell=new sap.ui.commons.layout.MatrixLayoutCell({content:[new sap.m.Label({text:highlight.getLabel()||highlight.getName()})]});
		keyCell.addStyleClass("RGFCellKey");
		var snippets=highlight.getSnippets().slice(0);
		var txt=snippets.shift();
		txt=txt.split("@!$PRE$!@").join("<span class=\'RGFHighlight\'>");
		txt=txt.split("@!$POST$!@").join("</span>");
		var firstSnippetCell=new sap.ui.commons.layout.MatrixLayoutCell({content:[new sas.ui.cpb3.widget.HTML({content:"<div>"+txt+"</div>"})]});
		highlights.addRow(new sap.ui.commons.layout.MatrixLayoutRow({cells:[keyCell,firstSnippetCell]}).addStyleClass("RGFSearchDetailsRow"));
		_.each(snippets,function(snippet){
			var blankCell=new sap.ui.commons.layout.MatrixLayoutCell();
			var snippetCell=new sap.ui.commons.layout.MatrixLayoutCell({content:[new sas.ui.cpb3.widget.HTML({content:"<div>"+snippet+"</div>"})]});
			highlights.addRow(new sap.ui.commons.layout.MatrixLayoutRow({cells:[blankCell,snippetCell]}));
		});
	});
	var highRows=highlights.getRows();
	highRows[highRows.length-1].addStyleClass("RGFSearchDetailsRowEnd");
	var detailsLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		items:[
       	detailsTitle,
       	highlights
      ]
	}).addStyleClass("RGFSearchDetailsMain");
   var detailsPopup = new sas.hc.m.ResponsivePopover({
   	showHeader:false,
   	verticalScrolling:true,
      placement: "Horizontal",
      modal: false,
      content: [detailsLayout],
//      afterClose:function(){ //uncomment if too many orphan popup divs are left behind
//      	this.destroy();
//      }
   });
   detailsPopup.addStyleClass("RGFSearchDetailsPopover");
	detailsPopup.openBy(button);
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createObjectDetailsTitle=function(item){
	var iconCell=new sap.ui.commons.layout.MatrixLayoutCell({rowSpan:2}).addStyleClass("RGFSearchItemObjectIcon");
	iconCell.addContent(new sas.hc.ui.core.Icon({
		src: this.getIconByType(item.getType())
	}));
	var titleCell=new sap.ui.commons.layout.MatrixLayoutCell({colSpan:4}).addStyleClass("RGFSearchItemObjectTitle");
	titleCell.addContent(new sap.ui.commons.TextView({text:item.getObjectName()}));
	var userCell=new sap.ui.commons.layout.MatrixLayoutCell({colSpan:2}).addStyleClass("RGFSearchItemObjectUser");
	userCell.addContent(new sap.ui.commons.TextView({text:item.getLastModifiedBy()}));
	var dateCell=new sap.ui.commons.layout.MatrixLayoutCell({colSpan:2}).addStyleClass("RGFSearchItemObjectDate");
	if (item.getLastModifiedDttm())
		dateCell.addContent(new sap.ui.commons.TextView({text:sas.risk.rgf.util.RGFUtil.toLocaleDateTimeString(item.getLastModifiedDttm())}));
	var idCell=new sap.ui.commons.layout.MatrixLayoutCell({}).addStyleClass("RGFSearchItemObjectId");
	idCell.addContent(new sap.ui.commons.TextView({text:item.getObjectId()}));
	var typeCell=new sap.ui.commons.layout.MatrixLayoutCell({}).addStyleClass("RGFSearchItemObjectType");
	typeCell.addContent(new sap.ui.commons.TextView({text:item.getTypeLabel()}));
	var detailsCell=new sap.ui.commons.layout.MatrixLayoutCell({}).addStyleClass("RGFSearchItemObjectDetails");
	if(item.getHighlights().length){
		var detailsButton=new sas.risk.common.widgets.Button({icon:sas.icons.HC.OBJECTINSPECTOR});
		detailsButton.attachPress($.proxy(this.openDetails,this,detailsButton,item));
		detailsButton.ontap=function(e){
			e.stopPropagation();//prevent the row select from triggering and navigating away
			return sap.m.Button.prototype.ontap.apply(this,arguments);
		};
		detailsCell.addContent(detailsButton);
	}
	var row1=new sap.ui.commons.layout.MatrixLayoutRow({
		cells:[iconCell,idCell,titleCell,detailsCell]
	});
	var row2=new sap.ui.commons.layout.MatrixLayoutRow({
		cells:[typeCell,userCell,dateCell]
	});
	var layout=new sap.ui.commons.layout.MatrixLayout({
		columns:7,
		rows:[row1,row2]
	}).addStyleClass("RGFSearchItemObject");
	return layout;
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createObjectItemWidget=function(item){
	return new sap.m.CustomListItem({
		content:this.createObjectDetailsTitle(item)
	});
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.navigateByRouteName=function(routeName,key){
	var key=key.split("|")[1];
	var app=sas.ui.mashup.App.getInstance();
	var currentTemplate=app.getCurrentTemplate();
	var isObjectEditor=currentTemplate.isInstanceOf(sas.ui.cpb3.template.CpbViewToolbarTemplate);
	var isObjectScreenEditor=!isObjectEditor && currentTemplate.isInstanceOf(sas.ui.mashup.template.ViewToolbarTemplate) && app.navUtil.navcurrent.index>0;
	var isQueryScreenEditor=!isObjectEditor && currentTemplate.isInstanceOf(sas.ui.mashup.template.ViewToolbarTemplate) && app.navUtil.navcurrent.index===0;
	
	var myDefaultNav = {
			routeName:routeName,
			context:{
				event_name:"object_selected",
				type:routeName,
				key:key
			},
			index:sas.ui.cpb3.nav.NavUtil.OBJROUTE_EDIT,
			navstyle:sas.ui.mashup.util.NavUtil.NAVSTYLE_REFRESH_ON_ARRIVAL
	};

	app.navUtil.navTo(
		routeName,
		{
			event_name:"object_selected",
			type:routeName,
			key:key
		},
		sas.ui.cpb3.nav.NavUtil.OBJROUTE_EDIT,
		sas.ui.mashup.util.NavUtil.NAVSTYLE_REFRESH_ON_ARRIVAL
	);
	if(isObjectEditor || isQueryScreenEditor){
		app.navUtil.popCurrentNavigation(myDefaultNav);
	}else if(isObjectScreenEditor){
		app.navUtil.popCurrentNavigation(myDefaultNav);
		app.navUtil.popCurrentNavigation(myDefaultNav);
	}
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.getIconByType=function(type){
	switch(type){
		case "policy":
			return sas.icons.HC.ALLOCATIONRULE;
		case "assessment":
			return sas.icons.HC.SCALES;
		case "controlInstance":
			sas.icons.HC.SECUREDFOLDER;//TODO: need icon for control
		case "customObject1":/*model*/
			return sas.icons.HC.MODEL;
		case "customObject2":/*model usage*/
			return sas.icons.HC.MODELRETRAIN;
		case "customObject3":/*non-model*/
			return sas.icons.HC.CALCULATEDMEASURE;
		case "customObject4":/*model change request*/
			return sas.icons.HC.CHANGE;
		case "customObject5":/*model candidate assessment*/
			return sas.icons.HC.MCA;
		case "customObject6":/*policy exception*/
			return sas.icons.HC.EXCEPTIONDEFINITION;
		case "customObject7":/*mrm correspondence*/
			return sas.icons.HC.CONVERSATION;
		case "customObject8":/*model variable*/
			return sas.icons.HC.PARAMETERS;
		case "customObject11":/*model review*/
			return sas.icons.HC.MODELREVIEW;
		case "customObject12":/*finding*/
			return sas.icons.HC.ISSUE;
		case "customObject13":/*action plan*/
			return sas.icons.HC.TASKLIST;
		case "customObject19":/*trim*/
			return sas.icons.HC.PERFORMANCEINSIGHT;
		case "customObject36":/*Audit*/
			return sas.icons.HC.ANALYZE;
		case "customObject86":/*Incident*/
			return sas.icons.HC.EVENT;
		case "customObject87":/*Financial Effect*/
			return sas.icons.HC.CURRENCY;
		case "customObject89":/*Direct Recovery*/
			return sas.icons.HC.DIRECTCASHRECOVERY;
		case "customObject90":/*InsuranceRecovery*/
			return sas.icons.HC.CHECK;
		case "customObject96":/*Risk Profile*/
			return sas.icons.HC.RISK;
		case "customObject97":/*Control*/
			return sas.icons.HC.CONTROL;
		case "customObject98":/*Cause*/
			return sas.icons.HC.RECORD;
		case "customObject100":/*Assessment*/
			return sas.icons.HC.SCALES;
		case "customObject102":/*Policy*/
			return sas.icons.HC.CONTRACT;
		case "customObject103":/*PolicyResponse*/
			return sas.icons.HC.POLICYRESPONSE;
		case "customObject105":/*Test Definition*/
			return sas.icons.HC.TEMPLATE;
		case "customObject107":/*Test*/
			return sas.icons.HC.ATTRIBUTES;
		case "customObject109":/*Correspondence*/
			return sas.icons.HC.CONVERSATION;
		case "customObject110":/*Issue*/
			return sas.icons.HC.ISSUE;
		case "customObject111":/*Action Plan*/
			return sas.icons.HC.TASKLIST;
		case "customObject113":/*Kri Indicator*/
			return sas.icons.HC.LINEPLOT;
		case "customObject114":/*Kri Observation*/
			return sas.icons.HC.TIMESERIESCHART;
//		case "customObject":/**/
//			return sas.icons.HC.;
		case "potentialImpact":
			return sas.icons.HC.EXPOSUREANALYSISCHART;
		case "risk":
			return sas.icons.HC.MODELRISK;
	}
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createFacetWidget=function(facet,filter){
	switch(facet.getName()){
		case "lastModifiedBy":
			return this.createLastModifiedByFacetWidget(facet,filter);
		case "options":
			return this.createOptionsFacetWidget(facet,filter);
		case "lastModifiedDttm":
			return this.createLastModifiedDttmFacetWidget(facet,filter);
		default:
			return sas.risk.common.widgets.search.FacetedSearch.prototype.createFacetWidget.apply(this,arguments);
	}
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createLastModifiedByFacetWidget=function(facet,filter){
	var facetWidget=sas.risk.common.widgets.search.FacetedSearch.prototype.createFacetWidget.apply(this,arguments);
	var section=facetWidget.getSections()[0];
	var rb=this.getResourceBundle();
	var userChooser=new sas.risk.rgf.widget.ObjectChooser({
		 collectionName:"users",
		 placeholderTxt:rb.getText("objectChooser.users.placeholder.txt"),
		 dialogTitleTxt:rb.getText("objectChooser.users.dialog.title"),
		 icon:sas.icons.HC.USER,
		 searchAttr:"displayNm",
		 change:$.proxy(this.doSearch,this,false)
	}).addStyleClass("RGFFacetedSearchLastModChooser");
	section.addContent(userChooser);
	var origUpdateFilter=facetWidget.updateFilter;
	var origReset=facetWidget.reset;
	facetWidget.updateFilter=function(filter){
		origUpdateFilter.call(facetWidget,filter);
		var value=userChooser.getValue();
		if(value!=null){
			var currentVal=filter[facet.getName()];
			currentVal=currentVal?currentVal+"|"+value.key:value.key;
			filter[facet.getName()]=currentVal;
		}
	};
	facetWidget.reset=function(){
		userChooser.setValue(null);
		userChooser.setValueLabel("");
		origReset.call(facetWidget);
	};
	return facetWidget;
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createOptionsFacetWidget=function(facet,filter){
	var facetWidget=this.createFacetEmptySection(facet);
	var section=facetWidget.getSections()[0];
	var selIndex=0;
	if(_.has(filter,'attachment')){
		selIndex=filter.attachment?1:2;
	}
	if(selIndex){
		facetWidget.getSections()[0].getExtra().setEnabled(true);
		this.resetAllLink.setEnabled(true);
	}
	var that=this;
	var rb=this.getResourceBundle();
	var rbkey="global.search.option.attachment";
	var items=[
     new sas.hc.ui.core.Item({key:"all",tooltip:rb.getText(rbkey+".all.tooltip.txt"),text:rb.getText(rbkey+".all.txt")}),
     new sas.hc.ui.core.Item({key:"attachonly",tooltip:rb.getText(rbkey+".attachonly.tooltip.txt"),text:rb.getText(rbkey+".attachonly.txt")}),
     new sas.hc.ui.core.Item({key:"noattach",tooltip:rb.getText(rbkey+".noattach.tooltip.txt"),text:rb.getText(rbkey+".noattach.txt")})
	];
	var selectedItem=items[selIndex];
	var attachmentSelect=new sas.risk.slimtable.widget.Select({
		name:"attachment",
		items:items,
		change:function(){
			that.doSearch();
		}
	});
	attachmentSelect.setSelectedKey(selectedItem.getKey());
	var attachmentLabel=new sas.hc.m.Label({
		text:rb.getText(rbkey+".txt"),
		labelFor:attachmentSelect
	}).addStyleClass("RGFFacetedSearchOptionsLabel");
	var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
		items:[attachmentLabel,attachmentSelect]
	});
	section.addContent(layout);
	var origUpdateFilter=facetWidget.updateFilter;
	var origReset=facetWidget.reset;
	facetWidget.updateFilter=function(filter){
		switch(attachmentSelect.getSelectedKey()){
			case "all":
				delete filter["attachment"];
			break;
			case "attachonly":
				filter.attachment=true;
				break;
			case "noattach":
				filter.attachment=false;
				break;
		}
	};
	facetWidget.reset=function(){
		attachmentSelect.setSelectedKey("all");
		that.doSearch();
	};
	return facetWidget;
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.createLastModifiedDttmFacetWidget=function(facet,filter){
	var that=this;
	var facetWidget=this.createFacetEmptySection(facet);
	var section=facetWidget.getSections()[0];
	var locale=sas.risk.rgf.util.RGFUtil.getLocale();
	var inputFormat = sap.ui.core.format.DateFormat.getInstance({style:"medium"});
	var getDateValue=function(){
		return inputFormat.parse(this.getValue());
	};
	var setDateValue=function(oDate){
		return this.setValue(inputFormat.format(oDate));
	};
	var fromPicker=new sas.hc.m.DatePicker({displayFormat:"medium",valueFormat:"medium"});
	fromPicker.getDateValue=getDateValue;
	fromPicker.setDateValue=setDateValue;
	var toPicker=new sas.hc.m.DatePicker({displayFormat:"medium",valueFormat:"medium"});
	toPicker.getDateValue=getDateValue;
	toPicker.setDateValue=setDateValue;
	var rb=this.getResourceBundle();
	var fromLabel=new sas.hc.m.Label({
		text:rb.getText("global.search.daterange.from.txt"),
		labelFor:fromPicker
	});
	var toLabel=new sas.hc.m.Label({
		text:rb.getText("global.search.daterange.to.txt"),
		labelFor:toPicker
	});
	fromPicker.attachChange(function(e){
		var val=e.getSource().getDateValue();
		toPicker.setMinDate(val);
		if(val){
			var toVal=toPicker.getDateValue();
			if(toVal && val.getTime()>toVal.getTime()){
				toPicker.setDateValue(val);
			}
		}
		that.doSearch();
	});
	toPicker.attachChange(function(e){
		var val=e.getSource().getDateValue();
		fromPicker.setMaxDate(val);
		if(val){
			var fromVal=fromPicker.getDateValue();
			if(fromVal && val.getTime()<fromVal.getTime()){
				fromPicker.setDateValue(val);
			}
		}
		that.doSearch();
	});
	var matrix=new sap.ui.commons.layout.MatrixLayout({layoutFixed:false}).addStyleClass("RGFFacetDateRange");
	matrix.createRow(fromLabel,fromPicker);
	matrix.createRow(toLabel,toPicker);
	section.addContent(matrix);
	
	var origUpdateFilter=facetWidget.updateFilter;
	var origReset=facetWidget.reset;
	facetWidget.updateFilter=function(filter){
		var val=fromPicker.getDateValue();
		if(val){
			val.setHours(0,0,0,0);
			filter.lastModifiedDttmStart=val.toISOString();
		}else{
			delete filter["lastModifiedDttmStart"];
		}
		val=toPicker.getDateValue();
		if(val){
			val.setHours(23,59,59,0);
			filter.lastModifiedDttmEnd=val.toISOString();
		}else{
			delete filter["lastModifiedDttmEnd"];
		}		
	};
	facetWidget.reset=function(){
		fromPicker.setValue(null);
		toPicker.setValue(null);
		that.doSearch();
	};
	if(filter["lastModifiedDttmStart"]){
		fromPicker.setDateValue(new Date(filter["lastModifiedDttmStart"]));
		toPicker.setMinDate(new Date(filter["lastModifiedDttmStart"]));
	}
	if(filter["lastModifiedDttmEnd"]){
		toPicker.setDateValue(new Date(filter["lastModifiedDttmEnd"]));
		fromPicker.setMaxDate(new Date(filter["lastModifiedDttmEnd"]));
	}
	if(fromPicker.getValue() || toPicker.getValue()){
		facetWidget.getSections()[0].getExtra().setEnabled(true);
		this.resetAllLink.setEnabled(true);
	}
	return facetWidget;
};
sas.risk.rgf.widget.RGFFacetedSearch.prototype.doSearchError=function(e){
	sas.risk.rgf.util.RGFUtil.handleFail(e);
	this.setBusy(false);
};