jQuery.sap.declare("sas.risk.slimtable.SlimTable");
jQuery.sap.require("sas.risk.slimtable.SlimTableState");
jQuery.sap.require("sas.risk.slimtable.util.SlimTableUtil");
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-core');
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-widget');
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-mouse');
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-draggable');
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-droppable');
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-sortable');
jQuery.sap.require("sas.risk.slimtable.SlimTableKeyboard");

sas.risk.slimtable.ClickScope={
	Row : "Row",
	Cell : "Cell"
};
sas.risk.slimtable.SortScope={
   MultiSort : "MultiSort",
   SingleSort :"SingleSort",
   None : "None"
};

sap.ui.core.Control.extend("sas.risk.slimtable.SlimTable",{
	metadata:{
		properties:{
			source:{type:"any"},
			clickScope:{type:"sas.risk.slimtable.ClickScope",defaultValue:sas.risk.slimtable.ClickScope.Row},
         sortScope:{type:"sas.risk.slimtable.SortScope",defaultValue:sas.risk.slimtable.SortScope.SingleSort},
			rowHeight:{type:"int",defaultValue:31}
		},
		events:{
			rowChange : {
				parameters : {
					rowIdx: {type: "int"},
					colIdx: {type: "int"},
					colMeta: {type: "any"},
					value: {type:"any"}
				}
			},
			rowClick : {
				parameters : {
					rowIdx: {type : "int"}
				}
			},
			cellClick: {
				parameters : {
					rowIdx: {type : "int"},
					colIdx: {type : "int"}
				}
			},
			rowSelect: {
				parameters : {
					rowIdx: {type : "int"}
				}
			},
			sort:{
				parameters : {
					colIdx: {type : "int"},
					direction:{type : "string"}
				}
			},
			renderComplete : {},
			rerender:{},
		}
	},
	constructor:function(){
		sap.ui.core.Control.apply(this,arguments);
		this.built=false;
	},
	renderer:function(rm, control){
		if(!control.getVisible()){return;}
		control.main=null;
		rm.write("<div tabindex='0' ");
		rm.addClass("SlimTable");
		rm.addClass("SlimTableAuto");
		if(sas.risk.slimtable.util.SlimTableUtil.isIE11()){
			rm.addClass("SlimTableIE11");
		}
		rm.writeClasses();
		rm.writeControlData(control);
		rm.write(">");
		rm.write("<div class='SlimTableSizer' style='width:100%'></div>");
		var calcHeight=control.getSource().getMaxVisibleRowCount()*control.getRowHeight()+(36+1)/*header*/;
		rm.write("<div class='SlimTableLoading' style='height:"+calcHeight+"px'></div>");
		rm.write("</div>");
		control.setBusy(true);
	}
});
sas.risk.slimtable.SlimTable.prototype.onBeforeRendering = function() {
	var source=this.getSource();
	if(source){
		source.init();
	}
};
sas.risk.slimtable.SlimTable.prototype.onAfterRendering = function() {
	var source=this.getSource();
	if(source&&this.getDomRef()){
		var maxVisRowCount=source.getMaxVisibleRowCount();
		if(maxVisRowCount===-1||this.autoRows){
			this.$().addClass("SlimTableAuto");
			var availHeight=Math.max(0,this.$().height()-36/*column header*/-2/*border between header and body*/);
			this.$().removeClass("SlimTableAuto");
			var autoRowCount=Math.floor(availHeight/(this.getRowHeight()*1.0));
			autoRowCount=Math.max(autoRowCount,source.getMinVisibleRowCount());
			if(maxVisRowCount!==autoRowCount){
				source.setMaxVisibleRowCount(autoRowCount);
				this.autoRows=true;
				this.fireRerender({prevRowCount:maxVisRowCount,rowCount:autoRowCount});
				return;				
			}
		}		
	}
	if(!source||!source.data||!this.getDomRef()){return;}
	this.state=new sas.risk.slimtable.SlimTableState(this);
	sas.risk.slimtable.SlimTable.createStructure(this);
	var that=this;
	this.prevWindowWidth=$(window).width();
	this.prevWindowHeight=$(window).height();
	var resizeTimer;
	$(window).resize(function(){
		clearTimeout(resizeTimer);
		resizeTimer=window.setTimeout($.proxy(that.onWindowResize,that),250);
	});
	this.built=true;
	this.setBusy(false);
	this.fireRenderComplete();
};
sas.risk.slimtable.SlimTable.prototype.destroy = function() {
	this.main=null;
};
sas.risk.slimtable.SlimTable.prototype.onWindowResize=function(){
	if(this.prevWindowWidth!==$(window).width()){
		this.prevWindowWidth=$(window).width();
		this.invalidate();
	}else	if(this.autoRows && this.prevWindowHeight!==$(window).height()){
		this.prevWindowHeight=$(window).height();
		this.invalidate();
	}
};
//S1266590: In DBCS languages like Japanese 1st enter key can be used for other selections instead of submission. Fix is to disable the enter key for submission.
// we may have to remove this if, this causes issues as documented for the issue above
sas.risk.slimtable.SlimTable.prototype.onkeyup=function(e){
	switch(e.keyCode){
		case 13:/*enter key*/ {
			this.onclick(e);
		}
	}
};
sas.risk.slimtable.SlimTable.prototype.onclick=function(e){
	if(sas.risk.slimtable.util.SlimTableUtil.getSelection()){return;}//if anything is selected on the page then we ignore the event
	var target=e.target;
	if(target.className==="SlimTableActionIcon"||target.className==="SlimTableExtraColumnCell"){return;}
	var clickScope=this.getClickScope();
	var i,rowIdx,colIdx,selected;
	for(i=0;i<10;i++){
		if(target.nodeName==="TD" && target.classList.contains("SlimTableSelectCell")){
			if(target.parentElement.classList.contains("SlimTableDisabledRow")){return;}
			rowIdx=parseInt(target.parentElement.dataset.row,10)+this.state.getFirstRow();
			if(rowIdx>=this.state.getRowCount()){return;}
			selected=this.getSource().rowClicked(rowIdx);
			sas.risk.slimtable.SlimTable.updateSelection(this);
			this.fireRowSelect({rowIdx:rowIdx,selected:selected});
			break;
		}
		if(target.nodeName==="TD" && clickScope===sas.risk.slimtable.ClickScope.Cell){
			if(target.parentElement.classList.contains("SlimTableDisabledRow")){return;}
			rowIdx=parseInt(target.parentElement.dataset.row,10)+this.state.getFirstRow();
			colIdx=parseInt(target.dataset.col,10)+this.state.getFirstCol();
			if(rowIdx>=this.state.getRowCount() || colIdx>=this.state.getColCount()){return;}
			this.fireCellClick({rowIdx:rowIdx,colIdx:colIdx,cell:target});
			break;
		}
		if(target.nodeName==="TH" && target.classList.contains("SlimTableSelectAll")){
			this.selectAll($(target.parentElement).hasClass("SlimTableSelectedRow"));
			break;
		}
		if(target.nodeName==="TR" && target.parentElement.nodeName==="TBODY"){
			if(target.classList.contains("SlimTableDisabledRow")){return;}
			rowIdx=parseInt(target.dataset.row,10)+this.state.getFirstRow();
			if(rowIdx>=this.state.getRowCount()){return;}
			sas.risk.slimtable.SlimTable.updateSelection(this);
			this.fireRowClick({rowIdx:rowIdx});
			break;
		}
		target=target.parentElement;
	}
};
sas.risk.slimtable.SlimTable.prototype.selectAll=function(isClear){
	var source=this.getSource();
	var j,isSelected,rowCount=source.getRowCount();
	var disableds=source.getDisableds();
	var selections=source.getSelections();
	for(j=0;j<rowCount;j++){
		if(disableds.indexOf(j)!==-1){continue;}//skip disabled rows
		isSelected=selections.indexOf(j)!==-1;
		if(isSelected && !isClear || !isSelected && isClear){continue;}//skip rows in correct selection state
		selected=this.getSource().rowClicked(j);
		this.fireRowSelect({rowIdx:j,selected:selected});
	}
	sas.risk.slimtable.SlimTable.updateSelection(this);
};
sas.risk.slimtable.SlimTable.prototype.rerender=function(){
	var ref=this.getDomRef();
	if(!ref){return;}
	if(ref.childNodes[1]){
//		var height=ref.offsetHeight;
//		ref.style.height=height+1/*dunno*/+"px";
		ref.removeChild(ref.childNodes[1]);
	}
	if(!this.getVisible()){return;}
	this.onAfterRendering();
};
sas.risk.slimtable.SlimTable.prototype.invalidate=function(){
	if(this.main){
		if(this.getDomRef()){
			this.getDomRef().removeChild(this.main);
		}
		this.main=null;
	}
	sap.ui.core.Control.prototype.invalidate.apply(this,arguments);
};
sas.risk.slimtable.SlimTable.prototype.isScrollingVertically=function(){
	return !!this.main.body.vsb;
};
sas.risk.slimtable.SlimTable.prototype.scrollToBottom=function(){
	if(this.isScrollingVertically()){
		this.setVSbPositionAndTrigger(100000);
	}
};
sas.risk.slimtable.SlimTable.prototype.isBuilt=function(){
	return this.built;
};
sas.risk.slimtable.SlimTable.prototype.setSource=function(source){
	this.setProperty("source",source);
	source.setTable(this);
};
sas.risk.slimtable.SlimTable.prototype.setVSbPositionAndTrigger=function(position){
	this.ignoreVSbEvent=false;
	var vsb=this.main.body.vsb;
	vsb.scroller.scrollTop=position;
};
sas.risk.slimtable.SlimTable.prototype.setVSbPosition=function(position){
	if(this.ignoreVSbEvent===null){this.ignoreVSbEvent=true;}//dont override if a trigger call is in queue
	var vsb=this.main.body.vsb;
	vsb.scroller.scrollTop=position;
};
//ACTIONS
sas.risk.slimtable.SlimTable.prototype.onColumnScroll=function(e){
	if(this.ignoreHSbEvent){this.ignoreHSbEvent=false;return;}
	var max=e.target.scrollWidth-e.target.offsetWidth;//IE doesnt calculate clientWidth
	var val=e.target.scrollLeft;
	var state=this.state;
	
//	var calcFirstCol=state.getFirstColMax()*val/max;
//	var delta=calcFirstCol-Math.round(calcFirstCol);
//	calcFirstCol=delta>0?Math.ceil(calcFirstCol):Math.floor(calcFirstCol);
//	this.ignoreHSbEvent=true;
//	e.target.scrollLeft=Math.round(calcFirstCol*max/state.getFirstColMax());
	
	var delta=e.target.scrollLeft-(this.oldScrollLeft||0);
	var minDelta=Math.ceil(max/state.getFirstColMax());
	var difference=minDelta-Math.abs(delta);	
	if(delta && difference>0){//scroll delta too small, must increase
		if(delta>0){//scroll left
			e.target.scrollLeft+=difference;
		}else{//scroll right
			e.target.scrollLeft-=difference;
		}
	}
	this.oldScrollLeft=e.target.scrollLeft;
	calcFirstCol=Math.floor(state.getFirstColMax()*val/max);
	calcFirstColAbs=Math.abs(calcFirstCol);
	state.setFirstCol(calcFirstColAbs);
	sas.risk.slimtable.SlimTable.updateHeader(this);
	sas.risk.slimtable.SlimTable.updateSortIcon(this);
	sas.risk.slimtable.SlimTable.updateBody(this);
};
sas.risk.slimtable.SlimTable.prototype.onRowScroll=function(e){
	if(this.ignoreVSbEvent){this.ignoreVSbEvent=null;return;}
	var state=this.state;
	var max=e.target.scrollHeight-(e.target.offsetHeight);//IE doesnt calculate clientHeight
	var delta=e.target.scrollTop-(this.oldScrollTop||0);
	var minDelta=Math.ceil(max/state.getFirstRowMax());
	var difference=minDelta-Math.abs(delta);
	if(delta && difference>0){//scroll delta too small, must increase
		if(delta>0){//scroll up
			e.target.scrollTop+=difference;
		}else{//scroll down
			e.target.scrollTop-=difference;
		}
	}
	var val=e.target.scrollTop;
	this.oldScrollTop=e.target.scrollTop;
	state.setFirstRow(Math.floor(state.getFirstRowMax()*val/max));
	sas.risk.slimtable.SlimTable.updateBody(this);
	sas.risk.slimtable.SlimTable.updateSelection(this);
};
sas.risk.slimtable.SlimTable.prototype.onWheelRowScroll=function(e){
	if(this.main.body.vsb){
		this.main.body.vsb.scroller.scrollTop+=e.deltaY;
	}
};
//STATICS
sas.risk.slimtable.SlimTable.createStructure=function(control){
	var rb=sas.risk.slimtable.util.SlimTableUtil.getBundle();
	var source=control.getSource();
	var state=control.state;
	var root=control.getDomRef();
	$(root).find(".SlimTableLoading").remove();
	var st=sas.risk.slimtable.SlimTable;
	var main=st.createElement("DIV",root,"main","SlimTableMain",control);
	var body=st.createElement("DIV",main,"body","SlimTableBody");
	var wrapper=st.createElement("DIV",body,"wrapper","SlimTableWrapper");
	var table=st.createElement("TABLE",wrapper,"table",source.isMultiSelect()?null:"SlimTableSingleSelect");
	table.onwheel=$.proxy(control.onWheelRowScroll,control);
	var thead=st.createElement("THEAD",table,"thead");
	var trHead=st.createElement("TR",thead,"tr");
	var ths=trHead.ths=[];
	var showSelectionControls=source.getShowSelectionControls();
	if(showSelectionControls){//add selection control column
		var multiSelect=source.isMultiSelect();
		var th0=st.createElement("TH",trHead,null,multiSelect?"SlimTableSelectCell SlimTableSelectAll":"SlimTableSelectCell");
		var div0=st.createElement("DIV",th0,null,multiSelect?"SlimTableSelectBox":"");
		if(multiSelect){
			div0.setAttribute("aria-checked",false);
			div0.setAttribute("role","checkbox");
			div0.setAttribute("aria-label",rb.getText("slimtable.selectall.select"));
		}
		var content0=multiSelect?st.createElement("SPAN",th0,null,"SlimTableSelectIcon"):document.createTextNode("");
		if(multiSelect){content0.dataset.sapUiIconContent="";}
		div0.appendChild(content0);
		ths.push({th:th0,div:div0,content:content0});
	}
	state.eachColumn(function(colIdx,cIdx,visible){
		var thClass=visible?(source.isSortable(colIdx)?"":"SlimTableNotSortable"):"SlimTableExtraColumn";
		var th=st.createElement("TH",trHead,null,thClass);
		if(!visible){th.setAttribute("aria-hidden","true");}
		th.tabIndex=(colIdx===0)?0:-1;
		th.onfocus=$.proxy(sas.risk.slimtable.SlimTableKeyboard.init,this,control);
		var prefWidth=source.getColPreferredWidth(colIdx);
		if(prefWidth){th.style.width=prefWidth;}
		var div=st.createElement("DIV",th,null,"SlimTableColLabel");
		div.dataset.col=colIdx;
		div.onmousedown=$.proxy(sas.risk.slimtable.SlimTable.setupColumnMenu,div,control);
		var txt=document.createTextNode(source.getColLabel(colIdx));
		div.appendChild(txt);
		var span=st.createElement("SPAN",th,null,"SlimTableColResizer");
		span.onmouseover=$.proxy(sas.risk.slimtable.SlimTable.setupColumnDragging,span,control);
		span.dataset.col=colIdx;
		var className="SlimTableSortIcon";
		if(visible && source.isSortable(colIdx)){
			className+=" SlimTable-"+(visible?source.getSortOrder(colIdx):"")+"-"+(visible?source.getSortDirection(colIdx):"None");
		} else {
			className+=" SlimTable-"+source.getSortDirection(colIdx);
		}
		var icon=st.createElement("SPAN",th,null,className);

		icon.dataset.sapUiIconContent="";
		ths.push({th:th,div:div,content:txt,icon:icon});
	});
	var tbody=st.createElement("TBODY",table,"tbody");
	var trs=tbody.trs=[];
	state.eachVisibleRow(function(rowIdx){
		var tr=st.createElement("TR",tbody,trs);
		tr.dataset.row=rowIdx;
		var tds=tr.tds=[];
		if(showSelectionControls){//add selection control column
			var td1=st.createElement("TD",tr,null,"SlimTableSelectCell");
			var div1=st.createElement("DIV",td1,null,"SlimTableSelectBox");
			div1.dataset.col=rowIdx;
			var content1=st.createElement("SPAN",td1,null,"SlimTableSelectIcon");
			content1.dataset.sapUiIconContent="";
			div1.appendChild(content1);
			tds.push({td:td1,div:div1});
		}
		state.eachColumn(function(colIdx,cIdx,visible){
			var td=st.createElement("TD",tr);
			td.tabIndex=(rowIdx===0 && colIdx===0)?0:-1;
			td.dataset.col=colIdx;
			var div=st.createElement("DIV",td);
			source.formatCell(rowIdx,colIdx,div);
			if(!visible){
				td.className="SlimTableExtraColumnCell";
				td.setAttribute("aria-hidden","true");
			}
			tds.push({td:td,div:div});
		});
	});
	if(state.showVSb()){
		st.createVSb(control);
	}
	state.calcAutoWidths();
	if(state.showHSb()){
		st.createHSb(control);
	}
	sas.risk.slimtable.SlimTable.updateSelection(control);
};
sas.risk.slimtable.SlimTable.createVSb=function(control){
	var state=control.state;
	var st=sas.risk.slimtable.SlimTable;
	var border=1/*right border of rightmost column*/;
	var vsbWidth=sas.risk.slimtable.SlimTable.meta.VSB_WIDTH;
	control.main.body.wrapper.style.width=(control.state.getMaxWidth()-vsbWidth-border)+"px";
	control.state.colWidths[control.state.colWidths.length-1]-=vsbWidth;/*remove extra width to accomodate the vsb*/
	var vsb=st.createElement("DIV",control.main.body,"vsb","SlimTableVScrollbar");
	var rowHeight=control.getRowHeight();
	vsb.style.height=(36/*header height*/+1/*border*/+state.getMaxVisibleRowCount()*(rowHeight+1/*collapse top/bottom border*/))+"px";
	var vscroller=st.createElement("DIV",vsb,"scroller");
	var vinner=st.createElement("DIV",vscroller,"inner");
	vinner.style.height=(36/*header height*/+1/*border*/+state.getRowCount()*(rowHeight+1/*collapse top/bottom border*/))+"px";
	vscroller.onscroll=_.throttle($.proxy(control.onRowScroll,control),150,{leading:false});
};
sas.risk.slimtable.SlimTable.createHSb=function(control){
	var state=control.state;
	var st=sas.risk.slimtable.SlimTable;
	var hasVSb=control.main.body.vsb;
	var hsb=st.createElement("DIV",control.main,"hsb",(hasVSb?"SlimTableHScrollbar":"SlimTableHScrollbar SlimTableNoVSb"));
	hsb.style.width=state.getMaxWidth()+"px";
	var hscroller=st.createElement("DIV",hsb,"scroller");
	var hinner=st.createElement("DIV",hscroller,"inner");
	hinner.style.width=state.getTotalColWidth()+"px";
	hscroller.onscroll=$.proxy(control.onColumnScroll,control);
	
};
sas.risk.slimtable.SlimTable.createElement=function(tagName,parentNode,id,className,container){
	var element=document.createElement(tagName);
	container=container||parentNode;
	if($.isArray(id)){
		id.push(element);
	}else if(id){
		container[id]=element;
	}
	if(className){
		element.className=className;
	}
	parentNode.appendChild(element);
	return element;
};
sas.risk.slimtable.SlimTable.setupColumnMenu=function(control,e){
	if(e.which===3/*right mouse click*/){
		e.stopPropagation();
		e.preventDefault();
		var target=e.target;
		var colIdx=parseInt(target.dataset.col,10)+control.state.getFirstCol();
		var source=control.getSource();
		if(!source.isSortable(colIdx)){return;}
		var direction=source.getSortDirection(colIdx);
		var sortInfo=source.getSortInfo();
		//var sortOrder=source.getSortOrder(colIdx);
		var totalSortedColumns = _.filter(sortInfo, function (column){return column.dir !== "None";}).length;
		jQuery.sap.require("sas.hc.ui.unified.Menu");
		jQuery.sap.require("sas.risk.common.widgets.MenuItem");
		var rb=sas.risk.slimtable.util.SlimTableUtil.getBundle();
		var menu=new sas.hc.ui.unified.Menu({
			itemSelect:function(e){
				var payload=e.getParameter("item").getPayload();
				source.sort(payload.colIdx,payload.action,payload.additionalSort);
			}
		});
		
		
		var sortScope=control.getSortScope();
		
        if (sortScope===sas.risk.slimtable.SortScope.MultiSort) {
   		    var item1 = new sas.risk.common.widgets.MenuItem({
   			    text:rb.getText('slimtable.sort.addAscending.txt'),
   			    payload:{colIdx:colIdx,control:control,action:"ascending", additionalSort:"add"},
				   enabled:(direction!=="ascending" && 
						   ((sortInfo[colIdx] !== undefined)?(sortInfo[colIdx].sortOrder === -1):true ) && 
						   (totalSortedColumns < 3))
   			});
   			menu.addItem(item1);
   			var item2 = new sas.risk.common.widgets.MenuItem({
   				text:rb.getText('slimtable.sort.addDescending.txt'),
   				payload:{colIdx:colIdx,control:control,action:"descending", additionalSort:"add"},
				   enabled:(direction!=="descending" && 
						   ((sortInfo[colIdx] !== undefined)?(sortInfo[colIdx].sortOrder === -1):true ) && 
						   (totalSortedColumns < 3))
   			});
   			menu.addItem(item2);
      	}
		
    	var item3 = new sas.risk.common.widgets.MenuItem({
    			text:rb.getText('slimtable.sort.ascending.txt'),
     			payload:{colIdx:colIdx,control:control,action:"ascending", additionalSort:"none"},
     			enabled:(direction!=="ascending")
  		});
    	menu.addItem(item3);
    	var item4 = new sas.risk.common.widgets.MenuItem({
	    	text:rb.getText('slimtable.sort.descending.txt'),
        	payload:{colIdx:colIdx,control:control,action:"descending", additionalSort:"none"},
        	enabled:(direction!=="descending")
    	});
    	menu.addItem(item4);
		var item5 = new sas.risk.common.widgets.MenuItem({
			text:rb.getText('slimtable.sort.remove.txt'),
			payload:{colIdx:colIdx,control:control,action:"None", additionalSort:"remove"},
			enabled:(direction!=="None")
		});
		menu.addItem(item5);		
		var eDock = sap.ui.core.Popup.Dock;
		
		var offset = "none none";
    	var pos = $(target).offset();
    	if (e.type === 'keydown') {
        	var $target = $(target),
            	iY = $target ? $target.outerHeight() : 0;
         	offset = "0 " + iY;
    	} else {
        	offset = (e.pageX - pos.left) + " " + (e.pageY - pos.top);
    	}
		
		menu.open(
				false /*First item already highlighted*/,
				target /*Dom reference which gets the focus back when the menu is closed*/,
				eDock.BeginTop /*"Edge" of the menu (see sap.ui.core.Popup)*/,
				eDock.BeginTop /*"Edge" of the related opener position (see sap.ui.core.Popup)*/,
				target /*Related opener position (see sap.ui.core.Popup)*/,
				offset/*stick it under the mouse cursor*/,
				"fit"/*Collision strategy*/
		);
}
};
sas.risk.slimtable.SlimTable.setupColumnDragging=function(control){
	var resizer=this;
	if(!resizer.anchor){
		var st=sas.risk.slimtable.SlimTable;
		st.createElement("DIV",resizer,"anchor","SlimTableColResizerAnchor");
		var bar=st.createElement("DIV",resizer,"bar","SlimTableColResizerBar");
		var $contentsTable=$(control.main.body.wrapper.table.tbody);
		bar.style.height=$contentsTable.height()+"px";
	}
	resizer.onmousedown=function(){
		$(document.activeElement).blur();//take away focus,good for in-place edit
		resizer.bar.style.visibility="visible";
	};
	resizer.onmouseup=function(){resizer.bar.style.visibility="hidden";};
	var $control=$(control.main);
	$(resizer).draggable({
		axis:"x",
		containment:$control,
		start:function(event){
			$(event.target).addClass("Dragging");
			$(event.target.children[0]).addClass("SlimTableColResizerAnchorHover");
		},
		stop:function(event,ui){
			var colIdx=parseInt(ui.helper.context.dataset.col,10);
			var state=control.state;
			//RGFRMWRK-831 Handle dragging column issue in RTL
			var dir = event.currentTarget.dir;
			var newColWidth = 0;
			if (dir === 'ltr')
				newColWidth = ui.position.left-ui.originalPosition.left;
			else{
				newColWidth = -(ui.position.left-ui.originalPosition.left);
				colIdx = colIdx-1;
			}				
			state.updateColWidth(colIdx,newColWidth);
			resizer.style.left="auto";//initial does not work in IE
			sas.risk.slimtable.SlimTable.updateHeader(control);
			sas.risk.slimtable.SlimTable.updateBody(control);
			sas.risk.slimtable.SlimTable.updateHSb(control);
			$(event.target).removeClass("Dragging");
			$(event.target.children[0]).removeClass("SlimTableColResizerAnchorHover");
			resizer.bar.style.visibility="hidden";
		}
	});
};
sas.risk.slimtable.SlimTable.updateBody=function(control){
	var trs=control.main.body.wrapper.table.tbody.trs;
	var state=control.state;
	var source=control.getSource();
	var selectionControlDelta=state.getSelectionControlDelta();
	state.eachVisibleRow(function(rowIdx,rowLoopIdx){
		var tds=trs[rowLoopIdx].tds;
		state.eachColumn(function(colIdx,colLoopIdx,isVisible){
			var td=tds[colLoopIdx+selectionControlDelta];
			if(isVisible){
				source.formatCell(rowIdx,colIdx,td.div);
				$(td.td).removeClass("SlimTableExtraColumnCell");
			}else{
				var empty=document.createTextNode("");
				sas.risk.slimtable.util.SlimTableUtil.empty(td.div);
				td.div.appendChild(empty);
				$(td.td).addClass("SlimTableExtraColumnCell");
			}
			state.setCellWidth(td,colIdx,isVisible);
		});
	});
};
sas.risk.slimtable.SlimTable.updateHeader=function(control){
	var ths=control.main.body.wrapper.table.thead.tr.ths;
	var state=control.state;
	var source=control.getSource();
	var selectionControlDelta=state.getSelectionControlDelta();
	state.eachColumn(function(colIdx,colLoopIdx,isVisible){
		var th=ths[colLoopIdx+selectionControlDelta];
		if(isVisible){
			th.content.nodeValue=source.getColLabel(colIdx);
			$(th.th).removeClass("SlimTableExtraColumn");
		}else{
			th.content.nodeValue="";
			$(th.th).addClass("SlimTableExtraColumn");
		}
		state.setCellWidth(th,colIdx,isVisible);
	});
};
sas.risk.slimtable.SlimTable.updateSortIcon=function(control){
	var ths=control.main.body.wrapper.table.thead.tr.ths;
	var state=control.state;
	var source=control.getSource();
	var selectionControlDelta=source.getShowSelectionControls()?1:0;
	state.eachColumn(function(colIdx,colLoopIdx,isVisible){
		var th=ths[colLoopIdx+selectionControlDelta];
		var $th=$(th);
		var className="SlimTableSortIcon";
		if(isVisible && source.isSortable(colIdx)){
			className+=" SlimTable-"+(isVisible?source.getSortOrder(colIdx):"")+"-"+(isVisible?source.getSortDirection(colIdx):"None");
			$th.removeClass("SlimTableNotSortable");
		}else{
			className+=" SlimTableNone";
			$th.addClass("SlimTableNotSortable");
		}
		th.icon.className=className;
	});	
};
sas.risk.slimtable.SlimTable.updateVSb=function(control){
	var state=control.state;
	var showVSb=state.showVSb();
	var body=control.main.body;
	if(showVSb && !body.vsb){
		sas.risk.slimtable.SlimTable.createVSb(control);
	}else if(!showVSb){
		if(body.vsb){
			body.removeChild(body.vsb);
			body.vsb=null;
			var vsbWidth=sas.risk.slimtable.SlimTable.meta.VSB_WIDTH;
			control.state.colWidths[control.state.colWidths.length-1]+=vsbWidth;/*add extra width that once accomodated the vsb*/
		}
		return;
	}
	var vsb=body.vsb;
	var rowHeight=control.getRowHeight();
	var max=36/*header height*/+1/*top border*/+state.getMaxVisibleRowCount()*(rowHeight+1);
	var maxPx=max+"px";
	if(vsb.style.height!==maxPx){
		vsb.style.height=maxPx;
	}
	var inner=vsb.scroller.inner;
	var total=36/*header height*/+1/*border*/+state.getRowCount()*(rowHeight+1);
	var totalPx=total+"px";
	if(inner.style.height!==totalPx){
		inner.style.height=totalPx;
	}
	var firstRowMax=state.getFirstRowMax();
	var val=firstRowMax===0?0:state.getFirstRow()*(total-max)/firstRowMax;
	if(vsb.scroller.scrollTop!==val){
		control.setVSbPosition(val);
	}
};
sas.risk.slimtable.SlimTable.updateHSb=function(control){
	var state=control.state;
	var showHSb=state.showHSb();
	var main=control.main;
	if(showHSb && !main.hsb){
		sas.risk.slimtable.SlimTable.createHSb(control);
	}else if(!showHSb){
		if(main.hsb){
			main.removeChild(main.hsb);
			main.hsb=null;
		}
		return;
	}
	var hsb=main.hsb;
	var max=state.getMaxWidth();
	var maxPx=max+"px";
	if(hsb.style.width!==maxPx){
		hsb.style.width=maxPx;
	}
	var inner=hsb.scroller.inner;
	var total=state.getTotalColWidth();
	var totalPx=total+"px";
	if(inner.style.width!==totalPx){
		inner.style.width=totalPx;
	}
	var firstColMax=state.getFirstColMax();
	var val=firstColMax===0?0:state.getFirstCol()*(total-max)/firstColMax;
	if(hsb.scroller.scrollLeft!==val){
		control.ignoreHSbEvent=true;
		hsb.scroller.scrollLeft=val;		
	}
};
sas.risk.slimtable.SlimTable.updateSelection=function(control){
	var source=control.getSource();
	var count=control.state.getRowCount();
	var selections=source.getSelections();
	var disableds=source.getDisableds();
	var state=control.state;
	var trs=control.main.body.wrapper.table.tbody.trs;
	var disabledUnselectedCount=0;
	state.eachVisibleRow(function(rowIdx,loopIdx){
		var tr=trs[loopIdx];
		var isSelected=selections.indexOf(rowIdx)!==-1;
		var isDisabled=disableds.indexOf(rowIdx)!==-1;
		if(!isSelected && isDisabled){disabledUnselectedCount++;}
		var isEmpty=rowIdx>=count;
		$(tr).toggleClass("SlimTableSelectedRow",isSelected)
			  .toggleClass("SlimTableEmptyRow",isEmpty)
			  .toggleClass("SlimTableDisabledRow",isDisabled);
	});
	var isChecked=(count>0 && count===(selections.length+disabledUnselectedCount));
	$(control.main.body.wrapper.table.thead.tr).toggleClass("SlimTableSelectedRow",isChecked);//select all
	$(control.main.body.wrapper.table.thead.tr.ths[0].div).attr("aria-checked",isChecked);
};
sas.risk.slimtable.SlimTable.meta={
	VSB_WIDTH:13
};
if(sas.risk.slimtable.util.SlimTableUtil.isIE11()){//IE11 overrides
	sas.risk.slimtable.SlimTable.meta.VSB_WIDTH=17;
}
sas.risk.slimtable.SlimTable.__skip__=["createElement"];
jQuery.sap.require("sap.ui.core.EnabledPropagator");
sap.ui.core.EnabledPropagator.call(sas.risk.slimtable.SlimTable.prototype);