jQuery.sap.declare("sas.risk.slimtable.SlimTableSource");
jQuery.sap.require("sas.risk.slimtable.util.SlimTableUtil");
jQuery.sap.require("sas.risk.slimtable.util.TableUtil");

sas.risk.slimtable.SlimTableSource=function(multiSelect,showSelectionControls,tableMeta,data){
	this.__LAST_UPDATE__=0;
	this.table=null;
	this.data=sas.risk.slimtable.util.TableUtil.timestampArray(data);/*optional*/
	this.originalData=data;/*optional*/
	this.tableMeta=tableMeta;/*optional*/
	this.query=null;
	this.selections=[];
	this.disableds=[];
	this.multiSelect=multiSelect;
	this.showSelectionControls=showSelectionControls;
	this.minVisibleRowCount=2;
	this.maxVisibleRowCount=5;
	this.sortInfo=null;
	this.preselection=null;
	this.disablement=null;
	this.ID=window.performance.now();
};
sas.risk.slimtable.SlimTableSource.prototype={
	init:function(){
		
	},
	setDisablement:function(disablement){
		this.disablement=disablement;
	},
	setPreselection:function(preselection){
		this.preselection=preselection;
	},
	setTable:function(table){
		this.table=table;
	},
	setTableMeta:function(tableMeta){
		this.tableMeta=tableMeta;
	},
	getShowSelectionControls:function(){
		return this.showSelectionControls;
	},
	setShowSelectionControls:function(value){
		this.showSelectionControls=value;
	},
	getTableMeta:function(){
		return this.tableMeta;
	},
	getColumnMeta:function(colIndex){
		return this.tableMeta.getVisibleAndFixedColumns()[colIndex];
	},
	getColumnFormatter:function(columnMeta){
		if(columnMeta && columnMeta.getType()){
			return columnMeta.getFormatter();
		}
		return null;
	},
	formatCell:function(rowIndex,colIndex,parent){
		parent.parentNode.className="SlimTableDataCell";
		var rowData=this.data[rowIndex];
		var columnMeta=this.getColumnMeta(colIndex);
		var formatter=this.getColumnFormatter(columnMeta);
		if(formatter && rowData){
			formatter.format(columnMeta,rowData,parent,{rowIndex:rowIndex,colIndex:colIndex,source:this});
			return;
		}
		var content=document.createTextNode("");
		sas.risk.slimtable.util.SlimTableUtil.empty(parent);
		parent.parentNode.className=null;
		parent.appendChild(content);
	},
	getRowCount:function(){
		return this.data.length;
	},
	setMinVisibleRowCount:function(count){
		this.minVisibleRowCount=count;
	},
	getMinVisibleRowCount:function(){
		return this.minVisibleRowCount;
	},
	setMaxVisibleRowCount:function(count){
		this.maxVisibleRowCount=count;
	},
	getMaxVisibleRowCount:function(){
		return this.maxVisibleRowCount;
	},
	getColLabel:function(idx){
		var visibleAndFixedColumns=this.tableMeta.getVisibleAndFixedColumns();
		if(idx<visibleAndFixedColumns.length){
			return this.tableMeta.getVisibleAndFixedColumns()[idx].getColLabel();
		}
		return "";
	},
	getColPreferredWidth:function(idx){
		var visibleAndFixedColumns=this.tableMeta.getVisibleAndFixedColumns();
		if(idx<visibleAndFixedColumns.length){
			return visibleAndFixedColumns[idx].getColPreferredWidth();
		}
		return 0;
	},
	sort:function(colIdx,action, additionalSort){
		var sortInfo=this.getSortInfo();
		var si = _.find(sortInfo, function(info, idx){return idx===colIdx;});
		var mso = 0;
		_.each(sortInfo, function(info){
			mso=(info.sortOrder > mso) ?info.sortOrder:mso;
		});

		if (si !== undefined) {			
			if (additionalSort === "add") {
				si.dir=action;
				si.sortOrder=mso+1;
//				console.log("sort: si.sortOrder:", si.sortOrder);
			} else if (additionalSort === "remove") {
				si.dir=action;
				si.sortOrder=-1;
				//Update the remainder columns to new order
				_.each(sortInfo, function(col) {
					if (col.colIdx > colIdx) {
						 col.sortOrder = col.sortOrder - 1;
					}
				});
			} 
			else if (additionalSort === "none") {
				si.dir=action;
				si.sortOrder = 0;
				_.each(sortInfo, function(col) {
					if (col.colIdx !== colIdx) {
						 col.sortOrder = -1;
						 col.dir = "None";
					}
				});
			}
		}

		//var that = this;
		var col = this.tableMeta.getVisibleAndFixedColumns()[colIdx];
		//this.tableMeta.setSortedColumn(that.tableMeta.getVisibleAndFixedColumns()[colIdx],action,additionalSort);	
		this.tableMeta.setSortedColumn(col,action,additionalSort);	
		this.table.fireSort({colIdx:colIdx,direction:action});
	},
	isSortable:function(colIdx){
		var visibleAndFixedColumns=this.tableMeta.getVisibleAndFixedColumns();
		return colIdx>=0&&colIdx<visibleAndFixedColumns.length&&visibleAndFixedColumns[colIdx].isColSortable();
	},
	getSortInfo:function(){
		var sortedColumns=this.tableMeta.getSortedColumns();
		if(sortedColumns && sortedColumns.length){
			var visibleColumns=this.tableMeta.getVisibleAndFixedColumns();
			this.sortInfo=[];
			var that = this;
		
			_.each(visibleColumns, function(sortedColumn) {
				that.sortInfo.push({
					colIdx:visibleColumns.indexOf(sortedColumn), 
					dir: sortedColumn.getSortDirection(),
					sortOrder:sortedColumn.getSortDirection() === "None"? -1: sortedColumn.sortOrder
				});
//				console.log("getSortInfo: colIdx:", that.sortInfo[that.sortInfo.length-1].colIdx,
//								" dir:", that.sortInfo[that.sortInfo.length-1].dir,
//								"SortOrder:", that.sortInfo[that.sortInfo.length-1].sortOrder);
			});
		}else{
			this.sortInfo={colIdx:0,dir:"None", sortOrder:-1};
			//console.log("getSortInfo: No sortedColumn");
		}
		return this.sortInfo;
	},
	retrieveSortInfo:function(){
		var sortedColumns=this.tableMeta.getSortedColumns();
		if(sortedColumns && sortedColumns.length){
			var visibleColumns=this.tableMeta.getVisibleAndFixedColumns();
			this.sortInfo=[];
			var that = this;
			
			_.each(visibleColumns, function(sortedColumn) {
				that.sortInfo.push({
					colIdx:visibleColumns.indexOf(sortedColumn), 
					dir: sortedColumn.getSortDirection(),
					sortOrder:sortedColumn.getSortDirection() === "None"? -1: sortedColumn.sortOrder
				});
//				console.log("retrieveSortInfo: colIdx:", that.sortInfo[that.sortInfo.length-1].colIdx,
//								" dir:", that.sortInfo[that.sortInfo.length-1].dir,
//								"SortOrder:", that.sortInfo[that.sortInfo.length-1].sortOrder);
			});
		}else{
			this.sortInfo={colIdx:0,dir:"None", sortOrder:-1};
//			console.log("retrieveSortInfo: No sortedColumn");
		}
		return this.sortInfo;
	},
	getSortDirection:function(colIdx){
		var sortInfo=this.getSortInfo();
		return sortInfo[colIdx]?sortInfo[colIdx].dir:"None";
	},
	getSortOrder:function(colIdx){
		var sortInfo=this.retrieveSortInfo();
		return sortInfo[colIdx]?sortInfo[colIdx].sortOrder:"-1";
	},
	setDisableds:function(disableds){
		this.disableds=disableds;
	},
	getDisableds:function(){
		return this.disableds;
	},
	setSelections:function(selections){
		this.selections=selections;
	},
	getSelections:function(){
		return this.selections;
	},
	isIndexSelected:function(index){
		return this.selections.indexOf(index)!==-1;
	},
	rowClicked:function(rowIdx){
		if(this.multiSelect){
			var index=this.selections.indexOf(rowIdx);
			if(index!==-1){
				this.selections.splice(index,1);
				return false;
			}else{
				this.selections.push(rowIdx);
			}
		}else{
			this.selections.length=0;
			this.selections.push(rowIdx);
		}
		return true;
	},
	isMultiSelect:function(){
		return this.multiSelect;
	},
	setMultiSelect:function(value){
		this.multiSelect=value;
	},
	getData:function(){
		return this.data;
	},
	getTable:function(){
		return this.table;
	},
	getOriginalData:function(){
		return this.originalData;
	},
	update:function(query,data,scrollToBottom,clearSelection){
		var isDifferent=!!(this.data && data && (data.length!==this.data.length || (this.__LAST_UPDATE__ < this.data.__TIMESTAMP__)));
		this.__LAST_UPDATE__=_.now();
		this.data=sas.risk.slimtable.util.TableUtil.timestampArray(data);
		var cols=query.getOrderedColumns();
		this.tableMeta.setVisibleColumns(cols);
		if(this.table.isBuilt() && this.query && this.table.getDomRef()){
			var delta=this.query.delta(query);
			if(isDifferent && !scrollToBottom){ //table re-render case
				delta.push("DATA_DIFF");
			}
			this.query=query;
			delta=delta.length?delta:["NO_CHANGE"];
			if(clearSelection){this.selections=[];}
			if(this.preselection){this.preselection(this);}
			if(this.disablement){this.disablement(this);}
			var augmented=_.map(delta,function(key){return sas.risk.slimtable.SlimTableSource.queryDeltaMap[key];});
			var maxEntry=_.max(augmented,function(entry){return entry.weight;});
			switch(maxEntry.action){
				case "UPDATE_ROWS":
				case "NO_CHANGE":
				case "UPDATE_TABLE":
					this.table.state.setFirstRow(0);
					sas.risk.slimtable.SlimTable.updateVSb(this.table);
					sas.risk.slimtable.SlimTable.updateHSb(this.table);
					if(scrollToBottom && this.table.isScrollingVertically()){
						this.table.scrollToBottom();
					}else{
						sas.risk.slimtable.SlimTable.updateBody(this.table);
						sas.risk.slimtable.SlimTable.updateSelection(this.table);							
					}
					sas.risk.slimtable.SlimTable.updateHeader(this.table);
					sas.risk.slimtable.SlimTable.updateSortIcon(this.table);
					this.table.fireRenderComplete();
				break;
				case "CREATE_TABLE":
					this.table.rerender();
				break;
			}
		}else{
			this.query=query;
			if(clearSelection){this.selections=[];}
			if(this.preselection){this.preselection(this);}
			if(this.disablement){this.disablement(this);}
			this.table.rerender();
		}
	}
};
sas.risk.slimtable.SlimTableSource.queryDeltaMap={
	"NO_CHANGE":{weight:1,action:"NO_CHANGE"},
	"LIMIT":{weight:1,action:"UPDATE_ROWS"},
	"START":{weight:1,action:"UPDATE_ROWS"},
	"FILTERS":{weight:1,action:"UPDATE_ROWS"},
	"LOCATION":{weight:1,action:"UPDATE_ROWS"},
	"SORT_NAME":{weight:2,action:"UPDATE_TABLE"},
	"SORT_DIRECTION":{weight:2,action:"UPDATE_TABLE"},
	"COLUMNS_ORDER":{weight:2,action:"UPDATE_TABLE"},
	"COLUMNS_VISIBLE":{weight:3,action:"CREATE_TABLE"},
	//"COLUMNS_SORTED":{weight:2,action:"UPDATE_TABLE"},
	"DATA_DIFF":{weight:4,action:"CREATE_TABLE"},
	
	// TODO: if using composite remove these lines
	"SORT_NAME2":{weight:2,action:"UPDATE_TABLE"},
	"SORT_DIRECTION2":{weight:2,action:"UPDATE_TABLE"},
	"SORT_NAME3":{weight:2,action:"UPDATE_TABLE"},
	"SORT_DIRECTION3":{weight:2,action:"UPDATE_TABLE"}
};
sas.risk.slimtable.SlimTableSource.__skip__=["formatCell"];