jQuery.sap.declare("sas.risk.slimtable.SlimTableState");

sas.risk.slimtable.SlimTableState=function(control){
	this.control=control;
	this.source=control.getSource();
	this.firstRow=0;
	this.firstCol=0;
	this.colWidths=[];
	this.totalColWidth=0;
	this.lastCol=null;
	this.firstColMax=null;
};
sas.risk.slimtable.SlimTableState.prototype={
	getFirstRow:function(){
		return this.firstRow;
	},
	setFirstRow:function(firstRow){
		this.firstRow=firstRow;
	},
	getFirstCol:function(){
		return this.firstCol;
	},
	setFirstCol:function(firstCol){
		this.firstCol=firstCol;
		this.calcColRange();
	},
	getLastCol:function(){
		return this.lastCol;
	},
	getFirstRowMax:function(){
		return Math.max(this.getRowCount()-this.getMaxVisibleRowCount(),0);/*largest number "firstRow" can be*/
	},
	getFirstColMax:function(){
		return this.firstColMax;
	},
	getMaxWidth:function(){
		var domRef=this.control.getDomRef();
		var fakeChild=domRef.children[0];
		domRef.children[1].style.width="10px";
		this.maxWidth=fakeChild.offsetWidth;
		domRef.children[1].style.width="";
		return this.maxWidth-2/*border of main, so that it matches client width of main*/;
	},
	getTotalColWidth:function(){
		return this.totalColWidth;
	},
	getRowCount:function(){
		return this.source.getRowCount();
	},
	getMinVisibleRowCount:function(){
		return this.source.getMinVisibleRowCount();
	},
	getMaxVisibleRowCount:function(){
		return this.source.getMaxVisibleRowCount();
	},
	getColCount:function(){
		var colCount=this.source.getTableMeta().getVisibleAndFixedColumns().length;
		if(this.lastCol==null){//first time 
			this.lastCol=Math.max(this.firstCol,colCount-this.getSelectionControlDelta());
		}else if(this.lastCol===0){//showing up as zero from section collapse
         this.lastCol=Math.max(this.firstCol,colCount-this.getSelectionControlDelta());
         if (colCount > 1) {
            this.calcAutoWidths();
         }
		}
		return colCount;
	},
	updateColWidth:function(colIdx,delta){
		var realIdx=colIdx+this.firstCol+this.getSelectionControlDelta();
		this.colWidths[realIdx]=Math.max(this.colWidths[realIdx]+delta,5);/*at least 5 pixels wide minimum*/
		this.calcTotalColWidth();
		this.calcFirstColMax();
		this.calcColRange();
	},
	eachVisibleRow:function(func){
		var i,row;
		var loadedData=this.source.getData().length;
		var lastRow=this.firstRow+Math.max(this.getMinVisibleRowCount(),Math.min(loadedData,this.getMaxVisibleRowCount()));
		for(i=0,row=this.firstRow;row<lastRow;row++,i++){
			func(row,i);
		}
	},
	eachColumn:function(func){
		var i,col;
		var colCount=this.getColCount();
		for(i=0,col=this.firstCol;col<colCount;col++,i++){
			func(col,i,this.isColVisible(col));
		}
		for(;i<colCount;i++){
			func(null,i,false);
		}
		func(colCount,colCount,false);//extra column 
	},
	isColVisible:function(colIdx){
		return colIdx>=this.firstCol && colIdx<=this.lastCol;
	},
	showVSb:function(){
		return (this.getRowCount()-this.getMaxVisibleRowCount())>0;
	},
	showHSb:function(){
		return (this.totalColWidth>this.getMaxWidth());
	},
	setCellWidth:function(cell,colIdx,isVisible){
		if(isVisible){
			var selectionControlDelta=this.getSelectionControlDelta();
			var border=( ((!selectionControlDelta)&&colIdx===this.firstCol) || (colIdx===this.lastCol) )?1:1/*border width of non-edge cell*/;
			var sortIcon=(!!cell.th)?(this.source.isSortable(colIdx)?18:0/*18 iconpad for sort*/):0;//add room for sort icon on header only
			cell.div.style.width=(this.colWidths[colIdx+selectionControlDelta]-16/*8pad+8pad*/-sortIcon-border)+"px";			
		}else{
			cell.div.style.width=this.getMaxWidth()+"px";
		}
	},
	getSelectionControlDelta:function(){
		return this.source.getShowSelectionControls()?1:0;
	},
	calcAutoWidths2:function(){
		
	},
	calcAutoWidths:function(){
//console.warn("-------------- calcAutoWidths --------------");
		var selectionShown=this.source.getShowSelectionControls();
//console.warn("selectionShown",selectionShown);
		var showingVSb=this.control.main.body.vsb;
//console.warn("showingVSb",showingVSb);
		var initialMaxWidth=this.getMaxWidth();
//console.warn("initialMaxWidth",initialMaxWidth);
		var maxWidth=initialMaxWidth-17/*vsb width*/-1/*right border of rightmost column*/;
		var wrapper=this.control.main.body.wrapper;
		var table=wrapper.table;
		var ths=table.thead.tr.ths;
		var headers=[];
		var weights=[];
		var totalHeaders=0;
		var i,j,tds,colIdx,border,sortIcon;
		var selectionControlDelta=this.getSelectionControlDelta();
		if($("#tempCaclWidth").length === 0){
			var tempElem=document.createElement("span");
			tempElem.setAttribute("id","tempCaclWidth");
			document.body.appendChild(tempElem);
		}
		for(i=0;i<ths.length-1;i++){
			colIdx=i-selectionControlDelta;
//console.warn("\tcolIdx",i);
			sortIcon=this.source.isSortable(colIdx)?18:0;/*18 iconpad for sort*/
			border=((!selectionControlDelta&&colIdx===this.firstCol)||colIdx===this.lastCol)?1:1/*border width of non-edge cell*/;
//console.warn("\tborder",border);
			$("#tempCaclWidth")[0].innerHTML=$(ths[i].th).text();
			var currentHeader=$($(ths[i].th).children()[0]);
			$("#tempCaclWidth").css("font-weight", currentHeader.css("font-weight"));
			$("#tempCaclWidth").css("font-size", currentHeader.css("font-size"));
			$("#tempCaclWidth").css("font-style", currentHeader.css("font-style"));
			$("#tempCaclWidth").css("font-family", currentHeader.css("font-family"));
			headers[i]=($("#tempCaclWidth").width() + 1/*Fix IE's width being off by 1px*/) + 16/*8pad+8pad*/ + border + sortIcon + ((i===0 && selectionShown)?15/*make room for the checkbox*/:0);
//console.warn("\theaders[i]",headers[i]);
			totalHeaders+=headers[i];
		}
//console.warn("totalHeaders",totalHeaders);
		$("#tempCaclWidth").remove();
		var trs=table.tbody.trs;
		for(i=0;i<trs.length;i++){
			tds=trs[i].tds;
			for(j=0;j<tds.length-1;j++){
				var currentColumnMeta=this.source.getColumnMeta(j-selectionControlDelta);
				if(currentColumnMeta){
				   var currentFormatter=this.source.getColumnFormatter(currentColumnMeta);
					weights[j]=Math.max((weights[j]||1),currentFormatter.calcWidthWeight(tds[j].td));
				}else{
					weights[j]=Math.max((weights[j]||0),0);
				}
//if(i===0||i===(trs.length-1)){console.warn("\tweights["+j+"]",weights[j]);}
			}
		}
		var deltaToSplitUp=maxWidth-totalHeaders;
//console.warn("deltaToSplitUp",deltaToSplitUp);
		if(deltaToSplitUp>0){
			var totalWeights=_.reduce(weights,function(memo,num){return memo+num;},0);
//console.warn("totalWeights",totalWeights);
			headers=_.map(weights,function(weight,index){return headers[index]+(deltaToSplitUp*weight/totalWeights);});
		}
		var k,th,colWidth,border,sortIcon;
		var lastIndex=ths.length-2;
//console.warn("lastIndex",lastIndex);
		this.totalColWidth=0;
		for(i=0;i<ths.length-1;i++){
			th=ths[i];
			colWidth=headers[i];
			if(!showingVSb && i===lastIndex){colWidth+=17;}/*Add the vsb width to the last col width*/
			this.colWidths[i]=colWidth;
//console.warn("\tcolWidths["+i+"]",this.colWidths[i]);
			this.totalColWidth+=colWidth;
		}
//console.warn("totalColWidth",this.totalColWidth);
		this.calcFirstColMax();
		this.calcColRange();
		var selectionControlDelta=this.getSelectionControlDelta();
		for(i=selectionControlDelta;i<ths.length-1;i++){
			this.setCellWidth(ths[i],i-selectionControlDelta,true);
			for(j=0;j<trs.length;j++){
				tds=trs[j].tds;
				for(k=selectionControlDelta;k<ths.length-1;k++){
					this.setCellWidth(tds[k],k-selectionControlDelta,true);
//if(i===(ths.length-2)&&j===0){console.warn("\tsetCellWidth",k,tds[k].div.style.width);}
				}
			}
		}
		maxWidth=this.getMaxWidth();
//console.warn("DELTA maxWidth",maxWidth-initialMaxWidth);
		if(maxWidth-initialMaxWidth!==0){
			setTimeout($.proxy(this.calcAutoWidths,this),0);//try again in a sec
		}
//console.warn("maxWidth",maxWidth);
		border=1/*right border of rightmost column*/;
		wrapper.style.width=showingVSb?(maxWidth-17/*vsb width*/-border)+"px":maxWidth-border+"px";
//console.warn("wrapper.style.width",wrapper.style.width);
//console.warn("--------------------------------------------");
	},
	calcTotalColWidth:function(){
		this.totalColWidth=0;
		var i;
		for(i=0;i<this.colWidths.length;i++){
			this.totalColWidth+=this.colWidths[i];
		}
	},
	calcFirstColMax:function(){
		var i,total=0;
		var selectionControlDelta=this.getSelectionControlDelta();
		var maxWidth=this.getMaxWidth();
		for(i=this.colWidths.length-1;i>=0;i--){
			total+=this.colWidths[i];
			if(total>maxWidth){
				break;
			}
		}
		i++;
		i=(i===0)?0:i-selectionControlDelta;
		this.firstColMax=i;
		this.firstCol=Math.min(this.firstCol,this.firstColMax);
	},
	calcColRange:function(){
		var selectionControlDelta=this.getSelectionControlDelta();
		var i,total=(!selectionControlDelta)?0:this.colWidths[0];
		var maxWidth=this.getMaxWidth();
		var colCount=this.getColCount();
		for(i=this.firstCol;i<colCount;i++){
			total+=this.colWidths[i+selectionControlDelta];
			if(total>maxWidth){
				break;
			}
		}
		this.lastCol=i;
	}
};
sas.risk.slimtable.SlimTableState.__skip__=[];