jQuery.sap.declare("sas.risk.slimtable.Table");
jQuery.sap.require("sas.risk.slimtable.TableRenderer");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.risk.slimtable.meta.QueryMeta");
jQuery.sap.require("sas.risk.slimtable.SlimTableSource");
jQuery.sap.require("sas.risk.slimtable.SlimTable");
jQuery.sap.require("sas.risk.slimtable.meta.TableMeta");
jQuery.sap.require("sas.risk.slimtable.util.ClientSideUtil");
jQuery.sap.require("sas.risk.common.utils.CompositeUtil");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.risk.slimtable.meta.ColumnFormatterFactory");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.slimtable.Table", {
   metadata:{
      properties:{
      	"formattingKey":{type:"string[]", defaultValue:[]},
         "meta":{type:"any"},
         "data":{type:"any"},
         "preselection":{type:"any"},
         "disablement":{type:"any"},
         "minVisibleRows":{type:"int", defaultValue:2},
         "visibleRows":{type:"int", defaultValue:5},
         "clientSide":{type:"boolean",defaultValue:true},
   		"multiSelect" : {type : "boolean", group : "Behavior", defaultValue : false},
   		"noDataText" : {type : "string", group : "Appearance", defaultValue : null, deprecated: true},
         "showSelectionControls": {type: "boolean", group: "Behavior", defaultValue: true},
         "clickScope":{type:"sas.risk.slimtable.ClickScope",defaultValue:sas.risk.slimtable.ClickScope.Row},
         "sortScope":{type:"sas.risk.slimtable.SortScope",defaultValue:sas.risk.slimtable.SortScope.SingleSort},
   		"title":{type:"string"},
   		"required":{type:"boolean"},
   		"hideActionColumnOnReadOnly":{type:"boolean",defaultValue:true},
   		"beforeRefreshAllHook":{type:"any"},
   		"afterRefreshAllHook":{type:"any"}
      },
		associations : {
			/**
			 * Association to controls / IDs which describe this control (see WAI-ARIA attribute aria-describedby).
			 */
			ariaDescribedBy : {type : "sap.ui.core.Control", multiple : true, singularName : "ariaDescribedBy"},
			/**
			 * Association to controls / IDs which label this control (see WAI-ARIA attribute aria-labelledby).
			 */
			ariaLabelledBy : {type : "sap.ui.core.Control", multiple : true, singularName : "ariaLabelledBy"}
		},
      aggregations:{
      	"plugins":{type:"sas.risk.slimtable.plugins.TablePlugin", singularName:"plugin"}
      },
      events:{
			rowChange : {
				parameters : {
					rowIdx: {type: "int"},
					colIdx: {type: "int"},
					colMeta: {type: "any"},
					value: {type:"any"}
				}
			},
			rowClick : {
				parameters : {
					rowIdx: {type : "int"}
				}
			},
			cellClick: {
				parameters : {
					rowIdx: {type : "int"},
					colIdx: {type : "int"}
				}
			},
			rowSelect: {
				parameters : {
					rowIdx: {type : "int"}
				}
			}
		}
   },
   renderer:"sas.risk.slimtable.TableRenderer"
});
sas.risk.slimtable.Table.prototype.getNewQueryMeta=function(){
	return new sas.risk.slimtable.meta.QueryMeta();
};
sas.risk.slimtable.Table.prototype.onBeforeRendering=function(){
	if(!this.topBeginLayout.getItems().length && !this.advSearchLayout.getItems().length && this.getTitle()){
		this.topBeginLayout.addItem(this.titleField);//Slide title down to the top begin position if there is nothing there already and no adv search pills
	}else if(this.advSearchLayout.getItems().length && this.getTitle() && !this.titleLayout.getItems().length){
		this.titleLayout.addItem(this.titleField);//add title back on title layout if there is content in adv search layout
	}
	//For components who create thisTable in onBeforeRendering block this code is called twice due to invalidation call caused by various reasons like setting attributes on the widget
	//This flag is used to prevent this code getting executed more than once during the multiple onBeforeRendering calls.
	if(this.renderedBefore){return;} 
	if(this.getEnabled()===false){
		this.table.getSource().setShowSelectionControls(false);
		this.fireEvent("riskTabledisabled",{"disabled":true});
	}
	if(this.getShowActionColumn()){
		this.addColumn(this.actionColumn);
	}
	this.renderedBefore=true;
};
sas.risk.slimtable.Table.prototype.getShowActionColumn=function(){
	return (!(this.getEnabled()===false && this.getHideActionColumnOnReadOnly()) && this.actionColumn);
};
sas.risk.slimtable.Table.prototype.init=function(){
	this.rb = sas.risk.slimtable.util.SlimTableUtil.getBundle();
	this.setFlexDirection(sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column);
	this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Stretch);
	this.footerLayout = this.createFooterLayout();
	this.addItem(this.createTopTitleArea());
	this.addItem(this.createHeaderLayout());
	this.addItem(this.createTable());
	this.addItem(this.footerLayout);
	this.filters = [];
	this.filterColumnValuesMap = {};
    this.doneFilterColumnValuesMap = false;
	this.loadedCount = 0;
	this.totalCount = 0;
    this.defaultTypeFilter = null;
	this.query = this.getNewQueryMeta();
	this.query._original_=true;
	this.addStyleClass("RiskTab");
	//COMPOSITES
	sas.risk.common.utils.CompositeUtil.addComposite(this,"rowChange",this.getTable);
	sas.risk.common.utils.CompositeUtil.addComposite(this,"rowClick",this.getTable);
	sas.risk.common.utils.CompositeUtil.addComposite(this,"rowSelect",this.getTable);
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cellClick",this.getTable);
	sas.risk.common.utils.CompositeUtil.addComposite(this,"clickScope",this.getTable);
   sas.risk.common.utils.CompositeUtil.addComposite(this,"sortScope",this.getTable);
   var that=this;
	$(window).resize(function(){
		$.proxy(that.actionOnWindowResize,that)();
	});
};
sas.risk.slimtable.Table.prototype.setTitle=function(value){
	this.setProperty("title",value);
	if(this.titleField){
		this.titleField.setText(value);
	}
};
/******
 * Plugins
 */
sas.risk.slimtable.Table.prototype.addPlugin=function(plugin){
	this.addAggregation("plugins",plugin, false);
	plugin.setTable(this);
	var position = plugin.getPosition();
	var pluginItem = plugin.getPluginLayout ? plugin.getPluginLayout() : plugin;
	if(position){
		switch(position){
			case sas.risk.slimtable.TableLayout.TOP_BEGIN:
				this.topBeginLayout.addItem(pluginItem);
				break;
			case sas.risk.slimtable.TableLayout.TOP_CENTER:
				this.topCenterLayout.addItem(pluginItem);
				break;
			case sas.risk.slimtable.TableLayout.TOP_END:
				this.topEndLayout.addItem(pluginItem);
				break;
			case sas.risk.slimtable.TableLayout.BOTTOM_BEGIN:
				this.bottomBeginLayout.addItem(pluginItem);
				break;
			case sas.risk.slimtable.TableLayout.BOTTOM_CENTER:
				this.bottomCenterLayout.addItem(pluginItem);
				break;
			case sas.risk.slimtable.TableLayout.BOTTOM_END:
				this.bottomEndLayout.addItem(pluginItem);
				break;
		}
	}
	return this;
};

/******
 * BUILDING
 */
sas.risk.slimtable.Table.prototype.build=function(){
	if(!this.built && this.query._original_){//first time building and the query is the plain vanilla one, then recreate it to catch location+
		this.doCreateQuery();
	}
	this.built=true;
	this.latch=true;
	var that=this;
	var data=this.getData();
	data=$.isFunction(data)?data:$.proxy(this.fetchData,this,data,this.query);
	data(this.query).done(function(json){
		//handle the client side filtering of data for the ones which support client side
		if(that.getClientSide()){
			json=that.actionFilterDataClient(json,that.query);
		}
		that.dataReceived(json);
	});
};
sas.risk.slimtable.Table.prototype.dataReceived=function(json){
	this.latch=false;
	this.totalCount=json.count||0;
	var source=this.table.getSource();
	var finalData=null;
	if(json.start){//APPEND DATA
		this.loadedCount+=json.items.length;
		var prevData=source.getData();
		prevData=prevData.concat(json.items);
		finalData=prevData;
		source.update(this.query,prevData,true/*scroll to bottom of table*/,false/*preserve selections*/);
	}else{//REPLACE DATA
		this.loadedCount=json.items.length;
		finalData=json.items;
		source.update(this.query,json.items,false,true/*clear selections*/);
	}
	this.fireEvent("dataReceived",{"data": finalData});
	var beforeRefreshAllHook=this.getBeforeRefreshAllHook();
	if(beforeRefreshAllHook && $.isFunction(beforeRefreshAllHook)){
		beforeRefreshAllHook(json);
	}
	this.refreshAll();
	var afterRefreshAllHook=this.getAfterRefreshAllHook();
	if(afterRefreshAllHook && $.isFunction(afterRefreshAllHook)){
		afterRefreshAllHook(json);
	}
};

sas.risk.slimtable.Table.prototype.fetchData=function(baseUrl,query){
	var url=baseUrl+(baseUrl.indexOf("?") > 0 ? "&" : "?")+query.toString();
	return $.getJSON(url);
};
/*************
 * CREATES
 *************/

sas.risk.slimtable.Table.prototype.createHeaderLayout=function(){
	this.advSearchLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		flexWrap:sas.risk.common.layout.flexlayout.FlexLayoutWrap.Wrap
	}).addStyleClass("riskTabPillBox");
	this.toolbarLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row,
		justifyContent:sas.risk.common.layout.flexlayout.FlexLayoutJustifyContent.SpaceBetween,
		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
		flexWrap:sas.risk.common.layout.flexlayout.FlexLayoutWrap.Wrap
	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
		alignSelf:sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.Stretch
	})).addStyleClass("RiskTabToolbarLayout");
	this.topBeginLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center
	});
	this.topCenterLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center
	});
	this.topEndLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center
	});
	
	this.toolbarLayout.addItem(this.topBeginLayout);
	this.toolbarLayout.addItem(this.topCenterLayout);
	this.toolbarLayout.addItem(this.topEndLayout);
	
	this.headerLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		layoutData: new sas.risk.common.layout.flexlayout.FlexLayoutData({
			flexShrink:0
		}),
		items:[this.advSearchLayout,this.toolbarLayout]
	});
	return this.headerLayout;
};
sas.risk.slimtable.Table.prototype.createFooterLayout=function(){
	this.footerLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row,
		layoutData: new sas.risk.common.layout.flexlayout.FlexLayoutData({
			flexShrink:0
		}),
		justifyContent:sas.risk.common.layout.flexlayout.FlexLayoutJustifyContent.SpaceBetween
	}).addStyleClass("RiskTabFooterLayout");
	this.bottomBeginLayout=new sas.risk.common.layout.flexlayout.FlexLayout({});
	this.bottomCenterLayout=new sas.risk.common.layout.flexlayout.FlexLayout({});
	this.bottomEndLayout=new sas.risk.common.layout.flexlayout.FlexLayout({});
	this.footerLayout.addItem(this.bottomBeginLayout);
	this.footerLayout.addItem(this.bottomCenterLayout);
	this.footerLayout.addItem(this.bottomEndLayout);
	return this.footerLayout;
};

sas.risk.slimtable.Table.prototype.createTopTitleArea=function(){
   this.titleField = new sas.hc.m.Label({
		textAlign: sap.ui.core.TextAlign.Begin, 
		design: sap.m.LabelDesign.Standard,
		text: this.getTitle()
	}).addStyleClass("RiskTabTitleLabel");
   this.titleLayout = new sas.risk.common.layout.flexlayout.FlexLayout({
      items: [this.titleField],
      layoutData: new sas.risk.common.layout.flexlayout.FlexLayoutData({
			flexShrink:0
		}),
      visible:false
   }).addStyleClass("RiskTabTitle");
   return this.titleLayout;
};

sas.risk.slimtable.Table.prototype.createTable=function(){
   this.table = new sas.risk.slimtable.SlimTable({
   	sort: $.proxy(this.actionInitSort,this),
   	rerender:$.proxy(this.doRerenderCallback,this)
	});
	var source=new sas.risk.slimtable.SlimTableSource(false,false);
	this.table.setSource(source);
   this.table.addStyleClass("RiskTabWidget");
   return this.table;
};

/******************
 * REFRESH
 *****************/
sas.risk.slimtable.Table.prototype.refreshAll=function(){
	this.setBusy(true);
	if(this.titleLayout && this.titleLayout.getVisible()){this.refreshTitle();}
	//Let all the plugins update themselves as needed from the query
	var i, plugins = this.getPlugins();
	for(i=0; i<plugins.length; i++){
		var plugin = plugins[i];
		if(plugin.refresh){
			plugin.refresh();
		}
	}
	this.setBusy(false);
};
sas.risk.slimtable.Table.prototype.refreshTitle=function(){
   //TITLE
	this.titleField.setText(this.getTitle());
	this.titleLayout.setVisible(!!this.getTitle());
};

/****************
 * ACTIONS
 ****************/
sas.risk.slimtable.Table.prototype.actionFilterTable=function(){
	this.loadedCount=0;
	this.doCreateQuery();
	this.build();
};
sas.risk.slimtable.Table.prototype.actionFilterDataClient=function(json,query){
	var data=json.items;
	var filteredData=sas.risk.slimtable.util.ClientSideUtil.filter(data,this);
	
	//SORT
	var sort=query.getSort();
	var sortName=sort.sortName;
	var sortDirection=sort.sortDirection;
	var colMeta=sort.columnMeta;
	if(sortName && sortDirection){
		sas.risk.slimtable.util.ClientSideUtil.sort(filteredData,colMeta);
	}
	
	json.count=filteredData.length;
	json.items=filteredData;
	return json;
};

sas.risk.slimtable.Table.prototype.ifAllDataLoaded=function(){
	return this.totalCount === this.loadedCount;
};

sas.risk.slimtable.Table.prototype.actionSort=function(){
	this.loadedCount=0;
	this.doCreateQuery();
	this.build();
};
sas.risk.slimtable.Table.prototype.actionInitSort=function(){
	this.actionSort();
};
sas.risk.slimtable.Table.prototype.actionOnWindowResize=function(){
	//TODO: not sure what needs to go here now
};
/************
 * DO
 ************/
sas.risk.slimtable.Table.prototype.doRefresh=function(previousDef){
	var def=previousDef||$.Deferred();
	var that=this;
	this.setBusyIndicatorDelay(0);
	this.setBusy(true);
	if(this.table.getSource().getMaxVisibleRowCount()===-1){
		var callback=function(event){
			that.table.detachRerender(callback);
			that.doRerenderCallback(event,def);
		};
		that.table.attachRerender(callback);
		return def;
	}
	this.loadedCount=0;
	this.doCreateQuery();
	this.table.attachRenderComplete(function abc(){
		if(that.latch){return;}
		that.table.detachRenderComplete(abc);
		that.setBusy(false);
		def.resolve();
	});
	this.build();
	return def;
};
sas.risk.slimtable.Table.prototype.doRerenderCallback=function(event,def){
	//compare event prevRowCount && rowCount
	var vizRows=this.getVisibleRows();
	if(vizRows!==event.rowCount){
		this.setVisibleRows(this.table.getSource().getMaxVisibleRowCount());
		this.doRefresh(def);
	}
};
sas.risk.slimtable.Table.prototype.doApplyDefaultQuery=function(query){
   this.doApplyQuery(query);
};
sas.risk.slimtable.Table.prototype.applyQueryHook=function(query){};
sas.risk.slimtable.Table.prototype.doApplyQuery=function(query){
	//Let all the plugins update themselves as needed from the query
	var i;
	var plugins = this.getPlugins();
	for(i=0; i<plugins.length; i++){
		var plugin=plugins[i];
		if(plugin.refresh){
			plugin.refresh(query);
		}
	}
   
   //SORT COLUMN
   var tableMeta=this.table.getSource().getTableMeta();
   var sort=query.getSort();
   var sort2=query.getSort2();
   var sort3=query.getSort3();

   //tableMeta.setSortInfo(tableMeta.getColumnByName(sort.sortName),sort.sortDirection,sort.columnMeta);
   //tableMeta.setSortInfo(tableMeta.getColumnByName(sort2.sortName),sort2.sortDirection,sort2.columnMeta);
   //tableMeta.setSortInfo(tableMeta.getColumnByName(sort3.sortName),sort3.sortDirection,sort3.columnMeta);

   var combinedSorts = [sort,sort2,sort3];

   /*
	tableMeta.setSortedColumn(tableMeta.getColumnByName(sort.sortName),sort.sortDirection);
	var sortedColumns = query.getSortedColumns();
	tableMeta.setSortedColumn(tableMeta.getColumnByName(sort.sortName),sort.sortDirection, "ignore");
	_.each(sortedColumns, function(sortedColumn, i){
		if ( i > 0) { // only do this for other sorted columns
			tableMeta.setSortedColumn(tableMeta.getColumnByName(sortedColumn.name),sortedColumn.sortDirection, "add");
		}
	});

	*/
   //ORDERED COLUMNS
	var visibleColumns=query.getOrderedColumns();
	tableMeta.setVisibleColumns(visibleColumns);
	
	// Sorted Columns
	//tableMeta.setSortedColumns(query.getSortedColumns());
	tableMeta.setSortedColumnsWithQuery(visibleColumns,combinedSorts);

	//Additional query apply hook
	this.applyQueryHook(query);
	
   this.doRefresh();
};
sas.risk.slimtable.Table.prototype.createQueryHook=function(query){};
sas.risk.slimtable.Table.prototype.doCreateQuery=function(){
	var query=this.getNewQueryMeta();
	
	//Let all the plugins update the query as needed
	var i;
	var plugins = this.getPlugins();
	for(i=0; i<plugins.length; i++){
		var plugin = plugins[i];
		if(plugin.updateQuery){
			plugin.updateQuery(query);
		}
	}
	//LIMIT
	query.setLimit(this.getVisibleRows());
	//SORT COLUMN
	var tableMeta=this.table.getSource().getTableMeta();
	var sortedColumns=tableMeta.getSortedColumns();

	if (sortedColumns.length > 1) {
		sortedColumns.sort(function (a,b) {
			return a.sortOrder-b.sortOrder;
		});
	}

	 if (sortedColumns[0]) {
		query.setSort(sortedColumns[0].getColName(),sortedColumns[0].getSortDirection(),sortedColumns[0]);
	 }
	 if (sortedColumns[1]) {
		query.setSort2(sortedColumns[1].getColName(),sortedColumns[1].getSortDirection(),sortedColumns[1]);
	 }

	 if (sortedColumns[2]) {
		query.setSort3(sortedColumns[2].getColName(),sortedColumns[2].getSortDirection(),sortedColumns[2]);
	 }

	 //DD: Testing a combined sortName/sortDirection
	 //query.setCombinedSortName();
	 //query.setCombinedSortDirection();
 
	//ORDERED COLUMNS
	var visibleColumns=tableMeta.getVisibleColumns();
	query.setOrderedColumns(visibleColumns);

	// sorted columns
	query.setSortedColumns(sortedColumns);
	//START
	query.setStart(this.loadedCount);
	//Additional query update hook
	this.createQueryHook(query);
	
	this.query=query;
	this.fireEvent("queryChange",{"query":this.query});
};
/***************
 * OPTIONS
 ***************/
sas.risk.slimtable.Table.prototype.setTitle=function(titleTxt){
	this.setProperty("title",titleTxt);
	this.titleLayout.setVisible(true);
};
sas.risk.slimtable.Table.prototype.setMinVisibleRows=function(value){
	if(!value){return;}
	this.setProperty("minVisibleRows",value);
	this.table.getSource().setMinVisibleRowCount(value);
};
sas.risk.slimtable.Table.prototype.setVisibleRows=function(value){
	if(!value){return;}
	this.setProperty("visibleRows",value);
   this.query.setLimit(value);
	this.table.getSource().setMaxVisibleRowCount(value);
};
sas.risk.slimtable.Table.prototype.setActionColumn=function(actionColumn){
	this.setProperty("actionColumn",actionColumn);
	this.actionColumn.setTemplate(actionColumn);
};
sas.risk.slimtable.Table.prototype.setFormattingKey=function(formattingKey){
	this.setProperty("formattingKey",formattingKey);
	var formatterFactory=sas.risk.slimtable.meta.ColumnFormatterRegistry.getFactory(formattingKey);
	var tableMeta=this.table.getSource().getTableMeta();
	if(tableMeta){//meta loaded before formattingKey
		tableMeta.setFormatterFactory(formatterFactory);
	}
};
sas.risk.slimtable.Table.prototype.setMeta=function(meta){
	this.setProperty("meta",meta);
	var that=this;
	var tableMeta=new sas.risk.slimtable.meta.TableMeta(meta);
	var formattingKey=this.getFormattingKey();
	if(formattingKey){//formattingKey loaded before meta
		var formatterFactory=sas.risk.slimtable.meta.ColumnFormatterRegistry.getFactory(formattingKey);
		tableMeta.setFormatterFactory(formatterFactory);
	}
	this.table.getSource().setTableMeta(tableMeta);
	this.query.setOrderedColumns(tableMeta.getVisibleColumns());
	//@DD
	//var sortedColumn=tableMeta.getSortedColumn();
	var sortedColumns=tableMeta.getSortedColumns();
	
	//DD: 5:37
	/* if(sortedColumns){
		_.each(sortedColumns, function(sortedColumn) {
			that.query.setSort(sortedColumn.getColName(),sortedColumn.getSortDirection(),sortedColumn);
		});	
	} */

	if (sortedColumns[0]) {
		that.query.setSort(sortedColumns[0].getColName(),sortedColumns[0].getSortDirection(),sortedColumns[0]);
	 }

	 if (sortedColumns[1]) {
		that.query.setSort2(sortedColumns[1].getColName(),sortedColumns[1].getSortDirection(),sortedColumns[1]);
	 }

	 if (sortedColumns[2]) {
		that.query.setSort3(sortedColumns[2].getColName(),sortedColumns[2].getSortDirection(),sortedColumns[2]);
	 }

	this.fireEvent("queryChange",{"query":this.query});
};
sas.risk.slimtable.Table.prototype.addColumn=function(columnMeta){
	this.table.getSource().getTableMeta().addColumn(columnMeta);
};
sas.risk.slimtable.Table.prototype.addActionColumn=function(columnMeta){
	this.actionColumn=columnMeta;
};
sas.risk.slimtable.Table.prototype.setMultiSelect=function(value){
	this.setProperty("multiSelect",value);
	this.table.getSource().setMultiSelect(value);
};
sas.risk.slimtable.Table.prototype.setNoDataText=function(value){
	this.setProperty("noDataText",value);
};
sas.risk.slimtable.Table.prototype.setShowSelectionControls=function(value){
	this.setProperty("showSelectionControls",value);
	this.table.getSource().setShowSelectionControls(value);
};
sas.risk.slimtable.Table.prototype.setPreselection=function(preselection){
	this.setProperty("preselection",preselection);
	this.table.getSource().setPreselection(preselection);
};
sas.risk.slimtable.Table.prototype.setDisablement=function(disablement){
	this.setProperty("disablement",disablement);
	this.table.getSource().setDisablement(disablement);
};
sas.risk.slimtable.Table.prototype.getSource=function(){
	return this.table.getSource();
};
sas.risk.slimtable.Table.prototype.getQuery=function(){
	return this.query;
};
sas.risk.slimtable.Table.prototype.getVisibleColumnNames=function(){
	return this.table.getSource().getTableMeta().getVisibleColumnNames();
};
sas.risk.slimtable.Table.prototype.getColumnNames=function(){
	return this.table.getSource().getTableMeta().getColumnNames();
};
/**************
 * Helpers
 **************/
sas.risk.slimtable.Table.prototype.getTable=function(){
	return this.table;
};
sas.risk.slimtable.Table.prototype.getTableMeta=function(){
	return this.table.getSource().getTableMeta();
};
sas.risk.slimtable.Table.prototype.getTitleField=function(){
	return this.titleField;
};
/**************
 * COMPOSITES
 **************/
sas.risk.common.utils.CompositeUtil.mixinPropertySupport(sas.risk.slimtable.Table);
sas.risk.common.utils.CompositeUtil.mixinAggregationSupport(sas.risk.slimtable.Table);
sas.risk.common.utils.CompositeUtil.mixinEventSupport(sas.risk.slimtable.Table);