jQuery.sap.declare("sas.risk.slimtable.meta.QueryMeta");
jQuery.sap.require("sap.ui.commons.Label");
jQuery.sap.require("sas.risk.slimtable.meta.filter.AndFilter");
jQuery.sap.require("sas.risk.slimtable.meta.filter.OrFilter");

sas.risk.slimtable.meta.QueryMeta=function(filters,sortName,sortDirection,limit,start){
	this.filters=filters||{};
	this.sortName=sortName;
	this.sortDirection=sortDirection;
	this.sortColumnMeta=null;
	this.limit=limit;
	this.start=start||0;
	this.orderedColumns=[];
	this.sortedColumns=[];
	this.orFilter=false;
	this.sortName2=null;
	this.sortDirection2=null;
	this.sortColumn2Meta=null;
	this.sortName3=null;
	this.sortDirection3=null;
	this.sortColumn3Meta=null;
};
sas.risk.slimtable.meta.QueryMeta.prototype={
	clone:function(){
		var cloneFilters=_.mapObject(this.filters,function(filter){return filter?filter.clone():filter;});
		var qMeta=new sas.risk.slimtable.meta.QueryMeta(cloneFilters,this.sortName,this.sortDirection,this.limit,this.start);
		qMeta.setOrderedColumns(this.getOrderedColumns() ? this.getOrderedColumns().slice(0) : []);
		qMeta.setSortedColumns(this.getSortedColumns() ? this.getSortedColumns().slice(0) : []);
		return qMeta;
	},
	setToStringHook:function(callback){
		this.toStringHook=callback;
	},
	toString:function(){
		var query={};
		
		if(this.getSortedColumns().length > 0) {
			var sortByQuery="";
			_.each(this.getSortedColumns(), function(sortedColumn) {
				sortByQuery += sortedColumn.getColName()+":"+sortedColumn.getSortDirection()+",";
			});

			query["sortBy"]=sortByQuery.slice(0,-1); //remove the last , from the string
		} 

		if(this.limit!==undefined){query["limit"]=this.limit;}
		if(this.start!==undefined){query["start"]=this.start;}
		var filters=_.filter(this.filters,function(filter){return !!filter;});
		var filters=_.mapObject(filters,function(filter){return filter.clone();});
		if($.isFunction(this.toStringHook)){
			filters=this.toStringHook(filters);
		}
		switch(_.size(filters)){
			case 0://no filters... nothing to add to query
			break;
			case 1://only 1 filter.. no need to AND anything
				_.each(filters,function(filter){ query["filter"]=filter.toString();});
			break;
			default://multiple filters.. AND them all
				var andAll=new sas.risk.slimtable.meta.filter.AndFilter();
				_.each(filters,function(filter){andAll.addFilter(filter);});
				query["filter"]=andAll.toString();
			break;
		}
		return $.param(query);
	},
	_mergeFilter:function(currentFilter,filter,isAnd){
		if(!currentFilter){
			currentFilter=filter;
		}else if(isAnd && currentFilter instanceof sas.risk.slimtable.meta.filter.AndFilter){
			currentFilter.addFilter(filter);
		}else if(!isAnd && currentFilter instanceof sas.risk.slimtable.meta.filter.OrFilter){
			currentFilter.addFilter(filter);
		}else{
			currentFilter=isAnd?new sas.risk.slimtable.meta.filter.AndFilter([currentFilter,filter])
								  :new sas.risk.slimtable.meta.filter.OrFilter([currentFilter,filter]);
		}
		return currentFilter;
	},
	addQueryFilter:function(filter,filterName,isAnd){
		this.filters[filterName]=this._mergeFilter(this.filters[filterName],filter,isAnd);
	},
	getQueryFilter: function(filterName){
	   return this.filters[filterName];
	},
	setQueryFilter: function(filter,filterName){
		if($.isArray(filter)){console.error("The setQueryFilter method takes a Filter object, not an array.");}
	   this.filters[filterName]=filter;
	},
	setSort:function(name,sortDirection,meta){
		this.sortName=name;
		this.sortDirection=sortDirection;
		this.sortColumnMeta=meta;
	},
	setSort2:function(name,sortDirection,meta){
		this.sortName2=name;
		this.sortDirection2=sortDirection;
		this.sortColumn2Meta=meta;
	},
	setSort3:function(name,sortDirection,meta){
		this.sortName3=name;
		this.sortDirection3=sortDirection;
		this.sortColumn3Meta=meta;
	},
	setLimit:function(limit){
		this.limit=limit;
	},
	setStart:function(start){
		this.start=start;
	},
	getOrderedColumns:function(){
		return this.orderedColumns;
	},
	setOrderedColumns:function(columns){
		this.orderedColumns=columns;
	},
	getSortedColumns:function(){
		return this.sortedColumns;
	},
	setSortedColumns:function(columns){
		this.sortedColumns=columns;
	},
	setCombinedSortName:function(){
		this.sortName = this.sortName +
						( (this.sortName2 !== null) ? this.sortName2 : "") +
						( (this.sortName3 !== null) ? this.sortName3 : "");
	},
	setCombinedSortDirection:function(){
		this.sortDirection=this.sortDirection + 
		                   ( (this.sortDirection2 !== null) ? this.sortDirection2 : "") + 
		                   ( (this.sortDirection3 !== null) ? this.sortDirection3 : "");
	},
	getSort:function(){
	   return {sortName:this.sortName,sortDirection:this.sortDirection,columnMeta:this.sortColumnMeta};
	},
	getSort2:function(){
		return {sortName:this.sortName2,sortDirection:this.sortDirection2,columnMeta:this.sortColumn2Meta};
	 },
	 getSort3:function(){
		return {sortName:this.sortName3,sortDirection:this.sortDirection3,columnMeta:this.sortColumn3Meta};
	 },
	getFilterByName:function(name,filterName){
		return !!this.filters[filterName] ? this.filters[filterName].findByName(name)
				               : [];
	},
	matchFilters:function(fils){
		var localFilters=_.pick(this.filters,function(val){return !!val;});
		var otherFilters=_.pick(fils,function(val){return !!val;});
		var sizeMatches=_.size(localFilters)===_.size(otherFilters);
		if(sizeMatches){
			return _.every(localFilters,function(val,key){
				var otherVal=otherFilters[key];
				return !!otherVal && val.equals(otherVal);
			});
		}
		return false;
	},
	matchVisibleColumns:function(columns){
		var thisCols=_.map(this.orderedColumns,function(c){return c.getColName();});
		var thatCols=_.map(columns,function(c){return c.getColName();});
		return _.isEqual(thisCols,thatCols);
	},
	matchOrderedColumns:function(columns){
		var thisCols=_.map(this.orderedColumns,function(c){return c.getColName();});
		var thatCols=_.map(columns,function(c){return c.getColName();});
		return _.isEqual(thisCols,thatCols);
	},
	matchSortedColumns:function(columns){
		var thisCols=_.map(this.sortedColumns,function(c){return c.getColName()+c.getSortDirection();});
		var thatCols=_.map(columns,function(c){return c.getColName()+c.getSortDirection();});
		return _.isEqual(thisCols,thatCols);
	},
	equal:function(qMeta,ignorePaging){
		console.warn("-----------------------------------------------");
		console.warn("FILTERS",this.filters,this.matchFilters(qMeta.filters),qMeta.filters);
		console.warn("SORTNAME",this.sortName,this.sortName===qMeta.sortName,qMeta.sortName);
		console.warn("SORTDIR",this.sortDirection,this.sortDirection===qMeta.sortDirection,qMeta.sortDirection);
		console.warn("SORTNAME2",this.sortName2,this.sortName2===qMeta.sortName2,qMeta.sortName2);
		console.warn("SORTDIR2",this.sortDirection2,this.sortDirection2===qMeta.sortDirection2,qMeta.sortDirection2);
		console.warn("SORTNAME3",this.sortName3,this.sortName3===qMeta.sortName3,qMeta.sortName3);
		console.warn("SORTDIR3",this.sortDirection3,this.sortDirection3===qMeta.sortDirection3,qMeta.sortDirection3);
		console.warn("MATCHORDEREDCOLUMNS",this.orderedColumns,this.matchOrderedColumns(qMeta.orderedColumns));
		//console.warn("MATCHSORTEDCOLUMNS",this.sortedColumns,this.matchSortedColumns(qMeta.sortedColumns));
	   return this.matchFilters(qMeta.filters)
	        && this.sortName===qMeta.sortName
	        && this.sortDirection===qMeta.sortDirection
	         && (ignorePaging||(this.limit===qMeta.limit))
	         && (ignorePaging||(this.start===qMeta.start))
			 && this.matchOrderedColumns(qMeta.orderedColumns)
			 //&& this.matchSortedColumns(qMeta.sortedColumns);

			 // TODO: if using composite remove these lines
			 && this.sortName2===qMeta.sortName2
			 && this.sortDirection2===qMeta.sortDirection2
			 && this.sortName3===qMeta.sortName3
			 && this.sortDirection3===qMeta.sortDirection3;
	},
	delta:function(qMeta){
		var deltas=[];
		if(!this.matchFilters(qMeta.filters)){deltas.push("FILTERS");}
		if(this.sortName!==qMeta.sortName){deltas.push("SORT_NAME");}
		if(this.sortDirection!==qMeta.sortDirection){deltas.push("SORT_DIRECTION");}
		if(this.limit!==qMeta.limit){deltas.push("LIMIT");}
		if(this.start!==qMeta.start){deltas.push("START");}
		if(!this.matchVisibleColumns(qMeta.orderedColumns)){deltas.push("COLUMNS_VISIBLE");}
		if(!this.matchOrderedColumns(qMeta.orderedColumns)){deltas.push("COLUMNS_ORDER");}
		//if(!this.matchSortedColumns(qMeta.sortedColumns)){deltas.push("COLUMNS_SORTED");}

		// TODO: if using composite remove these lines
		if(this.sortName2!==qMeta.sortName2){deltas.push("SORT_NAME2");}
		if(this.sortDirection2!==qMeta.sortDirection2){deltas.push("SORT_DIRECTION2");}
		if(this.sortName3!==qMeta.sortName3){deltas.push("SORT_NAME3");}
		if(this.sortDirection3!==qMeta.sortDirection3){deltas.push("SORT_DIRECTION3");}

		return deltas;
	}
};