jQuery.sap.declare("sas.risk.slimtable.meta.TableMeta");
jQuery.sap.require("sas.risk.slimtable.meta.ColumnMeta");

sas.risk.slimtable.meta.TableMeta = function(meta) {
	this.defaultTitle = meta.defaultTitle;
	this.columns = [];
	this.visibleColumns = [];
	this.defaultVisibleColumns = [];
	this.fixedColumns = [];
	this.filterableColumns = [];
	this.searchableColumns = [];
	this.queryableColumns = [];
	this.defaultSortedColumn=null;
	this.addColumns(sas.risk.slimtable.meta.ColumnMeta.createColumns(meta.columns,this));
	if(meta.defaultColumns){this.addDefaultColumns(sas.risk.slimtable.meta.ColumnMeta.createColumns(meta.defaultColumns,this));}
	this.formatterFactory=null;
	this.preferenceKey=meta.prefFieldKey;
};
sas.risk.slimtable.meta.TableMeta.prototype = {
	getDefaultSortedColumn : function() {
		return this.defaultSortedColumn;
	},
	getVisibleAndFixedColumns : function() {
		if(this.visibleAndFixedColumns){return this.visibleAndFixedColumns;}
		this.visibleAndFixedColumns=this.visibleColumns.slice(0);
		var i,col;
		for(i=0;i<this.fixedColumns.length;i++){
			col=this.fixedColumns[i];
			if(col.getDisplayOrder()===-1){
				this.visibleAndFixedColumns.unshift(col);
			}else{
				this.visibleAndFixedColumns.push(col);
			}
		}
		return this.visibleAndFixedColumns;
	},
	getVisibleColumns : function() {
		return this.visibleColumns;
	},
	getDefaultVisibleColumns : function() {
		return this.defaultVisibleColumns;
	},
	getHiddenColumns : function() {
		return this.hiddenColumns;
	},
	getFilterableColumns : function() {
		return this.filterableColumns;
	},
	getQueryableColumns : function() {
		return this.queryableColumns;
	},
	getSearchableColumns : function() {
		return this.searchableColumns;
	},
	getPreferenceKey : function(){
		return this.preferenceKey;
	},
	addDefaultColumns : function(defaultColumns) {
		for(var i=0; i<defaultColumns.length; i++){
			this.defaultVisibleColumns.push(defaultColumns[i]);
		}
		this.defaultVisibleColumns.sort(function(a, b){
		   return a.getDisplayOrder() - b.getDisplayOrder();
		});
	},
	addColumns : function(columns) {
		for (var i = 0; i < columns.length; i++) {
			this.addColumn(columns[i]);
		}
		this.visibleColumns.sort(function(a, b){
		   return a.getDisplayOrder() - b.getDisplayOrder();
		});
		
	},
	addColumn : function(column) {
		if (column.getSortDirection() !== "None"){
			this.defaultSortedColumn=column;
		}
		if(column.isColFixed()){
			this.fixedColumns.push(column);
		}else if (column.isColVisible()) {
			// this.visibleColumns[column.getDisplayOrder()]=column;
		   this.visibleColumns.push(column);
		}
		if (column.isColFilterable()) {
			this.filterableColumns.push(column);
		}
		if (column.isColSearchable()) {
			this.searchableColumns.push(column);
		}
		if (column.isColQueryable()) {
			this.queryableColumns.push(column);
		}
		if(this.formatterFactory){
			var formatter=this.formatterFactory.getFormatter(column.getType().toUpperCase(),column.getColName());
			column.setFormatter(formatter);			
		}
		this.columns.push(column);
	},
	removeColumn:function(column){
		var a=this.visibleColumns.indexOf(column);
		if(a !== -1){this.visibleColumns.splice(a,1);}
		var b=this.columns.indexOf(column);
		if(b !== -1){this.columns.splice(b,1);}
		if(this.defaultSortedColumn===column){this.defaultSortedColumn===null;}
		var d=this.defaultVisibleColumns.indexOf(column);
		if(d !== -1){this.defaultVisibleColumns.splice(d,1);}
		var e=this.filterableColumns.indexOf(column);
		if(e !== -1){this.filterableColumns.splice(e,1);}
		var f=this.searchableColumns.indexOf(column);
		if(f !== -1){this.searchableColumns.splice(f,1);}
	},
	setFormatterFactory:function(factory){
		this.formatterFactory=factory;
		var i,formatter,column;
		for(i=0;i<this.columns.length;i++){
			column=this.columns[i];
			formatter=factory.getFormatter(column.getType().toUpperCase(),column.getColName());
			column.setFormatter(formatter);
		}
	},
	setVisibleColumns:function(columns){
		var i;
		for(i=0;i<this.visibleColumns.length;i++){
			this.visibleColumns[i].setVisible(false);
		}
		for(i=0;i<columns.length;i++){
			columns[i].setVisible(true);
		}
		this.visibleColumns=columns;
		this.visibleAndFixedColumns=null;//clear it out so it can be recalculated next time its needed
	},
	getSortedColumn:function(){
		return _.find(this.columns,function(col){return col.isColSorted();});
	},
	getSortedColumns:function(){
		return _.filter(this.columns,function(col){return col.isColSorted();});
	},
	setSortInfo:function(column,direction, columnMeta){
		if (column !== undefined) {
			//column.direction = direction;
			column.sortDirection = direction;
			if (columnMeta !== null && columnMeta !== undefined) {
				column.sortOrder = columnMeta.sortOrder;
			}
		}
	},
	// setSortInfo2(column,direction){
	// 	column.direction = direction;
	// },
	// setSortInfo3(column,direction){

	// },
	setSortedColumnsWithQuery: function(visibleColumns, sortInfo) {
		_.each(visibleColumns, function(column) {
			if (column) {
				column.direction = "None";
				column.setSortDirection("None");
				column.sortOrder = -1;
			}
		});
		_.each(visibleColumns, function(column) {
			var col=_.find(sortInfo,function(col){return col.sortName===column.name;});
			if(col) {
				//Not sure if we need to set column.direction as that is not used for sortDirection
				column.setSortDirection(col.sortDirection);
				column.sortOrder= sortInfo.indexOf(col);
			}
		});
	},
	setSortedColumns: function(sortedColumns) {
		var theColumns= {};
		_.each(sortedColumns, function(column) {
			theColumns[column.name] = column;
		});
		_.each(this.getSortedColumns(), function(sortedColumn) {
			var theColumn = theColumns[sortedColumn.name];
			if (theColumn === undefined) {
				sortedColumn.direction = "None";
				sortedColumn.setSortDirection("None");
				sortedColumn.sortOrder = -1;
			}
		});
	},
	setSortedColumn:function(column,direction, additionalSort){

		// Find the max value of the sortOrder
		var maxSortOrder=0;

		_.each(this.getSortedColumns(), function(sortedColumn){
			maxSortOrder=(sortedColumn.sortOrder > maxSortOrder) ?sortedColumn.sortOrder:maxSortOrder;
		});

		if (additionalSort === undefined || additionalSort === "none") {
			_.each(this.getSortedColumns(), function(sortedColumn){
				sortedColumn.setSortDirection("None");
				sortedColumn.sortOrder = -1;
			});
		} 

		if(column && direction){
			column.setSortDirection(direction);
			if (additionalSort === undefined || additionalSort === "none") {
				column.sortOrder = 0;
			} else if (additionalSort ==="add") {
				column.sortOrder = maxSortOrder + 1;
			} else if (additionalSort === "remove") {
				//Update the rest of the columns on this to reflect the removed column
				_.each(this.getSortedColumns(), function(sortedColumn){
					if (sortedColumn.sortOrder > column.sortOrder) {
						sortedColumn.sortOrder = sortedColumn.sortOrder -1;
					}
				});
				column.sortOrder = -1;
			} 
		}
	},
	getSortableColumns:function(){
		return _.filter(this.columns,function(col){return col.isColSortable();});
	},
	getVisibleColumnNames:function(){
		var columnMetas=this.getVisibleColumns();
		var i,visibleFields=[];
		for(i=0;i<columnMetas.length;i++){
			visibleFields.push(columnMetas[i].getColName());
		}
		return visibleFields;
	},
	getColumns:function(){
		return this.columns;
	},
	getColumnNames:function(){
		var columnMetas=this.getColumns();
		var i,cols=[];
		for(i=0;i<columnMetas.length;i++){
			cols.push(columnMetas[i].getColName());
		}
		return cols;
	},
	getColumnByName:function(name,caseInsensitive){
		return _.find(this.getColumns(),function(column){
			if(caseInsensitive){
				return column.getColName().toUpperCase()===name.toUpperCase();
			}
			return column.getColName()===name;
		});
	},
	getOrderedColumnsByNames:function(names){
		var allColumns=this.getColumns();
		var columns1=_.filter(allColumns,function(col){
			return names.indexOf(col.getColName())!==-1;
		});
		var sortedColumns=_.sortBy(columns1,function(col){
			return names.indexOf(col.getColName());
		});
		return sortedColumns;
	}
};
//STATICS
sas.risk.slimtable.meta.TableMeta.updateTableMetaJsonUserDisplayColumns=function(origMeta,userDisplayColNames){
	var origColumns=origMeta.columns;
	var i,resultColumns=[];
	for(i=0;i<userDisplayColNames.length;i++){
		var colRawName=userDisplayColNames[i];
		var col=_.find(origColumns,function(col){return col.rawName===colRawName;});
		origColumns.splice(origColumns.indexOf(col),1);
		col.userDisplayOrder=i;
		col.userVisible=true;
		col.visible=true;
		resultColumns.push(col);
	}
	var j;
	for(j=0;j<origColumns.length;j++){
		var col=origColumns[j];
		col.userDisplayOrder=-1;
		col.userVisible=false;
		col.visible=false;
		resultColumns.push(col);
	}
	origMeta.columns=resultColumns;
};