jQuery.sap.declare("sas.risk.slimtable.plugins.Filter");
jQuery.sap.require("sas.risk.slimtable.plugins.TablePlugin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.risk.slimtable.widget.Select");
jQuery.sap.require("sas.risk.slimtable.util.TableUtil");
jQuery.sap.require("sas.hc.ui.core.Item");

sas.risk.slimtable.plugins.TablePlugin.extend("sas.risk.slimtable.plugins.Filter",{
	metadata:{
      properties:{
         "createFilterForSelectHook":{type:"any"},
         "checkSpecialFilterHook":{type:"any"},
         "checkCurrentFilterValHook":{type:"any"}
      }
	}
});

sas.risk.slimtable.plugins.Filter.prototype.init=function(){
	this.setPosition(sas.risk.slimtable.TableLayout.TOP_CENTER);
	this.rb=sas.risk.slimtable.util.SlimTableUtil.getBundle();
	this.filters=[];
};

sas.risk.slimtable.plugins.Filter.prototype.getPluginLayout=function(){
   this.filtersLayout = new sas.risk.common.layout.flexlayout.FlexLayout({
   });
   return this.filtersLayout;
};
sas.risk.slimtable.plugins.Filter.prototype.createToolbarFilter=function(column){
   var filterLabel = new sap.m.Label({
   	text:this.rb.getText("table.toolbar.filter.label.fmt.txt",[column.getColLabel()])
   }).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
        flexShrink:0
    }));
   var name=column.getColName();
   var filter=new sas.risk.slimtable.widget.Select({
      name: name,
      change:$.proxy(this.table.actionFilterTable,this.table)
   });
   filter.getEnabled=function(val){//filter should be always enabled irrespective of whether table is readonly or not
   	return true;
   };
   filterLabel.setLabelFor(filter);
   
   this.filters.push(filter);
	var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
		//RGFRMWRK-830 alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Baseline,
		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
		items:[filterLabel,filter]
	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
		flexShrink:0
	}));
	this.filtersLayout.addItem(layout);
	
	return filter;
};
sas.risk.slimtable.plugins.Filter.prototype.getFilter=function(column){
	if(this.filters) {
		for(var i=0; i < this.filters.length; i++) {
			if(this.filters[i].getProperty("name") === column.name) {
				return this.filters[i];
			}
		}
	}
	return null;
};
sas.risk.slimtable.plugins.Filter.prototype.refreshToolbarFilterChoices=function(column, filter) {
	var i,choice;
   var choices=column.getColChoices() || [];
   if(!filter) {
   	filter=this.getFilter(column);
   }
   var existingItems = filter.getItems(); // Keep track of the old items before we refresh with new ones.
                                          // Doing filter.destroyItems() actually seems more intuitive and does work, but it has an undesirable side effect 
                                          // that a terrible flicker occurs on screen as the items are all removed and then re-populated. (Select width goes
                                          // to 0 when items are destroyed, and then it goes back up to the width needed after items are refreshed.)
   
   var selectedKey=filter.getSelectedItem()&&filter.getSelectedItem().getKey();
   if(column.useAllFilter){
		filter.addItem(new sas.hc.ui.core.Item({
			key:-1,
			text:this.rb.getText("table.toolbar.filter.all.txt"),
			customData:[{
            key:"payload",
            value:{
            	value:-1
            }
			}]
		}));
   }
   var name=column.getColName();
   for(var i=0;i<choices.length;i++){
   	choice=choices[i];
   	choice.value=(choice.value==="true")?true:(choice.value==="false")?false:choice.value;
   	var itemToAdd=new sas.hc.ui.core.Item({
   	   key:choice.value,
   	   text:choice.label,
   	   customData:[{
   	   	key:"payload",
   	   	value:{
	   	   	value:choice.value,
	   	   	colMeta:column
   	   	}
   	   }]
   	});
   	filter.addItem(itemToAdd);
   	if(choice.value===selectedKey){
   		filter.setSelectedItem(itemToAdd);
   	}
   	var checkSpecialFilterHook=this.getCheckSpecialFilterHook();
   	if(checkSpecialFilterHook && $.isFunction(checkSpecialFilterHook)){
   		checkSpecialFilterHook(name, choice, filter);
   	}
   }
   
   // Now remove the old items, since they have been replaced/refreshed with new ones. See
   // comment above about screen flicker and why this is needed.
   if(existingItems) {
	   for(var i=0;i<existingItems.length;i++) {
	   	filter.removeItem(existingItems[i]);
	   }
	   filter._oSelectionOnFocus=filter.getSelectedItem();
   }
};

sas.risk.slimtable.plugins.Filter.prototype.refresh=function(query){
	var query=query || this.table.query;
   //FILTERS
	var filterableColumns=this.table.getSource().getTableMeta().getFilterableColumns();
	for(var i=0;i<filterableColumns.length;i++){
		var filter = this.getFilter(filterableColumns[i]);
		if(!filter) {
			filter = this.createToolbarFilter(filterableColumns[i]);
		}
		this.refreshToolbarFilterChoices(filterableColumns[i], filter);
	}
   
   var j,filter,filter,filterMeta,currentFilterVal;
   for(j=0; j<this.filters.length; j++){
      filter=this.filters[j];
      filters=query.getFilterByName(filter.getName(),"SIMPLE");
      filterMeta=_.find(filters||[],function(f){return f.getOperator()==="eq";});
      filter.setSelectedKey(-1);
      if(filterMeta){
	      currentFilterVal=filterMeta.getValue();
	      if(typeof currentFilterVal==="boolean")currentFilterVal=currentFilterVal.toString();
	      var checkCurrentFilterValHook=this.getCheckCurrentFilterValHook();
	      if(checkCurrentFilterValHook && $.isFunction(checkCurrentFilterValHook)){
	      	currentFilterVal = checkCurrentFilterValHook(currentFilterVal);
	      }
	      filter.setSelectedKey(currentFilterVal||-1);
      }
   }
};

sas.risk.slimtable.plugins.Filter.prototype.updateQuery = function(query){
	var i,f,selectedItem,payload;
	for(i=0;i<this.filters.length;i++){
		f=this.filters[i];
		selectedItem=f.getSelectedItem();
		if(selectedItem){
			payload=sas.risk.slimtable.util.TableUtil.findCustomData(selectedItem,"payload");
			if(payload&&payload.value!==-1){
				var createFilterForSelectHook=this.getCreateFilterForSelectHook();
				var selectFilter=(createFilterForSelectHook && $.isFunction(createFilterForSelectHook)) 
									? createFilterForSelectHook(f)
									: sas.risk.slimtable.util.TableUtil.createFilterForSelect(f);
				query.addQueryFilter(selectFilter,"SIMPLE",true);
			}
		}
	}
};
sas.risk.slimtable.plugins.Filter.prototype.getFilters=function(){
	return this.filters;
};