jQuery.sap.declare("sas.risk.slimtable.util.search.TimestampFilterHandler");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.hc.m.DatePicker");
jQuery.sap.require("sas.ui.cpb3.model.type.CpbDate");
jQuery.sap.require("sas.risk.slimtable.util.SlimTableUtil");
jQuery.sap.require("sas.risk.slimtable.meta.filter.BooleanFilter");
jQuery.sap.require("sas.ui.cpb3.model.format.CpbDateFormat");

sas.risk.slimtable.util.search.TimestampFilterHandler=function(colMeta){
	this.colMeta=colMeta;
};
sas.risk.slimtable.util.search.TimestampFilterHandler.prototype={
	setOperator:function(filter,operator){
		filter.setOperator(operator);
	},
	getOperators:function(){
		var rb = sas.risk.slimtable.util.SlimTableUtil.getBundle();
		return [
			{label:rb.getText("table.dialog.advsearch.operator.gt.txt"),key:"gt"},
			{label:rb.getText("table.dialog.advsearch.operator.lt.txt"),key:"lt"},
			{label:rb.getText("table.dialog.advsearch.operator.ge.txt"),key:"ge"},
			{label:rb.getText("table.dialog.advsearch.operator.le.txt"),key:"le"}
		];
	},
	getInput:function(filter){
		var that= this;
		filter.setType(sas.risk.slimtable.meta.filter.TimestampFilter);
		var value=filter.getValue();
		var operator=filter.getOperator();
		if(value===undefined||value===null){
			//value=sas.risk.slimtable.util.SlimTableUtil.today();
			value=that.adjustedDatetimeValue(operator,null);
		}		
		var picker=new sas.hc.m.DatePicker({
			dateValue : value,
			valueFormat:"yyyy-MM-dd",
			change:function(e){
				var value=e.getParameter("newValue");
				var operator=filter.getOperator();
				//var value=new Date(e.getParameter("newValue")+"T12:00:00.000Z");//noon-shift it
				value= that.adjustedDatetimeValue(operator,value,true);							
				if(isNaN(value.getTime())){
					filter.setValue(null);
					filter.setValueLabel(null);
				}else{
					filter.setValue(value);
					filter.setValueLabel(that.toLocaleDateString(value,true));
				}
			}
		}).addStyleClass("riskTabAdvDatePicker");
		if (_.isDate(picker.getDateValue())){
			var value = that.adjustedDatetimeValue(operator,picker.getDateValue(),false);
			filter.setValue(value);
			filter.setValueLabel(that.toLocaleDateString(value,true));
		}
		return picker;
	},
	updateFilterValueLabel:function(filter){
		var def=$.Deferred().resolve();
		var value=filter.getValue();
		var operator=filter.getOperator();
		if(value===undefined||value===null){
			//value=sas.risk.slimtable.util.SlimTableUtil.today();
			value=that.adjustedDatetimeValue(operator,null);
		}else if (_.isDate(value)){
			value=this.adjustedDatetimeValue(operator,value,false);
		}
		filter.setValueLabel(this.toLocaleDateString(value,true));
		return def;
	},
	toLocaleDateString:function(dateObject,bUTC){
		var locale=sas.risk.slimtable.util.SlimTableUtil.getLocale();
   	var inputFormat = sas.ui.cpb3.model.format.CpbDateFormat.getInstance({style: "medium"}, locale);
   	return inputFormat.format(dateObject,bUTC);
	},
	adjustedDatetimeValue:function(type,value,isISO){	
		var parts;
		if (value!==null&& isISO){
				parts = value.split('-');
		}
   	switch  (type){
   		case 'gt':
   		case 'le':
   			if (value&&isISO){
   				return new Date(Date.UTC(parts[0], parts[1]-1,parts[2], 23, 59, 59, 999));
   			}else if (value && !isISO)
   				return new Date(Date.UTC(value.getFullYear(),value.getMonth(),value.getDate(),23,59,59,999));
   			else{
   				var now = new Date();
   				return new Date(Date.UTC(now.getFullYear(),now.getMonth(),now.getDate(),23,59,59,999));
   			}   				
   			break;
   		case 'ge':
   		case 'lt':
   			if (value&&isISO){
   				return new Date(Date.UTC(parts[0], parts[1]-1,parts[2], 0, 0, 0, 0));
   			}else if (value && !isISO)
   				return new Date(Date.UTC(value.getFullYear(),value.getMonth(),value.getDate(),0,0,0,0));
   			else{
   				var now = new Date();
   				return new Date(Date.UTC(now.getFullYear(),now.getMonth(),now.getDate(),0,0,0,0));
   			}  
   			break;  			
   	}
   }
};