jQuery.sap.declare("sas.risk.slimtable.widget.AdvFilterRow");
jQuery.sap.require("sap.ui.commons.layout.MatrixLayoutRow");
jQuery.sap.require("sas.risk.slimtable.util.SlimTableUtil");
jQuery.sap.require("sas.risk.slimtable.util.search.FilterHandlerFactory");
jQuery.sap.require("sas.risk.slimtable.data.AdvSearch");
jQuery.sap.require("sas.risk.slimtable.data.AdvSearchFilterMeta");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.slimtable.meta.filter.ObjectLinkFilter");


sap.ui.commons.layout.MatrixLayoutRow.extend("sas.risk.slimtable.widget.AdvFilterRow",{
	metadata:{
		properties:{
			isAnd:{type:"boolean"},
			columns:{type:"any"},
			filterMeta:{type:"any"},
			isFirst:{type:"boolean",defaultValue:true},
			isLast:{type:"boolean",defaultValue:true}
		}
	}
});
sas.risk.slimtable.widget.AdvFilterRow.prototype.init=function(){
	this.colSelect=new sas.hc.m.Select({change:$.proxy(this.actionSelectColumn,this)});
	this.input=null;
	this.andOr=null;
	this.addButton=new sas.risk.common.widgets.Button({
		icon:sas.icons.HC.ADD,
		press:$.proxy(this.actionAddFilter,this)
	}).addStyleClass("riskTabAdvButton");
	this.delButton=new sas.risk.common.widgets.Button({
		icon:sas.icons.HC.DELETE,
		press:$.proxy(this.actionDelFilter,this)
	}).addStyleClass("riskTabAdvButton");
};
//ACTIONS
sas.risk.slimtable.widget.AdvFilterRow.prototype.actionSelectAndOr=function(){
	var selected=this.andOr.getSelectedKey();
	var mgr=sas.risk.slimtable.data.AdvSearch.getInstance(this);
	mgr.setIsAnd(selected==="AND");
	this.refresh();
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.actionSelectColumn=function(){
	var selectedItem=this.colSelect.getSelectedItem();
	var selectedColumn=selectedItem.getKey();
	var parse = selectedColumn.split('.', 2);

	var filterMeta=this.getFilterMeta();
	
	filterMeta.setName(selectedColumn);
	filterMeta.setValue(null);
	filterMeta.setOperator(null);
	filterMeta.setNameLabel(!!selectedColumn?selectedItem.getText():null);
	if(parse[0] === "linkType") {
		filterMeta.setType(sas.risk.slimtable.meta.filter.ObjectLinkFilter);
	}
	filterMeta.setValueLabel(null);
	this.refresh();
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.actionAddFilter=function(){
	var mgr=sas.risk.slimtable.data.AdvSearch.getInstance(this);
	mgr.addAdvSearchFilter(new sas.risk.slimtable.data.AdvSearchFilterMeta());
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.actionDelFilter=function(){
	var mgr=sas.risk.slimtable.data.AdvSearch.getInstance(this);
	mgr.delAdvSearchFilter(this.getFilterMeta());
};
//REFRESH
sas.risk.slimtable.widget.AdvFilterRow.prototype.refresh=function(){
	var filterMeta=this.getFilterMeta();
	var columns=this.getColumns();
	var isAnd=this.getIsAnd();
	if(filterMeta && columns && (isAnd!==null)){
		var index=this.getParent().getRows().indexOf(this);
		this.setIsFirst(index===0);
		this.setIsLast(this.getParent().getRows().length-1===index);
		this.refreshColumns();
		this.refreshOperators();
		this.refreshInput();
		this.refreshAndOr();
		this.refreshButtons();
		this.removeAllCells();
		this.addCell(new sap.ui.commons.layout.MatrixLayoutCell({content:[this.colSelect]}));
		this.addCell(new sap.ui.commons.layout.MatrixLayoutCell({content:[this.operSelect]}));
		this.addCell(new sap.ui.commons.layout.MatrixLayoutCell({content:[this.input]}).addStyleClass("riskTabAdvInputCol"));
		this.addCell(new sap.ui.commons.layout.MatrixLayoutCell({content:[this.andOr]}));
		this.addCell(new sap.ui.commons.layout.MatrixLayoutCell({content:[this.addButton]}));
		this.addCell(new sap.ui.commons.layout.MatrixLayoutCell({content:[this.delButton]}));
	}
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.refreshColumns=function(){
	var selectedColumn=this.getFilterMeta().getName();
	if(!this.colSelect.getItems().length){
		var i,col;
		var rb = sas.risk.slimtable.util.SlimTableUtil.getBundle();
		if(!selectedColumn){
			this.colSelect.addItem(new sas.risk.slimtable.widget.Item({
				key:null,
				text:rb.getText("table.dialog.advsearch.colselect.placeholder.txt")
			}));
		}
		var columns=this.getColumns();
		for(i=0;i<columns.length;i++){
			col=columns[i];
			this.colSelect.addItem(new sas.risk.slimtable.widget.Item({
				key:col.getColName(),
				text:col.getColLabel()
			}));
			if(selectedColumn===col.getColName()){
				this.getFilterMeta().setNameLabel(col.getColLabel());
			}
		}
		if(selectedColumn){
			this.colSelect.setSelectedKey(selectedColumn);
		}
	}
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.refreshOperators=function(){
	var mgr=sas.risk.slimtable.data.AdvSearch.getInstance(this);
	var selectedColumn=this.getFilterMeta().getName();
	var colMeta=_.find(this.getColumns(),function(c){return selectedColumn===c.getColName();});
	if(colMeta){
		var that=this;
		var handler=sas.risk.slimtable.util.search.FilterHandlerFactory.getInstance().getHandler(colMeta,mgr);
		this.operSelect=new sas.hc.m.Select({
			change:function(e){
				var colType = handler.colMeta.getType();
				handler.setOperator(that.getFilterMeta(),e.getParameter("selectedItem").getKey());
				if (colType === "java.sql.Timestamp"){
					that.refreshInput();
				}
			}
		});
		var operators=handler.getOperators();
		var i,op;
		for(i=0;i<operators.length;i++){
			op=operators[i];
			this.operSelect.addItem(new sas.risk.slimtable.widget.Item({
				key:op.key,
				text:op.label
			}));
		}
		var selectedOperator=this.getFilterMeta().getOperator();
		if(!selectedOperator){
			selectedOperator=this.operSelect.getItemAt(0).getKey();
			this.getFilterMeta().setOperator(selectedOperator);
		}
		this.operSelect.setSelectedKey(selectedOperator);
	}else{
		var rb = sas.risk.slimtable.util.SlimTableUtil.getBundle();
		this.operSelect=new sas.hc.m.Select({
			items:[
		      new sas.risk.slimtable.widget.Item({
					key:null,
					text:rb.getText("table.dialog.advsearch.operselect.placeholder.txt")
				})
			],
			enabled:false
		});
	}
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.refreshInput=function(){
	var mgr=sas.risk.slimtable.data.AdvSearch.getInstance(this);
	var selectedColumn=this.getFilterMeta().getName();
	var colMeta=_.find(this.getColumns(),function(c){return selectedColumn===c.getColName();});
	if(colMeta){
		var handler=sas.risk.slimtable.util.search.FilterHandlerFactory.getInstance().getHandler(colMeta,mgr);
		this.input=handler.getInput(this.getFilterMeta());
	}else{
		var rb = sas.risk.slimtable.util.SlimTableUtil.getBundle();
		this.input=new sas.hc.m.Input({
			value : rb.getText("table.dialog.advsearch.value.placeholder.txt"),
			enabled:false
		});
	}
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.refreshAndOr=function(){
	var rb = sas.risk.slimtable.util.SlimTableUtil.getBundle();
	var isAnd=this.getIsAnd();
	if(this.getIsLast()){
		this.andOr=new sas.hc.m.Label({text:""});
	}else if(this.getIsFirst()){
		this.andOr=new sas.hc.m.Select({
			items:[
		      new sas.risk.slimtable.widget.Item({
					key:"AND",
					text:rb.getText("table.dialog.advsearch.and.txt")
				}),
		      new sas.risk.slimtable.widget.Item({
					key:"OR",
					text:rb.getText("table.dialog.advsearch.or.txt")
				})
			],
			selectedKey:isAnd?"AND":"OR",
			change:$.proxy(this.actionSelectAndOr,this)
		});
	}else{
		this.andOr=new sas.hc.m.Label({text:isAnd?rb.getText("table.dialog.advsearch.and.txt"):rb.getText("table.dialog.advsearch.or.txt")});
	}
};
sas.risk.slimtable.widget.AdvFilterRow.prototype.refreshButtons=function(){
	if(this.getIsFirst() && this.getIsLast()){
		this.addButton.setVisible(true);
		this.delButton.setVisible(false);
	} else if(this.getIsFirst()) {
		this.addButton.setVisible(false);
		this.delButton.setVisible(true);
	} else if(this.getIsLast()) {
		this.addButton.setVisible(true);
		this.delButton.setVisible(true);
	} else {
		this.addButton.setVisible(false);
		this.delButton.setVisible(true);
	}
	this.addButton.setEnabled(!!this.getFilterMeta().getName());
};
//OVERRIDES
sas.risk.slimtable.widget.AdvFilterRow.prototype.clone=function(){
	return sap.ui.commons.layout.MatrixLayoutRow.prototype.clone.apply(this,[arguments[0],null,{cloneBindings:true}]);
};
