jQuery.sap.declare("sas.ui.cpb3.CPBForm");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sap.ui.model.json.JSONModel");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.ui.cpb3.CPBService");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.CPBForm", {
	metadata:{
		properties:{
			"cpbParent":{type:"any"},
			"serviceId":{type:"string"},
			"screenId":{type:"string"},
			"data":{type:"any"},
			"cpbModelManager":{type:"any"}
		},
		events:{
			modeled:{},
			onChangeFinished:{},
			mashup:{},
			checkingRequiredFields:{}
		}
	},
   constructor:function(){
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
	 },
	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});
sas.ui.cpb3.CPBForm.prototype.init=function(){
	this.setFlexDirection(sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column);
   this.addStyleClass("cpbCpbForm");
   this._registryByName={};
   this._registryById={};
   this.built=false;
};
sas.ui.cpb3.CPBForm.prototype.applySettings=function(){
	sas.risk.common.layout.flexlayout.FlexLayout.prototype.applySettings.apply(this,arguments);
	this.cpbService=sas.ui.cpb3.CPBService.getInstance(this.getServiceId());
	this.cpbAPI=this.cpbService.createObjectAPI(this);
};
sas.ui.cpb3.CPBForm.prototype.setCpbParent=function(cpbParent){
	this.setProperty("cpbParent",cpbParent);
	cpbParent.attachCheckingRequiredFields($.proxy(this.updateUIContextWithRequiredFields,this,this));
};
sas.ui.cpb3.CPBForm.prototype.getCpbAPI=function(){
	return this.cpbAPI;
};
sas.ui.cpb3.CPBForm.prototype.getQueryAPI=function(){
	return this.cpbService.getQueryAPI();
};
sas.ui.cpb3.CPBForm.prototype.getCpbService=function(){
	return this.cpbService;
};
sas.ui.cpb3.CPBForm.prototype.getRequiredFieldIds=function(context){
	var requiredControls = _.filter(_.values(this._registryById),function(control){return control.getCpbRequired() && _.isNumber(control.getCpbId())});
	return _.map(requiredControls, function(control){return control.getCpbId()});
};
sas.ui.cpb3.CPBForm.prototype.updateUIContextWithRequiredFields=function(context){
	var requiredControlIds = this.getRequiredFieldIds(context);
	this.getCpbModelManager().setRequiredFields(requiredControlIds,context);
	this.fireCheckingRequiredFields();
};
sas.ui.cpb3.CPBForm.prototype.setData=function(data){
	if(this.building){return;}
	data["BLUFF"]=new Date().getTime(); //This is to force the property update
	// data["INCLDUESTR"]="testing...";  not proceeding on with RGFRMWRK-489 story
	this.setProperty("data",data);
	var miscModel=this.getModel("cpb_misc");
	if(miscModel){
		var rebuild=(miscModel.getProperty("/rebuild"));
		if(rebuild){
			this.built=false;
			miscModel.setProperty("/rebuild",false);
			this.destroyItems();
		}
	}
	if(this.built){return;}
	this.loadObservers(this.getModel("cpb_observers").getData());
	var structure=this.getModel("cpb_structure").getData();
	//this.loadInclAndExclStr(this.getModel("cpb_structure").getData());  not proceeding on with RGFRMWRK-489 story
	var modelId=this.getBindingInfo("data").parts[0].model;
	var that=this;
	this._preprocessStructure(structure);
	this.building=true;
	this.buildStructure(that,structure,modelId).done(function(){
		that.building=false;
		that.built=true;
		that.fireModeled();
		that.setBusy(false);
	});
};
sas.ui.cpb3.CPBForm.prototype.getObservers=function(name){
	return this.nameToObservers[name&&name.toUpperCase()]||[];
};
sas.ui.cpb3.CPBForm.prototype.loadObservers=function(observers,nameToObservers){
	this.nameToObservers=nameToObservers||{};
	var that=this;
	_.each(observers,function(observer){
		_.each(observer.names,function(name){
			name=name.toUpperCase();
			var obsArray=that.nameToObservers[name]||[];
			obsArray.push(observer);
			that.nameToObservers[name]=obsArray;
		});
	});
};
sas.ui.cpb3.CPBForm.prototype.getInclStr=function(){
	return this.includeStr;
};
sas.ui.cpb3.CPBForm.prototype.getExclStr=function(){
	return this.excludeStr;
};
sas.ui.cpb3.CPBForm.prototype.loadInclAndExclStr=function(structureList){
	this.includeStr={};
	this.excludeStr={};
	var that=this;
	_.each(structureList,function(structItem){
		if(structItem.cpbIncludeStr) {
			that.includeStr={fieldName:structItem.cpbName,includeStr:structItem.cpbIncludeStr};
		}
		if(structItem.cpbExcludeStr) {
			that.excludeStr={fieldName:structItem.cpbName, excludeStr:structItem.cpbExcludeStr};
		}
	});
};
sas.ui.cpb3.CPBForm.prototype._preprocessStructure=function(structure){
	this._preprocessSubScreens(structure);
	this._preprocessMatrix(structure);
};
sas.ui.cpb3.CPBForm.prototype._preprocessSubScreens=function(structure){
	var hasSubScreens=false;
	var addProps=function(props,name,context){
		var model=context.getModel(name)
		model.setData(props,true);
	};
	var that=this;
	var process=function(payload,current){
		var children=current.cols||current.tabs||current.children;
		if(children){
			_.each(children.slice(0),$.proxy(process,this,{orig:children}));//clone the children so we can delete on real one and keep for-loop going
		}else{
			if(current.type==="SubScreen" && current.meta){
				hasSubScreens=true;
				var meta=current.meta;
				that.loadObservers(meta.observers,that.nameToObservers);
				addProps(meta.labels,"cpb_labels",this);
				addProps(meta.expressions,"cpb_exps",this);
				var args=[payload.orig.indexOf(current),1].concat(meta.structure);
				Array.prototype.splice.apply(payload.orig,args);//remove current one from its original location, replace with sub screen structure
			}
		}
	};
	do{
		hasSubScreens=false;
		_.each(structure.slice(0),$.proxy(process,this,{orig:structure}));
	}while(hasSubScreens);//allow for processing of nested subscreens
};
sas.ui.cpb3.CPBForm.prototype._preprocessMatrix=function(structure){
	var process=function(payload,current){
		var children=current.cols||current.tabs||current.children;
		if(children){
			payload.matrix=null;//processing a widget that doesnt participate in matrix layout, so lets clear the flag so that siblings that come later dont get added to matrix before this child (no skip-in-line)
			if(current.type!=="matrix"){//once is a matrix,than it has been preprocessed already
				_.each(children.slice(0),$.proxy(process,this,{parent:current,orig:children}));//clone the children so we can delete on real one and keep for-loop going
			}
		}else{
			if(payload.matrix){//we are already in matrix mode
				var total=payload.matrix.children.push(current);
				if(total===2){//achieved critical mass for a matrix layout. swap out first child for the matrix layout
					var origIndex=payload.orig.indexOf(payload.matrix.children[0]);
					payload.orig.splice(origIndex,1,payload.matrix);
				}
				var index=payload.orig.indexOf(current);
				payload.orig.splice(index,1);//remove current one from its original location, since it has now moved to the matrix layout
			}else if(!payload.parent || (payload.parent.type!=="gridLayout" && payload.parent.type!=="flexLayout")){//we are not in matrix mode, but are eligible to switch to matrix mode
				payload.matrix={type:"matrix",children:[]};
				payload.matrix.children.push(current);//add current as first child of this new matrix layout
			}
		}
	};
	_.each(structure.slice(0),$.proxy(process,this,{orig:structure}));
};
sas.ui.cpb3.CPBForm.prototype.buildStructure=function(parentWidget,children,modelId,def){
	var that=this;
	def=def?def:$.Deferred();
	var defs=[];
	_.each(children,function(child){
		defs.push(that._buildStructure(child,modelId));
	});
	$.when.apply($,defs).done(function(){
		var children=arguments?Array.prototype.slice.call(arguments):[];
		_.each(children,function(child){
			parentWidget.addItem(child);
		});
		def.resolve(parentWidget);
	});
	return def;
};
sas.ui.cpb3.CPBForm.prototype._buildStructure=function(currentJson,modelId){
	var def=$.Deferred();
	var that=this;
	var widgetClass=this.cpbService.getWidgetRegistry().getWidgetClass(currentJson.type);
	var createdWidget=widgetClass.create(currentJson,modelId,this);
	if(!createdWidget){//unregistered widget type returns null
		def.resolve(createdWidget);
	}else{
		$.when(createdWidget).then(function(currentWidget){
			if(currentWidget.getCpbName && currentWidget.getCpbName()){//register widget by name
				var name=currentWidget.getCpbName();
				currentWidget.addStyleClass("CPB_"+name.toUpperCase());
				currentWidget.addStyleClass&&currentWidget.addStyleClass("cpbName_"+name.toUpperCase());
				that._registryByName[name.toUpperCase()]=currentWidget;
			}
			else if(currentWidget.getCpbKey && currentWidget.getCpbKey()){//register widget by key (for layouts)
				var key=currentWidget.getCpbKey();
				currentWidget.addStyleClass&&currentWidget.addStyleClass("cpbKey_"+key.toUpperCase());
				that._registryByName[key.toUpperCase()]=currentWidget;
			}
			that._registryById[currentWidget.getCpbId()]=currentWidget;
			that.buildStructure(currentWidget,currentJson.children,modelId,def);
		});
	}
	return def;
};
sas.ui.cpb3.CPBForm.prototype.getView=function() {
	return this.getParent().getParent();
};
sas.ui.cpb3.CPBForm.prototype.getRootCPBForm=function(){
	var parent=this.getCpbParent();
	if(!parent)return this;
	return parent.getRootCPBForm();
};
sas.ui.cpb3.CPBForm.prototype.getCPBControl=function(name){
	return this._registryByName[name.toUpperCase()];
};
sas.ui.cpb3.CPBForm.prototype.getCPBControlById=function(cpbId){
	return this._registryById[parseInt(cpbId)];
};
sas.ui.cpb3.CPBForm.prototype.onStorageEvent=function(e){
	console.debug("STORAGE EVENT",e,arguments);
};
