jQuery.sap.declare("sas.ui.cpb3.CpbAPI");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");

sas.ui.cpb3.CpbAPI=function(){};
sas.ui.cpb3.CpbAPI.prototype={
	setCpbService:function(cpbService){
		this.cpbService=cpbService;
	},
	getCurrentServerTime:function(){
		var def=$.Deferred();
		var appId=this.cpbService.getServiceId();
		var url="rest/cpb/"+appId+"/currentTime";
		$.getJSON(url,function(json){
			def.resolve(json);
		});
		return def
	},
	evalExpression : function(expression,data,screenDef, cachedResult, GZIPCompressionThreshold) {
	   var def=$.Deferred();
	   var appId=this.cpbService.getServiceId();
	   var that=this;
	   var values={data:data};
	   if(screenDef){
	   	values.screenDef=screenDef;
	   }

	   if (!cachedResult) {
	   	var json = JSON.stringify(values);
	   	if (json.length > GZIPCompressionThreshold && (GZIPCompressionThreshold !== -1))
	   	{
	   	   	var compressed = pako.gzip(json);
		    var blob     = new Blob([ compressed ], { type: 'application/octet-stream'})
	   	    //alert("compressed gzip blob - " + blob);
			$.ajax({
			   	headers: {
					Accept: "application/json; charset=UTF-8",
					"Content-Type": "application/gzip"
				},
			    type: "POST",
			    url:"rest/cpb/"+appId+"/eval?"+$.param({expr:expression}),
			    dataType:"json",
			    data:blob,
				processData:false
			}).done(function(data, textStatus, jqXHR) {
			  def.resolve(data);
			}).fail(function(e,jqxhr) {
			   	var rb=that.cpbService.getBundle();
			    sas.ui.cpb3.CpbUtil.handleFail(e,jqxhr,rb);
			    def.reject();
			});
	   	}
	   	else
	   	{
			$.ajax({
			   	headers: {
					Accept: "application/json; charset=UTF-8",
					"Content-Type": "application/json; charset=UTF-8"
				},
			    type: "POST",
			    url:"rest/cpb/"+appId+"/eval?"+$.param({expr:expression}),
			    dataType:"json",
			    data:json
			}).done(function(data, textStatus, jqXHR) {
			    def.resolve(data);
			}).fail(function(e,jqxhr) {
			   	var rb=that.cpbService.getBundle();
			    sas.ui.cpb3.CpbUtil.handleFail(e,jqxhr,rb);
			    def.reject();
			});
	   	}
	   } else {
	   		def.resolve(cachedResult);
	   }
	   return def;
	},
	/**
	 * Evaluates a series of expressions
	 * The evals parameter is an array of objects having a name and expr attribute
	 * eg: [{name:"a",expr:"1+1+3"},{name:"b",expr:"2+2+a"}]
	 * The evals are evaluated serially, and the results of one eval is stored in the context 
	 * using the variable name given.
	 * The results is a json object that has a result containing the variables evaluated and their values,
	 * and a data containing the final ui context
	 */
	multieval : function(evals,data,screenDef, GZIPCompressionThreshold, multiEvalPayloadDebugging) {
	   var def=$.Deferred();
	   var appId=this.cpbService.getServiceId();
	   var that=this;
	
	   data["multiEvalDebuggingFlag"]=multiEvalPayloadDebugging;
	   var values={data:data};
	   if(screenDef){
	   	values.screenDef=screenDef;
	   }
        
	   var json = JSON.stringify({evals:evals,context:values});
	   if (json.length > GZIPCompressionThreshold && (GZIPCompressionThreshold !== -1))
	   {
			var compressed = pako.gzip(json);
			var blob     = new Blob([ compressed ], { type: 'application/octet-stream'})
	   	   	//alert("compressed gzip blob - " + blob);
			$.ajax({
			   	headers: {
						Accept: "application/json; charset=UTF-8",
						"Content-Type": "application/gzip"
						},
			    type: "POST",
			    url:"rest/cpb/"+appId+"/multieval",
			    dataType:"json",
			    data:blob,
				processData:false
			}).done(function(data, textStatus, jqXHR) {
			    //def.resolve(data);
				var resultsArr = data.results;
				//var filterdata= _.map(data, function(value){return value === null? undefined: value});
				var filterdata= _.map(resultsArr, function(value){return value === null? undefined: value});
	      		def.resolve(filterdata);
			}).fail(function(e,jqxhr) {
			   	var rb=that.cpbService.getBundle();
			    sas.ui.cpb3.CpbUtil.handleFail(e,jqxhr,rb);
			    def.reject();
			});
	   	}
		else {
	 		$.ajax({
	   			headers: {
					Accept: "application/json; charset=UTF-8",
					"Content-Type": "application/json; charset=UTF-8"
				},
	      		type: "POST",
	      		url:"rest/cpb/"+appId+"/multieval",
	      		dataType:"json",
	      		data:JSON.stringify({evals:evals,context:values})
	   		}).done(function(data, textStatus, jqXHR) {
				var resultsArr = data.results;
				//var filterdata= _.map(data, function(value){return value === null? undefined: value});
				var filterdata= _.map(resultsArr, function(value){return value === null? undefined: value});
	      		def.resolve(filterdata);
	   		}).fail(function(e,jqxhr) {
	   	 		var rb=that.cpbService.getBundle();
	       		sas.ui.cpb3.CpbUtil.handleFail(e,jqxhr,rb);
	       		def.reject();
	   		});
		}
	   return def;
	},
	loadJSLibraries : function() {
		var def=$.Deferred();
		var appId=this.cpbService.getServiceId();
		var url="rest/cpb/"+appId+"/loadJSLibs";
		$.getJSON(url,function(json){
			def.resolve(json);
		});
		return def;
	}
};