jQuery.sap.declare("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.ui.cpb3.CpbAPI");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.ui.cpb3.model.CpbModelManagerObject");

sas.ui.cpb3.CpbUtil={
	findCpbId:function(control){
		if(!control){return null;}
		if(control.getCpbId){
			return control.getCpbId();
		}
		return sas.ui.cpb3.CpbUtil.findCpbId(control.getParent());
	},
   findParent:function(child,parentClass){
   	for(var c=child;!!c.getParent();c=c.getParent()){
   		if(c.getParent() instanceof parentClass){
   			return c.getParent();
   		}
   	}
   	return null;
   },
   createMessagesPanel:function(){
   	return new sas.risk.common.layout.flexlayout.FlexLayout({
   		width : "100%",
   		flexDirection : sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
   		items : {
   			path : sas.ui.cpb3.model.CpbModelManagerObject.bindMessagePath('/messages'),
   			factory : function(sId, context) {
   				var message=context.getObject();
   				var messageBar = new sas.risk.common.layout.flexlayout.FlexLayout({
   					alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
   					items : [
   					   sas.icons.pool.createControlByURI(sas.icons.HC[message.status==="SUCCESS"?"COMPLETESTATUS":"ERRORSTATUS"]),
   			         new sas.hc.m.Label({
   							text : sas.ui.cpb3.model.CpbModelManagerObject.bindMessage("message")
   						}).addStyleClass("cpbMessage")
   					]
   				}).addStyleClass("cpbMessageBar");
   				switch(message.status){
   					case "SUCCESS":
   						messageBar.addStyleClass("cpbSuccessMessageBar");
   					break;
   					case "ERROR":
   					default:
   						messageBar.addStyleClass("cpbErrorMessageBar");
   					break;
   				}
   				return messageBar;
   			}
   		}
   	});
   },
   buildFormattingKey:function(cpbForm,klassName,fieldName,baseKeys){
   	var formattingKey=$.extend([],baseKeys||[]);
   	if(cpbForm){
	   	var findIt=function(current,target){
	   		if(!current){return null;}
				if(current instanceof target){return current;}
	   		return findIt(current.getParent(),target);
	   	};
	   	var view=findIt(cpbForm,sap.ui.core.mvc.JSView);
	   	if(view){//editable matrix rows have no views
				if(view.getViewType){
					formattingKey.push(view.getViewType());
				}
				var viewId=view.getId();
				if(viewId && viewId.indexOf("__")!==0)
					formattingKey.push(view.getId());
	   	}
			formattingKey.push(cpbForm.getScreenId());
   	}
		if(klassName){formattingKey.push(klassName);}
		if(fieldName){formattingKey.push(fieldName);}
   	return formattingKey;
   },
   buildDialogFormattingKey:function(viewId,cpbForm,klassName,fieldName,baseKeys){
   	var formattingKey=sas.ui.cpb3.CpbUtil.buildFormattingKey(cpbForm,klassName,fieldName,baseKeys);
   	formattingKey.push("Dialog");
   	formattingKey.push(viewId);
   	return formattingKey;
   },
	_locale:null,
	getLocale:function(){
		return sas.ui.cpb3.CpbUtil._locale;
	},
	setLocale:function(locale){
		sas.ui.cpb3.CpbUtil._locale=locale;
	},
	_stringifyFilter:function(key,val){
		return (val===null)?undefined:val;
	},
	stringify:function(obj){
		return JSON.stringify(obj,sas.ui.cpb3.CpbUtil._stringifyFilter);
	},
	handleFail:function(e,jqxhr,rb){
		console.error(arguments);
		var json=e.responseJSON||(jqxhr&&jqxhr.responseJSON);
		if(!json){
			if(jqxhr&&jqxhr.status===404){
				json={message:rb.getText("message.error.404.txt")};
			}else{
				json={message:rb.getText("message.error.500.txt")};
			}
		}
		jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");
    	sas.risk.common.widgets.MessageDetailsBox.error(json, sas.config.appName, rb);	
   },
   _session:{},
   sessionSet:function(key,value){
   	sas.ui.cpb3.CpbUtil._session[key]=value;
   },
   sessionGet:function(key,remove){
   	var value=sas.ui.cpb3.CpbUtil._session[key];
   	if(remove){
   		delete sas.ui.cpb3.CpbUtil._session[key];
   	}
   	return value;
   },
   sessionClear:function(keyRoot){
   	sas.ui.cpb3.CpbUtil._session=_.pick(sas.ui.cpb3.CpbUtil._session,function(value,key,object){
   		return key.substr(0,keyRoot.length).toUpperCase() !== keyRoot.toUpperCase();
   	});
   },
   silentUpdateHash:function(newHash){
      var orig=hasher.changed.dispatch;
   	hasher.changed.dispatch=function(){
   		hasher.changed.dispatch=orig;
		};
   	top.location.hash=newHash;
   },
   _eventQueues:{},
   _storageEventListener:function(event){
   	var eventId=event.key;
   	var queue=sas.ui.cpb3.CpbUtil._eventQueues[eventId];
   	if(queue && event.newValue){
   		var message=JSON.parse(event.newValue);
   		if(message!==null){
   			queue.fireEvent(eventId,{payload:message});
   		}
   	}
   },
   attachStorageEvent:function(eventId,callback,scope){
   	var eventQueues=sas.ui.cpb3.CpbUtil._eventQueues;
   	if(!eventQueues[eventId]){
   		eventQueues[eventId]=new sap.ui.base.EventProvider()
   	}
   	eventQueues[eventId].attachEvent(eventId,callback,scope);
   },
   detachStorageEvent:function(eventId,callback,scope){
   	var eventQueues=sas.ui.cpb3.CpbUtil._eventQueues;
   	if(eventQueues[eventId]){
   		eventQueues[eventId].detachEvent(eventId,callback,scope);
   	}   	
   },
   fireStorageEvent:function(eventId,message,fireLocally){
   	localStorage.setItem(eventId,null);
   	localStorage.setItem(eventId,JSON.stringify(message));
   	if(fireLocally){
      	var eventQueues=sas.ui.cpb3.CpbUtil._eventQueues;
      	if(eventQueues[eventId]){
      		eventQueues[eventId].fireEvent(eventId,{payload:message});
      	}
   	}
   },
   _contextPath:top.location.pathname,
   getContextPath:function(){
   	return sas.ui.cpb3.CpbUtil._contextPath;
   },
   _contextQuery:top.location.search||"",
   getContextQuery:function(useAmpersand){
   	if(useAmpersand && sas.ui.cpb3.CpbUtil._contextQuery){
   		return "&"+sas.ui.cpb3.CpbUtil._contextQuery.substring(1);
   	}
   	return sas.ui.cpb3.CpbUtil._contextQuery;
   },
   _loadedLibs:[],
   loadJSLibraries:function(){
   	sas.ui.cpb3.CpbAPI.loadJSLibraries().done(function(json){
   		_.each(json,function(lib){
   			lib["modules"][lib.name.replace(/\./g,"/")+".js"]="jQuery.sap.registerPreloadedModules("+JSON.stringify(lib)+");";
   			var libName=lib.name.substring(0,lib.name.length-"library-preload".length-1);
   			sas.ui.cpb3.CpbUtil._loadedLibs[libName]=lib;
   			sap.ui.getCore().loadLibrary(libName,"CPB_PRELOAD:"+libName+":");
//   			jQuery.sap.registerPreloadedModules(lib);
   			var css=lib.modules[libName.replace(/\./g,"/")+"/themes/sas_corporate/library.css"];
   			if(css){
					var styles=document.createElement("style");
					styles.id="sap-ui-theme-"+libName;
					styles.type="text/css";
					$(styles).html(css);
					document.getElementsByTagName("head")[0].appendChild(styles);
				};
//   			if(lib.init){
//   				jQuery.sap.require(lib.init);
//   				sap.ui.require(lib.init.replace(/\./g, "/")).initialize();
//   			}
   		});
   	});
   }
};
window.addEventListener('storage', sas.ui.cpb3.CpbUtil._storageEventListener, this);
$.ajaxPrefilter(function(options){
	if(options.url && options.url.startsWith("CPB_PRELOAD")){
		return "cpb";
	}
});
$.ajaxTransport("cpb",function(options,originalOptions,jqXHR){
	options.dataTypes.shift();//take "cpb" dataType off the list
	var mainType=options.dataTypes[0];
	var responses={};
	var split=options.url.split(":");
	var libName=split[1];
	var resource=libName.replace(/\./g,"/")+split[2];
	return {
		send: function(headers, completeCallback){
			var lib=sas.ui.cpb3.CpbUtil._loadedLibs[libName];
			if(lib){
				var json=lib.modules[resource];
				if(json){
					responses[mainType]=json;
					completeCallback(200,"success",responses);
				}else{
					completeCallback(404,"source "+resource+" in lib "+libName+" not found");
				}
			}else{
				completeCallback(404,"lib "+libName+" not found");
			}
		},
		abort:function(){
			console.log("Aborted ajax transport for cpb lib loading.");
		}
	}
});