jQuery.sap.declare("sas.ui.cpb3.OnChangeFactory");
jQuery.sap.require("sas.ui.cpb3.CpbAPI");
jQuery.sap.require("sas.ui.cpb3.model.CpbModelManagerObject");

sas.ui.cpb3.OnChangeFactory = {
  preprocessActions: function (jsonArray, cpbForm, infLoopCheck) {
    var actions = [];
    _.each(jsonArray, function (action) {
      switch (action.type) {
        case "SetValueAction":
          if (infLoopCheck.indexOf(action.name.toUpperCase()) !== -1) {
            console.error("Circular dependency on " + action.name, infLoopCheck);
            return;//skip this action but continue
          }
          actions.push(action);
          var observers = cpbForm.getObservers(action.name);
          if (observers.length) {
            infLoopCheck.push(action.name.toUpperCase());
            _.each(observers, function (observer) {
              var processedObsActions = sas.ui.cpb3.OnChangeFactory.preprocessActions(observer.actions, cpbForm, action.isOnChangeAction ? [action.name.toUpperCase()] : infLoopCheck);
              actions.push.apply(actions, processedObsActions);
            });
            infLoopCheck.pop();
          }
          break;
        default:
          actions.push(action);
      }
    });
    return actions;
  },
  create: function (jsonArray, fieldName, cpbForm) {
    var fieldObservers = cpbForm.getObservers(fieldName);
    var infLoopCheck = [];
    var enableMultiEval = false;
    var GZIPCompressionThreshold = 50000;
    var multiEvalPayloadDebugging = false;
    enableMultiEval = sas.risk.rgf.api.CommonAPI.getDefaultMultiEval(); // HARDCODING it to true always  RGFRMWRK-516
    GZIPCompressionThreshold = sas.risk.rgf.api.CommonAPI.getGZIPCompressionThreshold();
    multiEvalPayloadDebugging = sas.risk.rgf.api.CommonAPI.getMultiEvalPayloadDebugging();

    if (fieldObservers.length) {//if field has observers then concat their actions to the fields on-change actions
      infLoopCheck.push(fieldName.toUpperCase());
      if (!jsonArray || !$.isArray(jsonArray)) {
        jsonArray = [];
      }
      _.each(jsonArray, function (onChangeAction) { onChangeAction.isOnChangeAction = true; });//identify which actions are from the on-change vs observer
      var actionsArray = _.pluck(fieldObservers, "actions");
      var actions = [];
      _.each(actionsArray, function (obsActions) { actions.push.apply(actions, obsActions); });
      jsonArray.push.apply(jsonArray, actions);
    }
    if (jsonArray && $.isArray(jsonArray)) {
      var actions = sas.ui.cpb3.OnChangeFactory.preprocessActions(jsonArray, cpbForm, infLoopCheck);
      var funcs = [];
      var summary = { setValue: [], setValues: [], setRequired: [], setVisible: [], refresh: [], call: [] };
      var allactions=[];
      
      var i, json;
      for (i = 0; i < actions.length; i++) {
        json = actions[i];
        switch (json.type) {
          case "SetValueAction":
            funcs.push(function (name, value) {
              return function (cachedResult) {
                var control = sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbForm, name);
                var screenDef = sas.ui.cpb3.OnChangeFactory._getCPBFormScreenDef(cpbForm);
                var data = sas.ui.cpb3.OnChangeFactory._getCPBFormData(cpbForm);
                return cpbForm.getCpbService().getCpbAPI().evalExpression(value, data, screenDef, {result:cachedResult}, GZIPCompressionThreshold).done(function (value) {
                  if (control) {
                    control.setCPBValue(value.result);
                    try {
                      control.getCPBControl()._formatInputValue(); // makes sure input preserves value
                    } catch (e) { console.log(e); }
                  } else {
                    //cpbForm.getData()[name.toUpperCase()]=value.result;
                    cpbForm.getCpbModelManager().getUIContextModel(cpbForm).setProperty("/" + name.toUpperCase(), value.result);
                  }
                });
              };
            }(json.name, json.value));
            summary.setValue.push(json.name);
            allactions.push({name:json.name, expr: json.value, type:"set-value"});
            break;
          case "SetValuesAction":
            funcs.push(function (name, value) {
              return function (cachedResult) {
                var control = sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbForm, name);
                var screenDef = sas.ui.cpb3.OnChangeFactory._getCPBFormScreenDef(cpbForm);
                var data = sas.ui.cpb3.OnChangeFactory._getCPBFormData(cpbForm);
                return cpbForm.getCpbService().getCpbAPI().evalExpression(value, data, screenDef,{result:cachedResult}, GZIPCompressionThreshold).done(function (value) {
                  control.setCPBValues(value.result);
                });
              };
            }(json.name, json.values));
            summary.setValues.push(json.name);
            allactions.push({name:json.name, expr: json.values, type:"set-values"});
            break;
          case "SetRequiredAction":
            funcs.push(function (name, test) {
              return function (cachedResult) {
                var control = sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbForm, name);
                var screenDef = sas.ui.cpb3.OnChangeFactory._getCPBFormScreenDef(cpbForm);
                var data = sas.ui.cpb3.OnChangeFactory._getCPBFormData(cpbForm);
                return cpbForm.getCpbService().getCpbAPI().evalExpression(test, data, screenDef, {result: cachedResult}, GZIPCompressionThreshold).done(function (value) {
                  if (control !== undefined) {
                    control.setCPBRequired(value.result);
                  }
                });
              };
            }(json.name, json.test));
            summary.setRequired.push(json.name);
            allactions.push({name:json.name, expr: json.test, type:"required"});
            break;
          case "SetVisibleAction":
            funcs.push(function (name, test) {
              return function (cachedResult) {
                var control = sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbForm, name);
                var screenDef = sas.ui.cpb3.OnChangeFactory._getCPBFormScreenDef(cpbForm);
                var data = sas.ui.cpb3.OnChangeFactory._getCPBFormData(cpbForm);
                return cpbForm.getCpbService().getCpbAPI().evalExpression(test, data, screenDef, {result:cachedResult}, GZIPCompressionThreshold ).done(function (value) {
                  if (control !== undefined) {
                    control.setCPBVisible(value.result);
                  }
                });
              };
            }(json.name, json.test));
            summary.setVisible.push(json.name);
            allactions.push({name:json.name, expr: json.test, type:"visible"});
            break;
          case "RefreshAction":
            funcs.push(function (name, _if) {
              return function () {
                var control = sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbForm, name);
                var screenDef = sas.ui.cpb3.OnChangeFactory._getCPBFormScreenDef(cpbForm);
                var data = sas.ui.cpb3.OnChangeFactory._getCPBFormData(cpbForm);
                if (_if !== undefined) {
                  return cpbForm.getCpbService().getCpbAPI().evalExpression(_if, data, screenDef, GZIPCompressionThreshold).done(function (value) {
                    if (control !== undefined) {
                      control.doCPBRefresh();
                    }
                  });
                } else {
                  control.doCPBRefresh();
                  return $.Deferred().resolve({ data: cpbForm.getData() });
                }
              };
            }(json.name, json._if));
            summary.refresh.push(json.name);
            allactions.push({name:json.name, expr: json.value, type:"refresh"});
            break;
          case "CallAction":
            funcs.push(function (func, _if) {
              return function (cachedResult) {
                var deferred = $.Deferred();
                var cpbModelManager = cpbForm.getCpbModelManager();
                var control = sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbForm, name);
                var screenDef = sas.ui.cpb3.OnChangeFactory._getCPBFormScreenDef(cpbForm);
                var data = sas.ui.cpb3.OnChangeFactory._getCPBFormData(cpbForm);
                if (_if !== undefined) {
                  cpbForm.getCpbService().getCpbAPI().evalExpression(_if, data, screenDef, {result:cachedResult}, GZIPCompressionThreshold).done(function (value) {
                    if (value.result) {
                      cpbForm.getCpbService().getCpbAPI().evalExpression(func, data, screenDef, {result:cachedResult},GZIPCompressionThreshold).done(function (value2) {
                        cpbModelManager.updateData(value2.result);
                        deferred.resolve(value2);
                      });
                    } else {
                      deferred.resolve(value);
                    }
                  });
                } else {
                  cpbForm.getCpbService().getCpbAPI().evalExpression(func, data, screenDef, {result:cachedResult}, GZIPCompressionThreshold).done(function (value) {
                    cpbModelManager.updateData(value.result);
                    deferred.resolve(value);
                  });
                }
                return deferred;
              };
            }(json.func, json._if));
            summary.call.push(json.func);
            allactions.push({name:json.name, expr: json.func, type:"call"});
            break;
            case "enableMultiEval":
              enableMultiEval = json.value;
            break;
        }
      }
      return function () {
        var arr = (funcs || []).slice(0);
        var actions=(allactions || []).slice(0);
        //fill up cache using multi-eval
        // funcs = [set-val, refresh, refresh, set-val on currency];
        // [
        //   {name:"a", expr="1+2", action: "set-value"},
        //   {          expr="a eq 2", action: "refresh"}, // <refresh name="widget123" if="a eq 2" />
        //   {          expr="true", action:"refresh"} // <refresh name="widget123"/>
        // ]
        // answers:
        // [
        //   3,
        //   false,
        //   true
        // ]
        var cache = [/* answers */];
        var index=0;
        var app = sas.ui.mashup.App.getInstance();
        var func = function (def) {
          app.setBusy(true, 100);
          if (arr.length) {
            //if (point reached?)
            sas.ui.cpb3.OnChangeFactory._fillupCacheMap(cpbForm, cache, actions, enableMultiEval, GZIPCompressionThreshold, multiEvalPayloadDebugging).then(function() {
              //arr.shift()(cacheMap[index++]).then(function () {
              arr.shift()(cache.shift()).then(function () {
                func(def);
            });
          })
          } else {
            app.setBusy(false);
            def.resolve();
            cpbForm.fireOnChangeFinished({ cpbForm: cpbForm, summary: summary });
          }
        };
        var deferred = $.Deferred();
        func(deferred);
        return deferred;
      };
    }
    return null;
  },
  _createErrorMsg: function (value) {
    var json = {
      hasErrors: true,
      cpb_messages: {
        messages: [{
          message: value.error.message,
          status: value.error.type
        }]
      },
      cpb_data: value.data
    };
    return json;
  },
  _getCPBFormData: function (cpbForm) {
    if (cpbForm.getCpbParent()) {
      var data = $.extend({ PARENT: cpbForm.getCpbParent().getData() }, cpbForm.getData());
      return data;
    }
    return cpbForm.getData();
  },
  _getCPBFormScreenDef: function (cpbForm) {
    if (cpbForm.getCpbParent()) {
      var cpbFormComponent = cpbForm.getParent();
      var context = cpbFormComponent.getData()[cpbForm.getScreenId()];
      return context && context.screen;
    }
    return null;
  },
  _getCPBControl: function (cpbForm, name) {
    if (!cpbForm) { return null; }
    var control = cpbForm.getCPBControl(name);
    if (!control) {
      var split = name.split("|");
      if (split.length > 1) {
        var baseName = split.shift();
        if (baseName === "PARENT" && cpbForm.getCpbParent()) {
          return sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbForm.getCpbParent(), split.join("|"));
        }
        var cpbFormComponent = cpbForm.getCPBControl(baseName);
        if (cpbFormComponent && cpbFormComponent instanceof sas.ui.cpb3.components.CPBFormComponent) {
          return sas.ui.cpb3.OnChangeFactory._getCPBControl(cpbFormComponent.getComponent(), split.join("|"));
        }
      }
    }
    return control;
},
excludeComponents:[// for testing only "sas.ui.cpb3.field.StringField", 
                   "sas.risk.rgf.components.CurrencyInput",
                   "sas.ui.cpb3.components.CPBFormComponent"],

_fillupCacheMap: function (cpbForm, cache, actions, enableMultiEval, GZIPCompressionThreshold, multiEvalPayloadDebugging){
  var data = sas.ui.cpb3.OnChangeFactory._getCPBFormData(cpbForm);
  var def = $.Deferred();
  var itemTypeNoop=false;
  var evalsArrayCount=0;

    //Add logic here to create cache
	  if (cache.length) {
      def.resolve(true);
     } else { 
      var evals= [];
      while (actions.length) {
        var item = actions.shift();
        evals.push(item);
        evalsArrayCount++;
        
        if (item.type ==="call"  || item.type === "required") {
          itemTypeNoop = true;
          break;
        }
        if (!enableMultiEval) {
          break;
        }
        if (evalsArrayCount >=100) { //Limiting batchsize of multiEval to 100 to avoid out of memory recursion problem RGFRMWRK-602
          evalsArrayCount=0;
          break;
        }
        var cpbFormComponent = cpbForm.getCPBControl(item.name);
        if (cpbFormComponent && sas.ui.cpb3.OnChangeFactory.excludeComponents.indexOf(cpbFormComponent.getMetadata().getName()) !== -1  ) {
          //console.info(cpbFormComponent.getType());
          break;
        }
      }

      var screenDef = sas.ui.cpb3.OnChangeFactory._getCPBFormScreenDef(cpbForm);
		  
      if (enableMultiEval  && evals.length > 0) {
        cpbForm.getCpbService().getCpbAPI().multieval(evals, data, screenDef, GZIPCompressionThreshold, multiEvalPayloadDebugging).then(function(result){
          cache.splice.apply(cache, [0,0].concat(result));
          def.resolve(true);
        });
      } else {
      if (itemTypeNoop || evals.length == 0) {
        def.resolve(true);
        //cpbForm.getCpbService().getCpbAPI().evalExpression(evals, data, screenDef).then(function(result){
        //  cache.splice.apply(cache, [0,0].concat(result.result));
        //  def.resolve(true);
        //});  
      } else {
        cpbForm.getCpbService().getCpbAPI().evalExpression(evals[0].expr, data, screenDef, GZIPCompressionThreshold).then(function(result){
          cache.splice.apply(cache, [0,0].concat(result.result));
          def.resolve(true);
        });
      }
      }
    } 

    return def;
}
};  