jQuery.sap.declare("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.ui.cpb3.model.CpbModelManagerObject");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.ui.cpb3.OnChangeFactory");

sas.ui.cpb3.WidgetBindingUtil={
	getBasicSettings:function(fieldJson,modelId,cpbForm){
		return $.extend(
			sas.ui.cpb3.WidgetBindingUtil.getEnabledSettings(fieldJson,modelId),
			sas.ui.cpb3.WidgetBindingUtil.getHelpSettings(fieldJson,modelId),
			{
				cpbForm:cpbForm,
				cpbId:fieldJson.widgetId,
				cpbName:fieldJson.cpbName?fieldJson.cpbName.toUpperCase():fieldJson.cpbName,
				cpbLabel:{
					path:"cpb_labels>/label"+fieldJson.cpbLabel,
					formatter:sas.ui.cpb3.WidgetBindingUtil.formatterLabel
				},
				cpbOnChangeFn:sas.ui.cpb3.OnChangeFactory.create(fieldJson.cpbOnChange,fieldJson.cpbName,cpbForm),
				cpbRequired:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbRequired),
				cpbVisible:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbVisible),
				cpbErrored:{
					path:sas.ui.cpb3.model.CpbModelManagerObject.bindMessagePath("/errored"),
					formatter:sas.ui.cpb3.WidgetBindingUtil.formatterError
				},
				labelOverridePrefix:fieldJson.labelOverridePrefix,
				cpbIsFieldLike:!!fieldJson.cpbCompIsFieldLike
			}
		);
	},
	getFieldBasicSettings:function(fieldJson,modelId,cpbForm){
		var basicSettings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	   delete basicSettings.cpbIsFieldLike;
		return $.extend(
			basicSettings,
			{
				cpbAlign:fieldJson.cpbAlign
			}
		);
	},
	getEnabledSettings:function(fieldJson,modelId){
		return {
			cpbEnabled:{
         	parts:[sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,"SCREEN_READ_ONLY"), 
         	       sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,"IS_SCREEN_PREVIEW"), 
         	       sas.ui.cpb3.WidgetBindingUtil.getExpPath(fieldJson.cpbReadonly) 
				],
				formatter:function(screenReadOnly,screenPreview,fieldReadOnly){
					if(this.mixinType==="Component" || this.getParent && this.getParent() && this.getParent().mixinType==="Component"){
						var readonly=screenPreview ? screenPreview : fieldReadOnly!==undefined ? fieldReadOnly : screenReadOnly;
						return !readonly;
					}else{
						var readonly=screenReadOnly || fieldReadOnly;
						return !readonly;
					}
				}
         }
		};
	},
	getHelpSettings:function(fieldJson,modelId){
		var settings={};
		if(fieldJson.cpbHelp!==undefined){
			settings.cpbHelp={
				path:"cpb_labels>/label"+fieldJson.cpbHelp,
				formatter:sas.ui.cpb3.WidgetBindingUtil.formatterHelp
			};
		}
		return settings;
	},
	getLengthSettings:function(fieldJson,modelId,cpbForm){
		return {
			cpbLength:fieldJson.cpbLength?fieldJson.cpbLength:(fieldJson.cpbMaxLength?Math.min(fieldJson.cpbMaxLength,30):30),
			cpbMaxLength:fieldJson.cpbMaxLength
		};
	},
	getMinMaxSettings:function(fieldJson,modelId,cpbForm){
		return {
			cpbMin:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbMin),
			cpbMax:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbMax)
		};
	},
	//PATHS
	getDataBindingPath:function(dataModelId, name, noUpper) {
		return "{" + sas.ui.cpb3.WidgetBindingUtil.getDataPath(dataModelId, name, noUpper) +"}";
	},
	getDataPath:function(dataModelId,name,noUpper){
		var valuePath = "/"+(noUpper?name:name.toUpperCase());
		return dataModelId ? dataModelId + ">" + valuePath : valuePath;		
	},
	getExpBindingPath:function(id){
		if(id===undefined){return null;}
		return "{"+sas.ui.cpb3.WidgetBindingUtil.getExpPath(id)+"}";
	},
	getExpPath:function(id){
		return "cpb_exps>/expr"+id;
	},
	getChoicesBindingPath:function(name) {
		return "{" + sas.ui.cpb3.WidgetBindingUtil.getChoicesPath(name) +"}";
	},
	getChoicesPath:function(name){
		return "cpb_choices>/"+name.toUpperCase();
	},
	//FORMATTERS
	formatterLabel:function(value){
		if(value && value.message){
			return value.separatorVisible?jQuery.sap.formatMessage(value.message.replace(/'/g,"''")+"{0}",[":"]):value.message;
		}else{
			return "";
		}
	},
	formatterLabelNoSeparator:function(value){
		if(value && value.message){
			return value.message;
		}else{
			return "";
		}		
	},
	formatterTrueLabel:function(rb,value){
		var val=sas.ui.cpb3.WidgetBindingUtil.formatterLabelNoSeparator(value);
		return val==="" ? rb.getText("boolean.yes.txt") : val;
	},
	formatterFalseLabel:function(rb,value){
		var val=sas.ui.cpb3.WidgetBindingUtil.formatterLabelNoSeparator(value);
		return val==="" ? rb.getText("boolean.no.txt") : val;
	},
	formatterHelp:function(value){//TODO
		if(value && value.message){
			return value.message;
		}else{
			return "";
		}
	},
	formatterError:function(errored){
		var cpbId=sas.ui.cpb3.CpbUtil.findCpbId(this);
		if(cpbId===null){//control is not in a cpbform
			return false;
		}
		var screenId=sas.ui.cpb3.WidgetBindingUtil._findCpbForm(this).getScreenId();
		if(!errored){return false;}
		return (errored.indexOf(cpbId)!==-1 || errored.indexOf(screenId+"."+cpbId)!==-1);
	},
	_findCpbForm:function(control){
		if(!control){return null;}
		if(!(control.getCpbForm)){
			return sas.ui.cpb3.WidgetBindingUtil._findCpbForm(control.getParent());
		}
		return control.getCpbForm();
	}
}