jQuery.sap.declare("sas.ui.cpb3.api.CpbObjectAPI");
jQuery.sap.require("sas.ui.cpb3.api.UIContextAPI");

sas.ui.cpb3.api.CpbObjectAPI=function(cpbForm,uiContextAPI){
	this.cpbForm=cpbForm;
	this.uiContextAPI=uiContextAPI;
};
sas.ui.cpb3.api.CpbObjectAPI.prototype={
	setCpbService:function(cpbService){
		this.cpbService=cpbService;
	},
	/**************************************************************
	 *            LOAD OBJECT CONTEXT
	 **************************************************************/
	loadObjectContext : function(screenId,objectType,objectId,overrides) {
		var def=$.Deferred();
		var meta={screenId:screenId,objectType:objectType,objectId:objectId,overrides:overrides};
		this._getUIContextAPI().getContextForExisting(screenId,objectType,objectId,overrides)
		.done($.proxy(this._handleObjectResponse,this,def,meta))
		.fail($.proxy(this._handleFail,this,def));
		return def;
	},
	/**************************************************************
	 *            CREATE OBJECT CONTEXT
	 **************************************************************/
	createObjectContext : function(screenId,objectType,objectId,overrides) {
		var def=$.Deferred();
		var meta={screenId:screenId,objectType:objectType,objectId:objectId,overrides:overrides};
		this._getUIContextAPI().createContextForDraft(screenId,objectType,objectId,overrides)
		.done($.proxy(this._handleObjectResponse,this,def,meta))
		.fail($.proxy(this._handleFail,this,def));
		return def;
	},
	/**************************************************************
	 *            SAVE OBJECT CONTEXT
	 **************************************************************/
	saveObjectContext : function(objectType,objectId,data,changeReason,nestedSaves,nestedDeletes){
		var def=$.Deferred();
		var meta={data:data,objectType:objectType,objectId:objectId,changeReason:changeReason,nestedSaves:nestedSaves,nestedDeletes:nestedDeletes};
		this._getUIContextAPI().saveContext(objectType,objectId,data,changeReason,nestedSaves,nestedDeletes)
		.done($.proxy(this._handleObjectResponse,this,def,meta))
		.fail($.proxy(this._handleFail,this,def));
		return def;
	},
	/**************************************************************
	 *            VALIDATE OBJECT CONTEXT
	 **************************************************************/
   validateObjectContext : function(objectType,objectId,data,changeReason){
       var def=$.Deferred();
 		 var meta={data:data,objectType:objectType,objectId:objectId,changeReason:changeReason};
       var def2=this._getUIContextAPI().validateContext(objectType,objectId,data,changeReason);
       def2.done($.proxy(this._handleObjectResponse,this,def,meta))
           .fail($.proxy(this._handleFail,this,def));
       return def;
   },
   /**************************************************************
    *            HANDLE OBJECT RESPONSE
    **************************************************************/
	_handleObjectResponse:function(def,meta,response,isErrored){
		if(isErrored){
			this._handleObjectResponseError(def,meta,response);
		}else{
			this._handleObjectResponseSuccess(def,meta,response);
		}
	},
   _handleObjectResponseSuccess:function(def,meta,response,isErrored){
		response.messages={messages:response.messages||[]};
		response.misc=response.misc||{};//make sure the cpbform gets rebuilt from scratch
		response.misc.rebuild=true;
		var json={
			cpb_labels:response.labels,
			cpb_exps:response.expressions,
			cpb_structure:response.structure,
			cpb_observers:response.observers,
			cpb_choices:response.choices,
			cpb_messages:response.messages,
			cpb_misc:response.misc,
			cpb_data:response.data,
			cpb_nestedSaves:response.nestedSaves||{},
			cpb_nestedDeletes:response.nestedDeletes||{},
			hasErrors:false
		};
		def.resolve(json);
   },
   _handleObjectResponseError:function(def,meta,response){
		def.resolve({
			cpb_messages:this._processValidationErrors(response.messages),
			cpb_data:response.data,
			cpb_nestedSaves:response.nestedSaves||{},
			cpb_nestedDeletes:response.nestedDeletes||{},
			hasErrors:true
		});
   },
   /**************************************************************
    *            PROCESS VALIDATION ERRORS
    **************************************************************/
	_processValidationErrors:function(json){
		var errored=[];
		var i;
		for(i=0;i<json.length;i++){
			var controlId=json[i].parentWidgetId||json[i].widgetId;
			if(!controlId) {
				break;
			}
			var split=controlId.split(".");
			var control=this.cpbForm.getCPBControlById(split.shift());
			jQuery.sap.require("sas.ui.cpb3.components.CPBFormComponent");
			if(control instanceof sas.ui.cpb3.components.CPBFormComponent){
				var descendantInfo=control.getDescendantControlById(split.shift(),Number.parseInt(split.shift()),split.join("."));
				var pathArray=this._calcIdPath(descendantInfo.control);
				pathArray=_.map(pathArray,function(entry){return descendantInfo.screenId+"."+entry;});
				if(pathArray.length){
					errored=_.union(errored,this._calcIdPath(control));//add cpbform comp itself if children have errors
				}
				errored=_.union(errored,pathArray);
			}else{
				errored=_.union(errored,this._calcIdPath(control));
			}
		}
		if(errored.length === 0) {
			var errorLength = json.length;
			for(i=0;i<errorLength;i++) {
				errored.push(-1);
			}
			return {
				messages:json,
				errored: errored
			};
		} else {
			return {
				messages:json,
				errored:errored
			};
		}
	},
	_calcIdPath:function(control,array){
		array=array||[];
		if(!control){
			return array;
		}
		jQuery.sap.require("sas.ui.cpb3.layout.MatrixLayout");
		if(!(control instanceof sas.ui.cpb3.layout.MatrixLayout)){
			array.push(control.getCpbId());
		}
		return this._calcIdPath(control.getCpbParent(),array);
	},
	_handleFail:function(def,e){
		jQuery.sap.log.error( "Request Failed. " + e);
		def.reject(e);
	},
	_getUIContextAPI:function(){
		return this.uiContextAPI;
	}
};