jQuery.sap.declare("sas.ui.cpb3.components.AdvancedTableComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.ui.cpb3.widget.CpbTable");
jQuery.sap.require("sas.risk.slimtable.util.RowActionUtil");
jQuery.sap.require("sas.risk.slimtable.widget.menu.SeparatorItem");
jQuery.sap.require("sas.risk.slimtable.widget.menu.AdvSearchItem");
jQuery.sap.require("sas.risk.slimtable.widget.menu.ColumnsItem");
jQuery.sap.require("sas.ui.cpb3.nav.NavUtil");
jQuery.sap.require("sas.hc.m.MessageBox");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.risk.slimtable.plugins.TableOptions");
jQuery.sap.require("sas.risk.slimtable.plugins.TablePlugin");
jQuery.sap.require("sas.risk.slimtable.plugins.Counter");
jQuery.sap.require("sas.risk.slimtable.plugins.Pager");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.risk.rgf.api.RGFCpbObjectAPI");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.components.AdvancedTableComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
            "dataUrl":{type:"string"},
            "meta":{type:"any"},
            "metaUrl":{type:"string"},
            "visibleRows":{type:"int"},
            "queryTableType":{type:"string"},
            "relatedQueryTableType":{type:"any"},
            "showCreateButton":{type:"boolean",defaultValue:true},
            "showCreateFromAction":{type:"boolean",defaultValue:true},
            "showActionColumn":{type:"boolean",defaultValue:true},
            "hasCreatePermission":{type:"boolean"},
            "allowRowClickNavigation":{type:"boolean",defaultValue:true},
			formattingKey:{type:"any"},
			"quickLinkTypesForCreate":{type:"any"}
        }
    },
    constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 	 },
 	 renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.ui.cpb3.components.AdvancedTableComponent);
sas.ui.cpb3.components.AdvancedTableComponent.prototype._createComponent = function() {
	var that=this;
	this.getCpbForm().attachEvent('reload',$.proxy(this.onReload,this));

   var metaProp=this.getProperty("meta");
	if(!this.getCpbLabel() && metaProp.title){
      this.setCpbLabel(metaProp.title);
   }
   this.dataUrlProp=this.getProperty("dataUrl");
   var visibleRowsProp=this.getProperty("visibleRows");
   var queryTableType=this.getQueryTableType();
   var relatedQueryTableType=this.getRelatedQueryTableType();

   this.comp = this.getAdvTableComponent(queryTableType, visibleRowsProp, relatedQueryTableType);
   
   sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.comp,"enabled");
   //ACTION COLUMN
   var showActionColumn=this.getProperty("showActionColumn");
   if (showActionColumn) {
      var actionColumn=sas.risk.slimtable.util.RowActionUtil.createActionColumn($.proxy(this.createActionMenuItems,this),this.comp);
      this.comp.addActionColumn(actionColumn);
   }
   if(this.getAllowRowClickNavigation()){
   	this.comp.attachRowClick($.proxy(this.doRowClick,this));
   }else{
   	this.comp.addStyleClass("cpbTabNoClick");
   }
   return this.comp;
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.getTableTopActionsPlugin=function(){
	var that=this;
   this.createNewButton=new sas.risk.common.widgets.Button({
      press: $.proxy(this.doNewButtonClick,this),
      icon:sas.icons.HC.NEWSTARBURST
   }).addStyleClass("QueryTableCreateButton");
   
   this.tableTopActionsPlugin=new sas.risk.slimtable.plugins.TablePlugin({
     	position: sas.risk.slimtable.TableLayout.TOP_END,
   	items:[]
   });
   var showCreateButton=this.getProperty("showCreateButton");
   if (showCreateButton) {
      this.tableTopActionsPlugin.addItem(this.createNewButton);
   }
   this.tableTopActionsPlugin.handleDisabled=function(){
   	that.createNewButton.setVisible(false);
   };
   return this.tableTopActionsPlugin;
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.doNewButtonClick=function(oEvent){
   var type = this.getQueryTableType();
   sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
		this.getCpbForm(),
		{
			key:"drafts",
			type:type,
			event_name:"create_new"
		}
	);
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.getTableMenuItems=function(){
   var menuItems=[
     new sas.risk.slimtable.widget.menu.AdvSearchItem(),
     new sas.risk.slimtable.widget.menu.ColumnsItem(),
   ];
   return menuItems;
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.getAdvTableComponent=function(queryTableType, visibleRowsProp, relatedQueryTableType){
	return new sas.ui.cpb3.widget.CpbTable({
		formattingKey:["QueryTable",queryTableType,this.getCpbName(),relatedQueryTableType],
      meta: this.getMeta(),
      data: $.proxy(this._fetchData,this),
      visibleRows: visibleRowsProp,
      showSelectionControls:false,
      plugins:[this.getTableTopActionsPlugin(),
               new sas.risk.slimtable.plugins.TableOptions({
             	  menuItems: this.getTableMenuItems()
               }),
               new sas.risk.slimtable.plugins.Counter(),
               new sas.risk.slimtable.plugins.Pager()
      ]
   });
};

sas.ui.cpb3.components.AdvancedTableComponent.prototype.doRowClick=function(oEvent){
   var type = this.getQueryTableType();
   var rowData=this.comp.getSource().getData()[oEvent.getParameter("rowIdx")];
   sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
		this.getCpbForm(),
		{
			event_name:"object_selected",
			type:type,
			key:rowData.key
		}
	);
};

sas.ui.cpb3.components.AdvancedTableComponent.prototype.getCPBControl=function(){
	return this.comp;
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype._fetchData = function(query) {
	return this.getCPBControl().fetchData(this.dataUrlProp,query);
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.doCPBRefresh = function() {
	this.getCPBControl().doRefresh();
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.onReload = function(event) {
	var type=event.getParameter("type");
	var i;
	var doRefresh = (type===this.getQueryTableType() || type==="*");
	var relatedQueryTableTypes=this.getRelatedQueryTableType();
	if (!doRefresh && relatedQueryTableTypes) {
		for (i = 0; i < relatedQueryTableTypes.length; i++) {
			var relatedQueryTableType = relatedQueryTableTypes[i];
			if (type===relatedQueryTableType) {
				doRefresh = true;
				break;
			}
		}
	}
	if (doRefresh) {
		this.doCPBRefresh();
	}
};

//ACTION MENU
sas.ui.cpb3.components.AdvancedTableComponent.prototype.createActionMenuItems=function(payload){
	var rb = this.getCpbForm().getCpbService().getBundle();
	var i, menuItems=[], links=payload.rowData.links;
	var canUpdate=_.find(links,function(link){return link.rel.toUpperCase()==="UPDATE"});
	var canDelete=_.find(links,function(link){return link.rel.toUpperCase()==="DELETE"});
	var quickLinks = this.getQuickLinkTypesForCreate();
	if(canUpdate){
		var editItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
			text : rb.getText("queryTable.action.edit.txt"),
			callback : $.proxy(this.actionRowEdit,this)
		});
		menuItems.push(editItem);
	}
	var showCreateFromAction=this.getProperty("showCreateFromAction");
	if(showCreateFromAction && this.getHasCreatePermission()){
		var createNewFromItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({ //TODO: create permission from meta
			text : rb.getText("queryTable.action.createNewFrom.txt"),
			callback : $.proxy(this.actionRowCreateNewFrom,this)
		});
		menuItems.push(createNewFromItem);
	}
	if(canDelete){
		var deleteItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
			text : rb.getText("queryTable.action.delete.txt"),
			callback : $.proxy(this.actionRowDelete,this)
		});
		menuItems.push(deleteItem);
	}
	
	if(quickLinks) {
		var quickLinksSubMenu = [];
		var that = this;
		_.each(quickLinks, function(ql) {
			quickLinksSubMenu.push(sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
				text : ql.createLabel,
				callback : $.proxy(that.actionQuickLink,that, ql)
			}));
		});
		menuItems.push(quickLinksSubMenu);
	}
	return menuItems;
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.actionRowEdit=function(payload){
	sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
		this.getCpbForm(),
		{
			event_name:"edit_object",
			key:payload.rowData.key,
			type:this.getQueryTableType()
		}
	);   
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.actionQuickLink=function(link, source){
	var that = this;
	var sourceTypeName = link.linkType["LINKTYPE.BUSINESSOBJECTTYPENM1"]	
	if(link.sideOfLink !== 0 ){
		sourceTypeName = link.linkType["LINKTYPE.BUSINESSOBJECTTYPENM2"]
	}
	this.getCpbForm().getCpbAPI().loadObjectContext(sourceTypeName, this.getQueryTableType(), source.rowData.key, null).done(function(json) {
		that.hydrateStorageWithSourceObjectInfo(json)
		sas.risk.rgf.util.NavigationUtil.navTo(
			that.getCpbForm(),
			{
				key:"drafts",
				type:link.collectionName,
				linkTypeKey:link.linkTypeKey,
				sourceObjectKey:source.rowData.key,
				sideOfLink:link.sideOfLink,
				sourceTypeName: sourceTypeName===0?link.linkType["LINKTYPE.BUSINESSOBJECTTYPENM1"]:link.linkType["LINKTYPE.BUSINESSOBJECTTYPENM2"],
				targetMeta:link.table_meta_target,
				sourceSystemCd:link.sourceSystemCode,
				variance:"lbo",
				event_name:"create_new_lbo"
			}
		);  
	}).fail(function(json){
		console.error(json);
	})
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.hydrateStorageWithSourceObjectInfo=function(json){
	if(typeof(Storage) !== "undefined") {
		if(json && json.cpb_data){
			if(json.cpb_data.LINKEDBUSINESSOBJECTS) {
				sessionStorage.setItem("previous_Linked_BusinessObjects", JSON.stringify(json.cpb_data.LINKEDBUSINESSOBJECTS.objectLinks));
			}
			sessionStorage.setItem("previous_BusinessObject", JSON.stringify(json.cpb_data.BUSINESSOBJECT));
			var location = JSON.stringify(json.cpb_data.LOCATION);
			if (location) {
				sessionStorage.setItem("previous_Location", location);		
			}
			else {
				sessionStorage.setItem("previous_Location", "");  //this object does not have a location
			}
		}
   } else {
      console.warn("localStorage / sessionStorage not supported by this browser")
   }	
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.actionRowCreateNewFrom=function(payload){
	var type = this.getQueryTableType();
	sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
		this.getCpbForm(),
		{
			key:"drafts",
			event_name:"create_new_from",
			variance:payload.rowData.key,
			type:type
		}
	);
};
sas.ui.cpb3.components.AdvancedTableComponent.prototype.actionRowDelete=function(payload){
	var rb = this.getCpbForm().getCpbService().getBundle();
	var links=payload.rowData.links;
	var deleteLink=_.find(links,function(link){return link.rel.toUpperCase()==="DELETE"});
	var that=this;
	var deleteCallBack=function(event){
		var action=event.getParameter("origin").getMessageDialogAction();
		if(action==="CANCEL"){return;}
		that.getCpbForm().getQueryAPI().deleteQueryTableEntry(deleteLink.uri,deleteLink.method)
		.done(function(data){
			that.doCPBRefresh();
		});
	}
	var objLabel=this.getCPBControl().getMeta().bizObjLabel;
	var rowName=payload.rowData.name;
	if(objLabel && rowName){
		var deleteTitleKey="queryTable.action.delete.title.fmt.txt";
		var deleteTitleArg=objLabel;
		var deleteConfirmKey="queryTable.action.delete.confirmation.fmt.txt";
		var deleteConfirmArgs=[objLabel,rowName];
		sas.hc.m.MessageDialog.warning({
			text:rb.getText(deleteConfirmKey,deleteConfirmArgs),
			title:rb.getText(deleteTitleKey,deleteTitleArg),
			buttonActions:[sas.hc.m.MessageDialogAction.Delete,sas.hc.m.MessageDialogAction.Cancel],
			buttonPress:deleteCallBack
		});
	}else{
		deleteTitleKey="queryTable.action.delete.unknown.title.txt";
		deleteConfirmKey="queryTable.action.delete.unknown.confirmation.txt";
		sas.hc.m.MessageDialog.warning({
			text:rb.getText(deleteConfirmKey),
			title:rb.getText(deleteTitleKey),
			buttonActions:[sas.hc.m.MessageDialogAction.Delete,sas.hc.m.MessageDialogAction.Cancel],
			buttonPress:deleteCallBack
		});
	}
};
//STATICS
sas.ui.cpb3.components.AdvancedTableComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.components.AdvancedTableComponent._createSettings(fieldJson, modelId, cpbForm);
	return sas.ui.cpb3.components.AdvancedTableComponent._createComponent(settings,sas.ui.cpb3.components.AdvancedTableComponent);
};
sas.ui.cpb3.components.AdvancedTableComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=$.extend(
		sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm),
		{
			meta : fieldJson.table_meta,
			dataUrl : fieldJson.dataUrl,
			metaUrl : fieldJson.metaUrl,
			visibleRows : fieldJson.visibleRows,
			queryTableType : fieldJson.queryTableType,
			relatedQueryTableType : fieldJson.relatedQueryTableType,
			showCreateButton: fieldJson.showCreateButton,
			showCreateFromAction: fieldJson.showCreateFromAction,
			showActionColumn: fieldJson.showActionColumn,
			hasCreatePermission: fieldJson.hasCreatePermission,
			allowRowClickNavigation: fieldJson.allowRowClickNavigation,
			formattingKey: sas.ui.cpb3.CpbUtil.buildFormattingKey(cpbForm,fieldJson.type,fieldJson.cpbName),
			quickLinkTypesForCreate: fieldJson.quickLinkTypesForCreate
		}
	);
	return settings;
}
sas.ui.cpb3.components.AdvancedTableComponent._createComponent=function(settings,constructor) {
	var def=$.Deferred();
	if(!settings.meta){//no meta provided, load meta via metaUrl
		$.getJSON(settings.metaUrl,function(json){
			settings.meta=json;
			var advanceTable = new constructor(settings);
			def.resolve(advanceTable);
		});
	}else{
		def.resolve(new constructor(settings));
	}
	return def;
};