jQuery.sap.declare("sas.ui.cpb3.components.CPBFormComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.ui.cpb3.CPBForm");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");

/**
 * CPBForm Component 
 */
sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.components.CPBFormComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
      	  serviceId:{type:"string"},
      	  data:{type:"any"}
        }
    },
    constructor:function(settings){
 		this.supportsLayout=true;
 		this.supportsLabel=false;
		this.supportsAlign=false;
		this.neverRefreshed=true;
 		var parentCpbForm=settings.cpbForm;
 		var serviceId=parentCpbForm.getServiceId();
 		var that=this;
 		this.comp=new sas.ui.cpb3.CPBForm({
 			cpbParent:parentCpbForm,
 			serviceId:serviceId,
 			data:sas.ui.cpb3.model.CpbModelManager.bindData('/')
 		}).addStyleClass("cpbCpbFormComponent");
 		var cpbModelManager=this.comp.getCpbService().createModelManagerObject(this.comp);
 		var blankData={};
 		blankData[sas.ui.cpb3.model.CpbModelManager._nameLabels]={};
 		blankData[sas.ui.cpb3.model.CpbModelManager._nameExps]={};
 		blankData[sas.ui.cpb3.model.CpbModelManager._nameStructure]=[];
 		blankData[sas.ui.cpb3.model.CpbModelManager._nameObservers]={};
 		blankData[sas.ui.cpb3.model.CpbModelManager._nameData]={};
 		blankData[sas.ui.cpb3.model.CpbModelManagerObject._nameChoices]={};
 		blankData[sas.ui.cpb3.model.CpbModelManagerObject._nameMessages]={};
 		blankData[sas.ui.cpb3.model.CpbModelManagerObject._nameNestedSaves]={};
 		blankData[sas.ui.cpb3.model.CpbModelManagerObject._nameNestedDeletes]={};
 		blankData[sas.ui.cpb3.model.CpbModelManagerObject._nameClone]={};
 		blankData[sas.ui.cpb3.model.CpbModelManagerObject._nameMisc]={};
 		blankData[sas.risk.rgf.data.RGFModelManagerObject._nameWorkflow]={};
		cpbModelManager.refresh(blankData,this.comp);
 	   sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.comp,"enabled");
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 	 },
 	 renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.ui.cpb3.components.CPBFormComponent);
sas.ui.cpb3.components.CPBFormComponent.prototype.getComponent = function() {
	return this.comp;
};
sas.ui.cpb3.components.CPBFormComponent.prototype._createComponent = function() {
   return this.getComponent();
};
sas.ui.cpb3.components.CPBFormComponent.prototype.setCpbVisible=function(value) {
	this.setProperty("cpbVisible",value);
	if (this.comp) this.comp.setVisible(value);
	if(value==null){
		this.isHidden=true;
	}
};
sas.ui.cpb3.components.CPBFormComponent.prototype.setCPBValue=function(value) {
	this.setCpbValue(value);
};
sas.ui.cpb3.components.CPBFormComponent.prototype.setCpbValue=function(value) {
	var prevValue=this.getProperty("cpbValue");
	this.setProperty("cpbValue",value);
	if(!(jQuery.sap.equal(prevValue,value))){
		this.refreshMode="edit";
		this.invalidate();
	}
	if(this.isHidden){
		this.isHidden=false;
		this._refreshEdit();
	}
	else if (this.neverRefreshed && this.getCpbVisible()) {
		this._refreshEdit();
		this.refreshMode=null;
	}

};
sas.ui.cpb3.components.CPBFormComponent.prototype.setData=function(values) {
	var prevValue=this.getProperty("data");
	var key=this.getCpbValue();
	this.setProperty("data",values);
	if( key &&
		values &&
		values[key] &&
		!(jQuery.sap.equal(prevValue,values)) && 
		((!prevValue && values) || 
		(prevValue && _.differenceObj(prevValue,values,jQuery.sap.equal).length))){		
		this.refreshMode="edit";
		this._refreshEdit();
	}
};
sas.ui.cpb3.components.CPBFormComponent.prototype.getDescendantControlById=function(screenId,controlId,key) {
	var control=this.comp.getCPBControlById(controlId);
	this.refreshMode="error";
	if(key){
		var split=key.split(".");
		return control.getDescendantControlById(split.shift(),Number.parseInt(split.shift()),split.join("."));
	}
	return {control:control,screenId:screenId};
};
sas.ui.cpb3.components.CPBFormComponent.prototype.onBeforeRendering=function() {
	if(this.refreshMode==="edit"){
		this._refreshEdit();
	}else if(this.refreshMode==="error"){//error
		this._refreshError();
	}
};
sas.ui.cpb3.components.CPBFormComponent.prototype._refreshError=function(json) {
	var screenId=this.getCpbValue();
	json=json||{};
	var cpbModelManager=this.comp.getCpbModelManager();
	var uiContext=this.getData()[this.getCpbValue()].data;
	json.cpb_data=json.cpb_data||uiContext;//cpbModelManager.getUIContext();
	json.cpb_messages=json.cpb_messages||{};
	json.cpb_messages.errored=json.cpb_messages.errored||[];
	var rootCPBForm=this.comp.getRootCPBForm();
	_.each(rootCPBForm.getCpbModelManager().getErrored(screenId),function(controlId){
		json.cpb_messages.errored.push(controlId);
	});
	cpbModelManager.applyErrors(json);
};
sas.ui.cpb3.components.CPBFormComponent.prototype._refreshEdit=function() {
	//if(!this.getCpbVisible()){return;}
	// set the screen id
	this.neverRefreshed=false;
	var screenId=this.getCpbValue();
	if(!screenId){
		this.comp.setVisible(false);
		return;
	}
	this.comp.setVisible(this.getCpbVisible());
	this.comp.setScreenId(screenId);
	// clean new model manager
	var cpbModelManager=this.comp.getCpbService().createModelManagerObject(this.comp);
	this.comp.setCpbModelManager(cpbModelManager);
	// prepare the ui context overrides
	var values=this.getData()||{};
	if(_.isString(values)){
		values=JSON.parse(values);
	}
	if(!(values[screenId]) || !(values[screenId].data)){
		values[screenId]=values[screenId]||{};
		values[screenId].data={SCREENID:screenId};
	}
	this.setProperty("data",values);
	var parentUiCtxt=this.getCpbForm().getCpbModelManager().getUIContext();
	var overrides=$.extend({},values[screenId].data,{PARENT:parentUiCtxt});
	var that=this;
	this.comp.setBusy(true);
	this.comp.removeAllItems();
	this.comp.getQueryAPI().loadQueryTableModels(screenId,overrides,values[screenId].screen).done(function(json){
		json.cpb_data.SCREEN_READ_ONLY|=json.cpb_data.PARENT.SCREEN_READ_ONLY;
		delete json["cpb_data"]["PARENT"];
		json["cpb_misc"]=json["cpb_misc"]||{};
		json["cpb_misc"].rebuild=true;
		json["cpb_workflow"]={};//so that it doesnt accidentally find the parent one and clear it on refresh
		values[screenId].data=json.cpb_data;
		json.cpb_messages.errored=json.cpb_messages.errored||[];
		var rootCPBForm=that.comp.getRootCPBForm();
		_.each(rootCPBForm.getCpbModelManager().getErrored(screenId),function(controlId){
			json.cpb_messages.errored.push(controlId);
		});
		cpbModelManager.refresh(json,that.comp);
		if(that.comp.getRequiredFieldIds().length){//check and see if the nested form has required fields and mark the cpbform comp as required
			that.setCpbRequired(true);
		}
		delete that.refreshMode;
		that.comp.setBusy(false);
	});
};
//STATICS
sas.ui.cpb3.components.CPBFormComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	settings.data=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.dataField);
	settings.cpbValue=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName);
	return new sas.ui.cpb3.components.CPBFormComponent(settings);
};
