jQuery.sap.declare("sas.ui.cpb3.components.DependentObjectChooserComponent");
jQuery.sap.require("sas.ui.cpb3.components.ObjectChooserComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.utils.DialogManager");
jQuery.sap.require("sas.ui.cpb3.widget.InputField");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.StringFilter");
jQuery.sap.require("sas.risk.slimtable.widget.Item");
jQuery.sap.require("sas.risk.common.widgets.Button");

sas.ui.cpb3.components.ObjectChooserComponent.extend("sas.ui.cpb3.components.DependentObjectChooserComponent",{
    metadata:{
        properties:{
  			  dependentDataUrl:{type:"string"},
  			  linkTypeId:{type:"string"},
  			  side:{type:"string"}
        }
    },
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});
sas.ui.cpb3.components.DependentObjectChooserComponent.prototype.setDependentDataUrl=function(value){
	this.setProperty("dependentDataUrl",value)
	if(this.built){
		this.refreshDependants();
	}
};
sas.ui.cpb3.components.DependentObjectChooserComponent.prototype.onBeforeRendering=function(){
	this.refreshDependants();
	this.build=true;
};
sas.ui.cpb3.components.DependentObjectChooserComponent.prototype.refreshDependants=function(){
	var that=this;
	var side=this.getSide();
	var sideNumber=!!(side&&side==="businessObject2")?1:0;
	var linkTypeId=this.getLinkTypeId();
	var ssc=this.getCpbForm().getServiceId().toUpperCase();
	this.dependants=[];
	$.getJSON(this.getDependentDataUrl(),function(result){
		_.each(result.items,function(row){
			that.dependants.push("hasObjectLinkTo('"+ssc+"','"+linkTypeId+"',"+row.key+","+sideNumber+")");
		});
	});
	if(!this.dependants.length){
		this.dependants.push("hasObjectLinkTo('"+ssc+"','"+linkTypeId+"',-222,"+sideNumber+")");
	}
};
sas.ui.cpb3.components.DependentObjectChooserComponent.prototype.buildFilter = function(value){
	var filter=null;
	if(value){
		filter=new sas.risk.common.widgets.spreadsheet.filter.StringFilter(this.getNameAttr(),"contains",value);
	}
	filter=filter?filter.toString():"";
	if(this.dependants&&this.dependants.length){
		if(this.dependants.length===1){
			if(filter){
				filter="and("+filter+","+this.dependants[0]+")";
			}else{
				filter=this.dependants[0];
			}
		}else{
			if(filter){
				filter="and("+filter+",or("+this.dependants.join(",")+"))";
			}else{
				filter="or("+this.dependants.join(",")+")";
			}
		}
	}
	return filter;
};
//ACTIONS
sas.ui.cpb3.components.DependentObjectChooserComponent.prototype.onButtonPress=function(e){
	var rb=this.getCpbForm().getCpbService().getBundle();
	var dataUrl=this.getDataUrl();
	var hasFilter=!!(dataUrl.indexOf("?")!==-1);
	sas.risk.common.utils.DialogManager.open("sas.ui.cpb3.components.ObjectChooserDialog",{
		isSpreadsheet:true,
		metaUrl:this.getMetaUrl(),
		dataUrl:dataUrl+(hasFilter?"&":"?")+"filter="+this.buildFilter(),
		title:this.getDialogTitle()||rb.getText("objectChooser.dialog.title"),
		cancelButtonTxt:rb.getText("objectChooser.cancelBtn.txt")
	},$.proxy(this.selectAction,this),this);
};
//STATICS
sas.ui.cpb3.components.DependentObjectChooserComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.components.ObjectChooserComponent._createSettings(fieldJson,modelId,cpbForm);
	settings.dependentDataUrl=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.dependentDataUrlField);
	settings.side=fieldJson.side;
	settings.linkTypeId=fieldJson.linkTypeId;
	return new sas.ui.cpb3.components.DependentObjectChooserComponent(settings);
};