jQuery.sap.declare("sas.ui.cpb3.components.DiagramComponent");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.ui.cpb3.widget.diagram.FlowDiagram");
jQuery.sap.require("sas.ui.cpb3.widget.diagram.FlowTemplates");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sap.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.ui.cpb3.widget.diagram.DiagramFactory");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.components.DiagramComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata : {
		properties : {
			options:{type:"any"},
			diagramId:{type:"string"}
		}
	},
	constructor : function(settings) {
		this.supportsLayout=true;
		var that=this;
		var options=_.reduce(settings.options,function(memo,value,key){
			if(key.indexOf("callback")!==-1){
				memo[key]=$.proxy(that.execCallback,that,value[0]);
			}else{
				memo[key]=value;
			}
			return memo;
		},{});
		var optionsPlus={
			diagramId:settings.diagramId,
			options:options
		};
		this.cpbDiagram=sas.ui.cpb3.widget.diagram.DiagramFactory.getInstance().create(optionsPlus);
		sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbValues", this.cpbDiagram, "values");
		sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbValue", this.cpbDiagram, "value");
		this.cpbDiagram.attachObjectSingleClicked($.proxy(this.updateUICtxtVariable,this));
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
		this.cpbDiagram.setBundle(this.getCpbForm().getCpbService().getBundle());
		this.addStyleClass("cpbSuperWide");
	},
	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.ui.cpb3.components.DiagramComponent);
sas.ui.cpb3.components.DiagramComponent.prototype.setCpbValues = function(values){
	console.log("cpbValues : ",values);
	this.setProperty("cpbValues",values);
};
sas.ui.cpb3.components.DiagramComponent.prototype.updateUICtxtVariable = function(e) {
	var varName=this.getCpbName()+"_SELECTED";
	var g=e.getParameter("graphObject");
	var value=g.data&&g.data.text;
	while(g.panel&&!value){
		g=g.panel;
		value=g.data&&g.data.text;
	}
	if(value){
		this.getCpbForm().getCpbModelManager().getUIContextModel().setProperty("/"+varName,value);
	}
};
sas.ui.cpb3.components.DiagramComponent.prototype.wireOnChange = function(onChangeFn) {
	if(onChangeFn){
		this.cpbDiagram.attachObjectSingleClicked(onChangeFn);
	}
};
sas.ui.cpb3.components.DiagramComponent.prototype.setCPBValue = function(value) {
	this.setCpbValue(value);
};
sas.ui.cpb3.components.DiagramComponent.prototype.execCallback = function() {
	var payload=Array.prototype.slice.call(arguments);
	var funcName=payload.shift();
	var def=$.Deferred();
	this.setBusy(true);
	var cpbForm=this.getCpbForm();
	var cpbService=cpbForm.getCpbService();
	var screenId=cpbForm.getCpbModelManager().getUIContextValue("SCREENID");
	var that=this;
   cpbService.getCpbAPI().evalExpression(funcName+"()", {
   	diagram_options:this.getOptions(),
   	diagram_payload:payload,
   	uictxt:cpbForm.getCpbModelManager().getUIContext(),
   	SCREENID:screenId
	}).done(function(data){
   	that.setBusy(false);
   	def.resolve(data.result);
   });
   return def;
};
sas.ui.cpb3.components.DiagramComponent.prototype.setCPBValues = function(values) {
	this.setCpbValues(values);
};
sas.ui.cpb3.components.DiagramComponent.prototype._createComponent = function() {
	this.addStyleClass("cpbFlowDiagComponent");
	this.addItem(this._createHeader());
	this.addItem(this.cpbDiagram);
	return this.cpbDiagram;
};
sas.ui.cpb3.components.DiagramComponent.prototype._createHeader = function(){
	var rb = sas.risk.rgf.util.RGFUtil.getBundle();
	var that=this;
	zoomInButton = new sas.hc.m.Button({
   	icon: sas.icons.HC.ZOOMIN,
      tooltip: rb.getText("flowDiagram.zoomInButton.txt"),
      press: function () {
          var commandHandler = that.cpbDiagram.getDiagram().commandHandler;
          if (commandHandler.canIncreaseZoom()) {
              commandHandler.increaseZoom();
          }
      }
	});
	zoomOutButton = new sas.hc.m.Button({
      icon: sas.icons.HC.ZOOMOUT,
      tooltip: rb.getText("flowDiagram.zoomOutButton.txt"),
      press: function () {
          var commandHandler = that.cpbDiagram.getDiagram().commandHandler;
          if (commandHandler.canDecreaseZoom()) {
              commandHandler.decreaseZoom();
          }
      }
	});
	zoomFitButton = new sas.hc.m.Button({
      icon: sas.icons.HC.ZOOMTOFIT,
      tooltip: rb.getText("flowDiagram.zoomFitButton.txt"),
      press: function () {
          var commandHandler = that.cpbDiagram.getDiagram().commandHandler;
          commandHandler.zoomToFit();
      }
	});
	var zoomToolbar = new sas.hc.m.Toolbar({
      content: [
          new sap.m.ToolbarSpacer(),
          zoomInButton,
          zoomOutButton,
          zoomFitButton
      ]
	});
	var zoomToolbarLayout = new sas.risk.common.layout.flexlayout.FlexLayout({
		justifyContent: sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.FlexEnd,
		items: [zoomToolbar],
		width: "100%"
	}).addStyleClass("cpbFlowDiagToolbar");
	return zoomToolbarLayout; 
};
sas.ui.cpb3.components.DiagramComponent.prototype._createLabel = function() {
	// Overridden. Label will be part of widget and show in the window shade
	// section.
};
//STATICS
sas.ui.cpb3.components.DiagramComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	settings.diagramId=fieldJson.diagramId;
	settings.options=fieldJson.options||{};
	return new sas.ui.cpb3.components.DiagramComponent($.extend(
		settings,
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
			cpbValues : sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbValues)
		}
	));
};