jQuery.sap.declare("sas.ui.cpb3.components.DualSelector");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.hc.ui.commons.dualselector.DualSelector");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.hc.m.MessageDialog");
/**
 * Dual Selector Component 
 */
sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.components.DualSelector", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
      		maxSelectedItems:{type:"int",defaultValue:-1},
      	  	availableLabel:{type:"string"},
			selectedLabel:{type:"string"},
			enableSearch:{type: "boolean", defaultValue: false}
        }
    },
    constructor:function(args){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		this.supportsAlign=true;
 		var that=this;
 		this.compAvailableModel=new sap.ui.model.json.JSONModel();
 	  	this.comp=new sas.hc.ui.commons.dualselector.DualSelector({
 	   	responsive:false,
		contextPath:"/values",	
        showSearch: args && args.enableSearch ? true : false,
 	   	labelField:"label",
 			itemsMovedTo:function(){
 				that.syncComponent();
 			},
 			selectedItemsReordered:function(){
 				that.syncComponent();
 			},
 			dualSelectorCreationComplete:function(){
 				that.dsReady=true;
 				that.syncWidget();
 			}
 	   });
 	   this.comp.setModel(this.compAvailableModel);
 	   sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.comp,"enabled");
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.addStyleClass("cpbDualSelector");
 		//MAX SELECTED ITEMS override
 	   var rb=this.getCpbForm().getCpbService().getBundle();
 	   var orig=this.comp.onBeforeRendering;
 	   var origMove=sas.hc.ui.commons.dualselector.dsMove.MoveManager.prototype.moveItemsFromAvailableToSelectedList;
 	   this.comp.onBeforeRendering=function(){
 	   	orig.apply(this,arguments);
 	   	if(origMove===this.oMoveManager.moveItemsFromAvailableToSelectedList){
 	   		this.oMoveManager.moveItemsFromAvailableToSelectedList=function(selectedList){
 	 	   		var futureTargetLength = this.selectedList.getItems().length + selectedList.length;
 	 	   		var maxSelectedItems=that.getMaxSelectedItems();
 	 	   		if(maxSelectedItems>-1&&futureTargetLength>maxSelectedItems){
 	 	   			sas.hc.m.MessageDialog.warning({
 	 	   				text:rb.getText("dualSelector.selection.exceeds.maximum.fmt.txt",[maxSelectedItems]), 
 	 	   				title:rb.getText("dualSelector.selection.warning.txt"), 
 	 	   				buttonActions:[sas.hc.m.MessageDialogAction.Ok]
 	 	   			});
 	 	   		}else{
 	 	   			origMove.apply(this,arguments);
 	 	   		}
 	   		}
 	   	}
 	   };
 	 },
 	 renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.ui.cpb3.components.DualSelector);

sas.ui.cpb3.components.DualSelector.prototype.syncWidget=function(){
	if(!this.dsReady){return;}
	var choicesModel=this.getModel("cpb_choices");
	var values=this.getCpbValues()||(choicesModel&&choicesModel.getProperty("/"+this.getCpbName()));
	var value=this.getCpbValue();
	var available=_.filter(values,function(valueLabel){
		return !value||value.indexOf(valueLabel.value)===-1;
	});
	var selected=_.filter(values,function(valueLabel){
		return value&&value.indexOf(valueLabel.value)!==-1;
	});
	selected=_.sortBy(selected,function(valueLabel){
		return value.indexOf(valueLabel.value);
	});
	this.compAvailableModel.setProperty("/values",available);
	this.comp.getTargetList().getModel().setProperty("/itemList",selected);
};
sas.ui.cpb3.components.DualSelector.prototype.syncComponent=function(){
	var labelValues=this.comp.getTargetList().getModel().getProperty("/itemList");
	var value=_.pluck(labelValues,"value");
	this.setProperty("cpbValue",value);
};
/**
 * This is called by the CPB engine when it is time to create the component.
 * At this point the constructor,init,applySettings have all been called, but 
 * the model has not been populated yet with data.
 * Here we create the inner widget and tie in some of the components properties directly
 * to the inner widget.
 */
sas.ui.cpb3.components.DualSelector.prototype._createComponent = function() {
	if(this.getAvailableLabel()){
		this.comp.setSourceLabel(this.getAvailableLabel());
	}
	if(this.getSelectedLabel()){
		this.comp.setTargetLabel(this.getSelectedLabel());
	}
   return this.comp;
};
/**
 * OnChangeFn is a function that has the on-change info that was given in the screen def xml for this component.
 * You must override wireOnChange if the object returned by _createComponent does not have a "change" event.
 * Here we are attaching it directly on the inner widget
 */
sas.ui.cpb3.components.DualSelector.prototype.wireOnChange=function(onChangeFn){
	if(onChangeFn){
		this.comp.attachItemsMovedTo(onChangeFn);
		this.comp.attachSelectedItemsReordered(onChangeFn);
	}
};
/**
 * setCPBValue is called by the CPB engine when this component is involved in an on-change
 * event and it has a set-value on it. Here we are shuttling the value directly to the inner widget 
 */
sas.ui.cpb3.components.DualSelector.prototype.setCPBValue = function(value) {
	this.setCpbValue(value);
};
sas.ui.cpb3.components.DualSelector.prototype.setCpbValue = function(val){
	this.setProperty("cpbValue",val);
	this.syncWidget();
};
/**
 * setCPBValues is called by the CPB engine when this component is involved in an on-change
 * event and it has a set-values on it. Here we are shuttling the value directly to the inner widget 
 */
sas.ui.cpb3.components.DualSelector.prototype.setCPBValues = function(value) {
	this.setCpbValues(value);
};
sas.ui.cpb3.components.DualSelector.prototype.setCpbValues = function(val){
	this.setProperty("cpbValues",val);
	this.syncWidget();
};
//STATICS
sas.ui.cpb3.components.DualSelector._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	return $.extend(
			settings,
			{
				cpbValues:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbValues),
				maxSelectedItems:fieldJson.maxSelectedItems,
				availableLabel:fieldJson.availableLabel,
				selectedLabel:fieldJson.selectedLabel,
				cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
				enableSearch:fieldJson.enableSearch
			}
	);
};
sas.ui.cpb3.components.DualSelector.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.components.DualSelector._createSettings(fieldJson,modelId,cpbForm);
	return new sas.ui.cpb3.components.DualSelector(settings);
};
