jQuery.sap.declare("sas.ui.cpb3.components.ExprEditorComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.utils.DialogManager");
jQuery.sap.require("sas.ui.cpb3.widget.InputField");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.StringFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.NumberFilter");
jQuery.sap.require("sas.risk.slimtable.widget.Item");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.AndFilter");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.components.ExprEditorComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
  			  icon : {type : "sap.ui.core.URI", group : "Appearance", defaultValue : sas.icons.HC.SEARCH},
  			  placeholderText:{type:"string"},
  			  actionTooltip:{type:"string"},
  			  dialogTitle:{type:"string"},
  			  meta:{type:"any"}
        },
        events:{
      	  change:{}
        }
    },
 	constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		this.supportsAlign=true;
 		this.mainLayout=new sas.risk.common.layout.flexlayout.FlexLayout();
 		this.mainLayout.setWidth("100%");
 		sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.mainLayout,"enabled");
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Baseline);
 		this.addStyleClass("cpbFieldLikeComponent");
 		this.addStyleClass("cpbExprEditor");
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.ui.cpb3.components.ExprEditorComponent);

sas.ui.cpb3.components.ExprEditorComponent.prototype.setCPBValue=function(value){
	this.setCpbValue(value);
};
sas.ui.cpb3.components.ExprEditorComponent.prototype.setCpbValue=function(value){
	if(value && _.isString(value)){
		try{
			value=JSON.parse(value);
			
		}catch(e){
			console.error("Unable to parse JSON " + value )
		}
	}
    /*RGF IFRS-17 fixes*/
	if (value && _.isObject(value)){
        this.field.setValue(value.rawExpression);
        if (value.validationErrors && !value.validationErrors.hasErrors){
            // how do we set an error icon inside.. do we want to??
        }
    }
	this.setProperty("cpbValue",value);
};

sas.ui.cpb3.components.ExprEditorComponent.prototype._createComponent = function() {
	var rb=this.getCpbForm().getCpbService().getBundle();
   this.field = new sas.ui.cpb3.widget.InputField({
   	enabled:false
	});
	this.button=new sas.risk.common.widgets.Button({
		icon:this.getIcon(),
		press:$.proxy(this.onButtonPress,this)
	});
   this.mainLayout.removeAllItems();
   this.mainLayout.addItem(this.field);
   this.mainLayout.addItem(this.button);
   return this.mainLayout;
};
sas.ui.cpb3.components.ExprEditorComponent.prototype.wireOnChange=function(onChangeFn){
	if(onChangeFn){
		this.attachChange(onChangeFn);
	}
};
sas.ui.cpb3.components.ExprEditorComponent.prototype.selectAction=function(e){
	if(e.getParameter("isOk")){
		var val=e.getParameter("value");
		this.setCpbValue(val);
	   this.fireChange({newValue:val});
	}
};
//ACTIONS
sas.ui.cpb3.components.ExprEditorComponent.prototype.onButtonPress=function(e){
	var rb=this.getCpbForm().getCpbService().getBundle();
	sas.risk.common.utils.DialogManager.open("sas.risk.common.widgets.spreadsheet.control.ExpressionEditorDialog",{
		meta:this.getMeta(),
		value:this.getCpbValue(),
		callback:$.proxy(this.selectAction,this),
		title:this.getDialogTitle()||rb.getText("spreadsheet.cell.ee.title.txt")
	},$.proxy(this.selectAction,this),this);
};
//STATICS
sas.ui.cpb3.components.ExprEditorComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.components.ExprEditorComponent._createSettings(fieldJson,modelId,cpbForm);
	return new sas.ui.cpb3.components.ExprEditorComponent(settings);
};
sas.ui.cpb3.components.ExprEditorComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	var icon=(fieldJson.icon)?sas.icons.HC[fieldJson.icon]:sas.icons.HC.SEARCH;
	if(fieldJson.metaField){
		settings.meta=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.metaField);
	}
	return $.extend(
		settings,
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
			dialogTitle:fieldJson.dialogTitle,
			icon:icon
		}
	);
};