jQuery.sap.require("sas.risk.slimtable.plugins.Pager");
jQuery.sap.require("sas.risk.slimtable.plugins.Search");
jQuery.sap.require("sas.risk.slimtable.plugins.Filter");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsview("sas.ui.cpb3.components.MultiObjectChooserDialog", {
	getControllerName : function() {
       return "sas.ui.cpb3.components.MultiObjectChooserDialog";
   },
   createContent : function(controller) {
      var viewData=this.getViewData();
      var innerLayout=this.createSpreadsheetLayout();
      var layout = new sas.hc.ui.commons.layout.VerticalLayout({
         width : "100%",
         content : [ innerLayout]
      });
      this.createButtons(controller);
      return layout;
   },
   createButtons : function(controller) {
   	var rb=sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
      var viewData=this.getViewData();
      this.okButton = new sas.risk.common.widgets.Button({
      	text: rb.getText("ok.button.txt"),
      	enabled:false,
      	design: sas.hc.m.Button.Design.Primary,
      	press:$.proxy(controller.actionOk,controller) 
      });
      this.cancelButton = new sas.risk.common.widgets.Button({
      	text: viewData.cancelButtonTxt,
      	press:$.proxy(controller.actionCancel,controller) 
      });
   },
	getDialogSettings : function() {
		var dialog=this.getDialog();
		dialog.setBusyIndicatorDelay(0);
		dialog.setBusy(true);
		return {
			contentWidth:"80%",
			beginButton : this.okButton,
			endButton : this.cancelButton
		};
	},
	createSpreadsheetLayout:function(){
		var viewData=this.getViewData();
    	var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
    		minHeight:"400px",
   		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
   		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Stretch,
   		items:[]
   	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
   		alignSelf:sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.Stretch
   	}));
    	var that=this;
		$.getJSON(viewData.metaUrl,function(metas){
			var meta=that.loadMeta(metas);
			var spreadsheet=new sas.risk.common.widgets.spreadsheet.Spreadsheet({
				meta:meta,
				dataUrl:viewData.dataUrl,
				rowSelectionChange:function(e){
					var spreadsheet=e.getSource();
					var table=spreadsheet.getSelectedSheetTable();
					var indices=table.getSelectedIndices();
					if(indices.length){
						that.value=[];
						_.each(indices,function(index){
							var row=table.getContextByIndex(index).getObject();
							that.value.push(row);
						});
						that.okButton.setEnabled(true);
					}else{
						that.okButton.setEnabled(false);
					}
				}
			});
         that.spreadsheet=spreadsheet;			
			var toolbar1=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar");
			var toolbar2=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar");
			var footer=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar");
			var menu=new sas.hc.m.Menu();
			var pieces={spreadsheet:spreadsheet,toolbar1:toolbar1,toolbar2:toolbar2,footer:footer,menu:menu};
			toolbar2.addItem(new sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch({spreadsheet:pieces.spreadsheet}));
			toolbar2.addItem(new sas.hc.m.ToolbarSpacer());
			toolbar2.addItem(new sas.risk.common.widgets.spreadsheet.toolbar.Filters({spreadsheet:pieces.spreadsheet}));
			toolbar2.addItem(new sas.hc.m.ToolbarSpacer());
			footer.addItem(new sas.hc.m.ToolbarSpacer());
			footer.addItem(new sas.risk.common.widgets.spreadsheet.toolbar.Totals({spreadsheet:pieces.spreadsheet}));
			layout.addItem(pieces.toolbar1);
			layout.addItem(pieces.toolbar2);
			layout.addItem(pieces.spreadsheet);
			layout.addItem(pieces.footer);
			that.getDialog().setBusy(false);
		});
		return layout;
	},
	loadMeta:function(metas){
		metas=_.isArray(metas)?metas:[{id:"abc",meta:metas}];
		var metaArray=[];
		jQuery.sap.require("sas.ui.cpb3.components.SpreadsheetComponent");
		_.each(metas,function(metaInfo){
			metaInfo.meta=sas.ui.cpb3.components.SpreadsheetComponent.convertMeta(metaInfo.meta);
			var meta=new sas.risk.common.widgets.spreadsheet.SpreadsheetMeta(metaInfo.meta);
			meta.getTableOptions()["selectionMode"]="MultiToggle";
			metaArray.push({id:metaInfo.id,meta:meta});
		});
		return metaArray;
	}
});
sap.ui.controller("sas.ui.cpb3.components.MultiObjectChooserDialog", {
	actionCancel : function(){
		this.getView().getDialog().close();
	},
   actionOk: function(){   	
      var view=this.getView();
      if (view.getViewData() && view.getViewData().collection) {
         view.getDialog().setBusy(true);
         var arrayOfKeys=_.pluck(view.value,"key");
         var meta=view.spreadsheet.getMeta()[0].meta;
         var fields = meta.getColumnIds().join(","); 	 
         sas.risk.rgf.api.CommonAPI.searchCollection(view.getViewData().collection,"in(key," + arrayOfKeys + ")&fields=" + fields).done(function(results) {
   			_.each(results||[], function(item) {													//loop through the json results
      			var entry=_.find(view.value,function(val){return val.key===item.key;});//identify the spreadsheet row data
         		entry=$.extend(entry,item);
   			});
   			view.fireClosing({
   				item:view.value||[]
   			});
   			view.getDialog().setBusy(false);
   			view.getDialog().close();
         }).fail(function(){
         	view.getDialog().setBusy(false);
         });
      } else {
			view.fireClosing({
				item:view.value||[]
			});
			view.getDialog().close();
      }
   }
});    