jQuery.sap.declare("sas.ui.cpb3.components.ObjectChooserComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.utils.DialogManager");
jQuery.sap.require("sas.ui.cpb3.widget.InputField");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.StringFilter");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.NumberFilter");
jQuery.sap.require("sas.risk.slimtable.widget.Item");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.filter.AndFilter");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.components.ObjectChooserComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
  			  icon : {type : "sap.ui.core.URI", group : "Appearance", defaultValue : sas.icons.HC.SEARCH},
  			  placeholderText:{type:"string"},
  			  actionTooltip:{type:"string"},
  			  dialogTitle:{type:"string"},
  			  metaUrl:{type:"string"},
  			  dataUrl:{type:"string"},
  			  nameAttr:{type:"string",defaultValue:"name"},
  			  keyAttr:{type:"string",defaultValue:"key"},
  			  altKeyAttr:{type:"string"},
  			  fieldSize:{type:"int"}
        },
        events:{
      	  change:{}
        }
    },
 	constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		this.supportsAlign=true;
 		this.mainLayout=new sas.risk.common.layout.flexlayout.FlexLayout();
 		sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.mainLayout,"enabled");
 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Baseline);
 		this.addStyleClass("cpbFieldLikeComponent");
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.ui.cpb3.components.ObjectChooserComponent);

sas.ui.cpb3.components.ObjectChooserComponent.prototype.setCPBValue=function(value){
	this.setCpbValue(value);
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.setCpbValue=function(value){
	if(value && _.isString(value)){
		try{
			value=JSON.parse(value);
			
		}catch(e){
			console.error("Unable to parse JSON " + value )
		}
		
	}
	if (value && _.isObject(value)){
		if (value[this.getNameAttr()]  && (value.key != value[this.getNameAttr()])){
			this.field.setValue(value[this.getNameAttr()]);	
		}else {
			// if there is no name available. Fetch the latest name from the db
			var that = this;
			this.fetchItem(value).done(function(item){
				var latestVal = that.createItemLabel(item);
				if (latestVal){
					that.field.setValue(latestVal);
				}
			});
		}	
	}else{
		this.field.setValue("");
	}
		
	this.setProperty("cpbValue",value);
};

sas.ui.cpb3.components.ObjectChooserComponent.prototype._createComponent = function() {
	var rb=this.getCpbForm().getCpbService().getBundle();
   this.field = new sas.ui.cpb3.widget.InputField({
      showSuggestion: true,
      filterSuggests : false,
      suggest:$.proxy(this.onFieldSuggest,this),
      change:$.proxy(this.onFieldChange,this)
	});
   if (this.getFieldSize()) {
   	this.field.setSize(this.getFieldSize());
   }
	this.button=new sas.risk.common.widgets.Button({
		icon:this.getIcon(),
		press:$.proxy(this.onButtonPress,this)
	});
   this.mainLayout.removeAllItems();
   this.mainLayout.addItem(this.field);
   this.mainLayout.addItem(this.button);
   return this.mainLayout;
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.wireOnChange=function(onChangeFn){
	if(onChangeFn){
		this.attachChange(onChangeFn);
	}
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.selectAction=function(e){
	var item=e.getParameter("item");
	this.field.removeAllSuggestionItems();
	//this.field.setValue(this.createItemLabel(item));
	var val=this.transformValue(item);
	var nameAttr = this.getNameAttr();
	if (val[nameAttr] === undefined){
	   val[nameAttr] = item[nameAttr]||item.customFields[nameAttr]||item.key;
	}
	this.setCpbValue(val);
   this.fireChange({newValue:val});
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.transformValue=function(value){
	return value?{key:value[this.getKeyAttr()],name:this.createItemLabel(value), uri: this.buildRestURLForObject(value)}:value;
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.createItemLabel=function(item){
	return item[this.getNameAttr()]||item.customFields[this.getNameAttr()]||item.key;
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.buildFilter = function(value){
	if(value){
		return new sas.risk.common.widgets.spreadsheet.filter.StringFilter(this.getNameAttr(),"contains",value);
	}
	return null;
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.buildKeyFilter = function(value){
	if(value){
		if (value.key){
			return new sas.risk.common.widgets.spreadsheet.filter.NumberFilter(this.getKeyAttr(),"eq",value.key);
		} else if (this.getAltKeyAttr()){
			var altKeys=this.getAltKeyAttr().split(",");
			console.debug("Alternate keys =" + altKeys );
			var altKeysExist = _.reduce(altKeys, function(memo,altKey){
				  return  (memo && !!value[altKey]);  
			}, true);
			if (altKeysExist){
				console.debug("Alternate keys exist in value json");
				var andFilter = new sas.risk.common.widgets.spreadsheet.filter.AndFilter();
				 _.each(altKeys,function(altKey){
					return andFilter.addFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter(altKey,"eq",value[altKey]));
				}, new sas.risk.common.widgets.spreadsheet.filter.AndFilter());
				return andFilter;
			}else{
				console.debug("Cannot find Alternate keys  in value json");
			}
		} else if (value.sourceSystemCd && value.objectId){
			var andFilter=new sas.risk.common.widgets.spreadsheet.filter.AndFilter();
			andFilter.addFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter("sourceSystemCd","eq",value.sourceSystemCd));
			andFilter.addFilter(new sas.risk.common.widgets.spreadsheet.filter.StringFilter("objectId","eq",value.objectId));
			return andFilter;
		}else if (value.userId){
			return new sas.risk.common.widgets.spreadsheet.filter.StringFilter("userId","eq",value.userId);
		}
	}
	return null;
};

sas.ui.cpb3.components.ObjectChooserComponent.prototype.buildRestURLForObject = function (value) {
	var dataUrl = this.getDataUrl();
	if (dataUrl) {
		var hasParams = dataUrl.indexOf("?");
		if (hasParams > 0) {
			dataUrl = dataUrl.substring(0,hasParams);
		}
		var filter = this.buildKeyFilter(value);
		if (filter){
			dataUrl = dataUrl +  "?" + $.param({filter:filter.toString()});
			return dataUrl;
		}
	}
	return null;
}

sas.ui.cpb3.components.ObjectChooserComponent.prototype.fetchItem = function (value) {
	var def=$.Deferred();
	var dataUrl = this.buildRestURLForObject(value);
	if (dataUrl) {
			$.getJSON(dataUrl, function (results) {
				var data = results.items;
				if (_.isArray(data)){
					if (data.length > 1){
						console.error("Expecting 1 record for key but found " + data.length  + " records ");
						def.resolve(value);			
					}else if (data[0]){
						def.resolve(data[0]);
					}else{
						console.error("Unable to locate any record for url: "+ dataUrl);
						def.resolve(value);
					}
				}else{
					def.resolve(value);
				}
			}).fail(function(e){
				def.reject(e);
			});
	}else{
			def.resolve(value);
	}
	return def;
}


//ACTIONS
sas.ui.cpb3.components.ObjectChooserComponent.prototype.onFieldSuggest=function(e){
   var v = e.getParameter("suggestValue");
   var f = e.getSource();
   var dataUrl=this.getDataUrl();
   if(dataUrl){
   	var that=this;
   	var filter=this.buildFilter(v);
   	var url=this.getDataUrl();
   	var hasParams=url.indexOf("?")!==-1;
   	var dataUrl=this.getDataUrl()+(filter?(hasParams?"&":"?")+"filter="+filter:"");
      $.getJSON(dataUrl,function(results){
      	var data=results.items;
         f.removeAllSuggestionItems();
         _.each(data, function(item){
         	var suggestionItem=new sas.risk.slimtable.widget.Item({
               key: item.key,
               text: that.createItemLabel(item),
               payload: that.transformValue(item)
            });
            f.addSuggestionItem(suggestionItem);
         });
      });
   }
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.onFieldChange=function(e){
	if(this.field._oList && this.field.getSuggestionItems() && this.field.getSuggestionItems().length>0){ //If this change is coming from selecting a suggested item
		if(this.field._oList.getSelectedItem()){ //selection through key press
			var selItem=this.field._oList.getSelectedItem();
			var index=this.field._oList.indexOfItem(selItem);
			var selectedItem=this.field.getSuggestionItems()[index];
			this.setCpbValue(selectedItem.getPayload());
			this.fireChange({newValue:selectedItem.getPayload()});
		}else{
			var selection=$("#"+this.field._oList.getId()+">ul>li:hover");//selection through mouse click
			if(!selection || selection.length===0){//If user didn't select anything from the suggested items then we don't want to capture the user entered text as the value for user field
				this.fireChange({newValue:null});
				return;
			}else{//Hack to get the suggestion selected item
				var text=$("#"+this.field._oList.getId()+">ul>li:hover>.sapMLIBContent>.sapMSLITitleDiv>.sapMSLITitleOnly")[0].innerText;
				var suggestionItems=this.field.getSuggestionItems();
				var selectedSuggestionItem=_.find(suggestionItems,function(item){return text===item.getText();});
				this.setCpbValue(selectedSuggestionItem.getPayload());
				this.fireChange({newValue:selectedSuggestionItem.getPayload()});
			}
		}
	}else{
		this.setCpbValue(null);
		this.fireChange({newValue:null});
	}
};
sas.ui.cpb3.components.ObjectChooserComponent.prototype.onButtonPress=function(e){
	var rb=this.getCpbForm().getCpbService().getBundle();
	sas.risk.common.utils.DialogManager.open("sas.ui.cpb3.components.ObjectChooserDialog",{
		isSpreadsheet:true,
		metaUrl:this.getMetaUrl(),
		dataUrl:this.getDataUrl(),
		title:this.getDialogTitle()||rb.getText("objectChooser.dialog.title"),
		cancelButtonTxt:rb.getText("objectChooser.cancelBtn.txt")
	},$.proxy(this.selectAction,this),this);
};
//STATICS
sas.ui.cpb3.components.ObjectChooserComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.components.ObjectChooserComponent._createSettings(fieldJson,modelId,cpbForm);
	return new sas.ui.cpb3.components.ObjectChooserComponent(settings);
};
sas.ui.cpb3.components.ObjectChooserComponent._createSettings=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	var icon=(fieldJson.icon)?sas.icons.HC[fieldJson.icon]:sas.icons.HC.SEARCH;
	return $.extend(
		settings,
		{
			metaUrl:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.metaUrlField),
			dataUrl:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.dataUrlField),
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
			dialogTitle:fieldJson.dialogTitle,
			nameAttr:fieldJson.nameAttr,
			keyAttr:fieldJson.keyAttr,
			altKeyAttr:fieldJson.altKeyAttr,
			icon:icon,
			fieldSize:fieldJson.fieldSize
		}
	);
};