jQuery.sap.require("sas.risk.slimtable.plugins.Pager");
jQuery.sap.require("sas.risk.slimtable.plugins.Search");
jQuery.sap.require("sas.risk.slimtable.plugins.Filter");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsview("sas.ui.cpb3.components.ObjectChooserDialog", {
	getControllerName : function() {
       return "sas.ui.cpb3.components.ObjectChooserDialog";
   },
   createContent : function(controller) {
      var viewData=this.getViewData();
      var innerLayout=this.createSpreadsheetLayout();
      var layout = new sas.hc.ui.commons.layout.VerticalLayout({
         width : "100%",
         content : [ innerLayout]
      });
      this.createButtons(controller);
      return layout;
   },
   createButtons : function(controller) {
      var viewData=this.getViewData();
      this.cancelButton = new sas.risk.common.widgets.Button({
      	text: viewData.cancelButtonTxt,
      	press:$.proxy(controller.actionCancel,controller) 
      });
   },
	getDialogSettings : function() {
		var dialog=this.getDialog();
		dialog.setBusyIndicatorDelay(0);
		dialog.setBusy(true);
		return {
			contentWidth:"80%",
			endButton : this.cancelButton
		};
	},
	createSpreadsheetLayout:function(){
		var viewData=this.getViewData();
    	var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
    		minHeight:"400px",
   		flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
   		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Stretch,
   		items:[]
   	}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
   		alignSelf:sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.Stretch
   	}));
    	var that=this;
		$.getJSON(viewData.metaUrl,function(metas){
			var meta=that.loadMeta(metas);
			var spreadsheet=new sas.risk.common.widgets.spreadsheet.Spreadsheet({
				meta:meta,
				dataUrl:viewData.dataUrl,
				rowSelectionChange:function(e){
					var spreadsheet=e.getSource();
					var table=spreadsheet.getSelectedSheetTable();
					var indices=table.getSelectedIndices();
					if(indices.length){
						var row=table.getContextByIndex(indices[0]).getObject();
				      var fields = spreadsheet.getMeta()[0].meta.getColumnIds().join(",");
				      if (viewData.collection) {
							that.getDialog().setBusy(true);
					      sas.risk.rgf.api.CommonAPI.searchCollection(viewData.collection,"in(key," + row.key + ")&fields=" + fields).done(function(results) {
								_.each(results||[], function(item) {
					      		$.extend(row,item);
								});
								that.fireClosing({
									item:row
								});
								that.getDialog().setBusy(false);
								that.getDialog().close();
					      }).fail(function(){
								that.getDialog().setBusy(false);
					      });
				      } else {
							that.fireClosing({
								item:row
							});
							that.getDialog().close();
				      }
					}
				}
			});
			var toolbar1=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar");
			var toolbar2=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar");
			var footer=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar");
			var menu=new sas.hc.m.Menu();
			var pieces={spreadsheet:spreadsheet,toolbar1:toolbar1,toolbar2:toolbar2,footer:footer,menu:menu};
			toolbar2.addItem(new sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch({spreadsheet:pieces.spreadsheet}));
			toolbar2.addItem(new sas.hc.m.ToolbarSpacer());
			toolbar2.addItem(new sas.risk.common.widgets.spreadsheet.toolbar.Filters({spreadsheet:pieces.spreadsheet}));
			toolbar2.addItem(new sas.hc.m.ToolbarSpacer());
			footer.addItem(new sas.hc.m.ToolbarSpacer());
			footer.addItem(new sas.risk.common.widgets.spreadsheet.toolbar.Totals({spreadsheet:pieces.spreadsheet}));
			layout.addItem(pieces.toolbar1);
			layout.addItem(pieces.toolbar2);
			layout.addItem(pieces.spreadsheet);
			layout.addItem(pieces.footer);
			that.getDialog().setBusy(false);
		});
		return layout;
	},
	loadMeta:function(metas){
		metas=_.isArray(metas)?metas:[{id:"abc",meta:metas}];
		var metaArray=[];
		jQuery.sap.require("sas.ui.cpb3.components.SpreadsheetComponent");
		_.each(metas,function(metaInfo){
			metaInfo.meta=sas.ui.cpb3.components.SpreadsheetComponent.convertMeta(metaInfo.meta);
			var meta=new sas.risk.common.widgets.spreadsheet.SpreadsheetMeta(metaInfo.meta);
			meta.getTableOptions()["selectionMode"]="Single";
			metaArray.push({id:metaInfo.id,meta:meta});
		});
		return metaArray;
	}
});
sap.ui.controller("sas.ui.cpb3.components.ObjectChooserDialog", {
	actionCancel : function(){
		this.getView().getDialog().close();
	}
});    