jQuery.sap.declare("sas.ui.cpb3.components.SpreadsheetComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.Spreadsheet");
jQuery.sap.require("sas.hc.ui.table.TableRenderer");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetCell");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetMeta");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetColumnFactory");
jQuery.sap.require("sas.hc.ui.model.PageableJSONModel");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.model.SpreadsheetRestService");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.model.SpreadsheetPageableJSONModel");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.model.SpreadsheetPageableJSONTreeModel");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.FetchCriteria");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.Search");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.BroadSearch");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.Filters");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.View");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.Views");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.Totals");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.CreateNewButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.EditButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.RefreshButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.DeleteButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.AddRowButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.UndoButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.toolbar.SaveButton");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetFormulaContext");
jQuery.sap.require("sas.risk.common.widgets.spreadsheet.SpreadsheetToolbarFactory");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.components.SpreadsheetComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata : {
		properties : {
			meta:          {type:"any"},//array
			fetchCriteria: {type:"any"},//array
			tableModel:    {type:"any"},
			preferencesKey:{type:"any"},//map
			viewsKey:      {type:"any"},//array
			dataDelta:     {type:"any"},//array
			dataUndo:      {type:"any"},//array
			dataPatch:	   {type:"any"},//array
			toolbar:       {type:"string[]",defaultValue:[]},//shared
			footer:        {type:"string[]",defaultValue:[]},//shared
			dataBindId:    {type:"any"},//map (default "items")
			dataKey:       {type:"any"},//map (default: "key")
			arrayNames:    {type:"any"},//map (default: null)
			dataUrl:			{type:"any"},
			dataHeader:		{type:"any"}
		}
	},
	constructor : function() {
		this.supportsLayout=true;
      this.supportsLabel=true;
      this.supportsAlign=false;
      sas.risk.common.layout.flexlayout.FlexLayout.apply(this, arguments);
	},
	renderer: sas.risk.common.layout.flexlayout.FlexLayoutRenderer
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.ui.cpb3.components.SpreadsheetComponent);
sas.ui.cpb3.components.SpreadsheetComponent.prototype.setCPBValue = function(value) {
	this.setCpbValue(value);
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.getFetchCriteria = function() {
	var fc=this.getProperty("fetchCriteria")||{};//dig out the fetch criteria or default to an empty one
	_.each(fc,function(fcInfo,key){//let Views create a biz object out of the json representation
		fc[key]=sas.risk.common.widgets.spreadsheet.View.loadViews([fcInfo])[0].clone();
	});//we are careful to reuse the object instance "fc" so that the object in the model is the same one as the
	//one use by the widget, so that any changes made by the widget are reflected in the model
	this.setProperty("fetchCriteria",fc);//just in case we had to default to an empty one above
	return fc;
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.setFetchCriteria = function(fetchCriteria) {
	this.setProperty("fetchCriteria",fetchCriteria);
	if(this.spreadsheet){
		this.spreadsheet.setFetchCriteria(this.getFetchCriteria());
	}
};

sas.ui.cpb3.components.SpreadsheetComponent.prototype.setDataUrl = function(dataUrl) {
   this.setProperty("dataUrl",dataUrl);
   if(this.spreadsheet) {
      if (_.isString(dataUrl)) {
         this.spreadsheet.setDataUrl({abc:dataUrl});
         if(this.spreadsheet.models && this.spreadsheet.models.abc) {
            this.spreadsheet.models.abc.service.url = dataUrl;
         }
      }
      else {
         this.spreadsheet.setDataUrl(dataUrl);
         var that = this;
         _.each(dataUrl,function(url, modelId) {
            if(that.spreadsheet.models && that.spreadsheet.models[modelId])
               that.spreadsheet.models[modelId].service.url = dataUrl;
         });
      }
      // Trigger a refresh
      if(this.spreadsheet.initialized) {
         var fetchCriteria = this.spreadsheet.getFetchCriteria();
         this.spreadsheet.setFetchCriteria(fetchCriteria);
      }
   }
};

sas.ui.cpb3.components.SpreadsheetComponent.prototype.getMeta = function() {
	var metaProp=this.getProperty("meta");
	return metaProp&&sas.ui.cpb3.components.SpreadsheetComponent._parseMeta(metaProp);
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.setMeta = function(meta) {
	this.setProperty("meta",meta);
	if(this.spreadsheet){
		this.spreadsheet.setMeta(this.getMeta());
	}
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.formulaCallback=function() {
	var def=$.Deferred();
	var args=Array.prototype.slice.call(arguments);
	var funcName=args.shift();
	args=_.map(args,function(arg){
		return _.isNumber(arg)?arg:"'"+arg+"'";
	});
	var params=args.join(",");
	var cpbForm=this.getCpbForm();
	var cpbService=cpbForm.getCpbService();
	var screenId=cpbForm.getCpbModelManager().getUIContextValue("SCREENID");
	cpbService.getCpbAPI().evalExpression(funcName+"("+params+")", {
		SCREENID:screenId
	}).done(function(data){
		def.resolve(data.result);
	}).fail(function(error){
		def.reject(error);
	});
	return def;
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.setFilterPanelTitle = function() {
	var model=this.spreadsheet.getFilterModel();

	// Count how many filters are selected
	var selectedFilters = model.getProperty("/filterSelected").filter(function(e) { return e; }).length;

	var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();
	//old line
	//this.filterShadeSection.setTitle(rb.getText("table.toolbar.filter.title.txt", [selectedFilters, model.getProperty("/filterTotal")]));
	//new line
	this.filterShadeSection.setTitle(rb.getText("table.toolbar.filter.generic.title.txt"));
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype._createComponent = function() {
	this.getCpbForm().attachEvent('reload',$.proxy(this.onReload,this));
	var settings={
		meta:this.getMeta(),//metaUrl sets this ahead of time
		viewsKey:this.getViewsKey(),
		preferencesKey:this.getPreferencesKey(),
		dataBindId:this.getDataBindId(),
		dataUrl:this.getDataUrl(),
		dataHeader:this.getDataHeader(),
		arrayNames:this.getArrayNames(),
		dataKey:this.getDataKey(),
		mashup:$.proxy(this.handleActions,this),
		formulaContext:new sas.risk.common.widgets.spreadsheet.SpreadsheetFormulaContext($.proxy(this.formulaCallback,this)),
		change:$.proxy(this.spreadsheetUpdated,this),
		metaChange:$.proxy(this.spreadsheetMetaUpdated,this)
	};
	this.spreadsheet=new sas.risk.common.widgets.spreadsheet.Spreadsheet(settings);
	if(!_.size(this.getDataUrl())){
		sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbValue",this.spreadsheet,"values");//local
	}else{
		sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbValue",this.spreadsheet,"patch");//rest
	}
	var modelId=this.getCpbForm().getBindingInfo("data").parts[0].model;
	this.bindProperty("cpbValue",{path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,this.getCpbName())});
	var toolbar1=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar riskSpreadsheetToolbar1");
	var toolbar2=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar riskSpreadsheetToolbar2");
	var footer=new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("riskSpreadsheetToolbar riskSpreadsheetToolbarFooter");
	var menu=new sas.hc.m.Menu();
	var filterPanelContent = new sas.risk.common.layout.flexlayout.FlexLayout({
		layoutData: new sas.risk.common.layout.flexlayout.FlexLayoutData({
			alignSelf: sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.Stretch,
			flexGrow: 2
		})
	});

	var that = this;
	var pieces = { 
		spreadsheet: that.spreadsheet,
		toolbar1: toolbar1,
		toolbar2: toolbar2,
		footer: footer,
		menu: menu
	};
	_.each(this.getToolbar(),function(feature){
		var toolbarItem=sas.risk.common.widgets.spreadsheet.SpreadsheetToolbarFactory.getInstance().create(feature,pieces);
		var isComp=toolbarItem instanceof sap.ui.core.Control;

		var isFilter = toolbarItem instanceof sas.risk.common.widgets.spreadsheet.toolbar.Filters;
		if(toolbarItem && isFilter) {
			filterPanelContent.addItem(toolbarItem);
		} else if(toolbarItem && isComp){
			toolbar2.addItem(toolbarItem);
		} else if(toolbarItem) {
			console.error("Added non-control toolbar item named:"+feature);
		} else {
			console.error("Unknown spreadsheet toolbar item named:"+feature);
		}
		if(toolbarItem&&toolbarItem.getMenuItems){
			_.each(toolbarItem.getMenuItems(),function(menuItem){menu.addItem(menuItem);});
		}
	});
	_.each(this.getFooter(),function(feature){
		var toolbarItem=sas.risk.common.widgets.spreadsheet.SpreadsheetToolbarFactory.getInstance().create(feature,pieces);
		if(toolbarItem){
			footer.addItem(toolbarItem);
		}else{
			console.error("Unknown spreadsheet toolbar item named:"+feature);
		}
	});
	if(this.getViewsKey()){
		var views=new sas.risk.common.widgets.spreadsheet.toolbar.Views({spreadsheet:that.spreadsheet});
		toolbar2.addItem(views);
		_.each(views.getMenuItems(),function(menuItem){menu.addItem(menuItem);});
	}
	if(menu.getItems().length){
		toolbar2.addItem(new sas.hc.m.MenuButton({
			icon:sas.icons.HC.OVERFLOW,
			menu:menu
		}));
	}
	var rb = sas.risk.common.widgets.spreadsheet.SpreadsheetUtil.getBundle();

	this.filterShadeSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
		title: rb.getText("table.toolbar.filter.generic.title.txt"),
		content: filterPanelContent
	});

	var layoutItems = [toolbar1, toolbar2, this.spreadsheet, footer];

	// Set filter selected / totals
	var items = filterPanelContent.getItems()[0];
	var that = this;

	var filterWindowShade =	new sas.hc.ui.commons.windowshade.WindowShade({
			openFirstSection: false,
			width: '100%',
			sections: [that.filterShadeSection]
	});

	if(items) {
		items.getFilterColumns().then(function(res){
			var model = that.spreadsheet.getFilterModel();
			filterWindowShade.setOpenFirstSection(model.getProperty("/filterPanelOpen"));

			that.setFilterPanelTitle();
			// Update window shade title when filter model is changed
			var binding = new sap.ui.model.Binding(model, "/", model.getContext("/"));
			binding.attachChange(that.setFilterPanelTitle.bind(that));

			if(model.getProperty("/filterTotal") < 1) {
				filterWindowShade.addStyleClass('hideFilterPanel');
			} else {
				filterWindowShade.removeStyleClass('hideFilterPanel');
			}
		}).catch(function(e){ 
			console.log(e);
		});

		layoutItems.unshift(filterWindowShade.addStyleClass('riskSpreadsheetFilterPanel'));
	}

	var layout = new sas.risk.common.layout.flexlayout.FlexLayout({
		layoutData: new sas.risk.common.layout.flexlayout.FlexLayoutData({
			alignSelf: sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.Stretch,
			flexGrow: 2
		}),
		flexDirection: sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		alignItems: sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Stretch,
		items: layoutItems
	});
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",layout,"enabled");
	
	return layout;
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.onReload=function(event){
	var type=event.getParameter("type");
	console.error("TYPE",type);
	if(!this.spreadsheet.getParent().getParent().getCpbVisible())return;//dont reload if its not visible
	var sheetIds=this.spreadsheet.getSheetIds();
	var index=sheetIds.indexOf(type);
	var refreshSheetId=index!==-1?sheetIds[index]:(type==="*")?this.spreadsheet.getSelectedSheetId():null;
	if(refreshSheetId){
		this.spreadsheet.fetchData(refreshSheetId);
	}
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.handleActions=function(event){
	var action=null;
	var type="";//TODO:!! identify what custom object this spreadsheet is showing
	switch(event.getParameter("event_name")){
		case "create_new":
		case "object_selected":
			var params=event.getParameters();
			params.type=type;
			sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
				this.getCpbForm(),
				params
			);
		break;
		case "delete_objects":
			//maybe confirmation? followed by parallell deletes
		break;
	}
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.wireOnChange=function(onChangeFn){
	if(onChangeFn){
		this.spreadsheet.attachChange(onChangeFn);
	}
};
sas.ui.cpb3.components.SpreadsheetComponent.prototype.spreadsheetMetaUpdated=function(event) {
	if(this.getBindingInfo("meta")){//if bound to a cpb variable
		var current=this.getProperty("meta");
		var newmeta=event.getParameter("meta");
		if(_.isArray(current)){
			var tableId=current.getTableId();
			if(current[tableId] instanceof sas.risk.common.widgets.spreadsheet.SpreadsheetMeta){
				current[tableId]=newmeta;
			}else{
				current[tableId]=newmeta.toJson();
			}
		}else if(current instanceof sas.risk.common.widgets.spreadsheet.SpreadsheetMeta){
			this.setProperty("meta",newmeta);
		}else{
			this.setProperty("meta",newmeta.toJson());
		}
	}
};

sas.ui.cpb3.components.SpreadsheetComponent.prototype.spreadsheetUpdated=function(event) {
	var spreadsheet=event.getSource();
	var sheetId=spreadsheet.getSelectedSheetId();
	var model=spreadsheet.getModel(sheetId);
	if(model){
		var index=spreadsheet.getSelectedSheetIndex();
		var dataDelta=this.getDataDelta()||[];
		dataDelta[index]=model.getDelta();
		this.setDataDelta(dataDelta);
	
		var dataUndo=this.getDataUndo()||[];
		dataUndo[index]=model.getUndo();
		this.setDataUndo(dataUndo);
		
		var dataPatch=this.getDataPatch()||[];
		dataPatch[index]=model.getPatch();
		this.setDataPatch(dataPatch);
		
		var isMulti=this.spreadsheet.tabs.getItems().length>1;
		var service=model.getService();
		if(service instanceof sas.risk.common.widgets.spreadsheet.model.SpreadsheetLocalService){
			if(isMulti){
				var dataArray=this.getCpbValue()||[];
				dataArray[index]=service.getValues();
				this.spreadsheet.setProperty("values",dataArray);
			}else{
				var data=service.getValues();
				this.spreadsheet.setProperty("values",data);
			}
			this.spreadsheet.getModel("cpb_data").fireChange();
		}else{
			if(isMulti){
				var cpbValue=this.getCpbValue()||[];
				cpbValue[index]=model.getPatch();
				this.setCpbValue(cpbValue);
			}else{
				this.setCpbValue(model.getPatch());
			}
		}
	}	
};
//STATICS
sas.ui.cpb3.components.SpreadsheetComponent.create=function(fieldJson, modelId, cpbForm) {
	var def=$.Deferred();
	var rb = cpbForm.getCpbService().getBundle();
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	if(fieldJson.dataDeltaField){
		settings.dataDelta=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.dataDeltaField);
	}
	if(fieldJson.dataUndoField){
		settings.dataUndo=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.dataUndoField);
	}
	if(fieldJson.dataPatchField){
		settings.dataPatch=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.dataPatchField);
	}
	if(fieldJson.metaField){
		settings.meta=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.metaField);
		var spreadsheet=sas.ui.cpb3.components.SpreadsheetComponent._create(fieldJson,modelId,cpbForm,settings);
		def.resolve(spreadsheet);		
	}else{
		$.getJSON(fieldJson.metaUrl,function(metas){
			settings.meta=metas;
			var spreadsheet=sas.ui.cpb3.components.SpreadsheetComponent._create(fieldJson,modelId,cpbForm,settings);
			def.resolve(spreadsheet);
		});
	}
	return def;
};
sas.ui.cpb3.components.SpreadsheetComponent._parseMeta=function(metas) {
	metas=_.isString(metas)?JSON.parse(metas):metas;
	metas=_.isArray(metas)?metas:[{id:"abc",meta:metas}];
	var metaArray=[];
	_.each(metas,function(metaInfo){
		var meta=sas.ui.cpb3.components.SpreadsheetComponent.convertMeta(metaInfo.meta);
		meta=new sas.risk.common.widgets.spreadsheet.SpreadsheetMeta(meta,metaInfo.id);
		metaArray.push({id:metaInfo.id,meta:meta});
	});
	return metaArray;
};
sas.ui.cpb3.components.SpreadsheetComponent._create=function(fieldJson, modelId, cpbForm, settings) {
	if(fieldJson.fetchCriteriaField){
		settings.fetchCriteria=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, fieldJson.fetchCriteriaField);
	}else{
		settings.fetchCriteria={};
	}
	settings.preferencesKey=fieldJson.preferencesKey||{};
	if(_.isString(settings.preferencesKey)){
		settings.preferencesKey={abc:settings.preferencesKey};
	}
	settings.viewsKey=fieldJson.viewsKey;
	settings.dataUrl=fieldJson.dataUrl||{};
	if(_.isString(settings.dataUrl)){
		if(/^FIELD:.*$/i.test(settings.dataUrl)) {
		   var dataUrlField = settings.dataUrl.split(":")[1];
		   settings.dataUrl=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId, dataUrlField);
	    }
		else
		   settings.dataUrl={abc:settings.dataUrl};
	}
	settings.dataHeader=fieldJson.dataHeader||{};
	settings.arrayNames=fieldJson.dataChildArrayNames||{};
	if(_.isString(fieldJson.dataChildArrayNames)){
		settings.arrayNames={abc:[]};
		if(fieldJson.dataChildArrayNames.trim()){
			_.each(fieldJson.dataChildArrayNames.split(","),function(arrayName){
				arrayName=arrayName.trim();
				if(arrayName){
					settings.arrayNames.abc.push(arrayName);
				}
			});
		}
	}
	settings.dataBindId=_.isObject(fieldJson.dataBindId)||{};
	settings.dataKey=_.isObject(fieldJson.dataKey)||((fieldJson.dataKey)?{"abc":fieldJson.dataKey}:{});
	settings.toolbar=fieldJson.toolbar;
	settings.footer=fieldJson.footer;
//	settings.cpbValue=sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName);
	return new sas.ui.cpb3.components.SpreadsheetComponent(settings);
};
///////////////////////////////////////////////
// Conversion Utilities for previous formats //
///////////////////////////////////////////////
sas.ui.cpb3.components.SpreadsheetComponent.convertViews=function(viewsJson1){//TODO: types in filter types need to be converted to new package
	if(viewsJson1.length && !viewsJson1[0].fields){return viewJson1;}//detect new version, nothing to convert;
	var viewsJson2=[];
	_.each(viewsJson1,function(v1){
		var v2={
			filterSimple:v1.simpleFilter,
			filterAdvanced:v1.advancedFilter,
			columnIds:v1.fields,
			key:v1.key,
			name:v1.name,
			description:v1.description,
			isDefault:v1.isDefault,
			sortColumnId:v1.sortColumn
		};
		if(v1.sortDirection){
			v2.sortOrder=v1.sortDirection==="ascending"?"Ascending":"Descending";
		}
		viewsJson2.push(v2);
	});
	return viewsJson2;
};
sas.ui.cpb3.components.SpreadsheetComponent.convertMeta=function(m1){
	if(!m1.title&&!m1.defaultColumns){return m1;}//detect new version, nothing to convert
	var m2={table:{options:{},cellOptions:{}},columns:[]};
	m2.table.options.title=m1.title;
	m2.table.options.selectionMode="MultiToggle";
	m2.table.cellOptions.readonly=true;
	var bigInt=999999;
	var columns=_.filter(m1.columns,function(col){return !col.hidden;});
	_.each(columns,function(col){
		if(col.userDisplayOrder<0){
			col.userDisplayOrder=bigInt;
		}
	});
	var defaultColumns=_.object(_.pluck(m1.defaultColumns,"name"),m1.defaultColumns);
	var orderedColumns=_.sortBy(columns,"userDisplayOrder");
	_.each(orderedColumns,function(col){
		var columnOptions={
			id:col.name,
			label:col.label,
			visible:col.userDisplayOrder!==bigInt,
			defaultVisible:col.visible,
			quickSearch:col.quickSearch,
			filterable:col.filterable,
			queryable:col.queryable
		};
		if(col.sortable){
			columnOptions.sortProperty=col.name;
		}
		var defaultData=defaultColumns[col.name];
		if(defaultData){
			columnOptions.defaultOrder=defaultData.userDisplayOrder;
		}
		var cellOptions={
			cellType:col.type,
		};
		if(col.custom){
			cellOptions.path="customFields";
		}
		if(_.has(col,"choices")){
			cellOptions.choices=col.choices;
			columnOptions.useAllFilter=col.useAllFilter;
			cellOptions.cellType="dropdown";
		}
		if(_.has(col,"orderBy")){
			columnOptions.sorted=true;
			columnOptions.sortOrder=(!!col.orderBy)?sap.ui.table.SortOrder.Descending:sap.ui.table.SortOrder.Ascending;
		}
		m2.columns.push({options:columnOptions,cellOptions:cellOptions});
	});
	return m2;
};