jQuery.sap.declare("sas.ui.cpb3.field.ActionField");
jQuery.sap.require("sas.ui.cpb3.field.AFieldMixin");
jQuery.sap.require("sas.ui.cpb3.CpbAPI");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.hc.ui.commons.MessageBox");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.layout.VerticalLayout.extend("sas.ui.cpb3.field.ActionField", sas.ui.cpb3.field.AFieldMixin.mixinMeta({
      metadata : {
         properties : {
   			cpbKey:{type:"string"},
            cpbButtonLabel : {
               type : "string"
            },
            cpbUrlExp : {
               type : "string"
            },
            cpbUrlStr : {
               type : "string"
            },
            cpbUrlParams : {
               type : "any"
            },
            cpbOutputDestination : {
               type : "string"
            },
            cpbVerb : {
               type : "string"
            },
            cpbSuccessMessage : {
               type : "string"
            }
         }
      }
   }));
sas.ui.cpb3.field.AFieldMixin.mixin(sas.ui.cpb3.field.ActionField);

sas.ui.cpb3.field.ActionField.prototype._createField = function () {
   this.field = new sas.risk.common.widgets.Button({
         press : $.proxy(this.actionButtonClicked, this)
      }).addStyleClass("FormCloseButton"); //TODO
   sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbEnabled", this.field, "enabled");
   sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbButtonLabel", this.field, "text");

   return this.field;
};

sas.ui.cpb3.field.ActionField.prototype.actionButtonClicked = function () {
   var that = this;

   var url = this.getCpbUrlExp();
   if (!url)
      url = this.getCpbUrlStr();
   
   var target;
   var output = this.getCpbOutputDestination();
   if (output === undefined || output === "window") {
      var width = 800;
      var height = 600;
      var top = (screen.availHeight - height) / 2;
      var left = (screen.availWidth - width) / 2;

      if (top < 0) {
         top = 0;
      }
      if (left < 0) {
         left = 0;
      }
      target = window.open('', "sasActionPopup", 'height=' + height + ',width=' + width + ',left=' + left + ',top=' + top + ',scrollbars=yes,resizable=yes'); 
   }
   
   var params = this.getCpbUrlParams();
   this.evaluate(params).done(function (argsArray) {
      for (var i = 0; i < argsArray.length; i++)
      {
         var encoded = $.param(argsArray[i], true);
         if (encoded.length > 0) {
            if (url.indexOf("?") === -1)
               url += "?";
            url += "&" + encoded;
         }            
      }

      that.open(url, target);
   });

};

sas.ui.cpb3.field.ActionField.prototype.open = function(url, target) {
   var that = this;
   var output = this.getCpbOutputDestination();
   if (target) {
      this.openInNewWindow(url, target);
   } 
   else if (output && output === "link") {
      this.openLink(url);
   }
   else {
      var form = this.getCpbForm();
      var app=sas.ui.mashup.App.getInstance();
      app.setBusy(true);
      var verb = this.getCpbVerb();
      this.openInline(url, verb ? verb : "GET").done(function (data) {
         try {
            //parse params from data
            var parsedData = that.parseParams(data);
            for (var prop in parsedData) {
               //console.log(prop + " " + parsedData[prop]);
               var value = parsedData[prop];
               var control = form.getCPBControl(prop);
               if (control)
                  control.setCPBValue(value);
               else
                  form.getData()[prop] = value;
            }
         }
         finally {
         	app.setBusy(false);
         	
         	if (that.getCpbSuccessMessage()) {
         		var rb = that.getCpbForm().getCpbService().getBundle();
         		sas.hc.m.MessageDialog.information({
         			text:that.getCpbSuccessMessage(),
         			title:that.getCpbButtonLabel()
         		});
         	}
         }
      }).fail(function (e) {
      	app.setBusy(false);
         //Can't use CpbUtil.handleFail because the response may not be from our
         //REST services.  It could be anything.  Put up a generic error.
         var rb = that.getCpbForm().getCpbService().getBundle();
         var errorJson = "";
         if (e && e.status) {
            errorJson = {
               "message" : rb.getText("action.error.details.fmt.txt", [e.status, e.statusText])
            };
         } else {
            errorJson = {
               "message" : rb.getText("action.error.txt")
            };
         }
         sas.risk.common.widgets.MessageDetailsBox.error(errorJson, sas.config.appName, rb);
      });
   }
};

sas.ui.cpb3.field.ActionField.prototype.evaluate = function (rawParams) {
   var that = this;
   var form = this.getCpbForm();

   var def = $.Deferred();
   var evals = [];
   for (var key in rawParams) {
      var valuesArray = rawParams[key];
      _.each(valuesArray, function (value) {
            _.each(value, function (v) {
                  evals.push(that.evaluateOne(v, form, key));         
            });
      });
   }

   $.when.apply($, evals).done(function (test) {
      var argsArray = Array.prototype.slice.call(arguments);
      def.resolve(argsArray);
   });

   return def;
};

sas.ui.cpb3.field.ActionField.prototype.evaluateOne = function (param, form, key) {
   var def = $.Deferred();
   var obj = {};
   var that = this;
   var cpbService=this.getCpbForm().getCpbService();
   cpbService.getCpbAPI().evalExpression(param, form.getData()).done(function (data, textStatus, jqXHR) {
      obj[key] = data.result;
      def.resolve(obj);
   }).fail(function (e,jqxhr) {
   	var rb = cpbService.getBundle();
      sas.ui.cpb3.CpbUtil.handleFail(e,jqxhr,rb);
      def.reject();
   });
   return def;
};

sas.ui.cpb3.field.ActionField.prototype.parseParams = function (url) {
   if (!url || url === "")
      return {};
   var params = url.split("&");
   var b = {};
   for (var i = 0; i < params.length; ++i) {
      var p = params[i].split('=', 2);
      if (p[1])
         p[1] = decodeURIComponent(p[1].replace(/\+/g, " "));
      else
         p[1] = '';
      b[p[0].trim()] = p[1];
   }
   return b;
};

sas.ui.cpb3.field.ActionField.prototype.openInline = function (url, verb) {
   var def = $.Deferred();
   $.ajax({
      type : verb,
      url : url,
      contentType : false
   }).done(function (data, textStatus, jqXHR) {
      def.resolve(data);
   }).fail(function (e) {
      def.reject(e);
   });
   return def;
};

sas.ui.cpb3.field.ActionField.prototype.openInNewWindow = function (url, target) {
   if (target) {
      target.location = url;
      target.focus();
   }
};

sas.ui.cpb3.field.ActionField.prototype.openLink = function (url) {
   var a = document.createElement("a");
   a.href = url;
   document.body.appendChild(a);
   a.click();
};
//STATICS
sas.ui.cpb3.field.ActionField.create=function(fieldJson,modelId,cpbForm){
   var field=new sas.ui.cpb3.field.ActionField($.extend(
		sas.ui.cpb3.WidgetBindingUtil.getEnabledSettings(fieldJson,modelId),
	   {
			cpbForm:cpbForm,
         cpbId:fieldJson.widgetId,
         cpbKey:fieldJson.id,
         cpbButtonLabel:{
         	path:"cpb_labels>/label"+fieldJson.cpbLabel,
            formatter:sas.ui.cpb3.WidgetBindingUtil.formatterLabel
         },
         cpbVisible:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbVisible),
         cpbUrlExp:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbUrlExp),
         cpbUrlStr:fieldJson.cpbUrlStr,
         cpbUrlParams:fieldJson.cpbUrlParams,
         cpbOutputDestination:fieldJson.cpbOutputDestination,
         cpbVerb:fieldJson.cpbVerb,
         cpbSuccessMessage:fieldJson.cpbSuccessMessage,
         cpbErrored:{
         	path:sas.ui.cpb3.model.CpbModelManagerObject.bindMessagePath("/errored"),
            formatter:sas.ui.cpb3.WidgetBindingUtil.formatterError
         }
	   }
	));
   return field;
};