jQuery.sap.declare("sas.ui.cpb3.field.BooleanField");
jQuery.sap.require("sas.ui.cpb3.field.AFieldMixin");
jQuery.sap.require("sas.ui.cpb3.widget.RadioButtonGroup");
jQuery.sap.require("sas.hc.m.RadioButton");
jQuery.sap.require("sap.ui.commons.TextView");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");

sap.ui.layout.VerticalLayout.extend("sas.ui.cpb3.field.BooleanField",sas.ui.cpb3.field.AFieldMixin.mixinMeta({
	metadata:{
		properties:{
			cpbTrueLabel:{type:"string"},
			cpbFalseLabel:{type:"string"},
			cpbReadonlyText:{type:"string"},
			cpbReadonlyValue:{type:"boolean"},
			optionDirection:{type:"string"}
		}
	}
}));
sas.ui.cpb3.field.AFieldMixin.mixin(sas.ui.cpb3.field.BooleanField);

sas.ui.cpb3.field.BooleanField.prototype._createField = function() {
	var rb = this.getCpbForm().getCpbService().getBundle();
	this.yesBtn = new sas.hc.m.RadioButton().addCustomData(new sap.ui.core.CustomData({
		key: "val",value : "yes"
	})).addStyleClass("cpbRadioButton");
	this.noBtn = new sas.hc.m.RadioButton().addCustomData(new sap.ui.core.CustomData({
		key: "val",value : "no"
	})).addStyleClass("cpbRadioButton");
	this.field = new sas.ui.cpb3.widget.RadioButtonGroup({
		columns : 2,
		buttons : [ this.yesBtn, this.noBtn ]
	}).addAriaLabelledBy(this._label);
	if(this.getOptionDirection()!=="vertical"){
		this.field.addStyleClass("cpbBooleanRadioButtonGroup");
	}
	this.readonlyField = new sap.ui.commons.TextView().addStyleClass("cpbReadonlyField");
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbTrueLabel",this.yesBtn,"text");
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbFalseLabel",this.noBtn,"text");
	sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbValue", this.field, "selectedIndex");
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbReadonlyText",this.readonlyField,"text");
	return this.field;
};
sas.ui.cpb3.field.BooleanField.prototype.setCpbEnabled = function(val){
	this.setProperty("cpbEnabled",val);
	if(val===false){
		var index=this._layout.indexOfItem(this.field);
		this._layout.removeItem(this.field);
		this._layout.insertItem(this.readonlyField,index);
		this.setCPBControl(this.readonlyField);
	}
};
//OVERRIDE
sas.ui.cpb3.field.BooleanField.prototype.setCPBValue = function(value) {
	if(!this.getProperty("cpbEnabled")){
		this.setCpbReadonlyValue(value);
	}
	else{
		var formattedVal=sas.ui.cpb3.model.type.CpbBoolean.prototype.getFormattedValue(value,"int");
		this.getCPBControl().setSelectedIndex(formattedVal);
	}
};
//STATICS
sas.ui.cpb3.field.BooleanField.create=function(fieldJson,modelId,cpbForm){
	jQuery.sap.require("sas.ui.cpb3.model.type.CpbBoolean");
	var rb=cpbForm.getCpbService().getBundle();
	var settings=$.extend(
		sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm),
		{
			cpbTrueLabel:{
				path:"cpb_labels>/label"+fieldJson.cpbTrueLabel,
				formatter:$.proxy(sas.ui.cpb3.WidgetBindingUtil.formatterTrueLabel,this,rb)
			},
			cpbFalseLabel:{
				path:"cpb_labels>/label"+fieldJson.cpbFalseLabel,
				formatter:$.proxy(sas.ui.cpb3.WidgetBindingUtil.formatterFalseLabel,this,rb)
			},
			cpbReadonlyText:{
				parts:[sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,fieldJson.cpbName), 
				       "cpb_labels>/label"+fieldJson.cpbTrueLabel, 
				       "cpb_labels>/label"+fieldJson.cpbFalseLabel
				],
				formatter:function(value,trueLabel,falseLabel){
					var yesLabel=sas.ui.cpb3.WidgetBindingUtil.formatterTrueLabel(rb,trueLabel);
					var noLabel=sas.ui.cpb3.WidgetBindingUtil.formatterFalseLabel(rb,falseLabel);
					if(value==true) return yesLabel;
					if(value==false) return noLabel;
				}
			},
			cpbReadonlyValue:{
		   	path:sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,fieldJson.cpbName),
				type:'sas.ui.cpb3.model.type.CpbBoolean'
			},
			optionDirection: fieldJson.optionDirection

		}
	);
	var field=new sas.ui.cpb3.field.BooleanField(settings);
	field.bindProperty("cpbValue",{
   	path:sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,fieldJson.cpbName),
		type:'sas.ui.cpb3.model.type.CpbBoolean'//,
//		formatOptions: {field:field}
	});
	return field;
};