jQuery.sap.declare("sas.ui.cpb3.field.CheckBoxField");
jQuery.sap.require("sas.ui.cpb3.field.AFieldMixin");
jQuery.sap.require("sas.risk.common.widgets.CheckBoxGroup");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.hc.m.CheckBox");

sap.ui.layout.VerticalLayout.extend("sas.ui.cpb3.field.CheckBoxField",sas.ui.cpb3.field.AFieldMixin.mixinMeta({
	metadata:{
		properties:{
			optionDirection:{type:"string"}
		}
	}
}));
sas.ui.cpb3.field.AFieldMixin.mixin(sas.ui.cpb3.field.CheckBoxField);

sas.ui.cpb3.field.CheckBoxField.prototype._createField = function() {
	var rb = this.getCpbForm().getCpbService().getBundle();
	var bindPath=this.getBindingPath("cpbValue");
	var that=this;
	this.field=new sas.risk.common.widgets.CheckBoxGroup({
		flexDirection:this.getOptionDirection() === "horizontal"? sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row
			:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
		flexWrap:sas.risk.common.layout.flexlayout.FlexLayoutWrap.Wrap,
		items:{
			path:"cpb_choices>/"+this.getCpbName(),
			template:new sas.hc.m.CheckBox({
	         name: '{cpb_choices>value}',
	         text: '{cpb_choices>label}',
	         select: function(event){
	         	var selected=event.getParameters().selected;
	         	var selections=that.field.getCheckedValues();
	         	if (!selections) selections = [];
	         	var findVal=_.find(selections,function(val){return val===event.getSource().getName();});
	         	if(findVal && !selected){
	         		selections.splice(selections.indexOf(findVal),1);
	         	}
	         	else if(!findVal && selected){
	         		selections.push(event.getSource().getName());
	         	}
	         	//for comparisons  - don't save [], reset instead
	         	//sort selections by value, not by order they are clicked
	         	if (selections.length==0)
	         	   selections = null;
	         	else
	         		selections.sort();
	         	event.getSource().getParent().setCheckedValues(selections);
	         	event.getSource().getParent().fireChange();
	         }
	      })
		}
	});
    sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.field,"enabled");
    sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbValue",this.field,"checkedValues");
	return this.field;
};

sas.ui.cpb3.field.CheckBoxField.prototype.setCpbValues = function(val){
	this.setProperty("cpbValues",val);
	if(val && val.length>0){
		if(!this.field){
			this.field=new sas.risk.common.widgets.CheckBoxGroup({
				flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
				flexWrap:sas.risk.common.layout.flexlayout.FlexLayoutWrap.Wrap
			});
		}
		this.field.unbindAggregation("items");
		this.setItems(val);
	}
};

sas.ui.cpb3.field.CheckBoxField.prototype.onBeforeRendering=function(){
	if(this.getCpbAlign()==="inline"){
		this.field.setFlexDirection(sas.risk.common.layout.flexlayout.FlexLayoutDirection.Row);
		this.field.addStyleClass("cpbInlineCheckboxGroup");
	}
};


sas.ui.cpb3.field.CheckBoxField.prototype.setItems=function(values){
	var that=this;
	// Hang on this value.removeAllItems will erase the cpbValue
	var cpbValue=this.getCpbValue();
	this.field.removeAllItems();
	_.each(values, function(labelValuePair){
		var item=new sas.hc.m.CheckBox({
	         name: labelValuePair.value,
	         text: labelValuePair.label,
	         select: function(event){
	         	var selected=event.getParameters().selected;
	         	var selections=that.field.getCheckedValues();
	         	if (!selections) selections = [];
	         	var findVal=_.find(selections,function(val){return val===event.getSource().getName();});
	         	if(findVal && !selected){
	         		selections.splice(selections.indexOf(findVal),1);
	         	}
	         	else if(!findVal && selected){
	         		selections.push(event.getSource().getName());
	         	}
	         	event.getSource().getParent().setCheckedValues(selections);
	         	event.getSource().getParent().fireChange();
	         }
	      });
		
		that.field.addItem(
			item
			);
	});
	this.setCpbValue(cpbValue);
};

//CPB API
sas.ui.cpb3.field.CheckBoxField.prototype.setCPBRequired = function(val) {//OVERRIDEN
   this.setCpbRequired(val);
};

sas.ui.cpb3.field.CheckBoxField.prototype.setCPBValues = function(val) {//OVERRIDEN
	   this.setCpbValues(val);
};

sas.ui.cpb3.field.CheckBoxField.prototype.setCPBValue = function(val) {//OVERRIDEN
	   this.setCpbValue(val);
};
//STATICS
sas.ui.cpb3.field.CheckBoxField.create=function(fieldJson,modelId,cpbForm){
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	var field=new sas.ui.cpb3.field.CheckBoxField($.extend(
		settings,
		{
			cpbValues:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbValues),
			optionDirection: fieldJson.optionDirection
		}
	));
   field.bindProperty("cpbValue",{
		path:sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,fieldJson.cpbName)
   });
   return field;
};