jQuery.sap.declare("sas.ui.cpb3.field.DateField");
jQuery.sap.require("sas.ui.cpb3.field.AFieldMixin");
jQuery.sap.require("sas.ui.cpb3.widget.DatePicker");
jQuery.sap.require("sas.ui.cpb3.model.format.CpbDateFormat");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");

sap.ui.layout.VerticalLayout.extend("sas.ui.cpb3.field.DateField", sas.ui.cpb3.field.AFieldMixin.mixinMeta({
	metadata: {
		properties: {
			cpbMin: {
				type: "object"
			},
			cpbMax: {
				type: "object"
			},
			preventDateEntry: {
				type: "boolean"
			}
		}
	},
	constructor: function () {
		this.field = new sas.ui.cpb3.widget.DatePicker({
			width: "auto",
			showTodayButton: true,
			readOnly: false
		}).addStyleClass("cpbInputField");
		sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbValue", this.field, "value");
		sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbEnabled", this.field, "enabled");
		sap.ui.layout.VerticalLayout.apply(this, arguments);
	}
}));
sas.ui.cpb3.field.AFieldMixin.mixin(sas.ui.cpb3.field.DateField);

sas.ui.cpb3.field.DateField.prototype._createField = function () {
	return this.field;
};
//CPB API
sas.ui.cpb3.field.DateField.prototype.setCPBRequired = function (val) { //OVERRIDEN
	this.setCpbRequired(val);
};
//OVERRIDE
sas.ui.cpb3.field.DateField.prototype.setCPBValue = function (value) {
	var type = this.getCPBControl().getBinding("value").getType();
	type.updateDateFormat();
	var val = type.oInputFormat.parse(value);
	this.getCPBControl().setValue(val ? type.oOutputFormat.format(val) : val);
};

sas.ui.cpb3.field.DateField.prototype.setCpbMin = function (val) {
	var date = this.convertFromStringToDate(val);
	if (date) {
		date.setHours(0);
	}
	this.field.setMinDate(date);
};
sas.ui.cpb3.field.DateField.prototype.setCpbMax = function (val) {
	var date = this.convertFromStringToDate(val);
	if (date) {
		date.setHours(23);
		date.setMinutes(59);
		date.setSeconds(59);
		date.setMilliseconds(999);
	}
	this.field.setMaxDate(date);
};
sas.ui.cpb3.field.DateField.prototype.setPreventDateEntry = function (val) {
	this.field.setReadOnly(!!val);
}
sas.ui.cpb3.field.DateField.prototype.convertFromStringToDate = function (dateString) {
	var locale = sas.ui.cpb3.CpbUtil.getLocale();
	var inputFormat = sas.ui.cpb3.model.format.CpbDateFormat.getInstance({
		pattern: "yyyy-MM-ddz"
	}, locale);
	return inputFormat.parse(dateString);
};
//STATICS
sas.ui.cpb3.field.DateField.create = function (fieldJson, modelId, cpbForm) {
	jQuery.sap.require("sas.ui.cpb3.model.type.CpbDate");
	var settings = $.extend(
		sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson, modelId, cpbForm),
		sas.ui.cpb3.WidgetBindingUtil.getMinMaxSettings(fieldJson, modelId, cpbForm), {
			cpbMin: sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbMin),
			cpbMax: sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbMax)
		},
		sas.ui.cpb3.WidgetBindingUtil.getEnabledSettings(fieldJson, modelId), {
			preventDateEntry: sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbPreventDateEntry)
		}
	);
	var field = new sas.ui.cpb3.field.DateField(settings);
	field.bindProperty("cpbValue", {
		path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId, fieldJson.cpbName),
		type: new sas.ui.cpb3.model.type.CpbDate({
			field: field
		})
	});
	return field;
};