jQuery.sap.declare("sas.ui.cpb3.field.RadioField");
jQuery.sap.require("sas.ui.cpb3.field.AFieldMixin");
jQuery.sap.require("sas.ui.cpb3.widget.RadioButtonGroup");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.hc.m.RadioButton");

sap.ui.layout.VerticalLayout.extend("sas.ui.cpb3.field.RadioField",sas.ui.cpb3.field.AFieldMixin.mixinMeta({
	metadata:{
		properties:{
			optionDirection:{type:"string"}
		}
	}
}));
sas.ui.cpb3.field.AFieldMixin.mixin(sas.ui.cpb3.field.RadioField);

sas.ui.cpb3.field.RadioField.prototype._createField = function() {
	var rb = this.getCpbForm().getCpbService().getBundle();
	var bindPath=this.getBindingPath("cpbValue");
	this.oRadio = new sas.ui.cpb3.widget.RadioButtonGroup({
	   buttons:{
	   	path:"cpb_choices>/"+this.getCpbName(),
         template: new sas.hc.m.RadioButton({
            text: '{cpb_choices>label}',
            customData:new sap.ui.core.CustomData({
            	key:'val',
            	value:'{cpb_choices>value}'
            })
        })
	   }
	}).addAriaLabelledBy(this._label);
	sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbEnabled",this.oRadio,"enabled");
    sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbValue",this.oRadio,"selectedIndex");
	return this.oRadio;
};

sas.ui.cpb3.field.RadioField.prototype.setCpbValues = function(val){
	this.setProperty("cpbValues",val);
	if (val && val.length > 0)	{
		if (!this.oRadio){
			this.oRadio = new sas.ui.cpb3.widget.RadioButtonGroup({
			}).addAriaLabelledBy(this._label);
		}
		this.oRadio.unbindButtons();
		this.setItems(val);
	}
};

sas.ui.cpb3.field.RadioField.prototype.onBeforeRendering=function(){
	if(this.getCpbAlign()==="inline"){
		var num=this.oRadio.getButtons().length; 
		this.oRadio.setColumns(num);
	}
	if(this.getOptionDirection()==="horizontal"){
		this.oRadio.addStyleClass("cpbRadioFieldHorizontal");
	}
};

sas.ui.cpb3.field.RadioField.prototype.setItems=function(values){
	var that=this;
	// Hang on this value.removeAllItems will erase the cpbValue
	var selectedButton=this.oRadio.getSelectedButton();
	var prevValue=selectedButton?selectedButton.getCustomData()[0].getValue():null;
	this.oRadio.removeAllButtons();
	_.each(values, function(labelValuePair){
		var item=new sas.hc.m.RadioButton({
         text: labelValuePair.label
      }).addCustomData(new sap.ui.core.CustomData({
   		key: "val",value : labelValuePair.value
   	}));
		that.oRadio.addButton(item);
		if(prevValue===labelValuePair.value){
			that.oRadio.setSelectedButton(item);
		}
	});

};

//CPB API
sas.ui.cpb3.field.RadioField.prototype.setCPBRequired = function(val) {//OVERRIDEN
   this.setCpbRequired(val);
};

sas.ui.cpb3.field.RadioField.prototype.setCPBValues = function(val) {//OVERRIDEN
	   this.setCpbValues(val);
};

sas.ui.cpb3.field.RadioField.prototype.setCPBValue = function(val) {//OVERRIDEN
	var choices=this.oRadio.getButtons();
	var selection=_.find(choices,function(choice){return choice.getCustomData()[0].getValue()===val;});
	if(selection){
		var selIndex=choices.indexOf(selection);
		this.setCpbValue(selIndex);
	}
	else{
		this.setCpbValue(undefined);
	}
};
//STATICS
sas.ui.cpb3.field.RadioField.create=function(fieldJson,modelId,cpbForm){
	jQuery.sap.require("sas.ui.cpb3.model.type.CpbRadio");
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	var field=new sas.ui.cpb3.field.RadioField($.extend(
		settings,
		{
			cpbValues:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbValues),
			optionDirection: fieldJson.optionDirection
		}
	));
	field.bindProperty("cpbValue",{
		path: sas.ui.cpb3.WidgetBindingUtil.getDataPath(modelId,fieldJson.cpbName),
		type:'sas.ui.cpb3.model.type.CpbRadio',
		formatOptions: {field:field}
	});
	return field;
};