jQuery.sap.declare("sas.ui.cpb3.layout.TabSectionLayout");
jQuery.sap.require("sas.ui.cpb3.layout.ALayoutMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.ui.cpb3.layout.TabLayout");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.hc.m.IconTabBar");
jQuery.sap.require("sas.risk.common.utils.CustomDataUtil");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.ui.cpb3.layout.TabSectionLayout",sas.ui.cpb3.layout.ALayoutMixin.mixinMeta({
	metadata:{
		properties:{
			cpbKey:{type:"string"},
			upperCase:{type : "boolean",defaultValue : false}// so that it displays in 2.4
		},
		aggregations : {
			tabs:{type : "sas.ui.cpb3.layout.TabLayout",multiple : true,singularName : "tab"},
		}
	},
	constructor : function() {
		sas.risk.common.utils.CompositeUtil.addComposite(this, "tabs", this.getTabBar, "items");
		sas.risk.common.utils.CompositeUtil.addComposite(this, "cpbVisible", this, "visible");
		sas.ui.cpb3.layout.ALayoutMixin.meta.constructor.apply(this, arguments);
		this.setWidth("100%");
		this.addStyleClass("cpbTabLayout");
	},
	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.layout.ALayoutMixin.mixin(sas.ui.cpb3.layout.TabSectionLayout);

sas.ui.cpb3.layout.TabSectionLayout.prototype._createLayout = function() {
	return this;
};
sas.ui.cpb3.layout.TabSectionLayout.prototype.addItem = function(tabLayout) {
	this.children.push(tabLayout);
	this.tabBar.addItem(tabLayout);
	var tabContent = tabLayout.getPayload();
	if(this.children.length){
		tabContent.setVisible(false);
	}
	this.content.addItem(tabContent);
	tabLayout.setCpbParent(this);
};
sas.ui.cpb3.layout.TabSectionLayout.prototype.init = function() {
	this.setFlexDirection(sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column);
	this.tabBar=new sas.hc.m.IconTabBar({
		select : $.proxy(this.actionTabSelected, this)
	}).addStyleClass("cpbMenuBar");
	this.addAggregation("items",this.tabBar);
	this.content = new sas.risk.common.layout.flexlayout.FlexLayout({}).addStyleClass("cpbTabContent");
	this.addAggregation("items",this.content);
};
sas.ui.cpb3.layout.TabSectionLayout.prototype.findPrevSelectedTab = function(){
	var tabKey=sas.ui.cpb3.CpbUtil.sessionGet(hasher.getHash()+"|"+this.getCpbKey(),true);
	if(tabKey){
		return _.find(this.getTabs(),function(tab){return tab.getCpbKey()===tabKey;});
	}
	return null;
};
sas.ui.cpb3.layout.TabSectionLayout.prototype.onBeforeRendering = function() {
	sas.ui.cpb3.layout.ALayoutMixin.proto.onBeforeRendering.apply(this,arguments);
	var tabs=this.getTabs()||[];
	_.each(tabs,function(tab){
		tab.getErrored()?sas.risk.common.utils.CustomDataUtil.data(tab,"cpbError","true",true):sas.risk.common.utils.CustomDataUtil.remove(tab,"cpbError");
		tab.getRequired()?sas.risk.common.utils.CustomDataUtil.data(tab,"cpbRequired","true",true):sas.risk.common.utils.CustomDataUtil.remove(tab,"cpbRequired");
	});
	var i,selectedTab,hasActive;
	for(i=0;i<tabs.length;i++){
		if(i===0){
			selectedTab=tabs[0];
		}
		if(tabs[i].getActive()){
			hasActive=true;
			selectedTab=tabs[i];
			break;
		}
	}
	if(!hasActive){
		var prevTab=this.findPrevSelectedTab();
		if(prevTab && prevTab.getVisible()){
			this.actionTabSelected(prevTab);
			selectedTab=prevTab;
		}
	}
	var prevSelectedKey=this.tabBar.getSelectedKey();
	if(selectedTab && selectedTab.getKey()!==prevSelectedKey){
		this.tabBar.setSelectedKey(selectedTab.getKey());
		this.tabBar.fireSelect({item:selectedTab});
	}
};
sas.ui.cpb3.layout.TabSectionLayout.prototype.getTabBar = function() {
	return this.tabBar;
};
sas.ui.cpb3.layout.TabSectionLayout.prototype.actionTabSelected = function(e) {
	var prevSelected=_.find(this.getTabs(),function(tab){return tab.getActive();});
	if(prevSelected){
		prevSelected.setActive(false);
		sas.risk.common.utils.CustomDataUtil.remove(prevSelected,"tabselected");
		prevSelected.getPayload().setVisible(false);
	}
	var tab = e.getParameter?e.getParameter("item"):e;
	tab.setActive(true);
	sas.ui.cpb3.CpbUtil.sessionSet(hasher.getHash()+"|"+this.getCpbKey(),tab.getCpbKey());
	sas.risk.common.utils.CustomDataUtil.data(tab,"tabselected","true",true);
	var tabContent = tab.getPayload();
	tabContent.setVisible(true);
};
sas.risk.common.utils.CompositeUtil.mixinAggregationSupport(sas.ui.cpb3.layout.TabSectionLayout);
//STATICS
sas.ui.cpb3.layout.TabSectionLayout.create=function(fieldJson,modelId,cpbForm){
	var tabSection=new sas.ui.cpb3.layout.TabSectionLayout({
		cpbForm:cpbForm,
		cpbId:fieldJson.widgetId,
		cpbKey:fieldJson.id,
      cpbVisible:sas.ui.cpb3.WidgetBindingUtil.getExpBindingPath(fieldJson.cpbVisible)
	});
	return cpbForm.buildStructure(tabSection,fieldJson.tabs,modelId);
};