jQuery.sap.declare("sas.ui.cpb3.model.CpbModelManager");
jQuery.sap.require("sas.ui.cpb3.model.CpbJSONModel");

sas.ui.cpb3.model.CpbModelManager=function(defaultContext){
	this.defaultContext=defaultContext;
	if(defaultContext && defaultContext.setCpbModelManager){
		defaultContext.setCpbModelManager(this);
	}
};
sas.ui.cpb3.model.CpbModelManager._nameLabels="cpb_labels";
sas.ui.cpb3.model.CpbModelManager._nameExps="cpb_exps";
sas.ui.cpb3.model.CpbModelManager._nameStructure="cpb_structure";
sas.ui.cpb3.model.CpbModelManager._nameObservers="cpb_observers";
sas.ui.cpb3.model.CpbModelManager._nameData="cpb_data";

sas.ui.cpb3.model.CpbModelManager.prototype={
	_nameLabels : sas.ui.cpb3.model.CpbModelManager._nameLabels,
	_nameExps : sas.ui.cpb3.model.CpbModelManager._nameExps,
	_nameStructure : sas.ui.cpb3.model.CpbModelManager._nameStructure,
	_nameObservers : sas.ui.cpb3.model.CpbModelManager._nameObservers,
	_nameData : sas.ui.cpb3.model.CpbModelManager._nameData,
	refresh : function(json,context) {
		this._setModel(json.cpb_labels,this._nameLabels,context);
		this._setModel(json.cpb_exps,this._nameExps,context);
		this._setModel(json.cpb_structure,this._nameStructure,context);
		this._setModel(json.cpb_observers,this._nameObservers,context);
		this._setModel(json.cpb_data,this._nameData,context);
	},
	isInitialized:function(view){
		var dataModel=view.getModel(this._nameData)
		if(!!dataModel){
			var dataScreenId=dataModel.getProperty("/screenId")||dataModel.getProperty("/SCREENID");
			return view.getScreenId()===dataScreenId;
		}
		return false;
	},
	getStructure:function(context){
		var model=context.getModel(this._nameStructure);
		return model.getData();
	},
	getObservers:function(context){
		var model=context.getModel(this._nameObservers);
		return model.getData();
	},
	clear:function(context){
		this.refresh({},context);
	},
	_getModel:function(name,context){
		context=this._resolveContext(context);
		return context.getModel(name);
	},
	_setModel:function(json,name,context){
		context=this._resolveContext(context);
		var oldModel=context.getModel(name);
		if(oldModel){
			oldModel.destroy();
		}
		if(!json){
			context.setModel(null,name);
		}else{
			context.setModel(new sas.ui.cpb3.model.CpbJSONModel(json),name);
		}
	},
	getUIContextModel:function(context){
		context=this._resolveContext(context);
		var model=context.getModel(this._nameData);
		return model;
	},
	getUIContext:function(context){
		context=this._resolveContext(context);
		var model=context.getModel(this._nameData);
		var data=model && model.getData();
		return data;
	},
	getUIContextValue:function(name,context){
		context=this._resolveContext(context);
		var result, uiContext=this.getUIContext(context);
		if(uiContext){
			result=uiContext[name.toUpperCase()];
		}
		return result;
	},
	_resolveContext:function(context){
		return context||this.defaultContext;
	}
};
//BINDINGS
sas.ui.cpb3.model.CpbModelManager.bindData=function(path) {
	return "{" + sas.ui.cpb3.model.CpbModelManager.bindDataPath(path) + "}";
};
sas.ui.cpb3.model.CpbModelManager.bindDataPath=function(path){
	return sas.ui.cpb3.model.CpbModelManager._bindPath(path,sas.ui.cpb3.model.CpbModelManager._nameData);
};
sas.ui.cpb3.model.CpbModelManager.bindStructure=function(path) {
	return "{" + sas.ui.cpb3.model.CpbModelManager.bindStructurePath(path) + "}";
};
sas.ui.cpb3.model.CpbModelManager.bindStructurePath=function(path){
	return sas.ui.cpb3.model.CpbModelManager._bindPath(path,sas.ui.cpb3.model.CpbModelManager._nameStructure);
};
sas.ui.cpb3.model.CpbModelManager.bindExp=function(path) {
	return "{" + sas.ui.cpb3.model.CpbModelManager.bindExpPath(path) + "}";
};
sas.ui.cpb3.model.CpbModelManager.bindExpPath=function(path){
	return sas.ui.cpb3.model.CpbModelManager._bindPath(path,sas.ui.cpb3.model.CpbModelManager._nameExps);
};
sas.ui.cpb3.model.CpbModelManager.bindLabel=function(path) {
	return "{" + sas.ui.cpb3.model.CpbModelManager.bindLabelPath(path) + "}";
};
sas.ui.cpb3.model.CpbModelManager.bindLabelPath=function(path){
	return sas.ui.cpb3.model.CpbModelManager._bindPath(path,sas.ui.cpb3.model.CpbModelManager._nameLabels);
};
//HELPERS
sas.ui.cpb3.model.CpbModelManager._bindPath=function(path,name) {
	return name?name + ">" + path:path;
};
//FACTORY
sas.ui.cpb3.model.CpbModelManager.create=function(defaultContext){
	return new sas.ui.cpb3.model.CpbModelManager(defaultContext);
};