jQuery.sap.declare("sas.ui.cpb3.nav.ToolbarActionsObject");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutRenderer");
jQuery.sap.require("sas.ui.cpb3.model.CpbModelManagerObject");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.risk.common.utils.DialogManager");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.ui.cpb3.nav.NavUtil");
jQuery.sap.require("sas.ui.cpb3.nav.ToolbarActionsQuery");
jQuery.sap.require("sas.risk.common.widgets.Button");

sas.ui.cpb3.nav.ToolbarActionsQuery.extend("sas.ui.cpb3.nav.ToolbarActionsObject",{
	metadata:{
		properties:{
			changeReason: {type:"string"},
			nestedSaves: {type:"any"},
			nestedDeletes: {type:"any"},
			bizComp:{type:"any"}
		}
	},
	constructor:function(){
		sas.ui.cpb3.nav.ToolbarActionsQuery.apply(this,arguments);
	},
	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
});
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.applySettings=function(){
	sas.ui.cpb3.nav.ToolbarActionsQuery.prototype.applySettings.apply(this,arguments);
	this.removeAllItems();
	this.saveButton=this.createSaveButton();
	this.closeButton=this.createCloseButton();
	this.addItem(this.saveButton);
	this.addItem(this.closeButton);
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.createCloseButton=function(){
	var rb=this.getCpbForm().getCpbService().getBundle();
	return new sas.risk.common.widgets.Button({
		text:rb.getText("form.button.close.txt"),
		press: $.proxy(this.actionCloseCheck,this)
	}).addStyleClass("FormCloseButton").setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexShrink:0}));
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.createSaveButton=function(){
	this.bindProperty("nestedSaves",sas.ui.cpb3.model.CpbModelManagerObject.bindNestedSavesPath("/"));
	this.bindProperty("nestedDeletes",sas.ui.cpb3.model.CpbModelManagerObject.bindNestedDeletesPath("/"));
	var cpbForm=this.getCpbForm();
	var objectModel=cpbForm.getCpbModelManager();
	var rb=cpbForm.getCpbService().getBundle();
	var that=this;
	return new sas.risk.common.widgets.Button({
		icon: sas.icons.HC.SAVE,
		press: function(){
			that.updateObjectInfo();
			if(that.isChangeReasonRequired()){
				var viewData={
					create:true,
					title:rb.getText("form.changereason.txt"),
					reasonTxt:rb.getText("form.reasonText.txt"),
					saveButtonTxt:rb.getText("changeReason.add.dialog.save.txt"),
					cancelButtonTxt:rb.getText("changeReason.add.dialog.cancel.txt"),
					changeReasonMaxLen: objectModel.getChangeReasonMaxLen(),
					changeReason: that.getChangeReason()
				};
				sas.risk.common.utils.DialogManager.open("sas.ui.cpb3.nav.ChangeReasonDialog",viewData,that.actionSaveWithReason,that);	
			}else{
				that.actionSave();
			}
		},
		enabled:{
			parts:[sas.ui.cpb3.model.CpbModelManagerObject.bindMiscPath("/isDirty"),
			       sas.ui.cpb3.model.CpbModelManager.bindDataPath("/SCREEN_READ_ONLY"),
			       sas.ui.cpb3.model.CpbModelManager.bindDataPath("/__DIRTY__"),
			       sas.ui.cpb3.model.CpbModelManagerObject.bindMiscPath("/createNewFrom"),
			       sas.ui.cpb3.model.CpbModelManagerObject.bindNestedSavesPath("/"),
			       sas.ui.cpb3.model.CpbModelManagerObject.bindNestedDeletesPath("/")
	      ],
			formatter:function(isDirty,screenReadonly,initDirty,createNewFrom,nestedSaves,nestedDeletes){
				var hasNestedSaves=!!(_.find(nestedSaves,function(arr){return !!arr.length;}));
				var hasNestedDeletes=!!(_.find(nestedDeletes,function(arr){return !!arr.length;}));
				return (!!initDirty && !screenReadonly) || (!!isDirty && !screenReadonly) || !!createNewFrom || hasNestedSaves || hasNestedDeletes;
			}
		}
	}).addStyleClass("FormSaveButton");
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.navToScreenEditor=function(){
	var bizComp=this.getBizComp();
	var route=sas.ui.cpb3.nav.NavUtil.SCREENEDIT_OBJROUTE_EDIT;
	var key=bizComp.key;
	var variance=null;
	if(bizComp.isDraft){
		key="drafts";
		if(bizComp.isCreateLike){
			route=sas.ui.cpb3.nav.NavUtil.SCREENEDIT_OBJROUTE_DRAFT_LIKE;
			variance=bizComp.key;
		}else{
			route=sas.ui.cpb3.nav.NavUtil.SCREENEDIT_OBJROUTE_DRAFT;
		}
	}
	var type=sas.ui.mashup.App.getInstance().getCurrentTemplate().getPage().getPageId();
   sas.ui.cpb3.nav.NavUtil.getInstance().navTo(
		this.getCpbForm(),
		{
			event_name:"edit_screen",
			type:type,
			key:key,
			variance:variance,
			route:route
		}
	);	
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.updateObjectInfo=function(){
	var uiContext=this.getCpbForm().getCpbModelManager().getUIContext();
	this.objectType=uiContext.BUSINESSOBJECT.collectionName;
	this.objectId=uiContext.BUSINESSOBJECT.ID;
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.isChangeReasonRequired=function(){
	return this.getCpbForm().getCpbModelManager().getChangeReasonRequired();
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.saveObjectContext=function(uiContext){
	return this.getCpbForm().getCpbAPI().saveObjectContext(this.objectType,this.objectId,uiContext,this.getChangeReason(),this.getNestedSaves(),this.getNestedDeletes());
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.saveError=function(json,errorCallBack){
	this.getCpbForm().getCpbModelManager().applyErrors(json);
	$(this.getCpbForm().getDomRef()).parent().parent().scrollTop(0);
	if(errorCallBack){
		errorCallBack.call();
	}
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.draftSaveCallback=function(){
	sas.ui.cpb3.CpbUtil.silentUpdateHash("#/"+this.objectType+"/"+this.objectId);// need to update hash url from drafts to the object id
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.saveSuccess=function(json,neverBeenSaved,successCallBack){
	if(neverBeenSaved){
		this.draftSaveCallback();
	}
	sas.ui.cpb3.CpbUtil.fireStorageEvent('reload',{type:this.objectType,key:this.objectId},true);
	this.getCpbForm().getCpbModelManager().refresh(json);
	if(successCallBack){
		successCallBack.call();
	}
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.actionSaveWithReason = function(e) {
	var changeReason= e ? e.getParameter("changeReason") : null;
	this.setChangeReason(changeReason);
	var dialog=e.getParameter("dialog");
	var that=this;
	this.actionSave().done(function(response){
		if(!response.hasErrors){
			that.setChangeReason(null);
		}
		dialog.close();
	}).fail(function(e){
		dialog.close();
	});
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.actionSave=function(successCallBack,errorCallBack){
	sas.ui.mashup.App.getInstance().setBusy(true);
	var that=this;
	var objectModel=this.getCpbForm().getCpbModelManager();
	var uiContext=objectModel.getUIContext();
	var neverBeenSaved=objectModel.getNeverBeenSaved();
	this.getCpbForm().updateUIContextWithRequiredFields(); //Update the UIContext with required fields information needed during validation
	return this.saveObjectContext(uiContext).done(function(json){
		if(json.hasErrors){
			that.saveError(json,errorCallBack);
		}else{
			that.saveSuccess(json,neverBeenSaved,successCallBack);
		}
		sas.ui.mashup.App.getInstance().setBusy(false);
	}).fail(function(e){sas.ui.mashup.App.getInstance().setBusy(false);});
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.actionCloseCheck = function(e){
	if(this.saveButton.getEnabled()){
		var rb=this.getCpbForm().getCpbService().getBundle();
		var answer=confirm(rb.getText("form.unsaved.warning.txt"));
		if(answer){
			this.actionClose();
		}
	}else{
		this.actionClose();
	}
};
sas.ui.cpb3.nav.ToolbarActionsObject.prototype.actionClose = function(e) {
	sas.ui.cpb3.nav.NavUtil.getInstance().navTo(/*no way to know parent relationship in pure cpb-land*/
		this.getCpbForm(),
		{
			event_name:"close_object"
		}
	);
};