jQuery.sap.declare("sas.ui.cpb3.parser.Lexer");
jQuery.sap.require("sas.ui.cpb3.parser.Token");
jQuery.sap.require("sas.ui.cpb3.parser.TokenType");

sap.ui.base.Object.extend("sas.ui.cpb3.parser.Lexer",{
	constructor:function(text){
		this.index=0;
		this.text=text;
		this.punctuators={};
		var that=this;
		$.each(sas.ui.cpb3.parser.TokenType.values(),function(idx,tokenType){
			if(tokenType.punctuator){
				that.punctuators[tokenType.punctuator]=tokenType;
			}
		});
	}
});
sas.ui.cpb3.parser.Lexer.prototype.next=function(){
	while(this.index<this.text.length){
		var c=this.text[this.index++];
		var tokenType=this.punctuators[c];
		if(tokenType){
			switch(tokenType){
				case sas.ui.cpb3.parser.TokenType.LESS_THAN:
				case sas.ui.cpb3.parser.TokenType.GREATER_THAN:
					if(this.index<this.text.length){
						var cc=c+this.text[this.index];
						if(this.punctuators[cc]){
							this.index++;
							return new sas.ui.cpb3.parser.Token(this.punctuators[cc],cc);
						}
						break;
					}
				case sas.ui.cpb3.parser.TokenType.BANG:
				   if(this.index<this.text.length){
				      var cc=c+this.text[this.index];
				      var check=this._check(cc);
                      if(check){
                          this.index++;
                          return new sas.ui.cpb3.parser.Token(check,cc);
                      }
                      break;
				   }
				default:
			}
			return new sas.ui.cpb3.parser.Token(this.punctuators[c],c);
		} else if (this._isRecognizedChar(c)) {
			var start=this.index-1;
			var inQuote=(c==='"'||c==="'")?c:false;
			while(this.index<this.text.length){
				var c2=this.text[this.index];
				inQuote=(inQuote && c2!==c);
				if(!this._isRecognizedChar(c2)&&!inQuote){
					break;
				}
				this.index++;
				var check=this._check(this.text.substring(start,this.index));
				if(check)
				   return new sas.ui.cpb3.parser.Token(check, this.text.substring(start,this.index));
			}
			var name=this.text.substring(start,this.index);
			return new sas.ui.cpb3.parser.Token(sas.ui.cpb3.parser.TokenType.NAME,name);
		} else {/*ignore all other characters (whitespace, etc)*/}
	}
	return new sas.ui.cpb3.parser.Token(sas.ui.cpb3.parser.TokenType.EOF,'');
};
sas.ui.cpb3.parser.Lexer.prototype._check=function(str){
   var equal=/^EQ$|^eq$/, less=/^LT$|^lt$/, lessEQ=/^LE$|^le$/, greater=/^GT$|^gt$/,
         greaterEQ=/^GE$|^ge$/, notEQ=/^NE$|^ne$|^!=$/, not=/^NOT$|^not$/, and=/^AND$|^and$/, or=/^OR$|^or$/;
   if(this.index<this.text.length)
      if(this.text[this.index].match(/\.|\w/))
         return null;
   if(str.match(equal))
      return sas.ui.cpb3.parser.TokenType.EQUAL;
   if(str.match(lessEQ))
      return sas.ui.cpb3.parser.TokenType.LESS_THAN_EQUAL;
   if(str.match(less))
      return sas.ui.cpb3.parser.TokenType.LESS_THAN;
   if(str.match(greaterEQ))
      return sas.ui.cpb3.parser.TokenType.GREATER_THAN_EQUAL;
   if(str.match(greater))
      return sas.ui.cpb3.parser.TokenType.GREATER_THAN;
   if(str.match(notEQ))
      return sas.ui.cpb3.parser.TokenType.NOT_EQUAL;
   if(str.match(not))
      return sas.ui.cpb3.parser.TokenType.NOT;
   if(str.match(and))
      return sas.ui.cpb3.parser.TokenType.AND;
   if(str.match(or))
      return sas.ui.cpb3.parser.TokenType.OR;
};
sas.ui.cpb3.parser.Lexer.prototype._isRecognizedChar=function(str){
	return str.length === 1 && str.match(/[a-zA-Z0-9.$'_":|]/i);
};
