jQuery.sap.declare("sas.ui.cpb3.widget.diagram.FlowDiagram");
jQuery.sap.require("sas.ui.cpb3.widget.diagram.ADiagram");
jQuery.sap.require("sas.hc.ui.core.HTMLRenderer");

sas.ui.cpb3.widget.diagram.ADiagram.extend("sas.ui.cpb3.widget.diagram.FlowDiagram", {
	 metadata:{
       properties:{
           options:{type:"any"},
           layout:{type:"any"},
           nodeTemplates:{type:"any"},
           linkTemplates:{type:"any"},
           grid:{type:"any"},
           nodes:{type:"any",defaultValue:[]},
           links:{type:"any",defaultValue:[]}
       }
   },
	renderer: "sas.hc.ui.core.HTMLRenderer"
});
sas.ui.cpb3.widget.diagram.FlowDiagram.prototype.refreshNodesAndLinks=function(){
	var values=this.getValues();
	//links[] -> {from:<i>, to:<j>}
	//nodes[] -> {key:<i>, text:<label>, category:'', status:<ACTIVE,COMPLETE,NOT_STARTED>}
	var map=values;
	if(values && $.isArray(values)){
		map=this.convertFromLinear(values);
	}else if(_.isString(values)){
		map=JSON.parse(values);
	}
	var nodes=map&&map.nodes||[]
	var links=map&&map.links||[];
	this.activateNode(this.getValue(),nodes);
	this.setNodes(nodes);
	this.setLinks(links);
};
sas.ui.cpb3.widget.diagram.FlowDiagram.prototype.activateNode=function(value,nodes){
	var value = this.getValue();
	_.each(nodes,function(node){
		if(node.status==="ACTIVE"){
			node.status==="COMPLETE";
		}
		if(node.key===value){
			node.status==="ACTIVE";
		}
	});
};
sas.ui.cpb3.widget.diagram.FlowDiagram.prototype.convertFromLinear=function(values){
	//get nodes + links
	var nodes=[];
	var links=[];
	var value = this.getValue();
	var foundActive=false;
	var prevNode=null;
	_.each(values,function(obj){
		if(prevNode){
			links.push({from:prevNode.key,to:obj.value});
		}
		var status=value===obj.value?"ACTIVE":foundActive?"NOT_STARTED":"COMPLETE";
		prevNode={key:obj.value,text:obj.label,status:status,category:''}
		nodes.push(prevNode);
		if(status==="ACTIVE"){foundActive=true;}
	});
	return {links:links,nodes:nodes};
};
sas.ui.cpb3.widget.diagram.FlowDiagram.prototype.onBeforeRendering=function(){
	var templates = new sas.ui.cpb3.widget.diagram.FlowTemplates();
	var options=this.getOptions();
	if(options.height){
		this.height=options.height[0];
	}

	this.setContent('<div style="width:100%; height:100%; min-height:'+this.height+'"></div>');
	this.setPreferDOM(false);
	this.setOptions({
		allowDelete : false,
		allowMove : false,
		allowLink : false,
		allowRelink : false,
		allowVerticalScroll : true,
		allowHorizontalScroll : true,
		hasHorizontalScrollbar : true,
		hasVerticalScrollbar : false,
		allowUndo : false,
		initialAutoScale: go.Diagram.Uniform,
		contentAlignment : go.Spot.Left,
		"toolManager.mouseWheelBehavior" : go.ToolManager.WheelZoom,
		padding : 15
	});
	this.setLayout(templates.LAYOUT());
	this.setNodeTemplates({
		"" : templates.DEFAULT_TASK(),
		"_START_" : templates.START_NODE(),
		"_END_" : templates.END_NODE(),
		"_SYNC_":templates.SYNC_BAR()
	});
	this.setLinkTemplates({
		"" : templates.LINK()
	});
};
sas.ui.cpb3.widget.diagram.FlowDiagram.prototype.onAfterRendering=function(e){
	sas.ui.cpb3.widget.diagram.ADiagram.prototype.onAfterRendering.apply(this,arguments);
//    if(!this.firstTime){return;}//skip if not the first time
    if(this.getNodes().length && this.getLinks().length){
        this.diagram=go.GraphObject.make(go.Diagram,e.srcControl.getId(),this.getOptions());
        this.fixMouseWheel();
        this.enableSingleClick();
        this.enableDoubleClick();
        this.centerOnActiveNode();
        this.turnOffDragSelection();
        var d=this.diagram;
        d.layout=this.getLayout();
        _.each(this.getLinkTemplates(),function(value,key){
            d.linkTemplateMap.add(key,value);
        });
        _.each(this.getNodeTemplates(),function(value,key){
            d.nodeTemplateMap.add(key,value);
        });
        this.firstTime=false;
        this.diagram.model=new go.GraphLinksModel(this.getNodes(),this.getLinks());
    }
};
sas.ui.cpb3.widget.diagram.FlowDiagram.prototype.centerOnActiveNode=function(e){
   var that=this;
	// Center on the active node (if any)
   this.diagram.addDiagramListener("InitialLayoutCompleted", function (ev) {
       var nodes = that.getNodes();
       for (i = 0; i < nodes.length; i++) {
           if (nodes[i].status === "ACTIVE") {
               var activeNode = that.diagram.findPartForKey(nodes[i].key);
               that.diagram.centerRect(activeNode.actualBounds);
               that.diagram.select(activeNode);
           }
       }
   });
};
