jQuery.sap.declare("sas.ui.cpb3.widget.diagram.FlowStyles");
//sas.risk.rgf.widget
/**
 * Contains GoJS CSS style utility functions.
 */
sas.ui.cpb3.widget.diagram.FlowStyles = {

    getStyles: function() {

        var styles = [];

        //For cross-origin loaded CSS links, the js code to read or manipulate the styleSheets does not work
        try {
      	  var safeGetRules=function(styleSheet){
      		  var rules=[];
      		  try{
      			  rules=styleSheet.cssRules||styleSheet.rules||[];
      		  }catch(err){
                 console.warn("Loading stylesheet "+styleSheet.href+" failed due to the following message : " + ((!!err.message) ? err.message : err));
      		  }finally{
      			  return rules;
      		  }
      	  };
      	  var processRule=function(cssRule){
              if (cssRule) {
                 var selector = cssRule.selectorText;
                 if (selector) {
                     styles[selector.toLowerCase()] = cssRule;
                 }
             }
      	  };
      	  var processStyleSheet=function(styleSheet){
      		  _.each(safeGetRules(styleSheet),function(cssRule){
      			  if(cssRule instanceof CSSImportRule){
      				  processStyleSheet(cssRule.styleSheet);
      			  }else{
      				  processRule(cssRule);
      			  }
      		  });
      	  };
      	  _.each(document.styleSheets||[],function(styleSheet){
      		  processStyleSheet(styleSheet);
      	  });
        } catch (err) {
            console.log("The getStyles invocation failed due to the following message : " + ((!!err.message) ? err.message : err));
        }

        return styles;

    },

    getColor: function(className) {
        var css = this.styleMap[className.toLowerCase()];
        if (css) {
            var color = css.style["color"];
            if (!color) {
                color = css.style["background-color"];
            }
            if (color) {
                return go.GraphObject.make(go.Brush, go.Brush.Solid, {"color": color});

            }
        }
        return 'white'; // Default for unit tests
    },

    getGradient: function(startColorClassName, endColorClassName) {
        var startCss = this.styleMap[startColorClassName.toLowerCase()];
        var endCss = this.styleMap[endColorClassName.toLowerCase()];
        if (startCss && endCss) {
            var startColor = startCss.style["background-color"];
            var endColor = endCss.style["background-color"];
            if (startColor && endColor) {
                return go.GraphObject.make(go.Brush, go.Brush.Linear, {0: startColor, 1: endColor});
            }
        }
        return 'white'; // Default for unit tests
    },

    getFont: function(className) {
        var css = this.styleMap[className.toLowerCase()];
        if (css) {
            var fontStyle = (css.style["fontStyle"]) ? css.style["fontStyle"] : "";
            var fontVariant = (css.style["fontVariant"]) ? css.style["fontVariant"] : "";
            var fontWeight = (css.style["fontWeight"]) ? css.style["fontWeight"] : "";
            var fontSize = (css.style["fontSize"]) ? css.style["fontSize"] : "";
            var fontFamily = (css.style["fontFamily"]) ? css.style["fontFamily"] : "";
        }
        return fontStyle + " " + fontVariant + " " + fontWeight + " " + fontSize + " " + fontFamily;
    },

    getStrokeWidth: function(className) {
        var styles = this.styleMap[className.toLowerCase()];
        if (styles) {
            return parseInt(styles.style["width"]);
        }
        return 1.0; // Default for unit tests
    },

    getDimensions: function(className) {
        var dimensions = {};
        var styles = this.styleMap[className.toLowerCase()];
        if (styles) {
            dimensions.width = parseInt(styles.style["width"]);
            dimensions.height = parseInt(styles.style["height"]);
            dimensions.margin = parseInt(styles.style["margin"]);
        }
        return dimensions;
    },

    init: function(){

        this.styleMap = this.getStyles();
        var styles = {};

        // Start node styling
        styles.startNodeFillColor = this.getGradient(".cpbFlowDiagramStartNode", ".cpbFlowDiagramStartNodeEnd");
        styles.startNodeStrokeColor = this.getColor(".cpbFlowDiagramStartBorder");
        styles.startNodeStrokeWidth = this.getStrokeWidth(".cpbFlowDiagramStartBorder");
        styles.startNodeWidth = this.getDimensions(".cpbFlowDiagramStartNode").width;
        styles.startNodeHeight = this.getDimensions(".cpbFlowDiagramStartNode").height;

        // End node styling
        styles.endNodeFillColor = this.getGradient(".cpbFlowDiagramEndNode", ".cpbFlowDiagramEndNodeEnd");
        styles.endNodeStrokeColor = this.getColor(".cpbFlowDiagramEndBorder");
        styles.endNodeStrokeWidth = this.getStrokeWidth(".cpbFlowDiagramEndBorder");
        styles.endNodeWidth = this.getDimensions(".cpbFlowDiagramEndNode").width;
        styles.endNodeHeight = this.getDimensions(".cpbFlowDiagramEndNode").height;

        // Task node styling
        styles.taskNodeFillColor = this.getGradient(".cpbFlowDiagramTaskNode", ".cpbFlowDiagramTaskNodeEnd");
        styles.taskNodeActiveFillColor = this.getGradient(".cpbFlowDiagramActiveTaskNode", ".cpbFlowDiagramActiveTaskNodeEnd");
        styles.taskNodeStrokeColor = this.getColor(".cpbFlowDiagramTaskNodeBorder");
        styles.taskNodeStrokeWidth = this.getStrokeWidth(".cpbFlowDiagramTaskNodeBorder");
        styles.taskNodeWidth = this.getDimensions(".cpbFlowDiagramTaskNode").width;
        styles.taskNodeHeight = this.getDimensions(".cpbFlowDiagramTaskNode").height;
        styles.taskNodeFont = this.getFont(".cpbFlowDiagramTaskNodeText");
        styles.taskNodeTextColor = this.getColor(".cpbFlowDiagramTaskNodeText");
        styles.taskNodeTextMargin = this.getDimensions(".cpbFlowDiagramTaskNodeText").margin;

        // Link styling
        styles.linkStrokeColor = this.getColor(".cpbFlowDiagramLink");
        styles.linkStrokeWidth = this.getStrokeWidth(".cpbFlowDiagramLink");
        styles.linkFont = this.getFont(".cpbFlowDiagramLink");
        styles.linkMargin = this.getDimensions(".cpbFlowDiagramLink").margin;
        styles.linkBackground = this.getColor(".cpbFlowDiagramTooltip");

        // Common
        styles.tooltipFill = this.getColor(".toolTips");


        return styles;

    }
};
