jQuery.sap.declare("sas.ui.cpb3.widget.diagram.FlowTemplates");
jQuery.sap.require("sap.ui.base.Object");
jQuery.sap.require("sas.ui.cpb3.widget.diagram.FlowStyles");

sap.ui.base.Object.extend("sas.ui.cpb3.widget.diagram.FlowTemplates", {
    constructor: function () {
        this.GO = go.GraphObject.make;
        this.pfdStyles = sas.ui.cpb3.widget.diagram.FlowStyles.init();
    }
});

sas.ui.cpb3.widget.diagram.FlowTemplates.prototype.START_NODE = function () {
    return this.GO(go.Node, "Auto",
        new go.Binding("location", "", this.getLocation),
        {
            fromSpot: go.Spot.Right,
            selectionAdorned: false,
            locationObjectName: "START_NODE"
        },
        this.GO(go.Shape, "Circle", {
                name: "START_NODE",
                strokeWidth: this.pfdStyles.startNodeStrokeWidth,
                fill: this.pfdStyles.startNodeFillColor,
                stroke: this.pfdStyles.startNodeStrokeColor,
                width: this.pfdStyles.startNodeWidth,
                height: this.pfdStyles.startNodeHeight
            }
        )
    );
};
sas.ui.cpb3.widget.diagram.FlowTemplates.prototype.SYNC_BAR = function () {
   return this.GO(
       go.Node,
       "Auto",
       {
           fromSpot: go.Spot.Right,
           toSpot: go.Spot.LeftSide
       },
       this.GO(
           go.Shape,
           "Rectangle",
           {
               fill: this.pfdStyles.linkStrokeColor,
               stroke: this.pfdStyles.linkStrokeColor,
               desiredSize: new go.Size(5, 20)
           }
       )
   );
};
sas.ui.cpb3.widget.diagram.FlowTemplates.prototype.END_NODE = function () {
    return this.GO(go.Node, "Auto",
        new go.Binding("location", "", this.getLocation),
        {
            fromSpot: go.Spot.Right,
            selectionAdorned: false,
            locationObjectName: "END_NODE"
        },
        this.GO(go.Shape, "Circle", {
                name: "END_NODE",
                strokeWidth: this.pfdStyles.startNodeStrokeWidth,
                fill: this.pfdStyles.endNodeFillColor,
                stroke: this.pfdStyles.endNodeStrokeColor,
                width: this.pfdStyles.endNodeWidth,
                height: this.pfdStyles.endNodeHeight
            }
        )
    );
};

sas.ui.cpb3.widget.diagram.FlowTemplates.prototype.DEFAULT_TASK = function () {

    var that = this;

    return this.GO(go.Node, "Spot",
        {
            selectionAdornmentTemplate:
                this.GO(go.Adornment, "Auto",
                    this.GO(go.Shape, "RoundedRectangle",
                        {
                            fill: null,
                            stroke: this.pfdStyles.taskNodeStrokeColor,
                            strokeWidth: 3
                        }),
                    this.GO(go.Placeholder)
                )
        },
        new go.Binding("location", "", this.getLocation),
        this.GO(go.Panel, "Auto", {},
            this.GO(go.Shape, "RoundedRectangle",
                {
                    name: "nodeShape",
                    parameter1: 3, // Corner radius
                    fill: this.pfdStyles.taskNodeFillColor,
                    strokeWidth: this.pfdStyles.taskNodeStrokeWidth,
                    stroke: this.pfdStyles.taskNodeStrokeColor,
                    portId: "",
                    fromSpot: go.Spot.RightSide,
                    toSpot: go.Spot.LeftSide,
                    width: this.pfdStyles.taskNodeWidth,
                    height: this.pfdStyles.taskNodeHeight
                },
                new go.Binding("fill", "status", function (status) {
                    return (status === "NOT_STARTED") ? that.pfdStyles.taskNodeActiveFillColor : that.pfdStyles.taskNodeFillColor;
                }),
                new go.Binding("strokeWidth", "status", function (status) {
                    return (status === "ACTIVE") ? that.pfdStyles.taskNodeStrokeWidth + 2 : that.pfdStyles.taskNodeStrokeWidth;
                })
            ),
            this.GO(go.TextBlock, {
                    alignment: go.Spot.TopLeft,
                    maxSize: new go.Size(this.pfdStyles.taskNodeWidth - 30, this.pfdStyles.taskNodeHeight - 20),
                    margin: new go.Margin(5, 5, 5, 5),
                    wrap: go.TextBlock.WrapFit,
                    font: this.pfdStyles.taskNodeFont,
                    stroke: this.pfdStyles.taskNodeTextColor,
                    overflow: go.TextBlock.OverflowEllipsis
                },
                new go.Binding("text", "text")
            ),
            this.GO(go.Panel, "Viewbox",{
            		desiredSize: new go.Size(16,16),
            		alignment: go.Spot.BottomRight,
                    margin: 3
            	},
               this.GO(
                     go.Picture,
                     {
                         desiredSize: new go.Size(16,16),
                         source: sas.icons.HC.COMPLETESTATUS,
                         visible: false
                     },
                     new go.Binding("visible", "", function (task) {
                         return task.status !== "NOT_STARTED" && task.reset != true;
                     }),
                     new go.Binding("source", "", function (task) {
                         var status = task.status;
                         switch (status) {
                             case "ACTIVE":
                                 return sas.icons.HC.REVIEWSTATUS;
                                 break;
                             case "STARTED":
                                return sas.icons.HC.REVIEWSTATUS;
                                break;
                             case "COMPLETE":
                                 return sas.icons.HC.COMPLETESTATUS;
                                 break;
                             case "RUNNING":
                                return sas.icons.HC.ANIMATEDPROGRESSINDICATOR_GOJS;
                                break;
                             case "FAILED":
                                return sas.icons.HC.ERRORSTATUSNODE_GOJS;
                                break;
                             case "task.status.approved.txt":
                                 return sas.icons.HC.COMPLETESTATUS;
                                 break;
                             case "task.status.rejected.txt":
                                 return sas.icons.HC.INVALIDSTATUS;
                                 break;
                             case "Approved":
                                 return sas.icons.HC.COMPLETESTATUS;
                                 break;
                             default:
                                 // TODO: Use something different for custom?
                                 return sas.icons.HC.INFORMATIONSTATUS;
                                 break;
                         }
                     })
                 )
      		),
      		this.GO(go.Panel, "Viewbox", {
               desiredSize: new go.Size(16,16),
               alignment: go.Spot.BottomCenter,
               margin: new go.Margin(4,4,4,4)
             },
             this.GO(go.Picture, {
               desiredSize: new go.Size(16,16),
               visible: !0,
               source: sas.icons.HC.ADDSVG
             }), 
             new go.Binding("visible","",function(t) {
                return !!t.drill
             })
          )
        ),
        {toolTip:
            this.GO(go.Adornment, "Auto",
                this.GO(go.Shape, { fill: this.pfdStyles.tooltipFill }),
                this.GO(go.TextBlock, { margin: 4 },
                    new go.Binding("text", "text"))
            )
        }
    );
};


sas.ui.cpb3.widget.diagram.FlowTemplates.prototype.LINK = function () {
    return this.GO(go.Link, {
            routing: go.Link.Normal,
            curve: go.Link.JumpGap,
            corner: 10,
            reshapable: false,
            relinkableTo: false,
            toEndSegmentLength: 20,
            toolTip: this.GO(go.Adornment, "Auto",
                this.GO(go.Shape, {
                    fill: this.pfdStyles.linkBackground
                }),
                this.GO(go.TextBlock, {
                        margin: this.pfdStyles.linkMargin,
                        font: this.pfdStyles.linkFont,
                        stroke: this.pfdStyles.linkStrokeColor
                    },
                    new go.Binding("text", "text"),
                    new go.Binding("visible", "text", function (text) {
                        return text.length > 0;
                    })
                )
            )
        },
        this.GO(go.Shape, {
            isPanelMain: true,
            stroke: this.pfdStyles.linkStrokeColor,
            strokeWidth: this.pfdStyles.linkStrokeWidth

        }),
        this.GO(go.Shape, {
            toArrow: "Triangle",
            scale:.75,
            fill: this.pfdStyles.linkStrokeColor,
            stroke: this.pfdStyles.linkStrokeColor
        })
    );
};

sas.ui.cpb3.widget.diagram.FlowTemplates.prototype.LAYOUT = function () {
    return this.GO(
        go.LayeredDigraphLayout,
        {
            columnSpacing: 5,
            setsPortSpots: false,
            cycleRemoveOption: go.LayeredDigraphLayout.CycleGreedy,
            layeringOption:go.LayeredDigraphLayout.LayerOptimalLinkLength,
            packOption: go.LayeredDigraphLayout.PackStraighten
        }
    );
};

sas.ui.cpb3.widget.diagram.FlowTemplates.getLocation = function (data, node) {
    return new go.Point(data.x, data.y);
};
