jQuery.sap.declare("sas.ui.cpb3.widget.diagram.GraphTemplateFactory");
jQuery.sap.require("sas.ui.cpb3.widget.diagram.GraphStyles");
jQuery.sap.require("sas.ui.cpb3.widget.diagram.GraphUtils");

sas.ui.cpb3.widget.diagram.GraphTemplateFactory=function(rootKey,styleId,bundle,callback){
   this.rootKey=rootKey;
	this.styleId=styleId;
   this.bundle=bundle;
   this.callback=callback;
};
sas.ui.cpb3.widget.diagram.GraphTemplateFactory.prototype={
   createNodeTemplates:function(){
      this.flowStyles=new sas.ui.cpb3.widget.diagram.GraphStyles(this.styleId);
      var templateNames=this.flowStyles.getNodeTemplateNames();
      var map=new go.Map("string",go.Node);
      var that=this;
      _.each(templateNames,function(name){
         map.add(name,that.createNodeTemplate(name));
      });
      return map;
   },
   createLinkTemplates:function(){
      this.flowStyles=new sas.ui.cpb3.widget.diagram.GraphStyles(this.styleId);
      var templateNames=this.flowStyles.getLinkTemplateNames();
      var map=new go.Map("string",go.Link);
      var that=this;
      _.each(templateNames,function(name){
         map.add(name,that.createLinkTemplate(name));
      });
      return map;
   },
   createNodeTemplate:function(name){
//   	{
//   		font:"28px sas-icons",
//   		fontColor:"#4e4e4e",
//   		background:"linear-gradient(#f3d4d4,#fbeaea)",
//   		borderColor:"#dd5a57"
//    }
   	var style=this.flowStyles.getStyle(name,"N");
      var that=this;
      var icon=go.GraphObject.make(
         go.TextBlock,
         { 
            margin: 9,
            font:style.font,
            stroke:style.fontColor,
            text:this.flowStyles.getIcon(style,name,"N")
         }
      );
      var gradientInfo=/.*\((r.*),.*(r.*)\)/gm.exec(style.background);//"linear-gradient(rgb(240, 240, 240), rgb(233, 234, 235))"
      if(!gradientInfo){
      	gradientInfo=/.*\((#.*),.*(#.*)\)/gm.exec(style.background);//"linear-gradient(#geaded, #fefefe)"
      }
      var circle=go.GraphObject.make(
         go.Shape,
         "Circle",
         {
            spot1: new go.Spot(0, 0),
            spot2: new go.Spot(1, 1),
            strokeWidth: 1,
            fill:go.GraphObject.make(go.Brush, go.Brush.Linear, {"0": gradientInfo[1], "1": gradientInfo[2]})
         },
         new go.Binding("stroke", "selected", function(s) { 
            return s?"black":style.borderColor;
         })
      );
      var button=this.createButton(style,name);
      var tooltip=this.createTooltip(style,name);
      var selection=go.GraphObject.make(
         go.Panel,
         "Spot",
         go.GraphObject.make(
            go.Shape,
            "Circle",
            {
               margin:0,
               fill: null,
               stroke: style.borderColor,
               strokeWidth: 1 ,
               width:46,
               visible:false
            },
            new go.Binding("visible", "selected", function(s) { 
               return !!s;
            })
         ),
         go.GraphObject.make(
            go.Shape,
            "Circle",
            {
               margin:0,
               fill: null,
               stroke: "white",
               strokeWidth: 1 ,
               width:56,
               visible:false
            },
            new go.Binding("visible", "selected", function(s) { 
               return !!s;
            })
         )
         //go.GraphObject.make(go.Placeholder)
      );
      var nodeTemplate=go.GraphObject.make(
         go.Node, 
         "Auto",
         circle,
         selection,
         icon,
         //{selectionAdornmentTemplate:selection},
         {toolTip:tooltip}
      );
      if(this.callback){
      	nodeTemplate.add(button);
      }
      return nodeTemplate;
   },
   createButton:function(style,name){
   	var that=this;
   	var buttonText=go.GraphObject.make(
         go.TextBlock,
         {
            name:"ExpandButtonText",
            font:"13px sas-icons",
            stroke:style.fontColor,
            text:String.fromCharCode(62635)//62635 62632
         },
         new go.Binding("text","selected",function(s){
            return String.fromCharCode(s?62632:62635);
         })
      );
      var button=go.GraphObject.make(
         "Button",
         {
            name:"ExpandButton",
            "ButtonBorder.figure":"Circle",
            "ButtonBorder.fill":"white",
            "ButtonBorder.stroke":style.borderColor,
            "ButtonBorder.spot1": new go.Spot(0, 0),
            "ButtonBorder.spot2": new go.Spot(1, 1),
            width:16,
            height:16,
            alignment: go.Spot.TopRight,
            alignmentFocus: go.Spot.BottomLeft,
            click:function(e,obj){
               var buttonTxt=obj.findObject("ExpandButtonText");
               var isExpanded=buttonTxt.text.charCodeAt(0)===62632;
               buttonTxt.text=isExpanded?String.fromCharCode(62635):String.fromCharCode(62632);
               e.handled=true;
               if(that.callback){
                  var diagram=obj.diagram;
                  var model=diagram.model;
                  var objData=obj.part.data;
                  diagram.startTransaction("merge");
                  if(isExpanded){
                  	objData.expanded=false;
                     var collapseMap=sas.ui.cpb3.widget.diagram.GraphUtils.getCollapseableNodes(objData.key,that.rootKey,model.linkDataArray,model.nodeDataArray);
                     _.each(collapseMap.nodeKeys,function(newNodeKey){
                        model.removeNodeData(model.findNodeDataForKey(newNodeKey));
                     });
                     var newLinks=_.filter(model.linkDataArray,function(linkData){
                        return collapseMap.linkKeys.indexOf(linkData.key)!==-1;
                     });
                     var nodeKeysThatLostLinks=[];
                     _.each(newLinks,function(newLink){
                        nodeKeysThatLostLinks.push(newLink.to);
                        nodeKeysThatLostLinks.push(newLink.from);
                        model.removeLinkData(newLink);
                     });
                     _.each(_.uniq(nodeKeysThatLostLinks),function(nodeKey){
                        var node=diagram.findNodeForKey(nodeKey);
                        if(node){
                           node.findObject("ExpandButtonText").text=String.fromCharCode(62635);
                        }
                     });
                     diagram.commitTransaction("merge");
                  }else{
                  	objData.expanded=true;
                     that.callback(obj.part.data).done(function(data){
                        var newNodes=_.filter(data.nodes,function(node){
                           return !model.findNodeDataForKey(node.key);
                        });
                        var linkKeys=_.pluck(model.linkDataArray,"key");
                        var newLinks=_.filter(data.links,function(link){
                           return linkKeys.indexOf(link.key)===-1;
                        });
                        _.each(newNodes,function(newNode){
                           model.addNodeData(newNode);
                        });
                        _.each(newLinks,function(newLink){
                           model.addLinkData(newLink);
                        });
                        if(!newNodes.length && !newLinks.length){
                           obj.findObject("ExpandButton").visible=false;
                        }                     	
                        diagram.commitTransaction("merge");
                     });
                  }
                  //diagram.zoomToFit();
               }
            }
         },
         buttonText
      );
      return button;
   },
   createTooltip:function(style,name){
   	var that=this;
   	return go.GraphObject.make(
         go.Adornment,
         "Auto",
         go.GraphObject.make(
            go.Shape,
            {
               fill: "#EEEEEE"
            }
         ),
         go.GraphObject.make(
            go.Panel,
            "Vertical",
            go.GraphObject.make(
               go.TextBlock,
               { margin: 10, textAlign:"start",isMultiline:true },
               new go.Binding("text", "", function(obj) {
               	var overriden=that.bundle.hasText(that.styleId+"_graphdiagram.node.tooltip.fmt.txt");
               	var label=overriden?that.bundle.getText(that.styleId+"_graphdiagram.node.tooltip.fmt.txt",[obj.type,obj.id,obj.name,obj.key]):
         				 					  that.bundle.getText("graphdiagram.node.tooltip.fmt.txt",[obj.type,obj.id,obj.name,obj.key]);
               	return label; 
            	})
            )
         )
      );
   },
   createLinkTemplate:function(name){
   	/**
   	 * {
   	 * 	lineStyle:"solid",
   	 * 	lineColor:"#b4b4b4",
   	 * 	fromArrow:true,
   	 * 	toArrow:true
   	 * }
   	 */
   	var style=this.flowStyles.getStyle(name,"L");
      var borderStyle=style.lineStyle;
      var dashes=borderStyle==="solid"?[]:[6,6];
      var line=go.GraphObject.make(
         go.Shape,
         {
            strokeWidth: 1,
            stroke: style.lineColor,
            strokeDashArray: dashes
         }
      );
      var link=go.GraphObject.make(
         go.Link,
         { 
            relinkableFrom: false,
            relinkableTo: false
         },
         line
      );
      var toArrowHead=style.toArrow;
      if(toArrowHead){
	      var toArrow=go.GraphObject.make(
	         go.Shape,
	         {
	            toArrow: "Standard",
	            stroke: null,
	            fill: style.lineColor
	         }
	      );
   		link.toShortLength=3;
      	link.add(toArrow);
      }
      var fromArrowHead=style.fromArrow;
      if(fromArrowHead){
         var fromArrow=go.GraphObject.make(
            go.Shape,
            {
               fromArrow: "Backward",
               stroke: null,
               fill: style.lineColor
            }
         );
   		link.fromShortLength=3;
      	link.add(fromArrow);
      }
      return link;
   }
};