jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.mashup.api.AppAPI");
jQuery.sap.require("sas.ui.mashup.widget.LeftNav");
jQuery.sap.require("sas.ui.mashup.model.AppModel");
jQuery.sap.require("sas.ui.mashup.model.PageModel");
jQuery.sap.require("sas.ui.mashup.util.NavUtil");
jQuery.sap.require("sas.hc.m.IconTabBar");
jQuery.sap.require("sas.hc.m.IconTabFilter");

sas.hc.ui.jsappview("sas.ui.mashup.App",{
	getControllerName : function() {
		return "sas.ui.mashup.App";
	},
	detachBeforeMenuNavigation:function(f,o){
		this.detachEvent("beforeMenuNavigation",f,o);
	},
	attachBeforeMenuNavigation:function(d,f,o){
		this.attachEvent("beforeMenuNavigation",d,f,o);
	},
	fireBeforeMenuNavigation:function(p){
		return this.fireEvent("beforeMenuNavigation",p,true/*allowPreventDefault*/,false/*enableEventBubbling*/);
	},
	createContent : function(controller) {
		sas.ui.mashup.App._instance=this;
		this.appModel=new sas.ui.mashup.model.AppModel(this);
		this.appId=_MASHUP_contentId;//set on index.jps as global
		this.perspectiveId=_MASHUP_perspectiveId;
		this.navUtil=new sas.ui.mashup.util.NavUtil(this);
		this.layout = new sas.risk.common.layout.flexlayout.FlexLayout({
			width : "100%",
			height : "100%",
			flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column
		}).addStyleClass("MashupApp");
		this.layout.addItem(this.createLeftNav());
		this.layout.addItem(this.createToolbar());
		this.layout.addItem(this.createMenuBar());
		this.layout.addItem(this.createMenuBar2());
		this.target = new sas.risk.common.layout.flexlayout.FlexLayout('MashupAppArea',{
			width : "100%",
			height : "0px",
			flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column
		}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({flexGrow:2}));
		this.origAddItem=this.target.addItem;
		this.target.addItem=$.proxy(this.pageLoading,this);
		this.origRemoveAllItems=this.target.removeAllItems;
		this.target.removeAllItems=$.proxy(this.pageUnloading,this);
		this.layout.addItem(this.target);
		return this.layout;
	},
	createLeftNav:function(){
		var that=this;
		this.leftNav=new sas.ui.mashup.widget.LeftNav({
			visible:sas.ui.mashup.model.PageModel.bindLeftNavVisible(),
			menu:this.appModel.bindMenu(),
			selectedMenu:this.appModel.bindSelectedMenu(),
			beforeItemSelection:function(e){
				if(!that.fireBeforeMenuNavigation(e.getParameters())){
					e.preventDefault();
				}
			},
			selected:function(e){
				var menu=e.getParameter("menu");
				if(menu.getPageId()){
					that.navUtil.navTo(menu.getPageId());
				}else if(menu.getChildren().length){
					_.find(menu.getChildren(),function(child){
						targetMenu=child.getPageId()?child:_.find(child.getChildren(),function(grandChild){return !!grandChild.getPageId();});
						return targetMenu;
					});
					if(targetMenu){
						that.navUtil.navTo(targetMenu.getPageId());
					}
				}
			}
		});
		return this.leftNav;
	},
	createToolbar:function(){
		var that=this;
		this.title=new sas.hc.m.Heading('MashupMainToolbarTitle',{
			text:{
				parts:[
					sas.ui.mashup.model.PageModel.bindTitlePath(),
					this.appModel.bindSelectedMenuPath(),
					sas.ui.mashup.model.PageModel.bindMenuBarVisiblePath()],
				formatter:function(pageTitle,selectedMenu,visibleSetting){
					var menuBarShowing=false;
					if(selectedMenu&&visibleSetting){
						var secondLevel=selectedMenu.findLevel(2);
						menuBarShowing=!!secondLevel;
					}
					if(menuBarShowing){
						that.toolbar.removeStyleClass("MashupShowSepBar");
					}else{
						that.toolbar.addStyleClass("MashupShowSepBar");
					}
					return menuBarShowing?selectedMenu.findLevel(1).getText():pageTitle;
				}
			},
			level:2
		});
		this.actions=new sas.risk.common.layout.flexlayout.FlexLayout('MashupMainToolbarActions',{
			items:{
				path:sas.ui.mashup.model.PageModel.bindToolbarsPath(),
				factory:function(itemId,context){
					return context.getObject().clone();
				}
			}
		}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
			flexShrink:0
		}));
//		this.actions.bUseExtendedChangeDetection=true;
		this.toolbar=new sas.risk.common.layout.flexlayout.FlexLayout('MashupMainToolbar',{
			visible:sas.ui.mashup.model.PageModel.bindToolbarVisible(),
			justifyContent:sas.risk.common.layout.flexlayout.FlexLayoutJustifyContent.SpaceBetween,
			alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
			items:[this.title,this.actions]
		}).setLayoutData(new sas.risk.common.layout.flexlayout.FlexLayoutData({
			alignSelf:sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.Stretch
		}));
		return this.toolbar;
	},
	createMenuBar:function(){
		var that=this;
		this.menuBar=new sas.hc.m.IconTabBar("MashupMainMenu",{
			visible:{
				parts:[this.appModel.bindSelectedMenuPath(),sas.ui.mashup.model.PageModel.bindMenuBarVisiblePath()],
				formatter:function(selectedMenu,visibleSetting){
					if(selectedMenu&&visibleSetting){
						var secondLevel=selectedMenu.findLevel(2);
						return !!secondLevel;
					}
					return false;
				}
			},
			items:{
				path:this.appModel.bindSelectedSecondLevelMenusPath(),
				factory:function(sId,context){
					var menu=context.getObject();
					if(menu){
						var item=new sas.hc.m.IconTabFilter({
							text:menu.getText(),
							tooltip:menu.getText(),
							design: "Horizontal",
							icon:menu.getIcon(),
							key:menu.getId()
						});
						item.addCustomData(new sap.ui.core.CustomData({
							key:"menu",
							value:menu,
							writeToDom:false
						}));
						return item;
					}
					return null;
				}
			},
			selectedKey:{
				path:this.appModel.bindSelectedMenuPath(),
				formatter:function(selectedMenu){
					var secondLevel=selectedMenu&&selectedMenu.findLevel(2);
					return secondLevel?secondLevel.getId():null;
				}
			},
			enableDynamicStyle: true,
			tabChanging:function(e){
				if(!that.fireBeforeMenuNavigation(e.getParameters())){
					e.preventDefault();
				}
			},
			select:function(event){
				var customData=event.getParameter("item").getCustomData()[0];
				var menuDef=customData.getValue();
				if(menuDef.getChildren().length){
					menuDef=menuDef.getChildren()[0];
				}
				that.navUtil.navTo(menuDef.getPageId(),menuDef.getContext());
			}
		}).addStyleClass("MashupMenuBar");
		return this.menuBar;
	},
	createMenuBar2:function(){
		var that=this;
		this.menuBar2=new sas.hc.m.IconTabBar("MashupSecondMenu",{
			visible:{
				parts:[this.appModel.bindSelectedMenuPath(),sas.ui.mashup.model.PageModel.bindMenuBarVisiblePath()],
				formatter:function(selectedMenu,visibleSetting){
					if(selectedMenu&&visibleSetting){
						var thirdLevel=selectedMenu.findLevel(3);
						return !!thirdLevel;
					}
					return false;
				}
			},
			items:{
				path:this.appModel.bindSelectedThirdLevelMenusPath(),
				factory:function(sId,context){
					var menu=context.getObject();
					if(menu){
						var item=new sas.hc.m.IconTabFilter({
							text:menu.getText(),
							tooltip:menu.getText(),
							design: "Horizontal",
							icon:menu.getIcon(),
							key:menu.getId()
						});
						item.addCustomData(new sap.ui.core.CustomData({
							key:"menu",
							value:menu,
							writeToDom:false
						}));
						return item;
					}
					return null;
				}
			},
			selectedKey:{
				path:this.appModel.bindSelectedMenuPath(),
				formatter:function(selectedMenu){
					var thirdLevel=selectedMenu&&selectedMenu.findLevel(3);
					return thirdLevel?thirdLevel.getId():null;
				}
			},
			enableDynamicStyle: false,
			tabChanging:function(e){
				if(!that.fireBeforeMenuNavigation(e.getParameters())){
					e.preventDefault();
				}
			},
			select:function(event){
				var customData=event.getParameter("item").getCustomData()[0];
				var menuDef=customData.getValue();
				if(menuDef.getChildren().length){
					menuDef=menuDef.getChildren()[0];
				}
				that.navUtil.navTo(menuDef.getId(),menuDef.getContext());
			}
		}).addStyleClass("MashupMenuBar");
		return this.menuBar2;
	},
	getAppInfo:function(){
		return this.appInfo;
	},
	getCurrentTemplate:function(){
		return this.currentTemplate;
	},
	pageUnloading:function(){
		var removedItems=this.origRemoveAllItems.apply(this.target,arguments);
		return removedItems;
	},
	pageLoading:function(template){
		if(this.navUtil.isRefreshing()){
			template.getController().onLeave();
		}
		this.currentTemplate=template;
		var page=template.getPage();
		var pageId=page.getPageId();
		template.addStyleClass("Mashup_"+pageId);
		template.getPageModel().setActive(this);
		var appInfo=this.appInfo;
		var mainMenu=appInfo.getMenu();
		var menu=mainMenu.findMenuItem(page.getMenuAnchor());
		this.appModel.setSelectedMenu(menu);
		return this.origAddItem.apply(this.target,arguments);
	},
	setBusy:function(value,delay){
//		this.setBusyIndicatorDelay(delay||0);
//		sap.ui.core.Control.prototype.setBusy.call(this,value);
		this.layout.setBusyIndicatorDelay(delay||0);
		this.layout.setBusy(value);
	}
});
sap.ui.controller("sas.ui.mashup.App",{
	onInit:function(){
		var view=this.getView();
		view.setBusy(true);
  		$.when(
			sas.ui.mashup.api.AppAPI.getAppInfo()
		).done(function(appInfo){
			view.appInfo=appInfo;
			view.banner.setText(appInfo.getTitle());
			view.banner.setShowSearch(appInfo.getGlobalSearch());
			view.banner.setHelpUrl(appInfo.getHelpUrl());
			view.appModel.setMenu(appInfo.getMenu());
			var router=sap.ui.core.routing.Router.getRouter("router");
			_.each(appInfo.createRoutes(),function(route){
				router.addRoute(route);
			});
			router.attachEventOnce("routeMatched",function(e){
				view.navUtil.firstLoadPrimeTheStack(e.getParameter("name"),e.getParameter("arguments"));
			});
			router.initialize();
			view.setBusy(false);
		});
	},
	onMashup:function(e,template){
		var eventName=e.getParameter("event_name");
		var action=_.find(template.getPage().getActions(),function(action){return eventName===action.event;});
		var params=e.getParameters();
		if(action){
			var actionProcessor=sas.ui.mashup.ActionProcessorRegistry.get(action.processor);
			if(actionProcessor){
				action=actionProcessor.processEvent(e,action);
			}
			this.getView().navUtil.navTo(action.pageId,params,action.route,action.navstyle);
		}
	}
});
//STATICS
jQuery.sap.declare("sas.ui.mashup.App");
sas.ui.mashup.App={};
sas.ui.mashup.App._instance;
sas.ui.mashup.App.getInstance=function(){
	return sas.ui.mashup.App._instance;
};