jQuery.sap.declare("sas.ui.mashup.api.AppAPI");
jQuery.sap.require("sas.ui.mashup.biz.AppInfo");
jQuery.sap.require("sas.ui.mashup.util.PerspectiveUtil");

sas.ui.mashup.api.AppAPI={
	_loadedLibs:{},
	_loadedApps:{},
	_debugCache:null,
	_jsonCache:{},
	_propsCache:{},
	getAppInfo:function(){
		var def=$.Deferred();
		var appId=_MASHUP_contentId;
		if(sas.ui.mashup.api.AppAPI._loadedApps[appId]){
			def.resolve(sas.ui.mashup.api.AppAPI._loadedApps[appId]);
			return def;
		}
		var appKey=sas.ui.mashup.util.PerspectiveUtil.getAppKey();
		var url="rest/mashup/"+appKey;
		$.getJSON(url,function(json){
			var appInfo=sas.ui.mashup.biz.AppInfo.create(json);
			sas.ui.mashup.api.AppAPI._loadedApps[appId]=appInfo;
			def.resolve(appInfo);
		});
		return def;
	},
	loadAppLibs:function(appId){
		var def=$.Deferred();
		var debugOn=!!sap.ui.getCore().getConfiguration().getDebug();
		var appKey=sas.ui.mashup.util.PerspectiveUtil.getAppKey();
      var url="rest/mashup/"+appKey+"/libs?debug="+debugOn;
		$.getJSON(url,function(json){
   		_.each(json,function(lib){
   			var cssArray={};
   			var jsonArray={};
   			var propsArray={};
   			if(lib.modules){//js library
      			var libName=lib.name;
      			sas.ui.mashup.api.AppAPI._loadedLibs[libName]=lib;
      			if(debugOn){
      				sas.ui.mashup.api.AppAPI._debugCache={};
      				_.each(lib.modules,function(source,fileName){
      					sas.ui.mashup.api.AppAPI._debugCache[fileName]=libName;
      				});
      			}
      			sap.ui.getCore().loadLibrary(libName,"MASHUP_PRELOAD:"+libName+":");
      			_.each(lib.css,function(csslib){
      				cssArray[csslib.name]=csslib.css;
      			});
      			_.each(lib.json,function(jsonlib){
      				jsonArray["resources/"+jsonlib.name]=jsonlib.json;
      			});
      			_.each(lib.properties,function(propslib){
      				propsArray[propslib.name]=propslib.properties;
      			});
   			}else{//css library
   				cssArray[lib.name]=lib.css;
   			}
   			_.each(cssArray,function(css,cssName){
					var styles=document.createElement("style");
//					styles.id="sap-ui-theme-"+libName;
					styles.id=cssName;
					styles.type="text/css";
					$(styles).html(css);
					document.getElementsByTagName("head")[0].appendChild(styles);
   			});
   			_.each(jsonArray,function(jsonText,jsonName){
   				sas.ui.mashup.api.AppAPI._jsonCache[jsonName]=jsonText;
   			});
   			_.each(propsArray,function(propsText,propsName){
   				sas.ui.mashup.api.AppAPI._propsCache[propsName]=propsText;
   			});
   		});
   		def.resolve();
		});
		return def;		
	}
};
$.ajaxPrefilter(function(options){
	if(options.url && options.url.startsWith("MASHUP_PRELOAD")){
		return "mashup";
	}else if(options.url && sas.ui.mashup.api.AppAPI._jsonCache[options.url]){
		return "mashup_json";
	}else if(options.url && sas.ui.mashup.api.AppAPI._debugCache){
		var index=options.url.indexOf("/sas/");
		if(index!==-1){
			var path=options.url.substring(index+1);
			var lib=sas.ui.mashup.api.AppAPI._debugCache[path];
			if(lib){
				options.url="MASHUP_PRELOAD:"+lib+":"+path.substring();
				return "mashup";
			}
		}
	}
});
$.ajaxTransport("mashup",function(options,originalOptions,jqXHR){
	options.dataTypes.shift();//take "mashup" dataType off the list
	var mainType=options.dataTypes[0];
	var responses={};
	var split=options.url.split(":");
	var libName=split[1];
	var resource=(split[2].indexOf("sas/")===0)?split[2]:libName.replace(/\./g,"/")+split[2];
	return {
		send: function(headers, completeCallback){
			var lib=sas.ui.mashup.api.AppAPI._loadedLibs[libName];
			if(lib){
				var json=lib.modules[resource]||sas.ui.mashup.api.AppAPI._propsCache[resource];
				if(json){
					responses[mainType]=json;
					completeCallback(200,"success",responses);
				}else{
					completeCallback(404,"source "+resource+" in lib "+libName+" not found");
				}
			}else{
				completeCallback(404,"lib "+libName+" not found");
			}
		},
		abort:function(){
			console.log("Aborted ajax transport for mashup lib loading.");
		}
	};
});
$.ajaxTransport("mashup_json",function(options,originalOptions,jqXHR){
	options.dataTypes.shift();//take "mashup" dataType off the list
	var mainType=options.dataTypes[0];
	var responses={};
	return {
		send: function(headers, completeCallback){
			var json=JSON.parse(sas.ui.mashup.api.AppAPI._jsonCache[options.url]);
			responses[mainType]=json;
			completeCallback(200,"success",responses);
		},
		abort:function(){
			console.log("Aborted ajax transport for mashup json loading.");
		}
	};
});