jQuery.sap.declare("sas.ui.mashup.util.HashChanger");
jQuery.sap.require("sap.ui.core.routing.HashChanger");

sap.ui.core.routing.HashChanger.extend("sas.ui.mashup.util.HashChanger",{
	constructor:function(_hasher){
		this.hasher=_hasher;
		sap.ui.core.routing.HashChanger.apply(this);
	}
});

sas.ui.mashup.util.HashChanger.loadItUp=function(hasher){
	var current=sap.ui.core.routing.HashChanger.getInstance();
	if(!(current instanceof sas.ui.mashup.util.HashChanger)){
		var newone=new sas.ui.mashup.util.HashChanger(hasher);
		sap.ui.core.routing.HashChanger.replaceHashChanger(newone);
	}
};

/**
 * Will start listening to hashChanges with the parseHash function.
 * This will also fire a hashchanged event with the initial hash.
 *
 * @public
 * @return false if it was initialized before, true if it was initialized the first time
 */
sas.ui.mashup.util.HashChanger.prototype.init = function() {
	if (this._initialized) {
		jQuery.sap.log.info("this HashChanger instance has already been initialized.");
		return false;
	}

	this._initialized = true;

	this.hasher.changed.add(this.fireHashChanged, this); //parse hash changes

	if (!this.hasher.isActive()) {
		this.hasher.initialized.addOnce(this.fireHashChanged, this); //parse initial hash
		this.hasher.init(); //start listening for history change
	} else {
		this.fireHashChanged(this.hasher.getHash());
	}

	return this._initialized;
};

/**
 * Fires the hashchanged event, may be extended to modify the hash before fireing the event
 * @param {string} newHash the new hash of the browser
 * @param {string} oldHash - the previous hash
 * @protected
 */
sas.ui.mashup.util.HashChanger.prototype.fireHashChanged = function(newHash, oldHash) {
	this.fireEvent("hashChanged",{ newHash : newHash, oldHash : oldHash });
};

/**
 * Sets the hash to a certain value. When using the set function a browser history  entry is written.
 * If you do not want to have an entry in the browser history, please use set replaceHash function.
 * @param {string} sHash the hash
 * @public
 */
sas.ui.mashup.util.HashChanger.prototype.setHash = function(sHash) {
	this.fireEvent("hashSet", { sHash : sHash });
	this.hasher.setHash(sHash);
};

/**
 * Replaces the hash to a certain value. When using the replace function no browser history is written.
 * If you want to have an entry in the browser history, please use set setHash function.
 * @param {string} sHash the hash
 * @public
 */
sas.ui.mashup.util.HashChanger.prototype.replaceHash = function(sHash) {
	this.fireEvent("hashReplaced", { sHash : sHash });
	this.hasher.replaceHash(sHash);
};

/**
 * Gets the current hash
 *
 * @return {string} the current hash
 * @public
 */
sas.ui.mashup.util.HashChanger.prototype.getHash = function() {
	return this.hasher.getHash();
};

/**
 * Cleans the event registration
 * @see sap.ui.base.Object.prototype.destroy
 * @protected
 */
sas.ui.mashup.util.HashChanger.prototype.destroy = function() {
	this.hasher.changed.remove(this.fireHashChanged, this);
	EventProvider.prototype.destroy.apply(this, arguments);
};

/**
 * Disabled the hasher
 */
sas.ui.mashup.util.HashChanger.prototype.turnOff = function() {
	this.hasher.changed.remove(this.fireHashChanged, this);
};
/**
 * Enables the hasher
 */
sas.ui.mashup.util.HashChanger.prototype.turnOn = function() {
	this.hasher.changed.add(this.fireHashChanged, this);
};

/*AppSwitcher is switching to this app*/
sas.applicationSwitcher.subscribe(sas.applicationSwitcher.APP_SWITCH_MESSAGETYPE,function(id,event){
	if(event.appId===window._MASHUP_appId){
		console.warn("Switching Back to App "+event.appId);
		sap.ui.core.routing.HashChanger.getInstance().turnOn();
	}
});
sap.ui.getCore().attachInit(function(){
	/*AppSwitcher is switching away from this app*/
	sas.ui.mashup.App.getInstance().banner._appswitchMenu.attachItemSelected(function(event){
		var appId=event.getParameter("item").appId;
		if(appId!==window._MASHUP_appId){
			console.warn("Switching to App "+appId);
			delete top.sas.applicationSwitcher._applications.RGF_HUB_Home_RMC;
			sap.ui.core.routing.HashChanger.getInstance().turnOff();
		}
	});	
});
window.hasher=top.hasher;/*Breaking the iframes*/
sas.ui.mashup.util.HashChanger.loadItUp(top.hasher);/*updating the top hasher on hashchanger*/