jQuery.sap.declare("sas.ui.mashup.widget.LeftNav");
jQuery.sap.require("sas.ui.mashup.biz.Menu");
jQuery.sap.require("sas.hc.ui.commons.LeftNavItem");

sap.ui.core.Control.extend("sas.ui.mashup.widget.LeftNav",{
	metadata:{
		properties:{
			menu:{type:"sas.ui.mashup.biz.Menu"},
			selectedMenu:{type:"sas.ui.mashup.biz.Menu"}
		},
		events:{
			beforeItemSelection:{
            allowPreventDefault : true,
            parameters : {
                oldItem : {type : "sas.hc.ui.commons.LeftNavItem"},
                newItem : {type : "sas.hc.ui.commons.LeftNavItem"}
            }
			},
			selected :{
				parameters:{
					menu:{type:"sas.ui.mashup.biz.Menu"}
				}
			}
		}
	},
	constructor:function(){
		sap.ui.core.Control.apply(this,arguments);
		sas.ui.leftNav.addDelegate(this);
	},
	renderer:function(rm,control){
		rm.write("<div");
		rm.writeControlData(control);
		rm.write("></div>");
	}
});
sas.ui.mashup.widget.LeftNav.prototype._createLeftNavItem=function(menuItem){
	var collapsed=menuItem.getMenu().isCollapsed();
	var isHeader=collapsed?!menuItem.getParent()&&(!!menuItem.getChildren().length):false;
	var leftNavItem=new sas.hc.ui.commons.LeftNavItem({
      menu: menuItem,
      text: menuItem.getText(),
      id: menuItem.getId()
   });
	if(isHeader){
		leftNavItem.setType(sas.hc.ui.commons.LeftNavItem.TYPE_HEADER);
	}else{
		leftNavItem.setType(sas.hc.ui.commons.LeftNavItem.TYPE_ITEM);
		leftNavItem.setIcon(menuItem.getIcon());
	}
	return leftNavItem;
};
sas.ui.mashup.widget.LeftNav.prototype.invalidate=function(){
	return sap.ui.core.Control.prototype.invalidate.apply(this,arguments);
};
sas.ui.mashup.widget.LeftNav.prototype.updateVisibility=function(){
	//VISIBILITY
	if(this.getVisible()){
		sas.ui.leftNav.$().css("display","flex");//setVisible(true);
	}else{
		sas.ui.leftNav.$().css("display","none");//setVisible(false);
	}
};
sas.ui.mashup.widget.LeftNav.prototype.onAfterRendering=function(){
	this.updateVisibility();
};
sas.ui.mashup.widget.LeftNav.prototype.onBeforeRendering=function(){
	this.updateVisibility();
	var menu=this.getMenu();
	if(menu){
		//FIRST TIME ONLY
		if(!this.menuModel){
			//CREATING MODEL
			this.menuModel=[];
			var that=this;
			var menu=this.getMenu();
			var collapsed=menu.isCollapsed();
			_.each(menu.getRoots(),function(menuItem){
				that.menuModel.push(that._createLeftNavItem(menuItem));
				if(collapsed){
					_.each(menuItem.getChildren(),function(childItem){
						that.menuModel.push(that._createLeftNavItem(childItem));
					});
				}
			});
			sas.ui.leftNav.setModel(this.menuModel);
			//HOOKING UP EVENTS
			var that=this;
			sas.ui.leftNav.attachBeforeItemSelection(function(event){
				if(that.fireBeforeItemSelection(event.getParameters())){
					that.latch=true;//differentiate between clicking the left nav and programmatically setting it.
				}else{
					event.preventDefault();
				}
			});
			sas.ui.leftNav.attachItemSelected(function(event){
				if(that.latch){
					var selectedLeftNavItem = event.getParameter('item'); // Returns sas.hc.ui.commons.LeftNavItem instance
					var menu=selectedLeftNavItem.menu;
					that.fireSelected({menu:menu});
				}
				that.latch=false;
			},this);
		}
		//SELECTION
		var selectedMenu=this.getSelectedMenu();
		if(selectedMenu){
			var headMenu=this.getSelectedMenu().findLevel(1);
			setTimeout(function(){sas.ui.leftNav.setSelectedItem(headMenu.getId());},0);//strangeness without it
		}
	}
};