/*-------------------------------------------------------------------------*/
/* Copyright (c) 2008 by SAS Institute Inc., Cary, NC, USA.                */
/* All Rights Reserved.                                                    */
/*-------------------------------------------------------------------------*/
/* NAME:     PrepareDatabase.sql                                           */
/* VERSION:  4.1                                                           */
/* DESCRIPTION: batch sql file to prepare the database                     */
/*                                                                         */
/* PRODUCT:  SAS(r) OpRisk Monitor                                         */
/* USAGE:    Used initially when database is setup.                        */
/*                                                                         */
/*-------------------------------------------------------------------------*/

/* Run with DBA privileges */

/* If you want the datafiles to be created in some other location, */
/* instead of the Oracle default, prepend "&1.tables_1.ora" with */
/* full path to location.  Likewise for other ".ora" files. */

SET ECHO ON
CREATE TABLESPACE &1.TABLES_1
    LOGGING 
    DATAFILE '&1.tables_1.ora' SIZE 50M REUSE 
    AUTOEXTEND ON NEXT  51200K MAXSIZE  16383M 
    EXTENT MANAGEMENT LOCAL SEGMENT SPACE MANAGEMENT AUTO 
;

CREATE TABLESPACE &1.IDX_1 
    LOGGING 
    DATAFILE '&1.idx_1.ora' SIZE 50M REUSE 
    AUTOEXTEND ON NEXT  51200K MAXSIZE  16383M 
    EXTENT MANAGEMENT LOCAL SEGMENT SPACE MANAGEMENT AUTO
    BLOCKSIZE 32768
;

/* Each OpRisk user must be assigned this temp space unless local DBA has  
   other preference in which case this temp space need not be created.  In that
   case change the temp tablespace per user accordingly */
CREATE 
    TEMPORARY TABLESPACE &1.TEMP_1 TEMPFILE 
    '&1.temp_1.ora' SIZE 50M REUSE AUTOEXTEND 
    ON NEXT  1M  EXTENT MANAGEMENT LOCAL UNIFORM
    SIZE 1024K
;

CREATE USER &1  PROFILE "DEFAULT" 
    IDENTIFIED BY &1 DEFAULT TABLESPACE &1.TABLES_1 
    TEMPORARY TABLESPACE &1.TEMP_1 
    ACCOUNT UNLOCK
;
GRANT UNLIMITED TABLESPACE TO &1
;
GRANT CREATE SESSION TO &1
;
GRANT CREATE CLUSTER TO &1
;
GRANT CREATE INDEXTYPE TO &1
;
GRANT CREATE OPERATOR TO &1
;
GRANT CREATE PROCEDURE TO &1
;
GRANT CREATE SEQUENCE TO &1
;
GRANT CREATE TABLE TO &1
;
GRANT CREATE TRIGGER TO &1
;
GRANT CREATE TYPE TO &1
;
GRANT CREATE VIEW to &1
;
GRANT SELECT ANY TABLE to &1
;
GRANT UPDATE ANY TABLE to &1
;
GRANT INSERT ANY TABLE to &1
;
GRANT DELETE ANY TABLE to &1
;
GRANT DROP ANY TABLE to &1
;
GRANT CREATE ANY TABLE to &1
;
GRANT ALTER ANY TABLE to &1
;
GRANT CREATE ANY INDEX to &1
;
GRANT DROP ANY INDEX to &1
;
GRANT SELECT ANY SEQUENCE to &1
;
GRANT ALTER ANY SEQUENCE to &1
;
GRANT ANALYZE ANY to &1
;
exit

