/***************************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.              */
/*  All rights reserved.                                                   */
/***************************************************************************/
/*  NAME:        rskmgt_reload_custobj_data.sas                            */
/*  VERSION:     7.4                                                       */
/*  DESCRIPTION: Reloads the custom Object tables into LASR.               */
/*  PARAMETERS:  INTBLTYPE - lasr or audit custom object tables to reload  */
/*                           _l (live) tbls; _A (audit) tbls               */
/*  PRODUCT:     SAS Risk Management Server                                */
/***************************************************************************/

%macro rskmgt_reload_custobj_data(intbltype=);

%put NOTE: {START: &sysmacroname};

  /* Store the list of customObject numbers needed in a list */
  proc sql noprint;
    select cust_obj_tbl_no into : customObjectNums separated by ','
    from &contentId.ctl.custObj_tbl_ref;
  quit;

  /* Get a count of the number of customObjects that are enabled */
  %let customObjectList=%sysfunc(countw(%bquote(&customObjectNums)));

  %do z=1 %to &customObjectList;
  
    /* Get a value out of the customObjectList macro list */
    %let custObjValue=%trim(%qscan(%bquote(&customObjectNums),&z,%str(,)));
    
    /* Get the alias name, buildETLData and loadETLData properties */
    %let custObjAliasNm=;
    %let buildETLDataTbls=;
    %let loadETLDataTbls=;
    
    proc sql noprint;
      select cust_obj_alias_nm, buildETLData, loadETLData into : custObjAliasNm trimmed,
                                                               : buildETLDataTbls,
                                                               : loadETLDataTbls
      from &contentId.ctl.custobj_tbl_ref
      where cust_obj_tbl_nm="cust_obj_&custObjValue._l";
    quit;
      
    %let outdsn=%trim(&contentId._&custObjAliasNm.&intbltype);
          
    /* If buildETLDataTbls=true, then proceed.  Otherwise, move to the next customObject */
    %if &buildETLDataTbls=false %then %goto continue;
 
    %rskmgt_reload_lasr_data(intblnm=&outdsn,
                             feature=CUST_OBJ_&custObjValue);
     
    /* This table is only available in the (live) tables */
    %if &intbltype=_l %then
    %do;                  
      /* Load the MRM_Model_Model_L table when loading Models */
      %if (%upcase(&contentId)=MRM and (%upcase(&custObjAliasNm)=CUSTOMOBJECT1 OR %upcase(&custObjAliasNm)=MODEL)) %then
      %do;
        %rskmgt_reload_lasr_data(intblnm=mrm_model_model_l,
                                 feature=CUST_OBJ_1);
      %end;
    %end;
  
    %continue:
  %end;
     
%put NOTE: {END: &sysmacroname};

%mend rskmgt_reload_custobj_data;
