/**********************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.         */
/*  All rights reserved.                                              */
/**********************************************************************/
/*  NAME:        rskmgt_db_connect.sas                                */
/*  VERSION:     7.4                                                  */
/*  DESCRIPTION: Shared macro to build the SAS connect statement for  */
/*               explicit pass-through to an RDBMS.                   */
/*  PARAMETERS:  ConnectName = DB connection alias name               */
/*                 (default: &SRC_LIB._CONN)                          */
/*  PRODUCT:     SAS Risk Management Server                           */
/**********************************************************************/

%macro rskmgt_db_connect(ConnectName=&SRC_LIB._CONN);
  
%put NOTE: {START: &sysmacroname};
  
  %global DBConnectSQL DBConnectAlias;

  %let DBConnectAlias = &ConnectName;

  /*  For ORACLE: Build the DBConnectSQL macro variable */
  %if %upcase(&rskmgt_dbserver) = ORACLE %then 
  %do;
     %let DBConnectSQL = connect to &rskmgt_dbengine as &DBConnectAlias 
                        (path=&rskmgt_dbName authdomain="&rskmgt_dbAuth" connection=global);
  %end;

  /*  For PostgreSQL: Build the DBConnectSQL macro variable */
  %if %upcase(&rskmgt_dbserver) = POSTGRESQL %then
  %do;
    %let DBConnectSQL = connect to &rskmgt_dbengine as &DBConnectAlias 
                        (server="&rskmgt_dbserverNm" port="&rskmgt_dbport" database="&rskmgt_dbName" authdomain="&rskmgt_dbAuth" connection=global autocommit=no dbmax_text=4000);
  %end;
  
  /*  For SQLServer: Build the DBConnectSQL macro variable */
  %if %upcase(&rskmgt_dbserver) = SQLSERVER %then 
  %do;    
    %let DBConnectSQL = connect to &rskmgt_dbengine as &DBConnectAlias 
                        (dsn="&rskmgt_dbName" authdomain="&rskmgt_dbAuth" connection=global  autocommit=no);
  %end;
  
%put NOTE: {END: &sysmacroname};

%mend rskmgt_db_connect;
