/************************************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.                       */
/*  All rights reserved.                                                            */
/************************************************************************************/
/*  NAME:        rskmgt_get_rest_obj_names.sas                                      */
/*  VERSION:     7.4                                                                */
/*  DESCRIPTION: Get the custom object rest call names.                             */
/*  PARAMETERS:  business_obj_type_nm=business object type name                     */
/*  PRODUCT:    SAS Risk Management Server                                          */
/************************************************************************************/

%macro rskmgt_get_rest_obj_names(business_obj_type_nm=);

%put NOTE: {START: &sysmacroname};

%local nobs rc;
%let dsid = %sysfunc(open(work.&contentId._config_props));
%if &dsid %then
%do;
  %let nobs  = %sysfunc(attrn(&dsid, nlobs));
  %let rc = %sysfunc(close(&dsid));
%end;
%else %let nobs  = 0;

%if &nobs > 0 %then
%do;
  /* Loop through the rest object names */
  data _null_;
    set &contentId._config_props;

    if findw(key,'RESTURI')=0 then delete;
         
    %if &business_obj_type_nm ne %str() %then
    %do;
      if findw(key,"%upcase(&business_obj_type_nm).RESTURI")>0 then
      do;
        call symputx('restobj_alias_nm',value,'G');
        return;
      end;
      else call symputx('restobj_orig_nm',"&business_obj_type_nm"||'s','G');
    %end;
  run;
%end;
  
/* Put the value of the restObjectNm into a common macro variable */
%global restobj restobj_alias_nm restobj_orig_nm;

%if %symexist(restobj_alias_nm) or %symexist(restobj_orig_nm) %then 
%do;
  %if &restobj_alias_nm ne %str() %then
  %do;
    %let restobj=&restobj_alias_nm;
    %let restobj_alias_nm=;
    %let restobj_orig_nm=;
  %end;

  %if &restobj_orig_nm ne %str() %then
  %do;
    %let restobj=&restobj_orig_nm;
    %let restobj_orig_nm=;
    %let restobj_alias_nm=;
  %end;
%end;

%put NOTE: {END: &sysmacroname};

%mend rskmgt_get_rest_obj_names;