/**********************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.         */
/*  All rights reserved.                                              */
/**********************************************************************/
/*  NAME:        rskmgt_job_changereason_data.sas                     */
/*  VERSION:     7.4                                                  */
/*  DESCRIPTION: Create a table of change reasons.                    */
/*  PARAMETERS:                                                       */
/*  PRODUCT:     SAS Risk Management Server                           */
/**********************************************************************/

%macro rskmgt_job_changereason_data;

%put NOTE: {START: &sysmacroname}; 

  /* Get all users */
  proc sql noprint;
    %rskmgt_db_connect;                                                                                                                
    &DBConnectSQL;  

    create table work.&contentId._change_reason as 
      select * from connection to &src_lib._CONN
        (select *
         from 
         %if &&&contentId._schema ne %then &&&contentId._schema..change_reason;
         %else change_reason;
        ); 
        
    disconnect from &src_lib._CONN;
  quit;
  
  /* Check to see if there are any records in the change reason table */
  /* If so, continue; otherwise, exit the macro */
  %rskmgt_ds_exists(indsn=work.&contentId._change_reason,
                    output_nobs=chgRsnObjObs_cnt);
                    
  %if &chgRsnObjObs_cnt = 0 %then %goto exit; 
  
  /* Fix the date and datetime formats */
  %rskmgt_modify_date_time(indsn=work.&contentId._change_reason);
     
  /* Store a copy of the change_reason table in &contentId.dat */
  %if %eval(%sysfunc(exist(work.&contentId._change_reason)) = 1) %then 
  %do;
    data &contentId.dat.&contentId._change_reason;
      if 0 then set work.&contentId._change_reason;
      stop;
    run;
  %end;
      
  /* Update the &contentId.dat table */
  %if %eval(%sysfunc(exist(work.&contentId._change_reason)) = 1) %then 
  %do;
    proc append base=&contentId.dat.&contentId._change_reason data=work.&contentId._change_reason force;
    run;
  %end;
   
  /* Load LASR Table and Authorizations into Metadata w/ Group Permissions only */
  %rskmgt_load_lasr_tbls_grp_perms(intblnm=&contentId._change_reason,
                                   inlib=&contentId.dat);
                                                       
  /* Remove work tables */
  proc datasets lib=work nolist;
    save %let y=1;
    
    %let contentIds=%trim(%qscan(%bquote(&rgf_content_ids),&y,%str(,)));
    %do %while (&contentIds ne );
  
      &contentIds._Custommessages &contentIds._sm_security_acl &contentIds._config_props
         
      %let y=%eval(&y+1);
      %let contentIds=%trim(%qscan(%bquote(&rgf_content_ids),&y,%str(,))); 
    %end;
    %str(;)
  quit;
  run;

%put NOTE: {END: &sysmacroname}; 

%exit: %mend rskmgt_job_changereason_data;
