/**********************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.         */
/*  All rights reserved.                                              */
/**********************************************************************/
/*  NAME:        rskmgt_job_custobj_data.sas                          */
/*  VERSION:     7.4                                                  */
/*  DESCRIPTION: Create customObject data.                            */
/*  PARAMETERS:                                                       */
/*  PRODUCT:     SAS Risk Management Platform                         */
/**********************************************************************/

%macro rskmgt_job_custobj_data;

%put NOTE: {START: &sysmacroname};

  /* Store the list of customObject numbers needed in a list */
  proc sql noprint;
    select cust_obj_tbl_no into : customObjectNums separated by ','
    from &contentId.ctl.custObj_tbl_ref;
  quit;

  /* Get a count of the number of customObjects that are enabled */
  %let customObjectList=%sysfunc(countw(%bquote(&customObjectNums)));

  /* Lookup the custom object table names in metadata to see if they have been renamed */
  %do z=1 %to &customObjectList;
  
    /* Get a value out of the customObjectList macro list */
    %let custObjValue=%trim(%qscan(%bquote(&customObjectNums),&z,%str(,)));

    /* Get the alias name, buildETLData and loadETLData properties */
    %let custObjAliasNm=;
    %let buildETLDataTbls=;
    %let loadETLDataTbls=;
  
    proc sql noprint;
      select cust_obj_alias_nm, buildETLData, loadETLData into : custObjAliasNm trimmed, 
                                                               : buildETLDataTbls,
                                                               : loadETLDataTbls
      from &contentId.ctl.custobj_tbl_ref
      where cust_obj_tbl_nm="cust_obj_&custObjValue._l";
    quit;
       
    /* Store the output dataset name and rk column name in macros vars for */
    /* macro calls further downstream */
    %let outdsn=%trim(&contentId._&custObjAliasNm._L);
    %let out_rk_col=&custObjAliasNm._rk;
  
    /* If buildETLDataTbls=true, then proceed.  Otherwise, move to the next customObject */
    %if &buildETLDataTbls=false or &buildETLDataTbls=no %then %goto continue;

    %if %eval(%sysfunc(exist(&contentId.dat.&outdsn)) = 1) %then
    %do;
      proc datasets lib=&contentId.dat nolist;
        delete &outdsn;
      quit;
      run;
    %end;
   
    %if %eval(%sysfunc(exist(rgflasr.&outdsn)) = 1 ) %then
    %do;     
      proc datasets lib=rgflasr nolist;
        delete &outdsn;
      quit;
      run;   

      proc metalib;
        omr (liburi="SASLibrary?@libref='RGFLASR'");
        select ("&outdsn");
        update_rule=(delete);
      run;
    %end;
    
    %if %eval(%sysfunc(exist(rgfrload.&outdsn)) = 1 ) %then
    %do;
      proc datasets lib=rgfrload nolist;
        delete &outdsn;
      quit;
      run;     
    %end; 
    
  /* Rename the custom object table and fields to use the alias names */
  proc sql noprint;
    %rskmgt_db_connect;
    &DBConnectSQL;

    /* Get all custObj&custObjValue records and use the alias names */
    create table &custObjAliasNm as
      select * from connection to &src_lib._CONN
        (SELECT
           A.source_system_cd
           ,A.cust_obj_&custObjValue._id as &custObjAliasNm._id
           ,A.cust_obj_&custObjValue._rk as &custObjAliasNm._rk
           ,A.cust_obj_nm as &custObjAliasNm._nm
           ,A.cust_obj_desc as &custObjAliasNm._desc
           ,CASE
             WHEN A.cust_obj_&custObjValue._rk = U1.user_rk THEN U1.display_nm
             ELSE ''
           END AS user_name_1
           ,CASE
             WHEN A.cust_obj_&custObjValue._rk = U2.user_rk THEN U2.display_nm
             ELSE ''
           END AS user_name_2
           ,CASE
             WHEN A.cust_obj_&custObjValue._rk = U3.user_rk THEN U3.display_nm
             ELSE ''
           END AS user_name_3
           ,CASE
             WHEN A.cust_obj_&custObjValue._rk = U4.user_rk THEN U4.display_nm
             ELSE ''
           END AS user_name_4
           ,CASE
             WHEN A.cust_obj_&custObjValue._rk = U5.user_rk THEN U5.display_nm
             ELSE ''
           END AS user_name_5
           ,CASE
             WHEN A.creator_user_rk = U6.user_rk THEN U6.display_nm
             ELSE ''
           END AS created_user_name
           ,A.created_from_rk
           ,A.created_dttm
           ,CASE
             WHEN A.last_update_user_rk = U7.user_rk THEN U7.display_nm
             ELSE ''
           END AS last_update_user_name
         FROM
           %if &&&contentId._schema ne %then 
           %do;
             &&&contentId._schema..cust_obj_&custObjValue._l A
             LEFT JOIN &&&contentId._schema..orauser_l U1
               ON A.cust_user_1_rk = U1.user_rk
             LEFT JOIN &&&contentId._schema..orauser_l U2
               ON A.cust_user_2_rk = U2.user_rk
             LEFT JOIN &&&contentId._schema..orauser_l U3
               ON A.cust_user_3_rk = U3.user_rk
             LEFT JOIN &&&contentId._schema..orauser_l U4
               ON A.cust_user_4_rk = U4.user_rk
             LEFT JOIN &&&contentId._schema..orauser_l U5
               ON A.cust_user_5_rk = U5.user_rk
             LEFT JOIN &&&contentId._schema..orauser_l U6
               ON A.creator_user_rk = U6.user_rk
             LEFT JOIN &&&contentId._schema..orauser_l U7
               ON A.created_from_rk = U7.user_rk       
           %end;
           %else
           %do;
             cust_obj_&custObjValue._l A
             LEFT JOIN orauser_l U1
               ON A.cust_user_1_rk = U1.user_rk
             LEFT JOIN orauser_l U2
               ON A.cust_user_2_rk = U2.user_rk
             LEFT JOIN orauser_l U3
               ON A.cust_user_3_rk = U3.user_rk
             LEFT JOIN orauser_l U4
               ON A.cust_user_4_rk = U4.user_rk
             LEFT JOIN orauser_l U5
               ON A.cust_user_5_rk = U5.user_rk
             LEFT JOIN orauser_l U6
               ON A.creator_user_rk = U6.user_rk
             LEFT JOIN orauser_l U7
               ON A.created_from_rk = U7.user_rk 
           %end;
    
         ORDER BY
         A.cust_obj_&custObjValue._rk
         );
   
      disconnect from &src_lib._CONN;
    quit;
  
    /* Check to see if there are any records in the business object table */
    /* If so, continue; otherwise, exit the macro */
    %rskmgt_ds_exists(indsn=&custObjAliasNm,
                      output_nobs=busObjObs_cnt);
                    
    %if &busObjObs_cnt = 0 %then %goto continue;
 
    /* Add the Model Risk Assessment custom fields to the model table */
    /* for customObject1.  If not customObject1, do not add them.     */
    %if (%upcase(&contentId)=MRM and (%upcase(&custObjAliasNm)=CUSTOMOBJECT1 OR %upcase(&custObjAliasNm)=MODEL)) %then
    %do;
      /* Add the custom object and MRA custom fields */
      %rskmgt_add_cust_fields(indsn=&custObjAliasNm,
                              feature=CUST_OBJ_&custObjValue,
                              rk_column=&out_rk_col,
                              business_object_type_nm=customObject&custObjValue,
                              filter_lst=%str(x_co14_ops_tier_result,x_co14_ops_mdl_val_rating,x_co14_ops_inh_Risk_rating,x_co14_ops_control_score,x_co14_ops_residualrisk_score),
                              MRA=Y);
    %end;
    %else
    %do;
      /*Check to see if there are any custom macros by contentID */
      %let &contentId._Mac_Path=%substr(&RGF_CUSTOM_PATH,1,(%sysfunc(indexw("&RGF_CUSTOM_PATH",%qscan(%bquote(&RGF_CUSTOM_PATH),-2,'/\'),"/\"))-2))Source/&contentId./ucmacros;  
       
      filename custmac "&&&contentId._Mac_Path/&contentId._add_cust_fields.sas";
      
      %if %sysfunc(fexist(custmac))=1 %then
      %do;
        /* Execute the &contentId version of the macro */
        %&contentId._add_cust_fields(indsn=&custObjAliasNm,
                                     feature=CUST_OBJ_&custObjValue,
                                     rk_column=&out_rk_col,
                                     business_object_type_nm=customObject&custObjValue);
      %end;
      %else
      %do;      
        /* No custom &contentId macro found so run the RGF platform macro */
        %rskmgt_add_cust_fields(indsn=&custObjAliasNm,
                                feature=CUST_OBJ_&custObjValue,
                                rk_column=&out_rk_col,
                                business_object_type_nm=customObject&custObjValue);
      %end;
    %end;
 
    /* Fix the date and datetime formats */
    %rskmgt_modify_date_time(indsn=work.&custObjAliasNm);

    /* Get the links for all custom_object_&custObjValue records */
    /* and merge the custom_object_&custObjValue records with the */
    /* 360 links */
    %rskmgt_filter_link_inst_tbl(business_object_type_nm=customObject&custObjValue,
                                 indsn=&custObjAliasNm,
                                 rk_column=&out_rk_col);
   
    /* Add dimensions */
    %rskmgt_sp_unfold_dimensions(srclib=&src_lib,
                                 in=&custObjAliasNm,
                                 rk_column=&out_rk_col,
                                 feature=CUST_OBJ_&custObjValue,
                                 business_object_nm=CUST_OBJ_&custObjValue);

    /* Apply dimension path */
    %rskmgt_apply_dim_path(indsn=&custObjAliasNm,
                           feature=CUST_OBJ_&custObjValue);

    /* Sort the custObj&custObjValue_dtls by custObj&custObjValue_rk */
    proc sort tagsort data=work.&custObjAliasNm out=&custObjAliasNm._fnl(drop=dim_point_rk);
      by &out_rk_col;
    run;
  
    /* Update the _count variable to set the value to 1 for only 1 instance of each _rk */ 
    data work.&custObjAliasNm._fnl;
      set work.&custObjAliasNm._fnl;
      by &out_rk_col;

      if first.&out_rk_col then %sysfunc(substr(&out_rk_col,1,%sysfunc(length(&out_rk_col))-3))_count=1;
      else %sysfunc(substr(&out_rk_col,1,%sysfunc(length(&out_rk_col))-3))_count=0;
    run;

    /* Add custom object labels */
    %let custObjLabel=;
 
    /* Query the &contentId._Custommessages dataset to get the custom object labels */
    proc sql noprint;
      select text into : custObjLabel
      from work.&contentId._Custommessages
      where upcase(key) like "CUSTOMOBJECT&custObjValue..DISPLAYNAME.TXT";
    quit;

    /* Add the number of submodels to the models table */
    %if (%upcase(&contentId)=MRM and (%upcase(&custObjAliasNm)=CUSTOMOBJECT1 OR %upcase(&custObjAliasNm)=MODEL)) %then
    %do;
      /* Calculate the # of submodels */
      proc sql;
        create table &custObjAliasNm._fnl as
        select A.*,
               B.num_submodels
        from work.&custObjAliasNm._fnl A
        left join 
          (select 
             M1.cust_obj_1_rk as parent_model_rk, count(distinct M2.cust_obj_1_rk) as num_submodels
           from
              &src_lib..link_instance L
             ,&src_lib..link_type T
             ,&src_lib..cust_obj_1_l M1
             ,&src_lib..cust_obj_1_l M2
           where
            T.link_type_rk = L.link_type_rk
            AND link_type_id = 'model_model'
            AND business_object_rk_1 = M1.cust_obj_1_rk
            AND business_object_rk_2 = M2.cust_obj_1_rk 
           group by M1.cust_obj_1_rk) B
    
        %if %upcase(&custObjAliasNm) eq MODEL %then
        %do;
          on A.model_rk=B.parent_model_rk
          order by A.model_rk;
        %end;
        %else %if %upcase(&custObjAliasNm) eq CUSTOMOBJECT1 %then
        %do;
          on A.cust_obj_1_rk=B.parent_model_rk
          order by A.cust_obj_1_rk;
        %end; 
      quit;
    %end;
   
    data work.&custObjAliasNm._fnl;
      set work.&custObjAliasNm._fnl;

      label
      %if %bquote(&custObjAliasNm) ne %str() and %bquote(&custObjLabel) ne %str() %then
      %do;
        &custObjAliasNm._id="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Id, NOQUOTE))" 
        &custObjAliasNm._rk="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Rk, NOQUOTE))" 
        &custObjAliasNm._nm="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Nm, NOQUOTE))" 
        &custObjAliasNm._desc="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Desc, NOQUOTE))" 
        source_system_cd="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Source_System_Cd, NOQUOTE))"
      
        %if (%upcase(&contentId)=MRM and (%upcase(&custObjAliasNm)=CUSTOMOBJECT1 OR %upcase(&custObjAliasNm)=MODEL)) %then
        %do;
          num_submodels="%sysfunc(sasmsg(sashelp.mrmsrv,Num_Submodels, NOQUOTE))"
        %end;
      %end;
      %else %if %bquote(&custObjAliasNm) ne %str() and %bquote(&custObjLabel) eq %str() %then
      %do;
        &custObjAliasNm._id="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Id, NOQUOTE))" 
        &custObjAliasNm._rk="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Rk, NOQUOTE))" 
        &custObjAliasNm._nm="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Nm, NOQUOTE))" 
        &custObjAliasNm._desc="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Desc, NOQUOTE))" 
        source_system_cd="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Source_System_Cd, NOQUOTE))"
      
        %if (%upcase(&contentId)=MRM and (%upcase(&custObjAliasNm)=CUSTOMOBJECT1 OR %upcase(&custObjAliasNm)=MODEL)) %then
        %do;
          num_submodels="%sysfunc(sasmsg(sashelp.mrmsrv,Num_Submodels, NOQUOTE))"
       %end;
      %end;
 
      %do y=1 %to 5 %by 1;
        %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CUSTOMUSER&y, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CUSTOMUSER&y %then
        %do;
          user_name_&y="%sysfunc(sasmsg(sashelp.rskmgtsvr, User_Name_&y, NOQUOTE))"
        %end;
        %else
        %do;
          user_name_&y="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CUSTOMUSER&y, NOQUOTE))" 
        %end;
      %end;

      %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDFROM, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CREATEDFROM %then
      %do;
         created_from_rk="%sysfunc(sasmsg(sashelp.rskmgtsvr, Created_From_Rk, NOQUOTE))"
      %end;
      %else
      %do;
        created_from_rk="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDFROM, NOQUOTE))"
      %end;

      %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATOR, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CREATOR %then
      %do;
        created_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, Created_User_Name, NOQUOTE))"
      %end;
      %else
      %do;
        created_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATOR, NOQUOTE))"
      %end;

      %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_UPDATER, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_UPDATER %then
      %do;
        last_update_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, Last_Update_User_Name, NOQUOTE))"
      %end;
      %else
      %do;
        last_update_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_UPDATER, NOQUOTE))"
      %end;

      %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDDTTM, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CREATEDDTTM %then
      %do;
        created_dttm="%sysfunc(sasmsg(sashelp.rskmgtsvr, Created_Dttm, NOQUOTE))"
      %end;
      %else
      %do;
        created_dttm="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDDTTM, NOQUOTE))"
      %end;

      status_dttm="%sysfunc(sasmsg(sashelp.rskmgtsvr, Status_Dttm, NOQUOTE))"
      %str(;)
    run;

    /* Create the &contentId.dat.&custObjAliasNm dataset in order to add to metadata */
    %if %eval(%sysfunc(exist(work.&custObjAliasNm._fnl)) = 1) %then
    %do;
      data &contentId.dat.&outdsn;
        if 0 then set work.&custObjAliasNm._fnl;
        stop;
      run;
    %end;
     
    /* Update the &contentId.dat table */
    proc append base=&contentId.dat.&outdsn data=work.&custObjAliasNm._fnl force;
    run;
   
    /* Load LASR Table and Authorizations into Metadata */
    %rskmgt_load_lasr_tbls(intblnm=&outdsn,
                           feature=CUST_OBJ_&custObjValue); 
  
    /* Build the child to parent model table */
    %if (%upcase(&contentId)=MRM and (%upcase(&custObjAliasNm)=CUSTOMOBJECT1 OR %upcase(&custObjAliasNm)=MODEL)) %then
    %do;
      proc sql noprint;
        %rskmgt_db_connect;
        &DBConnectSQL;
           
        /* Get all child model details and parents */
        create table work.child_parent_models as
          select * from connection to &src_lib._CONN
            (SELECT         
               M2.source_system_cd as source_system_cd
             
               %if %upcase(&custObjAliasNm) eq MODEL %then
               %do;
                 ,M2.cust_obj_1_id as model_id
               %end;
               %else
               %do;
                 ,M2.cust_obj_1_id
               %end;
             
               %if %upcase(&custObjAliasNm) eq MODEL %then
               %do;
                 ,M2.cust_obj_1_rk as model_rk
               %end;
               %else
               %do;
                 ,M2.cust_obj_1_rk
               %end;
             
               %if %upcase(&custObjAliasNm) eq MODEL %then
               %do;
                 ,M2.cust_obj_nm as model_nm
               %end;
               %else
               %do;
                 ,M2.cust_obj_nm
               %end;
             
               %if %upcase(&custObjAliasNm) eq MODEL %then
               %do;
                ,M2.cust_obj_desc as model_desc
               %end;
               %else
               %do;
                 ,M2.cust_obj_desc
               %end;
             
              ,CASE
                WHEN M2.cust_user_1_rk = U1.user_rk THEN U1.display_nm
                ELSE ''
              END AS user_name_1
              ,CASE
                WHEN M2.cust_user_2_rk = U2.user_rk THEN U2.display_nm
                ELSE ''
              END AS user_name_2
              ,CASE
                WHEN M2.cust_user_3_rk = U3.user_rk THEN U3.display_nm
                ELSE ''
              END AS user_name_3
              ,CASE
                WHEN M2.cust_user_4_rk = U4.user_rk THEN U4.display_nm
                ELSE ''
              END AS user_name_4
              ,CASE
                WHEN M2.cust_user_5_rk = U5.user_rk THEN U5.display_nm
                ELSE ''
              END AS user_name_5
              ,CASE
                WHEN M2.creator_user_rk = U6.user_rk THEN U6.display_nm
                ELSE ''
              END AS created_user_name
              ,M2.created_from_rk
              ,M2.created_dttm
              ,CASE
                WHEN M2.last_update_user_rk = U7.user_rk THEN U7.display_nm
                ELSE ''
              END AS last_update_user_name
              ,M1.cust_obj_1_rk as parent_model_rk
              ,M1.cust_obj_1_id as parent_model_id
              ,M1.source_system_cd as parent_source_system_cd
             FROM
               %if &&&contentId._schema ne %then 
               %do;
                  &&&contentId._schema..LINK_INSTANCE_L L
                 ,&&&contentId._schema..LINK_TYPE_L T
                 ,&&&contentId._schema..CUST_OBJ_1_L M1
                 ,&&&contentId._schema..CUST_OBJ_1_l M2
                 LEFT JOIN &&&contentId._schema..ORAUSER_L U1
                   ON M2.CUST_USER_1_RK = U1.USER_RK
                 LEFT JOIN &&&contentId._schema..ORAUSER_L U2
                   ON M2.CUST_USER_2_RK = U2.USER_RK
                 LEFT JOIN &&&contentId._schema..ORAUSER_L U3
                   ON M2.CUST_USER_3_RK = U3.USER_RK
                 LEFT JOIN &&&contentId._schema..ORAUSER_L U4
                  ON M2.CUST_USER_4_RK = U4.USER_RK
                 LEFT JOIN &&&contentId._schema..ORAUSER_L U5
                   ON M2.CUST_USER_5_RK = U5.USER_RK
                 LEFT JOIN &&&contentId._schema..ORAUSER_L U6
                   ON M2.CREATOR_USER_RK = U6.USER_RK
                 LEFT JOIN &&&contentId._schema..ORAUSER_L U7
                   ON M2.LAST_UPDATE_USER_RK = U7.USER_RK       
              %end;
              %else
              %do;
                LINK_INSTANCE_L L
                ,LINK_TYPE_L T
                ,CUST_OBJ_1_L M1
                ,CUST_OBJ_1_l M2
                LEFT JOIN ORAUSER_L U1
                  ON M2.CUST_USER_1_RK = U1.USER_RK
                LEFT JOIN ORAUSER_L U2
                  ON M2.CUST_USER_2_RK = U2.USER_RK
                LEFT JOIN ORAUSER_L U3
                  ON M2.CUST_USER_3_RK = U3.USER_RK
                LEFT JOIN ORAUSER_L U4
                 ON M2.CUST_USER_4_RK = U4.USER_RK
                LEFT JOIN ORAUSER_L U5
                  ON M2.CUST_USER_5_RK = U5.USER_RK
                LEFT JOIN ORAUSER_L U6
                  ON M2.CREATOR_USER_RK = U6.USER_RK
                LEFT JOIN ORAUSER_L U7
                  ON M2.LAST_UPDATE_USER_RK = U7.USER_RK
              %end;
              
             WHERE
               T.LINK_TYPE_RK = L.LINK_TYPE_RK
               AND LINK_TYPE_ID = 'model_model'
               AND BUSINESS_OBJECT_RK_1 = M1.cust_obj_1_rk
               AND BUSINESS_OBJECT_RK_2 = M2.cust_obj_1_rk
             );
   
        disconnect from &src_lib._CONN;
      quit;    
    
      /* Check to see if there are any records in the business object table */
      /* If so, continue; otherwise, exit the macro */
      %rskmgt_ds_exists(indsn=work.child_parent_models,
                        output_nobs=busObjObs_cnt);
                    
      %if &busObjObs_cnt = 0 %then %goto continue;       

      /* Add the custom fields */
      %rskmgt_add_cust_fields(indsn=child_parent_models,
                              feature=CUST_OBJ_&custObjValue,
                              rk_column=&out_rk_col,
                              business_object_type_nm=customObject&custObjValue,
                              filter_lst=%str(x_co14_ops_tier_result,x_co14_ops_mdl_val_rating,x_co14_ops_inh_Risk_rating,x_co14_ops_control_score,x_co14_ops_residualrisk_score),
                              MRA=Y);
                            
      /* Get the links for all custom_object_1 records */
      /* and merge the custom_object_1 records with the */
      /* 360 links */
      %if %upcase(&custObjAliasNm) eq MODEL %then
      %do;
        %rskmgt_filter_link_inst_tbl(business_object_type_nm=customObject1,
                                     indsn=child_parent_models,
                                     rk_column=model_rk);
      %end;
      %else
      %do;
        %rskmgt_filter_link_inst_tbl(business_object_type_nm=customObject1,
                                     indsn=child_parent_models,
                                     rk_column=cust_obj_1_rk);
      %end;
   
      /* Fix the date and datetime formats */
      %rskmgt_modify_date_time(indsn=work.child_parent_models);
    
      /* Add dimensions */
      %rskmgt_sp_unfold_dimensions(srclib=&src_lib,
                                   in=child_parent_models,
                                   rk_column=&out_rk_col,
                                   feature=CUST_OBJ_&custObjValue,
                                   business_object_nm=CUST_OBJ_&custObjValue);

      /* Apply dimension path */
      %rskmgt_apply_dim_path(indsn=child_parent_models,
                             feature=CUST_OBJ_&custObjValue);

      /* Sort the custObj&custObjValue_dtls by custObj&custObjValue_rk */
      proc sort tagsort data=work.child_parent_models out=child_parent_models_fnl(drop=dim_point_rk);
        by &out_rk_col;
      run;
	  
	  /* Update the _count variable to set the value to 1 for only 1 instance of each _rk */ 
	  data work.child_parent_models_fnl;
        set work.child_parent_models_fnl;
        by &out_rk_col;

        if first.&out_rk_col then %sysfunc(substr(&out_rk_col,1,%sysfunc(length(&out_rk_col))-3))_count=1;
        else %sysfunc(substr(&out_rk_col,1,%sysfunc(length(&out_rk_col))-3))_count=0;
      run;

      /* Add custom object labels */
      %let custObjLabel=;
 
      /* Query the &contentId._Custommessages dataset to get the custom object labels */
      proc sql noprint;
        select text into : custObjLabel
        from work.&contentId._Custommessages
        where upcase(key) like "CUSTOMOBJECT1..DISPLAYNAME.TXT";
      quit;

      data work.model_model_fnl;
        set work.child_parent_models_fnl;

        label
        %if %bquote(&custObjAliasNm) ne %str() and %bquote(&custObjLabel) ne %str() %then
        %do;
          &custObjAliasNm._id="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Id, NOQUOTE))" 
          &custObjAliasNm._rk="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Rk, NOQUOTE))" 
          &custObjAliasNm._nm="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Nm, NOQUOTE))" 
          &custObjAliasNm._desc="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Desc, NOQUOTE))" 
          source_system_cd="%trim(&custObjLabel) %sysfunc(sasmsg(sashelp.rskmgtsvr, Source_System_Cd, NOQUOTE))"
        %end;
        %else %if %bquote(&custObjAliasNm) ne %str() and %bquote(&custObjLabel) eq %str() %then
        %do;
          &custObjAliasNm._id="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Id, NOQUOTE))" 
          &custObjAliasNm._rk="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Rk, NOQUOTE))" 
          &custObjAliasNm._nm="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Nm, NOQUOTE))" 
          &custObjAliasNm._desc="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Desc, NOQUOTE))" 
          source_system_cd="%upcase(&custObjAliasNm) %sysfunc(sasmsg(sashelp.rskmgtsvr, Source_System_Cd, NOQUOTE))"
        %end;
      
        %do y=1 %to 5 %by 1;
          %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CUSTOMUSER&y, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CUSTOMUSER&y %then
          %do;
            user_name_&y="%sysfunc(sasmsg(sashelp.rskmgtsvr, User_Name_&y, NOQUOTE))"
         %end;
         %else
         %do;
            user_name_&y="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CUSTOMUSER&y, NOQUOTE))" 
         %end;
       %end;

       %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDFROM, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CREATEDFROM %then
       %do;
         created_from_rk="%sysfunc(sasmsg(sashelp.rskmgtsvr, Created_From_Rk, NOQUOTE))"
       %end;
       %else
       %do;
          created_from_rk="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDFROM, NOQUOTE))"
       %end;

       %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATOR, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CREATOR %then
       %do;
         created_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, Created_User_Name, NOQUOTE))"
       %end;
       %else
       %do;
         created_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATOR, NOQUOTE))"
       %end;

       %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_UPDATER, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_UPDATER %then
       %do;
         last_update_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, Last_Update_User_Name, NOQUOTE))"
       %end;
       %else
       %do;
         last_update_user_name="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_UPDATER, NOQUOTE))"
       %end;

       %if %sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDDTTM, NOQUOTE))=CUSTOMOBJECT&custObjValue._OBJ_CREATEDDTTM %then
       %do;
         created_dttm="%sysfunc(sasmsg(sashelp.rskmgtsvr, Created_Dttm, NOQUOTE))"
       %end;
       %else
       %do;
         created_dttm="%sysfunc(sasmsg(sashelp.rskmgtsvr, CUSTOMOBJECT&custObjValue._OBJ_CREATEDDTTM, NOQUOTE))"
       %end;

       parent_model_rk="%sysfunc(sasmsg(sashelp.mrmsrv, Parent_Model_Rk, NOQUOTE))"
       parent_model_id="%sysfunc(sasmsg(sashelp.mrmsrv, Parent_Model_Id, NOQUOTE))"
       parent_source_system_cd="%sysfunc(sasmsg(sashelp.mrmsrv, Parent_Model_Source_System_Cd, NOQUOTE))"
       status_dttm="%sysfunc(sasmsg(sashelp.rskmgtsvr, Status_Dttm, NOQUOTE))"
       %str(;)
     run;

     /* Create the &contentId.dat.model_model dataset in order to add to metadata */
     %if %eval(%sysfunc(exist(work.model_model_fnl)) = 1) %then
     %do;      
       data &contentId.dat.&contentId._model_model_L;
         if 0 then set work.model_model_fnl;
         stop;
       run;
     %end;
    
     /* Update the &contentId.dat table */
     proc append base=&contentId.dat.&contentId._model_model_L data=work.model_model_fnl force;
     run;
 
     /* Load LASR Table and Authorizations into Metadata */
     %rskmgt_load_lasr_tbls(intblnm=&contentId._model_model_L,
                            feature=CUST_OBJ_1);  
    %end; /* End: Build the child to parent model table */
     
     /* Remove work tables */
     proc datasets lib=work nolist;
       save 
       
       %let y=1;
       %let contentIds=%trim(%qscan(%bquote(&rgf_content_ids),&y,%str(,)));
       %do %while (&contentIds ne );
  
         &contentIds._Custommessages &contentIds._sm_security_acl &contentIds._config_props
         
         %let y=%eval(&y+1);
         %let contentIds=%trim(%qscan(%bquote(&rgf_content_ids),&y,%str(,))); 
       %end;
       %str(;)
     quit;
     run;
   
     %continue:
   %end;
 
%put NOTE: {END: &sysmacroname};

%mend rskmgt_job_custobj_data;
