/***************************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.              */
/*  All rights reserved.                                                   */
/***************************************************************************/
/*  NAME:        rskmgt_load_lasr_tbls.sas                                 */
/*  VERSION:     7.4                                                       */
/*  DESCRIPTION: Loads tables into LASR and applies RLS permissions.       */
/*  PARAMETERS:  INTBLNM - business object dataset used to load into LASR  */
/*               FEATURE - business object feature name                    */
/*  PRODUCT:     SAS Risk Management Server                                */
/***************************************************************************/

%macro rskmgt_load_lasr_tbls(intblnm=,
                             feature=);

%put NOTE: {START: &sysmacroname}; 

%if (%upcase(&load_lasr)=YES or %upcase(&load_lasr)=TRUE) and
    (%upcase(&loadETLDataTbls)=YES or %upcase(&loadETLDataTbls)=TRUE) %then
%do;  
  /* Recreate the dimension formats to build the RLS permissions for the reload jobs only */
  %if %symexist(reload_perm) %then
  %do;
    %if %upcase(&reload_perm)=YES or %upcase(&reload_perm)=TRUE %then
    %do;
      /* Put the dimensions and levels into a list */
      proc sql noprint;     
        select count(*) into : DIMCOUNT
        from &contentId.ctl.feature_dimensions
        where upcase(feature_nm)=%upcase("&feature")
        ;
      
        %let DIMCOUNT=&DIMCOUNT;
      
        select dimension_nm into : DIMENSION1 - :DIMENSION&DIMCOUNT
        from &contentId.ctl.feature_dimensions
        where upcase(feature_nm)=%upcase("&feature")
        ;
        
        select dimension_alias_nm into : DIM_ALIAS1 - :DIM_ALIAS&DIMCOUNT
        from &contentId.ctl.feature_dimensions
        where upcase(feature_nm)=%upcase("&feature")
        ;
      quit;
      
        /* Begin dimension format generation */
        %macro rskmgt_create_dim_path_format(dim_nm=&&DIMENSION&I);
          
          %put NOTE: {START: &sysmacroname};
                      
          %global &dim_nm._lvls;
      
          proc sql noprint;
             select levels into : &dim_nm._lvls
             from &contentId.ctl.feature_dimensions
             where upcase(feature_nm)=%upcase("&feature")
                   and dimension_nm="&dim_nm";
          quit;
      
          %let &dim_nm._lvls=%trim(&&&dim_nm._lvls);
      
          %if %upcase(&dim_nm) = MANAGEMENT_ORG or %upcase(&dim_nm) = LEGAL_ORG %then         
            %let dim_column =organization_nm;        
          %else %let dim_column = &dim_nm._NM; 
               
          proc sql;
            create table &dim_nm._l as
            select &dim_nm._rk
              %if %upcase(&dim_nm) = MANAGEMENT_ORG or %upcase(&dim_nm) = LEGAL_ORG %then
              %do;
                ,case
                   when B.business_object_nm="&dim_nm" and 
                        A.&dim_nm._rk=B.business_object_rk and
                        upcase(B.language_cd) = "&localecd" then B.name_txt
                   else A.organization_nm
                 end as organization_nm len=%eval(200 * &char_multiplier)
              %end;
              %else
              %do;
                ,case
                   when B.business_object_nm="&dim_nm" and 
                        A.&dim_nm._rk=B.business_object_rk and
                        upcase(B.language_cd) = "&localecd" then B.name_txt
                  else A.&dim_nm._NM
                  end as &dim_nm._NM len=%eval(200 * &char_multiplier)
              %end;
              from &src_lib..&dim_nm._l A left join
                   &src_lib..full_sbo_trans B
              on B.business_object_nm="&dim_nm" and 
                 A.&dim_nm._rk=B.business_object_rk and
                 upcase(B.language_cd)="&localecd";
          quit;
      
          data &dim_nm.fmtin(keep=fmtname type start label);
            length fmtname $32 type $1 start $64;
            retain fmtname "&dim_nm.fmt" type 'N';
            set &dim_nm._l;
              
            start = &dim_nm._rk;
            %if %upcase(&dim_nm) = MANAGEMENT_ORG or %upcase(&dim_nm) = LEGAL_ORG %then
            %do;
              label = trim(left(organization_nm));
            %end;
            %else
            %do;
              label = trim(left(&dim_nm._nm));
            %end;
          run;
      
          proc format cntlin=&dim_nm.fmtin library=work;
          quit;
          run; 
            
          proc datasets nolist;
            delete &dim_nm._l &dim_nm.fmtin;
          quit;
          run;
      
          %put NOTE: {END: &sysmacroname};
       
        %mend rskmgt_create_dim_path_format;
        
        /* Format all dimensions */        
        %do I=1 %to &DIMCOUNT.;  
          %rskmgt_create_dim_path_format(dim_nm=&&DIMENSION&I);
        %end;  
    %end;
  %end; 
  
  /* Check to see if the rgflasr.&intblnm table exist and drop it */
  %if %sysfunc(exist(rgflasr.&intblnm)) %then
  %do;
    proc datasets library=rgflasr nolist;
      delete &intblnm;
    quit;
        
    /* delete the LASR table in metadata */
    proc metalib;
      omr(library="&lasr_meta_lib");  
      select("&intblnm"); 
      update_rule=(delete);
    run;
  %end;
  %else
  %do;
    %put %sysfunc(sasmsg(sashelp.rskmgtsvr, lasr_tbl_not_found, NOQUOTE));
  %end;
  
  /* Check to see if &contentId.dat.&intblnm exist before trying to load into LASR */
  %if %sysfunc(exist(&contentId.dat.&intblnm)) %then
  %do; 
    /* Load the rgflasr.&intblnm into LASR */ 
    %if %symexist(&contentId._cmpres_va_tbls) %then
    %do;
      %if %upcase(&&&contentId._cmpres_va_tbls)=YES or %upcase(&&&contentId._cmpres_va_tbls)=TRUE %then
      %do;
        /* Load the &intblnm into LASR */                       
        data rgflasr.&intblnm(SQUEEZE=YES);
          set &contentId.dat.&intblnm;
        run;
      %end;
      %else %if %upcase(&&&contentId._cmpres_va_tbls)=NO or %upcase(&&&contentId._cmpres_va_tbls)=FALSE %then
      %do; 
        /* Load the &intblnm into LASR */                       
        data rgflasr.&intblnm;
          set &contentId.dat.&intblnm;
        run;
      %end;
    %end;
    %else
    %do;
      /* Load the &intblnm into LASR */                       
      data rgflasr.&intblnm;
        set &contentId.dat.&intblnm;
      run;
    %end;   
      
    /* register the LASR table in metadata */
    proc metalib;
      omr(library="&lasr_meta_lib");  
      select("&intblnm"); 
    run; 
   
    /* Get macro variable bit flags. */
    %mdseccon();
    
    /* Subset the user permissions to just the &intblnm business object. */
    proc sql;
      create table work.&contentId._users as
      select distinct user_id
      from work.&contentId._sm_security_acl
      where ((upcase(business_object_nm)="%upcase(&intblnm)" or business_object_nm=''
            or upcase(business_object_nm)="%upcase(&feature)") and
            upcase(active_flg)='Y');
    quit;
   
    /* Set metadata permissions on the &intblnm */
    data _null_;
      format tc $20.;
      length uri user $256 condition wclause $32767;
      tc = '';
      uri='';
      n=1;
   
      /* Get the &intblnm PhysicalTable object in metadata. */
      rc=metadata_getnobj("omsobj:PhysicalTable?@Name='&intblnm'",n,uri);
   
      /* Start transaction on object created above using the URI. */
      rc=METASEC_BEGTRAN(uri,0,tc); 
      if (rc < 0 ) then 
      do;
        sysmsg = sysmsg();
        put sysmsg;
      end;
   
      /* Loop through the &contentId users w/ access to the business object to set permission conditions */
      do until (lastrec);
        set work.&contentId._users end=lastrec;
        user=strip(user_id);
   
        call execute('%get_user_sec_isUnder_constraints(feature=' || "&feature" || ', user_id=' || user || ', security_acl_lib=work' || ')');
           
        condition=symget('acc');
        if (strip(condition)='0' or strip(condition)='1' or strip(condition)='') then wclause="";
        else wclause='(' || strip(condition) || ')';
           
        /* Give users ReadMetadata permissions on the &intblnm table */
        rc = METASEC_SETAUTH(tc, "","Person", user,
                             "Grant", "ReadMetadata", wclause,0);
        if (rc < 0 ) then 
        do;
          sysmsg = sysmsg();
          put sysmsg;
        end;    
   
        /* Give users Read permissions on the &intblnm table */
        rc = METASEC_SETAUTH(tc, "","Person", user,
                              "Grant", "Read", wclause,0);
        if (rc < 0 ) then 
        do;
          sysmsg = sysmsg();
          put sysmsg;
        end;     
      end;
      
      /* Deny the "&&&contentId._GroupName ReadMetadata permissions on the &intblnm table */
      /* so that only users w/ access to &intblnm can see the table */ 
      %let grp=1;
      %let grp_name=%scan(%bquote(&&&contentId._GroupName),&grp,%str(","));

      %do %while (&grp_name ne );
  
        rc = METASEC_SETAUTH(tc, "","IdentityGroup", "%trim(&grp_name)",
                             "Deny", "ReadMetadata", "",0);
        if (rc < 0 ) then 
        do;
          sysmsg = sysmsg();
          put sysmsg;
        end; 
            
        %let grp=%eval(&grp+1);
        %let grp_name=%scan(%bquote(&&&contentId._GroupName),&grp,%str(","));
      %end; 
       
      /* End the transaction and commit any changes made to */
      /* the transaction since it was started.              */
      rc=METASEC_ENDTRAN(uri,tc, &_SECAD_COMMIT_TC ); 
      if (rc < 0 ) then
      do;
        sysmsg = sysmsg();
        put sysmsg;
      end;
    run;
     
    /* Check to see if the rgfrload.&intblnm table exist and drop it */
    %if %sysfunc(exist(rgfrload.&intblnm)) %then
    %do;
      proc datasets library=rgfrload nolist;
        delete &intblnm;
      quit;
     
      /* Put a copy of the rgflasr.&intblnm table in rgfrload */
      /* in order to update our backing store to support reload. */
      data rgfrload.&intblnm;
        set rgflasr.&intblnm;
      run;
    %end;
    %else
    %do;
      /* Put a copy of the rgflasr.&intblnm table in rgfrload */
      /* in order to update our backing store to support reload. */
      data rgfrload.&intblnm;
        set rgflasr.&intblnm;
      run;
    %end;
  %end;
%end;

%put NOTE: {END: &sysmacroname}; 
 
%mend rskmgt_load_lasr_tbls;
