/***************************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.              */
/*  All rights reserved.                                                   */
/***************************************************************************/
/*  NAME:        rskmgt_load_lasr_tbls_grp_perms.sas                       */
/*  VERSION:     7.4                                                       */
/*  DESCRIPTION: Loads &inlib..&intblnm tables into the                    */
/*               rgflasr LASR lib w/ &contentId_GroupName permissions      */
/*               applied.                                                  */
/*  PARAMETERS:  INTBLNM - table to be loaded into LASR                    */
/*               INLIB - library where INTBLNM lives                       */
/*  PRODUCT:     SAS Risk Management Server                                */
/***************************************************************************/

%macro rskmgt_load_lasr_tbls_grp_perms(intblnm=,
                                       inlib=);

%put NOTE: {START: &sysmacroname};

%if (%upcase(&load_lasr)=YES or %upcase(&load_lasr)=TRUE) %then
%do;
   /* Check to see if the rgflasr.&intblnm table exist and drop it */
   %if %sysfunc(exist(rgflasr.&intblnm)) %then
   %do;
     proc datasets library=rgflasr nolist;
       delete &intblnm;
     quit;
           
     /* delete the LASR table in metadata */
     proc metalib;
       omr(library="&lasr_meta_lib");  
       select("&intblnm"); 
       update_rule=(delete);
     run;
   %end;
   %else
   %do;
     %put %sysfunc(sasmsg(sashelp.rskmgtsvr, lasr_tbl_not_found, NOQUOTE));
   %end;
      
   /* Load the &intblnm into LASR */ 
   %if %symexist(&contentId._cmpres_va_tbls) %then
   %do;
     %if %upcase(&&&contentId._cmpres_va_tbls)=YES or %upcase(&&&contentId._cmpres_va_tbls)=TRUE %then
     %do;  
       /* Load the &intblnm into LASR */                       
       data rgflasr.&intblnm(SQUEEZE=YES);
         set &inlib..&intblnm;
       run;
     %end;
     %else %if %upcase(&&&contentId._cmpres_va_tbls)=NO or %upcase(&&&contentId._cmpres_va_tbls)=FALSE %then
     %do;  
       /* Load the &intblnm into LASR */                       
       data rgflasr.&intblnm;
         set &inlib..&intblnm;
       run;
     %end;
   %end;
   %else
   %do;
     /* Load the &intblnm into LASR */                       
     data rgflasr.&intblnm;
       set &inlib..&intblnm;
     run;
   %end;                       

   /* register the LASR table in metadata */
   proc metalib;
     omr(library="&lasr_meta_lib");  
     select("&intblnm"); 
   run; 

   /* Get macro variable bit flags. */
   %mdseccon();
       
   /* Set metadata permissions on the &intblnm */
   data _null_;
     format tc $20.;
     length uri $256;
     tc = '';
     uri='';
     n=1;
      
     /* Get the &intblnm PhysicalTable object in metadata. */
     rc=metadata_getnobj("omsobj:PhysicalTable?@Name='&intblnm'",n,uri);
      
     /* Start transaction on object created above using the URI. */
     rc=METASEC_BEGTRAN(uri,0,tc); 
     if (rc < 0 ) then 
     do;
       sysmsg = sysmsg();
       put sysmsg;
     end;
                
     /* Grant the "&&contentId_GroupName Read and ReadMetadata permissions on the &intblnm table */
     %let grp=1;
     %let grp_name=%scan(%bquote(&&&contentId._GroupName),&grp,%str(","));

     %do %while (&grp_name ne );
     
       rc = METASEC_SETAUTH(tc, "","IdentityGroup", "%trim(&grp_name)",
                            "Grant", "ReadMetadata", "",0);
       if (rc < 0 ) then 
       do;
         sysmsg = sysmsg();
         put sysmsg;
       end; 
     
       rc = METASEC_SETAUTH(tc, "","IdentityGroup", "%trim(&grp_name)",
                          "Grant", "Read", "",0);
       if (rc < 0 ) then 
       do;
         sysmsg = sysmsg();
         put sysmsg;
       end;
               
       %let grp=%eval(&grp+1);
       %let grp_name=%scan(%bquote(&&&contentId._GroupName),&grp,%str(","));
     %end; 
     
          
     /* End the transaction and commit any changes made to */
     /* the transaction since it was started.              */
     rc=METASEC_ENDTRAN(uri,tc, &_SECAD_COMMIT_TC ); 
     if (rc < 0 ) then
     do;
       sysmsg = sysmsg();
       put sysmsg;
     end;
   run;
     
   /* Check to see if the rgfrload.&intblnm table exist and drop it */
   %if %sysfunc(exist(rgfrload.&intblnm)) %then
   %do;
     proc datasets library=rgfrload nolist;
       delete &intblnm;
     quit;
      
     /* Put a copy of the rgflasr.&intblnm table in rgfrload */
     /* in order to update our backing store to support reload. */
     data rgfrload.&intblnm;
       set rgflasr.&intblnm;
     run;
   %end;
   %else
   %do;
     /* Put a copy of the rgflasr.&intblnm table in rgfrload */
     /* in order to update our backing store to support reload. */
     data rgfrload.&intblnm;
       set rgflasr.&intblnm;
     run;
   %end;
%end;
   
%put NOTE: {END: &sysmacroname};

%mend rskmgt_load_lasr_tbls_grp_perms;
