/**********************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.         */
/*  All rights reserved.                                              */
/**********************************************************************/
/*  NAME:        rskmgt_load_link_inst_tbl.sas                        */
/*  VERSION:     7.4                                                  */
/*  DESCRIPTION: Build the link instance table for reporting use.     */
/*  PARAMETERS:                                                       */
/*  PRODUCT:     SAS Risk Management Server                           */
/**********************************************************************/

/* Build the link instance table for all business objects */
%macro rskmgt_load_link_inst_tbl;

%put NOTE: {START: &sysmacroname};

/* Build the link instance table for all business objects */
  proc sql noprint;
    %rskmgt_db_connect;
    &DBConnectSQL;

    create table work.link_instances_l as
      select * from connection to &DBConnectAlias
        (select linktyp.link_type_rk,
                linktyp.link_type_id,
                linkinst.link_instance_rk,
                fullbo.name_txt as link_type_nm,
                linkinst.business_object_rk_1,
                linktyp.business_object_type_nm_1,
                linkinst.business_object_rk_2,
                linktyp.business_object_type_nm_2,
                case when upper(linktyp.business_object_type_nm_1)=upper(bot1.business_object_type_nm) then bot1.business_object_nm
                 else ''
                end as business_object_nm_1,
                case when upper(linktyp.business_object_type_nm_2)=upper(bot2.business_object_type_nm) then bot2.business_object_nm
                 else ''
                end as business_object_nm_2,
                case when upper(bot1.business_object_nm)='ASSESSMENT_PERIOD' then 'GLB_PERIOD_RK'
                     when upper(bot1.business_object_nm)='ASSESSMENT_QUESTION_MASTER' then 'GLB_ASSESSMENT_QUESTION_RK'
                     when upper(bot1.business_object_nm)='QUESTION_GRP_MASTER' then 'GLB_ASSESSMENT_QUESTION_GROUP_RK'
                     when upper(bot1.business_object_nm)='CHANGE_REASON' then 'GLB_CHANGE_REASON_RK'
                     when upper(bot1.business_object_nm)='CUST_FIELD_DEFN' then 'GLB_CUST_FIELD_DEF_RK'
                     when upper(bot1.business_object_nm)='CUST_FIELD_VALUE' then 'GLB_CUST_FIELD_VAL_RK'
                     when upper(bot1.business_object_nm)='FAVORITE_AREA' then 'GLB_FAVORITE_RK'
                     when upper(bot1.business_object_nm)='GENERIC_COMMENT' then 'GLB_COMMENT_RK'
                     when upper(bot1.business_object_nm)='OP_RISK_ISSUE' then 'GLB_ISSUE_RK'
                     when upper(bot1.business_object_nm)='OBS_REQUEST' then 'GLB_OBS_REQ_RK'
                     when upper(bot1.business_object_nm)='ORAUSER' then 'GLB_USER_RK'
                     when upper(bot1.business_object_nm)='ORM_POSITION' then 'GLB_POSITION_RK'
                     when upper(bot1.business_object_nm)='ORM_ROLE' then 'GLB_ROLE_RK'
                     when upper(bot1.business_object_nm)='AUDIT_INST' then 'GLB_AUDIT_RK'
                     when upper(bot1.business_object_nm)='POLICY_INST' then 'GLB_POLICY_RK'
                     when upper(bot1.business_object_nm)='POTENTIAL_IMPACT' then 'GLB_IMPACT_RK'
                     else 
                      
                     %if %upcase(&rskmgt_dbserver) = SQLSERVER %then 
                     %do; 
                       'GLB_'+upper(bot1.business_object_nm)+'_RK'
                     %end;
                     %else
                     %do;
                       'GLB_'||upper(bot1.business_object_nm)||'_RK'
                     %end;
                      
                end as business_object_rk_nm1,
                case when upper(bot2.business_object_nm)='ASSESSMENT_PERIOD' then 'GLB_PERIOD_RK'
                     when upper(bot2.business_object_nm)='ASSESSMENT_QUESTION_MASTER' then 'GLB_ASSESSMENT_QUESTION_RK'
                     when upper(bot2.business_object_nm)='QUESTION_GRP_MASTER' then 'GLB_ASSESSMENT_QUESTION_GROUP_RK'
                     when upper(bot2.business_object_nm)='CHANGE_REASON' then 'GLB_CHANGE_REASON_RK'
                     when upper(bot2.business_object_nm)='CUST_FIELD_DEFN' then 'GLB_CUST_FIELD_DEF_RK'
                     when upper(bot2.business_object_nm)='CUST_FIELD_VALUE' then 'GLB_CUST_FIELD_VAL_RK'
                     when upper(bot2.business_object_nm)='FAVORITE_AREA' then 'GLB_FAVORITE_RK'
                     when upper(bot2.business_object_nm)='GENERIC_COMMENT' then 'GLB_COMMENT_RK'
                     when upper(bot2.business_object_nm)='OP_RISK_ISSUE' then 'GLB_ISSUE_RK'
                     when upper(bot2.business_object_nm)='OBS_REQUEST' then 'GLB_OBS_REQ_RK'
                     when upper(bot2.business_object_nm)='ORAUSER' then 'GLB_USER_RK'
                     when upper(bot2.business_object_nm)='ORM_POSITION' then 'GLB_POSITION_RK'
                     when upper(bot2.business_object_nm)='ORM_ROLE' then 'GLB_ROLE_RK'
                     when upper(bot2.business_object_nm)='AUDIT_INST' then 'GLB_AUDIT_RK'
                     when upper(bot2.business_object_nm)='POLICY_INST' then 'GLB_POLICY_RK'
                     when upper(bot2.business_object_nm)='POTENTIAL_IMPACT' then 'GLB_IMPACT_RK'
                     else 
                      
                     %if %upcase(&rskmgt_dbserver) = SQLSERVER %then 
                     %do; 
                       'GLB_'+upper(bot2.business_object_nm)+'_RK'
                     %end;
                     %else
                     %do;
                       'GLB_'||upper(bot2.business_object_nm)||'_RK'
                     %end;
                 
                end as business_object_rk_nm2 
         from 
        
         %if &&&contentId._schema ne %then 
         %do;
           &&&contentId._schema..link_type_l linktyp
           LEFT JOIN &&&contentId._schema..link_instance_l linkinst
            ON linktyp.link_type_rk=linkinst.link_type_rk
           LEFT JOIN &&&contentId._schema..business_object_type bot1
            ON linktyp.business_object_type_nm_1=bot1.business_object_type_nm
           LEFT JOIN &&&contentId._schema..business_object_type bot2
            ON linktyp.business_object_type_nm_2=bot2.business_object_type_nm
           LEFT JOIN &&&contentId._schema..full_bo_trans fullbo
            ON fullbo.business_object_rk=linktyp.link_type_rk and
               fullbo.business_object_nm='LINK_TYPE' and
               upper(fullbo.language_cd)=%str(%')%upcase(&localecd)%str(%')
         %end;
         %else
         %do;
           link_type_l linktyp
           LEFT JOIN link_instance_l linkinst
            ON linktyp.link_type_rk=linkinst.link_type_rk
           LEFT JOIN business_object_type bot1
            ON linktyp.business_object_type_nm_1=bot1.business_object_type_nm
           LEFT JOIN business_object_type bot2
            ON linktyp.business_object_type_nm_2=bot2.business_object_type_nm
           LEFT JOIN full_bo_trans fullbo
            ON fullbo.business_object_rk=linktyp.link_type_rk and
               fullbo.business_object_nm='LINK_TYPE' and
               upper(fullbo.language_cd)=%str(%')%upcase(&localecd)%str(%')
         %end;

         order by linkinst.business_object_rk_1, linktyp.business_object_type_nm_1);
     
    disconnect from &DBConnectAlias;
  quit;

  /* Add labels to the output table */
  data work.link_instances_l;
    set work.link_instances_l;
    label business_object_rk_1="%sysfunc(sasmsg(sashelp.rskmgtsvr, Primary_Business_Object_Rk, NOQUOTE))"
          business_object_type_nm_1="%sysfunc(sasmsg(sashelp.rskmgtsvr, Primary_Business_Object_Type_Nm, NOQUOTE))"
          business_object_rk_2="%sysfunc(sasmsg(sashelp.rskmgtsvr, Secondary_Business_Object_Rk, NOQUOTE))" 
          business_object_type_nm_2="%sysfunc(sasmsg(sashelp.rskmgtsvr, Secondary_Business_Object_Type_Nm, NOQUOTE))" 
          business_object_rk_nm1="%sysfunc(sasmsg(sashelp.rskmgtsvr, Business_Object_Rk_Nm1, NOQUOTE))"
          business_object_rk_nm2="%sysfunc(sasmsg(sashelp.rskmgtsvr, Business_Object_Rk_Nm2, NOQUOTE))"
          link_type_id="%sysfunc(sasmsg(sashelp.rskmgtsvr, Link_Type_ID, NOQUOTE))"
          link_type_nm="%sysfunc(sasmsg(sashelp.rskmgtsvr, Link_Type_Nm, NOQUOTE))";
  run;

  /* Sort the link instances */
  proc sort data=work.link_instances_l out=work.&contentId._link_instances_l;
    by business_object_rk_1 business_object_type_nm_1;
  run;

  /* Reset the &contentId.dat table for appending the new records */
  data &contentId.dat.&contentId._link_instances_l;
    if 0 then set work.&contentId._link_instances_l;
    stop;
  run;

  /* Store the table in the &contentId.dat library */
  proc append base=&contentId.dat.&contentId._link_instances_l data=work.&contentId._link_instances_l force;
  run;

  %if (%upcase(&&&contentId._load_links)=YES or %upcase(&&&contentId._load_links)=TRUE) %then
  %do;  
    /* Load LASR Table and Authorizations into Metadata w/ Group Permissions only */
    %rskmgt_load_lasr_tbls_grp_perms(intblnm=&contentId._link_instances_l,
                                     inlib=&contentId.dat);
  %end;
                    
%put NOTE: {END: &sysmacroname};

%mend rskmgt_load_link_inst_tbl;
