/***************************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.              */
/*  All rights reserved.                                                   */
/***************************************************************************/
/*  NAME:        rskmgt_reload_lasr_data.sas                               */
/*  VERSION:     7.4                                                       */
/*  DESCRIPTION: Reloads a table into LASR.                                */
/*  PARAMETERS:  INTBLNM - business object dataset used to load into LASR  */
/*               FEATURE - business object feature name                    */
/*               RLS_PERM - set business object row level security         */
/*                          permissions for VA (Default: Y)                */
/*  PRODUCT:     SAS Risk Management Server                                */
/***************************************************************************/

%macro rskmgt_reload_lasr_data(intblnm=,
                               feature=,
                               rls_perm=Y);

%put NOTE: {START: &sysmacroname};

%if (%upcase(&load_lasr)=YES or %upcase(&load_lasr)=TRUE) and
    (%upcase(&loadETLDataTbls)=YES or %upcase(&loadETLDataTbls)=TRUE) %then
%do;
  /* Check the reload_perm flag: reload the lasr table w/ new permissions if TRUE; */
  /* else just reload the table w/ no permission updates */
  %if %upcase(&reload_perm)=NO or %upcase(&reload_perm)=FALSE %then
  %do;  
   /* Check to see if the table was unloaded or deleted: */
   /* If the table does not exist in metadata, it was deleted so, force a new load b/c permissions would have been removed from metadata */
   /* If the table does exist in metadata, it was unloaded so, we can reload from rgfrload b/c the permissions are still in metadata */
   data _null_;
     length TblUri $256 name $32767;
     TblUri='';
     name='';

     TblUri = "omsobj:PhysicalTable?@Name='" || "&intblnm" || "'";

     rc = metadata_getattr(TblUri, "TableName", name);
      if rc >= 0 then call symput('meta_exist','YES');
     else call symput('meta_exist','NO');
   run;

   /* Unload and Reload w/o permission changes */
    %if %sysfunc(exist(rgflasr.&intblnm)) and &meta_exist = YES %then
    %do;
      /* Unload the table before reloading */
      proc datasets library=rgflasr nolist;
        delete &intblnm;
      quit;
      
      %if %symexist(&contentId._cmpres_va_tbls) %then
      %do;
        %if %upcase(&&&contentId._cmpres_va_tbls)=YES or %upcase(&&&contentId._cmpres_va_tbls)=TRUE %then
        %do;
          /* Load the &intblnm into LASR */                       
          data rgflasr.&intblnm(SQUEEZE=YES);
            set rgfrload.&intblnm;
          run;
        %end;
        %else %if %upcase(&&&contentId._cmpres_va_tbls)=NO or %upcase(&&&contentId._cmpres_va_tbls)=FALSE %then
        %do; 
          /* Load the &intblnm into LASR */                       
          data rgflasr.&intblnm;
            set rgfrload.&intblnm;
          run;
        %end;
      %end;
      %else
      %do;
        /* Load the &intblnm into LASR */                       
        data rgflasr.&intblnm;
          set rgfrload.&intblnm;
        run;
      %end;
    
      /* register the LASR table in metadata */
      proc metalib;
       omr(library="&lasr_meta_lib");  
       select("&intblnm"); 
       update_rule=(noupdate);
      run;
    %end;
    %else /* Reload from rgfrload library w/o permission changes */
    %do;
      %if %sysfunc(exist(rgfrload.&intblnm)) and &meta_exist = YES %then
      %do;
        %if %symexist(&contentId._cmpres_va_tbls) %then
        %do;
          %if %upcase(&&&contentId._cmpres_va_tbls)=YES or %upcase(&&&contentId._cmpres_va_tbls)=TRUE %then
          %do;
            /* Load the &intblnm into LASR */                       
            data rgflasr.&intblnm(SQUEEZE=YES);
              set rgfrload.&intblnm;
            run;
          %end;
          %else %if %upcase(&&&contentId._cmpres_va_tbls)=NO or %upcase(&&&contentId._cmpres_va_tbls)=FALSE %then
          %do; 
            /* Load the &intblnm into LASR */                       
            data rgflasr.&intblnm;
              set rgfrload.&intblnm;
            run;
          %end;
        %end;
        %else
        %do;
          /* Load the &intblnm into LASR */                       
          data rgflasr.&intblnm;
            set rgfrload.&intblnm;
          run;
        %end;
       
        /* register the LASR table in metadata */
        proc metalib;
          omr(library="&lasr_meta_lib");  
          select("&intblnm"); 
          update_rule=(noupdate);
        run;
      %end;
      %else /* Force a new load since the rgflasr.&intblnm and &rgfrload.&intblnm tables do not exist */
      %do;
        %if &rls_perm=Y %then
        %do;
          /* Load w/ permissions updated */
          %rskmgt_load_lasr_tbls(intblnm=&intblnm,
                                 feature=&feature);
        %end;
        %else
        %do;
          /* Load LASR Table w/ only &contentID Group Authorizations into Metadata */
          %rskmgt_load_lasr_tbls_grp_perms(intblnm=&intblnm,
                                           inlib=&contentId.dat);
        %end;
      %end;
    %end;
  %end;
  %else /* reload_perm = TRUE (Force a new load since we are updating the permissions */
  %do;
     %if &rls_perm=Y %then
     %do;
       /* Load w/ permissions updated */
       %rskmgt_load_lasr_tbls(intblnm=&intblnm,
                              feature=&feature);
     %end;
     %else
     %do;
       /* Load LASR Table w/ only &contentID Group Authorizations into Metadata */
       %rskmgt_load_lasr_tbls_grp_perms(intblnm=&intblnm,
                                        inlib=&contentId.dat);
     %end;
  %end;
%end;

%put NOTE: {END: &sysmacroname};

%mend rskmgt_reload_lasr_data;
