/************************************************************************************/
/*  Copyright (c) 2018 by SAS Institute, Inc., Cary, NC, USA.                       */
/*  All rights reserved.                                                            */
/************************************************************************************/
/*  NAME:        rskmgt_var_exists.sas                                              */
/*  VERSION:     7.4                                                                */
/*  DESCRIPTION: Returns a list of space delimited variables in a dataset.          */
/*  PARAMETERS: libname=library where the dataset exist                             */
/*              memname=name of the dataset                                         */
/*              varListNm=global macro variable that contains the list of variables */
/*  PRODUCT:    SAS Risk Management Server                                          */
/************************************************************************************/

%macro rskmgt_var_exists(libname=,
                         memname=,
                         varListNm=);

%put NOTE: {START: &sysmacroname};

  %global &varListNm;

  proc sql noprint;
    select upcase(Name)
             into : &varListNm separated by ' '
    from  Dictionary.Columns
    where  upcase(Libname) = "%upcase(&libname.)"
      and  upcase(Memname) = "%upcase(&memname.)"
      and  upcase(MemType) = 'DATA';
  quit;

%put NOTE: {END: &sysmacroname};

%mend rskmgt_var_exists;